/*
for FITS conversion

1999/8/2 T.Handa ver.3.0
*/
/* ====================== */
/*  MACRO constant definitions */
#define FITS_VALUE_STRING 2
#define FITS_VALUE_SHORT_STRING 3
#define FITS_VALUE_BOOLIAN 1
#define FITS_VALUE 0

#define FITS_VALUE_INT     13
#define FITS_VALUE_FLOAT   12
#define FITS_VALUE_EXP     11
#define FITS_VALUE_NOTHING 10
#define FITS_VALUE_END    999

#define MAX_NAXIS 1000
#define MAX_PSIZE 1000
#define MAX_LWORD 80

/* file type */
#define FileExist 1
#define FileAbsent 0
#define FileNoexist 0
#define FileDirectory 11
#define FileInvalid 900
#define FileUnknown 9999

/* ====================== */
/*  structure definitions */
typedef struct { 
  int naxis;             /* number of elements in the axis */
  char ctype[MAX_LWORD]; /* meaning of the axis */
  double crpix;          /* reference point on the axis (pixel) */
  double crval;          /* reference value of the reference point */
  double cdelt;          /* value per pixel along the axis */
  double crota;          /* rotation angle of the axis (deg) */
  char cunit[MAX_LWORD]; /* unit of the axis */
} FITS_AXIS;

typedef struct {
  char bunit[80];	/* name of unit of the data */
  double bscale;	/* real=bscale*pixel+bzero */
  double bzero;
  double blank;	/* dummy data if pixel==blank */
} FITS_BSCALE; /* data scaling */

typedef struct {
  char ptype[MAX_LWORD];	/* name of the parameter */
  double pscale;	/* real=pscale*pixel+pzero */
  double pzero;
} FITS_RG_P;	/* paramter with random group FITS */

/* ====================== */
/*  function prototype declaration */

/*-------file names, parameter processing-------*/
#ifdef _NO_PROTO

extern int StarReadFitsBlock();
extern void StarWriteFitsRecord();
extern void StarAxis2FitsCtype();
extern void StarHeadAxisFromFitsHead();
extern void StarFitsCtype2Axis();

extern int StarWriteFitsHead4CubeHeadAxes();
extern int StarWriteFitsHead4CubeHeadAxis();

#else /* undef _NO_PROTO */

extern int StarReadFitsBlock(FILE *, char [36][80]);
extern void StarWriteFitsRecord(char *, char *, int, char *, FILE *);
extern void StarAxis2FitsCtype(char *, AXIS *);
extern void StarHeadAxisFromFitsHead(AXIS *, FITS_AXIS *);
extern void StarFitsCtype2Axis(AXIS *, char *);

extern int StarWriteFitsHead4CubeHeadAxes(FILE *, AXES *);
extern int StarWriteFitsHead4CubeHeadAxis(FILE *, AXIS *, int);
#endif /* endif of define _NO_PROTO */


#ifdef _NO_PROTO
extern void StarCutFitsComment();
extern int StarDivideFitsCard();

extern int StarFitsPixel2CubePixel();

#else
extern void StarCutFitsComment(char *);
extern int StarDivideFitsCard(char *, char *, char *);

extern int StarFitsPixel2CubePixel(FILE *, CUBE *, int);
#endif
