/*
UltraSTAR2006
standard include file

<StData.h>
StStd.h

2006/1/19 T.Handa
*/

#include "StFilename.h"
#ifndef STAR_ASTRON
#include <StAstron.h>
#define STAR_ASTRON
#endif

/* ====================== */
/*  MACRO constant definitions */
/*----- for sdb -----*/
#define	PIPE_FLOW 1000
#define	PIPE_FLOW_END 1001
#define	UNKNOWN_FLOW 9999
#define	BLANK_FLOW 9990
#define	SCN_FLOW 1
#define	SDB_FLOW 2
#define	CUB_FLOW 11
#define	CDB_FLOW 12

#define STAR_PIPE_IN	"STARpipe"
#define	STAR_PIPE_END	"STARpEnd"
#define	STAR_SCN_IN	"STAR.scn"
#define	STAR_CUB_IN	"STAR.cub"
#define	STAR_SDB_IN	"STAR.sdb"
#define	STAR_CDB_IN	"STAR.cdb"

/*----------for Cube---------*/
#define L_COLOR_STRING 128

#define XY_CUT 1
#define YX_CUT 11
#define YZ_CUT 2
#define ZY_CUT 12
#define ZX_CUT 3
#define XZ_CUT 13

#define PLOT_CELL_MAP 1
#define PLOT_CONTOUR_MAP 2

#define SQUARE_SCALE_MAP 1
#define NO_SQUARE_SCALE_MAP 0
#define RECTANGLE_SCALE_MAP 0

/* line pattern name */
#define MAX_CONTOUR_SETS 64

#define SOLID_LINE 10
#define AUTO_POSNEG_LINE 0
#define DASHED_LINE 1
#define DOTTED_LINE 2
#define DOTTED_DASHED_LINE 3

#define STD_LINE_WIDTH 1.0

/*----------for Astronomical Calculations---------*/
#ifndef PI
#define	PI	(3.14159265358979323846)
#endif
#ifndef LIGHT_SPEED
#define	LIGHT_SPEED	(2.99792458e5)	/* light speed in km/s */
#endif

/* ====================== */
/*  structure definitions */
/*----------for Cube---------*/

typedef struct { 
  int n;             /* number of contours */
  double start_level; /* start level */
  double interval;    /* contour interval */
  int line_type;      /* contour line type */
  double thickness;   /* thickness(width) of a drawn line */
  char color[32]; /* color name, by X11 Color (/usr/lib/X11/rgb.txt) or #rrggbb */
} Contours;

/*--------user-input-parameter processing---------*/
typedef struct	{	double	max;
			double	min;
		} RANGE;
typedef struct	{	double	dx;
			double	dy;
		} XY_SIZE;
#ifndef XY_POS_DEFINE
typedef struct	{	double	x;
			double	y;
		} XY_POS;
#define XY_POS_DEFINE
#endif
typedef struct	{	long	max;
			long	min;
		} RANGE_long;
typedef struct	{	long	dx;
			long	dy;
		} XY_SIZE_long;
typedef struct	{	long	x;
			long	y;
		} XY_POS_long;
typedef struct	{	RANGE	x;
			RANGE	y;
		} BOX_RANGE;
typedef struct	{	RANGE_long	x;
			RANGE_long	y;
		} BOX_RANGE_long;

/*----------for Astronomical Calculations---------*/
#ifndef XY_POS_DEFINE
typedef struct {
		double x; 	/* longitude(degree) */
		double y; 	/* latitude(degree) */
	} XY_POS;
#define XY_POS_DEFINE
#endif

typedef struct {
		long year;
		long month;
		long day;
		long hour;
		long min;
		long sec;
		} DATE_AND_TIME;

/* ====================== */
/*  function prototype declarations */
/*----------for Profile---------*/
#ifdef _NO_PROTO
extern int	StarProfileIn();
extern int	StarProfileOut();

extern int	StarProfileLoad();
extern int	StarProfileSave();

extern int	StarPrfheadLoad();

#else /* undef _NO_PROTO */
extern int	StarProfileIn(PROFILE *, int *, char *);
extern int	StarProfileOut(PROFILE *, int, char *);

extern int	StarProfileLoad(FILE *, PROFILE *);
extern int	StarProfileSave(FILE *, PROFILE *);

extern int	StarPrfheadLoad(FILE *, PRF_HEADER *);

#endif /* endif of define _NO_PROTO */

#ifdef _NO_PROTO
extern double	StarCalcRms();
extern void	StarProfileMaxMin();
extern int StarEstimateActualChRange();

#else /* undef _NO_PROTO */
extern double	StarCalcRms(PROFILE *, RANGE_long);
extern void	StarProfileMaxMin(PROFILE *,int, int ,double *, double *);
extern int StarEstimateActualChRange(PROFILE *, RANGE, int, RANGE_long *);

#endif /* endif of define _NO_PROTO */

/*--------option for scn---------*/
#ifdef _NO_PROTO
extern void StarProfHeadClear();
extern int StarProfHeadKeyWords();
extern void StarHeadOptProf();

extern void StarHeadOptSys_head();
extern void StarHeadOptObs_prm();
extern void StarHeadOptObject();
extern void StarHeadOptAnt_status();
extern void StarHeadOptRx_status();
extern void StarHeadOptBe_status();
extern void StarHeadOptObs_log();

extern void StarHeadClearSys_head();
extern void StarHeadClearObs_prm();
extern void StarHeadClearObject();
extern void StarHeadClearAnt_status();
extern void StarHeadClearRx_status();
extern void StarHeadClearBe_status();
extern void StarHeadClearObs_log();

#else /* undef _NO_PROTO */
extern void StarProfHeadClear(PRF_HEADER *);
extern int StarProfHeadKeyWords(char **, int *, int *);
extern void StarHeadOptProfile(char **,int *,int *,char **,int *,
	PRF_HEADER *,PRF_HEADER *,PRF_HEADER *);

extern void StarHeadOptSys_head(char **,int *,int *,char **,
	int,SYS_HEAD *,SYS_HEAD *,SYS_HEAD *);
extern void StarHeadOptObs_prm(char **,int *,int *,char **,int,
	OBS_PRM *,OBS_PRM *,OBS_PRM *);
extern void StarHeadOptObject(char **,int *,int *,char **,int,
	OBJECT *,OBJECT *,OBJECT *);
extern void StarHeadOptAnt_status(char **,int *,int *,char **,int,
	ANT_STATUS *,ANT_STATUS *,ANT_STATUS *);
extern void StarHeadOptRx_status(char **,int *,int *,char **,int,
	RX_STATUS *,RX_STATUS *,RX_STATUS *);
extern void StarHeadOptBe_status(char **,int *,int *,char **,int,
	BE_STATUS *,BE_STATUS *,BE_STATUS *);
extern void StarHeadOptObs_log(char **,int *,int *,char **,int,
	OBS_LOG *,OBS_LOG *,OBS_LOG *);

extern void StarHeadClearSys_head(SYS_HEAD *);
extern void StarHeadClearObs_prm(OBS_PRM *);
extern void StarHeadClearObject(OBJECT *);
extern void StarHeadClearAnt_status(ANT_STATUS *);
extern void StarHeadClearRx_status(RX_STATUS *);
extern void StarHeadClearBe_status(BE_STATUS *);
extern void StarHeadClearObs_log(OBS_LOG *);

#endif /* endif of define _NO_PROTO */


#ifdef _NO_PROTO
extern void StarCubeHeadClear();
extern int StarCubeHeadKeyWords();
extern void StarHeadOptCube();

extern void StarHeadOptAxes();
extern void StarHeadOptData_value();

extern void StarHeadClearAxis();
extern void StarHeadClearAxes();
extern void StarHeadClearData_value();

#else /* undef _NO_PROTO */
extern void StarCubeHeadClear(CUBE_HEAD *);
extern int StarCubeHeadKeyWords(char **, int *, int *);
extern void StarHeadOptCube(char **,int *,int *,char **,int *,
	CUBE_HEAD *,CUBE_HEAD *,CUBE_HEAD *);

extern void StarHeadOptAxes(char **,int *,int *,char **,int,
	AXES *,AXES *,AXES *);
extern void StarHeadOptData_value(char **,int *,int *,char **,int,
	DATA_VALUE *,DATA_VALUE *,DATA_VALUE *);

extern void StarHeadClearAxis(AXIS *);
extern void StarHeadClearAxes(AXES *);
extern void StarHeadClearData_value(DATA_VALUE *);

#endif /* endif of define _NO_PROTO */



#ifdef _NO_PROTO
extern unsigned long StarProfHeadKeyAddress();
extern void *StarSystemHeaderKeyAddress();
extern void *StarObsParamKeyAddress();
extern void *StarObjectKeyAddress();
extern void *StarAntennaKeyAddress();
extern void *StarReceiverKeyAddress();
extern void *StarBackendKeyAddress();
extern void *StarObsLogKeyAddress();

extern void StarProfHeadKeyAddress2Name();
extern void StarProfHeadKeyName2KeywordName();

#else /* undef _NO_PROTO */
extern unsigned long StarProfHeadKeyAddress(PRF_HEADER *, char *, int *, int *);
extern void *StarSystemHeaderKeyAddress(SYS_HEAD *, char *, int *, int *);
extern void *StarObsParamKeyAddress(OBS_PRM *, char *, int *, int *);
extern void *StarObjectKeyAddress(OBJECT *, char *, int *, int *);
extern void *StarAntennaKeyAddress(ANT_STATUS *, char *, int *, int *);
extern void *StarReceiverKeyAddress(RX_STATUS *, char *, int *, int *);
extern void *StarBackendKeyAddress(BE_STATUS *, char *, int *, int *);
extern void *StarObsLogKeyAddress(OBS_LOG *, char *, int *, int *);

extern void StarProfHeadKeyAddress2Name(PRF_HEADER *, unsigned long , char *);
extern void StarProfHeadKeyName2KeywordName(char *, char *);

#endif /* endif of define _NO_PROTO */

/*-------PROFILE header processing-------*/

#ifdef _NO_PROTO
extern int	StarSubtractProfile();
extern int	StarAddProfile();
extern void	StarMultiplyProfile();
extern int	StarLinearCombProfile();

extern int	StarAddHistory();

extern void	StarRReadBaselineFlags();
extern void	StarRReadSpuriousFlags();
extern void	StarProfileClearBaselineFlag();
extern void	StarRWriteBaselineFlags();
extern void	StarWriteBaselineFlag();
extern int	StarReadBaselineFlag();
extern void	StarProfileClearSpuriousFlag();
extern void	StarRWriteSpuriousFlags();
extern void	StarWriteSpuriousFlag();
extern int	StarReadSpuriousFlag();

extern int	StarReadBitMapFlag();
extern void	StarWriteBitMapFlag();
extern void	StarRWriteBitMapFlags();

#else /* undef _NO_PROTO */
extern int	StarSubtractProfile(PROFILE *, PROFILE *, PROFILE *);
extern int	StarAddProfile(PROFILE *, PROFILE *, PROFILE *);
extern void	StarMultiplyProfile(PROFILE *, double, PROFILE *);
extern int	StarLinearCombProfile(PROFILE *, double, PROFILE *, double, PROFILE *);

extern int	StarAddHistory(PROFILE *, HISTORY *);

extern void	StarRReadBaselineFlags(PROFILE *, int *, RANGE_long *);
extern void	StarRReadSpuriousFlags(PROFILE *, int *, RANGE_long *);
extern void	StarProfileClearBaselineFlag(PROFILE *, int);
extern void	StarRWriteBaselineFlags(PROFILE *, RANGE_long, int);
extern void	StarWriteBaselineFlag(PROFILE *, int, int);
extern int	StarReadBaselineFlag(PROFILE *, int);
extern void	StarProfileClearSpuriousFlag(PROFILE *, int);
extern void	StarRWriteSpuriousFlags(PROFILE *, RANGE_long, int);
extern void	StarWriteSpuriousFlag(PROFILE *, int, int);
extern int	StarReadSpuriousFlag(PROFILE *, int);

extern int	StarReadBitMapFlag(int, unsigned char *);
extern void	StarWriteBitMapFlag(unsigned char *, int, int, int);
extern void	StarRWriteBitMapFlags(unsigned char *, RANGE_long, int, int);

#endif /* endif of define _NO_PROTO */

/*-------frequency and velocity calcurations-------*/
#ifdef _NO_PROTO
extern double	StarBindedCh();
extern double	StarBEDeltaFreqOnBE();
extern double	StarFreqOnBE2Ch();
extern double	StarFrequency2Ch();
extern double	StarVelocity2Ch();

extern double	StarOriginalCh();	
extern double	StarCh2FreqOnBE();
extern double	StarCh2Frequency();
extern double	StarCh2Velocity();

#else /* undef _NO_PROTO */
extern double	StarBindedCh(PROFILE *, double);
extern double	StarBEDeltaFreqOnBE(PROFILE *, double);
extern double	StarFreqOnBE2Ch(PROFILE *, int *, double);
extern double	StarFrequency2Ch(PROFILE *, int *, double);
extern double	StarVelocity2Ch(PROFILE *, int *, double);

extern double	StarOriginalCh(PROFILE *, double);
extern double	StarCh2FreqOnBE(PROFILE *, double);
extern double	StarCh2Frequency(PROFILE *, double);
extern double	StarCh2Velocity(PROFILE *, double);
#endif /* endif of define _NO_PROTO */

/*--------PROFILE data processing--------*/
#ifdef _NO_PROTO
extern void	StarCopyProfile();

extern void	StarCopyAntstatus();
extern void	StarCopyBaseline();
extern void	StarCopyBestatus();
extern void	StarCopyHistory();
extern void	StarCopyObject();
extern void	StarCopyObslog();
extern void	StarCopyObsprm();
extern void	StarCopyProfdata();
extern void	StarCopyRxstatus();
extern void	StarCopySpurious();
extern void	StarCopySyshead();

#else /* undef _NO_PROTO */
extern void	StarCopyProfile(PROFILE *, PROFILE *);

extern void	StarCopyAntstatus(ANT_STATUS *, ANT_STATUS *);
extern void	StarCopyBaseline(unsigned int,
			unsigned char *, unsigned char *);
extern void	StarCopyBestatus(BE_STATUS *, BE_STATUS *);
extern void	StarCopyHistory(HISTORY *, HISTORY *);
extern void	StarCopyObject(OBJECT *, OBJECT *);
extern void	StarCopyObslog(OBS_LOG *, OBS_LOG *);
extern void	StarCopyObsprm(OBS_PRM *, OBS_PRM *);
extern void	StarCopyProfdata(unsigned int, float *, float *);
extern void	StarCopyRxstatus(RX_STATUS *, RX_STATUS *);
extern void	StarCopySpurious(unsigned int,
			unsigned char *, unsigned char *);
extern void	StarCopySyshead(SYS_HEAD *, SYS_HEAD *);
#endif /* endif of define _NO_PROTO */


#ifdef _NO_PROTO
extern void StarSetDefaultProfile();
extern void StarDefaultSyshead();
extern void StarDefaultObsprm();
extern void StarDefaultObject();
extern void StarDefaultAntStatus();
extern void StarDefaultRxStatus();
extern void StarDefaultBeStatus();
extern void StarDefaultObsLog();
extern void StarDefaultHistory();

#else /* undef _NO_PROTO */

extern void StarSetDefaultProfile(PROFILE *);
extern void StarDefaultSyshead(SYS_HEAD *);
extern void StarDefaultObsprm(OBS_PRM *);
extern void StarDefaultObject(OBJECT *);
extern void StarDefaultAntStatus(ANT_STATUS *);
extern void StarDefaultRxStatus(RX_STATUS *);
extern void StarDefaultBeStatus(BE_STATUS *);
extern void StarDefaultObsLog(OBS_LOG *);
extern void StarDefaultHistory(PROFILE *);
#endif /* endif of define _NO_PROTO */

/*----------for Cube---------*/
#ifdef _NO_PROTO
extern int StarCubeIn();
extern int StarCubeOut();

extern int StarCubeHeadLoad();
extern int StarCubeHeadSave();
extern int StarCubePixelLoad();
extern int StarCubePixelSave();

#else /* undef _NO_PROTO */
extern int StarCubeIn(CUBE *, int *, char *);
extern int StarCubeOut(CUBE *, int, char *);

extern int StarCubeHeadLoad(FILE *, CUBE_HEAD *);
extern int StarCubeHeadSave(FILE *, CUBE_HEAD *);
extern int StarCubePixelLoad(FILE *, CUBE *);
extern int StarCubePixelSave(FILE *, CUBE *);

#endif /* endif of define _NO_PROTO */

#ifdef _NO_PROTO
extern int StarSaveFitsPixel();
#else /* undef _NO_PROTO */
extern int StarSaveFitsPixel(FILE *, CUBE *);
#endif /* endif of define _NO_PROTO */

#ifdef _NO_PROTO
extern int StarAddHistory4Cube();
extern void StarAllocCube();
extern CUBE_PIXEL StarAllocCubePixel();
extern void StarFreeCubePixel();
extern float **StarAlloc2DFloat();
extern void StarFree2DFloat();
extern void StarPutCubePixel();
extern float StarGetCubePixel();
extern void StarCubePixelClear();
extern void StarPutCubePixelData();
extern double StarGetCubePixelData();
extern void StarPutCubePixelDummy();

#else /* undef _NO_PROTO */
extern int StarAddHistory4Cube(CUBE_HEAD *,HISTORY *);
extern void StarAllocCube(CUBE *);
extern CUBE_PIXEL StarAllocCubePixel(CUBE_HEAD *);
extern void StarFreeCubePixel(CUBE *);
extern float **StarAlloc2DFloat(int, int);
extern void StarFree2DFloat(float **, int);
extern void StarPutCubePixel(CUBE *, int, int, int, float);
extern float StarGetCubePixel(CUBE *, int, int, int);
extern void StarCubePixelClear(CUBE *,float);
extern void StarPutCubePixelData(CUBE *, int, int, int, double);
extern double StarGetCubePixelData(CUBE *, int, int, int, int *);
extern void StarPutCubePixelDummy(CUBE *, int, int ,int);

#endif /* endif of define _NO_PROTO */

#ifdef _NO_PROTO
extern int StarCubeFindLongLatVAxes();

extern double StarGetCubePixPosOnAxis();
extern void StarGetCubePixPosition();
extern double StarPixelIndexPositionOnAxis();
extern double StarPixelValuePositionOnAxis();

extern AXIS *StarCubeFindAxisByName();
extern AXIS *StarCubeFindAxisByType();

#else /* undef _NO_PROTO */
extern int StarCubeFindLongLatVAxes(AXES *,int *,AXIS *, AXIS *, AXIS *);

extern double StarGetCubePixPosOnAxis(AXIS *,int);
extern void StarGetCubePixPosition(AXES *, int, int, int, double *, double *, double *);
extern double StarPixelIndexPositionOnAxis(AXIS *,double);
extern double StarPixelValuePositionOnAxis(AXIS *,double);

extern AXIS *StarCubeFindAxisByName(AXES *, char *);
extern AXIS *StarCubeFindAxisByType(AXES *, char *);

#endif /* endif of define _NO_PROTO */

#ifdef _NO_PROTO
extern void StarCopyCubeHead();
extern void StarCopyAxes();
extern void StarCopyAxis();
extern void StarCopyDatavalue();
extern int StarCopyCubePixel();
extern int StarCopyCubePixelData();
extern void StarClearCubePixel();

#else /* undef _NO_PROTO */
extern void StarCopyCubeHead(CUBE_HEAD *, CUBE_HEAD *);
extern void StarCopyAxes(AXES *, AXES *);
extern void StarCopyAxis(AXIS *, AXIS *);
extern void StarCopyDatavalue(DATA_VALUE *, DATA_VALUE *);
extern int StarCopyCubePixel(CUBE *, CUBE *);
extern int StarCopyCubePixelData(CUBE *, CUBE *);
extern void StarClearCubePixel(CUBE *);

#endif /* endif of define _NO_PROTO */


#ifdef _NO_PROTO
extern void StarCubeSmoothAndConvert();
extern void StarCubePixelRescale();
extern void StarCubeScanDatavalue();

#else
extern void StarCubeSmoothAndConvert(CUBE *, CUBE *,double *,int *);
extern void StarCubePixelRescale(CUBE *,double,double);
extern void StarCubeScanDatavalue(CUBE *);

#endif

/*----- for PIPE flow -----*/
#ifdef _NO_PROTO

extern int	StarPipeHeadIn();
extern int	StarPipeHeadOut();
extern int	StarPipeHeadLoad();
extern int	StarPipeHeadSave();

extern int	StarPipeEndIn();
extern int	StarPipeEndOut();
extern int	StarPipeEndLoad();
extern int	StarPipeEndSave();

extern int	StarFlowdataIdentify();

#else /* undef _NO_PROTO */

extern int	StarPipeHeadIn(char *, int *);
extern int	StarPipeHeadOut(int);
extern int	StarPipeHeadLoad(char *, int *, FILE *);
extern int	StarPipeHeadSave(char *, int, FILE *);

extern int	StarPipeEndIn(char *);
extern int	StarPipeEndOut();
extern int	StarPipeEndLoad(char *, FILE *);
extern int	StarPipeEndSave(char *, FILE *);

extern int	StarFlowdataIdentify(char *);

#endif /* endif of define _NO_PROTO */

/*----- for sdb -----*/
#ifdef _NO_PROTO
extern int	StarSdbIn();
extern int	StarSdbOut();
extern int	StarSdbLoad();
extern int	StarSdbSave();
extern int	StarSdbHeadin();
extern int	StarSdbHeadout();
extern int	StarSdbHeadLoad();
extern int	StarSdbHeadSave();

#else /* undef _NO_PROTO */
extern int	StarSdbIn(char *, int *, char *);
extern int	StarSdbOut(char *, int, char *);
extern int	StarSdbLoad(FILE *, char *, int *, char *);
extern int	StarSdbSave(FILE *, char *, int, char *);
extern int	StarSdbHeadIn(char *, int *, int *);
extern int	StarSdbHeadOut(char *, int, int);
extern int	StarSdbHeadLoad(FILE *, int *);
extern int	StarSdbHeadSave(FILE *, int);

#endif /* endif of define _NO_PROTO */

/*-------file names, parameter processing-------*/
#ifdef _NO_PROTO

extern int StarStdFileExtName();
extern int StarDoesFileExist();
extern int StarGetSavename();

#else /* undef _NO_PROTO */

extern int StarStdFileExtName(char *, char *, char *, char *);
extern int StarDoesFileExist(char *);
extern int StarGetSavename(int,char **, char *, char *);

#endif /* endif of define _NO_PROTO */

/*-------file backup processing-------*/
#ifdef _NO_PROTO
extern void StarGetBackupname();
extern int StarRename4backup();

#else /* undef _NO_PROTO */
extern void StarGetBackupname(char *, char *, int, int, char *);
extern int StarRename4backup(char *, char *);

#endif /* endif of define _NO_PROTO */

/*--------user-input-parameter processing---------*/
#ifdef _NO_PROTO
extern int	StarDivideSetOfWords();
extern void	StarDividePrmRecord();

extern char	**AllocWordsStrage();
extern char	**AllocAllWords();
extern void	FreeAllWords();
extern int	AddNewWord();

extern int	StarOptionFlag();
extern int	StarOptionKeyArgPosition();
extern int	StarOptionKeyPosition();
extern char	*StarOptionValue();
extern void	StarOptionRange();
extern void	StarOptionAngleRange();
extern int	StarOptionRanges();
extern int	StarOptionRanges_long();
extern void	StarOptionRange_long();
extern int	StarKeylessArgPosition();
extern char	*StarKeylessOption();
extern char	*StarKeylessMultiOption();

#else /* undef _NO_PROTO */
extern int	StarDivideSetOfWords(char *,char *,char **);
extern void	StarDividePrmRecord(char *, char *, char *);

extern char	**AllocWordsStrage(int);
extern char	**AllocAllWords(int, int);
extern void	FreeAllWords(char **, int);
extern int	AddNewWord(char **, char *);

extern int	StarOptionFlag(int, int *, int *);
extern int	StarOptionKeyArgPosition(int, int *, int *);
extern int	StarOptionKeyPosition(int, int *, int *);
extern char 	*StarOptionValue(int, int *, char **, int *, int *, char **);
extern void	StarOptionRange(char *, RANGE *);
extern void	StarOptionAngleRange(char *, RANGE *);
extern int	StarOptionRanges(char *, int *, RANGE *);
extern void	StarOptionRange_long(char *, RANGE_long *);
extern int	StarOptionRanges_long(char *, int *, RANGE_long *);
extern int	StarKeylessArgPosition(int, char **, int, int *, int *);
extern char	*StarKeylessOption(int, char **, int, int *, int *);
extern char	*StarKeylessMultiOption(int, char **, int, int *, int *);
#endif /* endif of define _NO_PROTO */

#ifdef _NO_PROTO
extern int	StarGetOption();
extern int	StarGetPrm();
extern void	StarFreeAllPrms();

#else /* undef _NO_PROTO */

extern int	StarGetOption(int, char **, char *, int, char **, int *, char **);
extern int	StarGetPrm(FILE *, int, char **, int *, char **);
extern void	StarFreeAllPrms(int, int *, char **);

#endif /* endif of define _NO_PROTO */


#ifdef _NO_PROTO
extern int	StarInRange();
extern int	StarInLoopRange();
extern int	StarInRange_long();
extern int	StarInLoopRange_long();
extern double	StarStdAngle2deg();
extern void	Stardeg2StdAngle();

extern void	StarReduceString();
extern void	StarAdjustStrlen();

extern int StarIsInLongitudeRange();
extern int StarIsLeapYear();
extern long StarYmd2MJD();

#else /* undef _NO_PROTO */
extern int	StarInRange(double, RANGE);
extern int	StarInLoopRange(double, RANGE);
extern int	StarInRange_long(long, RANGE_long);
extern int	StarInLoopRange_long(long, RANGE_long);
extern double	StarStdAngle2deg(char *);
extern void	Stardeg2StdAngle(char *, double, int);

extern void	StarReduceString(char *);
extern void 	StarAdjustStrlen(char *, int, char);

extern int StarIsInLongitudeRange(RANGE *, double);
extern int StarIsLeapYear(int);
extern long StarYmd2MJD(int,int,int);

#endif /* endif of define _NO_PROTO */

/*--------general data processing---------*/
#ifdef _NO_PROTO
extern double	ipow();
extern void	LeastSquareFitFloat();
extern void	LeastSquareFit();
extern void	StarAutoScale();

#else /* undef _NO_PROTO */
extern double	ipow(float, int);
extern void	LeastSquareFitFloat(float *, float *, int, int, float *);
extern void	LeastSquareFit(double *, double *, int, int, double *);
extern void	StarAutoScale(RANGE, RANGE *, int *, double *);

#endif /* endif of define _NO_PROTO */

/*--------data sorting---------*/
#ifdef _NO_PROTO
extern void	StarSortRange();
extern void	StarSortRangeLong();
extern void	StarSortBoxRange();
extern void	StarSortBoxRangeLong();

#else /* undef _NO_PROTO */
extern void	StarSortRange(RANGE *);
extern void	StarSortRangeLong(RANGE_long *);
extern void	StarSortBoxRange(BOX_RANGE *);
extern void	StarSortBoxRangeLong(BOX_RANGE_long *);

#endif /* endif of define _NO_PROTO */

/*--------date/time conversion---------*/
#ifdef _NO_PROTO
extern int StarIsLeapYear();
extern long StarYmd2MJD();
extern void StarMJD2Ymd();
extern long StarFitsDate2MJD();
extern long StarFitsDate2Time1970();

#else
extern int StarIsLeapYear(int);
extern long StarYmd2MJD(int,int,int);
extern void StarMJD2Ymd(int *, int *, int *, long);
extern long StarFitsDate2MJD(char *);
extern long StarFitsDate2Time1970(char *);

#endif

/*--------c-string process---------*/
#ifdef _NO_PROTO
extern char *StarFileMainName();
extern int StarExtractPartialFilename();
extern void StarReduceBlankInCstrings();
extern int is_extra_visible_char();
extern int StarCheckCharType();
extern void minstrcpy();

#else
extern char *StarFileMainName(char *);
extern int StarExtractPartialFilename(char *,char *,int);
extern void StarReduceBlankInCstrings(char *);
extern int is_extra_visible_char(char);
extern int StarCheckCharType(char);
extern void minstrcpy(char *,char *,int);
#endif

/*--------time cotrol---------*/
#ifdef _NO_PROTO
extern long StarWaitMillisec();

#else
extern long StarWaitMillisec(long);

#endif

/*----------for Astronomical Calculations---------*//* celestial calculation */
/* astronomical converters */
#ifdef _NO_PROTO
extern int month_name2number();
extern int isleapyear();
extern int dayfromJan00();
extern int fromJan002date();
extern long ymd2mjd();
extern int mjd2ymd();

extern long mjd2ephem();
extern long ephem2mjd();
extern long mjd2time1970();

extern long month_number(); /* should be replaced by month_name2number() */
extern long date_and_time2utc();
extern long ctime2utc();
extern void ctime2date_and_time();

extern double stod();

#else /* undef _NO_PROTO */
extern int month_name2number(char *);
extern int isleapyear(int);
extern int dayfromJan00(int,int,int);
extern int fromJan002date(int,int,int *,int *);
extern long ymd2mjd(int,int,int);
extern int mjd2ymd(long, int *, int *, int *);

extern long mjd2ephem(long);
extern long ephem2mjd(long);
extern long mjd2time1970(long, long);

extern long month_number(char *);
extern long date_and_time2utc(DATE_AND_TIME);
extern long ctime2utc(char *);
extern void ctime2date_and_time(char *, DATE_AND_TIME *);

extern double stod(char *);

#endif /* endif of define _NO_PROTO */
