/*
Include files for Lineproc data conversion programs

1995/8/22 T.Handa
*/

#define MAX_HIST 50

#define EBC_char	unsigned char
typedef struct {unsigned char msp_byte[2];} MSP_short;
typedef struct {unsigned char msp_byte[4];} MSP_long;
typedef struct {unsigned char msp_byte[4];} MSP_float;
typedef struct {unsigned char msp_byte[8];} MSP_double;


/* MSP definitions */
typedef struct {
	EBC_char	id;	/* lineproc file ID '4' in EBCDIC code fixed */
	EBC_char	scan_mode;	/* M=multi-onoff, O=onoff */
	EBC_char	calib_mode;	/* R=R-sky, H=hot-cold */
	MSP_short	band;	/* band ID number */
	EBC_char	group_id[7];	/* observation Group ID */
	EBC_char	observer_name[12];	/* observer name */
	EBC_char	operator_name[12];	/* telescope operator name */
	EBC_char	unknown[6];
	MSP_short	integ_period;	/* integration period (sec) */
	MSP_short	be_active_time;	/* BackEnd active time (sec) */
	MSP_short	obs_year;	/* obs. year */
	MSP_short	obs_date;	/* obs. date from Jan. 1st */
	MSP_short	l_history;	/* No. of lines of history */
	MSP_long	start_time;	/* integ start time (sec) */
	MSP_long	stop_time;	/* integ stop time (sec) */
	MSP_long	process_start;	/* process start time (sec) */
	EBC_char	object_name[12];	/* object name */
	MSP_short	array_ch;	/* No. of array chanels? */
	char		dummy[2];	
	} LP_IDENTIFY;

typedef	struct {
	MSP_double	center_x;	/* tracking center (longitude) (rad) */
	MSP_double	center_y;	/* tracking center (latitude)  (rad) */
	MSP_double	on_x;	/* offset of source (long.) (rad) */
	MSP_double	on_y;	/* offset of source (lat.)  (rad) */
	MSP_double	off_x;	/* offset of reference pos. (long.) (rad) */
	MSP_double	off_y;	/* offset of reference pos. (lat.)  (rad) */
	MSP_double	correct_x;	/* offset position corrected (rad) */
	MSP_double	correct_y;
	EBC_char	coordinate_c;	/* coordinates R=RA-Dec, D=DlRA-Dec, A=AzEl, G=l-b */
	EBC_char	coordinate_on;
	EBC_char	coordinate_off;
	EBC_char	coordinate_corr;
	MSP_float	factor;	/* normarization factor */
	EBC_char	norm_id;	/* S=norm. factor set,  =norm. factor unset */
	char		dummy[7];	
	} LP_POSITION;

typedef struct {
	MSP_double	weight;	/* weight of profile */
	MSP_double	integ_time;	/* total integ time (sec) */
	EBC_char	mt_name[10];	/* MT name or obs. table name */
	MSP_short	whead_isig;	/* WHEAD.ISIG parameter (?) */
	MSP_float	rms;	/* RMS noise level */
	EBC_char	unknown1[4];
	MSP_float	baseline_prm[8];
	EBC_char	unknown2[6];
	char		dummy[6];
	} LP_Q_PROF;

typedef struct {
	MSP_double	trk_freq;	/* tracking frequency (GHz) */
	MSP_double	array_c_freq;	/* array center frequency (MHz) */
	MSP_float	v_lsr;	/* LSR velocity km/s */
	MSP_float	v_nro;	/* radial velocity of NRO (km/s) */
	EBC_char	array_name[2];	/* AOS array name */
	MSP_short	array_numb;	/* AOS array ID-number */
	MSP_short	processer_numb;	/* AOS array processer ID-number */
	MSP_short	side_band;	/* sideband mode &0x8000=0:SSB =1:DSB, &0x4000=0:LSB =1:USB */
	EBC_char	aos_cal_name[8];	/* AOS freq. cal. data member name */
	MSP_float	v_trk;	/* tracking LSR velocity (km/s) */
	char	unknown1[4];
	MSP_float	first_if;	/* first IF freq. (GHz) */
	char	unknown2[4];
	EBC_char	rx_name[3];	/* receiver name */
	char		dummy[21];
	} LP_AOS;


typedef struct {
	MSP_short	n;	/* No. of calibration scans */
	MSP_short	scan[39];	/* calib scan numbers */
	} LP_CALIB;

typedef struct {
	MSP_short	n;	/* No. of bad scans */
	MSP_short	scan[39];	/* bad scan numbers */
	} LP_BADSCAN;

typedef struct {

	LP_IDENTIFY	id;	/* file ID informations */
	EBC_char	scan_comment[80];	/* comment of scan */
	LP_POSITION	pos;	/* observation positions */
	EBC_char	aos_comment[80];	/* comment of AOS */
	LP_Q_PROF	prof_q;	/* profile quality */
	LP_AOS		aos;	/* AOS parameters */
	LP_CALIB	calib;	/* calib scans */
	LP_BADSCAN	bad[2];	/* bad scans */

	} LINEPROC_HEADER;

/* UNIX definitions */
typedef struct {
	char	id;	/* lineproc file ID '4' fixed */
	char	scan_mode;	/* M=multi-onoff, O=onoff */
	char	calib_mode;	/* R=R-sky, H=hot-cold */
	short	band;	/* band ID number */
	char	group_id[8];	/* observation Group ID */
	char	observer_name[13];	/* observer name */
	char	operator_name[13];	/* telescope operator name */
	short	integ_period;	/* integration period (sec) */
	short	be_active_time;	/* BackEnd active time (sec) */
	short	obs_year;	/* obs. year */
	short	obs_date;	/* obs. date from Jan. 1st */
	short	l_history;	/* No. of lines of history */
	long	start_time;	/* integ start time (sec) */
	long	stop_time;	/* integ stop time (sec) */
	long	process_start;	/* process start time (sec) */
	char	object_name[13];	/* object name */
	short	array_ch;	/* No. of array chanels? */
	} LPunix_IDENTIFY;

typedef	struct {
	double	center_x;	/* tracking center (longitude) (rad) */
	double	center_y;	/* tracking center (latitude)  (rad) */
	double	on_x;	/* offset of source (long.) (rad) */
	double	on_y;	/* offset of source (lat.)  (rad) */
	double	off_x;	/* offset of reference pos. (long.) (rad) */
	double	off_y;	/* offset of reference pos. (lat.)  (rad) */
	double	correct_x;	/* offset position corrected (rad) */
	double	correct_y;
	char	coordinate_c;	/* coordinates R=RA-Dec, D=DlRA-Dec, A=AzEl, G=l-b */
	char	coordinate_on;
	char	coordinate_off;
	char	coordinate_corr;
	float	factor;	/* normarization factor */
	char	norm_id;	/* S=norm. factor set,  =norm. factor unset */
	} LPunix_POSITION;

typedef struct {
	double	weight;	/* weight of profile */
	double	integ_time;	/* total integ time (sec) */
	char	mt_name[11];	/* MT name or obs. table name */
	short	whead_isig;	/* WHEAD.ISIG parameter (?) */
	float	rms;	/* RMS noise level */
	float	baseline_prm[8];
	} LPunix_Q_PROF;

typedef struct {
	double	trk_freq;	/* tracking frequency (GHz) */
	double	array_c_freq;	/* array center frequency (MHz) */
	float	v_lsr;	/* LSR velocity km/s */
	float	v_nro;	/* radial velocity of NRO (km/s) */
	char	array_name[3];	/* AOS array name */
	short	array_numb;	/* AOS array ID-number */
	short	processer_numb;	/* AOS array processer ID-number */
	short	side_band;	/* sideband mode 32767=SSB.USB, 1=SSB.LSB, -1=DSB.USB, -16385=DSB.LSB */
	char	aos_cal_name[9];	/* AOS freq. cal. data member name */
	float	v_trk;	/* tracking LSR velocity (km/s) */
	float	first_if;	/* first IF freq. (GHz) */
	char	rx_name[4];	/* receiver name */
	} LPunix_AOS;

typedef struct {
	short	n;	/* No. of calibration scans */
	short	scan[39];	/* calib scan numbers */
	} LPunix_CALIB;

typedef struct {
	short	n;	/* No. of bad scans */
	short	scan[78];	/* bad scan numbers */
	} LPunix_BADSCAN;

typedef struct {

	LPunix_IDENTIFY	id;	/* file ID informations */
	char	scan_comment[81];	/* comment of scan */
	LPunix_POSITION	pos;	/* observation positions */
	char	aos_comment[81];	/* comment of AOS */
	LPunix_Q_PROF	prof_q;	/* profile quality */
	LPunix_AOS		aos;	/* AOS parameters */
	LPunix_CALIB	calib;	/* calib scans */
	LPunix_BADSCAN	bad;	/* bad scans */

	} LINEPROCunix_HEADER;

/* prototype declarations */

/* MSP-UNIX conversions */
#ifdef _NO_PROTO
void str_ebc2asc();
extern char ebcdic2ascii();
extern short msp2unix_short();
extern long msp2unix_long();
extern float msp2unix_float();
extern double msp2unix_double();
#else /* undef _NO_PROTO */
void str_ebc2asc(char *, EBC_char *, int);
extern char ebcdic2ascii(EBC_char);
extern short msp2unix_short(MSP_short);
extern long msp2unix_long(MSP_long);
extern float msp2unix_float(MSP_float);
extern double msp2unix_double(MSP_double);
#endif /* endif of define _NO_PROTO */


/* LINEPROC readers */
#ifdef _NO_PROTO
extern int ReadLineprocHeader();
extern int ReadLineprocHistory();
extern int ReadLineprocProfData();
#else /* undef _NO_PROTO */
extern int ReadLineprocHeader(LINEPROCunix_HEADER *, FILE *);
extern int ReadLineprocHistory(short, char [MAX_HIST][81], FILE *);
extern int ReadLineprocProfData(float *, float *, FILE *);
#endif /* endif of define _NO_PROTO */

/* LINEPROC freq calib */
#ifdef _NO_PROTO
int calc_aos_prm();
int get_combdata();
int get_combdata_file();
#else /* undef _NO_PROTO */
int calc_aos_prm(PROFILE *, char *);
int get_combdata(long, char *, char *, int *, double *, double *, double *);
int get_combdata_file(char *, long, char *);
#endif /* endif of define _NO_PROTO */
