/*
library for conversion between newstar Spectrum Record and UltraSTAR scn data

1999/ 7/27 T.Handa (create) for SSC format
2004/11/22 T.Handa (revise) NSprofile is extent for SSD format
 */

#define TYPE_DOUBLE -8
#define TYPE_FLOAT -4
#define TYPE_LONG 4
#define TYPE_SHORT 2
#define TYPE_CHAR 1

#ifndef BIG_ENDIAN
#define BIG_ENDIAN 4321
#endif
#ifndef LITTLE_ENDIAN
#define LITTLE_ENDIAN 1234
#endif
#ifndef PDP_ENDIAN
#define PDP_ENDIAN 3412
#endif

#define	UTC_JST	(9)

#define LENGTH_SPECTRUM 10240
#define CBFILE_SIZE 2048

/*===for CB file===*/

typedef struct {
  char filename[13];
  char filetype[7];
  long n_sp;
  long max_ch;
} NS_CBdata;

/*===for SS file===*/

typedef struct {
  float x;
  float y;
  float z1;
  float z2;
} SUBREF_OFFSET; // 45m subreflector position [mm]
typedef struct {
  float speed;
  float direction;
} WIND_PRM; // wind condition

typedef struct	{
  XY_POS radec;
  XY_POS lb;
  XY_POS xy;
  XY_POS azel;
} POSITION;
typedef struct	{
  long n_range;
  long start[6];
  long stop[6];
} NS_CH_RANGES;

typedef	struct	{
  char schedule[3][9];
  char chapter[3][9];
  char observer[3][13];
  long start_date[3];
  long start_time[3];
  long stop_date[3];
  long stop_time[3];
  char sp_name[13];
  char version[5];
} NS_SYSHEAD;
typedef struct	{
  char name[17];
  char epoch[6];
  POSITION obj;
  char v_reference[5];
  char v_definition[5];
  double v;
  double v_radial;		
  char xy_coordinate[3];
  double xy_pa;
  POSITION d_on;
  POSITION on;
  POSITION d_off[2];
  POSITION off[2];
  float d_offbeam;
} NS_OBJECT;
typedef struct	{
  float hpbw;
  float eta_app;
  float eta_mb;
  float eta_ant;
  float eta_fss;
  float gain;
  float eta_couple;
  SUBREF_OFFSET subref_offset;
  XY_POS pointing_corr;
} NS_ANT;
typedef struct	{
  char name[9];
  char feed[3];
  double f_track;
  double first_if;
  char sideband[5];
  float ssb_ratio;
  float t_rx;
  float freq_fsw;
  float d_freq_fsw;
} NS_RX;
typedef struct	{
  char name[5];
  char function_table[9];
  long ch;
  float bandwidth;
  float resolution;
  float separation;
  double freq_rest;
  double center_freq;
  long n_freq_calib;
  double freq_calib[10];
  float ch_calib_center[10];
  float ch_calib_width[10];
  float t_hot;
} NS_BE;
typedef struct	{
  long total_integ_time;
  long n_orig_sp;
  double noise_level;
  NS_CH_RANGES rms;
  char basefit_function[5];
  long basefit_dimension;
  NS_CH_RANGES basefit;
  float t_sys;
  float tau_atm;
  float norm;
  F_METEO_PRM meteo;
  WIND_PRM wind;
  float tb_atm;
  char switch_mode[6];
} NS_DATA_STATUS;
typedef struct {
  double alc_time; /* ALC time constant */
  double alc_v; /* ALC control voltage */
  double norm4multrx; /* scaling factor for the multibeam RX */
  char lag_window[9]; /* AC45 lag window name [NONE,HANN,HAMM,BLCK] */
} NS_AC45_STATUS;

typedef struct	{
  NS_SYSHEAD NSsyshead;
  NS_OBJECT NSobj;
  NS_ANT NSant;
  NS_RX NSrx;
  NS_BE NSbe;
  NS_DATA_STATUS NSstatus;
  NS_AC45_STATUS NSac45status;
  char NShistory[10][81],comment[81];
  float data[2048];
} NS_PROFILE;

#ifdef _NO_PROTO
extern void XYrad2deg();
extern long hhmmss2sec();

extern int get_NSprofile();
extern void get_NS_CBfile();

extern double convert_endian_double();
extern float convert_endian_float();
extern long convert_endian_long();
extern short convert_endian_short();

#else
extern void XYrad2deg(XY_POS *);
extern long hhmmss2sec(long);

extern int get_NSprofile(NS_PROFILE *, int, int, FILE *);
extern void get_NS_CBfile(NS_CBdata *,int, FILE *);

extern double convert_endian_double(int,char *);
extern float convert_endian_float(int,char *);
extern long convert_endian_long(int,char *);
extern short convert_endian_short(int,char *);
#endif

#ifndef PI
#define	PI	(3.14159265358979323846)
#endif
