/*
STAR cube data I/O functions
Part 2. only for Cube data

1998/11/28 T.Handa
2000/1/26 T.Handa
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#include "StDataIO.c"

#define read_HeadSys(x,y) read_sys_head(x,y)
#define read_HeadObs(x,y,z) read_obs_prm(x,y,z)
#define read_HeadObj(x,y,z) read_object(x,y,z)
#define read_HeadAnt(x,y,z) read_ant_status(x,y,z)
#define read_HeadRx(x,y,z) read_rx_status(x,y,z)
#define read_HeadBe(x,y,z) read_be_status(x,y,z)
#define read_HeadLog(x,y,z) read_obs_log(x,y,z)
#define read_HeadHistory(x,y,z) read_history(x,y,z)
#define write_HeadSys(x,y) write_sys_head(x,y)
#define write_HeadObs(x,y,z) write_obs_prm(x,y,z)
#define write_HeadObj(x,y,z) write_object(x,y,z)
#define write_HeadAnt(x,y,z) write_ant_status(x,y,z)
#define write_HeadRx(x,y,z) write_rx_status(x,y,z)
#define write_HeadBe(x,y,z) write_be_status(x,y,z)
#define write_HeadLog(x,y,z) write_obs_log(x,y,z)
#define write_HeadHistory(x,y,z) write_history(x,y,z)

int	StarCubeIn(cube,sdb_flag,sdb_arg)
/*
Pipe-in a cube from stdin with auto allocation of cube_pixel

return
0 : normal
1 : garbage cube data
101 : too many history steps
102 : too many lines in a history step
111 : not "END" nor "end" for the last history step
900 : cannot alloc cube_pixel;
999 : invalid format data

1998/11/26 T.Handa ver.3.2 (create) from StarProfileIn() ver.3.2
*/
CUBE *cube;
int *sdb_flag;
char *sdb_arg;
{
  int	err_code,flow_code,l_file_name;
  char	*ret_char;
  char	buffer[MAX_IDX_RECORD_LENGTH];
  int	l_str;

/* get data idnetifier */
  ret_char=fgets(buffer,MAX_IDX_RECORD_LENGTH,stdin);
  if(ret_char==NULL) {
    (*cube).head.sys.l_file=0;
    *sdb_flag=BLANK_FLOW;
    *sdb_arg=(char)NULL;
    return(1);
  }
  buffer[MAX_IDX_RECORD_LENGTH-1]=(char)NULL;
  l_str=strlen(buffer);
  if(buffer[l_str-1]=='\n') buffer[l_str-1]=(char)NULL;

  flow_code=StarFlowdataIdentify(buffer);

  switch(flow_code) {
  case CUB_FLOW:
    break;
  default:
    fprintf(stderr,"not .cub. ID=%s\n",buffer);
    return(999);
  }

/* cube flow */
  strncpy((*cube).head.file_id,buffer,8);
  (*cube).head.file_id[8]=(char)NULL;

/* get file name */
  ret_char=fgets(&((*cube).head.file_name[0]),MAX_FILE_NAME_LENGTH,stdin);
  if(ret_char==NULL) return(999);
  l_file_name=strlen((*cube).head.file_name)-1;
  (*cube).head.file_name[l_file_name]=(char)NULL;

/* get cube flag and argument */
  ret_char=fgets(buffer,MAX_IDX_RECORD_LENGTH,stdin);
  if(ret_char==NULL) return(999);
  *sdb_flag=atoi(buffer);

  ret_char=fgets(sdb_arg,MAX_IDX_RECORD_LENGTH,stdin);
  if(ret_char==NULL) return(999);
  l_str=strlen(sdb_arg);
  if(sdb_arg[l_str-1]=='\n') sdb_arg[l_str-1]=(char)NULL;

  if(*sdb_flag==GOOD_SCAN || *sdb_flag==BAD_SCAN) {
/* get cube itself */
    err_code=StarCubeHeadLoad(stdin,&((*cube).head));
    if(err_code!=0) return(err_code);
    (*cube).pixel=StarAllocCubePixel(&((*cube).head));
    if((*cube).pixel==NULL) return(900);
    err_code=StarCubePixelLoad(stdin,cube);
    return(err_code);
  } else {
    return(0);
  }
}

int	StarCubeOut(cube,sdb_flag,sdb_arg)
/*
Pipe-out a cube from stdin without cube_pixel free
(Do StarFreeCubePixel() before end of the main(). )

return
0 : normal
1 : garbage cube data
999 : invalid format data

1998/11/26 T.Handa ver.3.2 (create) from StarProfileOut() ver.3.2
*/
CUBE *cube;
int sdb_flag;
char *sdb_arg;
{
  int	err_code,flow_code;
  int	l_str;

/* put data idnetifier */
  flow_code=StarFlowdataIdentify((*cube).head.file_id);
  switch(flow_code) {
  case CUB_FLOW:
    break;
  default:
    printf("NGID\n");
    return(999);
  }
  printf("%s\n",(*cube).head.file_id);

/* put file name */
  printf("%s\n",(*cube).head.file_name);

/* put cube flag and argument */
  l_str=strlen(sdb_arg);
  if(l_str>1 && sdb_arg[l_str-1]=='\n') sdb_arg[l_str-1]=(char)NULL;
  printf("%d\n%s\n",sdb_flag,sdb_arg);

  if(sdb_flag==GOOD_SCAN || sdb_flag==BAD_SCAN) {
/* get cube itself */
    err_code=StarCubeHeadSave(stdout,&((*cube).head));
    if(err_code!=0) return(err_code);
    err_code=StarCubePixelSave(stdout,cube);
    return(err_code);
  } else {
    return(0);
  }
}

int StarCubeHeadLoad(fp,head)
FILE *fp;
CUBE_HEAD *head;
/*
Load Header of the CUBE
1997/5/16 T.Handa 75byte for Cube-ver1
*/
{
  int	err_code;
  int	ii,n_hist;

  err_code=read_HeadSys(fp,&((*head).sys));
  if(err_code!=0) return(err_code);
  n_hist=(*head).sys.hist_step;
  err_code=read_HeadObs(fp,&((*head).sys),&((*head).obs));
  if(err_code!=0) return(err_code);
  err_code=read_HeadObj(fp,&((*head).sys),&((*head).object));
  if(err_code!=0) return(err_code);
  err_code=read_HeadAnt(fp,&((*head).sys),&((*head).ant));
  if(err_code!=0) return(err_code);
  err_code=read_HeadRx(fp,&((*head).sys),&((*head).rx));
  if(err_code!=0) return(err_code);
  err_code=read_HeadBe(fp,&((*head).sys),&((*head).be));
  if(err_code!=0) return(err_code);
  err_code=read_HeadLog(fp,&((*head).sys),&((*head).log));
  if(err_code!=0) return(err_code);
  err_code=read_HeadAxes(fp,&((*head).sys),&((*head).axis));
  if(err_code!=0) return(err_code);
  err_code=read_HeadDValue(fp,&((*head).sys),&((*head).data_value));
  if(err_code!=0) return(err_code);
  for(ii=0;ii<=n_hist;ii++) {
    err_code=read_HeadHistory(fp,&((*head).sys),&((*head).history[ii]));
  }
  if(err_code!=0) return(err_code);

	return(err_code);
}

int StarCubeHeadSave(fp,head)
FILE *fp;
CUBE_HEAD *head;
/*
Save Header of the CUBE
1997/5/16 T.Handa 75byte for Cube-ver1
*/
{
  int	err_code;
  int	ii,n_hist;

  err_code=write_HeadSys(fp,&((*head).sys));
  if(err_code!=0) return(err_code);
  n_hist=(*head).sys.hist_step;
  err_code=write_HeadObs(fp,&((*head).sys),&((*head).obs));
  if(err_code!=0) return(err_code);
  err_code=write_HeadObj(fp,&((*head).sys),&((*head).object));
  if(err_code!=0) return(err_code);
  err_code=write_HeadAnt(fp,&((*head).sys),&((*head).ant));
  if(err_code!=0) return(err_code);
  err_code=write_HeadRx(fp,&((*head).sys),&((*head).rx));
  if(err_code!=0) return(err_code);
  err_code=write_HeadBe(fp,&((*head).sys),&((*head).be));
  if(err_code!=0) return(err_code);
  err_code=write_HeadLog(fp,&((*head).sys),&((*head).log));
  if(err_code!=0) return(err_code);
  err_code=write_HeadAxes(fp,&((*head).sys),&((*head).axis));
  if(err_code!=0) return(err_code);
  err_code=write_HeadDValue(fp,&((*head).sys),&((*head).data_value));
  if(err_code!=0) return(err_code);
  for(ii=0;ii<=n_hist;ii++) {
    err_code=write_HeadHistory(fp,&((*head).sys),&((*head).history[ii]));
  }
  if(err_code!=0) return(err_code);

	return(err_code);
}

int StarCubePixelLoad(fp,cube)
/*
Load PixelData of the CUBE to fp
1998/11/18 T.Handa
*/
FILE *fp;
CUBE *cube;
{
  int jj;
  long nx,ny,nz;
  size_t total_pixel,read_pixel;

  total_pixel=0;

  nx=(long)(*cube).head.axis.x.size;
  ny=(long)(*cube).head.axis.y.size;
  nz=(long)(*cube).head.axis.z.size;

#ifdef INTEL_CPU
if(fp==stdin) {
#endif
  for(jj=0;jj<nx;jj++) {
    read_pixel=fread((*cube).pixel[jj],sizeof(float),ny*nz,fp);
    total_pixel=total_pixel+read_pixel;
    if(read_pixel!=ny*nz) break;
  }
#ifdef INTEL_CPU
} else {
  for(jj=0;jj<nx;jj++) {
    read_pixel=freadIntel((*cube).pixel[jj],sizeof(float),ny*nz,fp);
    total_pixel=total_pixel+read_pixel;
    if(read_pixel!=ny*nz) break;
  }
}
#endif

  if((long)total_pixel==nx*ny*nz) return(0);

  if(total_pixel==0) total_pixel= -1;
  return((int)total_pixel);
}

int StarCubePixelSave(fp,cube)
/*
Save PixelData of the CUBE to fp
1998/11/18 T.Handa
*/
FILE *fp;
CUBE *cube;
{
  int jj;
  long nx,ny,nz;

  size_t total_pixel,written_pixel;

  total_pixel=0;

  nx=(long)(*cube).head.axis.x.size;
  ny=(long)(*cube).head.axis.y.size;
  nz=(long)(*cube).head.axis.z.size;

#ifdef INTEL_CPU
if(fp==stdout) {
#endif
  for(jj=0;jj<nx;jj++) {
    written_pixel=fwrite((*cube).pixel[jj],sizeof(float),ny*nz,fp);
    total_pixel=total_pixel+written_pixel;
    if(written_pixel!=ny*nz) break;
  }
#ifdef INTEL_CPU
} else {
  for(jj=0;jj<nx;jj++) {
    written_pixel=fwriteIntel((*cube).pixel[jj],sizeof(float),ny*nz,fp);
    total_pixel=total_pixel+written_pixel;
    if(written_pixel!=ny*nz) break;
  }
}
#endif

  if((long)total_pixel==nx*ny*nz) return(0);

  if(total_pixel==0) total_pixel= -1;
  return((int)total_pixel);
}

int	StarAddHistory4Cube(cube_head,history)
/*
Add a new "HISTORY" entry to the cube_head.
END entry can be added by this function.
Automatically add the "END" entry.
0   normal end
100 too many history steps

1992/12/12 T.Handa
1998/9/9 T.Handa (revised) from StarAddHistory();
*/
CUBE_HEAD *cube_head;
HISTORY *history;
{
  int	lines;
  long	pres_time;
  int	current_step;
  int	add_size,add_byte;

  current_step=(*cube_head).sys.hist_step;

/* ----correct increasing the sizes---- */
/* previous last step is "END" --> adjust sizes */
  if(strcmp((*cube_head).history[current_step].step,"END")==0 ||
     strcmp((*cube_head).history[current_step].step,"end")==0) {
    (*cube_head).sys.hist_line--;
    (*cube_head).sys.l_file-=16;
    (*cube_head).sys.l_head-=16;
  }

  add_size=(*history).size+1;
  add_byte=add_size*16;

  (*cube_head).sys.hist_line+=add_size;
  (*cube_head).sys.l_file+=add_byte;
  (*cube_head).sys.l_head+=add_byte;

/* increase history step in normal history addition */
  if(strcmp((*history).step,"END")!=0 && strcmp((*history).step,"end")!=0) {
    (*cube_head).sys.hist_step++;
  } else {
    (*history).size=0;
  }

/* too many history steps */
  if((*cube_head).sys.hist_step>=MAX_HISTORY_STEP)
    return(100);

/* copy history to cube_head.history */
  (*cube_head).history[current_step].size=(*history).size;
  (*history).step[7]=(char)NULL;
  strcpy((*cube_head).history[current_step].step,(*history).step);
  time(&pres_time);
  (*cube_head).history[current_step].time=pres_time;
  for(lines=0;lines<(*history).size;lines++) {
    (*history).prm[lines][16]=(char)NULL;
    strcpy((*cube_head).history[current_step].prm[lines],(*history).prm[lines]);
  }

/* add history is alread "END" */
  if(strcmp((*history).step,"END")==0 || strcmp((*history).step,"end")==0)
    return(0);

/* ---add "END" step--- */

  (*cube_head).sys.hist_line++;
  (*cube_head).sys.l_file+=16;
  (*cube_head).sys.l_head+=16;
  current_step=(*cube_head).sys.hist_step;
/* too many history steps */
  if((*cube_head).sys.hist_step>=MAX_HISTORY_STEP)
    return(100);

  (*cube_head).history[current_step].size=0;
  strcpy((*cube_head).history[current_step].step,"END");	
  (*cube_head).history[current_step].time=pres_time;

  return(0);
}

void StarAllocCube(cube)
/*
alloc cube pixel automatically
1998/11/5 T.Handa
*/
CUBE *cube;
{
  int nx,ny,nz;

  nx=(*cube).head.axis.x.size;
  ny=(*cube).head.axis.y.size;
  nz=(*cube).head.axis.z.size;

  (*cube).pixel=StarAlloc2DFloat(nx,ny*nz);

  if((*cube).pixel==NULL) fprintf(stderr,"**ERR** StarAllocCube() failed.\n");

  return;
}

CUBE_PIXEL StarAllocCubePixel(cube_head)
/*
alloc cube pixel array
1998/11/5 T.Handa
*/
CUBE_HEAD *cube_head;
{
  int nx,ny,nz;
  CUBE_PIXEL data;

  nx=(*cube_head).axis.x.size;
  ny=(*cube_head).axis.y.size;
  nz=(*cube_head).axis.z.size;

  data=StarAlloc2DFloat(nx,ny*nz);

  if(data==NULL) fprintf(stderr,"**ERR** StarAllocCubePixel() failed.\n");

  return(data);
}

void StarFreeCubePixel(cube)
/*
free data array
1998/11/5 T.Handa
*/
CUBE *cube;
{
  int nx,ny,nz;
  CUBE_PIXEL data;

  nx=(*cube).head.axis.x.size;
  ny=(*cube).head.axis.y.size;
  nz=(*cube).head.axis.z.size;
  data=(*cube).pixel;

  StarFree2DFloat(data,nx);
}

float **StarAlloc2DFloat(ll,mm)
/*
alloc 2D float array
1998/10/18 T.Handa
*/
int ll,mm;
{
  float **data;
  int ii;

  data=(float **)malloc(sizeof(float *)*ll);
  for(ii=0;ii<ll;ii++) {
    data[ii]=(float *)malloc(sizeof(float)*mm);
    if(data[ii]==NULL) return(NULL);
  }
  return(data);
}

void StarFree2DFloat(data,ll)
float **data;
int ll;
/*
Free 2D float array
1998/10/18 T.Handa
*/
{
  int ii;

  for(ii=0;ii<ll;ii++) {
    free(data[ii]);
  }
  free(data);
}

/* pixel data I/O */
static void put_1d_float(data,iz,value)
float *data;
int iz;
float value;
/*
internal function for 1D float I/O
1998/11/4 T.Handa
*/
{
  data[iz]=value;
}

static float get_1d_float(data,iz)
float *data;
int iz;
/*
internal function for 1D float I/O
1998/11/4 T.Handa
*/
{
  float value;

  value=data[iz];
  return(value);
}

void StarPutCubePixel(cube,ix,iy,iz,value)
/*
Put pixel value to CUBE_PIXEL data
1998/11/5 T.Handa
*/
CUBE *cube;
int ix,iy,iz;
float value;
{
  int ny;

  ny=(*cube).head.axis.y.size;
  put_1d_float((*cube).pixel[ix],iy+iz*ny,value);

  return;
}

float StarGetCubePixel(cube,ix,iy,iz)
/*
Get pixel value to CUBE_PIXEL data
1998/11/5 T.Handa
*/
CUBE *cube;
int ix,iy,iz;
{
  float value;
  int ny;

  ny=(*cube).head.axis.y.size;
  value=get_1d_float((*cube).pixel[ix],iy+iz*ny);
  return(value);
}

void StarCubePixelClear(cube,value)
/*
clear cube pixel by an asigned value

1999/9/29 T.Handa
*/
CUBE *cube;
float value;
{
  int ix,iy,iz;
  int max_ix,max_iy,max_iz;

  max_ix=(*cube).head.axis.x.size;
  max_iy=(*cube).head.axis.y.size;
  max_iz=(*cube).head.axis.z.size;

  for(ix=0;ix<max_ix;ix++) {
    for(iy=0;iy<max_iy;iy++) {
      for(iz=0;iz<max_iz;iz++) {
	StarPutCubePixel(cube,ix,iy,iz,value);
      }
    }
  }
  return;
}


int StarSaveFitsPixel(fp,cube)
/*
Write pixel value in FITS format
1998/12/13 T.Handa
*/
FILE *fp;
CUBE *cube;
{
  int ix,iy,iz;
  int nx,ny,nz;
  int blank_byte,total_byte;
  int ii;
  char *blank;

  size_t written_pixel,written;

  float data,dummy;
  static long nan=0xffffffff; /* IEEE NaN */

  nx=(*cube).head.axis.x.size;
  ny=(*cube).head.axis.y.size;
  nz=(*cube).head.axis.z.size;
  dummy=(*cube).head.data_value.dummy;

  written_pixel=0;

  for(iz=0;iz<nz;iz++) {
    for(iy=0;iy<ny;iy++) {
      for(ix=0;ix<nx;ix++) {
	data=StarGetCubePixel(cube,ix,iy,iz);
	if(data==dummy) {
	  written=fwrite(&nan,sizeof(nan),1,fp);
	} else {
#ifdef INTEL_CPU
	  written=fwriteIntel(&data,sizeof(data),1,fp);
#else
	  written=fwrite(&data,sizeof(data),1,fp);
#endif
	}
	if(written!=1) return((int)written_pixel);
	written_pixel=written_pixel+written;
      }
    }
  }

/* estimate blank byte */
  total_byte=nx*ny*nz*sizeof(float);
  blank_byte=(int)ceil((double)total_byte/2880.0)*2880-total_byte;

  if(blank_byte>0) {
    blank=malloc(blank_byte);
    for(ii=0;ii<blank_byte;ii++) {
      blank[ii]=' ';
    }
    written=fwrite(blank,1,blank_byte,fp);
    free(blank);
    written_pixel=written+written_pixel;
    if(written!=blank_byte) return((int)written_pixel);
  }

  return(0);
}

void StarPutCubePixelData(cube,ix,iy,iz,value)
/*
Put data value to CUBE_PIXEL data
2000/1/21 T.Handa
*/
CUBE *cube;
int ix,iy,iz;
double value;
{
  float pixel;

  pixel=(float)((value-(*cube).head.data_value.offset)/(*cube).head.data_value.scale);
  StarPutCubePixel(cube,ix,iy,iz,pixel);

  return;
}

double StarGetCubePixelData(cube,ix,iy,iz,valid_flag)
/*
Get data value to CUBE_PIXEL data
2000/1/26 T.Handa
*/
CUBE *cube;
int ix,iy,iz; /* <in> */
int *valid_flag; /* <out> =FLAG_YES: valid data, =FLAG_NO: dummy pixel */
{
  double pixel;
  float pix;

  pix=StarGetCubePixel(cube,ix,iy,iz);
  if(pix==(*cube).head.data_value.dummy) {
    pixel=(double)pix;
    *valid_flag=FLAG_NO;
  } else {
    pixel=(double)pix*(*cube).head.data_value.scale+(*cube).head.data_value.offset;
    *valid_flag=FLAG_YES;
  }

  return(pixel);
}

void StarPutCubePixelDummy(cube,ix,iy,iz)
/*
Put pixel value to CUBE_PIXEL data
2006/ 2/9 T.Handa
*/
CUBE *cube;
int ix,iy,iz;
{
  float value;

  value=(*cube).head.data_value.dummy;

  StarPutCubePixel(cube,ix,iy,iz,value);
  return;
}
