/*
STAR Cube data processing functions

1999/9/28 T.Handa
2000/1/21 T.Handa
2002/5/11 T.Handa
2003/8/28 T.Handa (revise) StarCubeScanDatavalue()
2006/12/13 T.Handa (add) StarClearCubePixel,StarCopyCubePixel(),StarCopyCubePixelData()
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

void StarCubePixelRescale(cube,new_scale,new_offset)
CUBE *cube; /* <i/o> */
double new_scale; /* <in> */
double new_offset; /* <in> */
{
  int nx,ny,nz;
  int ix,iy,iz;
  float data,dummy;
  double old_scale,old_offset;
  double ta;

  nx=(*cube).head.axis.x.size;
  ny=(*cube).head.axis.y.size;
  nz=(*cube).head.axis.z.size;

  dummy=(*cube).head.data_value.dummy;
  old_scale=(*cube).head.data_value.scale;
  old_offset=(*cube).head.data_value.offset;

  for(iz=0;iz<nz;iz++) {
    for(iy=0;iy<ny;iy++) {
      for(ix=0;ix<nx;ix++) {
	data=StarGetCubePixel(cube,ix,iy,iz);
	if(data!=dummy) {
	  ta=(double)data*old_scale+old_offset;
	  data=(float)((ta-new_offset)/new_scale);
	  StarPutCubePixel(cube,ix,iy,iz,data);
	}
      }
    }
  }

  (*cube).head.data_value.scale=new_scale;
  (*cube).head.data_value.offset=new_offset;  

  return;
}

int StarCubeFindLongLatVAxes(axes,coord_code,long_axis,lat_axis,v_axis)
AXES *axes; /* <in> */
int *coord_code; /* <out> coordinate code of longitude-latitide */
AXIS *long_axis,*lat_axis,*v_axis; /* <out> */
/*
get longitude, latitude and velocity axis in AXES
return(); the number of found axes

1999/4/20 T.Handa 
*/
{
  AXIS *axis;
  int assigned_axes;

  assigned_axes=0;

  axis=StarCubeFindAxisByType(axes,"long");
  if(axis!=NULL) {
    StarCopyAxis(long_axis,axis);
    assigned_axes++;
  }
  axis=StarCubeFindAxisByType(axes,"lat");
  if(axis!=NULL) {
    StarCopyAxis(lat_axis,axis);
    assigned_axes++;
  }

  if(strcmp((*long_axis).name,"l")==0) {
    *coord_code=COORD_LB;
  } else if(strcmp((*long_axis).name,"RA")==0) {
    *coord_code=COORD_RADEC;
  } else if(strcmp((*long_axis).name,"az")==0) {
    *coord_code=COORD_AZEL;
  } else if(strcmp((*long_axis).name,"x")==0) {
    *coord_code=COORD_XY;
  }

  axis=StarCubeFindAxisByType(axes,"velocity");
  if(axis!=NULL) {
    StarCopyAxis(v_axis,axis);
    assigned_axes++;
  }

  return(assigned_axes);
}

double StarGetCubePixPosOnAxis(axis,ii)
AXIS *axis;
int ii; /* <in> position in pixel */
/*
Get the pixel position assigned by pixel index into physical unit.
not concerned for index is out of range

1999/7/28 T.Handa
*/
{
  double pos;

  pos=(*axis).origin+(*axis).delta*((double)ii-(*axis).pixel_origin);
  return(pos);
}

void StarGetCubePixPosition(axes,ix,iy,iz,xx,yy,zz)
AXES *axes;
int ix,iy,iz; /* <in> 3D position in pixel */
double *xx,*yy,*zz; /* <out> 3D position in physical units */
/*
Get the pixel 3D position assigned by pixel into physical units.

1999/4/18 T.Handa
*/
{
  *xx=StarGetCubePixPosOnAxis(&((*axes).x),ix);
  *yy=StarGetCubePixPosOnAxis(&((*axes).y),iy);
  *zz=StarGetCubePixPosOnAxis(&((*axes).z),iz);

  return;
}

double StarPixelIndexPositionOnAxis(axis,xx)
/*
Get pixel index position on axis

1999/ 9/26 T.Handa
 */
AXIS *axis; /* <in> */
double xx; /* <in> */
{
  double index_pos;

  index_pos=(xx-(*axis).origin)/(*axis).delta+(*axis).pixel_origin;

  return index_pos;
}

double StarPixelValuePositionOnAxis(axis,xx)
/*
Get pixel index position on axis

1999/ 9/26 T.Handa
 */
AXIS *axis; /* <in> */
double xx; /* <in> */
{
  double value_pos;

  value_pos=(xx-(*axis).pixel_origin)*(*axis).delta+(*axis).origin;

  return value_pos;
}

AXIS *StarCubeFindAxisByName(axes,axis_name)
AXES *axes;
char *axis_name; /* <in> name tag to assign */
/*
Get the pointer of AXIS by name tag.
return(NULL); not found the assigned name tag

1999/4/18 T.Handa
*/
{
  int ii;
  AXIS *axis;

  axis=NULL;

  for(ii=0;ii<(*axes).dimension;ii++) {
    switch(ii) {
    case 0:
      axis= &((*axes).x);
      break;
    case 1:
      axis= &((*axes).y);
      break;
    case 2:
      axis= &((*axes).z);
      break;
    }

    if(strcmp((*axis).name,axis_name)==0) {
      return(axis);
    }
  }
  return(NULL);
}

AXIS *StarCubeFindAxisByType(axes,axis_type)
AXES *axes;
char *axis_type; /* <in> type tag to assign */
/*
Get the pointer of AXIS by type tag.
return(NULL); not found the assigned type tag

1999/4/20 T.Handa
*/
{
  int ii;
  AXIS *axis;

  axis=NULL;

  for(ii=0;ii<(*axes).dimension;ii++) {
    switch(ii) {
    case 0:
      axis= &((*axes).x);
      break;
    case 1:
      axis= &((*axes).y);
      break;
    case 2:
      axis= &((*axes).z);
      break;
    }

    if(strcmp((*axis).type,axis_type)==0) {
      return(axis);
    }
  }
  return(NULL);
}

void StarCopyCubeHead(out_head,in_head)
CUBE_HEAD *out_head,*in_head;
/*
copy CUBE_HEAD
1999/4/18 T.Handa
*/
{
  unsigned int	ii;
  int err_code;

  unsigned short history_steps;

  err_code=0;

/* copy STAR pipeflow header */
  strcpy((*out_head).file_id,(*in_head).file_id);
  strcpy((*out_head).file_name,(*in_head).file_name);

/* copy STAR file system header */
  StarCopySyshead(&((*out_head).sys),&((*in_head).sys));
  history_steps=(unsigned short)(*out_head).sys.hist_step;

/* copy cube file header */
  StarCopyObsprm(&((*out_head).obs),&((*in_head).obs));
  StarCopyObject(&((*out_head).object),&((*in_head).object));
  StarCopyAntstatus(&((*out_head).ant),&((*in_head).ant));
  StarCopyRxstatus(&((*out_head).rx),&((*in_head).rx));
  StarCopyBestatus(&((*out_head).be),&((*in_head).be));
  StarCopyObslog(&((*out_head).log),&((*in_head).log));
  StarCopyAxes(&((*out_head).axis),&((*in_head).axis));
  StarCopyDatavalue(&((*out_head).data_value),&((*in_head).data_value));
/* copy HISTORY */
  for(ii=0;ii<=history_steps;ii++) {
      StarCopyHistory(&((*out_head).history[ii]),&((*in_head).history[ii]));
  }

  return;
}

/*========Copy Header part=========*/
/*
required with StProfileProc.c-->libStarProf.a
*/
void StarCopyAxes(out_axes,in_axes)
AXES *out_axes,*in_axes;
{
  (*out_axes).dimension=(*in_axes).dimension;
  strcpy((*out_axes).order,(*in_axes).order);
  StarCopyAxis(&((*out_axes).x),&((*in_axes).x));
  StarCopyAxis(&((*out_axes).y),&((*in_axes).y));
  StarCopyAxis(&((*out_axes).z),&((*in_axes).z));
}

void StarCopyAxis(out_axis,in_axis)
AXIS *out_axis,*in_axis;
{
  strcpy((*out_axis).name,(*in_axis).name);
  (*out_axis).size=(*in_axis).size;
  strcpy((*out_axis).type,(*in_axis).type);
  strcpy((*out_axis).unit,(*in_axis).unit);
  strcpy((*out_axis).project,(*in_axis).project);
  strcpy((*out_axis).refer,(*in_axis).refer);
  (*out_axis).delta=(*in_axis).delta;
  (*out_axis).origin=(*in_axis).origin;
  (*out_axis).pixel_origin=(*in_axis).pixel_origin;
  (*out_axis).rotation=(*in_axis).rotation;
  (*out_axis).resolution=(*in_axis).resolution;
  strcpy((*out_axis).psf,(*in_axis).psf);
  (*out_axis).res_orig=(*in_axis).res_orig;

  return;
}

void StarCopyDatavalue(out_dv,in_dv)
DATA_VALUE *out_dv,*in_dv;
{
  strcpy((*out_dv).type,(*in_dv).type);
  strcpy((*out_dv).unit,(*in_dv).unit);
  (*out_dv).scale=(*in_dv).scale;
  (*out_dv).offset=(*in_dv).offset;
  (*out_dv).dummy=(*in_dv).dummy;
  (*out_dv).max=(*in_dv).max;
  (*out_dv).min=(*in_dv).min;
  (*out_dv).mean=(*in_dv).mean;
  (*out_dv).rms=(*in_dv).rms;
  (*out_dv).noise=(*in_dv).noise;

  return;
}

int StarCopyCubePixel(out_cube,in_cube)
/*
copy pixel data between two cubes with the identical headers

<return> 
9001 unmatched x-size
9002 unmatched y-size
9003 unmatched z-size

2006/12/13 T.Handa
 */
CUBE *out_cube,*in_cube;
{
  int ix,iy,iz;
  int nx,ny,nz;
  float pixel_value;

  nx=(*in_cube).head.axis.x.size;
  ny=(*in_cube).head.axis.y.size;
  nz=(*in_cube).head.axis.z.size;

  if(nx!=(*out_cube).head.axis.x.size) return(9001);
  if(ny!=(*out_cube).head.axis.y.size) return(9002);
  if(nz!=(*out_cube).head.axis.z.size) return(9003);

  for(iz=0;iz<nz;iz++) {
    for(iy=0;iy<ny;iy++) {
      for(ix=0;ix<nx;ix++) {
	pixel_value=StarGetCubePixel(in_cube,ix,iy,iz);
	StarPutCubePixel(out_cube,ix,iy,iz,pixel_value);
      }
    }
  }

  return(0);
}

int StarCopyCubePixelData(out_cube,in_cube)
/*
copy pixel data between two cubes with the some different headers

<return> 
9001 unmatched x-size
9002 unmatched y-size
9003 unmatched z-size

2006/12/13 T.Handa
 */
CUBE *out_cube,*in_cube;
{
  int ix,iy,iz;
  int nx,ny,nz;
  double pixel_value;
  int valid_flag;
  float out_dummy;

  nx=(*in_cube).head.axis.x.size;
  ny=(*in_cube).head.axis.y.size;
  nz=(*in_cube).head.axis.z.size;

  if(nx!=(*out_cube).head.axis.x.size) return(9001);
  if(ny!=(*out_cube).head.axis.y.size) return(9002);
  if(nz!=(*out_cube).head.axis.z.size) return(9003);

  out_dummy=(*out_cube).head.data_value.dummy;

  for(iz=0;iz<nz;iz++) {
    for(iy=0;iy<ny;iy++) {
      for(ix=0;ix<nx;ix++) {
	pixel_value=StarGetCubePixelData(in_cube,ix,iy,iz,&valid_flag);
	if(valid_flag==FLAG_YES) {
	  StarPutCubePixelData(out_cube,ix,iy,iz,pixel_value);
	} else {
	  StarPutCubePixel(out_cube,ix,iy,iz,out_dummy);
	}
      }
    }
  }

  return(0);
}

void StarClearCubePixel(cube)
/*
2005/4/19 T.Handa
 */
CUBE *cube; /* <i/o> */
{
  int mx,my,mz;
  int ix,iy,iz;
  float dummy;

  mx=(*cube).head.axis.x.size;
  my=(*cube).head.axis.y.size;
  mz=(*cube).head.axis.z.size;

  dummy=(*cube).head.data_value.dummy;

  for(ix=0;ix<mx;ix++) {
    for(iy=0;iy<my;iy++) {
      for(iz=0;iz<mz;iz++) {
	StarPutCubePixel(cube,ix,iy,iz,dummy);	
      }
    }
  }

  return;
}

/*------------*/

void StarCubeScanDatavalue(cube)
/*
Estimate statistical data value by scan all data in the cube
1998/11/23 T.Handa
2002/5/11 T.Handa
2003/8/28 T.Handa (revise) rms using negative value pixels, statistics for pixel real value
*/
CUBE *cube;
{
  int nx,ny,nz;
  int ix,iy,iz;
  double ta;
  double max,min;
  double average,rms;
  double deviate;
  int n_data;
  double neg_average,noise;
  int neg_data;
  int valid_pixel_flag;
  double value0;

  nx=(*cube).head.axis.x.size;
  ny=(*cube).head.axis.y.size;
  nz=(*cube).head.axis.z.size;

  average=0.0;
  rms=0.0;
  n_data=0;
  neg_average=0.0;
  noise=0.0;
  neg_data=0;
  value0=0.0;

/* evaluate average */
  for(ix=0;ix<nx;ix++) {
    for(iy=0;iy<ny;iy++) {
      for(iz=0;iz<nz;iz++) {
	ta=StarGetCubePixelData(cube,ix,iy,iz,&valid_pixel_flag);
	if(valid_pixel_flag==FLAG_YES) {
	  if(n_data==0) value0=ta;
	  n_data++;
	  average=average+ta;
	  if(ta<0.0) {
	    neg_data++;
	    neg_average=neg_average+ta;
	  }
	}
      }
    }
  }

/* all data are dummy */
  if(n_data<=0) {
    (*cube).head.data_value.mean=0.0;
    (*cube).head.data_value.rms=0.0;
    (*cube).head.data_value.noise=0.0;
    (*cube).head.data_value.max=0.0;
    (*cube).head.data_value.min=0.0;
    return;
  }
  average=average/(double)n_data;
  neg_average=neg_average/(double)neg_data;

/* evaluate rms and max,min */
/* evaluate noise level using negative value pixels */
  max=value0;
  min=value0;
  for(ix=0;ix<nx;ix++) {
    for(iy=0;iy<ny;iy++) {
      for(iz=0;iz<nz;iz++) {
	ta=StarGetCubePixelData(cube,ix,iy,iz,&valid_pixel_flag);
	if(valid_pixel_flag==FLAG_YES) {
	  deviate=ta-average;
	  rms=rms+deviate*deviate;

	  if(max<ta) max=ta;
	  if(min>ta) min=ta;

	  if(ta<0.0) {
	    deviate=ta-neg_average;
	    noise=noise+deviate*deviate;
	  }
	}
      }
    }
  }

  rms=rms/(double)n_data;
  rms=sqrt(rms);

  noise=noise/(double)neg_data;
  noise=sqrt(noise);

  (*cube).head.data_value.mean=(float)average;
  (*cube).head.data_value.rms=(float)rms;
  (*cube).head.data_value.noise=(float)noise;
  (*cube).head.data_value.max=(float)max;
  (*cube).head.data_value.min=(float)min;

  return;
}
