/*
STAR Cube data showing functions

2002/4/22 T.Handa ver.3.10
2002/6/ 6 T.Handa ver.3.11 (debug) StarPlotCubeContourBar();
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#include <Pxplib.h>
#include <PxpGlib.h>
#include <StPxpShow.h>

static char contour_color[L_COLOR_STRING];
static char frame_color[L_COLOR_STRING],title_color[L_COLOR_STRING];
static int color_init=0;

/***********************/
/* internal functions  */
/***********************/
static void set_std_linecolor()
{
  if(color_init!=0) return;
  StarResetCubeMapShowColor("blue","black","black");
  color_init=1;
  return;
}

static void line_type_code2name(type_code,type_name,c_level)
int type_code; /* <in> line type code in StCubeShow.h */
char *type_name; /* <out> line type name in Pxplib */
double c_level; /* <in> contour level */
/*
conection between StCubeShow and Pxplib
2002/4/22 T.Handa
2002/ 6/ 6 T.Handa (level=0.0 in dashed line at auto mode)
 */
{
  if(type_code==AUTO_POSNEG_LINE) {
    if(c_level>0.0) strcpy(type_name,"solid");
    else strcpy(type_name,"dash");
  } else {

    switch(type_code) {
    case SOLID_LINE:
      strcpy(type_name,"solid");
      break;
    case DASHED_LINE:
      strcpy(type_name,"dash");
      break;
    case DOTTED_LINE:
      strcpy(type_name,"dot");
      break;
    case DOTTED_DASHED_LINE:
      strcpy(type_name,"dotted-dashed");
      break;
    default:
      strcpy(type_name,"solid");
    }
  }
  return;
}

static double cut_by_limit(limit,cell)
RANGE limit; /* <in> */
double cell; /* <in> */
{
  if(cell<limit.min) return(limit.min);
  else if(cell>limit.max) return(limit.max);
  else return(cell);
}

static double scale_for_bar(xx,plot_box,bar_range)
double xx;
RANGE *plot_box; /* <in> */
PxpRange *bar_range; /* <in> */
/*
convert value to plot position for bar
2002/6/6 T.Handa
*/
{
  double yy;
  double box_span,bar_span;

  box_span=(*plot_box).max-(*plot_box).min;
  bar_span=(*bar_range).max-(*bar_range).min;

  yy=(xx-(*bar_range).min)/bar_span*box_span+(*plot_box).min;

  return yy;
}

/***********************/
/* common for any maps */
/***********************/
double StarLinerInterporation(xx,x0,x1,y0,y1)
double xx;    /* argument position */
double x0,x1; /* given position */
double y0,y1; /* given value */
/*
Simple liner interporation for contouring

2002/4/22 T.Handa
 */
{
  double yy;

  if(x1==x0) {
    yy=y0;
  } else {
    yy=(y1-y0)/(x1-x0)*(xx-x0)+y0;
  }

  return yy;
}


void StarPlotCubeMapTitle(cube,cut_plane,plot_box,z_range)
CUBE *cube; /* <in> */
int cut_plane;	/* <in> XY_CUT,YX_CUT,YZ_CUT,ZY_CUT,ZX_CUT,XZ_CUT */
BOX_RANGE plot_box;	/* <in> plot and labeling position */
RANGE z_range; /* <in> */
{
  double center_x,center_y;
  char buffer[BUFFER_SIZE]; /* AXIS.name+AXIS.refer+AXIS.unit */
  AXIS *axis;

  switch(cut_plane) {
  case YZ_CUT: case ZY_CUT:
    axis= &((*cube).head.axis.x);
    break;
  case XZ_CUT: case ZX_CUT:
    axis= &((*cube).head.axis.y);
    break;
  case YX_CUT: case XY_CUT: default:
    axis= &((*cube).head.axis.z);
  }

  center_x=plot_box.x.max;
  center_y=plot_box.y.max+0.8;
  if(z_range.min==z_range.max)
    sprintf(buffer,"%s=%f(%s)",(*axis).name,z_range.min,(*axis).unit);
  else
    sprintf(buffer,"%s=%f to %f(%s)",(*axis).name,z_range.min,z_range.max,(*axis).unit);
  PxpDrawJustifiedString(center_x,center_y,buffer,"right","bottom");

  center_x=(plot_box.x.min+plot_box.x.max)/2.0;
  center_y=plot_box.y.max+1.5;
  sprintf(buffer,"%s, %s",(*cube).head.file_name,(*cube).head.object.name);
  PxpDrawJustifiedString(center_x,center_y,buffer,"center","bottom");

  return;
}

void StarPlotCubeMapLabel(phys_box,phys_cbox,plot_box,plot_cbox,x_step,y_step,x_axis,y_axis)
/*
plot only a label of cube-map
1997/12/11 T.Handa
*/
BOX_RANGE phys_box,phys_cbox;	/* <in> plot and labeling range (phys) */
BOX_RANGE plot_box,plot_cbox;	/* <in> plot and labeling position */
double x_step,y_step;	/* <in> label interval (phys) */
AXIS *x_axis,*y_axis;	/* <in> */
{
  PxpPoint phys0,phys1;
  PxpPoint ceil_phys0,ceil_phys1;
  PxpRange xc_range,yc_range;
  PxpPoint plot0,plot1;
  char x_lbl_format[80],y_lbl_format[80];
  double center_x,center_y;
  char buffer[BUFFER_SIZE]; /* AXIS.name+AXIS.refer+AXIS.unit */

  StarBoxRange2PxpPoint(plot_cbox,&plot0,&plot1);
  PxpPoint2Range(plot0,plot1,&xc_range,&yc_range);
  StarBoxRange2PxpPoint(phys_box,&phys0,&phys1);
  StarBoxRange2PxpPoint(phys_cbox,&ceil_phys0,&ceil_phys1);

  PxpGLabelFormat(x_lbl_format,xc_range,x_step,(char)NULL);
  PxpGLabelFormat(y_lbl_format,yc_range,y_step,(char)NULL);
  PxpGLabelInset(phys0,phys1,ceil_phys0,ceil_phys1,x_step,y_step,x_lbl_format,y_lbl_format);

  center_x=(plot_box.x.min+plot_box.x.max)/2.0;
  center_y=plot_box.y.min-0.5;
/*
  sprintf(buffer,"%s %s %s (%s)",(*x_axis).name,(*x_axis).refer,(*x_axis).project,(*x_axis).unit);
*/
  sprintf(buffer,"%s (%s)",(*x_axis).name,(*x_axis).unit);
  PxpDrawJustifiedString(center_x,center_y,buffer,"center","hanging");

  center_x=plot_box.x.min-0.5;
  center_y=plot_box.y.max+0.1;
/*
  sprintf(buffer,"%s %s %s (%s)",(*y_axis).name,(*y_axis).refer,(*y_axis).project,(*y_axis).unit);
*/
  sprintf(buffer,"%s(%s)",(*y_axis).name,(*y_axis).unit);
  PxpDrawJustifiedString(center_x,center_y,buffer,"left","bottom");

  return;
}

void StarPlotCubeMapFrame(phys_box,phys_cbox,x_step,y_step,tick_length)
/*
plot frame for cube-map
1997/12/12 T.Handa
*/
BOX_RANGE phys_box,phys_cbox;	/* <in> for plot and labeling range */
double x_step,y_step;	/* <in> label interval (phys value) */
double tick_length;	/* <in> */
{
  PxpPoint phys0,phys1;
  PxpPoint ceil_phys0,ceil_phys1;

  StarBoxRange2PxpPoint(phys_box,&phys0,&phys1);
  StarBoxRange2PxpPoint(phys_cbox,&ceil_phys0,&ceil_phys1);

  PxpGBox(phys0,phys1);
  PxpGTickMarkInset(phys0,phys1,ceil_phys0,ceil_phys1,x_step,y_step,tick_length);

  return;
}

int StarCubeMapAutoRangeAlongAxis(axis,x_r,pixel_r)
AXIS *axis;	/* <in> */
RANGE *x_r;	/* <i/o> range in (*axis).unit */
RANGE_long *pixel_r;	/* <out> corresponding pixel range (sorted) */
/*
auto set range in pixel from physical value
returnthe value of "x_r" is...>0 auto set, <=0 keep the input value
2000/2/11 T.Handa
*/
{
  RANGE x_range; /* range in (*axis).unit */
  RANGE_long pixel_range; /* range in pixel */
  int x_autoscale;

/* internal parameter setting */
  x_range.min=(*x_r).min;
  x_range.max=(*x_r).max;

/* default range is all */
  if(x_range.min==x_range.max) {
    x_range.min=StarGetCubePixPosOnAxis(axis,0);
    x_range.max=StarGetCubePixPosOnAxis(axis,(*axis).size-1);
    x_autoscale=1;
  } else {
    x_autoscale= -1;
  }

/* estimate corresponding channel range */
  pixel_range.min=(long)rint(StarPixelIndexPositionOnAxis(axis,x_range.min));
  pixel_range.max=(long)rint(StarPixelIndexPositionOnAxis(axis,x_range.max));
  StarSortRangeLong(&pixel_range);

/* over/under flow correction */
  if(pixel_range.min<0) 
    pixel_range.min=0;
  if(pixel_range.max>=(*axis).size)
    pixel_range.max=(*axis).size-1;

/* set return value */
  if(x_autoscale>0) {
    (*x_r).min=x_range.min;
    (*x_r).max=x_range.max;
  }

  (*pixel_r).min=pixel_range.min;
  (*pixel_r).max=pixel_range.max;

  return(x_autoscale);
}

void StarCubeMapAutoXYRange(cube,cut_plane,plot_mode,hori_auto,hori_r,hori_lbl_r,d_hori,vert_auto,vert_r,vert_lbl_r,d_vert,ii_r,jj_r)
/*
auto set range, label and tickmarks along both axes
2000/2/11 T.Handa
*/
CUBE *cube;	/* <in> */
int cut_plane;	/* <in> XY_CUT,YX_CUT,YZ_CUT,ZY_CUT,ZX_CUT,XZ_CUT */
int plot_mode; /* <in> PLOT_CELL_MAP,PLOT_CONTOUR_MAP */
int *hori_auto;	/* <out> done auto setting? */
RANGE *hori_r;	/* <i/o> horizontal range; outer envelope */
RANGE *hori_lbl_r;	/* <out> for horizontal labels and tickmarks (phys unit) */
double *d_hori;	/* <out> horizontal label and tickmark interval (phys unit) */
int *vert_auto;	/* <out> done auto setting? */
RANGE *vert_r;	/* <i/o> vertical range; outer envelope */
RANGE *vert_lbl_r;	/* <out> for vertical labels and tickmarks (phys unit) */
double *d_vert;	/* <out> vertical label and tickmark interval (phys unit) */
RANGE_long *ii_r;	/* <out> sorted horizontal pixel range */
RANGE_long *jj_r;	/* <out> sorted vertical pixel range */
{
  RANGE hori_range,vert_range;
  PxpRange org_range,hori_lbl_range,vert_lbl_range;
  int hori_autoscale,vert_autoscale;
  int n_step,m_step;
  double hori_step,vert_step;
  double ext_hori,ext_vert;

/* internal parameter setting */
  hori_range.min=(*hori_r).min;
  hori_range.max=(*hori_r).max;
  vert_range.min=(*vert_r).min;
  vert_range.max=(*vert_r).max;

/* estimate correstponding horizontal and range */
  switch(cut_plane) {
  case YZ_CUT:
    hori_autoscale=StarCubeMapAutoRangeAlongAxis(&((*cube).head.axis.y),&hori_range,ii_r);
    vert_autoscale=StarCubeMapAutoRangeAlongAxis(&((*cube).head.axis.z),&vert_range,jj_r);
    ext_hori=fabs((*cube).head.axis.y.delta)/2.0;
    ext_vert=fabs((*cube).head.axis.z.delta)/2.0;
    break;
  case ZY_CUT:
    hori_autoscale=StarCubeMapAutoRangeAlongAxis(&((*cube).head.axis.z),&hori_range,ii_r);
    vert_autoscale=StarCubeMapAutoRangeAlongAxis(&((*cube).head.axis.y),&vert_range,jj_r);
    ext_hori=fabs((*cube).head.axis.z.delta)/2.0;
    ext_vert=fabs((*cube).head.axis.y.delta)/2.0;
    break;
  case XZ_CUT:
    hori_autoscale=StarCubeMapAutoRangeAlongAxis(&((*cube).head.axis.x),&hori_range,ii_r);
    vert_autoscale=StarCubeMapAutoRangeAlongAxis(&((*cube).head.axis.z),&vert_range,jj_r);
    ext_hori=fabs((*cube).head.axis.x.delta)/2.0;
    ext_vert=fabs((*cube).head.axis.z.delta)/2.0;
    break;
  case ZX_CUT:
    hori_autoscale=StarCubeMapAutoRangeAlongAxis(&((*cube).head.axis.z),&hori_range,ii_r);
    vert_autoscale=StarCubeMapAutoRangeAlongAxis(&((*cube).head.axis.x),&vert_range,jj_r);
    ext_hori=fabs((*cube).head.axis.z.delta)/2.0;
    ext_vert=fabs((*cube).head.axis.x.delta)/2.0;
    break;
  case YX_CUT:
    hori_autoscale=StarCubeMapAutoRangeAlongAxis(&((*cube).head.axis.y),&hori_range,ii_r);
    vert_autoscale=StarCubeMapAutoRangeAlongAxis(&((*cube).head.axis.x),&vert_range,jj_r);
    ext_hori=fabs((*cube).head.axis.y.delta)/2.0;
    ext_vert=fabs((*cube).head.axis.x.delta)/2.0;
    break;
  case XY_CUT: default:
    hori_autoscale=StarCubeMapAutoRangeAlongAxis(&((*cube).head.axis.x),&hori_range,ii_r);
    vert_autoscale=StarCubeMapAutoRangeAlongAxis(&((*cube).head.axis.y),&vert_range,jj_r);
    ext_hori=fabs((*cube).head.axis.x.delta)/2.0;
    ext_vert=fabs((*cube).head.axis.y.delta)/2.0;
  }
/* range adjusting */
  if(hori_range.min<hori_range.max) {
    org_range.min=hori_range.min; org_range.max=hori_range.max;
  } else {
    org_range.min=hori_range.max; org_range.max=hori_range.min;
  }
  if(plot_mode==PLOT_CELL_MAP) {
    org_range.min=org_range.min-ext_hori;
    org_range.max=org_range.max+ext_hori;
  }
  PxpGCeilingRange(org_range,&hori_lbl_range,&n_step,&hori_step);

  if(vert_range.min<vert_range.max) {
    org_range.min=vert_range.min; org_range.max=vert_range.max;
  } else {
    org_range.min=vert_range.max; org_range.max=vert_range.min;
  }
  if(plot_mode==PLOT_CELL_MAP) {
    org_range.min=org_range.min-ext_vert;
    org_range.max=org_range.max+ext_vert;
  }
  PxpGCeilingRange(org_range,&vert_lbl_range,&m_step,&vert_step);

/* set return value */
  if(hori_autoscale>0) {
    (*hori_r).min=hori_range.min;
    (*hori_r).max=hori_range.max;
  }
  if(vert_autoscale>0) {
    (*vert_r).min=vert_range.min;
    (*vert_r).max=vert_range.max;
  }
  if(plot_mode==PLOT_CELL_MAP) {
    if((*hori_r).min<(*hori_r).max) {
      (*hori_r).min=(*hori_r).min-fabs(ext_hori);
      (*hori_r).max=(*hori_r).max+fabs(ext_hori);
    } else {
      (*hori_r).min=(*hori_r).min+fabs(ext_hori);
      (*hori_r).max=(*hori_r).max-fabs(ext_hori);
    }      
    if((*vert_r).min<(*vert_r).max) {
      (*vert_r).min=(*vert_r).min-fabs(ext_vert);
      (*vert_r).max=(*vert_r).max+fabs(ext_vert);
    } else {
      (*vert_r).min=(*vert_r).min+fabs(ext_vert);
      (*vert_r).max=(*vert_r).max-fabs(ext_vert);
    }
  }

  *hori_auto=hori_autoscale;
  *vert_auto=vert_autoscale;

  (*hori_lbl_r).min=hori_lbl_range.min;
  (*hori_lbl_r).max=hori_lbl_range.max;
  (*vert_lbl_r).min=vert_lbl_range.min;
  (*vert_lbl_r).max=vert_lbl_range.max;

  *d_hori=hori_step;
  *d_vert=vert_step;

  return;
}

void StarCubeMapAutoRange(cube,cut_plane,hori_r,vert_r,plot_mode,ii_r,jj_r,d_hori,d_vert,phys_b,phys_cb)
/*
auto setting of coordinate location to plot cube-map
2000/ 2/14 T.Handa
*/
CUBE *cube;	/* <in> */
int cut_plane;	/* <in> XY_CUT,YX_CUT,YZ_CUT,ZY_CUT,ZX_CUT,XZ_CUT */
RANGE hori_r,vert_r;	/* <in> range */
int plot_mode; /* <in> PLOT_CELL_MAP,PLOT_CONTOUR_MAP */
RANGE_long *ii_r,*jj_r;	/* <out> correspoided pixel range (sorted) */
double *d_hori,*d_vert;	/* <out> interval of labels and tickmarks (phys unit) */
BOX_RANGE *phys_b;	/* <out> range for ploting (phys unit) */
BOX_RANGE *phys_cb;	/* <out> display range of frame, labels, tickmarks (phys unit) */
{
  RANGE h_range,v_range;
  RANGE hc_r,vc_r;
  PxpRange hc_range,vc_range;
  RANGE_long act_ii_range,act_jj_range;
  int h_autoscale,v_autoscale;
  double h_step,v_step;

  BOX_RANGE phys_box;
  BOX_RANGE phys_cbox;

  PxpPoint phys0,phys1;
  PxpPoint ceil_phys0,ceil_phys1;

/* internal parameter setting */
  h_range.min=hori_r.min;
  h_range.max=hori_r.max;
  v_range.min=vert_r.min;
  v_range.max=vert_r.max;

/* estimate proper range */
  StarCubeMapAutoXYRange(cube,cut_plane,plot_mode,&h_autoscale,&h_range,&hc_r,&h_step,&v_autoscale,&v_range,&vc_r,&v_step,&act_ii_range,&act_jj_range);

  hc_range.min=hc_r.min;
  hc_range.max=hc_r.max;
  vc_range.min=vc_r.min;
  vc_range.max=vc_r.max;

  phys0.x=h_range.min; 
  phys1.x=h_range.max;
  phys0.y=v_range.min;
  phys1.y=v_range.max;
  StarPxpPoint2BoxRange(phys0,phys1,&phys_box);

  if(hc_range.min<hc_range.max) {
    ceil_phys0.x=hc_range.min;
    ceil_phys1.x=hc_range.max;
  } else {
    ceil_phys0.x=hc_range.max;
    ceil_phys1.x=hc_range.min;
  }
  if(vc_range.min<vc_range.max) {
    ceil_phys0.y=vc_range.min;
    ceil_phys1.y=vc_range.max;
  } else {
    ceil_phys0.y=vc_range.max;
    ceil_phys1.y=vc_range.min;
  }
  StarPxpPoint2BoxRange(ceil_phys0,ceil_phys1,&phys_cbox);

/* set return value */
  (*ii_r).min=act_ii_range.min;
  (*ii_r).max=act_ii_range.max;
  (*jj_r).min=act_jj_range.min;
  (*jj_r).max=act_jj_range.max;

  *d_hori=h_step;
  *d_vert=v_step;

  (*phys_b).x.min=phys_box.x.min;
  (*phys_b).x.max=phys_box.x.max;
  (*phys_b).y.min=phys_box.y.min;
  (*phys_b).y.max=phys_box.y.max;

  (*phys_cb).x.min=phys_cbox.x.min;
  (*phys_cb).x.max=phys_cbox.x.max;
  (*phys_cb).y.min=phys_cbox.y.min;
  (*phys_cb).y.max=phys_cbox.y.max;

  return;
}

void StarPlotCubeMapSetRange(plot_box,phys_box)
/*
set relation between plot position and phyics range for the profile
2000/2/11 T.Handa (cp from StarPlotProfileSetRange() 1997/12/12)
*/
BOX_RANGE phys_box,plot_box; /* <in> */
{
  PxpPoint phys0,phys1,cm0,cm1;

  StarBoxRange2PxpPoint(phys_box,&phys0,&phys1);
  StarBoxRange2PxpPoint(plot_box,&cm0,&cm1);
  PxpGSetRange(phys0,phys1,cm0,cm1);

  return;
}

void StarPlotCubeMapStdFrame(cube,cut_plane,h_range,v_range,cut_range,gray_range,plot_box,phys_box,plot_cbox,phys_cbox,h_step,v_step)
/*
cube-map frame for profile in the standard format
2000/2/11 T.Handa
*/
CUBE *cube;	/* <in> */
int cut_plane;	/* <in> XY_CUT,YX_CUT,YZ_CUT,ZY_CUT,ZX_CUT,XZ_CUT */
RANGE h_range,v_range;	/* <in> horizontal&vertical range; min=LeftBottom,max=RightTop */
RANGE cut_range; /* <in> cutting range */
RANGE gray_range; /* <in> color bar min/max */
BOX_RANGE plot_box;	/* <in> position of label and tick mark (in printing position, not in physics unit) */
BOX_RANGE phys_box;	/* <in> range plotted (in physics unit) */
BOX_RANGE plot_cbox,phys_cbox; /* <in> range for labels and tickmarks */
double h_step,v_step; /* <in> */
{
  double tick_length;
  double min_span;
  BOX_RANGE phys_fbox;

/* set the standard color set */
  set_std_linecolor();

/* adjust tick length */
  min_span=fabs(plot_box.x.max-plot_box.x.min);
  if(min_span>fabs(plot_box.y.max-plot_box.y.min)) 
    min_span=fabs(plot_box.y.max-plot_box.y.min);
  tick_length=0.2;
  if(tick_length*2.0>min_span) tick_length=min_span*0.4;

/* label position */
  phys_fbox=StarEnclosureBoxRange2InteriorBoxRange(phys_cbox,phys_box,h_step,v_step);

/* plot frame and label */
  PxpSetColor(frame_color);
  StarPlotCubeMapFrame(phys_box,phys_fbox,h_step,v_step,tick_length);

  switch(cut_plane) {
  case YZ_CUT:
    StarPlotCubeMapLabel(phys_box,phys_fbox,plot_box,plot_cbox,h_step,v_step,&((*cube).head.axis.y),&((*cube).head.axis.z));
    break;
  case ZY_CUT:
    StarPlotCubeMapLabel(phys_box,phys_fbox,plot_box,plot_cbox,h_step,v_step,&((*cube).head.axis.z),&((*cube).head.axis.y));
    break;
  case XZ_CUT:
    StarPlotCubeMapLabel(phys_box,phys_fbox,plot_box,plot_cbox,h_step,v_step,&((*cube).head.axis.x),&((*cube).head.axis.z));
    break;
  case ZX_CUT:
    StarPlotCubeMapLabel(phys_box,phys_fbox,plot_box,plot_cbox,h_step,v_step,&((*cube).head.axis.z),&((*cube).head.axis.x));
    break;
  case YX_CUT:
    StarPlotCubeMapLabel(phys_box,phys_fbox,plot_box,plot_cbox,h_step,v_step,&((*cube).head.axis.y),&((*cube).head.axis.x));
    break;
  case XY_CUT: default:
    StarPlotCubeMapLabel(phys_box,phys_fbox,plot_box,plot_cbox,h_step,v_step,&((*cube).head.axis.x),&((*cube).head.axis.y));
  }
  StarPlotCubeMapTitle(cube,cut_plane,plot_box,cut_range);

  return;
}

void StarCubeMapLongitudeFlip(cube,cut_plane,h_step,phys_box,phys_cbox)
/*
flip horisontal axis range if its type is longitude.
2000/2/14 T.Handa
*/
CUBE *cube; /* <in> */
int cut_plane;	/* <in> XY_CUT,YX_CUT,YZ_CUT,ZY_CUT,ZX_CUT,XZ_CUT */
double *h_step; /* <i/o> */
BOX_RANGE *phys_box,*phys_cbox; /* <i/o> */
{
  double tmp;
  int flip_flag;

  flip_flag=0;

  switch(cut_plane) {
  case YZ_CUT: case YX_CUT:
    if(strncmp((*cube).head.axis.y.type,"long",4)==0) {
      flip_flag=1;
    }
    break;
  case ZY_CUT: case ZX_CUT:
    if(strncmp((*cube).head.axis.z.type,"long",4)==0) {
      flip_flag=1;
    }
    break;
  case XZ_CUT: case XY_CUT: default:
    if(strncmp((*cube).head.axis.x.type,"long",4)==0) {
      flip_flag=1;
    }
  }

  if(flip_flag==1) {
      tmp=(*phys_box).x.max;
      (*phys_box).x.max=(*phys_box).x.min;
      (*phys_box).x.min=tmp;
      tmp=(*phys_cbox).x.max;
      (*phys_cbox).x.max=(*phys_cbox).x.min;
      (*phys_cbox).x.min=tmp;
      *h_step= -(*h_step);
  }
   
  return;
}

/***********************/
/*   gray scale map    */
/***********************/
void StarPlotCubeMapByGrayCells(cube,cut_plane,i_range,j_range,cut_level,plot_box,phys_box,gray_range)
CUBE *cube; /* <in> */
int cut_plane;	/* <in> XY_CUT,YX_CUT,YZ_CUT,ZY_CUT,ZX_CUT,XZ_CUT */
RANGE_long i_range,j_range;	/* <in> correspoided pixel range (sorted) */
int cut_level; /* <in> cut position in pixel */
BOX_RANGE plot_box; /* <in> */
BOX_RANGE phys_box; /* <in> min=LeftBottom, max=RightTop */
RANGE gray_range; /* <in> gray_scale; min=0=white, max=1=black */
{
  int ix,iy,iz;
  PxpPoint pt_blc,pt_trc;
  double dx,dy;
  float pixel_value,dummy_value;
  double b_scale,b_offset;
  double gray_density;
  double g_scale,g_offset;
  double ta;
  RANGE h_limit,v_limit;

/* data value */
  b_scale=(*cube).head.data_value.scale;
  b_offset=(*cube).head.data_value.offset;
  dummy_value=(*cube).head.data_value.dummy;

  g_scale=gray_range.max-gray_range.min;
  g_offset=gray_range.min;
  if(g_scale==0.0) {
    g_scale=(double)((*cube).head.data_value.max-(*cube).head.data_value.min)*b_scale;
    g_offset=(double)(*cube).head.data_value.min*b_scale+b_offset;
  }
  if(g_scale==0.0) {   /* flat image */
    g_scale=2.0;
    g_offset=1.0;
  }

  h_limit.min=phys_box.x.min;
  h_limit.max=phys_box.x.max;
  v_limit.min=phys_box.y.min;
  v_limit.max=phys_box.y.max;
  StarSortRange(&h_limit);
  StarSortRange(&v_limit);

  switch(cut_plane) {
  case YZ_CUT:
    ix=cut_level;
    if(ix<0) ix=0;
    else if(ix>=(*cube).head.axis.x.size) ix=(*cube).head.axis.x.size-1;
    dx=(*cube).head.axis.y.delta/2.0;
    dy=(*cube).head.axis.z.delta/2.0;

    for(iz=j_range.min;iz<=j_range.max;iz++) {
      pt_blc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.z),iz)-dy;
      pt_trc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.z),iz)+dy;
      pt_blc.y=cut_by_limit(v_limit,pt_blc.y);
      pt_trc.y=cut_by_limit(v_limit,pt_trc.y);

      for(iy=i_range.min;iy<=i_range.max;iy++) {
	pt_blc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.y),iy)-dx;
	pt_trc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.y),iy)+dx;
	pt_blc.x=cut_by_limit(h_limit,pt_blc.x);
	pt_trc.x=cut_by_limit(h_limit,pt_trc.x);

       	pixel_value=StarGetCubePixel(cube,ix,iy,iz);

	if(pixel_value==dummy_value) {
	  gray_density=(double)pixel_value;
	} else {
	  ta=(double)pixel_value*b_scale+b_offset;
	  gray_density=(ta-g_offset)/g_scale;
	}
	if(gray_density<0.0) gray_density=0.0;
	else if(gray_density>1.0) gray_density=1.0;

	PxpSetGray(gray_density);

	PxpGFilledBox(pt_blc,pt_trc);
      }
    }

    break;
  case ZY_CUT:
    ix=cut_level;
    if(ix<0) ix=0;
    else if(ix>=(*cube).head.axis.x.size) ix=(*cube).head.axis.x.size-1;
    dx=(*cube).head.axis.z.delta/2.0;
    dy=(*cube).head.axis.y.delta/2.0;

    for(iz=i_range.min;iz<=i_range.max;iz++) {
      pt_blc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.z),iz)-dx;
      pt_trc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.z),iz)+dx;
      pt_blc.x=cut_by_limit(h_limit,pt_blc.x);
      pt_trc.x=cut_by_limit(h_limit,pt_trc.x);

      for(iy=j_range.min;iy<=j_range.max;iy++) {
	pt_blc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.y),iy)-dy;
	pt_trc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.y),iy)+dy;
	pt_blc.y=cut_by_limit(v_limit,pt_blc.y);
	pt_trc.y=cut_by_limit(v_limit,pt_trc.y);

       	pixel_value=StarGetCubePixel(cube,ix,iy,iz);

	if(pixel_value==dummy_value) {
	  gray_density=(double)pixel_value;
	} else {
	  ta=(double)pixel_value*b_scale+b_offset;
	  gray_density=(ta-g_offset)/g_scale;
	}
	if(gray_density<0.0) gray_density=0.0;
	else if(gray_density>1.0) gray_density=1.0;

	PxpSetGray(gray_density);

	PxpGFilledBox(pt_blc,pt_trc);
      }
    }

    break;
  case XZ_CUT:
    iy=cut_level;
    if(iy<0) iy=0;
    else if(iy>=(*cube).head.axis.y.size) iy=(*cube).head.axis.y.size-1;
    dx=(*cube).head.axis.x.delta/2.0;
    dy=(*cube).head.axis.z.delta/2.0;
    for(ix=i_range.min;ix<=i_range.max;ix++) {
      pt_blc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.x),ix)-dx;
      pt_trc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.x),ix)+dx;
      pt_blc.x=cut_by_limit(h_limit,pt_blc.x);
      pt_trc.x=cut_by_limit(h_limit,pt_trc.x);

      for(iz=j_range.min;iz<=j_range.max;iz++) {
	pt_blc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.z),iz)-dy;
	pt_trc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.z),iz)+dy;
	pt_blc.y=cut_by_limit(v_limit,pt_blc.y);
	pt_trc.y=cut_by_limit(v_limit,pt_trc.y);

       	pixel_value=StarGetCubePixel(cube,ix,iy,iz);

	if(pixel_value==dummy_value) {
	  gray_density=(double)pixel_value;
	} else {
	  ta=(double)pixel_value*b_scale+b_offset;
	  gray_density=(ta-g_offset)/g_scale;
	}
	if(gray_density<0.0) gray_density=0.0;
	else if(gray_density>1.0) gray_density=1.0;

	PxpSetGray(gray_density);

	PxpGFilledBox(pt_blc,pt_trc);
      }
    }

    break;
  case ZX_CUT:
    iy=cut_level;
    if(iy<0) iy=0;
    else if(iy>=(*cube).head.axis.y.size) iy=(*cube).head.axis.y.size-1;
    dx=(*cube).head.axis.z.delta/2.0;
    dy=(*cube).head.axis.x.delta/2.0;

    for(ix=j_range.min;ix<=j_range.max;ix++) {
      pt_blc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.x),ix)-dy;
      pt_trc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.x),ix)+dy;
      pt_blc.y=cut_by_limit(v_limit,pt_blc.y);
      pt_trc.y=cut_by_limit(v_limit,pt_trc.y);

      for(iz=i_range.min;iz<=i_range.max;iz++) {
	pt_blc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.z),iz)-dx;
	pt_trc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.z),iz)+dx;
	pt_blc.x=cut_by_limit(h_limit,pt_blc.x);
	pt_trc.x=cut_by_limit(h_limit,pt_trc.x);

       	pixel_value=StarGetCubePixel(cube,ix,iy,iz);

	if(pixel_value==dummy_value) {
	  gray_density=(double)pixel_value;
	} else {
	  ta=(double)pixel_value*b_scale+b_offset;
	  gray_density=(ta-g_offset)/g_scale;
	}
	if(gray_density<0.0) gray_density=0.0;
	else if(gray_density>1.0) gray_density=1.0;

	PxpSetGray(gray_density);

	PxpGFilledBox(pt_blc,pt_trc);
      }
    }

    break;
  case YX_CUT:
    iz=cut_level;
    if(iz<0) iz=0;
    else if(iz>=(*cube).head.axis.z.size) iz=(*cube).head.axis.z.size-1;
    dx=(*cube).head.axis.y.delta/2.0;
    dy=(*cube).head.axis.x.delta/2.0;

    for(ix=j_range.min;ix<=j_range.max;ix++) {
      pt_blc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.x),ix)-dy;
      pt_trc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.x),ix)+dy;
      pt_blc.y=cut_by_limit(v_limit,pt_blc.y);
      pt_trc.y=cut_by_limit(v_limit,pt_trc.y);

      for(iy=i_range.min;iy<=i_range.max;iy++) {
	pt_blc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.y),iy)-dx;
	pt_trc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.y),iy)+dx;
	pt_blc.x=cut_by_limit(h_limit,pt_blc.x);
	pt_trc.x=cut_by_limit(h_limit,pt_trc.x);

       	pixel_value=StarGetCubePixel(cube,ix,iy,iz);

	if(pixel_value==dummy_value) {
	  gray_density=(double)pixel_value;
	} else {
	  ta=(double)pixel_value*b_scale+b_offset;
	  gray_density=(ta-g_offset)/g_scale;
	}
	if(gray_density<0.0) gray_density=0.0;
	else if(gray_density>1.0) gray_density=1.0;

	PxpSetGray(gray_density);

	PxpGFilledBox(pt_blc,pt_trc);
      }
    }

    break;
  case XY_CUT: default:
    iz=cut_level;
    if(iz<0) iz=0;
    else if(iz>=(*cube).head.axis.z.size) iz=(*cube).head.axis.z.size-1;
    dx=(*cube).head.axis.x.delta/2.0;
    dy=(*cube).head.axis.y.delta/2.0;

    for(ix=i_range.min;ix<=i_range.max;ix++) {
      pt_blc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.x),ix)-dx;
      pt_trc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.x),ix)+dx;
      pt_blc.x=cut_by_limit(h_limit,pt_blc.x);
      pt_trc.x=cut_by_limit(h_limit,pt_trc.x);

      for(iy=j_range.min;iy<=j_range.max;iy++) {
	pt_blc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.y),iy)-dy;
	pt_trc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.y),iy)+dy;
	pt_blc.y=cut_by_limit(v_limit,pt_blc.y);
	pt_trc.y=cut_by_limit(v_limit,pt_trc.y);

       	pixel_value=StarGetCubePixel(cube,ix,iy,iz);

	if(pixel_value==dummy_value) {
	  gray_density=(double)pixel_value;
	} else {
	  ta=(double)pixel_value*b_scale+b_offset;
	  gray_density=(ta-g_offset)/g_scale;
	}
	if(gray_density<0.0) gray_density=0.0;
	else if(gray_density>1.0) gray_density=1.0;

	PxpSetGray(gray_density);

	PxpGFilledBox(pt_blc,pt_trc);
      }
    }

  }
  PxpSetGray(1.0);

  return;
}

void StarPlotCubeGrayBar(cube,plot_box,gray_range)
CUBE *cube; /* <in> */
BOX_RANGE plot_box; /* <in> */
RANGE gray_range; /* <in> gray_scale; min=0=white, max=1=black */
/*
plot gray scale bar by gray scale cells
2000/ 2/16 T.Handa
*/
{
  double y_bottom,y_top;
  double y_step,y_level;
  double gray_density;
  double ta_offset,ta_span;
  int n_level,i_level,n_label;
  char buffer[BUFFER_SIZE],label_format[BUFFER_SIZE];
  double ta_step,ta;
  PxpRange ta_range,ta_ceil_range,ta_lbl_range;
  double l_tick;

  l_tick=0.15;

/* plot gray bar */
  n_level=100;

/* auto gray range */
  ta_range.min=gray_range.min;
  ta_range.max=gray_range.max;

  if(gray_range.max==gray_range.min) {
    ta_range.min=(*cube).head.data_value.min;
    ta_range.max=(*cube).head.data_value.max;
  }
  if(ta_range.max==ta_range.min) {
    ta_range.min=(*cube).head.data_value.min;
    ta_range.max=(*cube).head.data_value.min+2.0;
  }

/* scaling */
  ta_span=ta_range.max-ta_range.min;
  ta_offset=ta_range.min;
  y_step=fabs(plot_box.y.max-plot_box.y.min)/(double)n_level;
  ta_step=ta_span/(double)n_level;

/* gray scale cells */
  for(i_level=0;i_level<n_level;i_level++) {
    ta=ta_range.min+ta_step*(double)i_level;
    gray_density=(ta-ta_range.min)/ta_span;
    if(ta_span<0.0) gray_density=1.0-gray_density;
    y_level=scale_for_bar(ta,&(plot_box.y),&ta_range);
    y_bottom=y_level-y_step/2.0;
    y_top=y_level+y_step/2.0;

    PxpSetGray(gray_density);
    PxpDrawRectangle(plot_box.x.min,y_bottom,plot_box.x.max,y_top,1);
  }

  PxpSetGray(1.0);
/* outer frame */
  PxpDrawRectangle(plot_box.x.min,plot_box.y.min,plot_box.x.max,plot_box.y.max,0);

/* title */
  sprintf(buffer,"%s(%s)",(*cube).head.data_value.type,(*cube).head.data_value.unit);
  PxpDrawString(plot_box.x.min,plot_box.y.max+0.3,buffer);

/* tickmarks and Ta label */
  if(ta_span<0.0) {
    ta=ta_range.min;
    ta_range.min=ta_range.max;
    ta_range.max=ta;
    ta_span= -ta_span;
  }

  PxpGCeilingRange(ta_range,&ta_ceil_range,&n_label,&ta_step);
  PxpGLabelFormat(label_format,ta_ceil_range,ta_step,' ');
  ta_lbl_range=PxpGEnclosureRange2InteriorRange(ta_ceil_range,ta_range,ta_step);

  for(i_level=0;i_level<n_label;i_level++) {
    ta=ta_lbl_range.min+ta_step*(double)i_level;
    if(ta<=ta_lbl_range.max && ta<=ta_range.max) {
      y_level=scale_for_bar(ta,&(plot_box.y),&ta_range);
      PxpDrawLine(plot_box.x.max-l_tick,y_level,plot_box.x.max,y_level);
      sprintf(buffer,label_format,ta);
      PxpDrawJustifiedString(plot_box.x.max+0.1,y_level,buffer,"left","center");
    }
  }

  return;
}

/***********************/
/*     contour map     */
/***********************/
void StarPlotCubeMapByContours(cube,cut_plane,i_range,j_range,cut_level,plot_box,phys_box,n_contours,contours)
CUBE *cube; /* <in> */
int cut_plane;	/* <in> XY_CUT,YX_CUT,YZ_CUT,ZY_CUT,ZX_CUT,XZ_CUT */
RANGE_long i_range,j_range;	/* <in> correspoided pixel range (sorted) */
int cut_level; /* <in> cut position in pixel */
BOX_RANGE plot_box; /* <in> */
BOX_RANGE phys_box; /* <in> min=LeftBottom, max=RightTop */
int n_contours; /* <in> number of countours[] */
Contours *contours; /* <in> */
/*
plot cube-map by contours
2002/ 5/11 T.Handa
2002/ 6/14 T.Handa (add) for color
*/
{
  int ix,iy,iz;
  PxpPoint pt_blc,pt_trc;
  double dx,dy;
  float dummy_value;
  RANGE h_limit,v_limit;
  double ta4[4];
  double c_level;
  int i_cntr,j_cntr;
  char type_name[64];
  int valid_data_flag;

/* data value */
  dummy_value=(*cube).head.data_value.dummy;

  if(n_contours<=1 && contours[0].n==0) {
    n_contours=1;
    contours[0].n=10;
    contours[0].start_level=(double)(*cube).head.data_value.min;
    contours[0].interval=(double)((*cube).head.data_value.max-(*cube).head.data_value.min)/(double)contours[0].n;
  }
  if(contours[0].interval==0.0) {   /* flat image */
    contours[0].n=0;
  }

  h_limit.min=phys_box.x.min;
  h_limit.max=phys_box.x.max;
  v_limit.min=phys_box.y.min;
  v_limit.max=phys_box.y.max;
  StarSortRange(&h_limit);
  StarSortRange(&v_limit);

  switch(cut_plane) {
  case YZ_CUT:
    ix=cut_level;
    if(ix<0) ix=0;
    else if(ix>=(*cube).head.axis.x.size) ix=(*cube).head.axis.x.size-1;
    dx=(*cube).head.axis.y.delta;
    dy=(*cube).head.axis.z.delta;

    for(j_cntr=0;j_cntr<n_contours;j_cntr++) {
      for(i_cntr=0;i_cntr<contours[j_cntr].n;i_cntr++) {
	c_level=contours[j_cntr].start_level+(double)i_cntr*contours[j_cntr].interval;
	line_type_code2name(contours[j_cntr].line_type,type_name,c_level);
	PxpSetLineType(type_name);
	PxpSetLineWidth(contours[j_cntr].thickness);
	PxpSetColor(contours[j_cntr].color);

	for(iz=j_range.min;iz<j_range.max;iz++) {
	  pt_blc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.z),iz);
	  pt_trc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.z),iz)+dy;
	  pt_blc.y=cut_by_limit(v_limit,pt_blc.y);
	  pt_trc.y=cut_by_limit(v_limit,pt_trc.y);

	  for(iy=i_range.min;iy<i_range.max;iy++) {
	    pt_blc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.y),iy);
	    pt_trc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.y),iy)+dx;
	    pt_blc.x=cut_by_limit(h_limit,pt_blc.x);
	    pt_trc.x=cut_by_limit(h_limit,pt_trc.x);

	    ta4[0]=StarGetCubePixelData(cube,ix,iy,iz,&valid_data_flag);
	    ta4[1]=StarGetCubePixelData(cube,ix,iy+1,iz,&valid_data_flag);
	    ta4[2]=StarGetCubePixelData(cube,ix,iy,iz+1,&valid_data_flag);
	    ta4[3]=StarGetCubePixelData(cube,ix,iy+1,iz+1,&valid_data_flag);

	    StarPlotAContourInACell(ta4,pt_blc,pt_trc,c_level);
	  }
	}
      }
    }

    break;
  case ZY_CUT:
    ix=cut_level;
    if(ix<0) ix=0;
    else if(ix>=(*cube).head.axis.x.size) ix=(*cube).head.axis.x.size-1;
    dx=(*cube).head.axis.z.delta;
    dy=(*cube).head.axis.y.delta;

    for(j_cntr=0;j_cntr<n_contours;j_cntr++) {
      for(i_cntr=0;i_cntr<contours[j_cntr].n;i_cntr++) {
	c_level=contours[j_cntr].start_level+(double)i_cntr*contours[j_cntr].interval;
	line_type_code2name(contours[j_cntr].line_type,type_name,c_level);
	PxpSetLineType(type_name);
	PxpSetLineWidth(contours[j_cntr].thickness);
	PxpSetColor(contours[j_cntr].color);

	for(iz=i_range.min;iz<i_range.max;iz++) {
	  pt_blc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.z),iz);
	  pt_trc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.z),iz)+dx;
	  pt_blc.x=cut_by_limit(h_limit,pt_blc.x);
	  pt_trc.x=cut_by_limit(h_limit,pt_trc.x);

	  for(iy=j_range.min;iy<j_range.max;iy++) {
	    pt_blc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.y),iy);
	    pt_trc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.y),iy)+dy;
	    pt_blc.y=cut_by_limit(v_limit,pt_blc.y);
	    pt_trc.y=cut_by_limit(v_limit,pt_trc.y);

	    ta4[0]=StarGetCubePixelData(cube,ix,iy,iz,&valid_data_flag);
	    ta4[1]=StarGetCubePixelData(cube,ix,iy,iz+1,&valid_data_flag);
	    ta4[2]=StarGetCubePixelData(cube,ix,iy+1,iz,&valid_data_flag);
	    ta4[3]=StarGetCubePixelData(cube,ix,iy+1,iz+1,&valid_data_flag);

	    StarPlotAContourInACell(ta4,pt_blc,pt_trc,c_level);
	  }
	}
      }
    }

    break;
  case XZ_CUT:
    iy=cut_level;
    if(iy<0) iy=0;
    else if(iy>=(*cube).head.axis.y.size) iy=(*cube).head.axis.y.size-1;
    dx=(*cube).head.axis.x.delta;
    dy=(*cube).head.axis.z.delta;

    for(j_cntr=0;j_cntr<n_contours;j_cntr++) {
      for(i_cntr=0;i_cntr<contours[j_cntr].n;i_cntr++) {
	c_level=contours[j_cntr].start_level+(double)i_cntr*contours[j_cntr].interval;
	line_type_code2name(contours[j_cntr].line_type,type_name,c_level);
	PxpSetLineType(type_name);
	PxpSetLineWidth(contours[j_cntr].thickness);
	PxpSetColor(contours[j_cntr].color);

	for(ix=i_range.min;ix<i_range.max;ix++) {
	  pt_blc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.x),ix);
	  pt_trc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.x),ix)+dx;
	  pt_blc.x=cut_by_limit(h_limit,pt_blc.x);
	  pt_trc.x=cut_by_limit(h_limit,pt_trc.x);

	  for(iz=j_range.min;iz<j_range.max;iz++) {
	    pt_blc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.z),iz);
	    pt_trc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.z),iz)+dy;
	    pt_blc.y=cut_by_limit(v_limit,pt_blc.y);
	    pt_trc.y=cut_by_limit(v_limit,pt_trc.y);

	    ta4[0]=StarGetCubePixelData(cube,ix,iy,iz,&valid_data_flag);
	    ta4[1]=StarGetCubePixelData(cube,ix+1,iy,iz,&valid_data_flag);
	    ta4[2]=StarGetCubePixelData(cube,ix,iy,iz+1,&valid_data_flag);
	    ta4[3]=StarGetCubePixelData(cube,ix+1,iy,iz+1,&valid_data_flag);

	    StarPlotAContourInACell(ta4,pt_blc,pt_trc,c_level);
	  }
	}
      }
    }

    break;
  case ZX_CUT:
    iy=cut_level;
    if(iy<0) iy=0;
    else if(iy>=(*cube).head.axis.y.size) iy=(*cube).head.axis.y.size-1;
    dx=(*cube).head.axis.z.delta;
    dy=(*cube).head.axis.x.delta;

    for(j_cntr=0;j_cntr<n_contours;j_cntr++) {
      for(i_cntr=0;i_cntr<contours[j_cntr].n;i_cntr++) {
	c_level=contours[j_cntr].start_level+(double)i_cntr*contours[j_cntr].interval;
	line_type_code2name(contours[j_cntr].line_type,type_name,c_level);
	PxpSetLineType(type_name);
	PxpSetLineWidth(contours[j_cntr].thickness);
	PxpSetColor(contours[j_cntr].color);

	for(ix=j_range.min;ix<j_range.max;ix++) {
	  pt_blc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.x),ix);
	  pt_trc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.x),ix)+dy;
	  pt_blc.y=cut_by_limit(v_limit,pt_blc.y);
	  pt_trc.y=cut_by_limit(v_limit,pt_trc.y);

	  for(iz=i_range.min;iz<i_range.max;iz++) {
	    pt_blc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.z),iz);
	    pt_trc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.z),iz)+dx;
	    pt_blc.x=cut_by_limit(h_limit,pt_blc.x);
	    pt_trc.x=cut_by_limit(h_limit,pt_trc.x);

	    ta4[0]=StarGetCubePixelData(cube,ix,iy,iz,&valid_data_flag);
	    ta4[1]=StarGetCubePixelData(cube,ix,iy,iz+1,&valid_data_flag);
	    ta4[2]=StarGetCubePixelData(cube,ix+1,iy,iz,&valid_data_flag);
	    ta4[3]=StarGetCubePixelData(cube,ix+1,iy,iz+1,&valid_data_flag);

	    StarPlotAContourInACell(ta4,pt_blc,pt_trc,c_level);
	  }
	}
      }
    }

    break;
  case YX_CUT:
    iz=cut_level;
    if(iz<0) iz=0;
    else if(iz>=(*cube).head.axis.z.size) iz=(*cube).head.axis.z.size-1;
    dx=(*cube).head.axis.y.delta;
    dy=(*cube).head.axis.x.delta;

    for(j_cntr=0;j_cntr<n_contours;j_cntr++) {
      for(i_cntr=0;i_cntr<contours[j_cntr].n;i_cntr++) {
	c_level=contours[j_cntr].start_level+(double)i_cntr*contours[j_cntr].interval;
	line_type_code2name(contours[j_cntr].line_type,type_name,c_level);
	PxpSetLineType(type_name);
	PxpSetLineWidth(contours[j_cntr].thickness);
	PxpSetColor(contours[j_cntr].color);

	for(ix=j_range.min;ix<j_range.max;ix++) {
	  pt_blc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.x),ix);
	  pt_trc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.x),ix)+dy;
	  pt_blc.y=cut_by_limit(v_limit,pt_blc.y);
	  pt_trc.y=cut_by_limit(v_limit,pt_trc.y);

	  for(iy=i_range.min;iy<i_range.max;iy++) {
	    pt_blc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.y),iy);
	    pt_trc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.y),iy)+dx;
	    pt_blc.x=cut_by_limit(h_limit,pt_blc.x);
	    pt_trc.x=cut_by_limit(h_limit,pt_trc.x);

	    ta4[0]=StarGetCubePixelData(cube,ix,iy,iz,&valid_data_flag);
	    ta4[1]=StarGetCubePixelData(cube,ix,iy+1,iz,&valid_data_flag);
	    ta4[2]=StarGetCubePixelData(cube,ix+1,iy,iz,&valid_data_flag);
	    ta4[3]=StarGetCubePixelData(cube,ix+1,iy+1,iz,&valid_data_flag);

	    StarPlotAContourInACell(ta4,pt_blc,pt_trc,c_level);
	  }
	}
      }
    }

    break;
  case XY_CUT: default:
    iz=cut_level;
    if(iz<0) iz=0;
    else if(iz>=(*cube).head.axis.z.size) iz=(*cube).head.axis.z.size-1;
    dx=(*cube).head.axis.x.delta;
    dy=(*cube).head.axis.y.delta;

    for(j_cntr=0;j_cntr<n_contours;j_cntr++) {
      for(i_cntr=0;i_cntr<contours[j_cntr].n;i_cntr++) {
	c_level=contours[j_cntr].start_level+(double)i_cntr*contours[j_cntr].interval;
	line_type_code2name(contours[j_cntr].line_type,type_name,c_level);
	PxpSetLineType(type_name);
	PxpSetLineWidth(contours[j_cntr].thickness);
	PxpSetColor(contours[j_cntr].color);

	for(ix=i_range.min;ix<i_range.max;ix++) {
	  pt_blc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.x),ix);
	  pt_trc.x=StarPixelValuePositionOnAxis(&((*cube).head.axis.x),ix)+dx;
	  pt_blc.x=cut_by_limit(h_limit,pt_blc.x);
	  pt_trc.x=cut_by_limit(h_limit,pt_trc.x);

	  for(iy=j_range.min;iy<j_range.max;iy++) {
	    pt_blc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.y),iy);
	    pt_trc.y=StarPixelValuePositionOnAxis(&((*cube).head.axis.y),iy)+dy;
	    pt_blc.y=cut_by_limit(v_limit,pt_blc.y);
	    pt_trc.y=cut_by_limit(v_limit,pt_trc.y);

	    ta4[0]=StarGetCubePixelData(cube,ix,iy,iz,&valid_data_flag);
	    ta4[1]=StarGetCubePixelData(cube,ix+1,iy,iz,&valid_data_flag);
	    ta4[2]=StarGetCubePixelData(cube,ix,iy+1,iz,&valid_data_flag);
	    ta4[3]=StarGetCubePixelData(cube,ix+1,iy+1,iz,&valid_data_flag);

	    StarPlotAContourInACell(ta4,pt_blc,pt_trc,c_level);
	  }
	}
      }
    }

  }
  PxpSetColor("black");
  PxpSetLineType("solid");
  PxpSetLineWidth(STD_LINE_WIDTH);

  return;
}

void StarPlotCubeByContoursStd(cube,x_range,y_range,z_range,cut_plane,plot_box,n_contours,contours)
CUBE *cube;
RANGE x_range,y_range,z_range;
int cut_plane;	/* <in> XY_CUT,YX_CUT,YZ_CUT,ZY_CUT,ZX_CUT,XZ_CUT */
BOX_RANGE plot_box; /* <in> */
int n_contours; /* <in> number of countours[] */
Contours *contours; /* <in> */
/*
plot cube-map by contours in the standard format
2002/ 4/22 T.Handa
*/
{
  RANGE_long i_range,j_range;
  BOX_RANGE phys_box;
  RANGE h_range,v_range,cut_range;

  BOX_RANGE plot_cbox,phys_cbox;
  BOX_RANGE plot_box_org,cntrbar_box;
  double h_step,v_step;

  int square_map; /* SQUARE_SCALE_MAP,RECTANGLE_SCALE_MAP */
  RANGE contour_range;

  int cut_level;

  plot_box_org.x.min=plot_box.x.min;
  plot_box_org.x.max=plot_box.x.max;
  plot_box_org.y.min=plot_box.y.min;
  plot_box_org.y.max=plot_box.y.max;

  switch(cut_plane) {
  case YZ_CUT:
    h_range.min=y_range.min;
    h_range.max=y_range.max;
    v_range.min=z_range.min;
    v_range.max=z_range.max;
    cut_range.min=x_range.min;
    cut_range.max=x_range.max;
    cut_level=(int)rint(StarPixelIndexPositionOnAxis(&((*cube).head.axis.x),x_range.min));
    if(cut_level<0)
      cut_level=0;
    else if(cut_level>=(*cube).head.axis.x.size)
      cut_level=(*cube).head.axis.x.size-1;
    break;
  case ZY_CUT:
    h_range.min=z_range.min;
    h_range.max=z_range.max;
    v_range.min=y_range.min;
    v_range.max=y_range.max;
    cut_range.min=x_range.min;
    cut_range.max=x_range.max;
    cut_level=(int)rint(StarPixelIndexPositionOnAxis(&((*cube).head.axis.x),x_range.min));
    if(cut_level<0)
      cut_level=0;
    else if(cut_level>=(*cube).head.axis.x.size)
      cut_level=(*cube).head.axis.x.size-1;
    break;
  case XZ_CUT:
    h_range.min=x_range.min;
    h_range.max=x_range.max;
    v_range.min=z_range.min;
    v_range.max=z_range.max;
    cut_range.min=y_range.min;
    cut_range.max=y_range.max;
    cut_level=(int)rint(StarPixelIndexPositionOnAxis(&((*cube).head.axis.y),y_range.min));
    if(cut_level<0)
      cut_level=0;
    else if(cut_level>=(*cube).head.axis.y.size)
      cut_level=(*cube).head.axis.y.size-1;
    break;
  case ZX_CUT:
    h_range.min=z_range.min;
    h_range.max=z_range.max;
    v_range.min=x_range.min;
    v_range.max=x_range.max;
    cut_range.min=y_range.min;
    cut_range.max=y_range.max;
    cut_level=(int)rint(StarPixelIndexPositionOnAxis(&((*cube).head.axis.y),y_range.min));
    if(cut_level<0)
      cut_level=0;
    else if(cut_level>=(*cube).head.axis.y.size)
      cut_level=(*cube).head.axis.y.size-1;
    break;
  case YX_CUT:
    h_range.min=y_range.min;
    h_range.max=y_range.max;
    v_range.min=x_range.min;
    v_range.max=x_range.max;
    cut_range.min=z_range.min;
    cut_range.max=z_range.max;
    cut_level=(int)rint(StarPixelIndexPositionOnAxis(&((*cube).head.axis.z),z_range.min));
    if(cut_level<0)
      cut_level=0;
    else if(cut_level>=(*cube).head.axis.z.size)
      cut_level=(*cube).head.axis.z.size-1;
    break;
  case XY_CUT: default:
    h_range.min=x_range.min;
    h_range.max=x_range.max;
    v_range.min=y_range.min;
    v_range.max=y_range.max;
    cut_range.min=z_range.min;
    cut_range.max=z_range.max;
    cut_level=(int)rint(StarPixelIndexPositionOnAxis(&((*cube).head.axis.z),z_range.min));
    if(cut_level<0)
      cut_level=0;
    else if(cut_level>=(*cube).head.axis.z.size)
      cut_level=(*cube).head.axis.z.size-1;
  }

  StarCubeMapAutoRange(cube,cut_plane,h_range,v_range,PLOT_CELL_MAP,&i_range,&j_range,&h_step,&v_step,&phys_box,&phys_cbox);

  StarCubeMapLongitudeFlip(cube,cut_plane,&h_step,&phys_box,&phys_cbox);

  StarPlotCubeMapSetRange(plot_box,phys_box);

  square_map=RECTANGLE_SCALE_MAP;
  switch(cut_plane) {
  case YZ_CUT: case ZY_CUT:
    if(strcmp((*cube).head.axis.y.unit,(*cube).head.axis.z.unit)==0)
      square_map=SQUARE_SCALE_MAP;
  case ZX_CUT: case XZ_CUT:
    if(strcmp((*cube).head.axis.z.unit,(*cube).head.axis.x.unit)==0)
      square_map=SQUARE_SCALE_MAP;
    break;
  case XY_CUT: case YX_CUT: default:
    if(strcmp((*cube).head.axis.x.unit,(*cube).head.axis.y.unit)==0)
      square_map=SQUARE_SCALE_MAP;
  }

  if(square_map==SQUARE_SCALE_MAP) {
    StarCubeMapSquareScaleAdjust(&plot_box,phys_box,&phys_cbox,&h_step,&v_step);
  }

  StarPlotCubeMapByContours(cube,cut_plane,i_range,j_range,cut_level,plot_box,phys_box,n_contours,contours);

  contour_range.min=contours[0].start_level;
  contour_range.max=contours[0].start_level+(double)contours[0].n*contours[0].interval;

  StarPlotCubeMapStdFrame(cube,cut_plane,h_range,v_range,cut_range,contour_range,plot_box,phys_box,plot_cbox,phys_cbox,h_step,v_step);

  cntrbar_box.x.min=plot_box_org.x.max+1.0;
  cntrbar_box.x.max=cntrbar_box.x.min+0.5;
  cntrbar_box.y.min=plot_box_org.y.min;
  cntrbar_box.y.max=plot_box_org.y.max;
  StarPlotCubeContourBar(cube,cntrbar_box,n_contours,contours);

  return;
}

void StarPlotAContourInACell(ta,pt_blc,pt_trc,contour_level)
double *ta; /* <in> pixel values at 4 points */
PxpPoint pt_blc,pt_trc; /* <in> position of 4 points */
double contour_level; /* <in> */
/*
plot a contour in a cell with 4 corner values.
2002/4/22 T.Handa
*/
{
  PxpPoint cntr_pt[4]; /* contour points; 4 points max */
  int n_cntr_pt;

  n_cntr_pt=StarContourPointsInACell(ta,pt_blc,pt_trc,contour_level,cntr_pt);

  if(n_cntr_pt<=0) return;
  if(n_cntr_pt==4) {
    PxpGLines(cntr_pt,2);
    PxpGLines(&(cntr_pt[2]),2);
  } else {
    PxpGLines(cntr_pt,n_cntr_pt);
  }

  return;
}

static int StarContourPointsInACell_1(ta,pt_blc,pt_trc,contour_level,cntr)
double *ta; /* <in> pixel values at 4 points */
PxpPoint pt_blc,pt_trc; /* <in> position of 4 points */
double contour_level; /* <in> */
PxpPoint *cntr; /* <out> contour points; 4 points max */ 
/*
make a set of positions for a contour line
(left-bottom conour should be less than the contour level)
2002/4/22 T.Handa
*/
/* 
relative location of 4 points
   ta[0]=(0,0), ta[1]=(1,0), ta[2]=(0,1), ta[3]=(1,1)

   ta[0] must be < contour_level
*/
{
  int n_cntr_pt; /* <return> effective number of cntr[] */

  if(ta[1]>=contour_level) {
    cntr[0].x=StarLinerInterporation(contour_level,ta[0],ta[1],pt_blc.x,pt_trc.x);
    cntr[0].y=pt_blc.y;

    if(ta[2]>=contour_level) {
      if(ta[3]>=contour_level) { /* ta[0] is a dip. */
	n_cntr_pt=2;
	cntr[1].x=pt_blc.x;
	cntr[1].y=StarLinerInterporation(contour_level,ta[0],ta[2],pt_blc.y,pt_trc.y);

      } else { /* ta[3]<contour_level; siddle point, RightBottom-LeftTop ridge */
	n_cntr_pt=4;
	cntr[1].x=pt_blc.x;
	cntr[1].y=StarLinerInterporation(contour_level,ta[0],ta[2],pt_blc.y,pt_trc.y);

	cntr[2].x=pt_trc.x;
	cntr[2].y=StarLinerInterporation(contour_level,ta[1],ta[3],pt_blc.y,pt_trc.y);
	cntr[3].x=StarLinerInterporation(contour_level,ta[2],ta[3],pt_blc.x,pt_trc.x);
	cntr[3].y=pt_trc.y;
      }
    } else { /* ta[2]<contour_level */
      if(ta[3]<contour_level) { /* ta[1] is a peak. */
	n_cntr_pt=2;
	cntr[1].x=pt_trc.x;
	cntr[1].y=StarLinerInterporation(contour_level,ta[1],ta[3],pt_blc.y,pt_trc.y);
      } else { /* ta[3]<contour_level; RL-slop */
	n_cntr_pt=2;
	cntr[1].x=StarLinerInterporation(contour_level,ta[2],ta[3],pt_blc.x,pt_trc.x);
	cntr[1].y=pt_trc.y;
      }
    }
  } else { /* ta[1]<contour_level */
    if(ta[2]>=contour_level) {
      cntr[0].x=pt_blc.x;
      cntr[0].y=StarLinerInterporation(contour_level,ta[0],ta[2],pt_blc.y,pt_trc.y);
      if(ta[3]>=contour_level) { /* TB-slop */
	n_cntr_pt=2;
	cntr[1].x=pt_trc.x;
	cntr[1].y=StarLinerInterporation(contour_level,ta[1],ta[3],pt_blc.y,pt_trc.y);
      } else { /* ta[3]<contour_level; ta[2] is a peak. */
	n_cntr_pt=2;
	cntr[1].x=StarLinerInterporation(contour_level,ta[2],ta[3],pt_blc.x,pt_trc.x);
	cntr[1].y=pt_trc.y;
      }
    } else { /* ta[2]<contour_level */
      if(ta[3]>=contour_level) { /* ta[3] is a peak. */
	n_cntr_pt=2;
	cntr[0].x=pt_trc.x;
	cntr[0].y=StarLinerInterporation(contour_level,ta[1],ta[3],pt_blc.y,pt_trc.y);
	cntr[1].x=StarLinerInterporation(contour_level,ta[2],ta[3],pt_blc.x,pt_trc.x);
	cntr[1].y=pt_trc.y;
      } else { /* ta[3]<contour_level; all points below the contour_level. */
	n_cntr_pt=0;
      }
    }
  }
  return n_cntr_pt;
}

int StarContourPointsInACell(ta,pt_blc,pt_trc,contour_level,cntr_pt)
double *ta; /* <in> pixel values at 4 points */
PxpPoint pt_blc,pt_trc; /* <in> position of 4 points */
double contour_level;
PxpPoint *cntr_pt; /* <out> contour points; 4 points max */ 
/*
make a set of positions for a contour line
2002/4/22 T.Handa
 */
{
  double neg_ta[4],neg_cntr_lvl;
  int ii;
  int n_cntr_pt;


  if(ta[0]>contour_level) {
    for(ii=0;ii<4;ii++) 
      neg_ta[ii]= -ta[ii];
    neg_cntr_lvl= -contour_level;
    n_cntr_pt=StarContourPointsInACell_1(neg_ta,pt_blc,pt_trc,neg_cntr_lvl,cntr_pt);
  } else {
    n_cntr_pt=StarContourPointsInACell_1(ta,pt_blc,pt_trc,contour_level,cntr_pt);
  }

  return n_cntr_pt;
}

void StarPlotCubeContourBar(cube,plot_box,n_contours,contours)
CUBE *cube; /* <in> */
BOX_RANGE plot_box; /* <in> */
int n_contours; /* <in> number of countours[] */
Contours *contours; /* <in> */
/*
plot contour level bar
2002/ 4/22 T.Handa
2002/ 6/ 6 T.Handa (debug)
*/
{
  double y_level;
  int i_level,n_label;
  int i_cntr,j_cntr;
  double c_level;
  char buffer[BUFFER_SIZE],label_format[BUFFER_SIZE];
  double ta_step,ta,ta_span;
  RANGE cntr_1set_range;
  PxpRange ta_range,ta_bar_range,ta_ceil_range,ta_lbl_range;
  double l_tick;
  char type_name[64];
  double min_interval;

  l_tick=0.15;

/* plot a contour level bar */

/* auto full range */
  if(n_contours<=1 && contours[0].n==0) {
    ta_range.min=(*cube).head.data_value.min;
    ta_range.max=(*cube).head.data_value.max;
    ta=fabs(ta_range.max-ta_range.min)/50.0;
    min_interval=ta*2.0;
  } else {
    ta_range.min=contours[0].start_level;
    ta_range.max=ta_range.min+contours[0].interval*(double)(contours[0].n-1);
    min_interval=contours[0].interval;
    if(ta_range.min>ta_range.max) {
      ta=ta_range.min;
      ta_range.min=ta_range.max;
      ta_range.max=ta;
    }
    for(j_cntr=1;j_cntr<n_contours;j_cntr++) {
      cntr_1set_range.min=contours[j_cntr].start_level;
      cntr_1set_range.max=cntr_1set_range.min+contours[j_cntr].interval*(double)(contours[j_cntr].n-1);
      StarSortRange(&cntr_1set_range);
      if(ta_range.min>cntr_1set_range.min)
	ta_range.min=cntr_1set_range.min;
      if(ta_range.max<cntr_1set_range.max)
	ta_range.max=cntr_1set_range.max;
      ta_step=fabs(contours[j_cntr].interval);
      if(min_interval>ta_step) 
	min_interval=ta_step;
    }
    ta=fabs(min_interval)/2.0;
  }
  ta_bar_range.min=ta_range.min-ta;
  ta_bar_range.max=ta_range.max+ta;

/* scaling */
  ta_span=ta_range.max-ta_range.min;

/* draw levels */
  for(j_cntr=0;j_cntr<n_contours;j_cntr++) {
    for(i_cntr=0;i_cntr<contours[j_cntr].n;i_cntr++) {
      c_level=contours[j_cntr].start_level+(double)i_cntr*contours[j_cntr].interval;
      if(fabs(c_level)<min_interval*0.01) c_level=0.0; /* force to set 0.0 */
      line_type_code2name(contours[j_cntr].line_type,type_name,c_level);
      PxpSetLineType(type_name);
      PxpSetLineWidth(contours[j_cntr].thickness);
      PxpSetColor(contours[j_cntr].color);

      y_level=scale_for_bar(c_level,&(plot_box.y),&ta_bar_range);
      PxpDrawLine(plot_box.x.min,y_level,plot_box.x.max,y_level);
    }
  }
  PxpSetColor("black");
  PxpSetLineType("solid");
  PxpSetLineWidth(STD_LINE_WIDTH);

/* outer frame */
  PxpDrawRectangle(plot_box.x.min,plot_box.y.min,plot_box.x.max,plot_box.y.max,0);

/* title */
  sprintf(buffer,"%s(%s)",(*cube).head.data_value.type,(*cube).head.data_value.unit);
  PxpDrawString(plot_box.x.min,plot_box.y.max+0.3,buffer);

/* tickmarks and Ta label */
  if(ta_span<0.0) {
    ta=ta_range.min;
    ta_range.min=ta_range.max;
    ta_range.max=ta;
    ta_span= -ta_span;
  }

  PxpGCeilingRange(ta_range,&ta_ceil_range,&n_label,&ta_step);
  PxpGLabelFormat(label_format,ta_ceil_range,ta_step,' ');
  ta_lbl_range=PxpGEnclosureRange2InteriorRange(ta_ceil_range,ta_range,ta_step);

  for(i_level=0;i_level<n_label;i_level++) {
    ta=ta_step*(double)i_level+ta_lbl_range.min;
    if(ta<=ta_lbl_range.max && ta<=ta_range.max) {
      y_level=scale_for_bar(ta,&(plot_box.y),&ta_bar_range);
      PxpDrawLine(plot_box.x.max-l_tick,y_level,plot_box.x.max,y_level);
      sprintf(buffer,label_format,ta);
      PxpDrawJustifiedString(plot_box.x.max+0.1,y_level,buffer,"left","center");
    }
  }

  return;
}

/***********************/
/* common for any maps */
/***********************/
void StarCubeMapSquareScaleAdjust(plot_box,phys_box,phys_enclosure_box,h_step,v_step)
BOX_RANGE *plot_box; /* <i/o> */
BOX_RANGE phys_box; /* <in> */
BOX_RANGE *phys_enclosure_box; /* <i/o> */
double *h_step,*v_step; /* <i/o> */
/*
plot range and step adjust for a square scale map
2000/ 2/16 T.Handa
*/
{
  double h_scale,v_scale;
  double h_abscale,v_abscale;
  double h_abstep,v_abstep;
  PxpPoint new_trc,phys_trc;
  int adjust_axis;
  RANGE new_enclosure;

/* adjust plot scale */
  PxpGGetScale(&h_scale,&v_scale);
  h_abscale=fabs(h_scale);
  v_abscale=fabs(v_scale);
  if(h_abscale>v_abscale)
    h_abscale=v_abscale;
  else
    v_abscale=h_abscale;
  if(h_scale<0.0)
    h_scale= -h_abscale;
  else
    h_scale=h_abscale;
  if(v_scale<0.0)
    v_scale= -v_abscale;
  else
    v_scale=v_abscale;

  PxpGPutScale(h_scale,v_scale);

/* adjust phys_box */
  phys_trc.x=phys_box.x.max;
  phys_trc.y=phys_box.y.max;
  new_trc=PxpGPlotPosition(phys_trc);
  (*plot_box).x.max=new_trc.x;
  (*plot_box).y.max=new_trc.y;

/* adjust label step */
  h_abstep=fabs(*h_step);
  v_abstep=fabs(*v_step);
  if(h_abstep<v_abstep) {
    h_abstep=v_abstep;
    adjust_axis=1;
  } else {
    v_abstep=h_abstep;
    adjust_axis= -1;
  }

  if(*h_step<0.0)
    *h_step= -h_abstep;
  else
    *h_step=h_abstep;
  if(*v_step<0.0)
    *v_step= -v_abstep;
  else
    *v_step=v_abstep;

/* adjust enclosure box */
  if(adjust_axis>0) {
    new_enclosure=StarEnclosureRange(phys_box.x,*h_step);
    (*phys_enclosure_box).x.min=new_enclosure.min;
    (*phys_enclosure_box).x.max=new_enclosure.max;
    if(new_enclosure.min<new_enclosure.max)
      *h_step=fabs(*h_step);
    else
      *h_step= -fabs(*h_step);
  } else {
    new_enclosure=StarEnclosureRange(phys_box.y,*v_step);
    (*phys_enclosure_box).y.min=new_enclosure.min;
    (*phys_enclosure_box).y.max=new_enclosure.max;
    if(new_enclosure.min<new_enclosure.max)
      *v_step=fabs(*v_step);
    else
      *v_step= -fabs(*v_step);
  }

  return;
}

void StarPlotCubeByGrayCellsStd(cube,x_range,y_range,z_range,cut_plane,plot_box,gray_range)
CUBE *cube;
RANGE x_range,y_range,z_range;
int cut_plane;	/* <in> XY_CUT,YX_CUT,YZ_CUT,ZY_CUT,ZX_CUT,XZ_CUT */
BOX_RANGE plot_box; /* <in> */
RANGE gray_range; /* <in> gray_scale; min=0=white, max=1=black */
/*
plot cube-map by gray scale cells in the standard format
2000/ 2/16 T.Handa
*/
{
  RANGE_long i_range,j_range;
  BOX_RANGE phys_box;
  RANGE h_range,v_range,cut_range;

  BOX_RANGE plot_cbox,phys_cbox;
  BOX_RANGE plot_box_org,graybar_box;
  double h_step,v_step;

  int square_map; /* SQUARE_SCALE_MAP,RECTANGLE_SCALE_MAP */

  int cut_level;

  plot_box_org.x.min=plot_box.x.min;
  plot_box_org.x.max=plot_box.x.max;
  plot_box_org.y.min=plot_box.y.min;
  plot_box_org.y.max=plot_box.y.max;

  switch(cut_plane) {
  case YZ_CUT:
    h_range.min=y_range.min;
    h_range.max=y_range.max;
    v_range.min=z_range.min;
    v_range.max=z_range.max;
    cut_range.min=x_range.min;
    cut_range.max=x_range.max;
    cut_level=(int)rint(StarPixelIndexPositionOnAxis(&((*cube).head.axis.x),x_range.min));
    if(cut_level<0)
      cut_level=0;
    else if(cut_level>=(*cube).head.axis.x.size)
      cut_level=(*cube).head.axis.x.size-1;
    break;
  case ZY_CUT:
    h_range.min=z_range.min;
    h_range.max=z_range.max;
    v_range.min=y_range.min;
    v_range.max=y_range.max;
    cut_range.min=x_range.min;
    cut_range.max=x_range.max;
    cut_level=(int)rint(StarPixelIndexPositionOnAxis(&((*cube).head.axis.x),x_range.min));
    if(cut_level<0)
      cut_level=0;
    else if(cut_level>=(*cube).head.axis.x.size)
      cut_level=(*cube).head.axis.x.size-1;
    break;
  case XZ_CUT:
    h_range.min=x_range.min;
    h_range.max=x_range.max;
    v_range.min=z_range.min;
    v_range.max=z_range.max;
    cut_range.min=y_range.min;
    cut_range.max=y_range.max;
    cut_level=(int)rint(StarPixelIndexPositionOnAxis(&((*cube).head.axis.y),y_range.min));
    if(cut_level<0)
      cut_level=0;
    else if(cut_level>=(*cube).head.axis.y.size)
      cut_level=(*cube).head.axis.y.size-1;
    break;
  case ZX_CUT:
    h_range.min=z_range.min;
    h_range.max=z_range.max;
    v_range.min=x_range.min;
    v_range.max=x_range.max;
    cut_range.min=y_range.min;
    cut_range.max=y_range.max;
    cut_level=(int)rint(StarPixelIndexPositionOnAxis(&((*cube).head.axis.y),y_range.min));
    if(cut_level<0)
      cut_level=0;
    else if(cut_level>=(*cube).head.axis.y.size)
      cut_level=(*cube).head.axis.y.size-1;
    break;
  case YX_CUT:
    h_range.min=y_range.min;
    h_range.max=y_range.max;
    v_range.min=x_range.min;
    v_range.max=x_range.max;
    cut_range.min=z_range.min;
    cut_range.max=z_range.max;
    cut_level=(int)rint(StarPixelIndexPositionOnAxis(&((*cube).head.axis.z),z_range.min));
    if(cut_level<0)
      cut_level=0;
    else if(cut_level>=(*cube).head.axis.z.size)
      cut_level=(*cube).head.axis.z.size-1;
    break;
  case XY_CUT: default:
    h_range.min=x_range.min;
    h_range.max=x_range.max;
    v_range.min=y_range.min;
    v_range.max=y_range.max;
    cut_range.min=z_range.min;
    cut_range.max=z_range.max;
    cut_level=(int)rint(StarPixelIndexPositionOnAxis(&((*cube).head.axis.z),z_range.min));
    if(cut_level<0)
      cut_level=0;
    else if(cut_level>=(*cube).head.axis.z.size)
      cut_level=(*cube).head.axis.z.size-1;
  }

  StarCubeMapAutoRange(cube,cut_plane,h_range,v_range,PLOT_CELL_MAP,&i_range,&j_range,&h_step,&v_step,&phys_box,&phys_cbox);

  StarCubeMapLongitudeFlip(cube,cut_plane,&h_step,&phys_box,&phys_cbox);

  StarPlotCubeMapSetRange(plot_box,phys_box);

  square_map=RECTANGLE_SCALE_MAP;
  switch(cut_plane) {
  case YZ_CUT: case ZY_CUT:
    if(strcmp((*cube).head.axis.y.unit,(*cube).head.axis.z.unit)==0)
      square_map=SQUARE_SCALE_MAP;
  case ZX_CUT: case XZ_CUT:
    if(strcmp((*cube).head.axis.z.unit,(*cube).head.axis.x.unit)==0)
      square_map=SQUARE_SCALE_MAP;
    break;
  case XY_CUT: case YX_CUT: default:
    if(strcmp((*cube).head.axis.x.unit,(*cube).head.axis.y.unit)==0)
      square_map=SQUARE_SCALE_MAP;
  }

  if(square_map==SQUARE_SCALE_MAP) {
    StarCubeMapSquareScaleAdjust(&plot_box,phys_box,&phys_cbox,&h_step,&v_step);
  }

  StarPlotCubeMapByGrayCells(cube,cut_plane,i_range,j_range,cut_level,plot_box,phys_box,gray_range);

  StarPlotCubeMapStdFrame(cube,cut_plane,h_range,v_range,cut_range,gray_range,plot_box,phys_box,plot_cbox,phys_cbox,h_step,v_step);

  graybar_box.x.min=plot_box_org.x.max+1.0;
  graybar_box.x.max=graybar_box.x.min+0.5;
  graybar_box.y.min=plot_box_org.y.min;
  graybar_box.y.max=plot_box_org.y.max;
  StarPlotCubeGrayBar(cube,graybar_box,gray_range);

  return;
}



void StarResetCubeMapShowColor(cntr_clr,frm_clr,ttl_clr)
/*
Set line colors
2000/2/11 T.Handa
*/
char *cntr_clr; /* <in> for contour lines */
char *frm_clr; /* <in> for frame and labels on axes */
char *ttl_clr; /* <in> for title name characters */
{
  int l_str;

  l_str=strlen(cntr_clr);
  if(l_str>0 && l_str<L_COLOR_STRING)
    strcpy(contour_color,cntr_clr);
  l_str=strlen(frm_clr);
  if(l_str>0 && l_str<L_COLOR_STRING)
    strcpy(frame_color,frm_clr);
  l_str=strlen(ttl_clr);
  if(l_str>0 && l_str<L_COLOR_STRING)
    strcpy(title_color,ttl_clr);
  return;
}

void StarGetCubeMapShowColor(cntr_clr,frm_clr,ttl_clr)
/*
Get line colors
2000/2/11 T.Handa
*/
char *cntr_clr; /* <out> for contour lines */
char *frm_clr; /* <out> for frame and labels on axes */
char *ttl_clr; /* <out> for cube name characters */
{
  strcpy(cntr_clr,contour_color);
  strcpy(frm_clr,frame_color);
  strcpy(ttl_clr,title_color);
  return;
}
