/*
STAR observational data I/O library
Part 1. internal call functions
for PROFILE, CUBE

1992/4/16 T.Handa UNIX MiniSTAR
1992/12/19 T.Handa
1995/2/23 Tomo OKA	add Scaling factor, hereafter sys_head.version=2
1996/2/24 T.Handa : for data ver3
1997/11/14 T.Handa ver.3.0
1998/5/7 T.Handa ver.3.01
1998/10/31 T.Handa ver.3.02 (debug) Cube
1999/ 4/26 T.Handa ver.3.03 (adjust ver.4)
1999/ 7/23 T.Handa keep version
2001/ 9/28 T.Handa ver.3.04 (debug) strcpy() for equinox
*/

#ifdef _NO_PROTO

static int	read_ant_status();
static int 	read_baseline();
static int 	read_be_status();
static int 	read_history();
static int 	read_object();
static int 	read_obs_log();
static int 	read_obs_prm();
static int 	read_prof();
static int 	read_rx_status();
static int	read_spurious();
static int 	read_sys_head();

static int	write_ant_status();
static int 	write_baseline();
static int 	write_be_status();
static int 	write_history();
static int 	write_object();
static int 	write_obs_log();
static int 	write_obs_prm();
static int 	write_prof();
static int 	write_rx_status();
static int	write_spurious();
static int 	write_sys_head();

#else /* undef _NO_PROTO */

static int	read_ant_status(FILE *, SYS_HEAD *, ANT_STATUS *);
static int 	read_baseline(FILE *, SYS_HEAD *, int, unsigned char *);
static int 	read_be_status(FILE *, SYS_HEAD *, BE_STATUS *);
static int 	read_history(FILE *, SYS_HEAD *, HISTORY *);
static int 	read_object(FILE *, SYS_HEAD *, OBJECT *);
static int 	read_obs_log(FILE *, SYS_HEAD *, OBS_LOG *);
static int 	read_obs_prm(FILE *, SYS_HEAD *, OBS_PRM *);
static int 	read_prof(FILE *, SYS_HEAD *, int, float *);
static int 	read_rx_status(FILE *, SYS_HEAD *, RX_STATUS *);
static int	read_spurious(FILE *, SYS_HEAD *, int, unsigned char *);
static int 	read_sys_head(FILE *, SYS_HEAD *);

static int	write_ant_status(FILE *, SYS_HEAD *, ANT_STATUS *);
static int 	write_baseline(FILE *, SYS_HEAD *, int, unsigned char *);
static int 	write_be_status(FILE *, SYS_HEAD *, BE_STATUS *);
static int 	write_history(FILE *, SYS_HEAD *, HISTORY *);
static int 	write_object(FILE *, SYS_HEAD *, OBJECT *);
static int 	write_obs_log(FILE *, SYS_HEAD *, OBS_LOG *);
static int 	write_obs_prm(FILE *, SYS_HEAD *, OBS_PRM *);
static int 	write_prof(FILE *, SYS_HEAD *, int, float *);
static int 	write_rx_status(FILE *, SYS_HEAD *, RX_STATUS *);
static int	write_spurious(FILE *, SYS_HEAD *, int, unsigned char *);
static int 	write_sys_head(FILE *, SYS_HEAD *);

#endif /* endif of define _NO_PROTO */

#ifdef _NO_PROTO

static int read_HeadAxis();
static int read_HeadAxes();
static int read_HeadDValue();

static int write_HeadAxis();
static int write_HeadAxes();
static int write_HeadDValue();

#else /* undef _NO_PROTO */

static int read_HeadAxis(FILE *, SYS_HEAD *, AXIS *);
static int read_HeadAxes(FILE *, SYS_HEAD *, AXES *);
static int read_HeadDValue(FILE *, SYS_HEAD *, DATA_VALUE *);

static int write_HeadAxis(FILE *, SYS_HEAD *, AXIS *);
static int write_HeadAxes(FILE *, SYS_HEAD *, AXES *);
static int write_HeadDValue(FILE *, SYS_HEAD *, DATA_VALUE *);

#endif /* endif of define _NO_PROTO */

#ifdef INTEL_CPU
/*------ for INTEL CPU ------*/

static size_t freadIntel(pt,size,nn,fp)
char *pt;
size_t size;
int nn;
FILE *fp;
/*
Read Binary data written by Motrollor/MIPS CPU with Intel
1998/4/21 T.Handa
1998/7/14 T.Handa (return value revised)
*/
{
  char *buffer;
  size_t ret_code;
  int total_byte,ii,jj;

  if(size==1) {
    ret_code=fread(pt,1,nn,fp);
    return(ret_code);
  }

  total_byte=size*nn;
  buffer=malloc(total_byte);

  ret_code=fread(buffer,1,total_byte,fp);
  if(ret_code==total_byte) ret_code=nn;
  else ret_code=0;

  for(ii=0;ii<nn;ii++) {
    for(jj=0;jj<size;jj++) 
      pt[ii*size+jj]=buffer[(ii+1)*size-jj-1];
  }

  free(buffer);

  return(ret_code);
}

static size_t fwriteIntel(pt,size,nn,fp)
char *pt;
size_t size;
int nn;
FILE *fp;
/*
Write Binary data readable by Motrollor/MIPS CPU with Intel
1998/4/21 T.Handa
1998/7/14 T.Handa (return value revised)
*/
{
  char *buffer;
  size_t ret_code;
  int total_byte,ii,jj;

  if(size==1) {
    ret_code=fwrite(pt,1,nn,fp);
    return(ret_code);
  }

  total_byte=size*nn;
  buffer=malloc(total_byte);

  for(ii=0;ii<nn;ii++) {
    for(jj=0;jj<size;jj++)
      buffer[(ii+1)*size-jj-1]=pt[ii*size+jj];
  }

  ret_code=fwrite(buffer,1,total_byte,fp);
  if(ret_code==total_byte) 
    ret_code=nn;
  else 
    ret_code=0;

  free(buffer);
  return(ret_code);
}
#endif

/*------ for PROFILE, CUBE_HEAD ------*/

static int read_sys_head(fp,sys_head)
/*
1992/4/16 T.Handa for UNIX MiniSTAR
1992/9/19 T.Handa for UNIX STAR
1997/5/16 T.Handa 17byte for Cube-ver1
1999/7/23 T.Handa version number check
*/
FILE *fp;
SYS_HEAD *sys_head;
{
  char	dummy[1];
  int org_version;

#ifdef INTEL_CPU
if(fp==stdin) {
#endif
  fread(&(*sys_head).l_file,sizeof((*sys_head).l_file),1,fp);
  fread(&(*sys_head).l_head,sizeof((*sys_head).l_head),1,fp);
  fread(&(*sys_head).format_id,sizeof((*sys_head).format_id),1,fp);
  fread(&(*sys_head).version,sizeof((*sys_head).version),1,fp);
  fread(&(*sys_head).data_type,sizeof((*sys_head).data_type),1,fp);
  fread(&(*sys_head).hist_step,sizeof((*sys_head).hist_step),1,fp);
  fread(&(*sys_head).hist_line,sizeof((*sys_head).hist_line),1,fp);
  fread(&(*sys_head).scan,sizeof((*sys_head).scan),1,fp);
  fread(&(*sys_head).backup,sizeof((*sys_head).backup),1,fp);
  fread(&(*sys_head).num_scan,sizeof((*sys_head).num_scan),1,fp);
  fread(dummy,sizeof(dummy[0]),1,fp);
#ifdef INTEL_CPU
} else {
  freadIntel(&(*sys_head).l_file,sizeof((*sys_head).l_file),1,fp);
  freadIntel(&(*sys_head).l_head,sizeof((*sys_head).l_head),1,fp);
  freadIntel(&(*sys_head).format_id,sizeof((*sys_head).format_id),1,fp);
  freadIntel(&(*sys_head).version,sizeof((*sys_head).version),1,fp);
  freadIntel(&(*sys_head).data_type,sizeof((*sys_head).data_type),1,fp);
  freadIntel(&(*sys_head).hist_step,sizeof((*sys_head).hist_step),1,fp);
  freadIntel(&(*sys_head).hist_line,sizeof((*sys_head).hist_line),1,fp);
  freadIntel(&(*sys_head).scan,sizeof((*sys_head).scan),1,fp);
  freadIntel(&(*sys_head).backup,sizeof((*sys_head).backup),1,fp);
  freadIntel(&(*sys_head).num_scan,sizeof((*sys_head).num_scan),1,fp);
  freadIntel(dummy,sizeof(dummy[0]),1,fp);
}
#endif
  org_version=(*sys_head).version;
  if((*sys_head).format_id==SPECTRUM && (*sys_head).version>SCAN_FORMAT_VERSION){
    (*sys_head).version=SCAN_FORMAT_VERSION;
    fprintf(stderr,"Unsupported version %d: assume ver=%d\n",org_version,(*sys_head).version);
  }
  return(0);
}


static int write_sys_head(fp,sys_head)
/*
1992/4/16 T.Handa for UNIX MiniSTAR
1992/9/19 T.Handa for UNIX STAR
1995/2/23 Tomo OKA add Scaling factor (sys_head.version=2)
1996/2/24 T.Handa data ver3 : auto correction
1997/5/16 T.Handa 17byte for Cube-ver1
1999/7/23 T.Handa remove auto correction
*/
FILE *fp;
SYS_HEAD *sys_head;
{
  static char	dummy[1]={0x00};

/* data version auto up-date */
#ifdef INTEL_CPU
if(fp==stdout) {
#endif
  fwrite(&(*sys_head).l_file,sizeof((*sys_head).l_file),1,fp);
  fwrite(&(*sys_head).l_head,sizeof((*sys_head).l_head),1,fp);
  fwrite(&(*sys_head).format_id,sizeof((*sys_head).format_id),1,fp);
  fwrite(&(*sys_head).version,sizeof((*sys_head).version),1,fp);
  fwrite(&(*sys_head).data_type,sizeof((*sys_head).data_type),1,fp);
  fwrite(&(*sys_head).hist_step,sizeof((*sys_head).hist_step),1,fp);
  fwrite(&(*sys_head).hist_line,sizeof((*sys_head).hist_line),1,fp);
  fwrite(&(*sys_head).scan,sizeof((*sys_head).scan),1,fp);
  fwrite(&(*sys_head).backup,sizeof((*sys_head).backup),1,fp);
  fwrite(&(*sys_head).num_scan,sizeof((*sys_head).num_scan),1,fp);
  fwrite(dummy,sizeof(dummy[0]),1,fp);
#ifdef INTEL_CPU
} else {
  fwriteIntel(&(*sys_head).l_file,sizeof((*sys_head).l_file),1,fp);
  fwriteIntel(&(*sys_head).l_head,sizeof((*sys_head).l_head),1,fp);
  fwriteIntel(&(*sys_head).format_id,sizeof((*sys_head).format_id),1,fp);
  fwriteIntel(&(*sys_head).version,sizeof((*sys_head).version),1,fp);
  fwriteIntel(&(*sys_head).data_type,sizeof((*sys_head).data_type),1,fp);
  fwriteIntel(&(*sys_head).hist_step,sizeof((*sys_head).hist_step),1,fp);
  fwriteIntel(&(*sys_head).hist_line,sizeof((*sys_head).hist_line),1,fp);
  fwriteIntel(&(*sys_head).scan,sizeof((*sys_head).scan),1,fp);
  fwriteIntel(&(*sys_head).backup,sizeof((*sys_head).backup),1,fp);
  fwriteIntel(&(*sys_head).num_scan,sizeof((*sys_head).num_scan),1,fp);
  fwriteIntel(dummy,sizeof(dummy[0]),1,fp);
}
#endif

  return(0);
}


static int 	read_obs_prm(fp,sys,obs_prm)
/*
1992/4/16 T.Handa for UNIX MiniSTAR
1992/9/19 T.Handa for UNIX STAR
1992/12/7 T.Handa Add NULL to make char[]
1997/5/16 T.Handa 75byte for Cube-ver1
1999/4/26 T.Handa for ver.4
*/
FILE *fp;
SYS_HEAD *sys;
OBS_PRM *obs_prm;
{
	char dummy[1];

#ifdef INTEL_CPU
if(fp==stdin) {
#endif
  fread(&(*obs_prm).project[0],sizeof(char),16,fp);
  (*obs_prm).project[16]=(char)NULL;
  fread(&(*obs_prm).scd_file[0],sizeof(char),8,fp);
  (*obs_prm).scd_file[8]=(char)NULL;
  fread(&(*obs_prm).line_numb,sizeof((*obs_prm).line_numb),1,fp);
  fread(&(*obs_prm).sw_mode,sizeof(char),1,fp);
  if((*sys).version<=3) {
    fread(dummy,sizeof(dummy[0]),1,fp);
  }
  fread(&(*obs_prm).mjd,sizeof((*obs_prm).mjd),1,fp);
  fread(&(*obs_prm).start,sizeof((*obs_prm).start),1,fp);
  fread(&(*obs_prm).stop,sizeof((*obs_prm).stop),1,fp);
  fread(&(*obs_prm).on_time,sizeof((*obs_prm).on_time),1,fp);
  fread(&(*obs_prm).off_time,sizeof((*obs_prm).off_time),1,fp);
  fread(&(*obs_prm).observer1[0],sizeof(char),8,fp);
  fread(&(*obs_prm).observer2[0],sizeof(char),8,fp);
  fread(&(*obs_prm).observer3[0],sizeof(char),8,fp);
  fread(&(*obs_prm).observer4[0],sizeof(char),8,fp);
  (*obs_prm).observer1[8]=(char)NULL;
  (*obs_prm).observer2[8]=(char)NULL;
  (*obs_prm).observer3[8]=(char)NULL;
  (*obs_prm).observer4[8]=(char)NULL;
#ifdef INTEL_CPU
} else {
  freadIntel(&(*obs_prm).project[0],sizeof(char),16,fp);
  (*obs_prm).project[16]=(char)NULL;
  freadIntel(&(*obs_prm).scd_file[0],sizeof(char),8,fp);
  (*obs_prm).scd_file[8]=(char)NULL;
  freadIntel(&(*obs_prm).line_numb,sizeof((*obs_prm).line_numb),1,fp);
  freadIntel(&(*obs_prm).sw_mode,sizeof(char),1,fp);
  if((*sys).version<=3) {
    freadIntel(dummy,sizeof(dummy[0]),1,fp);
  }
  freadIntel(&(*obs_prm).mjd,sizeof((*obs_prm).mjd),1,fp);
  freadIntel(&(*obs_prm).start,sizeof((*obs_prm).start),1,fp);
  freadIntel(&(*obs_prm).stop,sizeof((*obs_prm).stop),1,fp);
  freadIntel(&(*obs_prm).on_time,sizeof((*obs_prm).on_time),1,fp);
  freadIntel(&(*obs_prm).off_time,sizeof((*obs_prm).off_time),1,fp);
  freadIntel(&(*obs_prm).observer1[0],sizeof(char),8,fp);
  freadIntel(&(*obs_prm).observer2[0],sizeof(char),8,fp);
  freadIntel(&(*obs_prm).observer3[0],sizeof(char),8,fp);
  freadIntel(&(*obs_prm).observer4[0],sizeof(char),8,fp);
  (*obs_prm).observer1[8]=(char)NULL;
  (*obs_prm).observer2[8]=(char)NULL;
  (*obs_prm).observer3[8]=(char)NULL;
  (*obs_prm).observer4[8]=(char)NULL;
}
#endif

  return(0);
}


static int 	write_obs_prm(fp,sys,obs_prm)
/*
1992/4/16 T.Handa for UNIX MiniSTAR
1997/5/16 T.Handa 75byte for Cube-ver1
1999/4/26 T.Handa for ver.4
*/
FILE *fp;
SYS_HEAD *sys;
OBS_PRM *obs_prm;
{
  static char dummy[1]={0x00};

#ifdef INTEL_CPU
if(fp==stdout) {
#endif
  fwrite(&(*obs_prm).project[0],sizeof(char),16,fp);
  fwrite(&(*obs_prm).scd_file[0],sizeof(char),8,fp);
  fwrite(&(*obs_prm).line_numb,sizeof((*obs_prm).line_numb),1,fp);
  fwrite(&(*obs_prm).sw_mode,sizeof(char),1,fp);
  if((*sys).version<=3) {
    fwrite(dummy,sizeof(dummy[0]),1,fp);
  }
  fwrite(&(*obs_prm).mjd,sizeof((*obs_prm).mjd),1,fp);
  fwrite(&(*obs_prm).start,sizeof((*obs_prm).start),1,fp);
  fwrite(&(*obs_prm).stop,sizeof((*obs_prm).stop),1,fp);
  fwrite(&(*obs_prm).on_time,sizeof((*obs_prm).on_time),1,fp);
  fwrite(&(*obs_prm).off_time,sizeof((*obs_prm).off_time),1,fp);
  fwrite(&(*obs_prm).observer1[0],sizeof(char),8,fp);
  fwrite(&(*obs_prm).observer2[0],sizeof(char),8,fp);
  fwrite(&(*obs_prm).observer3[0],sizeof(char),8,fp);
  fwrite(&(*obs_prm).observer4[0],sizeof(char),8,fp);
#ifdef INTEL_CPU
} else {
  fwriteIntel(&(*obs_prm).project[0],sizeof(char),16,fp);
  fwriteIntel(&(*obs_prm).scd_file[0],sizeof(char),8,fp);
  fwriteIntel(&(*obs_prm).line_numb,sizeof((*obs_prm).line_numb),1,fp);
  fwriteIntel(&(*obs_prm).sw_mode,sizeof(char),1,fp);
  if((*sys).version<=3) {
    fwriteIntel(dummy,sizeof(dummy[0]),1,fp);
  }
  fwriteIntel(&(*obs_prm).mjd,sizeof((*obs_prm).mjd),1,fp);
  fwriteIntel(&(*obs_prm).start,sizeof((*obs_prm).start),1,fp);
  fwriteIntel(&(*obs_prm).stop,sizeof((*obs_prm).stop),1,fp);
  fwriteIntel(&(*obs_prm).on_time,sizeof((*obs_prm).on_time),1,fp);
  fwriteIntel(&(*obs_prm).off_time,sizeof((*obs_prm).off_time),1,fp);
  fwriteIntel(&(*obs_prm).observer1[0],sizeof(char),8,fp);
  fwriteIntel(&(*obs_prm).observer2[0],sizeof(char),8,fp);
  fwriteIntel(&(*obs_prm).observer3[0],sizeof(char),8,fp);
  fwriteIntel(&(*obs_prm).observer4[0],sizeof(char),8,fp);
}
#endif

  return(0);
}


static int 	read_object(fp,sys,object)
/*
1992/2/22 T.Oka
1992/4/16 T.Handa UNIX MiniSTAR
1992/9/19 T.Handa for UNIX STAR
1992/12/7 T.Handa add NULL for char[]
1997/5/16 T.Handa 136byte for Cube-ver1
1999/4/26 T.Handa for ver.4
*/
FILE *fp;
SYS_HEAD *sys;
OBJECT *object;
{
	char dummy[16];

#ifdef INTEL_CPU
if(fp==stdin) {
#endif
  fread(&(*object).name[0],sizeof(char),15,fp);
  (*object).name[15]=(char)NULL;
  fread(&(*object).cood_sys,sizeof((*object).cood_sys),1,fp);
  fread(&(*object).on_lb.x,sizeof((*object).on_lb.x),1,fp);
  fread(&(*object).on_lb.y,sizeof((*object).on_lb.y),1,fp);
  fread(&(*object).on_radec.x,sizeof((*object).on_radec.x),1,fp);
  fread(&(*object).on_radec.y,sizeof((*object).on_radec.y),1,fp);
  if((*sys).version>3) {
    fread(&(*object).equinox[0],sizeof(char),8,fp);
    (*object).equinox[8]=(char)NULL;
  } else {
    strcpy((*object).equinox,"B1950.0");
  }
  fread(&(*object).on_azel.x,sizeof((*object).on_azel.x),1,fp);
  fread(&(*object).on_azel.y,sizeof((*object).on_azel.y),1,fp);
  fread(&(*object).off_lb.x,sizeof((*object).off_lb.x),1,fp);
  fread(&(*object).off_lb.y,sizeof((*object).off_lb.y),1,fp);
  fread(&(*object).off_radec.x,sizeof((*object).off_radec.x),1,fp);
  fread(&(*object).off_radec.y,sizeof((*object).off_radec.y),1,fp);
  fread(&(*object).orig_xy.x,sizeof((*object).orig_xy.x),1,fp);
  fread(&(*object).orig_xy.y,sizeof((*object).orig_xy.y),1,fp);
  fread(&(*object).pa_xy,sizeof((*object).pa_xy),1,fp);
  if((*sys).version<=3) {
    fread(dummy,sizeof(dummy[0]),8,fp);
  }
  fread(&(*object).on_xy.x,sizeof((*object).on_xy.x),1,fp);
  fread(&(*object).on_xy.y,sizeof((*object).on_xy.y),1,fp);
  if((*sys).version>3) {
    fread(&(*object).azel_offset.x,sizeof((*object).azel_offset.x),1,fp);
    fread(&(*object).azel_offset.y,sizeof((*object).azel_offset.y),1,fp);
  } else {
    (*object).azel_offset.x=0.0;
    (*object).azel_offset.y=0.0;
    fread(dummy,sizeof(dummy[0]),16,fp);
  }
#ifdef INTEL_CPU
} else {
  freadIntel(&(*object).name[0],sizeof(char),15,fp);
  (*object).name[15]=(char)NULL;
  freadIntel(&(*object).cood_sys,sizeof((*object).cood_sys),1,fp);
  freadIntel(&(*object).on_lb.x,sizeof((*object).on_lb.x),1,fp);
  freadIntel(&(*object).on_lb.y,sizeof((*object).on_lb.y),1,fp);
  freadIntel(&(*object).on_radec.x,sizeof((*object).on_radec.x),1,fp);
  freadIntel(&(*object).on_radec.y,sizeof((*object).on_radec.y),1,fp);
  if((*sys).version>3) {
    freadIntel(&(*object).equinox[0],sizeof(char),8,fp);
    (*object).equinox[8]=(char)NULL;
  } else {
    strcpy((*object).equinox,"B1950.0");
  }
  freadIntel(&(*object).on_azel.x,sizeof((*object).on_azel.x),1,fp);
  freadIntel(&(*object).on_azel.y,sizeof((*object).on_azel.y),1,fp);
  freadIntel(&(*object).off_lb.x,sizeof((*object).off_lb.x),1,fp);
  freadIntel(&(*object).off_lb.y,sizeof((*object).off_lb.y),1,fp);
  freadIntel(&(*object).off_radec.x,sizeof((*object).off_radec.x),1,fp);
  freadIntel(&(*object).off_radec.y,sizeof((*object).off_radec.y),1,fp);
  freadIntel(&(*object).orig_xy.x,sizeof((*object).orig_xy.x),1,fp);
  freadIntel(&(*object).orig_xy.y,sizeof((*object).orig_xy.y),1,fp);
  freadIntel(&(*object).pa_xy,sizeof((*object).pa_xy),1,fp);
  if((*sys).version<=3) {
    freadIntel(dummy,sizeof(dummy[0]),8,fp);
  }
  freadIntel(&(*object).on_xy.x,sizeof((*object).on_xy.x),1,fp);
  freadIntel(&(*object).on_xy.y,sizeof((*object).on_xy.y),1,fp);
  if((*sys).version>3) {
    freadIntel(&(*object).azel_offset.x,sizeof((*object).azel_offset.x),1,fp);
    freadIntel(&(*object).azel_offset.y,sizeof((*object).azel_offset.y),1,fp);
  } else {
    (*object).azel_offset.x=0.0;
    (*object).azel_offset.y=0.0;
    freadIntel(dummy,sizeof(dummy[0]),16,fp);
  }
}
#endif

  return(0);	
}


static int write_object(fp,sys,object)
/*
1992/2/22 T.Oka
1992/4/16 T.Handa UNIX MiniSTAR
1997/5/16 T.Handa 136byte for Cube-ver1
*/
FILE *fp;
SYS_HEAD *sys;
OBJECT *object;
{
  static char dummy[16]= {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
			  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};

#ifdef INTEL_CPU
if(fp==stdout) {
#endif
  fwrite(&(*object).name[0],sizeof(char),15,fp);
  fwrite(&(*object).cood_sys,sizeof((*object).cood_sys),1,fp);
  fwrite(&(*object).on_lb.x,sizeof((*object).on_lb.x),1,fp);
  fwrite(&(*object).on_lb.y,sizeof((*object).on_lb.y),1,fp);
  fwrite(&(*object).on_radec.x,sizeof((*object).on_radec.x),1,fp);
  fwrite(&(*object).on_radec.y,sizeof((*object).on_radec.y),1,fp);
  if((*sys).version>3) {
    fwrite(&(*object).equinox[0],sizeof(char),8,fp);
  }
  fwrite(&(*object).on_azel.x,sizeof((*object).on_azel.x),1,fp);
  fwrite(&(*object).on_azel.y,sizeof((*object).on_azel.y),1,fp);
  fwrite(&(*object).off_lb.x,sizeof((*object).off_lb.x),1,fp);
  fwrite(&(*object).off_lb.y,sizeof((*object).off_lb.y),1,fp);
  fwrite(&(*object).off_radec.x,sizeof((*object).off_radec.x),1,fp);
  fwrite(&(*object).off_radec.y,sizeof((*object).off_radec.y),1,fp);
  fwrite(&(*object).orig_xy.x,sizeof((*object).orig_xy.x),1,fp);
  fwrite(&(*object).orig_xy.y,sizeof((*object).orig_xy.y),1,fp);
  fwrite(&(*object).pa_xy,sizeof((*object).pa_xy),1,fp);
  if((*sys).version<=3) {
    fwrite(dummy,sizeof(dummy[0]),8,fp);
  }
  fwrite(&(*object).on_xy.x,sizeof((*object).on_xy.x),1,fp);
  fwrite(&(*object).on_xy.y,sizeof((*object).on_xy.y),1,fp);
  if((*sys).version>3) {
    fwriteIntel(&(*object).azel_offset.x,sizeof((*object).azel_offset.x),1,fp);
    fwriteIntel(&(*object).azel_offset.y,sizeof((*object).azel_offset.y),1,fp);
  } else {
    fwrite(dummy,sizeof(dummy[0]),16,fp);
  }
#ifdef INTEL_CPU
} else {
  fwriteIntel(&(*object).name[0],sizeof(char),15,fp);
  fwriteIntel(&(*object).cood_sys,sizeof((*object).cood_sys),1,fp);
  fwriteIntel(&(*object).on_lb.x,sizeof((*object).on_lb.x),1,fp);
  fwriteIntel(&(*object).on_lb.y,sizeof((*object).on_lb.y),1,fp);
  fwriteIntel(&(*object).on_radec.x,sizeof((*object).on_radec.x),1,fp);
  fwriteIntel(&(*object).on_radec.y,sizeof((*object).on_radec.y),1,fp);
  if((*sys).version>3) {
    fwriteIntel(&(*object).equinox[0],sizeof(char),8,fp);
  }
  fwriteIntel(&(*object).on_azel.x,sizeof((*object).on_azel.x),1,fp);
  fwriteIntel(&(*object).on_azel.y,sizeof((*object).on_azel.y),1,fp);
  fwriteIntel(&(*object).off_lb.x,sizeof((*object).off_lb.x),1,fp);
  fwriteIntel(&(*object).off_lb.y,sizeof((*object).off_lb.y),1,fp);
  fwriteIntel(&(*object).off_radec.x,sizeof((*object).off_radec.x),1,fp);
  fwriteIntel(&(*object).off_radec.y,sizeof((*object).off_radec.y),1,fp);
  fwriteIntel(&(*object).orig_xy.x,sizeof((*object).orig_xy.x),1,fp);
  fwriteIntel(&(*object).orig_xy.y,sizeof((*object).orig_xy.y),1,fp);
  fwriteIntel(&(*object).pa_xy,sizeof((*object).pa_xy),1,fp);
  if((*sys).version<=3) {
    fwriteIntel(dummy,sizeof(dummy[0]),8,fp);
  }
  fwriteIntel(&(*object).on_xy.x,sizeof((*object).on_xy.x),1,fp);
  fwriteIntel(&(*object).on_xy.y,sizeof((*object).on_xy.y),1,fp);
  if((*sys).version>3) {
    fwriteIntel(&(*object).azel_offset.x,sizeof((*object).azel_offset.x),1,fp);
    fwriteIntel(&(*object).azel_offset.y,sizeof((*object).azel_offset.y),1,fp);
  } else {
    fwriteIntel(dummy,sizeof(dummy[0]),16,fp);
  }
}
#endif

  return(0);
}


static int read_ant_status(fp,sys,ant_status)
/*
1992/2/22 Tomoharu Oka
1992/4/16 T.Handa UNIX MiniSTAR
1992/9/19 T.Handa for UNIX STAR
1992/12/7 T.Handa add NULL for char[]
1997/5/16 T.Handa 24byte for Cube-ver1
1999/4/26 T.Handa for ver.4
*/
FILE *fp;
SYS_HEAD *sys;
ANT_STATUS *ant_status;
{
  char dummy[8];

#ifdef INTEL_CPU
if(fp==stdin) {
#endif
  fread(&(*ant_status).tele[0],sizeof(char),8,fp);
  (*ant_status).tele[8]=(char)NULL;
  fread(&(*ant_status).ap_eff,sizeof((*ant_status).ap_eff),1,fp);
  fread(&(*ant_status).mb_eff,sizeof((*ant_status).mb_eff),1,fp);
  fread(&(*ant_status).fss_eff,sizeof((*ant_status).fss_eff),1,fp);
  fread(&(*ant_status).hpbw,sizeof((*ant_status).hpbw),1,fp);
  if((*sys).version<=3)
    fread(dummy,sizeof(dummy[0]),8,fp);
#ifdef INTEL_CPU
} else {
  freadIntel(&(*ant_status).tele[0],sizeof(char),8,fp);
  (*ant_status).tele[8]=(char)NULL;
  freadIntel(&(*ant_status).ap_eff,sizeof((*ant_status).ap_eff),1,fp);
  freadIntel(&(*ant_status).mb_eff,sizeof((*ant_status).mb_eff),1,fp);
  freadIntel(&(*ant_status).fss_eff,sizeof((*ant_status).fss_eff),1,fp);
  freadIntel(&(*ant_status).hpbw,sizeof((*ant_status).hpbw),1,fp);
  if((*sys).version<=3)
    freadIntel(dummy,sizeof(dummy[0]),8,fp);
}
#endif

  return(0);
}


static int write_ant_status(fp,sys,ant_status)
/*
1992/2/22 Tomoharu Oka
1992/4/16 T.Handa UNIX MiniSTAR
1997/5/16 T.Handa 24byte for Cube-ver1
1999/4/26 T.Handa for ver.4
*/
FILE		*fp;
SYS_HEAD *sys;
ANT_STATUS	*ant_status;
{
  static char	dummy[8]={0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};

#ifdef INTEL_CPU
if(fp==stdout) {
#endif
  fwrite(&(*ant_status).tele[0],sizeof(char),8,fp);
  fwrite(&(*ant_status).ap_eff,sizeof((*ant_status).ap_eff),1,fp);
  fwrite(&(*ant_status).mb_eff,sizeof((*ant_status).mb_eff),1,fp);
  fwrite(&(*ant_status).fss_eff,sizeof((*ant_status).fss_eff),1,fp);
  fwrite(&(*ant_status).hpbw,sizeof((*ant_status).hpbw),1,fp);
  if((*sys).version<=3)
    fwrite(dummy,sizeof(dummy[0]),8,fp);
#ifdef INTEL_CPU
} else {
  fwriteIntel(&(*ant_status).tele[0],sizeof(char),8,fp);
  fwriteIntel(&(*ant_status).ap_eff,sizeof((*ant_status).ap_eff),1,fp);
  fwriteIntel(&(*ant_status).mb_eff,sizeof((*ant_status).mb_eff),1,fp);
  fwriteIntel(&(*ant_status).fss_eff,sizeof((*ant_status).fss_eff),1,fp);
  fwriteIntel(&(*ant_status).hpbw,sizeof((*ant_status).hpbw),1,fp);
  if((*sys).version<=3)
    fwriteIntel(dummy,sizeof(dummy[0]),8,fp);
}
#endif

  return(0);
}


static int read_rx_status(fp,sys,rx_status)
/*
1992/4/16 T.Handa UNIX MiniSTAR
1992/9/19 T.Handa for UNIX STAR
1992/12/7 T.Handa add NULL for char[]
1995/2/23 Tomo OKA add Scaling factor (for sys_head.version=2)
1996/2/24 T.Handa data ver3
1997/5/16 T.Handa 118byte for Cube-ver1
1999/4/26 T.Handa for ver4
*/
FILE *fp;
SYS_HEAD *sys;
RX_STATUS *rx_status;
{
  char dummy[4];

#ifdef INTEL_CPU
if(fp==stdin) {
#endif
  fread(&(*rx_status).name[0],sizeof(char),7,fp);
  (*rx_status).name[7]=(char)NULL;
  fread(&(*rx_status).sb_mode,sizeof((*rx_status).sb_mode),1,fp);
  fread(&(*rx_status).v_lsr,sizeof((*rx_status).v_lsr),1,fp);
  if((*sys).format_id!=SPECTRUM || (*sys).version>2) {
    fread(&(*rx_status).v_off,sizeof((*rx_status).v_off),1,fp);
  } else {
    (*rx_status).v_off=(*rx_status).v_lsr;
  }
  fread(&(*rx_status).rest_freq,sizeof((*rx_status).rest_freq),1,fp);
  fread(&(*rx_status).obs_freq,sizeof((*rx_status).obs_freq),1,fp);
  fread(&(*rx_status).lo_freq1,sizeof((*rx_status).lo_freq1),1,fp);
  fread(&(*rx_status).if_freq1,sizeof((*rx_status).if_freq1),1,fp);
  fread(&(*rx_status).lo_freq2,sizeof((*rx_status).lo_freq2),1,fp);
  fread(&(*rx_status).if_freq2,sizeof((*rx_status).if_freq2),1,fp);
  fread(&(*rx_status).lo_freq3,sizeof((*rx_status).lo_freq3),1,fp);
  fread(&(*rx_status).if_freq3,sizeof((*rx_status).if_freq3),1,fp);
  fread(&(*rx_status).mltplx1,sizeof((*rx_status).mltplx1),1,fp);
  fread(&(*rx_status).mltplx2,sizeof((*rx_status).mltplx2),1,fp);
  fread(&(*rx_status).mltplx3,sizeof((*rx_status).mltplx3),1,fp);
  if((*sys).version<=3)
    fread(dummy,sizeof(dummy[0]),1,fp);
  fread(&(*rx_status).tsys,sizeof((*rx_status).tsys),1,fp);
  fread(&(*rx_status).trx,sizeof((*rx_status).trx),1,fp);
  fread(&(*rx_status).rms,sizeof((*rx_status).rms),1,fp);
  if((*sys).version<=3)
    fread(dummy,sizeof(dummy[0]),4,fp);
  fread(&(*rx_status).calib,sizeof((*rx_status).calib),1,fp);
  fread(&(*rx_status).weight,sizeof((*rx_status).weight),1,fp);
  if((*sys).format_id!=SPECTRUM || (*sys).version>1) {
    fread(&(*rx_status).scale,sizeof((*rx_status).scale),1,fp);
  }else{
    (*rx_status).scale=1.0;
  }
#ifdef INTEL_CPU
} else {
  freadIntel(&(*rx_status).name[0],sizeof(char),7,fp);
  (*rx_status).name[7]=(char)NULL;
  freadIntel(&(*rx_status).sb_mode,sizeof((*rx_status).sb_mode),1,fp);
  freadIntel(&(*rx_status).v_lsr,sizeof((*rx_status).v_lsr),1,fp);
  if((*sys).format_id!=SPECTRUM || (*sys).version>2) {
    freadIntel(&(*rx_status).v_off,sizeof((*rx_status).v_off),1,fp);
  } else {
    (*rx_status).v_off=(*rx_status).v_lsr;
  }
  freadIntel(&(*rx_status).rest_freq,sizeof((*rx_status).rest_freq),1,fp);
  freadIntel(&(*rx_status).obs_freq,sizeof((*rx_status).obs_freq),1,fp);
  freadIntel(&(*rx_status).lo_freq1,sizeof((*rx_status).lo_freq1),1,fp);
  freadIntel(&(*rx_status).if_freq1,sizeof((*rx_status).if_freq1),1,fp);
  freadIntel(&(*rx_status).lo_freq2,sizeof((*rx_status).lo_freq2),1,fp);
  freadIntel(&(*rx_status).if_freq2,sizeof((*rx_status).if_freq2),1,fp);
  freadIntel(&(*rx_status).lo_freq3,sizeof((*rx_status).lo_freq3),1,fp);
  freadIntel(&(*rx_status).if_freq3,sizeof((*rx_status).if_freq3),1,fp);
  freadIntel(&(*rx_status).mltplx1,sizeof((*rx_status).mltplx1),1,fp);
  freadIntel(&(*rx_status).mltplx2,sizeof((*rx_status).mltplx2),1,fp);
  freadIntel(&(*rx_status).mltplx3,sizeof((*rx_status).mltplx3),1,fp);
  if((*sys).version<=3)
    freadIntel(dummy,sizeof(dummy[0]),1,fp);
  freadIntel(&(*rx_status).tsys,sizeof((*rx_status).tsys),1,fp);
  freadIntel(&(*rx_status).trx,sizeof((*rx_status).trx),1,fp);
  freadIntel(&(*rx_status).rms,sizeof((*rx_status).rms),1,fp);
  if((*sys).version<=3)
    freadIntel(dummy,sizeof(dummy[0]),4,fp);
  freadIntel(&(*rx_status).calib,sizeof((*rx_status).calib),1,fp);
  freadIntel(&(*rx_status).weight,sizeof((*rx_status).weight),1,fp);
  if((*sys).format_id!=SPECTRUM || (*sys).version>1) {
    freadIntel(&(*rx_status).scale,sizeof((*rx_status).scale),1,fp);
  }else{
    (*rx_status).scale=1.0;
  }
}
#endif
  return(0);
}


static int write_rx_status(fp,sys,rx_status)
/*
1992/4/16 T.Handa UNIX MiniSTAR
1992/9/19 T.Handa for UNIX STAR
1995/2/23 Tomo OKA add Scaling factor(sys_head.version=2)
1996/2/24 T.Handa	data ver3
1997/5/16 T.Handa 118byte for Cube-ver1
1999/4/26 T.Handa for ver.4
*/
FILE *fp;
SYS_HEAD *sys;
RX_STATUS *rx_status;
{
  static char dummy[4]={0x00,0x00,0x00,0x00};

#ifdef INTEL_CPU
if(fp==stdout) {
#endif
  fwrite(&(*rx_status).name[0],sizeof(char),7,fp);
  fwrite(&(*rx_status).sb_mode,sizeof((*rx_status).sb_mode),1,fp);
  fwrite(&(*rx_status).v_lsr,sizeof((*rx_status).v_lsr),1,fp);
  if((*sys).format_id!=SPECTRUM || (*sys).version>2){
    fwrite(&(*rx_status).v_off,sizeof((*rx_status).v_off),1,fp);
  }
  fwrite(&(*rx_status).rest_freq,sizeof((*rx_status).rest_freq),1,fp);
  fwrite(&(*rx_status).obs_freq,sizeof((*rx_status).obs_freq),1,fp);
  fwrite(&(*rx_status).lo_freq1,sizeof((*rx_status).lo_freq1),1,fp);
  fwrite(&(*rx_status).if_freq1,sizeof((*rx_status).if_freq1),1,fp);
  fwrite(&(*rx_status).lo_freq2,sizeof((*rx_status).lo_freq2),1,fp);
  fwrite(&(*rx_status).if_freq2,sizeof((*rx_status).if_freq2),1,fp);
  fwrite(&(*rx_status).lo_freq3,sizeof((*rx_status).lo_freq3),1,fp);
  fwrite(&(*rx_status).if_freq3,sizeof((*rx_status).if_freq3),1,fp);
  fwrite(&(*rx_status).mltplx1,sizeof((*rx_status).mltplx1),1,fp);
  fwrite(&(*rx_status).mltplx2,sizeof((*rx_status).mltplx2),1,fp);
  fwrite(&(*rx_status).mltplx3,sizeof((*rx_status).mltplx3),1,fp);
  if((*sys).version<=3)
    fwrite(dummy,sizeof(dummy[0]),1,fp);
  fwrite(&(*rx_status).tsys,sizeof((*rx_status).tsys),1,fp);
  fwrite(&(*rx_status).trx,sizeof((*rx_status).trx),1,fp);
  fwrite(&(*rx_status).rms,sizeof((*rx_status).rms),1,fp);
  if((*sys).version<=3)
    fwrite(dummy,sizeof(dummy[0]),4,fp);
  fwrite(&(*rx_status).calib,sizeof((*rx_status).calib),1,fp);
  fwrite(&(*rx_status).weight,sizeof((*rx_status).weight),1,fp);
  if((*sys).format_id!=SPECTRUM || (*sys).version>1){
    fwrite(&(*rx_status).scale,sizeof((*rx_status).scale),1,fp);
  }
#ifdef INTEL_CPU
} else {
  fwriteIntel(&(*rx_status).name[0],sizeof(char),7,fp);
  fwriteIntel(&(*rx_status).sb_mode,sizeof((*rx_status).sb_mode),1,fp);
  fwriteIntel(&(*rx_status).v_lsr,sizeof((*rx_status).v_lsr),1,fp);
  if((*sys).format_id!=SPECTRUM || (*sys).version>2){
    fwriteIntel(&(*rx_status).v_off,sizeof((*rx_status).v_off),1,fp);
  }
  fwriteIntel(&(*rx_status).rest_freq,sizeof((*rx_status).rest_freq),1,fp);
  fwriteIntel(&(*rx_status).obs_freq,sizeof((*rx_status).obs_freq),1,fp);
  fwriteIntel(&(*rx_status).lo_freq1,sizeof((*rx_status).lo_freq1),1,fp);
  fwriteIntel(&(*rx_status).if_freq1,sizeof((*rx_status).if_freq1),1,fp);
  fwriteIntel(&(*rx_status).lo_freq2,sizeof((*rx_status).lo_freq2),1,fp);
  fwriteIntel(&(*rx_status).if_freq2,sizeof((*rx_status).if_freq2),1,fp);
  fwriteIntel(&(*rx_status).lo_freq3,sizeof((*rx_status).lo_freq3),1,fp);
  fwriteIntel(&(*rx_status).if_freq3,sizeof((*rx_status).if_freq3),1,fp);
  fwriteIntel(&(*rx_status).mltplx1,sizeof((*rx_status).mltplx1),1,fp);
  fwriteIntel(&(*rx_status).mltplx2,sizeof((*rx_status).mltplx2),1,fp);
  fwriteIntel(&(*rx_status).mltplx3,sizeof((*rx_status).mltplx3),1,fp);
  if((*sys).version<=3)
    fwriteIntel(dummy,sizeof(dummy[0]),1,fp);
  fwriteIntel(&(*rx_status).tsys,sizeof((*rx_status).tsys),1,fp);
  fwriteIntel(&(*rx_status).trx,sizeof((*rx_status).trx),1,fp);
  fwriteIntel(&(*rx_status).rms,sizeof((*rx_status).rms),1,fp);
  if((*sys).version<=3)
    fwriteIntel(dummy,sizeof(dummy[0]),4,fp);
  fwriteIntel(&(*rx_status).calib,sizeof((*rx_status).calib),1,fp);
  fwriteIntel(&(*rx_status).weight,sizeof((*rx_status).weight),1,fp);
  if((*sys).format_id!=SPECTRUM || (*sys).version>1){
    fwriteIntel(&(*rx_status).scale,sizeof((*rx_status).scale),1,fp);
  }
}
#endif
  return(0);
}


static int read_be_status(fp,sys,be_status)
/*
1991/4/16 T.Handa UNIX MiniSTAR
1992/9/19 T.Handa for UNIX STAR
1992/12/7 T.Handa add NULL for char[]
1996/2/24 T.Handa	data ver3
1997/5/16 T.Handa 105byte for Cube-ver1
1999/4/26 T.Handa for ver.4
*/
FILE *fp;
SYS_HEAD *sys;
BE_STATUS *be_status;
{
  char dummy[1];

#ifdef INTEL_CPU
if(fp==stdin) {
#endif
  fread(&(*be_status).name[0],sizeof(char),8,fp);
  (*be_status).name[8]=(char)NULL;
  fread(&(*be_status).be_ch,sizeof((*be_status).be_ch),1,fp);

  if((*be_status).be_ch==0)
    (*be_status).be_ch=MAX_CH; /* debug for ASTROS 1.11 */

  fread(&(*be_status).basis_ch,sizeof((*be_status).basis_ch),1,fp);
  fread(&(*be_status).sign,sizeof((*be_status).sign),1,fp);
  if((*sys).version<=3)
    fread(dummy,sizeof(dummy[0]),1,fp);
  fread(&(*be_status).data_ch,sizeof((*be_status).data_ch),1,fp);

  if((*be_status).data_ch==0)
    (*be_status).data_ch=MAX_CH; /* debug for ASTROS 1.11 */

  if((*sys).format_id!=SPECTRUM || (*sys).version>2) {
    fread(&(*be_status).center_ch,sizeof((*be_status).center_ch),1,fp);
  } else {
    (*be_status).center_ch=(double)(*be_status).basis_ch;
  }
  fread(&(*be_status).center_freq,sizeof((*be_status).center_freq),1,fp);
  fread(&(*be_status).disp0,sizeof((*be_status).disp0),1,fp);
  fread(&(*be_status).disp1,sizeof((*be_status).disp1),1,fp);
  fread(&(*be_status).disp2,sizeof((*be_status).disp2),1,fp);
  fread(&(*be_status).disp3,sizeof((*be_status).disp3),1,fp);
  fread(&(*be_status).disp4,sizeof((*be_status).disp4),1,fp);
  fread(&(*be_status).freq_res,sizeof((*be_status).freq_res),1,fp);
  fread(&(*be_status).ch_width,sizeof((*be_status).ch_width),1,fp);
  fread(&(*be_status).ch_orig,sizeof((*be_status).ch_orig),1,fp);
#ifdef INTEL_CPU
} else {
  freadIntel(&(*be_status).name[0],sizeof(char),8,fp);
  (*be_status).name[8]=(char)NULL;
  freadIntel(&(*be_status).be_ch,sizeof((*be_status).be_ch),1,fp);

  if((*be_status).be_ch==0)
    (*be_status).be_ch=MAX_CH; /* debug for ASTROS 1.11 */

  freadIntel(&(*be_status).basis_ch,sizeof((*be_status).basis_ch),1,fp);
  freadIntel(&(*be_status).sign,sizeof((*be_status).sign),1,fp);
  if((*sys).version<=3)
    freadIntel(dummy,sizeof(dummy[0]),1,fp);
  freadIntel(&(*be_status).data_ch,sizeof((*be_status).data_ch),1,fp);

  if((*be_status).data_ch==0)
    (*be_status).data_ch=MAX_CH; /* debug for ASTROS 1.11 */

  if((*sys).format_id!=SPECTRUM || (*sys).version>2) {
    freadIntel(&(*be_status).center_ch,sizeof((*be_status).center_ch),1,fp);
  } else {
    (*be_status).center_ch=(double)(*be_status).basis_ch;
  }
  freadIntel(&(*be_status).center_freq,sizeof((*be_status).center_freq),1,fp);
  freadIntel(&(*be_status).disp0,sizeof((*be_status).disp0),1,fp);
  freadIntel(&(*be_status).disp1,sizeof((*be_status).disp1),1,fp);
  freadIntel(&(*be_status).disp2,sizeof((*be_status).disp2),1,fp);
  freadIntel(&(*be_status).disp3,sizeof((*be_status).disp3),1,fp);
  freadIntel(&(*be_status).disp4,sizeof((*be_status).disp4),1,fp);
  freadIntel(&(*be_status).freq_res,sizeof((*be_status).freq_res),1,fp);
  freadIntel(&(*be_status).ch_width,sizeof((*be_status).ch_width),1,fp);
  freadIntel(&(*be_status).ch_orig,sizeof((*be_status).ch_orig),1,fp);
}
#endif

  return(0);
}


static int write_be_status(fp,sys,be_status)
/*
1991/4/16 T.Handa UNIX MiniSTAR
1996/2/24 T.Handa	data ver3
1997/5/16 T.Handa 105byte for Cube-ver1
1999/4/26 T.Handa for ver.4
*/
FILE *fp;
SYS_HEAD *sys;
BE_STATUS *be_status;
{
  static char dummy=0x00;

#ifdef INTEL_CPU
if(fp==stdout) {
#endif
  fwrite(&(*be_status).name[0],sizeof(char),8,fp);
  fwrite(&(*be_status).be_ch,sizeof((*be_status).be_ch),1,fp);
  fwrite(&(*be_status).basis_ch,sizeof((*be_status).basis_ch),1,fp);
  fwrite(&(*be_status).sign,sizeof((*be_status).sign),1,fp);
  if((*sys).version<=3)
    fwrite(&dummy,sizeof(dummy),1,fp);
  fwrite(&(*be_status).data_ch,sizeof((*be_status).data_ch),1,fp);
  if((*sys).format_id!=SPECTRUM || (*sys).version>2) {
    fwrite(&(*be_status).center_ch,sizeof((*be_status).center_ch),1,fp);
  }
  fwrite(&(*be_status).center_freq,sizeof((*be_status).center_freq),1,fp);
  fwrite(&(*be_status).disp0,sizeof((*be_status).disp0),1,fp);
  fwrite(&(*be_status).disp1,sizeof((*be_status).disp1),1,fp);
  fwrite(&(*be_status).disp2,sizeof((*be_status).disp2),1,fp);
  fwrite(&(*be_status).disp3,sizeof((*be_status).disp3),1,fp);
  fwrite(&(*be_status).disp4,sizeof((*be_status).disp4),1,fp);
  fwrite(&(*be_status).freq_res,sizeof((*be_status).freq_res),1,fp);
  fwrite(&(*be_status).ch_width,sizeof((*be_status).ch_width),1,fp);
  fwrite(&(*be_status).ch_orig,sizeof((*be_status).ch_orig),1,fp);
#ifdef INTEL_CPU
} else {
  fwriteIntel(&(*be_status).name[0],sizeof(char),8,fp);
  fwriteIntel(&(*be_status).be_ch,sizeof((*be_status).be_ch),1,fp);
  fwriteIntel(&(*be_status).basis_ch,sizeof((*be_status).basis_ch),1,fp);
  fwriteIntel(&(*be_status).sign,sizeof((*be_status).sign),1,fp);
  if((*sys).version<=3)
    fwriteIntel(&dummy,sizeof(dummy),1,fp);
  fwriteIntel(&(*be_status).data_ch,sizeof((*be_status).data_ch),1,fp);
  if((*sys).format_id!=SPECTRUM || (*sys).version>2) {
    fwriteIntel(&(*be_status).center_ch,sizeof((*be_status).center_ch),1,fp);
  }
  fwriteIntel(&(*be_status).center_freq,sizeof((*be_status).center_freq),1,fp);
  fwriteIntel(&(*be_status).disp0,sizeof((*be_status).disp0),1,fp);
  fwriteIntel(&(*be_status).disp1,sizeof((*be_status).disp1),1,fp);
  fwriteIntel(&(*be_status).disp2,sizeof((*be_status).disp2),1,fp);
  fwriteIntel(&(*be_status).disp3,sizeof((*be_status).disp3),1,fp);
  fwriteIntel(&(*be_status).disp4,sizeof((*be_status).disp4),1,fp);
  fwriteIntel(&(*be_status).freq_res,sizeof((*be_status).freq_res),1,fp);
  fwriteIntel(&(*be_status).ch_width,sizeof((*be_status).ch_width),1,fp);
  fwriteIntel(&(*be_status).ch_orig,sizeof((*be_status).ch_orig),1,fp);
}
#endif

  return(0);
}


static int read_obs_log(fp,sys,obs_log)
/*
1992/2/22 Tomoharu Oka
1992/4/16 T.Handa UNIX MiniSTAR
1992/9/19 T.Handa for UNIX STAR
1992/12/7 T.Handa add NULL for char[]
1997/5/16 T.Handa 152byte for Cube-ver1
1999/4/26 T.Handa for ver.4
*/
FILE *fp;
SYS_HEAD *sys;
OBS_LOG *obs_log;
{
  char dummy[8];
  float tmp,press,water;

#ifdef INTEL_CPU
if(fp==stdin) {
#endif
  fread(&(*obs_log).site_name[0],sizeof(char),8,fp);
  (*obs_log).site_name[8]=(char)NULL;
  fread(&tmp,sizeof(tmp),1,fp);
  fread(&press,sizeof(press),1,fp);
  fread(&water,sizeof(water),1,fp);
  fread(&(*obs_log).tau0,sizeof((*obs_log).tau0),1,fp);
  if((*sys).version<=3)
    fread(dummy,sizeof(dummy[0]),8,fp);
  fread(&(*obs_log).memo[0],sizeof(char),64,fp);
  (*obs_log).memo[64]=(char)NULL;
  fread(&(*obs_log).comment[0],sizeof(char),64,fp);
  (*obs_log).comment[64]=(char)NULL;

  (*obs_log).meteo.tmp=tmp;
  (*obs_log).meteo.press=press;
  (*obs_log).meteo.water=water;
#ifdef INTEL_CPU
} else {
  freadIntel(&(*obs_log).site_name[0],sizeof(char),8,fp);
  (*obs_log).site_name[8]=(char)NULL;
  freadIntel(&tmp,sizeof(tmp),1,fp);
  freadIntel(&press,sizeof(press),1,fp);
  freadIntel(&water,sizeof(water),1,fp);
  freadIntel(&(*obs_log).tau0,sizeof((*obs_log).tau0),1,fp);
  if((*sys).version<=3)
    freadIntel(dummy,sizeof(dummy[0]),8,fp);
  freadIntel(&(*obs_log).memo[0],sizeof(char),64,fp);
  (*obs_log).memo[64]=(char)NULL;
  freadIntel(&(*obs_log).comment[0],sizeof(char),64,fp);
  (*obs_log).comment[64]=(char)NULL;

  (*obs_log).meteo.tmp=tmp;
  (*obs_log).meteo.press=press;
  (*obs_log).meteo.water=water;
}
#endif

  return(0);
}


static int write_obs_log(fp,sys,obs_log)
/*
1992/2/22 Tomoharu Oka
1992/4/16 T.Handa UNIX MiniSTAR
1997/5/16 T.Handa 152byte for Cube-ver1
1999/4/26 T.Handa for ver.4
*/
FILE *fp;
SYS_HEAD *sys;
OBS_LOG *obs_log;
{
  static char dummy[8]={0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
  float tmp,press,water;

  tmp=(*obs_log).meteo.tmp;
  press=(*obs_log).meteo.press;
  water=(*obs_log).meteo.water;

#ifdef INTEL_CPU
if(fp==stdout) {
#endif
  fwrite(&(*obs_log).site_name[0],sizeof(char),8,fp);
  fwrite(&tmp,sizeof(tmp),1,fp);
  fwrite(&press,sizeof(press),1,fp);
  fwrite(&water,sizeof(water),1,fp);
  fwrite(&(*obs_log).tau0,sizeof((*obs_log).tau0),1,fp);
  if((*sys).version<=3)
    fwrite(dummy,sizeof(dummy[0]),8,fp);
  fwrite(&(*obs_log).memo[0],sizeof(char),64,fp);
  fwrite(&(*obs_log).comment[0],sizeof(char),64,fp);
#ifdef INTEL_CPU
} else {
  fwriteIntel(&(*obs_log).site_name[0],sizeof(char),8,fp);
  fwriteIntel(&tmp,sizeof(tmp),1,fp);
  fwriteIntel(&press,sizeof(press),1,fp);
  fwriteIntel(&water,sizeof(water),1,fp);
  fwriteIntel(&(*obs_log).tau0,sizeof((*obs_log).tau0),1,fp);
  if((*sys).version<=3)
    fwriteIntel(dummy,sizeof(dummy[0]),8,fp);
  fwriteIntel(&(*obs_log).memo[0],sizeof(char),64,fp);
  fwriteIntel(&(*obs_log).comment[0],sizeof(char),64,fp);
}
#endif

  return(0);
}


static int read_history(fp,sys,history)
/*
return
0 : normal
102 : too many lines in a history step

1992/2/22 Tomoharu Oka
1992/4/16 T.Handa UNIX MiniSTAR
1992/8/22 T.Handa check also by "END" keyword in the history
1992/9/19 T.Handa for UNIX STAR
1992/12/7 T.Handa add NULL for char[]
1997/5/16 T.Handa 16byte + 16 byte per "size" for Cube-ver1
         always 16byte for END step at least
1999/4/26 T.Handa for ver.4
*/
FILE		*fp;
SYS_HEAD *sys;
HISTORY		*history;
{
  int ii;
  char dummy[4];

#ifdef INTEL_CPU
if(fp==stdin) {
#endif
  fread(&(*history).size,sizeof((*history).size),1,fp);
  fread(&(*history).step[0],sizeof(char),7,fp);
  (*history).step[7]=(char)NULL;
  fread(&(*history).time,sizeof((*history).time),1,fp);
  fread(dummy,sizeof(dummy[0]),4,fp);

  if((*history).size > MAX_HISTORY_LINES_PER_STEP)
    return(102);

  for(ii=0;ii<(*history).size;ii++) {
    fread(&(*history).prm[ii][0],sizeof((*history).prm[ii][0]),16,fp);
    (*history).prm[ii][16]=(char)NULL;
  }
#ifdef INTEL_CPU
} else {
  freadIntel(&(*history).size,sizeof((*history).size),1,fp);
  freadIntel(&(*history).step[0],sizeof(char),7,fp);
  (*history).step[7]=(char)NULL;
  freadIntel(&(*history).time,sizeof((*history).time),1,fp);
  freadIntel(dummy,sizeof(dummy[0]),4,fp);

  if((*history).size > MAX_HISTORY_LINES_PER_STEP)
    return(102);

  for(ii=0;ii<(*history).size;ii++) {
    freadIntel(&(*history).prm[ii][0],sizeof((*history).prm[ii][0]),16,fp);
    (*history).prm[ii][16]=(char)NULL;
  }
}
#endif

  return(0);
}


static int write_history(fp,sys,history)
/*
1992/2/22 Tomoharu Oka
1992/4/16 T.Handa UNIX MiniSTAR
1992/8/22 T.Handa check also by "END" keyword in the history
1997/5/16 T.Handa 16byte + 16 byte per "size" for Cube-ver1
         always 16byte for END step at least
1999/4/26 T.Handa for ver.4
*/
FILE *fp;
SYS_HEAD *sys;
HISTORY *history;
{
  int ii;
  static char dummy[4]={0x00,0x00,0x00,0x00};

#ifdef INTEL_CPU
if(fp==stdout) {
#endif
  fwrite(&(*history).size,sizeof((*history).size),1,fp);
  fwrite(&(*history).step[0],sizeof(char),7,fp);
  fwrite(&(*history).time,sizeof((*history).time),1,fp);
  fwrite(dummy,sizeof(dummy[0]),4,fp);

  for(ii=0;ii<(*history).size;ii++)
    fwrite(&(*history).prm[ii][0],sizeof((*history).prm[ii][0]),16,fp);
#ifdef INTEL_CPU
} else {
  fwriteIntel(&(*history).size,sizeof((*history).size),1,fp);
  fwriteIntel(&(*history).step[0],sizeof(char),7,fp);
  fwriteIntel(&(*history).time,sizeof((*history).time),1,fp);
  fwriteIntel(dummy,sizeof(dummy[0]),4,fp);

  for(ii=0;ii<(*history).size;ii++)
    fwriteIntel(&(*history).prm[ii][0],sizeof((*history).prm[ii][0]),16,fp);
}
#endif

  return(0);
}


/*------ for PROFILE ------*/

static int read_spurious(fp,sys,data_ch,spurious)
/*
1992/2/22 Tomoharu Oka
1992/4/16 T.Handa UNIX MiniSTAR
*/
FILE *fp;
SYS_HEAD *sys;
int data_ch;
unsigned char *spurious;
{
  int sp_ch;
  int err_code;

  sp_ch=(data_ch-1)/(sizeof(unsigned char)*8)+1;
#ifdef INTEL_CPU
if(fp==stdin) {
#endif
  err_code=fread(spurious,sizeof(unsigned char),sp_ch,fp);
#ifdef INTEL_CPU
} else {
  err_code=freadIntel(spurious,sizeof(unsigned char),sp_ch,fp);
}
#endif

 if(err_code!=sp_ch) return(-err_code);
 return(0);
}


static int write_spurious(fp,sys,data_ch,spurious)
/*
1992/2/22 Tohomaru Oka
1992/4/16 T.Handa UNIX MiniSTAR
*/
FILE *fp;
SYS_HEAD *sys;
int data_ch;
unsigned char *spurious;
{
  int sp_ch;
  int err_code;

  sp_ch=(data_ch-1)/(sizeof(unsigned char)*8)+1;
#ifdef INTEL_CPU
if(fp==stdout) {
#endif
  err_code=fwrite(spurious,sizeof(unsigned char),sp_ch,fp);
#ifdef INTEL_CPU
} else {
  err_code=fwriteIntel(spurious,sizeof(unsigned char),sp_ch,fp);
}
#endif

  if(err_code!=sp_ch) return(-err_code);
  return(0);
}


static int read_baseline(fp,sys,data_ch,baseline)
/*
1992/2/22 Tomoharu Oka
1992/4/16 T.Handa UNIX MiniSTAR
*/
FILE *fp;
SYS_HEAD *sys;
int data_ch;
unsigned char *baseline;
{
  int bs_ch;
  int err_code;

  bs_ch=(data_ch-1)/(sizeof(unsigned char)*8)+1;
#ifdef INTEL_CPU
if(fp==stdin) {
#endif
  err_code=fread(baseline,sizeof(unsigned char),bs_ch,fp);
#ifdef INTEL_CPU
} else {
  err_code=freadIntel(baseline,sizeof(unsigned char),bs_ch,fp);
}
#endif

  if(err_code!=bs_ch) return(-err_code);
  return(0);
}


static int write_baseline(fp,sys,data_ch,baseline)
/*
1992/2/22 Tomoharu Oka
1992/4/16 T.Handa UNIX MiniSTAR
*/
FILE *fp;
SYS_HEAD *sys;
int data_ch;
unsigned char *baseline;
{
  int bs_ch;
  int err_code;

  bs_ch=(data_ch-1)/(sizeof(unsigned char)*8)+1;
#ifdef INTEL_CPU
if(fp==stdout) {
#endif
  err_code=fwrite(baseline,sizeof(unsigned char),bs_ch,fp);
#ifdef INTEL_CPU
} else {
  err_code=fwriteIntel(baseline,sizeof(unsigned char),bs_ch,fp);
}
#endif

  if(err_code!=bs_ch) return(-err_code);
  return(0);
}


static int read_prof(fp,sys,data_ch,profile)
/*
1992/2/22 Tomoharu Oka
1992/4/16 T.Handa UNIX MiniSTAR
*/
FILE *fp;
SYS_HEAD *sys;
int data_ch;
float profile[];
{
  int err_code;

#ifdef INTEL_CPU
if(fp==stdin) {
#endif
  err_code=fread(profile,sizeof(profile[0]),data_ch,fp);
#ifdef INTEL_CPU
} else {
  err_code=freadIntel(profile,sizeof(profile[0]),data_ch,fp);
}
#endif
  if(err_code!=data_ch) return(-err_code);
  return(0);
}


static int write_prof(fp,sys,data_ch,profile)
/*
1992/2/22 Tomoharu Oka
1992/4/16 T.Handa UNIX MiniSTAR
*/
FILE *fp;
SYS_HEAD *sys;
int data_ch;
float profile[];
{
  int err_code;

#ifdef INTEL_CPU
if(fp==stdout) {
#endif
  err_code=fwrite(profile,sizeof(profile[0]),data_ch,fp);
#ifdef INTEL_CPU
} else {
  err_code=fwriteIntel(profile,sizeof(profile[0]),data_ch,fp);
}
#endif
  if(err_code!=data_ch) return(-err_code);
  return(0);
}


/*------ for CUBE_HEAD ------*/

static int read_HeadAxis(fp,sys,axis)
FILE *fp;
SYS_HEAD *sys;
AXIS *axis;
/*
Read Datacube Axis in Datacube Axes in Header
1998/10/18 T.Handa 60byte for Cube-ver1
1998/12/21 T.Handa for Cube-ver3
1999/4/26 T.Handa for ver4
*/
{
  int err_code;

  err_code=0;

#ifdef INTEL_CPU
if(fp==stdin) {
#endif
  fread(&((*axis).name[0]),sizeof((*axis).name[0]),16,fp);
  (*axis).name[16]=(char)NULL;
  fread(&((*axis).size),sizeof((*axis).size),1,fp);
  fread(&((*axis).type[0]),sizeof((*axis).type[0]),8,fp);
  (*axis).type[8]=(char)NULL;
  fread(&((*axis).unit[0]),sizeof((*axis).unit[0]),8,fp);
  (*axis).unit[8]=(char)NULL;
  fread(&((*axis).project[0]),sizeof((*axis).project[0]),8,fp);
  (*axis).project[8]=(char)NULL;
  fread(&((*axis).refer[0]),sizeof((*axis).refer[0]),8,fp);
  (*axis).refer[8]=(char)NULL;
  fread(&((*axis).delta),sizeof((*axis).delta),1,fp);
  fread(&((*axis).origin),sizeof((*axis).origin),1,fp);
  fread(&((*axis).pixel_origin),sizeof((*axis).pixel_origin),1,fp);
  fread(&((*axis).rotation),sizeof((*axis).rotation),1,fp);
  fread(&((*axis).resolution),sizeof((*axis).resolution),1,fp);
  fread(&((*axis).psf[0]),sizeof((*axis).psf[0]),8,fp);
  fread(&((*axis).res_orig),sizeof((*axis).res_orig),1,fp);
#ifdef INTEL_CPU
} else {
  freadIntel(&((*axis).name[0]),sizeof((*axis).name[0]),16,fp);
  (*axis).name[16]=(char)NULL;
  freadIntel(&((*axis).size),sizeof((*axis).size),1,fp);
  freadIntel(&((*axis).type[0]),sizeof((*axis).type[0]),8,fp);
  (*axis).type[8]=(char)NULL;
  freadIntel(&((*axis).unit[0]),sizeof((*axis).unit[0]),8,fp);
  (*axis).unit[8]=(char)NULL;
  freadIntel(&((*axis).project[0]),sizeof((*axis).project[0]),8,fp);
  (*axis).project[8]=(char)NULL;
  freadIntel(&((*axis).refer[0]),sizeof((*axis).refer[0]),8,fp);
  (*axis).refer[8]=(char)NULL;
  freadIntel(&((*axis).delta),sizeof((*axis).delta),1,fp);
  freadIntel(&((*axis).origin),sizeof((*axis).origin),1,fp);
  freadIntel(&((*axis).pixel_origin),sizeof((*axis).pixel_origin),1,fp);
  freadIntel(&((*axis).rotation),sizeof((*axis).rotation),1,fp);
  freadIntel(&((*axis).resolution),sizeof((*axis).resolution),1,fp);
  freadIntel(&((*axis).psf[0]),sizeof((*axis).psf[0]),8,fp);
  freadIntel(&((*axis).res_orig),sizeof((*axis).res_orig),1,fp);
}
#endif

  return(err_code);
}

static int write_HeadAxis(fp,sys,axis)
FILE *fp;
SYS_HEAD *sys;
AXIS *axis;
/*
Write Datacube Axis in Datacube Axes in Header
1998/10/18 T.Handa 60byte for Cube-ver1
*/
{
  int	err_code;

  err_code=0;

#ifdef INTEL_CPU
if(fp==stdout) {
#endif
  fwrite(&((*axis).name[0]),sizeof((*axis).name[0]),16,fp);
  fwrite(&((*axis).size),sizeof((*axis).size),1,fp);
  fwrite(&((*axis).type[0]),sizeof((*axis).type[0]),8,fp);
  fwrite(&((*axis).unit[0]),sizeof((*axis).unit[0]),8,fp);
  fwrite(&((*axis).project[0]),sizeof((*axis).project[0]),8,fp);
  fwrite(&((*axis).refer[0]),sizeof((*axis).refer[0]),8,fp);
  fwrite(&((*axis).delta),sizeof((*axis).delta),1,fp);
  fwrite(&((*axis).origin),sizeof((*axis).origin),1,fp);
  fwrite(&((*axis).pixel_origin),sizeof((*axis).pixel_origin),1,fp);
  fwrite(&((*axis).rotation),sizeof((*axis).rotation),1,fp);
  fwrite(&((*axis).resolution),sizeof((*axis).resolution),1,fp);
  fwrite(&((*axis).psf[0]),sizeof((*axis).psf[0]),8,fp);
  fwrite(&((*axis).res_orig),sizeof((*axis).res_orig),1,fp);
#ifdef INTEL_CPU
} else {
  fwriteIntel(&((*axis).name[0]),sizeof((*axis).name[0]),16,fp);
  fwriteIntel(&((*axis).size),sizeof((*axis).size),1,fp);
  fwriteIntel(&((*axis).type[0]),sizeof((*axis).type[0]),8,fp);
  fwriteIntel(&((*axis).unit[0]),sizeof((*axis).unit[0]),8,fp);
  fwriteIntel(&((*axis).project[0]),sizeof((*axis).project[0]),8,fp);
  fwriteIntel(&((*axis).refer[0]),sizeof((*axis).refer[0]),8,fp);
  fwriteIntel(&((*axis).delta),sizeof((*axis).delta),1,fp);
  fwriteIntel(&((*axis).origin),sizeof((*axis).origin),1,fp);
  fwriteIntel(&((*axis).pixel_origin),sizeof((*axis).pixel_origin),1,fp);
  fwriteIntel(&((*axis).rotation),sizeof((*axis).rotation),1,fp);
  fwriteIntel(&((*axis).resolution),sizeof((*axis).resolution),1,fp);
  fwriteIntel(&((*axis).psf[0]),sizeof((*axis).psf[0]),8,fp);
  fwriteIntel(&((*axis).res_orig),sizeof((*axis).res_orig),1,fp);
}
#endif

  return(err_code);
}

static int read_HeadAxes(fp,sys,axis)
FILE *fp;
SYS_HEAD *sys;
AXES *axis;
/*
Read Datacube Axes in Header
1998/10/18 T.Handa 186byte for Cube-ver1
*/
{
  int err_code;

#ifdef INTEL_CPU
if(fp==stdin) {
#endif
  fread(&((*axis).dimension),sizeof((*axis).dimension),1,fp);
  fread(&((*axis).order[0]),sizeof((*axis).order[0]),4,fp);
  (*axis).order[4]=(char)NULL;
#ifdef INTEL_CPU
} else {
  freadIntel(&((*axis).dimension),sizeof((*axis).dimension),1,fp);
  freadIntel(&((*axis).order[0]),sizeof((*axis).order[0]),4,fp);
  (*axis).order[4]=(char)NULL;
}
#endif

  err_code=read_HeadAxis(fp,sys,&((*axis).x));
  if(err_code!=0) return(err_code);
  err_code=read_HeadAxis(fp,sys,&((*axis).y));
  if(err_code!=0) return(err_code);
  err_code=read_HeadAxis(fp,sys,&((*axis).z));
  if(err_code!=0) return(err_code);

  return(0);
}

static int write_HeadAxes(fp,sys,axis)
FILE *fp;
SYS_HEAD *sys;
AXES *axis;
/*
Write Datacube Axes in Header
1998/10/18 T.Handa 186byte for Cube-ver1
*/
{
  int err_code;

#ifdef INTEL_CPU
if(fp==stdout) {
#endif
  fwrite(&((*axis).dimension),sizeof((*axis).dimension),1,fp);
  fwrite(&((*axis).order[0]),sizeof((*axis).order[0]),4,fp);
#ifdef INTEL_CPU
} else {
  fwriteIntel(&((*axis).dimension),sizeof((*axis).dimension),1,fp);
  fwriteIntel(&((*axis).order[0]),sizeof((*axis).order[0]),4,fp);
}
#endif

  err_code=write_HeadAxis(fp,sys,&((*axis).x));
  if(err_code!=0) return(err_code);
  err_code=write_HeadAxis(fp,sys,&((*axis).y));
  if(err_code!=0) return(err_code);
  err_code=write_HeadAxis(fp,sys,&((*axis).z));
  if(err_code!=0) return(err_code);

  return(0);
}

static int read_HeadDValue(fp,sys,data_value)
FILE *fp;
SYS_HEAD *sys;
DATA_VALUE *data_value;
/*
Read Pixel Data Value in Header
1998/10/18 T.Handa 52byte for Cube-ver1
*/
{
  int err_code;

  err_code=0;

#ifdef INTEL_CPU
if(fp==stdin) {
#endif
  fread(&((*data_value).type[0]),sizeof((*data_value).type[0]),8,fp);
  (*data_value).type[8]=(char)NULL;
  fread(&((*data_value).unit[0]),sizeof((*data_value).unit[0]),8,fp);
  (*data_value).unit[8]=(char)NULL;
  fread(&((*data_value).scale),sizeof((*data_value).scale),1,fp);
  fread(&((*data_value).offset),sizeof((*data_value).offset),1,fp);
  fread(&((*data_value).dummy),sizeof((*data_value).dummy),1,fp);
  fread(&((*data_value).max),sizeof((*data_value).max),1,fp);
  fread(&((*data_value).min),sizeof((*data_value).min),1,fp);
  fread(&((*data_value).mean),sizeof((*data_value).mean),1,fp);
  fread(&((*data_value).rms),sizeof((*data_value).rms),1,fp);
  fread(&((*data_value).noise),sizeof((*data_value).noise),1,fp);
#ifdef INTEL_CPU
} else {
  freadIntel(&((*data_value).type[0]),sizeof((*data_value).type[0]),8,fp);
  (*data_value).type[8]=(char)NULL;
  freadIntel(&((*data_value).unit[0]),sizeof((*data_value).unit[0]),8,fp);
  (*data_value).unit[8]=(char)NULL;
  freadIntel(&((*data_value).scale),sizeof((*data_value).scale),1,fp);
  freadIntel(&((*data_value).offset),sizeof((*data_value).offset),1,fp);
  freadIntel(&((*data_value).dummy),sizeof((*data_value).dummy),1,fp);
  freadIntel(&((*data_value).max),sizeof((*data_value).max),1,fp);
  freadIntel(&((*data_value).min),sizeof((*data_value).min),1,fp);
  freadIntel(&((*data_value).mean),sizeof((*data_value).mean),1,fp);
  freadIntel(&((*data_value).rms),sizeof((*data_value).rms),1,fp);
  freadIntel(&((*data_value).noise),sizeof((*data_value).noise),1,fp);
}
#endif

  return(err_code);
}

static int write_HeadDValue(fp,sys,data_value)
FILE *fp;
SYS_HEAD *sys;
DATA_VALUE *data_value;
/*
Write Pixel Data Value in Header
1998/10/18 T.Handa 52byte for Cube-ver1
*/
{
  int err_code;

  err_code=0;

#ifdef INTEL_CPU
if(fp==stdout) {
#endif
  fwrite(&((*data_value).type[0]),sizeof((*data_value).type[0]),8,fp);
  fwrite(&((*data_value).unit[0]),sizeof((*data_value).unit[0]),8,fp);
  fwrite(&((*data_value).scale),sizeof((*data_value).scale),1,fp);
  fwrite(&((*data_value).offset),sizeof((*data_value).offset),1,fp);
  fwrite(&((*data_value).dummy),sizeof((*data_value).dummy),1,fp);
  fwrite(&((*data_value).max),sizeof((*data_value).max),1,fp);
  fwrite(&((*data_value).min),sizeof((*data_value).min),1,fp);
  fwrite(&((*data_value).mean),sizeof((*data_value).mean),1,fp);
  fwrite(&((*data_value).rms),sizeof((*data_value).rms),1,fp);
  fwrite(&((*data_value).noise),sizeof((*data_value).noise),1,fp);
#ifdef INTEL_CPU
} else {
  fwriteIntel(&((*data_value).type[0]),sizeof((*data_value).type[0]),8,fp);
  fwriteIntel(&((*data_value).unit[0]),sizeof((*data_value).unit[0]),8,fp);
  fwriteIntel(&((*data_value).scale),sizeof((*data_value).scale),1,fp);
  fwriteIntel(&((*data_value).offset),sizeof((*data_value).offset),1,fp);
  fwriteIntel(&((*data_value).dummy),sizeof((*data_value).dummy),1,fp);
  fwriteIntel(&((*data_value).max),sizeof((*data_value).max),1,fp);
  fwriteIntel(&((*data_value).min),sizeof((*data_value).min),1,fp);
  fwriteIntel(&((*data_value).mean),sizeof((*data_value).mean),1,fp);
  fwriteIntel(&((*data_value).rms),sizeof((*data_value).rms),1,fp);
  fwriteIntel(&((*data_value).noise),sizeof((*data_value).noise),1,fp);
}
#endif

  return(err_code);
}
