/*
STAR file I/O functions
for file name processing

1997/11/13 T.Handa
2002/ 6/10 T.Handa (revise) StarDoesFileExist()
2003/ 1/ 2 T.Handa (add) StarGetBackupname(), StarRename4backup(); (revise) StarStdFileExtName()
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/stat.h>
#include <unistd.h>

#include <StData.h>
#include <StStd.h>

int StarStdFileExtName(env_name,file_name,std_file_name,extension)
/*
add extension file

5001:too long file name

1992/4/29 T.Handa
1992/11/26 T.Handa (revise) add ENV paramter
2003/ 1/ 2 T.Handa (revise) omit current dir
*/
char *env_name;	/* <in> Environmant for default directory */
char *file_name;	/* <in> short filename */
char *std_file_name;	/* <out> full filename */
char *extension;	/* <in> standard file extension name */
{
  int	l_ext, l_file_name,l_dir;
  char 	*env_p,dir_name[MAX_FILE_NAME_LENGTH];

  strcpy(std_file_name,file_name);
  l_file_name=strlen(std_file_name);
  l_ext=strlen(extension);

/* length check */
  if(l_file_name+l_ext>=MAX_FILE_NAME_LENGTH)
    return(5001);

  if(env_name[0]!=(char)NULL) {
/* add ENV parameter */
    env_p=getenv(env_name);
    if(env_p!=NULL && file_name[0]!='/') {
      l_dir=strlen(env_p);
      if(l_dir+l_file_name+1>=MAX_FILE_NAME_LENGTH)
	return(5001);
      if(strcmp(env_p,".")==0) {
	strcpy(dir_name,"");
      } else if(strncmp(env_p,"./",2)==0) {
	strcpy(dir_name,&(env_p[2]));
      } else {
	strcpy(dir_name,env_p);
      }
      l_dir=strlen(dir_name);
      if(l_dir>0 && dir_name[l_dir-1]!='/')
	strcat(dir_name,"/");
      strcpy(std_file_name,dir_name);
      strcat(std_file_name,file_name);

      /* recheck file name length */
      l_file_name=strlen(std_file_name);
      if(l_file_name+l_ext>=MAX_FILE_NAME_LENGTH)
	return(5001);
    }
  }

/* check extension part */
  if(l_file_name>l_ext) {
    if(strcmp(&std_file_name[l_file_name-l_ext],extension)!=0)
      strcat(std_file_name,extension);
  } else {
    strcat(std_file_name,extension);
  }
  return(0);	
}

int StarDoesFileExist(filename)
char *filename;	/* <in> */
/*
check the file existance
1997/12/15 T.Handa
2002/6/11 T.Handa (revise) use stat()
*/
{
  struct stat f_stat;
  int ret_code,err_code;

  int l_str,ii;
  char *buffer;

  ret_code=stat(filename,&f_stat);
  if(ret_code==0) { /*  accessible filename */
    if(f_stat.st_mode & S_IFDIR) return FileDirectory;
    else return FileExist;
  }

  /* check exist directory */
  l_str=strlen(filename);
  buffer=(char *)malloc(l_str);
  if(buffer==NULL) return FileUnknown; /* failed to process */
  strcpy(buffer,filename);

  for(ii=l_str-1;ii>=0;ii--) {
    if(buffer[ii]=='/') {
      buffer[ii]=(char)NULL;
      l_str=ii;
      goto directory_check;
    }
  }
  free(buffer);
  return FileAbsent;

directory_check:


  err_code=stat(buffer,&f_stat);
  if(err_code==0) { /*  accessible filename */
    if(f_stat.st_mode & S_IFDIR) 
      ret_code=FileAbsent;
    else 
      ret_code=FileInvalid;
  } else {
    ret_code=FileInvalid;
  }

  free(buffer);
  return ret_code;

/*
  FILE *fp;

  fp=fopen(filename,"r");
  if(fp!=NULL) {
    fclose(fp);
    return FLAG_YES;
  }
  return FLAG_NO;
*/
}

void StarGetBackupname(backup_fname,filename,bkver,withbkver,extension)
char *backup_fname; /* <out> filename to backup (with full path) */
char *filename; /* <in> filename to save */
int bkver; /* <in> backup version number */
int withbkver; /* <in> with bkver if STAR_BACKUP is assigned */
char *extension; /* <in> extension with '.' */
/*
2003/1/2 T.Handa
 */
{
  char fname4work[MAX_FILE_NAME_LENGTH];
  char back_dir[MAX_FILE_NAME_LENGTH];
  char *env_p;
  int l_str,l_ext;

  l_str=strlen(filename);
  l_ext=strlen(extension);
  if(l_str>=l_ext && strcmp(&(filename[l_str-l_ext]),extension)==0) {
    filename[l_str-l_ext]=(char)NULL;
  }

  /* check ENVIRONMENT valiable for backup */
  env_p=getenv("STAR_BACKUP");
  if(env_p==NULL) {
    back_dir[0]=(char)NULL;
  } else {
    if(strncmp(env_p,"./",2)==0) {
      strcpy(back_dir,&(env_p[2]));
    } else if(strcmp(env_p,".")==0) {
      strcpy(back_dir,&(env_p[1]));
    } else {
      strcpy(back_dir,env_p);
    }
  }
  if(back_dir[0]==(char)NULL || strcmp(back_dir," ")==0) {
    sprintf(backup_fname,"%s.%d+%s",filename,bkver,extension);
  } else {
    if(withbkver==FLAG_YES) {
      sprintf(backup_fname,"%s.%d+",filename,bkver);
      strcpy(fname4work,backup_fname);
    } else {
      strcpy(fname4work,filename);
    }
    StarStdFileExtName("STAR_BACKUP",fname4work,backup_fname,extension);
  }
  if(back_dir[0]!='/') {
    strcpy(fname4work,backup_fname);
    StarStdFileExtName("STAR_DATA",fname4work,backup_fname,extension);
  }

  return;
}

int StarRename4backup(org_fname,new_fname)
char *org_fname; /* <in> */
char *new_fname; /* <in> */
/*
2003/ 1/ 2 T.Handa
 */
{
  int err_code;
  int exist;

  if(StarDoesFileExist(org_fname)==FileAbsent) return(0);

  exist=StarDoesFileExist(new_fname);
  if(exist==FileExist) {
    fprintf(stderr,"Previous backup file %s was overwritten.\n",new_fname);
  }

  err_code=rename(org_fname,new_fname);
  if(err_code!=0) {
    fprintf(stderr,"Failed to rename backup file %s to %s\n",org_fname,new_fname);
    return(92);
  }
  return(exist);
}

int StarGetSavename(argc,argv, default_name,filename)
int argc; /* <in> the number of option arguments */
char **argv; /* <in> option arguments */
char *default_name; /* <in> */
char *filename; /* <out> */
/*
set a proper filename

2003/1/4 T.Handa
 */
{
  static int argi=0;

/* next argument */
  argi++;

/* all arguments have been used */
  if(argi>=argc) {
    strcpy(filename,default_name);
    return(1);
  }

/* skip for keywords */
  while(argv[argi][0]=='-') {
    if(strcmp(argv[argi],"-prm")==0) argi++;
    argi++;

    /* all arguments have been used */
    if(argi>=argc) {
      strcpy(filename,default_name);
      return(1);
    }
  }

/* from one of the arguments */
  strcpy(filename,argv[argi]);

  return(0);
}

