/*
Ultra/STAR FITS I/O functions
convert between PROFILE,CUBE, and FITS

1999/ 8/12 T.Handa : ver.1.00
1999/ 9/23 T.Handa : ver.1.01 (revise) for CfA South FITS
1999/ 9/25 T.Handa : ver.1.02 (debug) rx.scale for long/short/char pixel data
1999/12/27 T.Handa : ver.1.03 (modefy) standard keyword list
2001/ 9/28 T.Handa : ver.1.04 (debug) NAXISn are in correct position for the SIMPLE format
2003/ 8/28 T.Handa : ver.1.05 (debug) for NaN value in bitpix<0 data in StarFitsPixel2CubePixel() using finite()
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>
#include <StFitsIO.h>

#ifdef INTEL_CPU
/*-------Top of StDataIO.c import---------*/
/*------ for INTEL CPU ------*/

static size_t freadIntel(pt,size,nn,fp)
char *pt;
size_t size;
int nn;
FILE *fp;
/*
Read Binary data written by Motrollor/MIPS CPU with Intel
1998/4/21 T.Handa
1998/7/14 T.Handa (return value revised)
*/
{
  char *buffer;
  size_t ret_code;
  int total_byte,ii,jj;

  if(size==1) {
    ret_code=fread(pt,1,nn,fp);
    return(ret_code);
  }

  total_byte=size*nn;
  buffer=malloc(total_byte);

  ret_code=fread(buffer,1,total_byte,fp);
  if(ret_code==total_byte) ret_code=nn;
  else ret_code=0;

  for(ii=0;ii<nn;ii++) {
    for(jj=0;jj<size;jj++) 
      pt[ii*size+jj]=buffer[(ii+1)*size-jj-1];
  }

  free(buffer);

  return(ret_code);
}

static size_t fwriteIntel(pt,size,nn,fp)
char *pt;
size_t size;
int nn;
FILE *fp;
/*
Write Binary data readable by Motrollor/MIPS CPU with Intel
1998/4/21 T.Handa
1998/7/14 T.Handa (return value revised)
*/
{
  char *buffer;
  size_t ret_code;
  int total_byte,ii,jj;

  if(size==1) {
    ret_code=fwrite(pt,1,nn,fp);
    return(ret_code);
  }

  total_byte=size*nn;
  buffer=malloc(total_byte);

  for(ii=0;ii<nn;ii++) {
    for(jj=0;jj<size;jj++)
      buffer[(ii+1)*size-jj-1]=pt[ii*size+jj];
  }

  ret_code=fwrite(buffer,1,total_byte,fp);
  if(ret_code==total_byte) ret_code=nn;
  else ret_code=0;

  free(buffer);
  return(ret_code);
}
/*-------End of StDataIO.c import---------*/
#else
#define freadIntel(x1,x2,x3,x4) fread(x1,x2,x3,x4)
#define fwriteIntel(x1,x2,x3,x4) fwrite(x1,x2,x3,x4)
#endif

int StarReadFitsBlock(fp,block)
/*
Read a FITS block from a FITS file.

1999/8/13 T.Handa
*/
FILE *fp; /* <in> */
char block[36][80]; /* <out> */
{
  int ii;
  size_t read_size;

  for(ii=0;ii<36;ii++) {
    read_size=freadIntel(block[ii],sizeof(block[ii][0]),80,fp);
    if(read_size!=80) return(999);
  }
  return(0);
}

void StarWriteFitsRecord(keyword,value,v_type,comment,fp)
/*
Write a FITS header record into a file.

1999/7/31 T.Handa
*/
char *keyword; /* <in> keyword for FITS header record */
char *value;   /* <in> value word for FITS header record */
int v_type;    /* <in> value type for *value */
  /* FITS_VALUE_BOOLIAN for boolian, FITS_VALUE_STRING for char, 
     FITS_VALUE_SHORT_STRING for char followed by blanks,
     FITS_VALUE numerical value but in ASCII char */
char *comment; /* <in> comment in FITS header record */
FILE *fp; /* <in> for FITS file */
{
  int l_rec,l_str,l_key;
  char r_key[9],r_value[70];
  char buffer[81];
  char record[81];

/* keyword */
  l_key=strlen(keyword);
  if(l_key>8) keyword[8]=(char)NULL;
  strcpy(r_key,keyword);
  StarAdjustStrlen(r_key,8,' ');

/* value */
  l_str=strlen(value);
  if(l_str>69) value[69]=(char)NULL;
  if(l_str>0) {
    if(v_type==FITS_VALUE_BOOLIAN) {
      switch(value[0]) {
      case 't': case 'T': case 'y': case 'Y':
	strcpy(r_value,"                   T");
	break;
      case 'f': case 'F': case 'n': case 'N':
	strcpy(r_value,"                   F");
	break;
      }
    }else if(v_type==FITS_VALUE_STRING || v_type==FITS_VALUE_SHORT_STRING) {
      if(l_str<=8) {
	strcpy(buffer,value);
	StarAdjustStrlen(buffer,8,' ');
	sprintf(r_value,"\'%s\'",buffer);
      }else{
	sprintf(r_value,"\'%s\'",value);
      }
      if(v_type==FITS_VALUE_SHORT_STRING) {
	StarAdjustStrlen(r_value,20,' ');
      }
    }else{
      strcpy(r_value,value);
    }

    if(strcmp(keyword,"HISTORY")==0) {
      sprintf(record,"%s %s",r_key,r_value);
    } else if(l_key>0) {
      sprintf(record,"%s= %s ",r_key,r_value);
    } else {
      sprintf(record,"          %s",r_value);
    }
  }else{
    strcpy(record,r_key);
  }

/* comment */
  l_rec=strlen(record);
  l_str=strlen(comment);
  if(l_str>69) value[69]=(char)NULL;
  if(l_str>0 && l_rec+l_str>78) {
    value[l_str+l_rec-79]=(char)NULL;
    l_str=strlen(value);
  }
  if(l_str>0) {
    strcat(record,"/ ");
    strcat(record,comment);
  }

  StarAdjustStrlen(record,80,' ');

  fprintf(fp,"%s",record);

  return;
}

/* keyword conversion between CUBE_HEAD.AXIS and FITS.CTYPE */
#define N_LongLat 6
#define N_Projection 9

static char FitsCtypeKey4LongLat1[N_LongLat][10]=
  {"GLON","GLAT","RA","DEC","AZ","EL"};
static char StAxisNameKey4LongLat[N_LongLat][9] =
  {"l"   ,"b"   ,"RA","Dec","az","el"};
static char StAxisRefer4LongLat[N_LongLat/2][9]=
  {"lb2","","azel"};
static char FitsCtypeKey4LongLat2[N_Projection][4]=
  {"TAN","SIN","STG"   ,"ARC",
   "MER"     ,"GLS"     ,"PAR"    ,"AIT"   ,"MOL"};
static char StAxisProjKey[N_Projection][9]=
  {"tan","sin","stereo","linear",
   "Mercator","Sinusoid","Parabol","Aitoff","Molweide"};

#define N_AxisType 3
static char FitsCtypeKey[N_AxisType][10]=
  {"VELOCITY","VELO-LSR","FREQUENCY"};
static char StAxisType[N_AxisType][9] =
  {"velocity","velocity","freq"};

/* CUBE to FITS */

static void StarLongLatAxis2FitsCtype(fits,x)
/*
Make FITS-CTYPE keyword from cube.axis (long/lat) information

1999/7/31 T.Handa
*/
char *fits; /* <out> */
AXIS *x; /* <in> */
{
  int ii,iflag;

  iflag=0;
  for(ii=0;ii<N_LongLat;ii++) {
    if(strcmp((*x).name,StAxisNameKey4LongLat[ii])==0) {
      strcpy(fits,FitsCtypeKey4LongLat1[ii]);
      iflag=1;
      break;
    }
  }

  if(iflag==0) {
    strcpy(fits,(*x).name);
  }

  iflag=0;
  for(ii=0;ii<N_Projection;ii++) {
    if(strcmp((*x).project,StAxisProjKey[ii])==0) {
      fits[5]=(char)NULL;
      StarAdjustStrlen(fits,5,'-');
      strcat(fits,FitsCtypeKey4LongLat2[ii]);
      iflag=1;
      break;
    }
  }

  return;
}

void StarAxis2FitsCtype(fits,x)
/*
Make FITS-CTYPE keyword from cube.axis information

1999/7/31 T.Handa
*/
char *fits; /* <out> FITS CTYPE keyword */
AXIS *x;    /* <in> */
{
  int ii,iflag;

/* axis with spheric-plane projection */
  if(strcmp((*x).type,"long")==0 || strcmp((*x).type,"lat")==0) {
    StarLongLatAxis2FitsCtype(fits,x);
    return;
  }

  iflag=0;
  for(ii=0;ii<N_AxisType;ii++) {
    if(strcmp((*x).type,StAxisType[ii])==0) {
      strcpy(fits,FitsCtypeKey[ii]);
      iflag=1;
      return;
    }
  }

  if(iflag==0) {
    strcpy(fits,(*x).type);
  }
  return;
}

/* Make FITS header records */

int StarWriteFitsHead4CubeHeadAxes(fp,axis)
/*
Write Fits header corresponding cube.axis

1999/7/31 T.Handa
2001/9/28 T.Handa (for correct order to SIMPLE)
*/
FILE *fp; /* <in> for FITS */
AXES *axis; /* <in> cube.axis */
{
  int n_line;
  char buffer[80],tag_key[9];

  n_line=0;
  StarWriteFitsRecord("NAXIS","                   3",FITS_VALUE,"NUMBER OF AXES",fp);
  n_line++;

  sprintf(buffer,"%20ld",(*axis).x.size);
  sprintf(tag_key,"NAXIS1");
  StarWriteFitsRecord(tag_key,buffer,FITS_VALUE," ",fp);
  n_line++;
  sprintf(buffer,"%20ld",(*axis).y.size);
  sprintf(tag_key,"NAXIS2");
  StarWriteFitsRecord(tag_key,buffer,FITS_VALUE," ",fp);
  n_line++;
  sprintf(buffer,"%20ld",(*axis).z.size);
  sprintf(tag_key,"NAXIS3");
  StarWriteFitsRecord(tag_key,buffer,FITS_VALUE," ",fp);
  n_line++;

  n_line=n_line+StarWriteFitsHead4CubeHeadAxis(fp,&((*axis).x),1);
  n_line=n_line+StarWriteFitsHead4CubeHeadAxis(fp,&((*axis).y),2);
  n_line=n_line+StarWriteFitsHead4CubeHeadAxis(fp,&((*axis).z),3);

  return(n_line);
}

int StarWriteFitsHead4CubeHeadAxis(fp,x,xyz)
/*
Write Fits header corresponding cube.axis.x,y,z

1999/7/31 T.Handa
2001/9/28 T.Handa (for correct order to SIMPLE)
*/
FILE *fp; /* <in> for FITS */
AXIS *x; /* <in> cube.axis */
int xyz; /* <in> 1=x, 2=y, 3=z */
{
  int n_line;
  char buffer[80],tag_key[9];

  n_line=0;
  StarAxis2FitsCtype(buffer,x);
  sprintf(tag_key,"CTYPE%d",xyz);
  StarWriteFitsRecord(tag_key,buffer,FITS_VALUE_SHORT_STRING,"Name of axis",fp);
  n_line++;
  sprintf(tag_key,"CUNIT%d",xyz);
  StarWriteFitsRecord(tag_key,(*x).unit,FITS_VALUE_SHORT_STRING,"Unit of axis",fp);
  n_line++;
  sprintf(buffer,"%20.15f",(*x).pixel_origin+1.0);
  sprintf(tag_key,"CRPIX%d",xyz);
  StarWriteFitsRecord(tag_key,buffer,FITS_VALUE,"Reference in Index",fp);
  n_line++;
  sprintf(buffer,"%20.15f",(*x).origin);
  sprintf(tag_key,"CRVAL%d",xyz);
  StarWriteFitsRecord(tag_key,buffer,FITS_VALUE,"at reference pixel",fp);
  n_line++;
  sprintf(buffer,"%20.15f",(*x).delta);
  sprintf(tag_key,"CDELT%d",xyz);
  StarWriteFitsRecord(tag_key,buffer,FITS_VALUE,"",fp);
  n_line++;
  sprintf(buffer,"%20.15f",(*x).rotation);
  sprintf(tag_key,"CROTA%d",xyz);
  StarWriteFitsRecord(tag_key,buffer,FITS_VALUE,"",fp);
  n_line++;

  return n_line;
}

/* FITS to CUBE */

static void StarFitsCtype2LongLatAxis(x,fits)
/*
Make cube.axis (long/lat) information from FITS-CTYPE keyword

1999/8/2 T.Handa
*/
AXIS *x; /* <out> */
char *fits; /* <in> FITS value e.g. RA---ARC */
{
  int ii,iflag;
  int l_str,l_str_name;
  char fits_axis_name[81],fits_axis_project[81];

/* divide FITS axis name into axis.name, axis.project */
  strcpy(fits_axis_name,"");
  strcpy(fits_axis_project,"");
  l_str=strlen(fits);
  if(l_str>80) l_str=80;
  strncpy(fits_axis_name,fits,80);
  fits_axis_name[80]=(char)NULL;
  StarReduceString(fits_axis_name);
  for(ii=0;ii<l_str;ii++) {
    if(fits_axis_name[ii]=='-') {
      fits_axis_name[ii]=(char)NULL;
      break;
    }
  }
  l_str_name=strlen(fits_axis_name);

  for(ii=l_str_name;ii<l_str;ii++) {
    if(fits[ii]!='-') {
      strcpy(fits_axis_project,&(fits[ii]));
      break;
    }
  }
  StarReduceString(fits_axis_project);

/* axis name and type */
  iflag=0;
  for(ii=0;ii<N_LongLat;ii++) {
    if(strcmp(fits_axis_name,FitsCtypeKey4LongLat1[ii])==0) {
      strcpy((*x).name,StAxisNameKey4LongLat[ii]);
      if(ii==ii/2*2) 
	strcpy((*x).type,"long");
      else 
	strcpy((*x).type,"lat");
      strcpy((*x).refer,StAxisRefer4LongLat[ii/2]);
      iflag=1;
      break;
    }
  }

  if(iflag==0) {
    strncpy((*x).name,fits_axis_name,8);
    (*x).name[8]=(char)NULL;
    strcpy((*x).type,"");
  }

/* axis projection */
  iflag=0;
  for(ii=0;ii<N_Projection;ii++) {
    if(strcmp(fits_axis_project,FitsCtypeKey4LongLat2[ii])==0) {
      strcpy((*x).project,StAxisProjKey[ii]);
      iflag=1;
      break;
    }
  }

  if(iflag==0) {
    strcpy((*x).project,"linear");
  }

  return;
}

void StarFitsCtype2Axis(x,fits)
/*
Make cube.axis information from FITS-CTYPE keyword

1999/8/3 T.Handa
*/
AXIS *x;    /* <out> */
char *fits; /* <in> FITS CTYPE keyword */
{
  int ii,iflag;

/* axis with spheric-plane projection */
  StarFitsCtype2LongLatAxis(x,fits);
  if(strcmp((*x).type,"long")==0 || strcmp((*x).type,"lat")==0) {
    return;
  }

  iflag=0;
  for(ii=0;ii<N_AxisType;ii++) {
    if(strcmp(fits,FitsCtypeKey[ii])==0) {
      strcpy((*x).type,StAxisType[ii]);
      iflag=1;
      return;
    }
  }

  if(iflag==0) {
    strcpy((*x).type,fits);
  }


  return;
}

void StarHeadAxisFromFitsHead(x,fits_axis)
/*
convert cube.axis.x,y,z from FITS header record

1999/8/3 T.Handa
*/
AXIS *x; /* <out> cube.axis */
FITS_AXIS *fits_axis; /* <in> for FITS */
{
  (*x).size=(*fits_axis).naxis;
  (*x).delta=(*fits_axis).cdelt;
  (*x).origin=(*fits_axis).crval;
  (*x).pixel_origin=(*fits_axis).crpix-1.0;
  (*x).rotation=(*fits_axis).crota;
  strcpy((*x).unit,(*fits_axis).cunit);
  StarReduceString((*x).unit);
  if(strcmp((*x).unit,"")==0) strcpy((*x).unit,"pixel");

  StarFitsCtype2Axis(x,(*fits_axis).ctype);
  StarReduceString((*x).name);
  StarReduceString((*x).type);
  StarReduceString((*x).project);
  StarReduceString((*x).refer);
  StarReduceString((*x).psf);

  if(strcmp((*x).unit,"")==0 || strcmp((*x).unit,"pixel")==0) { 
    if(strcmp((*x).type,"long")==0 || strcmp((*x).type,"lat")==0) {
      strcpy((*x).unit,"degree");
    } else if(strcmp((*x).type,"velocity")==0) {
      strcpy((*x).unit,"m/s");
    }
  }

  if(strcmp((*x).type,"velocity")==0) {
    strcpy((*x).refer,"LSR");
  }

  return;
}

void StarCutFitsComment(card)
char *card; /* <i/o> FITS card image */
/*
Omitt the comment on FITS card image

1999/8/3 T.Handa
*/
{
  int ii;
  int v_flag;

  v_flag=0;
  for(ii=9;ii<80;ii++) {
    if(v_flag==0) {
      if(card[ii]=='\'')  v_flag=1;
      else if(card[ii]=='/') {
	card[ii]=(char)NULL;
	return;
      }
    } else if(v_flag==1 && card[ii]=='\'') {
      v_flag=0;
    }
  }

  return;
}

int StarDivideFitsCard(card,key,value)
char *card; /* <in> FITS card image without comments */
char *key; /* <out> FITS keyword */
char *value; /* <out> FITS value word */
/*
Divide keyword and its value word in a FITS card image
return value type
FITS_VALUE_END,FITS_VALUE_STRING,FITS_VALUE_FLOAT,FITS_VALUE_EXP,FITS_VALUE_BOOLIAN,FITS_VALUE_NOTHING

1999/8/3 T.Handa
*/
{
  int ii,jj,l_card,l_value;
  int v_flag,i_bgn,i_end;
  int bool_flag;

/* get keyword */
  for(ii=0;ii<8;ii++) {
    key[ii]=card[ii];
    if(key[ii]==' ' || key[ii]=='=') {
      key[ii]=(char)NULL;
      key[8]=(char)NULL;
      break;
    }
  }
  if(strcmp(key,"END")==0) return(FITS_VALUE_END);

  i_bgn=0;
  i_end=0;

/* get value */
  l_card=strlen(card);
  if(card[8]=='=' || strcmp(key,"HISTORY")==0) { /* A card has a value */
    v_flag=0; /* before BLK/quart */
    for(ii=9;ii<l_card;ii++) {
      if(v_flag==0 && card[ii]!=' ') {
	v_flag=1;
	i_bgn=ii;
	if(card[ii]=='\'') {
	  v_flag=11;
	  i_bgn=ii+1;
	}
      } else if(v_flag==11 && card[ii]=='\'') {
	v_flag=12;
	i_end=ii;
      } else if(v_flag==1 && card[ii]==' ') {
	v_flag=2;
	i_end=ii;
      }
      if(v_flag==2 || v_flag==12) break;
    }

    for(ii=i_bgn,jj=0;ii<i_end;ii++,jj++) {
      value[jj]=card[ii];
    }
    l_value=jj;
    value[l_value]=(char)NULL;

/* check type of value */
    if(v_flag==12) return(FITS_VALUE_STRING); /* character */

    for(ii=0;ii<l_value;ii++) {
      if(value[ii]=='e' || value[ii]=='E') return(FITS_VALUE_EXP); /* %e */
    }
    for(ii=0;ii<l_value;ii++) {
      if(value[ii]=='.') return(FITS_VALUE_FLOAT); /* %f */
    }
    bool_flag=0;
    for(ii=0;ii<l_value;ii++) {
      if(value[ii]!=' ' && bool_flag==0) {
	if(value[ii]=='T' || value[ii]=='F') 
	  bool_flag=1;
	else 
	  bool_flag=2;
      }
    }
    if(bool_flag==1) return(FITS_VALUE_BOOLIAN);
    return(FITS_VALUE_INT); /* default %d */

/* keyword without value */
  } else {
    strcpy(value,"");
    return(FITS_VALUE_NOTHING);
  }
}

int StarFitsPixel2CubePixel(fp,cube,bitpix)
FILE *fp; /* <in> */
CUBE *cube; /* <i/o> cube.head=in, cube.pixel=out */
int bitpix; /* <in> FITS bitpix */
/*
Read pixel value in FITS to cube.pixel

1999/8/3 T.Handa
2003/8/28 T.Handa (debug) for NaN value in bitpix<0 data
*/
{
  int nx,ny,nz;
  int ix,iy,iz;
  double data_double;
  float data_float,data;
  long data_long;
  short data_short;
  unsigned char data_u_char;
  double scale,offset;

  nx=(*cube).head.axis.x.size;
  ny=(*cube).head.axis.y.size;
  nz=(*cube).head.axis.z.size;

  scale=(*cube).head.data_value.scale;
  offset=(*cube).head.data_value.offset;

  switch(bitpix) {
  case -64: /* double */
    for(iz=0;iz<nz;iz++){
      for(iy=0;iy<ny;iy++) {
	for(ix=0;ix<nx;ix++) {
	  freadIntel(&data_double,sizeof(data_double),1,fp);
	  if(finite(data_double)==0)
	    data=(*cube).head.data_value.dummy;
	  else
	    data=(float)data_double;
	  StarPutCubePixel(cube,ix,iy,iz,data);
	}
      }
    }
    break;
  case -32: /* float */
    for(iz=0;iz<nz;iz++){
      for(iy=0;iy<ny;iy++) {
	for(ix=0;ix<nx;ix++) {
	  freadIntel(&data_float,sizeof(data_float),1,fp);
	  if(finite((double)data_float)==0)
	    data=(*cube).head.data_value.dummy;
	  else
	    data=(float)data_float;
	  StarPutCubePixel(cube,ix,iy,iz,data);
	}
      }
    }
    break;
  case 32: /* long */
    for(iz=0;iz<nz;iz++){
      for(iy=0;iy<ny;iy++) {
	for(ix=0;ix<nx;ix++) {
	  freadIntel(&data_long,sizeof(data_long),1,fp);
	  data_double=(double)data_long*scale+offset;
	  data=(float)data_double;
	  StarPutCubePixel(cube,ix,iy,iz,data);
	}
      }
    }
    break;
  case 16: /* short */
    for(iz=0;iz<nz;iz++){
      for(iy=0;iy<ny;iy++) {
	for(ix=0;ix<nx;ix++) {
	  freadIntel(&data_short,sizeof(data_short),1,fp);
	  data_double=(double)data_short*scale+offset;
	  data=(float)data_double;
	  StarPutCubePixel(cube,ix,iy,iz,data);
	}
      }
    }
    break;
  case 8: /* unsigned char */
    for(iz=0;iz<nz;iz++){
      for(iy=0;iy<ny;iy++) {
	for(ix=0;ix<nx;ix++) {
	  freadIntel(&data_u_char,sizeof(data_u_char),1,fp);
	  data_double=(double)data_u_char*scale+offset;
	  data=(float)data_double;
	  StarPutCubePixel(cube,ix,iy,iz,data);
	}
      }
    }
    break;
  }

  if(bitpix>0) {
    (*cube).head.data_value.scale=1.0;
    (*cube).head.data_value.offset=0.0;

    (*cube).head.rx.scale=1.0;
  }

  return(0);
}
