/*
STAR index file processing functions

1997/11/19 T.Handa
1998/ 4/10 T.Handa : ver.3.20 (revise) command/comment lines
1998/ 8/17 T.Handa : ver.3.21 (revise) max length of default_sdbname
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <StData.h>
#include <StStd.h>

#define	SEPS	" ,\t\n"

/*
update history

1992/12/21 T.Handa
1998/ 4/10 T.Handa : ver.3.20 (revise) command/comment lines support
*/


static char GLargument[MAX_IDX_RECORD_LENGTH];
static int GLarg_flag=0;

int	StarSdbHeadIn(default_sdbfile,n_line,back_ver)
/*
Get default filename

1998/ 4/10 T.Handa : ver.3.20 (revise) backup_version
*/
char *default_sdbfile;
int *n_line;
int *back_ver;
{
  int err_code,flow_id;
  char buffer[MAX_IDX_RECORD_LENGTH];
  int l_str;
  char *ret_char;

  *n_line=0;

  ret_char=fgets(buffer,MAX_IDX_RECORD_LENGTH,stdin);
  if(ret_char==NULL) return(1);

  l_str=strlen(buffer);
  if(l_str>0 && buffer[l_str-1]=='\n') buffer[l_str-1]=(char)NULL;
/*  strncpy(default_sdbfile,buffer,9);
  default_sdbfile[9]=(char)NULL; */
  if(l_str>=MAX_FILE_NAME_LENGTH) {
    strncpy(default_sdbfile,buffer,MAX_FILE_NAME_LENGTH);
    default_sdbfile[MAX_FILE_NAME_LENGTH-1]=(char)NULL;
  } else {
    strcpy(default_sdbfile,buffer);
  }

  flow_id=StarFlowdataIdentify(buffer);
  if(flow_id!=SDB_FLOW) {
    fprintf(stderr,"Broken pipe. input ID=%s\n",buffer);
    return(999);
  }

/* get default file name */
  ret_char=fgets(buffer,MAX_IDX_RECORD_LENGTH,stdin);
  if(ret_char==NULL) return(999);

  l_str=strlen(buffer);
  if(l_str>0 && buffer[l_str-1]=='\n') buffer[l_str-1]=(char)NULL;
/*  strncpy(default_sdbfile,buffer,9);
  default_sdbfile[9]=(char)NULL; */
  if(l_str>MAX_FILE_NAME_LENGTH) {
    strncpy(default_sdbfile,buffer,MAX_FILE_NAME_LENGTH-1);
    default_sdbfile[MAX_FILE_NAME_LENGTH-1]=(char)NULL;
  } else {
    strcpy(default_sdbfile,buffer);
  }

/* get line numbers */
  ret_char=fgets(buffer,MAX_IDX_RECORD_LENGTH,stdin);
  if(ret_char==NULL) return(999);

  *n_line=atol(buffer);

/* get backup version number */
  err_code=StarSdbHeadLoad(stdin,back_ver);
  return(err_code);
}

int StarSdbHeadOut(default_sdbfile,n_line,back_ver)
/*
Put default filename
1992/12/21 T.Handa
1998/ 4/10 T.Handa : ver.3.20 (revise) backup_version
*/
char *default_sdbfile;
int n_line;
int back_ver;
{
  int err_code;

/* put data idnetifier, default file name, and line_numbers */
  printf("%s\n%s\n%d\n",STAR_SDB_IN,default_sdbfile,n_line);

/* put backup version number */
  err_code=StarSdbHeadSave(stdout,back_ver);

  return(0);
}


int StarSdbHeadLoad(fp,back_ver)
/*
Get default filename
1992/12/21 T.Handa
1998/ 4/10 T.Handa : ver.3.20 (revise) backup_version
*/
FILE *fp;
int *back_ver;
{
  char buffer[MAX_IDX_RECORD_LENGTH];
  int l_str;
  char *ret_char;

/* get data idnetifier */
  ret_char=fgets(buffer,MAX_IDX_RECORD_LENGTH,fp);
  if(ret_char==NULL) return(1);

  l_str=strlen(buffer);
  if(l_str>0 && buffer[l_str-1]=='\n') buffer[l_str-1]=(char)NULL;

  if(strncmp(buffer,"!backup=",7)==0) {
    *back_ver=atol(&(buffer[8]));
    GLargument[0]=(char)NULL;
    GLarg_flag=0;
  } else {
    *back_ver=0;
    strcpy(GLargument,buffer);
    GLarg_flag=1;
  }

/* get backup version number */
  return(0);
}

int StarSdbHeadSave(fp,back_ver)
/*
Put default filename
1992/12/21 T.Handa
1998/ 4/10 T.Handa : ver.3.20 (revise) backup_version
*/
FILE *fp;
int back_ver;
{
/* put data idnetifier */
  fprintf(fp,"!backup=%d\n",back_ver);

  return(0);
}

int StarSdbIn(filename,flag,argument)
/*
get filename, flag, and etc. from .sdb pipeflow

1997/11/19 T.Handa
*/
char *filename;
int *flag;
char *argument;
{
  int err_code;

  err_code=StarSdbLoad(stdin,filename,flag,argument);
  return(err_code);
}

int StarSdbOut(filename,flag,argument)
/*
get filename, flag, and etc. from .sdb pipeflow

1998/ 4/ 3 T.Handa 
*/
char *filename;
int flag;
char *argument;
{
  int err_code;

  err_code=StarSdbSave(stdout,filename,flag,argument);
  return(err_code);
}

int StarSdbLoad(fp_index,filename,flag,argument)
FILE *fp_index;
char *filename;
int *flag;
char *argument;
/*
function)
load a line in sdb file

return)
	0	:	nomal end
	11	:	blank line
	100	:	detect EOF (ERR_EOF_DETECT)
	201	:	wrong field number (too few)
	301	:	wrong flag assign

1992/12/1  Tomoharu Oka
1992/12/11 T.Handa (filename in c-string)
1997/11/19 T.Handa
1998/ 4/ 8 T.Handa : ver.3.20 (revise) command/comment lines
1998/ 4/10 T.Handa : ver.3.20 (revise) backup version
2002/ 5/ 8 T.Handa : ver.3.21 (debug) for blank line
*/
{
  char	buffer[MAX_IDX_RECORD_LENGTH];
  char	filename_buff[MAX_IDX_RECORD_LENGTH];
  char	flag_buff[MAX_IDX_RECORD_LENGTH];
  char	*ret_chr,*token;
  int	err_code,l_str;


  if(GLarg_flag==0) {

/* get 1 line */
    ret_chr=fgets(buffer,MAX_IDX_RECORD_LENGTH,fp_index);
    if(ret_chr==NULL)
      return(ERR_EOF_DETECT);

  } else {
    strcpy(buffer,GLargument);
    GLarg_flag=0;
  }

/* first character check */
  l_str=strlen(buffer);
  if(l_str>0 && buffer[l_str-1]=='\n')
    buffer[l_str-1]=(char)NULL;
  switch(buffer[0]) {
  case '#':
    *flag=IDX_COMMENT;
    strcpy(argument,buffer);
    filename[0]=(char)NULL;
    return(0);
  case '!':
    *flag=IDX_COMMAND;
    strcpy(argument,buffer);
    filename[0]=(char)NULL;
    return(0);
  }
  if(buffer[0]==(char)NULL) {
    *flag=IDX_COMMENT;
    argument[0]=(char)NULL;
    filename[0]=(char)NULL;
    return(11);
  }

/* divide a line by token */
/* first field is filename */
  token=strtok(buffer,SEPS);
  strcpy(filename_buff,token);
  strncpy(filename,filename_buff,strlen(filename_buff));
  if(filename[0]==(char)NULL) {
    *flag=GOOD_SCAN;
    argument[0]=(char)NULL;
    return(11);
  }

/* second field is flag */
  token=strtok(NULL,SEPS);
  filename[strlen(filename_buff)]=(char)NULL;
  if(token==NULL){
    *flag=GOOD_SCAN;
    argument[0]=(char)NULL;
    return(201);

  }
  strcpy(flag_buff,token);

/* interprete the flag */
  if((strncasecmp(flag_buff,"g",1)==0)|| (strncmp(flag_buff,"0",1)==0)){
    *flag=GOOD_SCAN;
    err_code=0;
  } else if((strncasecmp(flag_buff,"b",1)==0)||(strncmp(flag_buff,"1",1)==0)){
    *flag=BAD_SCAN;
    err_code=0;
  } else	{
    *flag=GOOD_SCAN;
    err_code=301;
  }

/* the followings are as arguments */
  token=strtok(NULL,"\n");
  if(token==NULL) argument[0]=(char)NULL;
  else strcpy(argument,token);
  return(err_code);
}

int StarSdbSave(fp_index,filename,flag,argument)
FILE *fp_index;
char *filename;
int flag;
char *argument;
/*
function)
save a line in sdb file

return)
	0	:	normal end
	101	:	invalid flag value (must be 0 or 1)

1992/12/1	Tomoharu Oka
1997/11/19 T.Handa
1998/ 4/ 8 T.Handa : ver.3.20 (revise) command/comment lines
*/
{
  char flag_word[5];

  switch(flag) {
  case IDX_COMMENT: case IDX_COMMAND:
    fprintf(fp_index,"%s\n",argument);
    return(0);

  case GOOD_SCAN:
    strcpy(flag_word,"good");
    break;
  case BAD_SCAN:
    strcpy(flag_word,"bad");
    break;
  default:
    return(101);
  }
  if(argument[0]==(char)NULL)
    fprintf(fp_index,"%s\t%s\n",filename,flag_word);
  else
    fprintf(fp_index,"%s\t%s\t%s\n",filename,flag_word,argument);
  return(0);
}
