/*
for Pipeline data I/O

1997/11/14 T.Handa ver.3.0
1998/ 4/ 7 T.Handa ver.3.2 (mixed .sdb,.scn)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <StData.h>
#include <StStd.h>

int StarPipeHeadIn(id_str,n_file)
/*
Get in the multi-Pipeline header from the pipeline

1997/11/20 T.Handa
*/
char *id_str; /* <out> header ID strings */
int *n_file; /* <out> multipulicity */
{
  int err_code;

  err_code=StarPipeHeadLoad(id_str,n_file,stdin);
  return(err_code);
}

int StarPipeHeadOut(n_file)
/*
Put out the multi-Pipeline header into the pipeline

1997/11/20 T.Handa
*/
int n_file; /* <in> multipulicity */
{
  int err_code;

  err_code=StarPipeHeadSave(STAR_PIPE_IN,n_file,stdout);
  return(err_code);
}

int StarPipeHeadLoad(id_str,n_file,fp)
/*
Load the multi-Pipeline header from a file

1997/11/20 T.Handa
*/
char *id_str; /* <out> header ID strings */
int *n_file; /* <out> multipulicity */
FILE *fp;
{
  char buffer[BUFFER_SIZE];
  char *f_get;
  int err_code;
  int l_str;

  err_code=0;

  f_get=fgets(buffer,BUFFER_SIZE,fp);
  if(f_get==NULL) return(ERR_EOF_DETECT);

  l_str=strlen(buffer);
  if(buffer[l_str-1]=='\n') buffer[l_str-1]=(char)NULL;

  strcpy(id_str,buffer);
  if(StarFlowdataIdentify(buffer)!=PIPE_FLOW) {
    return(1);
 }

  f_get=fgets(buffer,BUFFER_SIZE,fp);
  if(f_get==NULL) return(ERR_EOF_DETECT);
  *n_file=atoi(buffer);

  return(err_code);
}

int StarPipeHeadSave(id_str,n_file,fp)
/*
Save the multi-Pipeline header into a file

1997/11/20 T.Handa
*/
char *id_str; /* <in> header ID strings */
int n_file; /* <in> multipulicity */
FILE *fp;
{
  fprintf(fp,"%s\n%d\n",id_str,n_file);
  return(0);
}

int StarPipeEndIn(id_str)
/*
Get in the multi-Pipeline header from the pipeline

1997/11/20 T.Handa
*/
char *id_str; /* <out> header ID strings */
{
  int err_code;

  err_code=StarPipeEndLoad(id_str,stdin);
  return(err_code);
}

int StarPipeEndOut()
/*
Put out the multi-Pipeline header into the pipeline

1997/11/20 T.Handa
*/
{
  int err_code;

  err_code=StarPipeEndSave(STAR_PIPE_END,stdout);
  return(err_code);
}

int StarPipeEndLoad(id_str,fp)
char *id_str; /* <out> header ID strings */
FILE *fp;
/*
Load the multi-Pipeline terminator from a file

1997/11/20 T.Handa
*/
{
  char buffer[BUFFER_SIZE];
  char *f_get;
  int err_code;
  int l_str;

  err_code=0;

  f_get=fgets(buffer,BUFFER_SIZE,fp);
  if(f_get==NULL) return(ERR_EOF_DETECT);

  l_str=strlen(buffer);
  if(buffer[l_str-1]=='\n') buffer[l_str-1]=(char)NULL;

  if(StarFlowdataIdentify(buffer)!=PIPE_FLOW_END) {
    strcpy(id_str,buffer);
    return(1);
 }

  return(err_code);
}

int StarPipeEndSave(id_str,fp)
/*
Save the multi-Pipeline terminator into a file

1997/11/20 T.Handa
*/
char *id_str; /* <in> header ID strings */
FILE *fp;
{
  fprintf(fp,"%s\n",id_str);
  return(0);
}

/*------------------------*/

int StarFlowdataIdentify(id_str)
char *id_str;
/*

1998/4/7 T.Handa
*/
{
  int id_code;

  if(strcmp(id_str,STAR_PIPE_IN)==0) {
    id_code=PIPE_FLOW;
  } else if(strcmp(id_str,STAR_PIPE_END)==0) {
    id_code=PIPE_FLOW_END;
  } else if(strcmp(id_str,STAR_SCN_IN)==0) {
    id_code=SCN_FLOW;
  } else if(strcmp(id_str,STAR_CUB_IN)==0) {
    id_code=CUB_FLOW;
  } else if(strcmp(id_str,STAR_SDB_IN)==0) {
    id_code=SDB_FLOW;
  } else if(strcmp(id_str,STAR_CDB_IN)==0) {
    id_code=CDB_FLOW;
  } else {
    id_code=UNKNOWN_FLOW;
  }
  return(id_code);
}
