/*
STAR profile data I/O functions
Part 2. only for Profile data

1997/11/28 T.Handa
1998/4/23 T.Handa (revise) .sdb
1998/5/15 T.Handa (debug) StarProfileIn();
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <StData.h>
#include <StStd.h>

#include "StDataIO.c"

int StarProfileIn(profile,sdb_flag,sdb_arg)
/*
profile from the pipeline
return
0 : normal
1 : garbage profile data
101 : too many history steps
102 : too many lines in a history step
111 : not "END" nor "end" for the last history step
999 : invalid format data

1992/12/19 T.Handa
1998/ 4/10 T.Handa ver.3.2 (revise) .scn extension for .sdb
*/
PROFILE	*profile;
int *sdb_flag;
char *sdb_arg;
{
  int	err_code,flow_code,
    l_file_name;
  char	*ret_char;
  char	buffer[MAX_IDX_RECORD_LENGTH];
  int	l_str;

/* get data idnetifier */
  ret_char=fgets(buffer,MAX_IDX_RECORD_LENGTH,stdin);
  if(ret_char==NULL) {
    (*profile).sys_head.l_file=0;
    *sdb_flag=BLANK_FLOW;
    *sdb_arg=(char)NULL;
    return(1);
  }
  buffer[MAX_IDX_RECORD_LENGTH-1]=(char)NULL;
  l_str=strlen(buffer);
  if(buffer[l_str-1]=='\n') buffer[l_str-1]=(char)NULL;

  flow_code=StarFlowdataIdentify(buffer);

  switch(flow_code) {
  case SCN_FLOW:
    break;
  default:
    fprintf(stderr,"not .scn. ID=%s\n",buffer);
    return(999);
  }

/* profile flow */
  strncpy((*profile).file_id,buffer,8);
  (*profile).file_id[8]=(char)NULL;

/* get file name */
  ret_char=fgets(&((*profile).file_name[0]),MAX_FILE_NAME_LENGTH,stdin);
  if(ret_char==NULL) return(999);
  l_file_name=strlen((*profile).file_name)-1;
  (*profile).file_name[l_file_name]=(char)NULL;

/* get profile flag and argument */
  ret_char=fgets(buffer,MAX_IDX_RECORD_LENGTH,stdin);
  if(ret_char==NULL) return(999);
  *sdb_flag=atoi(buffer);

  ret_char=fgets(sdb_arg,MAX_IDX_RECORD_LENGTH,stdin);
  if(ret_char==NULL) return(999);
  l_str=strlen(sdb_arg);
  if(sdb_arg[l_str-1]=='\n') sdb_arg[l_str-1]=(char)NULL;

  if(*sdb_flag==GOOD_SCAN || *sdb_flag==BAD_SCAN) {
/* get profile itself */
    err_code=StarProfileLoad(stdin,profile);
    return(err_code);
  } else {
    return(0);
  }
}

int StarProfileOut(profile,sdb_flag,sdb_arg)
/*
profile into the pipeline
return
0 : normal
1 : garbage profile data
999 : invalid format data

1992/12/19 T.Handa
1998/ 4/ 8 T.Handa ver.3.2 (revise) .scn extension for .sdb
*/
PROFILE	*profile;
int sdb_flag;
char *sdb_arg;
{
  int	err_code,flow_code;
  int	l_str;

  err_code=0;

/* put data idnetifier */
  flow_code=StarFlowdataIdentify((*profile).file_id);
  switch(flow_code) {
  case SCN_FLOW:
    break;
  default:
    printf("NGID\n");
    return(999);
  }
  printf("%s\n",(*profile).file_id);

/* put file name */
  printf("%s\n",(*profile).file_name);

/* put profile flag and argument */
  l_str=strlen(sdb_arg);
  if(l_str>1 && sdb_arg[l_str-1]=='\n') sdb_arg[l_str-1]=(char)NULL;
  printf("%d\n%s\n",sdb_flag,sdb_arg);

  if(sdb_flag==GOOD_SCAN || sdb_flag==BAD_SCAN) {
/* put profile itself */
    err_code=StarProfileSave(stdout,profile);
  }

  return(err_code);
}

int StarProfileLoad(file_p,profile)
/*
load profile from a file

return
0 : normal
1 : garbage profile data
101 : too many history steps
102 : too many lines in a history step
111 : not "END" nor "end" for the last history step
999 : invalid format data


1992/2/20 T.Oka
1992/4/16 T.Handa UNIX MiniSTAR
1992/5/1  T.Handa garbage data treatment : return(1)
1992/8/19 T.Oka	  (debug) in the history step loop, hist_line-->hist_step)
1992/8/26 T.Handa structurize
1992/8/27 T.OKA	(debug)	in the history step loop, "END" handling
1992/12/7 T.Handa debug for ASTROS ver.1.31,1.32
1995/2/23 Tomo OKA	(modify) add Scaling factor, hereafter sys_head.version=2
1996/2/24 T.Handa	data ver3
1999/8/13 T.Handa (debug) auto correct for ver.3 data
*/
FILE *file_p;
PROFILE *profile;
{
  unsigned int	ii;
  int err_code;
  unsigned short history_steps;
  int data_ch;

/* read STAR file system header */
  (*profile).sys_head.l_file=0L;
  err_code=read_sys_head(file_p,&((*profile).sys_head));
  if(err_code!=0) {
    fprintf(stderr,"invalid format\n");
    return(999);
  }

  if((*profile).sys_head.l_file==0L) {
    return(1);	/* garbage data */
  }
  history_steps=(unsigned short)(*profile).sys_head.hist_step;
  if(history_steps>MAX_HISTORY_STEP) {
    return(101);	/* too large history */
  }

/* read scan file header */
  err_code=read_obs_prm(file_p,&((*profile).sys_head),&((*profile).obs_prm));
  err_code=read_object(file_p,&((*profile).sys_head),&((*profile).object));
  err_code=read_ant_status(file_p,&((*profile).sys_head),&((*profile).ant_status));
  err_code=read_rx_status(file_p,&((*profile).sys_head),&((*profile).rx_status));
  err_code=read_be_status(file_p,&((*profile).sys_head),&((*profile).be_status));
  err_code=read_obs_log(file_p,&((*profile).sys_head),&((*profile).obs_log));

  data_ch=(*profile).be_status.data_ch;

/* read history */
  for(ii=0;ii<=history_steps;ii++) {
    err_code=read_history(file_p,&((*profile).sys_head),&((*profile).history[ii]));
    if(err_code!=0) {
      return(102);	/* too many lines in a history step */
    }
  }
/*  if(strncmp((*profile).history[history_steps].step,"END",3)!=0
  && strncmp((*profile).history[history_steps].step,"end",3)!=0) {
    return(111);
  }
*/

/* read spurious flag bitmap */
  err_code=read_spurious(file_p,&((*profile).sys_head),data_ch,(*profile).spurious);

/* read baseline flag bitmap */
  err_code=read_baseline(file_p,&((*profile).sys_head),data_ch,(*profile).baseline);

/* read profile data */
  err_code=read_prof(file_p,&((*profile).sys_head),data_ch,(*profile).data);

/* adjust bug for ASTROS 1.31,1.32 */
  if(strncmp((*profile).history[history_steps].step,"END",3)!=0
     && strncmp((*profile).history[history_steps].step,"end",3)!=0) {
    strcpy((*profile).history[history_steps].step,"END");
    return(111);
  }
  if((*profile).history[history_steps].step[3]!=(char)NULL) {
    (*profile).history[history_steps].step[3]=(char)NULL;
  }
/* adjust bug for ver.3 data */
  if((*profile).sys_head.version<4 && (*profile).be_status.center_ch==0.0) {
    (*profile).be_status.center_ch=(double)(*profile).be_status.basis_ch;
  }
  return(0);
}

int StarPrfheadLoad(file_p,prfhead)
/*
Load PRF_HEADER

return
0 : normal
1 : garbage profile data
101 : too many history steps
102 : too many lines in a history step
111 : not "END" nor "end" for the last history step
999 : invalid format data

1993/9/29 T.Handa
1995/2/23 Tomo OKA : add Scaling factor entry for sys_head.version=2
1996/2/24 T.Handa : data ver3
*/
FILE		*file_p;
PRF_HEADER	*prfhead;
{
  unsigned int ii;
  int err_code;

  unsigned short history_steps;

/* read STAR file system header */
  (*prfhead).sys_head.l_file=0L;
  err_code=read_sys_head(file_p,&((*prfhead).sys_head));
  if(err_code!=0) {
    fprintf(stderr,"invalid format\n");
    return(999);
  }

  if((*prfhead).sys_head.l_file==0L) {
    return(1);	/* garbage data */
  }
  history_steps=(unsigned short)(*prfhead).sys_head.hist_step;
  if(history_steps>MAX_HISTORY_STEP) {
    return(101);	/* too large history */
  }

/* read scan file header */
  err_code=read_obs_prm(file_p,&((*prfhead).sys_head),&((*prfhead).obs_prm));
  err_code=read_object(file_p,&((*prfhead).sys_head),&((*prfhead).object));
  err_code=read_ant_status(file_p,&((*prfhead).sys_head),&((*prfhead).ant_status));
  err_code=read_rx_status(file_p,&((*prfhead).sys_head),&((*prfhead).rx_status));
  err_code=read_be_status(file_p,&((*prfhead).sys_head),&((*prfhead).be_status));
  err_code=read_obs_log(file_p,&((*prfhead).sys_head),&((*prfhead).obs_log));
/* read history */
  for(ii=0;ii<=history_steps;ii++) {
    err_code=read_history(file_p,&((*prfhead).sys_head),&((*prfhead).history[ii]));
    if(err_code!=0) {
      return(102);	/* too many lines in a history step */
    }
  }
/*  if(strncmp((*prfhead).history[history_steps].step,"END",3)!=0
    && strncmp((*prfhead).history[history_steps].step,"end",3)!=0)
    {
    return(111);
    }
*/
  return(0);
}


int StarProfileSave(file_p,profile)
/*
save a PROFILE into a file

1992/2/20 T.Oka
1992/4/16 T.Handa UNIX MiniSTAR
1992/5/1  T.Handa darbage data treatment : return(1)
1992/8/19 T.Oka	  (debug) loop for history step (hist_line-->hist_step)
1992/8/26 T.Handa structurize
1995/2/23 Tomo OKA (modify) add the Scaling factor for sys_head.version=2
1996/2/24 T.Handa (modify) data ver3
*/
FILE *file_p;
PROFILE *profile;
{
  unsigned int ii;
  int err_code;
  unsigned short history_steps;
  int data_ch;

/* write STAR file system header */
  err_code=write_sys_head(file_p,&((*profile).sys_head));
  if(err_code!=0) {
    fprintf(stderr,"invalid format\n");
    return(999);
  }

  if((*profile).sys_head.l_file==0L) {
    return(1);	/* garbage data */
  }
  history_steps=(unsigned short)(*profile).sys_head.hist_step;
/* write scan file header */
  err_code=write_obs_prm(file_p,&((*profile).sys_head),&((*profile).obs_prm));
  err_code=write_object(file_p,&((*profile).sys_head),&((*profile).object));
  err_code=write_ant_status(file_p,&((*profile).sys_head),&((*profile).ant_status));
  err_code=write_rx_status(file_p,&((*profile).sys_head),&((*profile).rx_status));
  err_code=write_be_status(file_p,&((*profile).sys_head),&((*profile).be_status));
  err_code=write_obs_log(file_p,&((*profile).sys_head),&((*profile).obs_log));

  data_ch=(*profile).be_status.data_ch;
/* write history */
  for(ii=0;ii<=history_steps;ii++) {
    err_code=write_history(file_p,&((*profile).sys_head),&((*profile).history[ii]));
  }

/* write spurious bitmap */
  err_code=write_spurious(file_p,&((*profile).sys_head),data_ch,(*profile).spurious);

/* write baseline bitmap */
  err_code=write_baseline(file_p,&((*profile).sys_head),data_ch,(*profile).baseline);

/* write profile data */
  err_code=write_prof(file_p,&((*profile).sys_head),data_ch,(*profile).data);
  return(0);
}
