/*
STAR Profile data showing functions

1997/12/12 T.Handa
1998/ 4/23 T.Handa (debug) StarPlotProfile();
1998/ 5/18 T.Handa (debug) StarPlotProfileBaselineRange(),StarPlotProfileSpuriousRange();
                  (revise) StarPlotProfileStd();
1999/ 8/ 6 T.Handa StarGetProfileShowColor(),StarResetProfileShowColor();
2000/ 2/16 T.Handa (debug) StarPlotProfileStdFrame();
   (add) StarEnclosureBoxRange2InteriorBoxRange(),StarEnclosureRange(),StarInteriorRange();
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#include <Pxplib.h>
#include <PxpGlib.h>
#include <StPxpShow.h>

static char profile_color[L_COLOR_STRING],zero_level_color[L_COLOR_STRING];
static char frame_color[L_COLOR_STRING],prfname_color[L_COLOR_STRING];
static char baseline_box_color[L_COLOR_STRING],spurious_box_color[L_COLOR_STRING];
static int color_init=0;

static void set_std_linecolor()
{
  if(color_init!=0) return;
  StarResetProfileShowColor("blue","pink","black","black","cyan","pink");
  color_init=1;
  return;
}

void StarPlotProfile(profile,x_unit,x_range)
PROFILE *profile;	/* <in>  */
int x_unit;	/* <in> unit of x_range */
RANGE x_range;	/* <in> auto sorting range */
/*
plot profile data of a range in asigned unit

1997/12/20 T.Handa
*/
{
  RANGE_long ch_range;

  StarEstimateActualChRange(profile,x_range,x_unit,&ch_range);

  if(ch_range.min<1) ch_range.min=1;
  if(ch_range.max>(*profile).be_status.data_ch)
    ch_range.max=(*profile).be_status.data_ch;

  StarPlotProfileInCh(profile,ch_range,x_unit);
}

void StarPlotProfileInCh(profile,ch_range,x_unit)
PROFILE *profile;	/* <in> */
RANGE_long ch_range;	/* <in> range in channel (must be sorted) */
int x_unit;	/* <in> */
/*
plot profile data

1997/12/10 T.Handa
1998/1/5 T.Handa : use rx_status.scale
*/
{
  int n_ch,ii,jj,ch;
  PxpPoint *plot;
  double xx;
  double scale;

  scale=(*profile).rx_status.scale;
  n_ch=ch_range.max-ch_range.min+1;
  plot=(PxpPoint *)malloc(sizeof(PxpPoint)*n_ch);

  for(ch=ch_range.min,jj=0;ch<=ch_range.max;ch++,jj++) {
    ii=ch-1;
    plot[jj].y=(*profile).data[ii]*scale;
  }

  switch(x_unit) {
  case X_VELOCITY:
    for(ch=ch_range.min,jj=0;ch<=ch_range.max;ch++,jj++) {
      xx=(double)ch;
      plot[jj].x=StarCh2Velocity(profile,xx);
    }
    break;
  case X_RF_FREQ:
    for(ch=ch_range.min,jj=0;ch<=ch_range.max;ch++,jj++) {
      xx=(double)ch;
      plot[jj].x=StarCh2Frequency(profile,xx);
    }
    break;
  case X_BE_FREQ:
    for(ch=ch_range.min,jj=0;ch<=ch_range.max;ch++,jj++) {
      xx=(double)ch;
      plot[jj].x=StarCh2FreqOnBE(profile,xx);
    }
    break;
  case X_CHANNEL: default:
    for(ch=ch_range.min,jj=0;ch<=ch_range.max;ch++,jj++) {
      xx=(double)ch;
      plot[jj].x=xx;
    }
  }

/* plot actually */
  PxpGLines(plot,n_ch);

  free(plot);
}


void StarPlotProfileLabel(phys_box,phys_cbox,plot_box,plot_cbox,x_step,y_step,x_unit)
/*
plot labels of profile
1997/12/11 T.Handa
*/
BOX_RANGE phys_box,phys_cbox;	/* <in> range to plot a profile and labels (phys unit) */
BOX_RANGE plot_box,plot_cbox;	/* <in> range to plot a profile and labels (plot unit) */
double x_step;	/* <in> label interval (phys unit) */
double y_step;	/* <in> label interval (phys unit) */
int x_unit;	/* <in> phyics unit ID */
{
  PxpPoint phys0,phys1;
  PxpPoint ceil_phys0,ceil_phys1;
  PxpRange xc_range,yc_range;
  PxpPoint plot0,plot1;
  char x_lbl_format[80],y_lbl_format[80];
  double center_x,center_y;

  StarBoxRange2PxpPoint(plot_cbox,&plot0,&plot1);
  PxpPoint2Range(plot0,plot1,&xc_range,&yc_range);
  StarBoxRange2PxpPoint(phys_box,&phys0,&phys1);
  StarBoxRange2PxpPoint(phys_cbox,&ceil_phys0,&ceil_phys1);

  PxpGLabelFormat(x_lbl_format,xc_range,x_step,(char)NULL);
  PxpGLabelFormat(y_lbl_format,yc_range,y_step,(char)NULL);
  PxpGLabelInset(phys0,phys1,ceil_phys0,ceil_phys1,x_step,y_step,x_lbl_format,y_lbl_format);

  center_x=(plot_box.x.min+plot_box.x.max)/2.0;
  center_y=plot_box.y.min-0.5;

  switch(x_unit) {
  case X_CHANNEL:
    PxpDrawJustifiedString(center_x,center_y,"channel","center","hanging");
    break;
  case X_VELOCITY:
    PxpDrawJustifiedString(center_x,center_y,"Vlsr (km/s)","center","hanging");
    break;
  case X_RF_FREQ:
    PxpDrawJustifiedString(center_x,center_y,"radio freq (GHz)","center","hanging");
    break;
  case X_BE_FREQ:
    PxpDrawJustifiedString(center_x,center_y,"backend freq (MHz)","center","hanging");
    break;
  }
  PxpDrawJustifiedString(plot_box.x.min,plot_box.y.max+0.3,"Ta*(K)","right","bottom");

  return;
}

void StarPlotProfileFrame(phys_box,phys_cbox,x_step,y_step,tick_length)
/*
plot frame for profile
1997/12/12 T.Handa
*/
BOX_RANGE phys_box,phys_cbox;	/* <in> range to plot a profile and labels (phys unit) */
double x_step;	/* <in> label interval (phys unit) */
double y_step;	/* <in> label interval (phys unit) */
double tick_length;	/* <in> in plot unit */
{
  PxpPoint phys0,phys1;
  PxpPoint ceil_phys0,ceil_phys1;

  StarBoxRange2PxpPoint(phys_box,&phys0,&phys1);
  StarBoxRange2PxpPoint(phys_cbox,&ceil_phys0,&ceil_phys1);

  PxpGBox(phys0,phys1);
  PxpGTickMarkInset(phys0,phys1,ceil_phys0,ceil_phys1,x_step,y_step,tick_length);

  return;
}

int StarProfileAutoXRange(profile,x_r,x_unit,ch_r)
PROFILE *profile;	/* <in> profile */
RANGE *x_r;	/* <i/o> range along x-axis */
int x_unit;	/* <in> phyics unit ID */
RANGE_long *ch_r;	/* <out> channel range (sorted) */
/*
auto set x-range of a profile
return) x_r is... >0 revised, <=0 keep original
1997/12/12 T.Handa
*/
{
  RANGE x_range;
  RANGE_long act_ch_range;
  int ch_max,ch_min;
  int x_autoscale;

/* internal parameter setting */
  x_range.min=(*x_r).min;
  x_range.max=(*x_r).max;

  ch_max=0;
  ch_min=0;

/* default range is all */
  if(x_range.min==x_range.max) {
    if(x_unit==X_VELOCITY) {
      ch_min=1;
      ch_max=(*profile).be_status.data_ch;
      x_range.min=StarCh2Velocity(profile,(double)ch_min);
      x_range.max=StarCh2Velocity(profile,(double)ch_max);
      StarSortRange(&x_range);
    } else if(x_unit==X_RF_FREQ) {
      ch_min=1;
      ch_max=(*profile).be_status.data_ch;
      x_range.min=StarCh2Frequency(profile,(double)ch_min);
      x_range.max=StarCh2Frequency(profile,(double)ch_max);
      StarSortRange(&x_range);
    } else if(x_unit==X_BE_FREQ) {
      ch_min=1;
      ch_max=(*profile).be_status.data_ch;
      x_range.min=StarCh2FreqOnBE(profile,(double)ch_min);
      x_range.max=StarCh2FreqOnBE(profile,(double)ch_max);
      StarSortRange(&x_range);
    } else {
      x_range.min=1;
      x_range.max=(*profile).be_status.data_ch;
    }
    x_autoscale=1;
  } else {
    x_autoscale= -1;
  }

/* estimate corresponding channel range */
  StarEstimateActualChRange(profile,x_range,x_unit,&act_ch_range);
  if(x_autoscale>0 && x_unit==X_VELOCITY) {
    act_ch_range.min=ch_min;
    act_ch_range.max=ch_max;
  }

/* over/under flow correction */
  if(act_ch_range.min<1) act_ch_range.min=1;
  if(act_ch_range.max>(*profile).be_status.data_ch)
    act_ch_range.max=(*profile).be_status.data_ch;

/* set return value */
  (*ch_r).min=act_ch_range.min;
  (*ch_r).max=act_ch_range.max;
  if(x_autoscale>0) {
    (*x_r).min=x_range.min;	(*x_r).max=x_range.max;
  }

  return(x_autoscale);
}

int StarProfileAutoYRange(profile,y_r,ch_range)
PROFILE *profile;	/* <in> profile */
RANGE *y_r;	/* <i/o> range along y-axis */
RANGE_long ch_range;	/* <in> channel range (sorted) */
/*
auto set y-range of a profile
return) y_r is... >0 revised, <=0 keep original
1997/12/12 T.Handa
1997/12/17 T.Handa (revised) give +-1.0 for completely flat profile
*/
{
  int ch_max,ch_min;
  int y_autoscale;
  RANGE y_range;

/* internal parameter setting */
  y_range.min=(*y_r).min;	y_range.max=(*y_r).max;
  ch_min=ch_range.min;	ch_max=ch_range.max;

/* default range is all */
  if(y_range.min==y_range.max) {
    StarProfileMaxMin(profile,ch_max,ch_min,&(y_range.max),&(y_range.min));
    y_autoscale=1;
  } else {
    y_autoscale= -1;
  }

/* reset return value */
  if(y_autoscale>0) {
    if(y_range.min==y_range.max) {
      y_range.min=y_range.min-1.0;
      y_range.max=y_range.max+1.0;
    }
    (*y_r).min=y_range.min;	(*y_r).max=y_range.max;
  }

  return(y_autoscale);
}

void StarProfileAutoXYRange(profile,x_unit,x_auto,x_r,xc_r,dx,y_auto,y_r,yc_r,dy,ch_r)
/*
auto setting of ranges for labels and tickmarks on both axes 
1997/12/12 T.Handa
*/
PROFILE *profile;	/* <in> profile */
int x_unit;	/* <in> unit ID */
int *x_auto;	/* <out> done auto setting? */
RANGE *x_r;	/* <i/o> range */
RANGE *xc_r;	/* <out> for labels and tickmarks (phys unit) */
double *dx;	/* <out> label and tickmark interval (phys unit) */
int *y_auto;	/* <out> done auto setting? */
RANGE *y_r;	/* <i/o> range */
RANGE *yc_r;	/* <out> for labels and tickmarks (phys unit) */
double *dy;	/* <out> lable and tickmark interval (phys unit) */
RANGE_long *ch_r;	/* <out> ch range (sorted) */
{
  RANGE x_range,y_range;
  PxpRange org_range,xc_range,yc_range;
  RANGE_long act_ch_range;
  int x_autoscale,y_autoscale;
  int n_step,m_step;
  double x_step,y_step;

/* internal parameter setting */
  x_range.min=(*x_r).min;
  x_range.max=(*x_r).max;
  y_range.min=(*y_r).min;
  y_range.max=(*y_r).max;

/* estimate correstponding x range */
  x_autoscale=StarProfileAutoXRange(profile,&x_range,x_unit,&act_ch_range);

/* estimate correstponding y range */
  y_autoscale=StarProfileAutoYRange(profile,&y_range,act_ch_range);

/* range adjusting */
  if(x_range.min<x_range.max) {
    org_range.min=x_range.min; org_range.max=x_range.max;
  } else {
    org_range.min=x_range.max; org_range.max=x_range.min;
  }
  PxpGCeilingRange(org_range,&xc_range,&n_step,&x_step);

  if(y_range.min<y_range.max) {
    org_range.min=y_range.min; org_range.max=y_range.max;
  } else {
    org_range.min=y_range.max; org_range.max=y_range.min;
  }
  PxpGCeilingRange(org_range,&yc_range,&m_step,&y_step);
  if(y_autoscale>0) {
    y_range.min=yc_range.min;
    y_range.max=yc_range.max;
  }

/* set return value */
  (*ch_r).min=act_ch_range.min;
  (*ch_r).max=act_ch_range.max;

  if(x_autoscale>0) {
    (*x_r).min=x_range.min;
    (*x_r).max=x_range.max;
  }
  if(y_autoscale>0) {
    (*y_r).min=y_range.min;
    (*y_r).max=y_range.max;
  }

  *x_auto=x_autoscale;
  *y_auto=y_autoscale;

  (*xc_r).min=xc_range.min;
  (*xc_r).max=xc_range.max;
  (*yc_r).min=yc_range.min;
  (*yc_r).max=yc_range.max;

  *dx=x_step;
  *dy=y_step;

  return;
}


void StarProfileAutoRange(profile,x_unit,x_r,y_r,ch_r,dx,dy,plot_cb,phys_b,phys_cb)
/*
auto setting of coordinate location to plot profile
1997/12/12 T.Handa
*/
PROFILE *profile;	/* <in> profile */
int x_unit;	/* <in> unit ID */
RANGE x_r,y_r;	/* <in> ranges */
RANGE_long *ch_r;	/* <out> corresponded ch range (sorted) */
double *dx,*dy;	/* <out> interval of labels and tickmarks (phys unit) */
BOX_RANGE *plot_cb;	/* <out> position range of lables and tick marks (plot position) */
BOX_RANGE *phys_b;	/* <out> range for ploting (phys unit) */
BOX_RANGE *phys_cb;	/* <out> display range of frame, labels, tickmarks (phys unit) */
{
  RANGE x_range,y_range;
  RANGE xc_r,yc_r;
  PxpRange xc_range,yc_range;
  RANGE_long act_ch_range;
  int x_autoscale,y_autoscale;
  double x_step,y_step;

  BOX_RANGE phys_box;
  BOX_RANGE phys_cbox,plot_cbox;

  PxpPoint phys0,phys1;
  PxpPoint ceil_phys0,ceil_phys1;

/* internal parameter setting */
  x_range.min=x_r.min;
  x_range.max=x_r.max;
  y_range.min=y_r.min;
  y_range.max=y_r.max;

/* estimate proper range */
  StarProfileAutoXYRange(profile,x_unit,&x_autoscale,&x_range,&xc_r,&x_step,&y_autoscale,&y_range,&yc_r,&y_step,&act_ch_range);

  xc_range.min=xc_r.min;
  xc_range.max=xc_r.max;
  yc_range.min=yc_r.min;
  yc_range.max=yc_r.max;

  plot_cbox.x.min=xc_range.min;
  plot_cbox.x.max=xc_range.max;
  plot_cbox.y.min=yc_range.min;
  plot_cbox.y.max=yc_range.max;

  phys0.x=x_range.min; phys1.x=x_range.max;
  phys0.y=y_range.min; phys1.y=y_range.max;
  StarPxpPoint2BoxRange(phys0,phys1,&phys_box);

  if(xc_range.min<xc_range.max) {
    ceil_phys0.x=xc_range.min;
    ceil_phys1.x=xc_range.max;
  } else {
    ceil_phys0.x=xc_range.max;
    ceil_phys1.x=xc_range.min;
  }
  if(yc_range.min<yc_range.max) {
    ceil_phys0.y=yc_range.min;
    ceil_phys1.y=yc_range.max;
  } else {
    ceil_phys0.y=yc_range.max;
    ceil_phys1.y=yc_range.min;
  }
  StarPxpPoint2BoxRange(ceil_phys0,ceil_phys1,&phys_cbox);

/* set return value */
  (*ch_r).min=act_ch_range.min;
  (*ch_r).max=act_ch_range.max;

  *dx=x_step;
  *dy=y_step;

  (*plot_cb).x.min=plot_cbox.x.min;
  (*plot_cb).x.max=plot_cbox.x.max;
  (*plot_cb).y.min=plot_cbox.y.min;
  (*plot_cb).y.max=plot_cbox.y.max;

  (*phys_b).x.min=phys_box.x.min;
  (*phys_b).x.max=phys_box.x.max;
  (*phys_b).y.min=phys_box.y.min;
  (*phys_b).y.max=phys_box.y.max;

  (*phys_cb).x.min=phys_cbox.x.min;
  (*phys_cb).x.max=phys_cbox.x.max;
  (*phys_cb).y.min=phys_cbox.y.min;
  (*phys_cb).y.max=phys_cbox.y.max;

  return;
}

void StarPlotProfileSetRange(plot_box,phys_box)
/*
set relation between plot position and phyics range for the profile
1997/12/12 T.Handa
*/
BOX_RANGE phys_box,plot_box; /* <in> */
{
  PxpPoint phys0,phys1,cm0,cm1;

  StarBoxRange2PxpPoint(phys_box,&phys0,&phys1);
  StarBoxRange2PxpPoint(plot_box,&cm0,&cm1);
  PxpGSetRange(phys0,phys1,cm0,cm1);
}

void StarPlotProfileStdFrame(profile,x_unit,x_range,y_range,plot_box,ch_range,phys_bx)
/*
plot frame for profile in the standard format
1997/12/11 T.Handa
2000/ 2/16 T.Handa (debug) adjust for PxpGlib
*/
PROFILE *profile;	/* <in> */
int x_unit;	/* <in> */
RANGE x_range,y_range;	/* <in> */
BOX_RANGE plot_box;	/* <in> position of label and tick mark (in printing position, not in physics unit) */
RANGE_long *ch_range;	/* <out> */
BOX_RANGE *phys_bx;	/* <out> range plotted (in physics unit) */
{
  BOX_RANGE phys_box;
  BOX_RANGE phys_cbox,plot_cbox,phys_fbox;
  RANGE_long act_ch_range;
  double x_step,y_step;
  double tick_length;
  double min_span;

/* set the standard color set */
  set_std_linecolor();

/* adjust tick length */
  min_span=fabs(plot_box.x.max-plot_box.x.min);
  if(min_span>fabs(plot_box.y.max-plot_box.y.min)) 
    min_span=fabs(plot_box.y.max-plot_box.y.min);
  tick_length=0.2;
  if(tick_length*2.0>min_span) tick_length=min_span*0.4;

/* estimate x- and y- physics values */
  StarProfileAutoRange(profile,x_unit,x_range,y_range,&act_ch_range,&x_step,&y_step,&plot_cbox,&phys_box,&phys_cbox);

/* set physical coordinate */
  StarPlotProfileSetRange(plot_box,phys_box);

/* label position */
  phys_fbox=StarEnclosureBoxRange2InteriorBoxRange(phys_cbox,phys_box,x_step,y_step);

/* plot frame and label */
  PxpSetColor(frame_color);
  StarPlotProfileFrame(phys_box,phys_fbox,x_step,y_step,tick_length);
  StarPlotProfileLabel(phys_box,phys_fbox,plot_box,plot_cbox,x_step,y_step,x_unit);

/* 0K level */
  if(y_range.min*y_range.max<0.0) {
    PxpSetColor(zero_level_color);
    PxpGLine(x_range.min,0.0,x_range.max,0.0);
  }
  PxpSetColor("black");

/* set back used values */
  (*phys_bx).x.min=phys_box.x.min;
  (*phys_bx).x.max=phys_box.x.max;
  (*phys_bx).y.min=phys_box.y.min;
  (*phys_bx).y.max=phys_box.y.max;

  (*ch_range).min=act_ch_range.min;
  (*ch_range).max=act_ch_range.max;

  return;
}

void StarPlotProfileStd(profile,x_unit,x_range,y_range,plot_box,base,spur)
/*
plot profile data in a standard format
1997/12/11 T.Handa
1998/ 5/18 T.Handa (modify) base/spur plot switch
*/
PROFILE *profile;	/* <in>  */
int x_unit;	/* <in>  */
RANGE x_range,y_range;	/* <in>  */
BOX_RANGE plot_box;	/* <in> position of label and tick mark (in printing position, not in physics unit) */
int base,spur; /* <in> base,spur plot switch (0=off, 1=on) */
{
  BOX_RANGE phys_box;
  RANGE_long ch_range;
  PxpPoint name_pos;

/* set the standard color set */
  set_std_linecolor();

/* profile name position */
  name_pos.x=plot_box.x.min+0.2;
  name_pos.y=plot_box.y.max-0.5;

/* plot frame */
  StarPlotProfileStdFrame(profile,x_unit,x_range,y_range,plot_box,&ch_range,&phys_box);

  if(base>0) {
/* plot baseline range */
    PxpSetColor(baseline_box_color);
    StarPlotProfileBaselineRange(profile,x_unit,phys_box,base);
  }

  if(spur>0) {
/* plot spurious range */
    PxpSetColor(spurious_box_color);
    StarPlotProfileSpuriousRange(profile,x_unit,phys_box,spur);
  }

  PxpSetColor(prfname_color);
/* profile name */
  PxpDrawString(name_pos.x,name_pos.y,(*profile).file_name);

/* profile itself */
  PxpSetColor(profile_color);
  StarPlotProfileInCh(profile,ch_range,x_unit);
  PxpSetColor("black");

  return;
}

void StarPlotProfileBaselineRange(profile,x_unit,phys_box,show_mode)
/*
show baseline range of profile by box of 3 rms height
1998/5/18 T.Handa
*/
PROFILE *profile;	/* <in> */
int x_unit;	/* <in> */
BOX_RANGE phys_box;	/* <in> plotting range in physics value */
int show_mode; /* <in> 0=no plot, 1=filled box, 2=line box */
{
  RANGE_long base_range[MAX_CH];
  int n_range;
  double ch;
  int ii;
  BOX_RANGE fit_box;
  PxpPoint fit_box_min,fit_box_max;
  RANGE	plot_x,bs_x;

  if(show_mode<=0) return;

  plot_x.min=phys_box.x.min;
  plot_x.max=phys_box.x.max;
  StarSortRange(&plot_x);

/* adjust vertical scale */
  fit_box.y.max= (*profile).rx_status.rms*3.0;	/* 3 rms box */
  fit_box.y.min= -fit_box.y.max;
  if(fit_box.y.max>phys_box.y.max || fit_box.y.max<phys_box.y.min)
    fit_box.y.max=phys_box.y.max;
  if(fit_box.y.min>phys_box.y.max || fit_box.y.min<phys_box.y.min)
    fit_box.y.min=phys_box.y.min;

/* get baseline ranges */
  StarRReadBaselineFlags(profile,&n_range,base_range);

  for(ii=0;ii<n_range;ii++) {
    fit_box.x.min=base_range[ii].min;
    fit_box.x.max=base_range[ii].max;
    if(x_unit==X_VELOCITY) {
      ch=fit_box.x.min;
      fit_box.x.min=StarCh2Velocity(profile,ch);
      ch=fit_box.x.max;
      fit_box.x.max=StarCh2Velocity(profile,ch);
    }

    bs_x.min=fit_box.x.min;
    bs_x.max=fit_box.x.max;
    StarSortRange(&bs_x);

    if(!(bs_x.max<plot_x.min || bs_x.min>plot_x.max)) {
      if(bs_x.min<plot_x.min) bs_x.min=plot_x.min;
      if(bs_x.max>plot_x.max) bs_x.max=plot_x.max;

      fit_box_min.x=bs_x.min;
      fit_box_min.y=fit_box.y.min;
      fit_box_max.x=bs_x.max;
      fit_box_max.y=fit_box.y.max;
      PxpGBox(fit_box_min,fit_box_max);
      if(show_mode==1)
	PxpGFilledBox(fit_box_min,fit_box_max);
    }
  }
}


void StarPlotProfileSpuriousRange(profile,x_unit,phys_box,show_mode)
/*
show spurious range of profile by box
1998/5/18 T.Handa
*/
PROFILE *profile;	/* <in>  */
int x_unit;	/* <in> */
BOX_RANGE phys_box;	/* <in> plotting range in physics value */
int show_mode; /* <in> 0=no plot, 1=filled box, 2=line box */
{
  RANGE_long sp_range[MAX_CH];
  int n_range;
  double ch;
  int ii;
  BOX_RANGE sp_box;
  PxpPoint sp_box_min,sp_box_max;
  RANGE	plot_x,sp_x;

  if(show_mode<=0) return;

  plot_x.min=phys_box.x.min;
  plot_x.max=phys_box.x.max;
  StarSortRange(&plot_x);

/* adjust vertical scale */
  sp_box.y.max=phys_box.y.max;
  sp_box.y.min=phys_box.y.min;

/* get spurious ranges */
  StarRReadSpuriousFlags(profile,&n_range,sp_range);

  for(ii=0;ii<n_range;ii++) {
    sp_box.x.min=sp_range[ii].min;
    sp_box.x.max=sp_range[ii].max;
    if(x_unit==X_VELOCITY) {
      ch=sp_box.x.min;
      sp_box.x.min=StarCh2Velocity(profile,ch);
      ch=sp_box.x.max;
      sp_box.x.max=StarCh2Velocity(profile,ch);
    }

    sp_x.min=sp_box.x.min;
    sp_x.max=sp_box.x.max;
    StarSortRange(&sp_x);

    if(!(sp_x.max<plot_x.min || sp_x.min>plot_x.max)) {
      if(sp_x.min<plot_x.min) sp_x.min=plot_x.min;
      if(sp_x.max>plot_x.max) sp_x.max=plot_x.max;

      sp_box_min.x=sp_x.min;
      sp_box_min.y=sp_box.y.min;
      sp_box_max.x=sp_x.max;
      sp_box_max.y=sp_box.y.max;

      PxpGBox(sp_box_min,sp_box_max);
      if(show_mode==1)
	PxpGFilledBox(sp_box_min,sp_box_max);
    }
  }
}

void StarResetProfileShowColor(prf_clr,zero_clr,frm_clr,prfn_clr,base_clr,spur_clr)
/*
Set line colors
1999/8/6 T.Handa
*/
char *prf_clr; /* <in> for profile data line */
char *zero_clr; /* <in> for profile zero intensity level */
char *frm_clr; /* <in> for frame and labels on axes */
char *prfn_clr; /* <in> for profile name characters */
char *base_clr; /* <in> for baseline range box */
char *spur_clr; /* <in> for spurious range box */
{
  int l_str;

  l_str=strlen(prf_clr);
  if(l_str>0 && l_str<L_COLOR_STRING)
    strcpy(profile_color,prf_clr);
  l_str=strlen(zero_clr);
  if(l_str>0 && l_str<L_COLOR_STRING)
    strcpy(zero_level_color,zero_clr);
  l_str=strlen(frm_clr);
  if(l_str>0 && l_str<L_COLOR_STRING)
    strcpy(frame_color,frm_clr);
  l_str=strlen(prfn_clr);
  if(l_str>0 && l_str<L_COLOR_STRING)
    strcpy(prfname_color,prfn_clr);
  l_str=strlen(base_clr);
  if(l_str>0 && l_str<L_COLOR_STRING)
    strcpy(baseline_box_color,base_clr);
  l_str=strlen(spur_clr);
  if(l_str>0 && l_str<L_COLOR_STRING)
    strcpy(spurious_box_color,spur_clr);
  return;
}

void StarGetProfileShowColor(prf_clr,zero_clr,frm_clr,prfn_clr,base_clr,spur_clr)
/*
Get line colors
1999/8/6 T.Handa
*/
char *prf_clr; /* <out> for profile data line */
char *zero_clr; /* <out> for profile zero intensity level */
char *frm_clr; /* <out> for frame and labels on axes */
char *prfn_clr; /* <out> for profile name characters */
char *base_clr; /* <out> for baseline range box */
char *spur_clr; /* <out> for spurious range box */
{
  strcpy(prf_clr,profile_color);
  strcpy(zero_clr,zero_level_color);
  strcpy(frm_clr,frame_color);
  strcpy(prfn_clr,prfname_color);
  strcpy(base_clr,baseline_box_color);
  strcpy(spur_clr,spurious_box_color);
  return;
}

BOX_RANGE StarEnclosureBoxRange2InteriorBoxRange(enclosure_box,frame_box,x_step,y_step)
/*
2000/ 2/16 T.Handa
*/
BOX_RANGE enclosure_box; /* <in> */
BOX_RANGE frame_box; /* <in> */
double x_step,y_step; /* <in> */
{
  PxpRange enclosure_range,interior_range,frame_range;
  BOX_RANGE interior_box;

/* x-axis */
  frame_range.min=frame_box.x.min;
  frame_range.max=frame_box.x.max;
  enclosure_range.min=enclosure_box.x.min;
  enclosure_range.max=enclosure_box.x.max;
  interior_range=PxpGEnclosureRange2InteriorRange(enclosure_range,frame_range,x_step);
  interior_box.x.min=interior_range.min;
  interior_box.x.max=interior_range.max;

/* y-axis */
  frame_range.min=frame_box.y.min;
  frame_range.max=frame_box.y.max;
  enclosure_range.min=enclosure_box.y.min;
  enclosure_range.max=enclosure_box.y.max;
  interior_range=PxpGEnclosureRange2InteriorRange(enclosure_range,frame_range,y_step);
  interior_box.y.min=interior_range.min;
  interior_box.y.max=interior_range.max;

  return interior_box;
}

RANGE StarEnclosureRange(range,quantum)
/*
2000/ 2/16 T.Handa
*/
RANGE range;
double quantum;
{
  double abs_quant;
  RANGE enclo_range;

  abs_quant=fabs(quantum);
  if(range.min<range.max) {
    enclo_range.min=floor(range.min/abs_quant)*abs_quant;
    enclo_range.max=ceil(range.max/abs_quant)*abs_quant;
  } else {
    enclo_range.min=ceil(range.min/abs_quant)*abs_quant;
    enclo_range.max=floor(range.max/abs_quant)*abs_quant;
  }

  return enclo_range;
}

RANGE StarInteriorRange(range,quantum)
/*
2000/ 2/16 T.Handa
*/
RANGE range;
double quantum;
{
  double abs_quant;
  RANGE inter_range;

  abs_quant=fabs(quantum);
  if(range.min<range.max) {
    inter_range.min=ceil(range.min/abs_quant)*abs_quant;
    inter_range.max=floor(range.max/abs_quant)*abs_quant;
  } else {
    inter_range.min=floor(range.min/abs_quant)*abs_quant;
    inter_range.max=ceil(range.max/abs_quant)*abs_quant;
  }

  return inter_range;
}
