/*
STAR stdlib functions
for user-input-parameter processing

1997/11/18 T.Handa
1998/2/28 T.Handa : ver.1.01 debug for SUN-OS4.3.1
1998/4/20 T.Handa : ver.1.02 (add) StarStdAngle2deg(),StarOptionAngleRange();
1998/4/21 T.Handa : ver.1.03 (add) StarInRange(),StarInRange_long(),
	StarInLoopRange(),StarInLoopRange_long(),StarReduceString();
1998/12/22 T.Handa : ver.1.04 (add) StarOptionKeyArgPosition();
1999/ 4/22 T.Handa : ver.1.05 (add) StarIsInLongitudeRange(),StarIsLeapYear(),StarYmd2MJD();
1999/ 8/ 2 T.Handa : ver.1.06 (add) StarAdjustStrlen(),StarFitsDate2MJD(),StarFitsDate2Time1970(),StarMJD2Ymd();
2000/ 1/16 T.Handa : ver.1.07 (add) StarFileMainName()
2000/ 2/11 T.Handa : ver.1.08 (add) StarWaitMillisec()
2002/ 5/28 T.Handa : ver.1.09 (add) StarExtractPartialFilename();
2002/ 6/23 T.Handa : ver.1.10 (add) StarDivideSetOfWords(),StarReduceBlankInCstrings()
2004/12/ 1 T.Handa : ver.1.11 (add) is_extra_visible_char(),StarCheckCharType(),minstrcpy(), (debug)StarExtractPartialFilename()
2004/12/ 8 T.Handa : ver.1.12 (debug) minstrcpy()
2007/11/20 T.Handa : ver.1.13 (add) Stardeg2StdAngle()
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/time.h>
#include <ctype.h>

#include <StData.h>
#include <StStd.h>

int StarGetPrm(prm_file,n_key,key,key_flag,value)
/*
Get a value for keyword from the .prm file

1997/11/17 T.Handa
*/
FILE *prm_file;
int n_key;	/* <in>  The total number of keys */
char **key;	/* <in>  word list of keys */
int *key_flag;	/* <out> Is key in .prm file? =0 No, <0 Yes. */
char **value;	/* <out> list of values in character */
{
  char	record[MAX_PRM_RECORD_LENGTH],
    keyword[MAX_PRM_RECORD_LENGTH],
    keyvalue[MAX_PRM_RECORD_LENGTH];
  char	*f_rec;
  int ii,jj;

/* clear read flags */
  for(ii=0;ii<n_key;ii++) key_flag[ii]=0;

  jj=0;
  while(1){
    f_rec=fgets(record,MAX_PRM_RECORD_LENGTH-1,prm_file);
    if(f_rec==NULL) return 0;
    if(record[0]!='#') {
      jj++;
      StarDividePrmRecord(record,keyword,keyvalue);
      for(ii=0;ii<n_key;ii++) {
	if(strcmp(keyword,key[ii])==0){
	  if(key_flag[ii]==0) {
	    value[ii]=(char *)malloc(sizeof(char)*MAX_PRM_RECORD_LENGTH);
	    key_flag[ii]= -jj;
	  }
	  strcpy(value[ii],keyvalue);
	}
      }
    }
  }
}

int StarGetOption(argc,argv,default_prmfile,n_key,key,key_flag,value)
/*
Get a value for a keyword from arguments and the .prm file

1997/11/13 T.Handa
*/
int argc;
char **argv;
char *default_prmfile;/* <in> */
int n_key;	/* <in>  max number of keywords */
char **key;	/* <in>  keywords' list */
int *key_flag;	/* <out> =0 not asigned, >0 argv index, <0 .prm index */
char **value;
{
  static char yes[]="yes";

  int ii,jj;
  char option_key[MAX_PRM_RECORD_LENGTH];
  char prm_filename[MAX_FILE_NAME_LENGTH];
  FILE *prm_file;
  int err_code;

  prm_file=NULL;
/* clear key_flag */
  for(ii=0;ii<n_key;ii++) key_flag[ii]=0;

/* search default prm file */
  err_code=StarStdFileExtName("STAR_PARM",default_prmfile,prm_filename,".prm");
  prm_file=fopen(prm_filename,"r");
	
/* scan paramter file name */
  for(ii=1;ii<argc;ii++) {
    if(strcmp(argv[ii],"-prm")==0) {
      ii++;
      err_code=StarStdFileExtName("STAR_PRM",argv[ii],prm_filename,".prm");
      if(prm_file!=NULL) fclose(prm_file);
      prm_file=fopen(prm_filename,"r");
      if(prm_file==NULL)
	fprintf(stderr,"Open %s fail!\n",prm_filename);
      break;
    } else if(strcmp(argv[ii],"-keywordlist")==0
	      ||strcmp(argv[ii],"-keyword_list")==0) {
      fprintf(stderr,"%s ",argv[0]);
      for(jj=0;jj<n_key;jj++) {
	if(strcmp(key[jj],"-")==0)
	  fprintf(stderr,"%s ",key[jj]);
	else
	  fprintf(stderr,"-%s ",key[jj]);
      }
      fprintf(stderr,"\n");
      exit(1);
    }
  }

/* read prm file */
  if(prm_file!=NULL) {
    StarGetPrm(prm_file,n_key,key,key_flag,value);
    fclose(prm_file);
  }

/* scan option keys */
  for(jj=0;jj<n_key;jj++) {
    strcpy(option_key,"-");
    if(strcmp(key[jj],"-")!=0) {
      strcat(option_key,key[jj]);
    }
    for(ii=1;ii<argc;ii++) {
      if(strcmp(argv[ii],option_key)==0) {
	if(key_flag[jj]!=0) free(value[jj]);
	key_flag[jj]=ii;
	if(ii==argc-1) {
	  value[jj]=(char *)yes;
	} else {
	  value[jj]=argv[ii+1];
	}
      }
    }
  }
  return 0;
}

int StarKeylessArgPosition(argc,argv,n_key,key_flag,key_type)
/*
Get the last value without a keyword from argv[].
If not exist, return value is 0.

1997/11/20 T.Handa
*/
int argc;
char **argv;
int n_key;	/* <in>  max number of keywords */
int *key_flag;	/* <in> =0 not asigned, >0 argv index, <0 .prm index */
int *key_type;	/* <in> =0 key without prm, !=0 key with a prm */
{
  int ii,jj;
  int keypos;
  int found_flag;

  keypos=0;

  for(ii=1;ii<argc;ii++) {
    found_flag=0;
    for(jj=0;jj<n_key;jj++) {
      if(ii==key_flag[jj]) {
	if(key_type[jj]!=0) ii++;
	found_flag=1;
	break;
      }
    }
    if(found_flag==0) {
      keypos=ii;
    }
  }
  if(keypos>=argc) keypos=0;
  return keypos;
}

char *StarKeylessOption(argc,argv,n_key,key_flag,key_type)
/*
Get the last value without a keyword from argv[]

1997/11/20 T.Handa
*/
int argc;
char **argv;
int n_key;	/* <in>  max number of keywords */
int *key_flag;	/* <in> =0 not asigned, >0 argv index, <0 .prm index */
int *key_type;	/* <in> =0 key without prm, !=0 key with a prm */
{
  char *found_word;
  int keypos;
  static char null_str[]="";

  keypos=StarKeylessArgPosition(argc,argv,n_key,key_flag,key_type);

  if(keypos>0) found_word=argv[keypos];
  else found_word=null_str;

  return found_word;
}

char *StarKeylessMultiOption(argc,argv,n_key,key_flag,key_type)
/*
Get a value without a keyword from argv[] in the order from argv[1]
to argv[argc-1]
Followed keyless words can be gotten by repeated call of this function.

1997/11/20 T.Handa
*/
int argc;
char **argv;
int n_key;	/* <in>  max number of keywords */
int *key_flag;	/* <in> =0 not asigned, >0 argv index, <0 .prm index */
int *key_type;	/* <in> =0 key without prm, !=0 key with a prm */
{
  static int start_ii=1;

  char *found_word;
  int ii,jj;
  int keypos;
  int found_flag;
  static char null_str[]="";

  keypos=0;

  for(ii=start_ii;ii<argc;ii++) {
    found_flag=0;
    for(jj=0;jj<n_key;jj++) {
      if(ii==key_flag[jj]) {
	if(key_type[jj]!=0) ii++;
	found_flag=1;
	break;
      }
    }
    if(found_flag==0) {
      keypos=ii;
      break;
    }
  }
  if(keypos>0) found_word=argv[keypos];
  else found_word=null_str;

  start_ii=keypos+1;

  return found_word;
}

void StarFreeAllPrms(n_key,key_flag,value)
/*
free memory area for value[] read from the .prm file.
Not affect on memories which are already free in StarGetOption();

1997/11/17 T.Handa
*/
int n_key;
int *key_flag;	/* <in> <0 be free, (=0 not malloc(), >0 correcpond to argv[]) */
char **value;	/* <in> list of value words */
{
  int ii;

  for(ii=0;ii<n_key;ii++) {
    if(key_flag[ii]<0) free(value[ii]);
  }

  return;
}

void FreeAllWords(words,n_word)
/*
free memory area for words[]

1997/11/17 T.Handa
*/
char **words;	/* <in> the pointer for the word array */
int n_word;	/* <in> the number of entries in the **words */
{
  char *str;
  int ii;

  for(ii=0;ii<n_word;ii++) {
    str=words[ii];
    if(str==NULL) break;
    free(str);
  }
  free(words);

  return;
}

char **AllocWordsStrage(n_word)
/*
allocate memory area for words[]

1997/11/17 T.Handa
*/
int n_word;	/* <in> max number of entries */
{
  char **words;	/* entried keywords */
  words=(char **)malloc(sizeof(char *)*n_word);

  return words;
}

char **AllocAllWords(n_word,l_word)
/*
allocate memory area for words[]

1997/11/19 T.Handa
*/
int n_word;	/* <in> max number of entries */
int l_word;	/* <in> longest word length */
{
  char **words;	/* entried keywords */
  int ii;

  words=(char **)malloc(sizeof(char *)*n_word);

  for(ii=0;ii<n_word;ii++) {
    words[ii]=(char *)malloc(sizeof(char)*l_word);
  }

  return words;
}

int AddNewWord(words,new_word)
/*
add a pointer of a new word for words[]  
<return> current number of entried words

1997/11/17 T.Handa
*/
char **words;	/* <in> entried keywords */
char *new_word;	/* <in> a word to entry */
{
  static int i_pos=0;

  words[i_pos]=new_word;
  i_pos++;

  return i_pos;
}

int StarOptionFlag(n_scan,logic_flag,key_flag)
/*
check option flags with positive and negative logic
valid with StarGetOption();

logic_flag[]
   its abs value shows the keyword position to check in key_flag[]
   A positive or 0 value means positive logic.
   A negative means negative logic.
   (Therefore, keyword[0] must be positive logic.)
key_flag[]
   stores the result of keyword scan with StarGetOption().
   its abs value shows the parameter position.
   The larger value is effective because of the later assignment.
   Negative value means the parameter is assigned in the prm file.
   Positive value means the paramter is assigned in argv[] array.

<return>
FLAG_YES/FLAG_NO
  The resultant assignment by the prm file and argv[] array
  after the check shown above.

1997/11/17 T.Handa
*/
int n_scan;	/* <in>  total size of the logic_flag[] array */
int *logic_flag;	/* <in>  logic and keyword poisions */
int *key_flag;	/* <in>  parameter scan resultant flags */
{
  int ii,jj;
  int final_key_pos; /* final position in argv/.prm for key_flag[] */
  int i_pos; /* final position in logic_flag[] */

  final_key_pos=0;

/* scan for argv[] */
  i_pos=0;
  for(ii=0;ii<n_scan;ii++) {
    if(logic_flag[ii]<0) jj= -logic_flag[ii];
    else jj=logic_flag[ii];

    if(key_flag[jj]>0 && final_key_pos < key_flag[jj]) {
      i_pos=ii;
      final_key_pos=key_flag[jj];
    }
  }

  if(final_key_pos>0) {	/* found in argv[] */
    if(logic_flag[i_pos]<0) return FLAG_NO;
    return FLAG_YES;
  }

/* scan for .prm */
  for(ii=0;ii<n_scan;ii++) {
    if(logic_flag[ii]<0) jj= -logic_flag[ii];
    else jj=logic_flag[ii];

    if(key_flag[jj]<0 && final_key_pos < -key_flag[jj]) {
      i_pos=ii;
      final_key_pos= -key_flag[jj];
    }
  }

  if(final_key_pos>0) {
    if(logic_flag[i_pos]<0) return FLAG_NO;
    return FLAG_YES;
	}
  return 0;
}

char *StarOptionValue(n_scan,scan_flag,key,key_type,key_flag,value)
/*
check option parameter values with redundant keywords
valid with StarGetOption();

scan_flag[]
  keyword position in key_flag[] to check
key_flag[]
   stores the result of keyword scan with StarGetOption().
   its abs value shows the parameter position.
   The larger value is effective because of the later assignment.
   Negative value means the parameter is assigned in the prm file.
   Positive value means the paramter is assigned in argv[] array.
value[]
  store the result of value scan with StarGetOption();

<return>
  The pointer of the resultant key position after the check shown above.
  return the pointer to the empty character array if no assignment

1997/11/20 T.Handa
*/
int n_scan;	/* <in>  total size of the scan_flag[] array */
int *scan_flag;	/* <in>  keyword poisions list */
char **key;	/* <in>  keywords' list */
int *key_flag;	/* <in>  parameter scan resultant flags */
int *key_type;	/* <in> =0 key without prm, !=0 key with a prm */
char **value;	/* <in>  parameter scan resultant values */
{
  static char null_str[]="";

  int j_pos;

  j_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(j_pos<0) {
    return(null_str);
  } else {
    if(key_type[j_pos]==0) 
      return(key[j_pos]);
    else 
      return(value[j_pos]);
  }
}

int StarOptionKeyArgPosition(n_scan,scan_flag,key_flag)
/*
get option parameter key position which should finally used
valid with StarGetOption();

scan_flag[]
  keyword position in key_flag[] to check
key_flag[]
   stores the result of keyword scan with StarGetOption().
   its abs value shows the parameter position.
   The larger value is effective because of the later assignment.
   Negative value means the parameter is assigned in the prm file.
   Positive value means the paramter is assigned in argv[] array.

<return>
  The resultant key position after the check shown above.
  0 for no assignment

1998/12/22 T.Handa
*/
int n_scan;	/* <in>  total size of the scan_flag[] array */
int *scan_flag;	/* <in>  keyword poisions list */
int *key_flag;	/* <in>  parameter scan resultant flags */
{
  int key_pos,final_pos;

  key_pos=(int)StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  final_pos=key_flag[key_pos];

  return final_pos;
}

int StarOptionKeyPosition(n_scan,scan_flag,key_flag)
/*
check option parameter values with redundant key position
valid with StarGetOption();

scan_flag[]
  keyword position in key_flag[] to check
key_flag[]
   stores the result of keyword scan with StarGetOption().
   its abs value shows the parameter position.
   The larger value is effective because of the later assignment.
   Negative value means the parameter is assigned in the prm file.
   Positive value means the paramter is assigned in argv[] array.

<return>
  The resultant key position after the check shown above.
  -1 for no assignment

1997/11/20 T.Handa
*/
int n_scan;	/* <in>  total size of the scan_flag[] array */
int *scan_flag;	/* <in>  keyword poisions list */
int *key_flag;	/* <in>  parameter scan resultant flags */
{
  int ii,jj;
  int final_key_pos; /* final position in argv/.prm */
  int j_pos; /* final position in value[] */

  final_key_pos=0;

/* scan for argv[] */
  jj=scan_flag[0];
  j_pos=jj;
  for(ii=0;ii<n_scan;ii++) {
    jj=scan_flag[ii];

    if(key_flag[jj]>0 && final_key_pos < key_flag[jj]) {
      j_pos=jj;
      final_key_pos=key_flag[jj];
    }
  }

  if(final_key_pos>0) { /* found in argv[] */
    return(j_pos);
  }

/* scan for .prm */
  for(ii=0;ii<n_scan;ii++) {
    jj=scan_flag[ii];

    if(key_flag[jj]<0 && final_key_pos < -key_flag[jj]) {
      j_pos=jj;
      final_key_pos= -key_flag[jj];
    }
  }

  if(final_key_pos>0) {
    return(j_pos);
  }
  return(-1);
}

void StarOptionRange(word,range)
/*
divide paramter for ranges
separator is one of the following 6 characters
  , --> comma
    --> space
  : --> colon
  ; --> semiccolon
  \n --> CR
  \t --> tab

1997/11/18 T.Handa
1998/ 4/16 T.Handa (modify) give the same values for word without any separator
*/
char *word;	/* <in>  parameter scan resultant value */
RANGE *range;	/* <out> the resultant ranges */
{
  char *token;
  static char separator[]=", :;\n\t";

  token=strtok(word,separator);
  if(token==NULL) return;
  (*range).min=atof(token);
  token=strtok(NULL,separator);
  if(token==NULL) {
    (*range).max=(*range).min;
    return;
  }
  (*range).max=atof(token);
}

void StarOptionAngleRange(word,range)
/*
Divide paramter for a range
Each separated parameter is in STAR standard angle format 
or a simple floating number.
Separators are the following 6 characters; 
,(comma) (black):(colon);(semicolon)\n(CR)\t(tab)

1998/ 4/20 T.Handa 
*/
char *word;	/* <in>  parameter scan resultant value */
RANGE *range;	/* <out> the resultant ranges (two doubles) */
{
  char *token;
  static char separator[]=", :;\n\t";

  token=strtok(word,separator);
  if(token==NULL) return;
  (*range).min=StarStdAngle2deg(token);
  token=strtok(NULL,separator);
  if(token==NULL) {
    (*range).max=(*range).min;
    return;
  }
  (*range).max=StarStdAngle2deg(token);
  return;
}

int StarOptionRanges(word,n_range,range)
/*
Divide a paramter word for several ranges
Each range is separated by :(colon) or ;(semicolon).
Each separated parameter in a range is in a simple floating number.
Separators are the following 6 characters; 
,(comma) (black)\n(CR)\t(tab)

1997/11/18 T.Handa
*/
char *word;	/* <in>  parameter scan resultant value */
int *n_range;	/* <out> the number of resultant ranges */
RANGE *range;	/* <out> the resultant ranges */
{
  int ii,jj;
  int l_str;
  char *buffer;
  int i_pos;

  l_str=strlen(word);
  buffer=(char *)malloc(sizeof(char)*l_str);
  strcpy(buffer,word);
  ii=0;
  i_pos=0;
  for(jj=0;jj<=l_str;jj++) {
    switch(buffer[jj]) {
    case ':': case ';':
      buffer[jj]=(char)NULL;
    case (char)NULL:
      StarOptionRange(&(buffer[i_pos]),&(range[ii]));
      i_pos=jj+1;
      ii++;
      if(ii>MAX_RANGE_NUMBER) {
	*n_range=ii;
	return(ERR_TOO_MANY_PRMS);
      }
    }
  }

  free(buffer);
  *n_range=ii;
  return 0;
}

void StarOptionRange_long(word,range)
/*
separate a paramter for ranges with long_integer values.
the separater is one of the following 6 characters ", :;\n\t"

1997/11/18 T.Handa
*/
char *word;	/* <in>  parameter scan resultant value */
RANGE_long *range;	/* <out> the resultant ranges */
{
  char *token;
  static char separator[]=", :;\n\t";

  token=strtok(word,separator);
  if(token==NULL) return;
  (*range).min=atol(token);
  token=strtok(NULL,separator);
  if(token==NULL) {
    (*range).max=(*range).min;
    return;
  }
  (*range).max=atol(token);
}

int StarOptionRanges_long(word,n_range,range)
/*
separate a paramter for ranges with long_integer ranges.
the separater is one of the following 5 characters " :;\n\t"

1997/11/18 T.Handa
*/
char *word;	/* <in>  parameter scan resultant value */
int *n_range;	/* <out> the number of resultant ranges */
RANGE_long *range;	/* <out> the resultant ranges */
{
  int ii,jj;
  int l_str;
  char *buffer;
  int i_pos;

  l_str=strlen(word);
  buffer=(char *)malloc(sizeof(char)*l_str);
  strcpy(buffer,word);
  ii=0;
  i_pos=0;
  for(jj=0;jj<=l_str;jj++) {
    switch(buffer[jj]) {
    case ':': case ';':
      buffer[jj]=(char)NULL;
    case (char)NULL:
      StarOptionRange_long(&(buffer[i_pos]),&(range[ii]));
      i_pos=jj+1;
      ii++;
      if(ii>MAX_RANGE_NUMBER) {
	*n_range=ii;
	return(ERR_TOO_MANY_PRMS);
      }
    }
  }

  free(buffer);
  *n_range=ii;
  return 0;
}

int StarDivideSetOfWords(word,sep,tops)
/*
Divide a paramter word for several ranges
Each range is separated by :(colon) or ;(semicolon).

<return> the number of divided parts

2002/ 6/14 T.Handa
*/
char *word;	/* <i/o>  parameter scan for resultant value */
char *sep; /* <in> separators */
char **tops; /* <out> pointers to accesss each divided part */
{
  int ii,jj,i_sep,n_sep;
  int l_str;
  int i_pos;

  l_str=strlen(word);
  n_sep=strlen(sep);
  ii=0;
  i_pos=0;

  for(jj=0;jj<=l_str;jj++) {
    for(i_sep=0;i_sep<n_sep;i_sep++) {
      if(word[jj]==sep[i_sep] || word[jj]==(char)NULL) {
	word[jj]=(char)NULL;
	tops[ii]=&(word[i_pos]);
	i_pos=jj+1;
	ii++;
	if(ii>MAX_SET_OF_WORDS)
	  return ii;
	break;
      }
    }
  }

  return ii;
}

void StarDividePrmRecord(record,keyword,value)
/*
divide a prm record into keyword and value by =
omit tab and space

1992/11/27 T.Handa
*/
char	record[];
char	keyword[];
char	value[];
{
  static char	delimiter='=';
  int	i_rec,i_key,i_value;

/* find first nonNULL character */
  i_rec=0;
  while(record[i_rec]!=(char)NULL) {
      if(record[i_rec]!=' ' && record[i_rec]!='\t') break;
      i_rec++;
    }

/* make keyword */
  i_key=0;
  while(record[i_rec]!=(char)NULL && record[i_rec]!=delimiter) {
    /* cutout comment part */
    if(record[i_rec]=='/' && record[i_rec+1]=='*') break;
    /* cutout CR for fgets */
    if(record[i_rec]=='\n') break;

    keyword[i_key]=record[i_rec];
    i_rec++; i_key++;
  }

  keyword[i_key]=(char)NULL;

/* discard additional space & tab */
  i_key--;
  for(;i_key>0;i_key--) {
    if(keyword[i_key]!=' ' && keyword[i_key]!='\t') break;
    keyword[i_key]=(char)NULL;
  }

/* make value */
  if(record[i_rec]==delimiter) i_rec++;
  while(record[i_rec]==' ' || record[i_rec]=='\t')
    i_rec++;

  i_value=0;
  while(record[i_rec]!=(char)NULL) {
    /* cutout comment part */
    if(record[i_rec]=='/' && record[i_rec+1]=='*') break;
    /* cutout CR for fgets */
    if(record[i_rec]=='\n') break;
    value[i_value]=record[i_rec];
    i_rec++; i_value++;
  }
  value[i_value]=(char)NULL;

/* discard additional space & tab */
  i_value--;
  for(;i_value>0;i_value--) {
    if(value[i_value]!=' ' && value[i_value]!='\t') break;
    value[i_value]=(char)NULL;
  }

  return;
}


void StarAutoScale(orig_range,ceil_range,n_step,step)
/*
auto scaling to a round value with a roud step for tick marks

1992/9/7 T.Handa
*/
RANGE	orig_range,	/* original range */
	*ceil_range;	/* ceiled range */
int	*n_step;	/* number of steps */
double	*step;		/* interval of steps */
{
  double	span;

  span=orig_range.max-orig_range.min;

  *step=pow(10.0, floor( log10(span) ))/10.0;
  *n_step=floor(span/(*step));

  if((*n_step)>20) { 
    *step=(*step)*5.0; 
    *n_step=floor(span/(*step));
  }
  if((*n_step)>10) { 
    *step=(*step)*2.0; 
    *n_step=floor(span/(*step));
  }

  (*ceil_range).min=floor(orig_range.min/(*step)) * (*step);
  (*ceil_range).max=ceil(orig_range.max/(*step)) * (*step);

  *n_step=floor(((*ceil_range).max-(*ceil_range).min)/(*step))+1;

  return;
}


void StarSortRange(range)
/*
sorting the value range

1992/12/21 T.Handa
*/
RANGE	*range;
{
  double dummy;

  if((*range).min>(*range).max) {
    dummy=(*range).min;
    (*range).min=(*range).max;
    (*range).max=dummy;
  }
  return;
}

void StarSortRangeLong(range)
/*
sorting the value range

1992/12/21 T.Handa
*/
RANGE_long	*range;
{
  long dummy;

  if((*range).min>(*range).max) {
    dummy=(*range).min;
    (*range).min=(*range).max;
    (*range).max=dummy;
  }

  return;
}

void StarSortBoxRange(range)
/*
sorting the value range

1992/12/21 T.Handa
*/
BOX_RANGE	*range;
{
  StarSortRange(&((*range).x));
  StarSortRange(&((*range).y));
  return;
}

void StarSortBoxRangeLong(range)
/*
sorting the value range

1992/12/21 T.Handa
*/
BOX_RANGE_long	*range;
{
  StarSortRangeLong(&((*range).x));
  StarSortRangeLong(&((*range).y));
  return;
}

double StarStdAngle2deg(std_ang)
char *std_ang; /* <in> StdAngule String */
/*
Convert C-strings to degree value

example)
-12d34m56s --> -45296.0/3600.0=-12.582222
12d34'56" --> 45296.0/3600.0=12.582222
34'56" --> 2096.0/3600.0=0.582222
12h30m  --> 675000.0/3600.0=187.5
30m15s  --> 27225.0/3600.0=7.5625
1.0rad --> 57.297

-0d30m  --> -0.5
-10d-30m  --> -10.5
10d-30m  --> -10.5

12.5d --> 12.5
12.5d10.2s --> 45010.2/3600.0=12.502833

0 for inproper input

1998/4/20 T.Handa
*/
{
  int ii,l_str,val_pt[3],next_pt;
  int sign,unit;
  double ang_val[3];
  double deg;
  char *token;

  unit=0;	/* degree */
  deg=0.0;

  l_str=strlen(std_ang);
  if(l_str<=0) return(0.0);

  if(l_str>3 && strcmp(&(std_ang[l_str-3]),"rad")==0) {
    deg=atof(std_ang);
    deg=deg/PI*180.0;
    return deg;
  } else {
    switch(std_ang[l_str-1]) {
    case '0': case '1': case '2': case '3': case '4':
    case '5': case '6': case '7': case '8': case '9':
    case '.':
      deg=atof(std_ang);
      return deg;
    }
  }


  if(std_ang[0]=='-') sign= -1;
  else sign=1;

  for(ii=0;ii<3;ii++) val_pt[ii]= -1;
  next_pt=0;

  for(ii=0;ii<l_str;ii++) {
    switch(std_ang[ii]) {
    case '0': case '1': case '2': case '3': case '4':
    case '5': case '6': case '7': case '8': case '9':
    case '.': case '-': case '+':
      break;

    case 'd':
      unit=1; /* degree */
      val_pt[0]=next_pt;
      next_pt=ii+1;
      break;

    case '\'': case 'M':
      unit=2; /* arcmin */
      val_pt[1]=next_pt;
      next_pt=ii+1;
      break;

    case '\"': case 'S':
      unit=3; /* arcsec */
      val_pt[2]=next_pt;
      break;

    case 'h':
      unit=11; /* hour */
      val_pt[0]=next_pt;
      next_pt=ii+1;
      break;

    case 'm':
      if(unit==0) unit=12; /* minute */
      else if(unit<10) unit=2; /* arcmin */
      else unit=12; /* minute */
      val_pt[1]=next_pt;
      next_pt=ii+1;
      break;

    case 's':
      if(unit==0) unit=13; /* second */
      else if(unit<10) unit=3; /* arcsec */
      else unit=13; /* second */
      val_pt[2]=next_pt;
      break;

    }
  }
  for(ii=0;ii<3;ii++) {
    if(val_pt[ii]<0) {
      ang_val[ii]=0.0;
    } else {
      token= &std_ang[val_pt[ii]];
      ang_val[ii]=atof(token);
      if(ang_val[ii]<0.0) {
	ang_val[ii]= -ang_val[ii];
	sign= -1;
      }
    }
  }
  deg=ang_val[0]+ang_val[1]/60.0+ang_val[2]/3600.0;

  if(unit>10) deg=deg*15.0;

  if(sign<0) deg= -deg;
  return deg;
}

void Stardeg2StdAngle(str,angle,mode)
/*
convert ASTROS-std angle format to its value in deg

return:angle in degrees
input format: such as "-20d35'20.3"

2007/11/20 T.Handa
*/
char *str; /* <out> ASTROS-std angle format */
double angle; /* <in> [deg] */
int mode; /* <in> =0 long in d'", =1 long in hms, =2 lat in d'" */
{
  char sign;
  int deg,min;
  double sec;

  if(angle<0.0) {
    sign='-';
    angle= -angle;
  } else if(angle==0.0) {
    sign=' ';
  } else {
    sign='+';
  }
  if(mode==1) angle=angle/15.0;

  deg=(int)angle;
  angle=(angle-(double)deg)*60.0;
  min=(int)angle;
  angle=(angle-(double)min)*60.0;
  sec=angle;

  if(mode==0) {
    sprintf(str,"%dd%02d\'%05.2f\"",deg,min,sec);
  } else if(mode==1) {
    sprintf(str,"%dh%02dm%05.2fs",deg,min,sec);
  } else if(mode==2) {
    sprintf(str,"%c%dd%02d\'%05.2f\"",sign,deg,min,sec);
  } else {
    strcpy(str,"");
  }
  return;
}

int StarInRange(xx,range)
double xx;
RANGE range;
/*
 xx in the range?
=0 : No, =1: Yes
*/
{
  if(range.min>range.max) {
    if(xx<=range.min && xx>=range.max) return(1);
  } else {
    if(xx>=range.min && xx<=range.max) return(1);
  }
  return 0;
}

int StarInLoopRange(xx,range)
double xx;
RANGE range;
/*
 xx in the range?
=0 : No, =1: Yes
*/
{
  if(range.min>range.max) {
    if(xx>=range.min || xx<=range.max) return(1);
  } else {
    if(xx>=range.min && xx<=range.max) return(1);
  }
  return 0;
}

int StarInRange_long(xx,range)
long xx;
RANGE_long range;
/*
 xx in the range?
=0 : No, =1: Yes
*/
{
  if(range.min>range.max) {
    if(xx<=range.min && xx>=range.max) return 1;
  } else {
    if(xx>=range.min && xx<=range.max) return 1;
  }
  return 0;
}

int StarInLoopRange_long(xx,range)
long xx;
RANGE_long range;
/*
 xx in the range?
=0 : No, =1: Yes
*/
{
  if(range.min>range.max) {
    if(xx>=range.min || xx<=range.max) return 1;
  } else {
    if(xx>=range.min && xx<=range.max) return 1;
  }
  return 0;
}

void StarReduceString(text)
char *text;
/*
reduce the length of text omitting ' ' and '\t' characters in the tail
*/
{
  int l_str,ii,i_end,flag;

  flag=0;

  l_str=strlen(text);
  i_end=l_str+1;

  for(ii=l_str;ii>=0;ii--) {
    if(text[ii]!=' ' && text[ii]!='\t' && text[ii]!=(char)NULL) {
      flag=1;
      i_end=ii+1;
      break;
    }
  }
  if(flag<1) return;

  if(i_end>0) text[i_end]=(char)NULL;
  return;
}

/*
general perpus library

update history of the file
1993/1/12 T.Handa
1992/12/21 T.Handa
*/

double ipow(p, n)
/*
floatѿpѾ롣

Programmed by H. Tanioka
Editted by Neko
*/
float	p;
int  	n;
{
  int		k;
  double 	s = 1;
  
  for(k = 1; k <= n; k++) {
    s = s * p;
  }
	
  return s;
}

void LeastSquareFitFloat (x, y, ndata, order, coefficient)
/*
Function LeastSquareFit
data are given as "ndata" pairs of (x,y).
give the coefficient
after fitting by polinomial with the given order.

Programmed by H. Tanioka
Editted by Neko
*/

float *x, *y; /* <in> data pairs */
int ndata;	/* <in> the number of data pairs */
int order;	/* <in> the order of the polinomial */
float *coefficient;	/* <out> fitted result */
{
  double a[10][12];	/* coefficient matrix */
  double s[21], t[11];	/* work array */
	
  int i, j, k;
  double p, d;

/* Initializing the work-matrix s[] and t[] */

  for (i = 0; i <= 2*order; ++i)
    s[i] = 0;

  for (i = 0; i <= order; ++i)
    t[i] = 0;

/* Initializing the coefficient matrix a[][] */

  for (i = 0; i < order; ++i) {
    for (j = 0; j < order + 2; ++j) {
      a[i][j] = 0;
    }
  }


/* Calculating the value of s[] and t[] */

  for (i = 0; i < ndata; ++i) {
    for (j = 0; j <= 2*order; ++j) {
      s[j] = s[j] + ipow (x[i], j);
    }
    for (j = 0; j <= order; ++j) {
      t[j] = t[j] + ipow (x[i], j) * y[i];
    }
  }

/* Put the value of s[] and t[] into a[][] */

  for (i = 0; i <= order; ++i) {
    for (j = 0; j <= order; ++j) {
      a[i][j] = s[i+j];
    }
    a[i][order+1] = t[i];
  }

/* Sweep-out the matrix */

  for (k = 0; k <= order; ++k) {
    p = a[k][k];

    for (j = k; j <= order+1; ++j)
      a[k][j] = a[k][j] / p;

    for (i = 0; i <= order; ++i) {				
      if (i != k) {
	d = a[i][k];
	
	for (j = k; j <= order+1; ++j)
	  a[i][j] = a[i][j] - d * a[k][j];
      }
    }
  }


/* Put the results into the coefficient vector */

  for (i = 0; i <= order; ++i) {
    coefficient[i] = a[i][order+1];
  }

  return;
}

void LeastSquareFit(xx,yy,ndata,order,coefficient)
/*
Function LeastSquareFit
data are given as "ndata" pairs of (xx,yy).
give the coefficient
after fitting by polinomial with the given order.

Programmed by H. Tanioka
Editted by Neko
*/

double *xx,*yy; /* <in> data pairs */
int ndata;	/* <in> the number of data pairs */
int order;	/* <in> the order of the polinomial */
double *coefficient;	/* <out> fitted result */
{
  double matrix[10][12];	/* coefficient matrix */
  double wv1[21],wv2[11];	/* work array */
  int ii,jj,kk;
  double pp,dd;

/* Initializing the work-matrix wv1[] and wv2[] */

  for(ii=0;ii<=2*order;++ii)
    wv1[ii]=0;

  for(ii=0;ii<=order;++ii)
    wv2[ii]=0;

/* Initializing the coefficient matrix matrix[][] */

  for(ii=0;ii<order;++ii) {
    for(jj=0;jj<order+2;++jj) {
      matrix[ii][jj]=0;
    }
  }


/* Calculating the value of wv1[] and wv2[] */
  for(ii=0;ii<ndata;++ii) {
    for(jj=0;jj<=2*order;++jj) {
      wv1[jj]=wv1[jj]+ipow (xx[ii],jj);
    }
    for(jj=0;jj<=order;++jj) {
      wv2[jj]=wv2[jj]+ipow (xx[ii],jj)*yy[ii];
    }
  }

/* Put the value of wv1[] and wv2[] into matrix[][] */
  for(ii=0;ii<=order;++ii) {
    for(jj=0;jj<=order;++jj) {
      matrix[ii][jj]=wv1[ii+jj];
    }
    matrix[ii][order+1]=wv2[ii];
  }

/* Sweep-out the matrix */
  for(kk=0;kk<=order;++kk) {
    pp=matrix[kk][kk];

    for(jj=kk;jj<=order+1;++jj)
      matrix[kk][jj]=matrix[kk][jj]/pp;

    for(ii=0;ii<=order;++ii) {				
      if (ii!=kk) {
	dd=matrix[ii][kk];
	
	for(jj=kk;jj<=order+1;++jj)
	  matrix[ii][jj]=matrix[ii][jj]-dd*matrix[kk][jj];
      }
    }
  }


/* Put the results into the coefficient vector */

  for(ii=0;ii<=order;++ii) {
    coefficient[ii]=matrix[ii][order+1];
  }

  return;
}

int StarIsInLongitudeRange(range,target)
RANGE *range;
double target;
/*
check target position in or out of the range

range and target in degree.
(*range).min>(*range).max means through 0deg
Positions in negative value are fully supported.
return(1); in the range,
return(0); out of the range.

1999/4/20 T.Handa
*/
{
  double fold_x;

/* target value is folded between 0.0 to 360.0 */
  fold_x=target-floor(target/360.0)*360.0;

  if((*range).min<=(*range).max) {
    if((*range).min>0.0) { /* normal ranging */
      if((*range).min<=fold_x && fold_x<=(*range).max) 
	return(1);
      else 
	return(0);
    } else if((*range).max>0.0) { /* negative min, positive max */
      if((*range).min+360.0<=fold_x || fold_x<=(*range).max) 
	return(1);
      else 
	return(0);
    } else { /* negative min&max */
      if((*range).min+360.0<=fold_x && fold_x<=(*range).max+360.0) 
	return(1);
      else 
	return(0);
    }
  } else { /* through 0deg=360deg */
    if((*range).max>0.0) { /* through the origin */
      if(fold_x<=(*range).max || fold_x>=(*range).min)
	return(1);
      else
	return(0);
    } else if((*range).min<0.0) {
      if(fold_x<=(*range).max+360.0 || fold_x>=(*range).min+360.0)
	return(1);
      else
	return(0);
    } else {
      if(fold_x<=(*range).max+360.0 && fold_x>=(*range).min)
	return(1);
      else
	return(0);
    }
  }
}

/*----date/time conversion----*/

int StarIsLeapYear(yy)
int yy;
/*
Judge the reap year based on Gregorious calender
=0 for normal, =1 for leap year.
yy must be in full AD year like 1959 or 2001

1999/4/13 T.Handa
*/
{
  int y4,y100,y400;

  y4=yy/4; y4=y4*4;
  y100=yy/100; y100=y100*100;
  y400=yy/400; y400=y400*400;
  
  if(yy!=y4) /* normal year; the last day of Feb is 28 */
    return(0);
  else if(yy==y100 && yy!=y400)
    return(0);

  return(1);
}

long StarYmd2MJD(yy,mm,dd)
int yy,mm,dd;
/*
convert yy,mm,dd (year,month,day) to MJD (modified Julian Day)
Year must be in full AD year; like 1959 or 2001.
Valid after 1900AD.

1999/4/13 T.Handa
*/
{
  long mjd,mjd0;
  int ii;

/* day from the first of the month */
  mjd=(long)dd;

/* day from the first of the year */
  switch(mm) {
  case 12:
    mjd=mjd+30L;
  case 11:
    mjd=mjd+31L;
  case 10:
    mjd=mjd+30L;
  case 9:
    mjd=mjd+31L;
  case 8:
    mjd=mjd+31L;
  case 7:
    mjd=mjd+30L;
  case 6:
    mjd=mjd+31L;
  case 5:
    mjd=mjd+30L;
  case 4:
    mjd=mjd+31L;
  case 3:
    mjd=mjd+28L;
  case 2:
    mjd=mjd+31L;
  case 1:
    break;
  }
  if(mm>2) mjd=mjd+StarIsLeapYear(yy);

/* day from 1900AD */
  mjd0=15020L;
  for(ii=1900;ii<yy;ii++) {
    mjd0=mjd0+365L+(long)StarIsLeapYear(ii);
  }

  mjd=mjd+mjd0;

  return(mjd);
}

void StarAdjustStrlen(word,l_word,pad)
/*
Adjust to the asigned length C-string at any cases.

1998/11/19 T.Handa
*/
char *word; /* must have enough space; word[l_word+1] */
int l_word; /* forced length */
char pad;   /* padding character */
{
  int l_str;
  int ii;

  l_str=strlen(word);

  if(l_str==l_word)
    return;

  else if(l_str<l_word) {
    for(ii=l_str;ii<l_word;ii++)
      word[ii]=pad;
  } 
  word[l_word]=(char)NULL;
  return;
}

long StarFitsDate2MJD(fitsdate)
char *fitsdate;
/*
convert FITS data format to obs_prm value

1999/4/13 T.Handa
*/
{
  char date_char[20];
  char *token;
  int l_str,yy,mm,dd;

  strncpy(date_char,fitsdate,19);
  date_char[19]=(char)NULL;

  token=strtok(date_char,"-/");
  if(token==NULL) {
    fprintf(stderr,"**ERR** invalid date format in FITS!\n");
    return(0L);
  }
  l_str=strlen(token);
  if(l_str==2) { /* old FITS style; DD/MM/YY */
    dd=atoi(token);
    token=strtok(NULL,"/");
    if(token==NULL) {
      fprintf(stderr,"**ERR** invalid date format in FITS!\n");
      return(0L);
    }
    mm=atoi(token);
    token=strtok(NULL,"/ ");
    if(token==NULL) {
      fprintf(stderr,"**ERR** invalid date format in FITS!\n");
      return(0L);
    }
    yy=atoi(token)+1900;
  } else { /* new FITS style; CCYY-MM-DDThh:mm:ss */
    yy=atoi(token);
    token=strtok(NULL,"-");
    if(token==NULL) {
      fprintf(stderr,"**ERR** invalid date format in FITS!\n");
      return(0L);
    }
    mm=atoi(token);
    token=strtok(NULL,"T ");
    if(token==NULL) {
      fprintf(stderr,"**ERR** invalid date format in FITS!\n");
      return(0L);
    }
    dd=atoi(token);
  }

  return(StarYmd2MJD(yy,mm,dd));
}


long StarFitsDate2Time1970(fitsdate)
char *fitsdate;
/*
convert FITS data format to obs_prm value

1999/4/13 T.Handa
*/
{
  long day,sec;
  char date_char[20];
  char *token;
  int hour,min;

  day=StarFitsDate2MJD(fitsdate)-StarYmd2MJD(1970,1,1);
  sec=day*86400L;

  strncpy(date_char,fitsdate,19);
  date_char[19]=(char)NULL;
  token=strtok(date_char,"T ");
  if(token==NULL) {
    fprintf(stderr,"**ERR** invalid date format in FITS!\n");
    return(0);
  }
  token=strtok(NULL,": ");
  if(token==NULL) return(sec);
  hour=atoi(token);
  sec=sec+hour*3600L;
  token=strtok(NULL,": ");
  if(token==NULL) return(sec);
  min=atoi(token);
  sec=sec+min*60L;
  token=strtok(NULL,".: ");
  if(token==NULL) return(sec);
  sec=sec+atoi(token);
  
  return(sec);
}

void StarMJD2Ymd(yy,mm,dd,mjd)
int *yy,*mm,*dd; /* <out> */
long mjd; /* <in> */
/*
convert MJD (modified Julian Day) to yy,mm,dd (year,month,day)
Year must be in full AD year; like 1959 or 2001.
Valid after 1900AD

1999/8/2 T.Handa
*/
{
  int ii;
  long mjd2000, /* MJD from 2000 */
    day_from400yr, /* day from Jan 1 of the latest 400 Gregorian year step */
    day_from100yr, /* day from Jan 1 of the latest 100 Gregorian year step */
    day_from4yr, /* day from Jan 1 of the latest reap year */
    day_fromJan1; /* day from Jan 1 of the year */
  int i_400yr, /* step in 400-Gregorian-year from nominal year AD.0 */
    i_100yr, /* step in century from 400 Gregorian year step */
    i_4yr, /* step in 4-year from the end of the last century */ 
    i_1yr; /* step in year from the latest 4-year step */
  long reap_year_flag; /* =0 ordinal year, =1 reap year */

  static long day_in_month[13]={0,31,59,90,120,151,181,212,243,273,304,334,365}; /* day number list in a non-reap year */

  static long day_in_year[5]={0,365,730,1095,1460}; /* day number for 4 non-reap years */

/* day difference */
  mjd2000=mjd-51545; /* MJD51545=2000/1/1 */

/* estimate year */
/* 400 Gregorian years in days */
  i_400yr=mjd2000/146097;
  day_from400yr=mjd2000-i_400yr*146097;
  if(day_from400yr<0) {
    i_400yr=i_400yr-1;
    day_from400yr=mjd2000-i_400yr*146097;
  }
  i_400yr=i_400yr+5;

/* 100 Gregorian years in days */
  if(day_from400yr<36525) { /* first century of the 400-year step */
    i_100yr=0;
    day_from100yr=day_from400yr;
  } else {
    i_100yr=(day_from400yr-36525)/36524+1;
    day_from100yr=day_from400yr-i_100yr*36524-1;
  }

/* 4 Gregorian years in days */
  if(i_100yr==0) { /* first century in every 400 Gregorian years */
    i_4yr=day_from100yr/1461;
    day_from4yr=day_from100yr-i_4yr*1461;
    reap_year_flag=1; /* reap year if i_1yr==0 */
  } else { /* second, third and 4th century */
    if(day_from100yr<=1460) {
      i_4yr=0;
      day_from4yr=day_from100yr;
      reap_year_flag=0; /* ordinal year even if i_1yr==0 */
    } else {
      i_4yr=(day_from100yr-1460)/1461+1;
      day_from4yr=day_from100yr-i_4yr*1461+1;
      reap_year_flag=1; /* reap year if i_1yr==0 */
    }
  }

/* get the year from the last_reap_year */
  i_1yr=0;
  day_fromJan1=0;
  for(ii=0;ii<4;ii++) {
    if(day_from4yr<day_in_year[ii+1]+reap_year_flag) {
      i_1yr=ii;
      if(ii==0 && reap_year_flag==1) {
	day_fromJan1=day_from4yr-day_in_year[ii];
      } else {
	day_fromJan1=day_from4yr-(day_in_year[ii]+reap_year_flag);
      }
      break;
    }
  }

  if(i_1yr!=0)
    reap_year_flag=0;

  *yy=i_400yr*400+i_100yr*100+i_4yr*4+i_1yr;

/* reap year */
  if(reap_year_flag==1) {
    if(day_fromJan1<31) {
      *mm=1;
      *dd=day_fromJan1+1;
      return;
    } else if(day_fromJan1<60) {
      *mm=2;
      *dd=day_fromJan1-30;
      return;
    }

    for(ii=3;ii<13;ii++) {
      if(day_fromJan1<=day_in_month[ii]) {
	*mm=ii;
	*dd=day_fromJan1-day_in_month[ii-1];
	return;
      }
    }

/* not reap year */
  } else {
    for(ii=1;ii<13;ii++) {
      if(day_fromJan1<day_in_month[ii]) {
	*mm=ii;
	*dd=day_fromJan1-day_in_month[ii-1]+1;
	return;
      }
    }
  }
}

/*--------c-string process---------*/

char *StarFileMainName(fullpassname)
char *fullpassname; /* <in> */
/*
extract main part of the filename if it is assigned in absorute pass name.
2000/1/16 T.Handa
*/
{
  int l_str,ii,i_top;

  i_top=0;
  l_str=strlen(fullpassname);
  for(ii=l_str;ii>=0;ii--) {
    if(fullpassname[ii]=='/') {
      i_top=ii+1;
      break;
    }
  }
  return(&(fullpassname[i_top]));
}

int StarExtractPartialFilename(fullname,partname,mode_flag)
char *fullname; /* <in> */
char *partname; /* <out> */
int mode_flag; /* <in> */
/*
extract main part for a filename, which is separated by '.' and '/'.

return: length of an output c-string.

mode_flag
FileNameProcess_OnlyFullName	0: filename without the directory part
FileNameProcess_OnlyMainName	1: filename without dir and ext
FileNameProcess_OnlyDir 	2: only the directory part
FileNameProcess_OnlyExt 	3: only the extention part 
FileNameProcess_WithoutExt	4: filename with dir but without ext

2001/7/2 T.Handa
2004/12/1 T.Handa (debug)
*/
{
  int l_str,i_start;
  int ii;
  int i_slash;

  l_str=strlen(fullname);

  if(l_str<=0) {
    strcpy(partname,"");
    return(0);
  }

/* without ext and with dir */ 
  if(mode_flag==FileNameProcess_WithoutExt) {
    strcpy(partname,fullname);
    for(ii=l_str;ii>=0;ii--) {
      if(partname[ii]=='.') {
	partname[ii]=(char)NULL;
	return(ii);
      }
    }
  }

/* find the lowest directory level */
  i_slash= -1;
  for(ii=l_str-1;ii>=0;ii--) {
    if(fullname[ii]=='/') {
      i_slash=ii;
      break;
    }
  }

  i_start=ii+1;

/* ext name */
  if(mode_flag==FileNameProcess_OnlyExt) {
    for(ii=l_str-1;ii>=i_start;ii--) {
      if(fullname[ii]=='.') {
	strcpy(partname,&(fullname[ii+1]));
	return strlen(partname);
      }
    }
    return(0);
  }

/* dir name */
  else if(mode_flag==FileNameProcess_OnlyDir) {
    if(i_slash<0) {
      strcpy(partname,"");
      return(0);
    } else {
      strncpy(partname,fullname,ii);
      partname[i_slash]=(char)NULL;
      return(i_start);
    }
  }

/* file name */
  strcpy(partname,&(fullname[i_start]));

  if(mode_flag==FileNameProcess_OnlyFullName) {
    return(l_str-i_start);
  }

/* clear extension */
  l_str=l_str-i_start;
  for(ii=l_str;ii>=0;ii--) {
    if(partname[ii]=='.') {
      partname[ii]=(char)NULL;
      return(ii);
    }
  }
  return(0);
}

void StarReduceBlankInCstrings(name)
char *name; /* <i/o> */
/*
reduce top/bottom blanks in c-strings

2002/6/23 T.Handa
 */
{
  int ii,l_str,i_top;
  char *buffer;

  l_str=strlen(name);
 
  for(ii=0;ii<l_str;ii++) {
    if(name[ii]=='\n' || name[ii]=='\r' || name[ii]=='\t')
      name[ii]=' ';
  }

  i_top=l_str;
  for(ii=0;ii<l_str;ii++) {
    if(name[ii]!=' ') {
      i_top=ii;
      break;
    }
  }

  if(i_top==l_str) return;

  for(ii=l_str-1;ii>=i_top;ii--) {
    if(name[ii]!=' ') {
      name[ii+1]=(char)NULL;
      break;
    }
  }

  buffer=(char *)malloc((l_str+1)*sizeof(name[0]));
  if(buffer==NULL) return;

  strcpy(buffer,&(name[i_top]));
  buffer[l_str]=(char)NULL;
 
  strcpy(name,buffer);

  free(buffer);

  return;
}

int is_extra_visible_char(letter)
char letter; /* <in> letter to check */
/*
check the character whether non-alphanumeric but visible ASCII or not
<return> =1 Yes, =0 No
2004/12/1 T.Handa
 */
{
  static char sample[32]="!?\"#$%&\'()~=|+-*/^_`{}@[];:<>,.";
  static int n_sample=31;
  int ii;

  for(ii=0;ii<n_sample;ii++) {
    if(letter==sample[ii]) {
      return(1);
    }
  }

  return(0);
}

int StarCheckCharType(letter)
char letter; /* <in> letter to check */
/*
check the character
<return>
1 aplphabet (a-zA-Z)
2 digit (0-9)
3 extraASCII but visible
4 space
0 others

2004/12/1 T.Handa
 */
{
  if(is_extra_visible_char(letter)==1)
    return(3);
  if(isalpha((int)letter)!=0)
    return(1);
  if(isdigit((int)letter)!=0)
    return(2);
  if(letter==' ')
    return(4);
  return(0);
}

void minstrcpy(outp,inp,n_str)
char *outp; /* <out> output c-strings */
char *inp; /* <in> input characters */
int n_str; /* <in> effective length of input */
/*
copy minimum part of the c-strings

2004/12/ 8 T.Handa
 */
{
  int ii;
  int l_str;
  int i_type;

  l_str=strlen(inp);
  if(l_str<n_str) n_str=l_str;
  strncpy(outp,inp,n_str);
  outp[n_str]=(char)NULL;

  /* find the first invisible char */
  for(ii=0;ii<n_str;ii++) {
    i_type=StarCheckCharType(outp[ii]);
    if(i_type==0 || i_type>4) {
      outp[ii]=(char)NULL;
      n_str=ii;
      if(ii==0) return;
      break;
    }
  }

  /* find the last visible char*/
  for(ii=n_str-1;ii>=0;ii--) {
    i_type=StarCheckCharType(outp[ii]);
    if(i_type>0 && i_type<4) {
      outp[ii+1]=(char)NULL;
      break;
    }
  }

  return;
}

/*--------time cotrol---------*/

long StarWaitMillisec(wait_msec)
long wait_msec; /* <in> */
/*
wait/sleep in msec
2000/2/11 T.Handa
*/
{
  struct timeval tv1,tv2;
  struct timezone tz;

  long delta_msec;

  gettimeofday(&tv1,&tz);

  while(1) {
    gettimeofday(&tv2,&tz);

    delta_msec=(tv2.tv_usec-tv1.tv_usec)/1000L;

    if(delta_msec<=0)
      delta_msec=delta_msec+(tv2.tv_sec-tv1.tv_sec)*1000;

    if(delta_msec>=wait_msec)
      return(delta_msec);
  }
}
