/*
UltraASTROS

date:2004/8/17 T.Handa
2004/12/17 T.Handa (add) refract_optical() in opt/IR regeme by WataruTanaka
2005/4/17 T.Handa (debug) argument order of aberration() in apparent()
2005/9/30 T.Handa (debug) cleanup for gcc -Wall
2005/11/4 T.Handa (add) calc_lst_hour()
2007/11/14 T.Handa (add) calc_lst2() : use the usec clock
2007/11/20 T.Handa (add) B1950toJ2000(),J2000toB1950(),B1950toJ2000_dircos(),J2000toB1950_dircos()
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include <ncurses.h>
#include <StData.h>

#ifdef ULTRAASTROS
#include <file.tel>
#include <UltraASTROS.h>

#else
#include <StStd.h>

#endif

void ab_param(te,s_lambda)
/*
derive the aberration parameters at the asigned epoch

1989. 3.23. by Takeshi Ito
1990/10/24 Toshihiro Handa : revise
1991/12/30 T.Handa
*/
double te; /* <in> elapse time from 1899/12/31 12:00(ET) [Julian century] */
double *s_lambda; /* <out> right ascension of the mean sun [rad] */
{
  double ll, /* mean celestial longitude of the sun  [arcsec] */
    gg, /* mean anomaly [deg] */
    /* g1, mean anomaly of Mercury [deg] */
    g2, /* mean anomaly of Vnus [deg] */
    g3, /* mean anomaly of Mars [deg] */
    g4,	/* mean anomaly of Jupiter [deg] */
    g5, /* mean anomaly of Saturn [deg] */
    g6, /* mean anomaly of the moon [deg] */
    dd, /* [deg]  */
    ud, /* [deg] */
    dl1, /* equation of center (difference between mean and true sun) in ecliptic longitude ( arcsec )  */
    dl2, /* periodic perturbation by plantes in ecliptic longitude [arcsec]  */
    dl2a,dl2b,dl2c,dl2d,
    dl3, /* perturbation by Moon in ecliptic longitude [arcsec] */
    dl4, /* secular perturbation in mean ecliptic longitude [arcsec]  */
    sl; /* ( arcsec )  */
  static double d2r=PI/180.0; /* deg --> rad */

  ll = 1006908.04 + (129602768.13 + 1.0890 * te)* te; /* [arcsec]  */

  gg = (1290513.00 +(129596579.10 + (- 0.5400 - 0.0120 * te) * te) * te
	+ (1.882 - 0.016 * te) * sin((57.24 + 150.27 * te) * d2r)
	+ 6.40 * sin((231.19 + 20.20 * te) * d2r)
	+ 0.266 * sin((31.8 + 119.00 * te) * d2r)) / 3600; /*  [deg]  */

  /*	g1 = 102.20 + 149472.350 * te;   */
  g2 = 212.45 +  58517.493 * te;
  g3 = 319.58 +  19139.977 * te;
  g4 = 225.28 +   3034.583 * te + (0.332 - 0.0015 * te) * sin((133.91 + 38.394 * te) * d2r);
  g5 = 175.60 +   1221.794 * te;
  g6 = (1065976.59 + (1717915856.79 + (33.0900 + 0.0518 * te) * te) * te) / 3600.0;

  dd = (1262654.95 + (1602961611.18 + (-5.1700 + 0.0068 * te) * te) * te) / 3600.0;
  ud = (  73848.25 + ( 136565679.36 + (-6.3910 + 0.0080 * te) * te) * te) / 3600.0;

  /* get a remainder of a 360 degree mode   */
  ll = ll - (int)(ll / 1296000.0) * 1296000.0;   /*  [arcsec]  */
  gg = gg - (int)(gg / 360.0) * 360.0;           /*  [deg]  */
  /*	g1 = g1 - (int)(g1 / 360.0) * 360.0;        [deg]  */
  g2 = g2 - (int)(g2 / 360.0) * 360.0;           /*  [deg]  */
  g3 = g3 - (int)(g3 / 360.0) * 360.0;           /*  [deg]  */
  g4 = g4 - (int)(g4 / 360.0) * 360.0;           /*  [deg]  */
  g5 = g5 - (int)(g5 / 360.0) * 360.0;           /*  [deg]  */
  g6 = g6 - (int)(g6 / 360.0) * 360.0;           /*  [deg]  */
  dd = dd - (int)(dd / 360.0) * 360.0;           /*  [deg]  */
  ud = ud - (int)(ud / 360.0) * 360.0;           /*  [deg]  */

  /* [deg] --> [rad]  */
  gg = gg * d2r;              /*  [rad]  */
  /*	g1 = g1 * d2r;                  [rad]  */
  g2 = g2 * d2r;              /*  [rad]  */
  g3 = g3 * d2r;              /*  [rad]  */
  g4 = g4 * d2r;              /*  [rad]  */
  g5 = g5 * d2r;              /*  [rad]  */
  g6 = g6 * d2r;              /*  [rad]  */
  dd = dd * d2r;              /*  [rad]  */
  ud = ud * d2r;              /*  [rad]  */

  dl1 = (6910.057 + (-17.242 - 0.0520 * te) * te) * sin(gg)
    + (72.339 - 0.361 * te) * sin(2.0 * gg)
    + (1.050 - 0.008 * te) * sin(3.0 * gg)
    + 0.017 * sin(4.0 * gg); /*  [arcsec]  */

  dl2a=  4.838 * cos(299.102 * d2r + 1.0 * g2 - 1.0 * gg)
    + 0.116 * cos(148.900 * d2r + 2.0 * g2 - 1.0 * gg)
    + 5.526 * cos(148.313 * d2r + 2.0 * g2 - 2.0 * gg)
    + 2.497 * cos(315.943 * d2r + 2.0 * g2 - 3.0 * gg)
    + 0.666 * cos(177.710 * d2r + 3.0 * g2 - 3.0 * gg)
    + 1.559 * cos(345.253 * d2r + 3.0 * g2 - 4.0 * gg)
    + 1.024 * cos(318.150 * d2r + 3.0 * g2 - 5.0 * gg)
    + 0.210 * cos(206.200 * d2r + 4.0 * g2 - 4.0 * gg)
    + 0.144 * cos(195.400 * d2r + 4.0 * g2 - 5.0 * gg);
  dl2b= 0.152 * cos(343.800 * d2r + 4.0 * g2 - 6.0 * gg)
    + 0.123 * cos(195.300 * d2r + 5.0 * g2 - 7.0 * gg)
    + 0.154 * cos(359.600 * d2r + 5.0 * g2 - 8.0 * gg)
    + 0.273 * cos(217.700 * d2r - 1.0 * g3 + 1.0 * gg)
    + 2.043 * cos(343.888 * d2r - 2.0 * g3 + 2.0 * gg)
    + 1.770 * cos(200.402 * d2r - 2.0 * g3 + 1.0 * gg)
    + 0.129 * cos(294.200 * d2r - 3.0 * g3 + 3.0 * gg)
    + 0.425 * cos(338.880 * d2r - 3.0 * g3 + 2.0 * gg)
    + 0.500 * cos(105.180 * d2r - 4.0 * g3 + 3.0 * gg);
  dl2c= 0.585 * cos(334.060 * d2r - 4.0 * g3 + 2.0 * gg)
    + 0.204 * cos(100.800 * d2r - 5.0 * g3 + 3.0 * gg)
    + 0.154 * cos(227.400 * d2r - 6.0 * g3 + 4.0 * gg)
    + 0.101 * cos( 96.300 * d2r - 6.0 * g3 + 3.0 * gg)
    + 0.106 * cos(222.700 * d2r - 7.0 * g3 + 4.0 * gg)
    + 0.163 * cos(198.600 * d2r - 1.0 * g4 + 2.0 * gg)
    + 7.208 * cos(179.532 * d2r - 1.0 * g4 + 1.0 * gg)
    + 2.600 * cos(263.217 * d2r - 1.0 * g4)
    + 2.731 * cos( 87.145 * d2r - 2.0 * g4 + 2.0 * gg);
  dl2d= 1.610 * cos(109.493 * d2r - 2.0 * g4 + 1.0 * gg)
    + 0.164 * cos(170.500 * d2r - 3.0 * g4 + 3.0 * gg)
    + 0.556 * cos( 82.650 * d2r - 3.0 * g4 + 2.0 * gg)
    + 0.210 * cos( 98.500 * d2r - 3.0 * g4 + 1.0 * gg)
    + 0.419 * cos(100.580 * d2r - 1.0 * g5 + 1.0 * gg)
    + 0.320 * cos(269.460 * d2r - 1.0 * g5)
    + 0.108 * cos(290.600 * d2r - 2.0 * g5 + 2.0 * gg)
    + 0.112 * cos(293.600 * d2r - 2.0 * g5 + 1.0 * gg);

  dl2=dl2a+dl2b+dl2c+dl2d; /*  [arcsec]  */

  dl3 =  6.454 * sin(dd) + 0.013 * sin(3.0 * dd)
    + 0.177 * sin(dd + g6)
    - 0.424 * sin(dd - g6)
    + 0.039 * sin(3.0 * dd - g6)
    - 0.064 * sin(dd + gg)
    + 0.172 * sin(dd - gg)
    - 0.013 * sin(dd - g6 - gg)
    - 0.013 * sin(2.0 * ud); /*  [arcsec]  */

  dl4 =  0.266 * sin(( 31.80 + 119.00 * te) * d2r)
    + 6.400 * sin((231.19 +  20.20 * te) * d2r)
    + (1.882 - 0.016 * te) * sin((57.24 + 150.27 * te) * d2r)
    + 0.202 * sin((315.60 + 893.30 * te) * d2r); /*  [arcsec]  */

  sl = ll + dl1 + dl2 + dl3 + dl4; /*  [arcsec]  */

  /*  [arcsec] --> [deg]  */
  sl = sl / 3600.0; /*  [deg]   */

  /*  [deg] --> [rad]  */
  *s_lambda = sl * d2r;

  return;
}

#define KAPPA 9.936508e-5 /* the constant of aberration   20.49552 arcsec = 9.936508e-5  */

void aberration(true_cos, epsilon, s_lambda, app_cos)
/*
do aberration correction

1989/3/20  by Takeshi Ito
1990/10/24  by Toshihiro Handa
1991/12/30 T.Handa
*/
DIRECT_COS true_cos; /* <in>  true position of the equinox of the day [direct cosine] */
double epsilon, /* <in> mean inclination of the equliptic plane [rad]  */
			s_lambda; 	/* <in> right ascension of the mean sun [rad]  */
DIRECT_COS *app_cos; /* <out> apparent position of the equinox of the day [direct cosine] */
{
  double sinl, cosl, sine, cose;

  sinl = sin(s_lambda);
  cosl = cos(s_lambda);
  sine = sin(epsilon);
  cose = cos(epsilon);

  (*app_cos).l = true_cos.l
    + KAPPA * ((1.0 - true_cos.l * true_cos.l)* sinl
	       + true_cos.l * true_cos.m * cosl * cose
	       + true_cos.l * true_cos.n * cosl * sine);
  (*app_cos).m = true_cos.m
    + KAPPA * (     - true_cos.l * true_cos.m * sinl
		    - (1.0 - true_cos.m * true_cos.m)* cosl * cose
		    + true_cos.m * true_cos.n * cosl * sine);
  (*app_cos).n = true_cos.n
    + KAPPA * (     - true_cos.n * true_cos.l * sinl
		    + true_cos.m * true_cos.n * cosl * cose
		    - (1.0 - true_cos.n * true_cos.n)* cosl * sine);
  return;
}

double angle_distance(xy1,xy2)
/*
rough estimatation of the angular distance between two point on the sphere

1992/1/4 T.Handa
1992/4/15 T.Handa debug yy (deg-->rad)
 */
XY_POS xy1,xy2;
{
  double dx,dy,yy;
  dx=xy1.x-xy2.x;
  dy=xy1.y-xy2.y;

  yy=(xy1.y+xy2.y)/2.0;
  yy=yy*PI/180.0;

  /* correction by latitude */
  dx=dx*cos(yy);

  return(sqrt(dx*dx+dy*dy));
}

void apparent(et,radec50_cos,radec_cos,nut_prm)
/*
estimate an apparent RADec in equinox of the day
from a mean RADec in B1950.0
       
1989/4/14  by Takeshi Ito
1990/10/24 Toshihiro Handa
1991/12/30 T.Handa (revise) revise arguents of the function
*/
EP_TIME et; /* <in> ephemeris time for culculation */
DIRECT_COS radec50_cos, /* <in> mean RADec in B1950.0 [direct cosin] */
  *radec_cos; /* <out> apparent RADec in equinox of the day [direct cosine] */
NUTATION_PRM *nut_prm; /* <out> nutation parameters */
{
  DIRECT_COS mean_cos, /* mean RADec in equinox of the day [direct cosine] */
    true_cos; /* true position in equinox of the day [direct cosine] */
  double s_lambda; /* right ascension of mean sun [rad] */

  /* estimate the nutation parameters */
  nut_param(et, nut_prm);

  /* estimate the aberration parameters */
  ab_param(et.t, &s_lambda);

  /* correct the precession */
  precession(radec50_cos, et.t50, &mean_cos);

  /* correct the nutation */
  nutation(mean_cos, *nut_prm, &true_cos);

  /* correct the aberration */
  aberration(true_cos, (*nut_prm).epsilon, s_lambda, radec_cos);

  return;
}

void radec2azel(lst, radec_cos, obs_site, azel)
/*
estimate the AzEl without atmospheric refraction at an asigned site
from an apparent RADec in equinox of the day

1989/3/20   by Takeshi Ito
1989/10/24  Toshihiro Handa
1990/11/25  Toshihiro Handa
1991/12/30 T.Handa
2004/12/3 T.Handa (modify) remove refract()
*/
double lst; /* <in> local sidereal time [rad] */
DIRECT_COS radec_cos; /* <in> apparent RADec in equinox of the day [direct cosine] */
SITE_POS obs_site; /* <in>  location of an observing site */
XY_POS *azel; /* <out> AzEl of the object [deg] (az=0 is for North. positive in eastbound. el=90deg at zenith) */
{
  double cosp, sinp, cosl, sinl;
  DIRECT_COS 	azel_cos;

  /* convert to the horizontal coordinate */
  cosp = cos(obs_site.y * PI / 180.0);
  sinp = sin(obs_site.y * PI / 180.0);
  cosl = cos(lst);
  sinl = sin(lst);

  azel_cos.l = sinp * cosl * radec_cos.l + sinp * sinl * radec_cos.m - cosp * radec_cos.n;
  azel_cos.m =      - sinl * radec_cos.l        + cosl * radec_cos.m;
  azel_cos.n = cosp * cosl * radec_cos.l + cosp * sinl * radec_cos.m + sinp * radec_cos.n;

  /* convert in degrees */
  direction(azel_cos,azel);
  /* convert to Az=0 for North; south=0,eastbound positive-->north=0,eastbound positive */
  if((*azel).x<180.0)
    (*azel).x=180.0-(*azel).x;
  else
    (*azel).x=540.0-(*azel).x;

  return;
}

double besselian(dos_ut,te)
/*
estimate Besselian century (100 Bessel years) 
from UNIX/DOS-UT time (elapse seconds from 1970/1/1 0h UT)

1989/3/12  by Takeshi Ito
1990/10/24 Toshihiro Handa
1991/12/30 T.Handa
*/
long dos_ut; /* <in> UNIX/DOS-UT time [sec] */
double te; /* <in> elapse Julan century from 1899/12/31 12h ET */
{
  double m_s_alpha_e, /* right ascension of ephemeris mean sun [sec] */
    sy, /* decimal faction fo Bessel year */
    yy; /* current year in AD */
  double t50; /* elapse time from B1950.0 in 100 Bessel year (Besselian century) */

  m_s_alpha_e=mean_sun(te);
  if(m_s_alpha_e < 67200.0)
    m_s_alpha_e = m_s_alpha_e + 86400.0;
  
  sy = (m_s_alpha_e - 67200.0) / 86400.0;

  yy = (double)(dos_ut - 599184000) / 31536000.0 +1989.0;
  /* 599184000 sec = 19 Besselian year
     31536000 sec = 356 days = 1 Besselian year */

  yy=floor(yy);
  t50 = (yy+sy - 1950.0) / 100.0;
  return(t50);
}

double calc_lst2(dos_ut2, obs_site, nut_prm)
/*
calculation of local sidereal time in radian
after correction of precession and nutation

1989/3/10 by Takeshi Ito
1990/10/24  Toshihiro Handa
1990/11/25  Toshihiro Handa
1991/12/30 T.Handa
2007/8/30 Hirofumi.Okuno
*/
double dos_ut2; /* <in> UNIX/DOS-UT time (elapse seconds from 1970/1/1 0h UT) [sec] */
SITE_POS obs_site; /* <in> location of an obsering site */
NUTATION_PRM nut_prm; /* <in> nutation parameters */
{
  double tu; /* elapse Julian century from 1899/12/31 12h (without leap sec) */
  double lst; /* local sidereal time [rad] */

  double ut, /* UTC in a day [sec] */
    eq, /* differenc of equinoxes [rad] */
    gmst; /* Greenwich mean sidereal time [sec] */

  /* sec until 1970/1/1 0h-->Julian century until 1899/12/31 12h */
  tu=(25567.5+(dos_ut2)/86400.0)/36525.0;

  /* estimate Greenwich mean sidereal time */
  ut = fmod( dos_ut2 , 86400.0 );
  gmst = ut + 43200.0 + mean_sun(tu);
  gmst = fmod(gmst,86400.0);
  gmst = gmst * PI / 43200.0; /* [sec] --> [rad] */

  /* estimate difference of equinoxes */
  eq = nut_prm.d_psi * cos(nut_prm.epsilon + nut_prm.d_epsilon);

  /* estimate local sidereal time */
  lst = gmst + eq + (obs_site.x * PI / 180.0) ;
  lst=fmod(lst,2.0*PI);

  return(lst);
}

double calc_lst(dos_ut, obs_site, nut_prm)
/*
calculation of local sidereal time in radian
after correction of precession and nutation

1989/3/10 by Takeshi Ito
1990/10/24  Toshihiro Handa
1990/11/25  Toshihiro Handa
1991/12/30 T.Handa
2006/2/9 T.Handa (revise) use get_leapsec()
*/
long dos_ut; /* <in> UNIX/DOS-UT time (elapse seconds from 1970/1/1 0h UT) [sec] */
SITE_POS obs_site; /* <in> location of an obsering site */
NUTATION_PRM nut_prm; /* <in> nutation parameters */
{
  double tu; /* elapse Julian century from 1899/12/31 12h (without leap sec) */
  double lst; /* local sidereal time [rad] */

  double ut, /* UTC in a day [sec] */
    eq, /* differenc of equinoxes [rad] */
    gmst; /* Greenwich mean sidereal time [sec] */

  /* sec until 1970/1/1 0h-->Julian century until 1899/12/31 12h */
  tu=(25567.5+((double)dos_ut)/86400.0)/36525.0;

  /* estimate Greenwich mean sidereal time */
  ut = fmod( (double)dos_ut , 86400.0 );
  gmst = ut + 43200.0 + mean_sun(tu);
  gmst = fmod(gmst,86400.0);
  gmst = gmst * PI / 43200.0; /* [sec] --> [rad] */

  /* estimate difference of equinoxes */
  eq = nut_prm.d_psi * cos(nut_prm.epsilon + nut_prm.d_epsilon);

  /* estimate local sidereal time */
  lst = gmst + eq + (obs_site.x * PI / 180.0) ;
  lst=fmod(lst,2.0*PI);

  return(lst);
}

double calc_lst_hour(utc70,obs_site)
time_t utc70; /* <in> */
SITE_POS obs_site; /* <in> */
/*
<return>
-1.0 : error on LEAP_SEC_FILE

2005/11/4 T.Handa
 */
{
  EP_TIME et;
  NUTATION_PRM nut_prm;
  double lst;
  int err_code;

  err_code=ephem_time(utc70,&et);
  if(err_code!=0) return(-1.0);

  nut_param(et,&nut_prm);

  lst=calc_lst(utc70,obs_site,nut_prm);
  lst=lst/PI*12.0; // rad-->hour
  while(lst<0.0) lst=lst+24.0;
  while(lst>=24.0) lst=lst-24.0;

  return(lst);
}

void dir_cos(xy,lmn)
/*
derive a direct cosine from a position on a spherical coordinate
1991/12/24 T.Handa
*/
XY_POS xy; /* <in> position on a spherical coordinate [deg] */
DIRECT_COS *lmn; /* <out> direct cosine */
{
  XY_POS rad;

  rad.x=xy.x*PI/180.0;
  rad.y=xy.y*PI/180.0;

  (*lmn).l=cos(rad.y) * cos(rad.x);
  (*lmn).m=cos(rad.y) * sin(rad.x);
  (*lmn).n=sin(rad.y);

  return;
}

void direction(lmn,xy)
/*
derive a position on a spherical coordinatea from  direct cosine

1989/3/12  by Takeshi Ito
1990/10/24 Toshihiro Handa
1991/12/30 T.Handa
*/
DIRECT_COS lmn; /* <in> direct cosine */
XY_POS *xy; /* <out> position on a spherical coordinate [deg] */
{
  static double r2d=180.0/PI;

  if(lmn.l == 0.0  &&  lmn.m == 0.0) /* at the pole */ {
    (*xy).x = 0.0;
    if(lmn.n < 0.0)
      (*xy).y = -90.0;
    else
      (*xy).y = 90.0;
    return;
  } else {
    (*xy).x = atan2(lmn.m, lmn.l);
    if((*xy).x < 0.0)
      (*xy).x = (*xy).x + 2.0 * PI; }
  if(lmn.n > 1.0)
    (*xy).y =  PI / 2.0;
  else if(lmn.n < -1.0)
    (*xy).y = -PI / 2.0;
  else
    (*xy).y = asin(lmn.n);
  
  (*xy).x = (*xy).x * r2d; /* [rad]-->[deg]  */
  (*xy).y = (*xy).y * r2d; /* [rad]-->[deg]  */

  return;
}


#define SIZE_DM 80

time_t get_leapsec(dos_ut)
time_t dos_ut; /* <in> time from 1970Jan1 0h UTC */
/*
extract gap between ephemeris time and UTC
using a leap_sec file, which is in

expire_sec(in dos_ut unit) time_gap

i.e.
    64 # 1999/1/1 0h UTC

use updateleapsec command.

1999/5/9 T.Handa
*/
{
  FILE *fp;
  char *token;
  char buffer[SIZE_DM],*file_stat;
  time_t expire_time;
  static time_t leap_sec,prev_leap_sec;
  int ii;

  fp=fopen(LEAP_SEC_FILE,"r");
  if(fp==NULL) return(0);

  for(ii=0;;ii++) {
    file_stat=fgets(buffer,SIZE_DM,fp);
    if(file_stat==NULL) break;

    if(buffer[0]!='#' && strncmp(buffer,"/*",2)!=0 && strncmp(buffer,"//",2)!=0) {
      token=strtok(buffer," \t\r#");
      if(token==NULL) {
	leap_sec=0;
	break;
      }
      prev_leap_sec=atol(token);

      token=strtok(NULL," \t\r#");
      if(token==NULL) {
	leap_sec=0;
	break;
      }
      expire_time=atol(token);

      if(expire_time!=0 && dos_ut>=expire_time) {
	break;
      }
      leap_sec=prev_leap_sec;
    }
  }

  fclose(fp);
  return(leap_sec);
}

int ephem_time(dos_ut, et)
/*
estimate time in EP_TIME (elapse seconds until 1899/12/31 12:00 ET 
with leap sec and elapse time until B1950.0)
from UNIX/DOS-UT time (elapse seconds until 1970/1/1 0:00 UT without leap sec)

1989/3/10  by Takeshi Ito
1990/8/28  Toshihiro Handa
1991/12/30 T.Handa
2006/2/9 T.Handa (revise) use get_leapsec()
*/
long dos_ut; /* <in> UNIX/DOS UT time [sec] */
EP_TIME *et; /* <out> elapse ephemeris time (ET) */
{
  static long leap_sec; /* leap sec (difference between ET and UNIX/DOS UT) [sec] */
  long et_sec; /* elapse ephemeris time until 1970/1/1 0:00(UT) [sec] */

  /* get an integrated leap sec */
  leap_sec=get_leapsec(dos_ut);
  if(leap_sec==0) {
    return(8212);
  }
  
  /* correct the leap sec */
  et_sec=dos_ut+leap_sec;

  /* elapse sec until 1970/1/1 0h UT-->elapse day and Julian century until 1899/12/31 12h */
  (*et).d=25567.5+(double)et_sec/86400.0;
  (*et).t=(*et).d/36525.0;

  /* Besserial century until B1950.0 */ 
  (*et).t50=besselian(dos_ut,(*et).t);

  return(0);
}

int lb2radec50(radec50,lb)
/*
convert position from Galactic coordinate to RADec(B1950.0)

1991/12/31 T.Handa
2007/12/05 (revise) T.Handa
*/
XY_POS *radec50; /* <out> mean RADec in B1950.0 [deg] */
XY_POS lb; /* <in> in galactic coordinate [deg] */
{
  XY_POS radec_off,lb_off;
  DIRECT_COS radec_cos,lb_cos;

  static double inclination=1.092576112, /* 62.6 deg : inclination of the Galactic plane against the equatorial plane [rad] */
    ra_node=282.25, /* 18h49m : right ascension of the ascending node [deg] */
    l_node=33.0; /* 33deg  : galactic longitude of the ascending node [deg] */

  /* lb --> radec50 */
  /* lb in the galactic coordinate --> direct cosine in offset-galactic */
  lb_off.x=lb.x - l_node;
  if(lb_off.x < -180.0)
    lb_off.x = lb_off.x + 360.0;
  else if(lb_off.x > 180.0)
    lb_off.x = lb_off.x - 360.0;
  lb_off.y=lb.y;
  dir_cos(lb_off,&lb_cos);
  /* rotate by the inclination angle */
  radec_cos.l = lb_cos.l;
  radec_cos.m = cos(inclination) * lb_cos.m - sin(inclination) * lb_cos.n;
  radec_cos.n = sin(inclination) * lb_cos.m + cos(inclination) * lb_cos.n;
  /* direct cosine in offset-equitoria --> RADec in equatorial coordinate */
  direction(radec_cos,&radec_off);
  (*radec50).x=radec_off.x + ra_node;
  if((*radec50).x < 0.0)
    (*radec50).x = (*radec50).x + 360.0;
  else if((*radec50).x > 360.0)
    (*radec50).x = (*radec50).x - 360.0;
  (*radec50).y=radec_off.y;
  if((*radec50).y == 90.0 || (*radec50).y == -90.0)
    (*radec50).x=0.0;

  return(0);
}

int radec502lb(lb,radec50)
/*
convert position from RADec(B1950.0) to galactic coordinate

1991/12/31 T.Handa
2007/12/05 (revise) T.Handa
*/
XY_POS *lb; /* <out> in galactic coordinate [deg] */
XY_POS radec50; /* <in> mean RADec in B1950.0 [deg] */
{
  XY_POS 	radec_off,lb_off;
  DIRECT_COS 	radec_cos,lb_cos;

  static double inclination=1.092576112, /* 62.6 deg : inclination of the Galactic plane against the equatorial plane [rad] */
    ra_node=282.25, /* 18h49m : right ascension of the ascending node [deg] */
    l_node=33.0; /* 33deg  : galactic longitude of the ascending node [deg] */

  /* radec50 --> lb */
  /* RADec in equatorial coordinate-->direct cosine in offset-equitoria */
  radec_off.x=radec50.x - ra_node;
  if(radec_off.x < -180.0)
    radec_off.x = radec_off.x + 360.0;
  else if(radec_off.x > 180.0)
    radec_off.x = radec_off.x - 360.0;
  radec_off.y=radec50.y;
  dir_cos(radec_off,&radec_cos);
  /* rotate by the inclination angle */
  lb_cos.l = radec_cos.l;
  lb_cos.m =  cos(inclination) * radec_cos.m + sin(inclination) * radec_cos.n;
  lb_cos.n = -sin(inclination) * radec_cos.m + cos(inclination) * radec_cos.n;
  /* direct cosine in offset-galactic--> lb in the galactic coordinate */
  direction(lb_cos,&lb_off);
  (*lb).x=lb_off.x + l_node;
  if((*lb).x < 0.0)
    (*lb).x = (*lb).x + 360.0;
  else if((*lb).x > 360.0)
    (*lb).x = (*lb).x - 360.0;
  (*lb).y=lb_off.y;

    return(0);
}

int B1950toJ2000_dircos(radec2k,radec50)
DIRECT_COS *radec2k; /* <out> RA-Dec J2000 */
DIRECT_COS radec50; /* <in> RA-Dec B19500 */
/*
conversion using Aoki S., Soma M, Konoshita H, Inoue K., (1983) AA 128, 263
referred to
"Tentai-no-ichikeisan Zouho-ban (Astrometrical calculation)"
K.Nagasawa (Chijin-shokan) 1988, p.238
ISBN4-8052-0225-4 C3044
 */
{
  static double matrix[9]={0.9999256782, -0.0111820610, -0.0048579477,
                           0.0111820609,  0.9999374784, -0.0000271765,
                           0.0048579479, -0.0000271474,  0.9999881997};
  DIRECT_COS aa;
  double scale;

  aa.l=matrix[0]*radec50.l+matrix[1]*radec50.m+matrix[2]*radec50.n;
  aa.m=matrix[3]*radec50.l+matrix[4]*radec50.m+matrix[5]*radec50.n;
  aa.n=matrix[6]*radec50.l+matrix[7]*radec50.m+matrix[8]*radec50.n;

  scale=1.0/sqrt(aa.l*aa.l+aa.m*aa.m+aa.n*aa.n);
 
  /* re-normaliztion */

  (*radec2k).l=aa.l*scale;
  (*radec2k).m=aa.m*scale;
  (*radec2k).n=aa.n*scale;

  return(0);
}

int B1950toJ2000(radec2k,radec50)
XY_POS *radec2k; /* <out> RA-Dec J2000 */
XY_POS radec50; /* <in> RA-Dec B19500 */
/*
conversion using Aoki S., et al. (1983) AA 128, 263
referred to
"Tentai-no-ichikeisan Zouho-ban (Astrometrical calculation)"
K.Nagasawa (Chijin-shokan) 1988, p.238
ISBN4-8052-0225-4 C3044
 */
{
  DIRECT_COS lmn2k,lmn50;

  dir_cos(radec50,&lmn50);
  B1950toJ2000_dircos(&lmn2k,lmn50);
  direction(lmn2k,radec2k);

  return(0);

}

int J2000toB1950_dircos(radec50,radec2k)
DIRECT_COS *radec50; /* <out> RA-Dec B19500 */
DIRECT_COS radec2k; /* <in> RA-Dec J2000 */
/*
conversion using Aoki S., Soma M, Konoshita H, Inoue K., (1983) AA 128, 263
referred to
"Tentai-no-ichikeisan Zouho-ban (Astrometrical calculation)"
K.Nagasawa (Chijin-shokan) 1988, p.238
ISBN4-8052-0225-4 C3044
 */
{
  static double matrix[9]={0.9999256782, -0.0111820610, -0.0048579477,
                           0.0111820609,  0.9999374784, -0.0000271765,
                           0.0048579479, -0.0000271474,  0.9999881997};
  /* This matrix is orthogonal.  The inverse matrix is the translated one. */
  DIRECT_COS aa;
  double scale;

  aa.l=matrix[0]*radec2k.l+matrix[3]*radec2k.m+matrix[6]*radec2k.n;
  aa.m=matrix[1]*radec2k.l+matrix[4]*radec2k.m+matrix[7]*radec2k.n;
  aa.n=matrix[2]*radec2k.l+matrix[5]*radec2k.m+matrix[8]*radec2k.n;

  scale=1.0/sqrt(aa.l*aa.l+aa.m*aa.m+aa.n*aa.n);
 
  /* re-normaliztion */

  (*radec50).l=aa.l*scale;
  (*radec50).m=aa.m*scale;
  (*radec50).n=aa.n*scale;

  return(0);
}

int J2000toB1950(radec50,radec2k)
XY_POS *radec50; /* <out> RA-Dec B19500 */
XY_POS radec2k; /* <in> RA-Dec J2000 */
/*
conversion using Aoki S., et al. (1983) AA 128, 263
referred to
"Tentai-no-ichikeisan Zouho-ban (Astrometrical calculation)"
K.Nagasawa (Chijin-shokan) 1988, p.238
ISBN4-8052-0225-4 C3044
 */
{
  DIRECT_COS lmn2k,lmn50;

  dir_cos(radec2k,&lmn2k);
  J2000toB1950_dircos(&lmn50,lmn2k);
  direction(lmn50,radec50);

  return(0);

}

double mean_sun(t)
/*
estimate a right ascension of the mean sun or the ephemeric mean sun
from elapse time until 1899/12/31 UT or ET

formulation: m_s_alpha = 18h38m45.836s + 8640184.542s * t + 0.0929s * t * t
		= 67125.836s + 8640184.542s * t + 0.0929s * t * t  

1989/3/10 by Takeshi Ito
1990/10/24 by Toshihiro Handa
1991/12/30 T.Handa
*/
double t; /* elapse time until 1899/12/31 12h [Julian century] */
{
  double m_s_alpha; /* right ascension of the mean sun [sec] */
  double a; /* the second term of the formula */		

  a = 8640184.542 * t;	
  a = fmod(a,86400.0); /* remainder of 24 hours */
  m_s_alpha = 67125.836 + a + 0.0929 * t * t;
  m_s_alpha=fmod(m_s_alpha,86400.0);
  return(m_s_alpha);
}

long m_julian_day(dos_ut)
/*
estimate MJD (modified julian day) 
from UNIX/DOS UT (elapse sec until 1970/1/1 0:00UT without leap sec)
The fractional part is neglected.

1992/2/19 T.Handa
*/
long dos_ut; /* <in> UNIX/DOS UT [sec] */
{
  long mjd;

  mjd=dos_ut/3600L/24L+40587L;
  return(mjd);
}

void nut_param(et, nut_prm)
/*
estimate the nutation parameters at the time

1989/3/25    by Takeshi Ito
1990/10/24 Toshihiro Handa
1991/12/30 T.Handa
*/
EP_TIME et; /* <in> elapse ephemeris time until 1899/12/31 12:00(ET) */
NUTATION_PRM *nut_prm; /* <out> the nutation paramters */
{
  double ll, /* mean anomaly of the sun [rad] */
    ff, /* ecliptic longitude of Moon from the ascending node [rad] */
    dd, /* mean anglar distance between Sun and Moon [rad] */
    omega; /* ecliptic longitude of mean luna ascending node [rad] */
  static double d2r=PI/180.0; /* deg --> rad */

  ll    = 358.475833 + ( 0.9856002669 + (-0.112e-12 - 6.8e-20 * et.d) * et.d) * et.d;
  ff    =  11.250889 + (13.2293504490 + (-2.407e-12 - 0.7e-20 * et.d) * et.d) * et.d;
  dd    = 350.737486 + (12.1907491914 + (-1.076e-12 + 3.9e-20 * et.d) * et.d) * et.d;
  omega = 259.183275 + (-0.0529539222 + ( 1.557e-12 + 4.6e-20 * et.d) * et.d) * et.d;

  ll=ll*d2r;
  ff=ff*d2r;
  dd=dd*d2r;
  omega=omega*d2r;

  (*nut_prm).d_psi =  (-17.2327 - 0.01737 * et.t) * sin(omega)
    +(-1.2729 - 0.00013 * et.t) * sin(2.0 * (ff - dd + omega))
    +( 0.2088 + 0.00002 * et.t) * sin(2.0 * omega)
    +( 0.1261 - 0.00031 * et.t) * sin(ll)
    +(-0.2037 - 0.00002 * et.t) * sin(2.0 * (ff + omega)); /* [arcsec] */

  (*nut_prm).d_epsilon =  ( 9.2100 + 0.00091 * et.t) * cos(omega)
    +( 0.5522 - 0.00029 * et.t) * cos(2.0 * (ff - dd + omega))
    +(-0.0904 + 0.00004 * et.t) * cos(2.0 * omega)
    +( 0.0884 - 0.00005 * et.t) * cos(2.0 * (ff + omega)); /* [arcsec] */

  (*nut_prm).epsilon = 23.452294
    +(-0.0130125 + (-0.00000164 + 0.000000503 * et.t) * et.t) * et.t; /* [deg] */

  (*nut_prm).d_psi = (*nut_prm).d_psi /3600.0 * d2r; 			/* [arcsec] --> [rad] */
  (*nut_prm).d_epsilon = (*nut_prm).d_epsilon /3600.0 * d2r; 	/* [arcsec] --> [rad] */
  (*nut_prm).epsilon = (*nut_prm).epsilon * d2r; 				/* [deg] --> [rad] */

  return;
}

void nutation(mean_cos, nut_prm, true_cos)
/*
correct nutation

1989/3/18 by Takeshi Ito
1990/10/24 Toshihiro Handa
1991/12/30 T.Handa
*/
DIRECT_COS mean_cos; /* <in> mean position in equinox of the day [direct cosine] */
NUTATION_PRM nut_prm; /* <in>  the nutation parameters [rad] */
DIRECT_COS *true_cos; /* <out> true position in equinox of the day [direct cosine] */
{
  double ce, se;
	
  ce = cos(nut_prm.epsilon);
  se = sin(nut_prm.epsilon);

  (*true_cos).l = mean_cos.l
    - nut_prm.d_psi * ce * mean_cos.m
    - nut_prm.d_psi * se * mean_cos.n;
  
  (*true_cos).m = nut_prm.d_psi * ce * mean_cos.l
    +                      mean_cos.m
    - nut_prm.d_epsilon  * mean_cos.n;

  (*true_cos).n =   nut_prm.d_psi * se * mean_cos.l
    + nut_prm.d_epsilon  * mean_cos.m
    +                      mean_cos.n;

  return;
}

void precession(radec50_cos, t50, mean_cos)
/*
correct precession and estimate
mean position in equinox of the day
from mean position in B1950.0

1989/3/18 by Takeshi Ito
1990/10/24 Toshihiro Handa
1991/12/30 T.Handa
*/
DIRECT_COS radec50_cos; /* <in>  mean position in B1950.0 [direct cosine] */
double t50; /* <in>  elapse time until B1950.0 [Besselian century] */
DIRECT_COS *mean_cos; /* <out> mean position in equonox of the day [direct cosine] */
{
  double zeta0,
    z,
    theta,
    czeta0, szeta0, cz, sz, ctheta, stheta;
  static double s2r=PI/648000.0;
  
  zeta0 = (2304.948  + ( 0.302 + 0.018 * t50) * t50) * t50; /* [sec] */
  z     = (2304.948  + ( 1.093 + 0.019 * t50) * t50) * t50; /* [sec] */
  theta = (2004.2555 + (-0.462 - 0.042 * t50) * t50) * t50; /* [sec] */

  zeta0 = zeta0 * s2r; /* [sec] --> [rad] */ 
  z     = z     * s2r; /* [sec] --> [rad] */ 
  theta = theta * s2r; /* [sec] --> [rad] */ 

  czeta0 = cos(zeta0);
  szeta0 = sin(zeta0);
  cz = cos(z);
  sz = sin(z);
  ctheta = cos(theta);
  stheta = sin(theta);

  (*mean_cos).l =  ( czeta0 * cz * ctheta - szeta0 * sz)* radec50_cos.l
    +(-szeta0 * cz * ctheta - czeta0 * sz)* radec50_cos.m
    +(-cz * stheta)                       * radec50_cos.n;

  (*mean_cos).m =  ( czeta0 * sz * ctheta + szeta0 * cz)* radec50_cos.l
    +(-szeta0 * sz * ctheta + czeta0 * cz)* radec50_cos.m
    +(-sz * stheta)                       * radec50_cos.n;

  (*mean_cos).n =   czeta0 * stheta   * radec50_cos.l
    +(-szeta0 * stheta) * radec50_cos.m
    + ctheta            * radec50_cos.n;

  return;
}

double refract(el, meteo)
/*
apply an elevation correction by the atmospheric refraction 

1989. 4.13 by Takeshi Ito  
1990/8/28 Toshihiro Handa
1991/12/30 T.Handa
*/
double el; /* <in> elevation of an apparent position [deg] */
METEO_PRM meteo; /* <in> the meteorological paramters */
{
  double el_obs, /* elevation after corrected the atmospheric refraction [deg] */
    atm_tmp0, /* atmospheric temperature [K] */
    n0, /* refraction index of atmosphere (1.0e-6) */
    z, /* zenith distance [rad] */
    tanz,
    del; /* elevation correction due to atmospheric refraction [rad] */

  atm_tmp0 = 273.15 + meteo.tmp;
  n0 = (3.73e5 * meteo.water / (atm_tmp0*atm_tmp0) + 77.6 * meteo.press / atm_tmp0) * 1.0e-6;
  z = (90.0 - el) * PI / 180.0; 
  tanz = tan(z);

  del = n0 * (1.0 - 0.0011 * tanz * tanz) * tanz;
  el_obs = el + del / PI * 180.0;

  return(el_obs);
}

double gravity1980(phi)
double phi; /* <in> geographical latitude [deg] */
/*
estimate standard gravity using Chebyshev approximation of gravity1980 eq.
see http://wwwsoc.nii.ac.jp/geod-soc/web-text/ Chap.2 2-1-6, 2-2-1-4

979.813469[gal] at NRO phi=35.933333 deg

2005/3/18 T.Handa
 */
{
  double sin_phi,sin_phi2;
  static double coeff[5]={1.0,0.0052790414,0.0000232718,0.0000001262,0.0000000007};
  double xx,gravity;
  int ii;

  sin_phi=sin(phi/180.0*PI);
  sin_phi2=sin_phi*sin_phi;

  xx=coeff[4];
  for(ii=0;ii<4;ii++)
    xx=coeff[3-ii]+xx*sin_phi2;

  gravity=978.03267715*xx;

  return(gravity);
}

double refract_optical(el,gravity,meteo,lambda)
/*
correction due to atmospheric refraction in optical/NIR regime
revised from atmref(z,g,p,t,f,l,pr) based on WataruTanaka's calculation

<return>
elevation to observe [deg] 

2003/5/14 Wataru Tanaka,Jun Ichi Morino (for SUBARU)
2005/3/18 T.Handa (transfered for UltraASROS) 
*/
double el; /* <in> apparent elevation [deg] */
double gravity; /* <in> gravity constant [gal=cm/s^2] */
METEO_PRM meteo; /* <in> meteorological paramters */
double lambda; /* <in> obs wavelength [um] */
{
  double el_obs;
  double z_dist; /* zenith distance [deg] */

  static double bsl=0.00130; /* Bessel constant */
  double bsl2; /* bsl^2 */
  static double std_gravity=980.665;

  double press; /* atm pressure [hPa] */
  double p_correct; /* atmosphare pressure corrected by gravity */ 
  double temp; /* atm temperature [K] */
  double p_wv; /* water vaper pressure [hPa] */

  double ds,dw,a0,a1,a2;

  double tz,tz3,tz5;
  double lmd2,lmd4,lmd6; /* 1/(lambda^2),1/(lamdba^4),1/(lambda^6) */
  double temp2,temp3; /* temp^2,temp^3 */
  double rho,rho2,rho3;
  /*  double index_ref; *//* refraction index */
  double d_zenith; /* zenith_vacuum - zenith_actual [deg] */

  /* pre-calculation for powers */
  press=meteo.press;
  temp=meteo.tmp+273.14;
  p_wv=meteo.water;
  bsl2=bsl*bsl;
  lmd2=1.0/(lambda*lambda);
  lmd4=lmd2*lmd2;
  lmd6=lmd4*lmd2;
  temp2=temp*temp;
  temp3=temp2*temp;

  /* eq(12) in the Morino report */
  p_correct=press*std_gravity/gravity;

  /* eq(10) */
  ds=(p_correct-p_wv)*(1.0+(p_correct-p_wv)*(57.90e-8-0.00093250/temp+0.25844/temp2))/temp;

  /* eq(11) */
  dw= -0.00237321+2.23366/temp-710.792/temp2+77514.1/temp3;
  dw=p_wv*(1.0+p_wv*(1.0+0.00037*p_wv)*dw)/temp;

  /* eq(9) */
  rho=1.0e-8*(2371.34+683939.7/(130.0-lmd2)+4547.3/(38.9-lmd2))*ds
    +1.0e-8*(6487.31+58.058*lmd2-0.71150*lmd4+0.08851*lmd6)*dw;

  rho2=rho*rho;
  rho3=rho2*rho;

  /* eq(8); Owens eq. ; not used */
  //  index_ref=1.0+rho;

  /* eq(4) */
  a0=rho-rho*bsl+2.0*rho*bsl2-0.5*rho2*bsl;
  /* eq(5) */
  a1=0.5*rho2+rho3/6.0-rho*bsl-2.75*rho2*bsl+5.0*rho*bsl2;
  /* eq(6) */
  a2=0.5*rho3-2.25*rho2*bsl+3.0*rho*bsl2;

  /* eq(2) */
  z_dist=90.0-el;
  tz=tan(z_dist/180.0*PI);
  tz3=tz*tz*tz;
  tz5=tz3*tz*tz;

  /* eq(3) */
  d_zenith=(a0*tz+a1*tz3+a2*tz5)/PI*180.0;

  el_obs=el+d_zenith;

  return(el_obs);
}

void trans_sphere(old_lambda,new_lambda,inclination,old,new)
/*
convertion angular position between in two spherical coodinates

1990/12/25 T.Handa
1991/12/30 T.Handa (revise) use structured value
*/
double old_lambda, /* longitude of the ascending node in the original coordinate [rad] */
 new_lambda, /* longitude of the ascending node in the target coordinate [rad] */
 inclination; /* inclination of the target equatorial plane against the original [rad] */
XY_POS_RAD old,*new; /* position in the original (old) and target (new) coordinates [rad] */
{
  double along_o1; /* longitude offsets from the ascending point */
  DIRECT_COS old_cos,new_cos; /* direction cosines */

  /* get longitude offset from ascending point on the old coordinate */
  along_o1=old.x_rad-old_lambda;
  if(along_o1 < - PI)
    along_o1=along_o1+2.0*PI;
  else if(along_o1 > PI)
    along_o1=along_o1-2.0*PI;
  
  /* direction --> direction cosines */
  old_cos.l=cos(old.y_rad)*cos(along_o1);
  old_cos.m=cos(old.y_rad)*sin(along_o1);
  old_cos.n=sin(old.y_rad);

  /* rotating matrix */
  new_cos.l=old_cos.l;
  new_cos.m= old_cos.m*cos(inclination)+old_cos.n*sin(inclination);
  new_cos.n=-old_cos.m*sin(inclination)+old_cos.n*cos(inclination);

  /* direction cosines --> direction */
  if(new_cos.l==0.0 && new_cos.m==0.0) {
    (*new).x_rad=0.0;
    if(new_cos.n < 0.0)
      (*new).y_rad= -PI/2.0;
    else
      (*new).y_rad=  PI/2.0;
  } else {
    (*new).x_rad=atan2(new_cos.m,new_cos.l)+new_lambda;

    if((*new).x_rad<0.0)
      (*new).x_rad= (*new).x_rad + 2.0 * PI;
    else if((*new).x_rad>2.0*PI)
      (*new).x_rad= (*new).x_rad - 2.0 * PI;
    
    if(new_cos.n > 1.0)
      (*new).y_rad =  PI / 2.0;
    else if(new_cos.n < -1.0)
      (*new).y_rad = -PI / 2.0;
    else
      (*new).y_rad=asin(new_cos.n);
  }

  return;
}

double v_earth_moon(et,eclip)
/*
estimate line-of-site velocity due to Earth rotation around
the balicenter of Earth and Moon

<return> velocity [km/s]

author: base upon "Methods of Experimental Physics vol12-C" DOP-program
T.Minezaki: convert into C-language
1990/12/24 T.Handa: in C-language function
1991/12/23 T.Handa: use C-language structure valuables
*/
EP_TIME et; /* <in> ephemerical time */
XY_POS_RAD eclip; /* <in> ecliptic longitude and latitude of the target object [rad] */
{
  double et_t2; /* et.t**2 */
  double v_moon;
  static double ellipt=0.054900489; /* eccentricity of Moon orbit */
  double ellipt2,ellipt3;
  static double mean_ang_vel=2.0*PI/27.321661, /* mean angular velocity of Moon [rad/day] */
    a_orbit=60.2665*6378.388, /* semimajor axis of Moon orbit [km] (about 60 times of the Earth's equator radius) */
    inclination=5.1453964*PI/180.0; /* inclination of Moon orbit against the ecliptic plane [rad] */

  double omega, /* mean ecliptic longitude of the luna ascending node [deg] */
    a_moon, /* mean ecliptic longitude of luna ascending node + mean lunar longitude of Moon [deg] */
    gamma_moon;	/* mean ecliptic longitude of luna ascending node + lunar longitude of perigee [deg] */

  double pi_moon, /* mean lunar longitude of perigee [rad] */
    moon_lambda_moon, /* mean lunar longitude of Moon [rad] */
    mean_anomaly_moon, /* mean anomaly of Moon [rad] */
    true_anomaly_moon, /* true anomaly of Moon (rough estimation) [rad] */
    true_lambda_moon; /* true anomaly of Moon [rad] */

  double hop_moon; /* rotation velocity of Moon */
  XY_POS_RAD lunatic; /* target position in the lunar coodinate [rad] */

  et_t2=(et.t)*(et.t);

  ellipt2=ellipt*ellipt;
  ellipt3=ellipt2*ellipt;

  /* mean ecliptic longitude of luna ascending node [deg] */
  omega=259.183275-0.0529539222*(et.d)+(0.002078+0.000002*(et.t))*et_t2;
/* mean ecliptic longitude of luna ascending node + mean lunar longitude of Moon [deg] */
  /*     ---remarks: aberration is already corrected. ---  */
  a_moon=270.434164+13.176396527*(et.d)+(-0.001133+0.0000019*(et.t))*et_t2;
/* mean ecliptic longitude of luna ascending node + lunar longitude of perigee [deg] */
  /*     ---remarks: aberration is already corrected. ---  */
  gamma_moon=334.329556+0.1114040803*(et.d)+(-0.010325-0.000012*(et.t))*et_t2;

  /* mean lunar longitude of perigee [rad] */
  pi_moon=(gamma_moon-omega)*PI/180.0;
  /* mean lunar longitude of Moon [rad] */
  moon_lambda_moon=(a_moon-omega)*PI/180.0;
  /* mean anomaly of Moon [rad] */
  mean_anomaly_moon=moon_lambda_moon-pi_moon;
  /* true anomaly of Moon (rough estimation) [rad] */
  true_anomaly_moon=mean_anomaly_moon
    +(2.0-0.25*ellipt2)*ellipt*sin(mean_anomaly_moon)
    +1.25*ellipt2*sin(2.0*mean_anomaly_moon)
    +13.0/12.0*ellipt3*sin(3.0*mean_anomaly_moon);
  /* true anomaly of Moon [rad] */
  true_lambda_moon=pi_moon+true_anomaly_moon;
  
  /* target position: ecliptic coordinate -----> lunar coordinate */
  trans_sphere(omega*PI/180.0,0.0,inclination,eclip,&lunatic);
  /* rotation velocity of Moon [km/day] */
  hop_moon=mean_ang_vel*a_orbit/sqrt(1.0-ellipt2);
  /* [km/day] -----> [km/sec] */
  hop_moon=hop_moon/86400.0;
  /* line-of-sight velocity due to Moon rotation [km/sec] */
  /*     81.3: mass ratio of Moon and Earth        */
  v_moon=-hop_moon/81.30*cos(lunatic.y_rad)*
    (sin(true_lambda_moon-lunatic.x_rad)
     -ellipt*sin(pi_moon-lunatic.x_rad));

  return(v_moon);
}

double v_earth_revolution(et,radec,eclip)
/*
estimate line-of-site velocity due to Earth rotation around Sun
<return> velocity [km/s]

author: base upon "Methods of Experimental Physics vol12-C" DOP-program
T.Minezaki: convert into C-language
1990/12/25 T.Handa: in C-language function
1991/12/23 T.Handa: use C-language structure valuables
*/
EP_TIME et; /* <in> ephemerical time */
XY_POS radec; /* <in> apparent RADec in equinox of the day [deg] */
XY_POS_RAD *eclip; /* <out> target position in the equliptic coordinate [rad] */
{
  XY_POS_RAD radec_rad; /* apperent RADec in equinox of the day [rad] */
  double et_t2; /* et.t**2 */
  double v_revol;
  static double astron_unit=149598500.0, /* semimajor axis of Earth orbit [km] */
    mean_ang_vel=2.0*PI/365.2564; /* mean angular velocity [rad/day] */
  double ellipt,ellipt2,ellipt3; /* eccentricity of Earth orbit */
  double mean_anomaly, /* mean anomaly of Earth [rad] */
    pi_e_sun, /* ecliptic longitude of perihelion from Sun [rad] */
    true_anomaly, /* rough estimation of true anomaly [rad] */
    inclination, /* inclination of Earth orbit [rad] */
    lambda_earth, /* true ecliptic longitude of Earth from Sun [rad] */
    lambda_sun,	/* trun ecliptic longitude of Sun from Earth [rad] */
    hop; /* Earth rotation velocity [km/sec] */


  et_t2=(et.t)*(et.t);

  /* mean anomaly of Earht [rad] */
  mean_anomaly=358.47583+0.9856002670*(et.d)+(-0.000150-0.000003*(et.t))*et_t2;
  mean_anomaly=mean_anomaly*PI/180.0;
  /* ecliptic longitude of Earth perihelion from Sun [rad] */
  pi_e_sun=101.22083+0.0000470684*(et.d)+(0.000453+0.000003*(et.t))*et_t2;
  pi_e_sun=pi_e_sun*PI/180.0;

  /* eccentricity of Earth orbit */
  ellipt=0.01675104+(-0.00004180-0.000000126*(et.t))*(et.t);
  ellipt2=ellipt*ellipt;
  ellipt3=ellipt2*ellipt;
  /* inclination of Earth orbit against Earth equtoric plane [rad] */
  inclination=23.452294+(-0.0130125+(-0.00000164+0.000000503*(et.t))*(et.t))*(et.t);
  inclination=inclination*PI/180.0;
  /* rough estimation of true anomaly [rad] */
  true_anomaly=mean_anomaly+(2.0-0.25*ellipt2)*ellipt*sin(mean_anomaly)
    +1.25*ellipt2*sin(2.0*mean_anomaly)
    +13.0/12.0*ellipt3*sin(3.0*mean_anomaly);
  /* true ecliptic longitude of Earth from Sun [rad] */
  lambda_earth=pi_e_sun+true_anomaly;
  /* true ecliptic longitude of Sun from Earth [rad] */
  lambda_sun=lambda_earth+PI;

  /* equitorial coordinate ---> ecliptic coordinate (radec[rad]-->*eclip[rad]) */
  radec_rad.x_rad=radec.x/180.0*PI;
  radec_rad.y_rad=radec.y/180.0*PI;
  trans_sphere(0.0,0.0,inclination,radec_rad,eclip);
  
  /* rotation velocity [km/sec] */
  hop=mean_ang_vel*astron_unit/sqrt(1.0-ellipt2);
  hop=hop/86400.0;

  /* line-of-sight velocity due to Earth rotation [km/sec] */
  v_revol=-hop * cos( (*eclip).y_rad ) *
    (  sin(lambda_sun - (*eclip).x_rad)
       - ellipt * sin(pi_e_sun - (*eclip).x_rad) );

  return(v_revol);
}

double v_earth_spin(lst,radec,obs_site)
/*
estimate line-of-site velocity due to Earth spin
<return> velocity [km/s]

author: base upon "Methods of Experimental Physics vol12-C" DOP-program
T.Minezaki: convert into C-language
1990/12/24 T.Handa: in C-language function
1991/12/30 T.Handa: use C-language structure valuables
*/
double lst; /* local sidereal time [rad] */
XY_POS radec; /* apparent RADec in equinox of the day [deg] */
SITE_POS obs_site; /* location of an observing site  [deg and m] */
{
  double obs_y,
    v_obs,
    r_ho,
    v_r_ho,
    d_lat;
  static double c1=0.997269566414; /* conversion factor between sun and sidereal times */

  obs_y=obs_site.y*PI/180.0; /* latitude of the site [rad] */

  /* distance of the site from Earth center [m] */
  r_ho=6378160.0*(0.998327073+0.001676438*cos(2.0*obs_y)
		  -0.00000351*cos(4.0*obs_y)+0.000000008*cos(6.0*obs_y))
    +obs_site.z;

  /* spin velocity [m/sidereal_day --> km/s] */
  v_r_ho=2.0*PI*r_ho;
  v_r_ho=v_r_ho/24.0/1000.0/3600.0/c1;
  /* difference of geocentric latitude from geological latitude (arcsec) */
  d_lat=-692.7430*sin(2.0*obs_y)+1.1633*sin(4.0*obs_y)-0.0026*sin(6.0*obs_y);
  /* geocentric latitude [rad] */
  obs_y=obs_y+d_lat*PI/3600.0/180.0;
  
  v_obs=v_r_ho*cos(obs_y)*cos(radec.y*PI/180.0)*sin(lst-radec.x*PI/180.0);
  return(v_obs);
}

double v_solar_motion(et,code_sw,radec_cos)
/*
estimate line-of-site velocity due to Sun proper motion
<return> velocity [km/s]

author: base upon "Methods of Experimental Physics vol12-C" DOP-program
T.Minezaki: convert into C-language
1990/12/25 T.Handa: in C-language function
1991/12/23 T.Handa: use C-language structure valuables
*/
EP_TIME et; /* <in> ephemerical time (required only if code_sw=10) */
int code_sw; /* <in> coordinate code 2=mean position in B1950.0, 10=apparent position in equnox of the day */
DIRECT_COS radec_cos; /* <in> RADec of the target */
{
  /* proper motion of Sun (LSR definition) */
  static double v_sun_motion=20.0;
  XY_POS sol_apex50; /* Sun motion against LSR in RADec(B1950.0); LSR definition */
  DIRECT_COS apex50_cos, /* its direct cosine(mean in B1950.0) */
    apex_cos; /* (apparent in equinox of the day) */
  double v_sun_lsr;
  NUTATION_PRM 	nut_prm;

  /* solar apex against LSR; definition of LSR */
  sol_apex50.x=270.479431;
  sol_apex50.y=30.001165;
  /* corresponds RA=18h,Dec=+30deg in B1900.0 */

  dir_cos(sol_apex50,&apex50_cos);

  if(code_sw==10) /* apparent RADec in equinox of the day of the solar apex */ {
    apparent(et,apex50_cos,&apex_cos,&nut_prm);
    apex50_cos.l=apex_cos.l;
    apex50_cos.m=apex_cos.m;
    apex50_cos.n=apex_cos.n;
  }

  v_sun_lsr =	 apex50_cos.l*radec_cos.l
    +apex50_cos.m*radec_cos.m
    +apex50_cos.n*radec_cos.n;
  
  v_sun_lsr= -v_sun_lsr * v_sun_motion;

  return(v_sun_lsr);
}

double vel_shift(et,lst,radec,code_sw,radec_cos,obs_site,v_helio_rad)
/*
estimate line-of-site velocity of the observing site

formulation:
The caluclated velocity is 
against the local standard of rest (LSR) defined by IAU.
Recession is positive.

We considers the following factors:
 proper motion of Sun against LSR
 rotation of the Earth-Moon system around the balicenter of the solar system
 Earth rotation around the balicenter of Earth and Moon
 Earth spin

We do not consider the other factors such as:
 difference between balicenter of the solar system and Sun
 perturbation due to planets

Accuracy is about 0.02km/s.

<return> line-of-site velocity of the observing site against LSR [km/s]

AuthorFbase upon "Methods of Experimental Physics vol12-C" DOP-program
T.Minezaki: convert into C-language
1990/12/25 T.Handa: in C-language function
1991/12/30 T.Handa: use C-language structure valuables
*/
EP_TIME et; /* <in> ephemerical time [day,Jcent,Bcent] */
double lst; /* <in> local sidereal time [rad] */
XY_POS radec; /* <in> apparent RADec of a target in equnox of the day [deg] */
int code_sw; /* <in> code for radec_cos; 2=mean in B1950.0, 10=apparent in equnox of the day */
DIRECT_COS radec_cos; /* <in> mean RADec of the target in B1950.0 or apparent RADec in equnox of the day [direct cos] */
SITE_POS obs_site; /* <in> location of the observing site [deg and m] */
double *v_helio_rad; /* <out> heliocentric velocity [km/sec] */
{
  XY_POS_RAD eclip; /* target position in ecliptic coordinate [rad] */

  double v_sun,v_moon,v_spin,v_revol,v_lsr_shift;
	
  /* proper motion of Sun (difference between heliocentric and LSR velocities) */
  v_sun=v_solar_motion(et,code_sw,radec_cos);

  /* due to Earth spin */
  v_spin=v_earth_spin(lst,radec,obs_site);

  /* due to Earth rotation around Sun */
  v_revol=v_earth_revolution(et,radec,&eclip);

  /* due to Earth rotation around balicenter of Earth-Moon */
  v_moon=v_earth_moon(et,eclip);

  /* overall velocity [km/s] */
  *v_helio_rad=v_revol+v_moon+v_spin;
  v_lsr_shift=*v_helio_rad+v_sun;
	
  return(v_lsr_shift);
}

void xy2azel(azel,et,lst,obs_site,coord_sys,xy)
/*
estimate AzEl from position in the asigned coordinate
remarks: A full calculation will be done in each function call.
No atmospheric fraction effect is considered.

1997/6/23 T.Handa
*/
XY_POS *azel; /* <out> */
EP_TIME	et; /* <in> ephemerical time[rad] */
double	lst; /* <in> local sidereal time [rad] */
SITE_POS obs_site; /* <in> location of an observing site */
int coord_sys; /* <in> code of asigned coordinate */
XY_POS xy; /* <in> longitude and latitude [deg] */
{
  XY_POS radec;

  NUTATION_PRM nut_prm; /* the nutation paramaters */

  DIRECT_COS radec_cos,radec_app_cos;

  /* convert the coordinate */
  switch(coord_sys) {
  case COORD_LB: /* lb */
    lb2radec50(&radec,xy);
    break;

  default:
    radec.x=xy.x;
    radec.y=xy.y;
    break;
  }

  /* convert to direct cosine */
  dir_cos(radec,&radec_cos);

  /* convert to apparent position in equnox of the day */
  switch(coord_sys) {
  case COORD_RADEC_APP:
    radec_app_cos.l=radec_cos.l;
    radec_app_cos.m=radec_cos.m;
    radec_app_cos.n=radec_cos.n;
    break;

  default:
    apparent(et,radec_cos,&radec_app_cos,&nut_prm);
  }

  /* estimate AzEl position */
  radec2azel(lst,radec_app_cos,obs_site,azel);

  return;
}

/* ==== sphare transformation ==== */
double dircos_inner_product(xx,yy)
DIRECT_COS xx,yy;
/*
inner product of unit vectors xx and yy
return=xx(dot)yy;

1996/5/1 T.Handa
*/
{
  double inner;

  inner=xx.l*yy.l+xx.m*yy.m+xx.n*yy.n;
  return(inner);
}

void dircos_outer_product(xx_yy,xx,yy)
DIRECT_COS xx,yy,*xx_yy;
/*
outer product from unit vector xx to yy.
xx_yy=xx(cross)yy;

1996/5/1 T.Handa
*/
{
  (*xx_yy).l=xx.m*yy.n-xx.n*yy.m;
  (*xx_yy).m=xx.n*yy.l-xx.l*yy.n;
  (*xx_yy).n=xx.l*yy.m-xx.m*yy.l;

  return;
}

double VectorAngleOnUnitSphere(xx,aa,bb)
DIRECT_COS xx,aa,bb;
/*
angle between xx-->aa and xx-->bb on the unit sphere.
xx,aa,bb are unit vectors

1996/5/1 T.Handa
*/
{
  double yy,zz,angle;
  DIRECT_COS vector;

  dircos_outer_product(&vector,aa,bb);
  yy=dircos_inner_product(vector,xx);

  zz=dircos_inner_product(aa,bb)
    -dircos_inner_product(xx,aa)*dircos_inner_product(xx,bb);

  angle=atan2(yy,zz);

  return(angle);
}


double DistanceOnUnitSphere(xx,yy)
DIRECT_COS xx,yy;
/*
angular distance between vectors xx and yy

1996/5/9 T.Handa
*/
{
  double angle,inp;

  inp=dircos_inner_product(xx,yy);
  angle=acos(inp);

  return(angle);
}

double AzEl2PA(azel_pos, cnp_pos)
/*
calculate PositionAngle against the RaDec(B1950.0) coordinate from AzEl
in degree

1996/5/14 T.Handa
*/
XY_POS	azel_pos;
XY_POS	cnp_pos;	/* CNP in AzEl */ 
{
  DIRECT_COS azel,cnp;
  double angle;
  DIRECT_COS zenith;

  zenith.l=0.0; zenith.m=0.0; zenith.n=1.0;

  dir_cos(azel_pos,&azel);
/* estimate PA */
  dir_cos(cnp_pos,&cnp);
  angle=VectorAngleOnUnitSphere(azel,cnp,zenith);
  angle=angle/PI*180.0;

  return(angle);	
}
