/*
library for the NEWSTAR data converter

1996/1/9 Toshihiro HANDA : ver.1.00
1998/7/14 T.Handa : ver.1.01 (Linux support)
2000/7/21 J.I.Morino, T.Handa : ver.1.11 (debug)
2004/12/1 T.Handa : ver.1.20 (add) newstar SSDxxx format
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <StData.h>
#include <StStd.h>
#include <ns2star.h>

double convert_endian_double(endian,inp)
int endian; /* <in> endian BIG_ENDIAN,LITTLE_ENDIAN */
char *inp; /* <in> input binary */
{
  int host_endian;
  int ii;
  static int byte_width=8;

  union	{
    double dd;
    char cc[8];
  } cmn;

#ifdef INTEL_CPU
  host_endian=LITTLE_ENDIAN;
#else
  host_endian=BIG_ENDIAN;
#endif

  if(endian==host_endian) {
    for(ii=0;ii<byte_width;ii++)
      cmn.cc[ii]=inp[ii];

  } else { /* endian conversion */

    for(ii=0;ii<byte_width;ii++) 
      cmn.cc[ii]=inp[byte_width-ii-1];
  }

  return(cmn.dd);
}

float convert_endian_float(endian,inp)
int endian; /* <in> endian BIG_ENDIAN,LITTLE_ENDIAN */
char *inp; /* <in> input binary */
{
  int host_endian;
  int ii;
  static int byte_width=4;

  union	{
    float ff;
    char cc[4];
  } cmn;

#ifdef INTEL_CPU
  host_endian=LITTLE_ENDIAN;
#else
  host_endian=BIG_ENDIAN;
#endif

  if(endian==host_endian) {
    for(ii=0;ii<byte_width;ii++)
      cmn.cc[ii]=inp[ii];

  } else { /* endian conversion */

    for(ii=0;ii<byte_width;ii++) 
      cmn.cc[ii]=inp[byte_width-ii-1];
  }

  return(cmn.ff);
}

long convert_endian_long(endian,inp)
int endian; /* <in> endian BIG_ENDIAN,LITTLE_ENDIAN */
char *inp; /* <in> input binary */
{
  int host_endian;
  int ii;
  static int byte_width=4;

  union	{
    long ll;
    char cc[4];
  } cmn;

#ifdef INTEL_CPU
  host_endian=LITTLE_ENDIAN;
#else
  host_endian=BIG_ENDIAN;
#endif

  if(endian==host_endian) {
    for(ii=0;ii<byte_width;ii++)
      cmn.cc[ii]=inp[ii];

  } else { /* endian conversion */

    for(ii=0;ii<byte_width;ii++) 
      cmn.cc[ii]=inp[byte_width-ii-1];
  }

  return(cmn.ll);
}

short convert_endian_short(endian,inp)
int endian; /* <in> endian BIG_ENDIAN,LITTLE_ENDIAN */
char *inp; /* <in> input binary */
{
  int host_endian;
  int ii;
  static int byte_width=2;

  union	{
    short ss;
    char cc[2];
  } cmn;

#ifdef INTEL_CPU
  host_endian=LITTLE_ENDIAN;
#else
  host_endian=BIG_ENDIAN;
#endif

  if(endian==host_endian) {
    for(ii=0;ii<byte_width;ii++)
      cmn.cc[ii]=inp[ii];

  } else { /* endian conversion */

    for(ii=0;ii<byte_width;ii++) 
      cmn.cc[ii]=inp[byte_width-ii-1];
  }

  return(cmn.ss);
}

void XYrad2deg(xy)
/*
convert radian-->degree for both value in XY_POS
1995/12/2 Toshihiro HANDA : ver.1.00
*/
XY_POS	*xy; /* <i/o> rad-->deg */
{
  (*xy).x = (*xy).x/PI*180.0;
  (*xy).y = (*xy).y/PI*180.0;

  return;
}

long hhmmss2sec(hhmmss)
long	hhmmss;
/*
2000/7/21 J.I.Morino (debug) 24L-->60L
 */
{
  long	hh,mm,ss;
  long	sec;

  hh=hhmmss/10000;
  hhmmss=hhmmss-hh*10000;
  mm=hhmmss/100;
  ss=hhmmss-mm*100;

  sec=((hh*60L)+mm)*60L+ss;

  return(sec);
}

int get_NSprofile(NSprofile,max_ch,endian,in_file)
NS_PROFILE *NSprofile; /* <out> profile data */
int max_ch; /* <in> data ch number (must < 2048) */
int endian; /* <in> binary endian of the data */
FILE	*in_file; /* <in> input file */
/*
<return>
0 normal
>=100 successfully read only a part; read byte+100 
10 invalid ID key (the first 4bytes)
99 required max_ch is too large

1998/7/14 T.Handa : err_code, INTEL_CPU
2004/11/22 T.handa : (revise) for SSD version (based on http://www.nro.nao.ac.jp/~cosmos/45m/newstar/spectrum.html, mpu5m:/home0/newstar/src/lib/nfile/ZFSHED.f)
*/
{
  char buffer[LENGTH_SPECTRUM];
  char work_buffer[256];

  int ii;
  int err_code;
  long id_key;

  int l_record;

  l_record=2048+max_ch*sizeof(float);
  if(l_record>LENGTH_SPECTRUM) return(99);

/* read a record for a profile */
  err_code=fread(buffer,sizeof(buffer[0]),l_record,in_file);
  if(err_code!=l_record) return(100+l_record);

/* dummy? */
  id_key=convert_endian_long(endian,buffer);
  if(id_key<0) return(10);

/* observation scheduling */
  for(ii=0;ii<3;ii++) {
    minstrcpy((*NSprofile).NSsyshead.schedule[ii],&(buffer[8*ii-4]),8);
    minstrcpy((*NSprofile).NSsyshead.chapter[ii],&(buffer[20+8*ii]),8);
    minstrcpy((*NSprofile).NSsyshead.observer[ii],&(buffer[40+12*ii]),12);

/* observation date */
    minstrcpy(work_buffer,&(buffer[80+8*ii]),8);
    (*NSprofile).NSsyshead.start_date[ii]=atoi(work_buffer);
    minstrcpy(work_buffer,&(buffer[107+6*ii]),6);
    (*NSprofile).NSsyshead.start_time[ii]=atoi(work_buffer);
    minstrcpy(work_buffer,&(buffer[126+8*ii]),8);
    (*NSprofile).NSsyshead.stop_date[ii]=atoi(work_buffer);
    minstrcpy(work_buffer,&(buffer[149+6*ii]),6);
    (*NSprofile).NSsyshead.stop_time[ii]=atoi(work_buffer);
  }

/* total integration time */
  (*NSprofile).NSstatus.total_integ_time=convert_endian_long(endian,&(buffer[172]));
/* total number of orignal spectra */
  (*NSprofile).NSstatus.n_orig_sp=convert_endian_long(endian,&(buffer[176]));

/* object */
/* object name */
  minstrcpy((*NSprofile).NSobj.name,&(buffer[180]),16);
/* epoch */
  minstrcpy((*NSprofile).NSobj.name,&(buffer[196]),5);
  /* dummy for padding ; buffer[201]-buffer[207] */
/* position of the object */
  (*NSprofile).NSobj.obj.radec.x=convert_endian_double(endian,&(buffer[208]));
  (*NSprofile).NSobj.obj.radec.y=convert_endian_double(endian,&(buffer[216]));
  (*NSprofile).NSobj.obj.lb.x=convert_endian_double(endian,&(buffer[224]));
  (*NSprofile).NSobj.obj.lb.y=convert_endian_double(endian,&(buffer[232]));
/* radial velocity */
  (*NSprofile).NSobj.v=convert_endian_double(endian,&(buffer[240]));
  minstrcpy((*NSprofile).NSobj.v_reference,&(buffer[248]),4);
  minstrcpy((*NSprofile).NSobj.v_definition,&(buffer[252]),4);
/* on position; offset from the mapping center */
  (*NSprofile).NSobj.d_on.radec.x=convert_endian_double(endian,&(buffer[256]));
  (*NSprofile).NSobj.d_on.radec.y=convert_endian_double(endian,&(buffer[264]));
  (*NSprofile).NSobj.d_on.lb.x=convert_endian_double(endian,&(buffer[272]));
  (*NSprofile).NSobj.d_on.lb.y=convert_endian_double(endian,&(buffer[280]));
  (*NSprofile).NSobj.d_on.azel.x=convert_endian_double(endian,&(buffer[288]));
  (*NSprofile).NSobj.d_on.azel.y=convert_endian_double(endian,&(buffer[296]));
/* obs on position */
  (*NSprofile).NSobj.on.radec.x=convert_endian_double(endian,&(buffer[304]));
  (*NSprofile).NSobj.on.radec.y=convert_endian_double(endian,&(buffer[312]));
  (*NSprofile).NSobj.on.lb.x=convert_endian_double(endian,&(buffer[320]));
  (*NSprofile).NSobj.on.lb.y=convert_endian_double(endian,&(buffer[328]));
  (*NSprofile).NSobj.on.azel.x=convert_endian_double(endian,&(buffer[336]));
  (*NSprofile).NSobj.on.azel.y=convert_endian_double(endian,&(buffer[344]));
  (*NSprofile).NSobj.on.xy.x=convert_endian_double(endian,&(buffer[352]));
  (*NSprofile).NSobj.on.xy.y=convert_endian_double(endian,&(buffer[360]));

/* convert rad-->deg */
  XYrad2deg(&((*NSprofile).NSobj.obj.radec));
  XYrad2deg(&((*NSprofile).NSobj.obj.lb));
  XYrad2deg(&((*NSprofile).NSobj.d_on.radec));
  XYrad2deg(&((*NSprofile).NSobj.d_on.lb));
  XYrad2deg(&((*NSprofile).NSobj.d_on.azel));

  XYrad2deg(&((*NSprofile).NSobj.on.radec));
  XYrad2deg(&((*NSprofile).NSobj.on.lb));
  XYrad2deg(&((*NSprofile).NSobj.on.azel));
  XYrad2deg(&((*NSprofile).NSobj.on.xy));

/* reference of the local grid */
  minstrcpy((*NSprofile).NSobj.xy_coordinate,&(buffer[368]),2);
  /* dummy for padding ; buffer[370]-buffer[375] */
  (*NSprofile).NSobj.xy_pa=convert_endian_double(endian,&(buffer[376]));

/* switching mode */
  minstrcpy((*NSprofile).NSstatus.switch_mode,&(buffer[384]),5);
  /* dummy for padding ; buffer[389]-buffer[391] */

/* off position */
  for(ii=0;ii<2;ii++) {
/* off position; offset from the mapping center */
    (*NSprofile).NSobj.d_off[ii].radec.x=convert_endian_double(endian,&(buffer[392+ii*8]));
    (*NSprofile).NSobj.d_off[ii].radec.y=convert_endian_double(endian,&(buffer[408+ii*8]));
    (*NSprofile).NSobj.d_off[ii].lb.x=convert_endian_double(endian,&(buffer[424+ii*8]));
    (*NSprofile).NSobj.d_off[ii].lb.y=convert_endian_double(endian,&(buffer[440+ii*8]));
    (*NSprofile).NSobj.d_off[ii].azel.x=convert_endian_double(endian,&(buffer[456+ii*8]));
    (*NSprofile).NSobj.d_off[ii].azel.y=convert_endian_double(endian,&(buffer[472+ii*8]));

/* obs off position */
    (*NSprofile).NSobj.off[ii].radec.x=convert_endian_double(endian,&(buffer[488+ii*8]));
    (*NSprofile).NSobj.off[ii].radec.y=convert_endian_double(endian,&(buffer[504+ii*8]));
    (*NSprofile).NSobj.off[ii].lb.x=convert_endian_double(endian,&(buffer[520+ii*8]));
    (*NSprofile).NSobj.off[ii].lb.y=convert_endian_double(endian,&(buffer[536+ii*8]));
    (*NSprofile).NSobj.off[ii].azel.x=convert_endian_double(endian,&(buffer[552+ii*8]));
    (*NSprofile).NSobj.off[ii].azel.y=convert_endian_double(endian,&(buffer[568+ii*8]));
    (*NSprofile).NSobj.off[ii].xy.x=convert_endian_double(endian,&(buffer[584+ii*8]));
    (*NSprofile).NSobj.off[ii].xy.y=convert_endian_double(endian,&(buffer[600+ii*8]));
  }

/* convert rad-->deg */
  for(ii=0;ii<2;ii++){
    XYrad2deg(&((*NSprofile).NSobj.d_off[ii].radec));
    XYrad2deg(&((*NSprofile).NSobj.d_off[ii].lb));
    XYrad2deg(&((*NSprofile).NSobj.d_off[ii].azel));

    XYrad2deg(&((*NSprofile).NSobj.off[ii].radec));
    XYrad2deg(&((*NSprofile).NSobj.off[ii].lb));
    XYrad2deg(&((*NSprofile).NSobj.off[ii].azel));
    XYrad2deg(&((*NSprofile).NSobj.off[ii].xy));
  }

/* beam/freq switch */
/* switching freq [Hz] */
  (*NSprofile).NSrx.freq_fsw=convert_endian_float(endian,&(buffer[616]));
/* freq switching offset [Hz] */
  (*NSprofile).NSrx.d_freq_fsw=convert_endian_float(endian,&(buffer[620]));
/* beam throw angle */
  (*NSprofile).NSobj.d_offbeam=convert_endian_float(endian,&(buffer[624]));
  (*NSprofile).NSobj.d_offbeam=(*NSprofile).NSobj.d_offbeam/PI*180.0;

/* antenna */
  minstrcpy((*NSprofile).NSrx.feed,&(buffer[628]),2);
  /* dummy for padding ; buffer[630]-buffer[631] */
  (*NSprofile).NSant.hpbw=convert_endian_float(endian,&(buffer[632]));
  (*NSprofile).NSant.hpbw=(*NSprofile).NSant.hpbw/PI*10800.0; /* rad-->arcmin */
  (*NSprofile).NSant.eta_app=convert_endian_float(endian,&(buffer[636]));
  (*NSprofile).NSant.eta_mb=convert_endian_float(endian,&(buffer[640]));
  (*NSprofile).NSant.eta_ant=convert_endian_float(endian,&(buffer[644]));
  (*NSprofile).NSant.eta_fss=convert_endian_float(endian,&(buffer[648]));
  (*NSprofile).NSant.eta_couple=convert_endian_float(endian,&(buffer[652]));
  (*NSprofile).NSant.gain=convert_endian_float(endian,&(buffer[656]));

/* pointing offset */
  (*NSprofile).NSant.pointing_corr.x=convert_endian_float(endian,&(buffer[660]));
  (*NSprofile).NSant.pointing_corr.y=convert_endian_float(endian,&(buffer[664]));
  XYrad2deg(&((*NSprofile).NSant.pointing_corr));
  (*NSprofile).NSant.pointing_corr.x=(*NSprofile).NSant.pointing_corr.x*60.0;
  (*NSprofile).NSant.pointing_corr.y=(*NSprofile).NSant.pointing_corr.y*60.0;

/* subreflector position */
  (*NSprofile).NSant.subref_offset.x=convert_endian_float(endian,&(buffer[668]));
  (*NSprofile).NSant.subref_offset.y=convert_endian_float(endian,&(buffer[672]));
  (*NSprofile).NSant.subref_offset.z1=convert_endian_float(endian,&(buffer[676]));
  (*NSprofile).NSant.subref_offset.z2=convert_endian_float(endian,&(buffer[680]));
  
/* meteorological data */
  (*NSprofile).NSstatus.meteo.tmp=convert_endian_float(endian,&(buffer[684]));
  (*NSprofile).NSstatus.meteo.press=convert_endian_float(endian,&(buffer[688]));
  (*NSprofile).NSstatus.meteo.water=convert_endian_float(endian,&(buffer[692]));

  (*NSprofile).NSstatus.wind.speed=convert_endian_float(endian,&(buffer[696]));
  (*NSprofile).NSstatus.wind.direction=convert_endian_float(endian,&(buffer[700]));

  (*NSprofile).NSstatus.tau_atm=convert_endian_float(endian,&(buffer[704]));

/* receiver */
  minstrcpy((*NSprofile).NSrx.name,&(buffer[708]),8);
  /* dummy for padding ; buffer[716]-buffer[719] */
  (*NSprofile).NSrx.f_track=convert_endian_double(endian,&(buffer[720]));
  (*NSprofile).NSrx.first_if=convert_endian_double(endian,&(buffer[728]));
  minstrcpy((*NSprofile).NSrx.sideband,&(buffer[736]),4);
  /* dummy for padding ; buffer[740]-buffer[747] */
  (*NSprofile).NSstatus.t_sys=convert_endian_float(endian,&(buffer[748]));
  (*NSprofile).NSstatus.tb_atm=convert_endian_float(endian,&(buffer[752]));

/* backend */
  (*NSprofile).NSbe.ch=convert_endian_long(endian,&(buffer[756]));
  minstrcpy((*NSprofile).NSbe.function_table,&(buffer[760]),8);
  minstrcpy((*NSprofile).NSbe.name,&(buffer[768]),4);
  /* backend bandwidth [Hz] */
  (*NSprofile).NSbe.bandwidth=convert_endian_float(endian,&(buffer[772]));
  (*NSprofile).NSbe.resolution=convert_endian_float(endian,&(buffer[776]));
  /* channel interval */
  (*NSprofile).NSbe.separation=convert_endian_float(endian,&(buffer[780]));
  (*NSprofile).NSbe.freq_rest=convert_endian_double(endian,&(buffer[784]));
  (*NSprofile).NSbe.n_freq_calib=convert_endian_long(endian,&(buffer[792]));
  /* dummy for padding ; buffer[796]-buffer[799] */
  (*NSprofile).NSbe.center_freq=convert_endian_double(endian,&(buffer[800]));
  for(ii=0;ii<10;ii++) {
    (*NSprofile).NSbe.freq_calib[ii]=convert_endian_double(endian,&(buffer[808+ii*8]));
    (*NSprofile).NSbe.ch_calib_center[ii]=convert_endian_float(endian,&(buffer[888+ii*4]));
    (*NSprofile).NSbe.ch_calib_width[ii]=convert_endian_float(endian,&(buffer[928+ii*4]));
  }
  (*NSprofile).NSstatus.noise_level=convert_endian_double(endian,&(buffer[968]));
  (*NSprofile).NSstatus.rms.n_range=convert_endian_long(endian,&(buffer[976]));
  for(ii=0;ii<6;ii++) {
    (*NSprofile).NSstatus.rms.start[ii]=convert_endian_long(endian,&(buffer[980+ii*4]));
    (*NSprofile).NSstatus.rms.stop[ii]=convert_endian_long(endian,&(buffer[1004+ii*4]));
  }
  minstrcpy((*NSprofile).NSstatus.basefit_function,&(buffer[1028]),4);
  (*NSprofile).NSstatus.basefit_dimension=convert_endian_long(endian,&(buffer[1032]));
  (*NSprofile).NSstatus.basefit.n_range=convert_endian_long(endian,&(buffer[1036]));
  for(ii=0;ii<6;ii++) {
    (*NSprofile).NSstatus.basefit.start[ii]=convert_endian_long(endian,&(buffer[1040+ii*4]));
    (*NSprofile).NSstatus.basefit.stop[ii]=convert_endian_long(endian,&(buffer[1064+ii*4]));
  }
  (*NSprofile).NSbe.t_hot=convert_endian_float(endian,&(buffer[1088]));
  (*NSprofile).NSstatus.norm=convert_endian_float(endian,&(buffer[1092]));

/* history */
  for(ii=0;ii<10;ii++){
    minstrcpy((*NSprofile).NShistory[ii],&(buffer[1096]),80);
  }
  minstrcpy((*NSprofile).comment,&(buffer[1896]),80);
  minstrcpy((*NSprofile).NSsyshead.version,&(buffer[1976]),4);

/* AC45 entension : 52 byte*/
  (*NSprofile).NSac45status.alc_time=convert_endian_double(endian,&(buffer[1984]));
  (*NSprofile).NSac45status.alc_v=convert_endian_double(endian,&(buffer[1992]));
  (*NSprofile).NSac45status.norm4multrx=convert_endian_double(endian,&(buffer[2000]));
  /* dummy for padding ; buffer[2008]-buffer[2027] */
  minstrcpy((*NSprofile).NSac45status.lag_window,&(buffer[2028]),8);

/* sp_name */
  minstrcpy((*NSprofile).NSsyshead.sp_name,&(buffer[2036]),12);

/* Vesrion dependent data */
  if(strcmp((*NSprofile).NSsyshead.version,"V001")==0){
    (*NSprofile).NSrx.ssb_ratio=convert_endian_float(endian,&(buffer[740]));
    (*NSprofile).NSrx.t_rx=convert_endian_float(endian,&(buffer[744]));;
    (*NSprofile).NSobj.v_radial=0.0;
  } else if(strcmp((*NSprofile).NSsyshead.version,"V002")==0){
    (*NSprofile).NSrx.ssb_ratio=0.0;
    (*NSprofile).NSrx.t_rx=0.0;
    (*NSprofile).NSobj.v_radial=convert_endian_double(endian,&(buffer[740]));
  } else{
    strcpy((*NSprofile).NSsyshead.version,"V001");
    (*NSprofile).NSrx.ssb_ratio=convert_endian_float(endian,&(buffer[740]));
    (*NSprofile).NSrx.t_rx=convert_endian_float(endian,&(buffer[744]));;
    (*NSprofile).NSobj.v_radial=0.0;
  }

/* correction of total data ch */
  if((*NSprofile).NSbe.ch>2048 || (*NSprofile).NSbe.ch<0) {
    (*NSprofile).NSbe.ch=2048;
  }
  if((*NSprofile).NSbe.ch>max_ch) {
    (*NSprofile).NSbe.ch=max_ch;
  }

/* data */
  for(ii=0;ii<(*NSprofile).NSbe.ch;ii++) {
    (*NSprofile).data[ii]=convert_endian_float(endian,&(buffer[2048+ii*4]));
  }

  /* padding by 0.0 for blank channels */
  if((*NSprofile).NSbe.ch<max_ch) {
    for(ii=(*NSprofile).NSbe.ch;ii<max_ch;ii++){
      (*NSprofile).data[ii]=0.0;
    }
  }
  
  return(0);
}

void get_NS_CBfile(nscb,endian,fp)
NS_CBdata *nscb; /* <out> information in newstar CB file */
int endian; /* <in> data endian */
FILE *fp; /* <in> newstar CB file */
{
  char buffer[CBFILE_SIZE];
  size_t read_byte;

  read_byte=fread(buffer,sizeof(buffer[0]),CBFILE_SIZE,fp);

  minstrcpy((*nscb).filename,&(buffer[424]),12);
  minstrcpy((*nscb).filetype,&(buffer[436]),6);
  (*nscb).n_sp=convert_endian_long(endian,&(buffer[796]));
  (*nscb).max_ch=convert_endian_long(endian,&(buffer[804]));

  return;
}
