/*
set default type scn name in .scn pipe-flow

1993/3/14 T.Handa ver.1.0
1997/11/28 T.Handa : ver.3.00 multi_pipe and new library
1998/3/19 T.Handa : ver.3.01 debug for Solaris2 fcc
1998/4/ 8 T.Handa : ver.3.20 (revise) .scn,.sdb
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"autoname.prm"
#define N_KEY 30
#define N_BLOCK	10

#define MAX_TAG_LENGTH	10

int options(argc,argv,tag,sys_coord)
/*
get parameters from options
*/
int argc;
char *argv[];
char *tag;
unsigned char *sys_coord;
{
  int ii,next_pos;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char buffer[MAX_PRM_RECORD_LENGTH];

  int coord;

/* default values */
  strcpy(tag,"");
  coord=0;	/* not assigned */

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"tag");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"coordinate");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"lb");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"radec");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"radec-app");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"azel");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"xy");	key_type[n_key-1]=0;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

  next_pos=0;
/* tag */
  n_scan=1;
  for(ii=0;ii<n_scan;ii++) {    
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    strcpy(buffer,StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value));
    if(strlen(buffer)>MAX_TAG_LENGTH)
      buffer[MAX_TAG_LENGTH-1]=(char)NULL;
    strcpy(tag,buffer);
    tag[MAX_TAG_LENGTH-1]=(char)NULL;
  }

/* coordinate */
  n_scan=5;
  for(ii=0;ii<n_scan;ii++) {    
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    strcpy(buffer,StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value));
    if(strcmp(buffer,"lb")==0) {
      coord=COORD_LB;
    } else if(strcmp(buffer,"radec")==0) {
      coord=COORD_RADEC;
    } else if(strcmp(buffer,"radec-app")==0) {
      coord=COORD_RADEC_APP;
    } else if(strcmp(buffer,"azel")==0) {
      coord=COORD_AZEL;
    } else if(strcmp(buffer,"xy")==0) {
      coord=COORD_XY;
    }
  }

  *sys_coord=(unsigned char)coord;

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

void change_filename(profile,tag,coord)
PROFILE *profile;
char *tag;
unsigned char coord;
{
  char	buffer[MAX_FILE_NAME_LENGTH];

  switch(coord){
  case COORD_LB:	/* lb */
    sprintf(buffer,"l%+08.3fb%+07.3f%s",(*profile).object.on_lb.x,(*profile).object.on_lb.y,tag);
    break;
  case COORD_RADEC:	/* radec B1950 */
    sprintf(buffer,"a%+08.3fd%+07.3f%s",(*profile).object.on_radec.x,(*profile).object.on_radec.y,tag);
    break;
  case COORD_RADEC_APP:	/* radec app */
    sprintf(buffer,"a%+08.3fp%+07.3f%s",(*profile).object.on_radec.x,(*profile).object.on_radec.y,tag);
    break;
  case COORD_AZEL:	/* azel */
    sprintf(buffer,"a%+08.3fe%+07.3f%s",(*profile).object.on_azel.x,(*profile).object.on_azel.y,tag);
    break;
  case COORD_XY:	/* local xy */
    sprintf(buffer,"x%+08.3fy%+07.3f%s",(*profile).object.on_xy.x,(*profile).object.on_xy.y,tag);
  }

  strcpy((*profile).file_name,strtok(buffer,"."));
  strcat((*profile).file_name,strtok(NULL,"."));
  strcat((*profile).file_name,strtok(NULL,"\n"));
}

int main(argc,argv)
int argc;
char *argv[];
{
  PROFILE profile;
  int i_file,n_file;
  char id_str[BUFFER_SIZE];
  int err_code;

  char tag[MAX_TAG_LENGTH];
  unsigned char sys_coord;

  int sdb_flag;
  char sdb_arg[MAX_IDX_RECORD_LENGTH];

/* get option */
  options(argc,argv,tag,&sys_coord);

/* Pipeline head */
  StarPipeHeadIn(id_str,&n_file);
  StarPipeHeadOut(n_file);

  for(i_file=0;i_file<n_file;i_file++) {
/* ProfileIn */
    err_code=StarProfileIn(&profile,&sdb_flag,sdb_arg);

    if(err_code!=0)	{
      StarPipeEndOut();
      exit(1);
    }

    if(sdb_flag==GOOD_SCAN || sdb_flag==BAD_SCAN) {
      if(sys_coord==0) sys_coord=profile.object.cood_sys;

/* change default name */
      change_filename(&profile,tag,sys_coord);
    }

/* ProfileOut */
    err_code=StarProfileOut(&profile,sdb_flag,sdb_arg);
  }

/* End of Pipeline */
  err_code=StarPipeEndIn(id_str);
  if(err_code!=0)
    fprintf(stderr,"%s: Pipeline end invalid=%s\n",argv[0],id_str);

  StarPipeEndOut();
  exit(0);
}
