/*
Baseline Fit : with interactive and background modes

1995/1/20 T.Oka & O.Kimura & T.Handa : created
1997/12/25 T.Handa : ver.3.00 multi_pipe and new library
1998/3/19 T.Handa : ver.3.01 debug for Solaris2 fcc
1998/4/ 8 T.Handa : ver.3.20 (revise) .scn,.sdb
1998/4/30 T.Handa : ver.3.21 (debug) fit range conversion in do_fit();
1998/5/20 T.Handa : ver.3.22 (revise) range show, range adjust
1998/5/21 T.Handa : ver.3.25 (revise) multi-unit for ranges
1998/12/21 T.Handa : ver.3.26 (revise) neglect next button before process
1999/ 3/ 4 T.Handa : ver.3.27 (debug) CUI baseline process
2002/ 5/13 T.Handa : ver.3.28 (revise) command order in the menubar
2008/ 5/27 T.Handa : ver.3.29 (debug) CUI baseline clear
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>

#include <Xm/DrawingA.h>
#include <Xm/MainW.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#include <Xm/PushBG.h>

#include <StData.h>
#include <StStd.h>

#include <Pxxlib.h>
#include <Pxplib.h>
#include <Ppslib.h>
#include <PxpGlib.h>
#include <StPxpShow.h>

#define	DEFAULT_PRMFILE	"baseline.prm"
#define N_KEY 25
#define	MAX_RANGES	256
#define MAX_UNIT 2

static PROFILE GLprofile;
static int GLsdb_flag;
static char GLsdb_arg[MAX_IDX_RECORD_LENGTH];
static PROFILE backup_profile;
static PROFILE baseline_profile;
static int GLn_file=0;
static int GLx_unit;
static RANGE GLx_range;
static RANGE GLy_range;

static RANGE GLfit_ch_range[MAX_RANGES];
static RANGE GLfit_range[MAX_UNIT][MAX_RANGES]; /* [0]ch, [1]v */
static int GLn_ch_range,GLrange_unit,GLf_order;
static int GLn_range[MAX_UNIT]; /* [0]ch, [1]v */
static int GLrangeclear_flag;
static int GLautorange_x,GLautorange_y,GLautorange_fit;
static BOX_RANGE GLphys_box;
static int GLrange_clear=1;

#define NOT_PROCESSED 0
#define RANGE_ASSIGNED 1
#define FIT_EXECUTED 2
#define SUBTRACTED 10
static int process_status=NOT_PROCESSED; /* 0=no process, 1=range assigned, 2=done fitting, 10=done subtraction */

static double coefficent[11]={0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0};

/*-----------undo---------------------*/
void do_reset()
/*
reset profile data as the initial
1997/12/17 T.Handa
*/
{
/* reverse copy from the backup profile */
  StarCopyProfile(&GLprofile,&backup_profile);
  process_status=RANGE_ASSIGNED;

  return;
}

/*-----------baseline fitting process---------------------*/
void do_baseline_fit(profile,fit_order,coef)
/*
make a fit using baseline flag by polynomial
1997/12/17 T.Handa
*/
PROFILE *profile;
int fit_order;
double *coef;
{
  int flagbit,ch;
  int ndata; /* the number of data to fit */
  double xx[MAX_CH],yy[MAX_CH];
  int data_ch;

  data_ch=(*profile).be_status.data_ch;
/* get channel flags with baseline flag */
  ndata=0;
  for(ch=1;ch<=data_ch;ch++){
    flagbit=StarReadBaselineFlag(profile,ch);
    if(flagbit==FLAGGED){
      xx[ndata]=ch;
      yy[ndata]=(*profile).data[ch-1];
      ndata++;
    }
  }

/* do least square fitting */
  LeastSquareFit(xx,yy,ndata,fit_order,coef);

  return;
}

float get_baselevel(ch,fit_order,coef)
/*
derive the baseline level at the channel using the baseline polynomial
1997/12/17 T.Handa
*/
double ch;
int fit_order;
double *coef;
{
  double baselevel;
  float level;
  int ii;

  baselevel=0;
  for(ii=0;ii<=fit_order;ii++) {
    baselevel=baselevel*ch+coef[fit_order-ii];
  }
  level=(float)baselevel;
  return(level);
}

void do_fit()
/*
do fit
1997/12/17 T.Handa
1998/4/24 T.Handa : ver.3.21 (debug) fit range conversion in do_fit();
*/
{
  int ch;
  int ii,n_range,range_unit;
  RANGE_long ch_range;
  int max_ch;
  int x_unit;
  RANGE x_range;

/* convert values from global valuables to local valuables */
  n_range=GLn_ch_range;
  range_unit=GLrange_unit;
  x_unit=GLx_unit;
  x_range.min=GLx_range.min;
  x_range.max=GLx_range.max;

  max_ch=GLprofile.be_status.data_ch;

  for(ii=0;ii<n_range;ii++) {
/* copy fit range to baseline flag */
    ch_range.min=(long)(rint(GLfit_ch_range[ii].min));
    ch_range.max=(long)(rint(GLfit_ch_range[ii].max));
    if(ch_range.min>=1 && ch_range.max<=max_ch)
      StarRWriteBaselineFlags(&GLprofile,ch_range,FLAGGED);
  }

/* remove baseline flag at the spurious channel */
  for(ch=1;ch<=max_ch;ch++) {
    if(StarReadSpuriousFlag(&GLprofile,ch)==FLAGGED)
      StarWriteBaselineFlag(&GLprofile,ch,UNFLAGGED);
  }

/* do baseline fit */
  do_baseline_fit(&GLprofile,GLf_order,coefficent);

/* make baseline profile */
  StarCopyProfile(&baseline_profile,&GLprofile);
  for(ii=0;ii<max_ch;ii++) {
    baseline_profile.data[ii]=get_baselevel((double)(ii+1),GLf_order,coefficent);
  }

  process_status=FIT_EXECUTED;

  return;
}

void plot_baseline()
/*
draw fitted baseline
1997/12/25 T.Handa
*/
{
  RANGE full_plot_x;

  full_plot_x.min=GLphys_box.x.min;
  full_plot_x.max=GLphys_box.x.max;
  PxpSetColor("red");
  StarPlotProfile(&baseline_profile,GLx_unit,full_plot_x);
  PxpSetColor("black");

  return;
}

void do_subtract_baseline()
/*
baseline subtraction with GLprofile=GLprofile-baseline_profile
1997/12/17 T.Handa
*/
{
  RANGE_long ch_range;
  double rms;

/* baseline subtraction */
  StarSubtractProfile(&GLprofile,&backup_profile,&baseline_profile);
  process_status=SUBTRACTED;

/* estimate rms noise level */
  ch_range.min=1;
  ch_range.max=GLprofile.be_status.data_ch;

  rms=StarCalcRms(&GLprofile,ch_range);

  return;
}

void plot_last_bsbox(profile,x_unit,fit_x,phys_box)
/*
This function should be removed after labber box cursoring support.
*/
PROFILE *profile; /* <in> */
int x_unit; /* <in> code of the unit of the x-axis */
double fit_x; /* <in> */
BOX_RANGE phys_box; /* <in> maxmum box size to plot */
{
  RANGE range_bar;
  double bar_x;

/* set and adjudt y range of boxes */
  range_bar.max= (*profile).rx_status.rms*3.0;
  range_bar.min= -range_bar.max;

  if(range_bar.min<phys_box.y.min)
    range_bar.min=phys_box.y.min;
  if(range_bar.max>phys_box.y.max)
    range_bar.max=phys_box.y.max;

  bar_x=fit_x;

  PxpGLine(bar_x,range_bar.min,bar_x,range_bar.max);

  return;
}

void plot_range_bsbox(profile,x_unit,n_range,fit_range,phys_box)
PROFILE *profile; /* <in> */
int x_unit; /* <in> code of the unit of the x-axis */
int n_range; /* <in> the numb of ranges */
RANGE *fit_range; /* <in> */
BOX_RANGE phys_box; /* <in> maxmum box size to plot */
{
  RANGE plot_x,bs_x;
  BOX_RANGE fit_box;
  PxpPoint fit_box_min,fit_box_max;

  int ii;

  plot_x.min=phys_box.x.min;
  plot_x.max=phys_box.x.max;
  StarSortRange(&plot_x);

/* set and adjudt y range of boxes */
  fit_box.y.max= (*profile).rx_status.rms*3.0;
  fit_box.y.min= -fit_box.y.max;

  if(fit_box.y.min<phys_box.y.min)
    fit_box.y.min=phys_box.y.min;
  if(fit_box.y.max>phys_box.y.max)
    fit_box.y.max=phys_box.y.max;

/* for each box */
  for(ii=0;ii<n_range;ii++) {
    if(x_unit==X_VELOCITY) {
      fit_box.x.min=StarCh2Velocity(profile,fit_range[ii].min);
      fit_box.x.max=StarCh2Velocity(profile,fit_range[ii].max);
    } else {
      fit_box.x.min=fit_range[ii].min;
      fit_box.x.max=fit_range[ii].max;
    }

    bs_x.min=fit_box.x.min;
    bs_x.max=fit_box.x.max;
    StarSortRange(&bs_x);

    if(!(bs_x.max<plot_x.min || bs_x.min>plot_x.max)) {
      if(bs_x.min<plot_x.min) bs_x.min=plot_x.min;
      if(bs_x.max>plot_x.max) bs_x.max=plot_x.max;
    }

    fit_box_min.x=bs_x.min;
    fit_box_min.y=fit_box.y.min;
    fit_box_max.x=bs_x.max;
    fit_box_max.y=fit_box.y.max;
    PxpGBox(fit_box_min,fit_box_max);
  }

  return;
}

/*-----------reset fitting range---------------------*/
void clear_range()
/*
clear baseline flag
1997/12/17 T.Handa
*/
{
  int ii, i_unit;

/* clear already assigned range */
  for(ii=0;ii<GLn_ch_range;ii++) {
    GLfit_ch_range[ii].min=0;
    GLfit_ch_range[ii].max=0;
  }
  GLn_ch_range=0;

  for(i_unit=0;i_unit<MAX_UNIT;i_unit++) {
    for(ii=0;ii<GLn_range[i_unit];ii++) {
      GLfit_range[i_unit][ii].min=0;
      GLfit_range[i_unit][ii].max=0;
    }
    GLn_range[i_unit]=0;
  }

/* clear flags in the profile */
  StarProfileClearBaselineFlag(&GLprofile,UNFLAGGED);

/* set status flag; range clear */
  GLrange_clear=1;

  process_status=NOT_PROCESSED;

  return;
}

/*-----------assigne the fitting range---------------------*/
void all_range_in_ch()
/*
convert all ranges asigned into the actual fitting ranges (in ch). 
All parameters are used by GLOBAL.

1998/5/21 T.Handa
*/
{
  int i_fit,ii,err[2];
  RANGE fit_ch;
  static char cmd[]="baseline";
  int data_ch;

  data_ch=GLprofile.be_status.data_ch;

/* ch to ch */
  for(ii=0,i_fit=0;ii<GLn_range[0];ii++) {
    GLfit_ch_range[i_fit].min=GLfit_range[0][ii].min;
    GLfit_ch_range[i_fit].max=GLfit_range[0][ii].max;
    i_fit++;
  }

/* v to ch */
  for(ii=0;ii<GLn_range[1];ii++) {
    fit_ch.min=StarVelocity2Ch(&GLprofile,&(err[0]),GLfit_range[1][ii].min);
    fit_ch.max=StarVelocity2Ch(&GLprofile,&(err[1]),GLfit_range[1][ii].max);

/* adjust over ranges */
/* actually all ranges */
    if((err[0]==OVER_FLOW && err[1]==UNDER_FLOW)
    || (err[0]==UNDER_FLOW && err[1]==OVER_FLOW)) {
      fprintf(stderr,"%s:cover all ch range. Adjust. : %s\n",cmd,GLprofile.file_name);
      fit_ch.min=1.0;
      fit_ch.max=(double)data_ch;
/* half over range */
    } else if(err[0]==NORMAL_END && err[1]==OVER_FLOW) {
      fprintf(stderr,"%s:v=%f is over range. Adjust: %s",cmd,GLfit_range[1][ii].max,GLprofile.file_name);
      fit_ch.max=(double)data_ch;
    } else if(err[0]==OVER_FLOW && err[1]==NORMAL_END) {
      fprintf(stderr,"%s:v=%f is over range. Adjust: %s",cmd,GLfit_range[1][ii].min,GLprofile.file_name);
      fit_ch.min=(double)data_ch;
/* half under range */
    } else if(err[0]==NORMAL_END && err[1]==UNDER_FLOW) {
      fprintf(stderr,"%s:v=%f is over range. Adjust: %s",cmd,GLfit_range[1][ii].max,GLprofile.file_name);
      fit_ch.max=1.0;
    } else if(err[0]==UNDER_FLOW && err[1]==NORMAL_END) {
      fprintf(stderr,"%s:v=%f is over range. Adjust: %s",cmd,GLfit_range[1][ii].min,GLprofile.file_name);
      fit_ch.min=1.0;
    } else if(!(err[0]==NORMAL_END && err[1]==NORMAL_END)) {
      fprintf(stderr,"%s:v=%f,%f is over range. Not use: %s",cmd,GLfit_range[1][ii].min,GLfit_range[1][ii].max,GLprofile.file_name);
      goto loop_btm;
    }

    GLfit_ch_range[i_fit].min=fit_ch.min;
    GLfit_ch_range[i_fit].max=fit_ch.max;

    i_fit++;
loop_btm:;
  }

  GLn_ch_range=i_fit;

  for(ii=0;ii<GLn_ch_range;ii++) {
    StarSortRange(&(GLfit_ch_range[ii]));
    if(GLfit_ch_range[ii].min<1.0 && GLfit_ch_range[ii].max>1.0)
      GLfit_ch_range[ii].min=1.0;
    if(GLfit_ch_range[ii].max>(double)data_ch && GLfit_ch_range[ii].min<(double)data_ch)
      GLfit_ch_range[ii].max=(double)data_ch;
  }

  return;
}

void add_fit_range(x_unit,new_range)
/*
Add fit range arrays

1998/5/21 T.Handa
*/
int x_unit;
RANGE new_range;
{
  static char cmd[]="baseline";
  int i_unit;

  switch(x_unit) {
  case X_CHANNEL:
    i_unit=0;
    break;
  case X_VELOCITY: default:
    i_unit=1;
    break;
  }

  if(GLn_range[i_unit]>=MAX_RANGES || GLn_range[0]+GLn_range[1]>=MAX_RANGES) {
    fprintf(stderr,"%s: too many ranges.\n",cmd);
    return;
  }
  GLfit_range[i_unit][GLn_range[i_unit]].min=new_range.min;
  GLfit_range[i_unit][GLn_range[i_unit]].max=new_range.max;
  GLn_range[i_unit]++;

  all_range_in_ch();

  return;
}

int click_position_phys(ii,jj,result_phys_pos,x_unit,profile)
int ii,jj; /* <in> pixel position */
PxpPoint *result_phys_pos; /* <out> */
int x_unit; /* <in> */
PROFILE *profile; /* <in> */
{
  double xx,yy;
  PxpPoint plot_pos,phys_pos;
  int data_ch,ch;
  int err_code;

  PxxGetPlotPosition(ii,jj,&xx,&yy);
  plot_pos.x=xx;
  plot_pos.y=yy;
  phys_pos=PxpGPhysicsValue(plot_pos);

  data_ch=(*profile).be_status.data_ch;

  if(x_unit==X_CHANNEL) {

  } else if(x_unit==X_VELOCITY) {
    ch=(int)rint(StarVelocity2Ch(profile,&err_code,phys_pos.x));
    switch(err_code) {
    case OVER_FLOW:
      ch=(int)((*profile).be_status.data_ch);
      break;
    case UNDER_FLOW:
      ch=1.0;
      break;
    case NORMAL_END:
      break;
    default:
      return(err_code);
    }
    phys_pos.x=StarCh2Velocity(profile,(double)ch);
  }

  (*result_phys_pos).x=phys_pos.x;
  (*result_phys_pos).y=phys_pos.y;
  return(NORMAL_END);
}

void ClickActions(parentWDG,gc,cdata)
Widget	parentWDG;
GC	gc;
XmDrawingAreaCallbackStruct	*cdata;
/*
define actions at click
1997/12/17 T.Handa ver.1.00
1998/5/21 T.Handa ver.1.10
*/
{
  int ii,jj;
  double xx;
  PxpPoint phys_pos;
  static RANGE new_range;
  static int onoff=1;

  Cursor	draw_cursor;
	
/* change the griph of cursor */
  draw_cursor=XCreateFontCursor(PxxGetDisplay(),XC_crosshair);
  XDefineCursor(PxxGetDisplay(),PxxGetWindow(),draw_cursor);
  PxxFlush();

/* just after range clear */
  if(GLrange_clear>0) {
    onoff=1;
    GLrange_clear= -1;
  }

/* responce at click */
  switch(cdata->reason) {
  case XmCR_INPUT:
    if(cdata->event->type==ButtonPress) {
      ii=cdata->event->xbutton.x;
      jj=cdata->event->xbutton.y;
      click_position_phys(ii,jj,&phys_pos,GLx_unit,&GLprofile);
      xx=phys_pos.x;

      if(onoff>0) {
	new_range.min=xx;
      } else {
	new_range.max=xx;
	add_fit_range(GLx_unit,new_range);
process_status=RANGE_ASSIGNED;
      }
      onoff= -onoff;
    }

/* display at click and refresh */
  case XmCR_EXPOSE:

    PxpSetColor("cyan");
    StarPlotProfileBaselineRange(&GLprofile,GLx_unit,GLphys_box,2);

    plot_range_bsbox(&GLprofile,GLx_unit,GLn_ch_range,GLfit_ch_range,GLphys_box);
    /* plot the last line */
    if(onoff<0) 
      plot_last_bsbox(&GLprofile,GLx_unit,new_range.min,GLphys_box);

    PxpSetColor("black");
  }

  return;
}

/*-----------draw profile---------------------*/

void local_StarPlotProfileStd(profile,x_unit,x_range,y_range,plot_box,base,spur)
/*
draw a profile in the standard format
1997/12/11 T.Handa
*/
PROFILE *profile;	/* <in> profile */
int x_unit;	/* <in> physical unit along x-axis */
RANGE x_range,y_range;	/* <in> range in both axes */
BOX_RANGE plot_box;	/* <in> print positions of labels and tick marks */
int base,spur; /* <in> base/spur plot switch (0=off,1=on) */
{
/*  BOX_RANGE phys_box;*/
  RANGE_long ch_range;
  PxpPoint name_pos;

/* profile name position */
  name_pos.x=plot_box.x.min+0.2;
  name_pos.y=plot_box.y.max-0.5;

/* plot frame */
  StarPlotProfileStdFrame(profile,x_unit,x_range,y_range,plot_box,&ch_range,&GLphys_box);

  if(base>0) {
/* plot baseline range */
    PxpSetColor("cyan");
    StarPlotProfileBaselineRange(profile,x_unit,GLphys_box,2);
  }

  if(spur>0) {
/* plot spurious range */
    PxpSetColor("pink");
    StarPlotProfileSpuriousRange(profile,x_unit,GLphys_box,1);
  }

  if(GLn_ch_range>0) {
/* plot ranges assigned in this procedure */
    PxpSetColor("cyan");
    plot_range_bsbox(&GLprofile,GLx_unit,GLn_ch_range,GLfit_ch_range,GLphys_box);

  }

  PxpSetColor("black");
/* profile name */
  PxpDrawString(name_pos.x,name_pos.y,(*profile).file_name);

/* profile itself */
  if(process_status<SUBTRACTED)
    PxpSetColor("blue");	/* not yet processed */
  else
    PxpSetColor("green");	/* already processed */
  StarPlotProfileInCh(profile,ch_range,x_unit);
  PxpSetColor("black");

  return;
}

/*-----------change assigned order of the polynomial---------------------*/
void OrderSelectActions(parentWDG,client_data,call_data)
Widget	parentWDG;
caddr_t	client_data,call_data;
{
  GLf_order=atol(client_data);

  return;
}

/*--------from Pxxlib--------------------*/
static void	(*do_plot)(), (*do_exit)(), (*do_help)();

/* define actions for buttons in the menu bar */
void star_PxxPushButtonActions(menubarWDG,client_data,call_data)
Widget	menubarWDG;
caddr_t	client_data,call_data;
/*
define actions for buttons in the menu bar
1997/10/12 T.Handa ver.1.00
1997/12/17 T.Handa ver.1.00 revise for baseline
*/
{
/*  RANGE full_plot_x;*/

  if(strcmp(client_data,"Exit")==0) {
    if(process_status<FIT_EXECUTED) {
      fprintf(stderr,"Baseline: cannot go next before process.\n");
    } else {
      XClearWindow(PxxGetDisplay(),PxxGetWindow());
      if(do_exit!=NULL) (*do_exit)();
      if(do_plot!=NULL) (*do_plot)();
    }
  }
  else if(strcmp(client_data,"Quit")==0) {
    exit(0);
  } else if(strcmp(client_data,"Fit")==0)	{ /* baseline fit */
    do_fit();
    plot_baseline();
  } else if(strcmp(client_data,"Reshow")==0){ /* show applied profile */
    XClearWindow(PxxGetDisplay(),PxxGetWindow());
    do_subtract_baseline();
    (*do_plot)();
  } else if(strcmp(client_data,"Clear")==0) { /* clear fitting range */
    XClearWindow(PxxGetDisplay(),PxxGetWindow());
    clear_range();
    (*do_plot)();
  } else if(strcmp(client_data,"Retry")==0) { /* reset profile data */
    XClearWindow(PxxGetDisplay(),PxxGetWindow());
    do_reset();
    (*do_plot)();
  } else if(strcmp(client_data,"Print")==0) {
    PxxExecPrintOut();
  } else if(strcmp(client_data,"Help")==0) {
    if(do_help!=NULL) (*do_help)();
  }

  return;
}

#define MAX_ARG 100
#define	CHAR_SIZE_DEFAULT_RESOURCE	128

/* display for X11 */
void star_PxxDisplayAll(argc,argv, 
	app_class,width,height,plt_fnct,exit_fnct,help_fnct)
/*
display a window in X11 in the default format
1997/10/12 T.Handa ver.1.00
1997/12/17 T.Handa ver.1.00 revise for baseline
1998/ 5/ 6 T.Handa ver.1.02 (debug)
*/
int argc;
char *argv[];
char app_class[];	/* name of this application class(for .Xresource) */
double width,height;
void (*plt_fnct)(),(*exit_fnct)(),(*help_fnct)();
{
  Widget  toplevelWDG, main_windowWDG, drawing_areaWDG, menubarWDG, mbBtn[9];
  Arg     args[10];
  int     ii;
  GC      graphic_context;

  int	mainwin_x,mainwin_y;

  int i_btn_id;
  int i_order_submenu;

  XtAppContext    app_context;
  Arg fixed_res_args[MAX_ARG];     /* resource arguments */
  Cardinal        fixed_res_n_args;
  static char	window_w[CHAR_SIZE_DEFAULT_RESOURCE],
    window_h[CHAR_SIZE_DEFAULT_RESOURCE];
  static String   fb_res[] = {
    "*mainWindow.width:        500 ",
    "*mainWindow.height:       300 ",
    "*drawArea.background:	white",
    "*menubar.topAttachment:	ATTACH_FORM",
    "*menubar.rightAttachment:	ATTACH_FORM",
    "*menubar.leftAttachment:	ATTACH_FORM",
    NULL};
/* addendum for revision */
  static char psfile_name[]="";
  static char prt_name[]="";
  Widget pulldownWDG,gpb[10];

/* Change initial window size */
  mainwin_x=(int)(width*CM2INCH*DVI_SCALE+0.5);
  mainwin_y=(int)(height*CM2INCH*DVI_SCALE+0.5);
  if(mainwin_x>0 && mainwin_x<9999999) {
    sprintf(window_w,"*mainWindow.width: %8d",mainwin_x);
    fb_res[0]=window_w;
  }
  if(mainwin_y>0 && mainwin_y<9999999) {
    sprintf(window_h,"*mainWindow.height:%8d",mainwin_y);
    fb_res[1]=window_h;
  }

/*	PxxView_width=width; */
/*	PxxView_height=height; */
/*	PxxView_printer[0]=(char)NULL; */	/* set default printer */
/*	PxxView_psfile[0]=(char)NULL;*/	/* set temporaly file */
/* addendum for revision */
  PxxPutWindowSizeInPixel(width,height);
  PxxSetPrinterName(prt_name);
  PxxSetPSFName(psfile_name);


/* printer assigne by the command option */
  for(ii=1;ii<argc;ii++) {
    if(strcmp(argv[ii],"-P")==0) PxxSetPrinterName(argv[ii+1]);
    else if(argv[ii][0]=='-' && argv[ii][1]=='P')
      PxxSetPrinterName(&(argv[ii][2]));
  }

/* output PS filename by the command option */
  for(ii=1;ii<argc;ii++) {
    if(strcmp(argv[ii],"-F")==0) PxxSetPSFName(argv[ii+1]);
    else if(argv[ii][0]=='-' && argv[ii][1]=='F')
      PxxSetPSFName(&(argv[ii][2]));
  }

/* set plot, exit, and help functions */
  PxxSetPlotFunction(plt_fnct);
  PxxSetExitFunction(exit_fnct);
  PxxSetHelpFunction(help_fnct);
/* addendum for the revision */
  do_plot=plt_fnct;
  do_exit=exit_fnct;
  do_help=help_fnct;

/* create X11 */
  fixed_res_n_args=0;
  toplevelWDG=XtAppInitialize(&app_context,app_class, NULL, 0, &argc, argv, fb_res,fixed_res_args,fixed_res_n_args);

  main_windowWDG=XmCreateMainWindow(toplevelWDG,"mainWindow",NULL,(Cardinal)NULL);
  XtManageChild(main_windowWDG);

/* create menubar */
  menubarWDG=XmCreateMenuBar(main_windowWDG,"menubar",NULL,(Cardinal)NULL);
  XtManageChild(menubarWDG);

  i_btn_id=0;
  mbBtn[i_btn_id]=XmCreateCascadeButton(menubarWDG,"help", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, star_PxxPushButtonActions,"Help");
  i_btn_id++;

  mbBtn[i_btn_id]=XmCreateCascadeButton(menubarWDG,"next", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, (XtCallbackProc)star_PxxPushButtonActions,"Exit");
  i_btn_id++;
  mbBtn[i_btn_id]=XmCreateCascadeButton(menubarWDG,"fit", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, (XtCallbackProc)star_PxxPushButtonActions,"Fit");
  i_btn_id++;
  mbBtn[i_btn_id]=XmCreateCascadeButton(menubarWDG,"reshow", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, (XtCallbackProc)star_PxxPushButtonActions,"Reshow");
  i_btn_id++;
  mbBtn[i_btn_id]=XmCreateCascadeButton(menubarWDG,"order", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, (XtCallbackProc)star_PxxPushButtonActions,"Order");
  i_order_submenu=i_btn_id;
  i_btn_id++;
  mbBtn[i_btn_id]=XmCreateCascadeButton(menubarWDG,"clear", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, (XtCallbackProc)star_PxxPushButtonActions,"Clear");
  i_btn_id++;
  mbBtn[i_btn_id]=XmCreateCascadeButton(menubarWDG,"retry", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, (XtCallbackProc)star_PxxPushButtonActions,"Retry");
  i_btn_id++;
  mbBtn[i_btn_id]=XmCreateCascadeButton(menubarWDG,"print", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, (XtCallbackProc)star_PxxPushButtonActions,"Print");
  i_btn_id++;
  mbBtn[i_btn_id]=XmCreateCascadeButton(menubarWDG,"quit", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, (XtCallbackProc)star_PxxPushButtonActions,"Quit");
  i_btn_id++;
  XtManageChildren(mbBtn,i_btn_id);

  XtSetArg(args[0],XmNmenuHelpWidget,(XtArgVal)mbBtn[0]);
  XtSetValues(menubarWDG,args,1);

/* create the pulldown menu for order of the polynomial fit*/ 
  pulldownWDG=XmCreatePulldownMenu(menubarWDG,"OrderMenu",NULL,0);

  XtSetArg(args[0],XmNsubMenuId,pulldownWDG);
  XtSetValues(mbBtn[i_order_submenu],args,1);

  gpb[0]=XmCreatePushButtonGadget(pulldownWDG,"order=1",args,0);
  gpb[1]=XmCreatePushButtonGadget(pulldownWDG,"order=2",args,0);
  gpb[2]=XmCreatePushButtonGadget(pulldownWDG,"order=3",args,0);
  gpb[3]=XmCreatePushButtonGadget(pulldownWDG,"order=4",args,0);

  XtAddCallback(gpb[0],XmNactivateCallback,OrderSelectActions,"1");
  XtAddCallback(gpb[1],XmNactivateCallback,OrderSelectActions,"2");
  XtAddCallback(gpb[2],XmNactivateCallback,OrderSelectActions,"3");
  XtAddCallback(gpb[3],XmNactivateCallback,OrderSelectActions,"4");

  XtManageChildren(gpb,4);

/* create drawing area */
  drawing_areaWDG=XmCreateDrawingArea(main_windowWDG,"drawArea",args,0);
  XtManageChild(drawing_areaWDG);

/* assign X11 alignement */
  XmMainWindowSetAreas(main_windowWDG,menubarWDG,NULL,NULL,NULL,drawing_areaWDG);

/* realize X11 */
  XtRealizeWidget(toplevelWDG);

  graphic_context=XCreateGC(XtDisplay(drawing_areaWDG),XtWindow(drawing_areaWDG),0,0);

  XtAddCallback(drawing_areaWDG,XmNexposeCallback,(XtCallbackProc)PxxExecDraw,graphic_context);

  XtAddCallback(drawing_areaWDG,XmNresizeCallback,(XtCallbackProc)PxxExecDraw,graphic_context);

/* action at click */
  graphic_context=XCreateGC(XtDisplay(drawing_areaWDG),XtWindow(drawing_areaWDG),0,0);
  XtAddCallback(drawing_areaWDG,XmNinputCallback,ClickActions,graphic_context);
  XtAddCallback(drawing_areaWDG,XmNexposeCallback,ClickActions,graphic_context);

/* keep display */
  XtAppMainLoop(app_context);
}

/*--------from Pxplib--------------------*/
/* X11ޤPostScriptϤμ¹ */
void star_PxpDisplayAll(argc,argv, output, filename,
	app_class,width,height,plt_fnct,exit_fnct,help_fnct)
/*
create an X11 window or a PostScript file
1997/10/14 T.Handa ver.1.00
1997/12/02 T.Handa ver.1.00 modify for display
*/
int argc;
char *argv[];
int output;
char *filename;	/* if "-" or NULL-string, stdout; if filename==NULL, temporaly work file */
char app_class[];	/* application class used in .Xresource */
double width,height;
void (*plt_fnct)(),(*exit_fnct)(),(*help_fnct)();
{
  void (*plot_function)();

  plot_function=plt_fnct;
/*  PxpView_output=output;*/
  PxpPutOutputMode(output);

  switch(output) {
  case X11WIN:
    star_PxxDisplayAll(argc,argv, app_class,width,height,plt_fnct,exit_fnct,help_fnct);
    break;
  case POSTSCRIPT:
    PpsInitialize(filename,width,height);
    plot_function();
    PpsPrintPage();
    PpsDoPrint();
  }

  return;
}

/*---------------------------------*/

int options(argc,argv, w_height,w_width,x_unit,x_range,T_range,
	range_unit,n_range,fit_range,f_order,rangeclear_flag,gui_flag)
/*
get parameters from options
*/
int argc;
char *argv[];
double	*w_height, *w_width;	/* window size(cm) */
int	*x_unit;
RANGE	*x_range,	/* channel range */
	*T_range;	/* Ta range */
int	*range_unit;	/* fit range unit */
int	*n_range;	/* number of ranges */
RANGE	fit_range[MAX_UNIT][MAX_RANGES];	/* fit range */
int	*f_order;	/* order for fitting polynomical */
int	*rangeclear_flag;
int	*gui_flag;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *token;
  int opt_pos;
  char buffer[MAX_PRM_RECORD_LENGTH];

  int flag;
  double window_x,window_y;

  char *got_value;

/* default values */
  *w_height=15.0;	*w_width=20.0;
  *x_unit=X_CHANNEL;
  (*x_range).min=0.0;	(*x_range).max=0.0;
  (*T_range).min=0.0;	(*T_range).max=0.0;
  n_range[0]=0; n_range[1]=0;
  *f_order=1;	/* the order of polynomical to use baseline fit */
  *rangeclear_flag=FLAG_NO; /* keep the previous baseline flags? yes */
  *gui_flag= -1;	/* GUI? no */

  window_x=0.0;
  window_y=0.0;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"size");	key_type[n_key-1]=1; /* No.0 */
  n_key=AddNewWord(keyword,"ch");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"v");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"lsr");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"x_unit");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"T");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"GUI");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"gui");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"nogui");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"CUI");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"cui");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"mute");	key_type[n_key-1]=0; /* No.11*/
  n_key=AddNewWord(keyword,"order");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"ch_fit");	key_type[n_key-1]=1; /* No.13*/
  n_key=AddNewWord(keyword,"ch_fit_range");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"v_fit");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"v_fit_range");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"clear");	key_type[n_key-1]=0; /* No.17*/
  n_key=AddNewWord(keyword,"keep");	key_type[n_key-1]=0;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* window size */
  scan_flag[0]=0;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(buffer,got_value);
    token=strtok(buffer,"xX ,\t\n");
    if(token!=NULL) {
      window_x=atof(token);
      token=strtok(NULL," ,\t\n");
      if(token!=NULL) window_y=atof(token);
    }
    *w_height=window_y;
    *w_width=window_x;
  }

/* ch,v,lsr,x_unit */
  scan_flag[0]=1;	scan_flag[1]=2; scan_flag[2]=3; scan_flag[3]=4;
  n_scan=4;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    if(strcmp(keyword[opt_pos],"v")==0
       || strcmp(keyword[opt_pos],"lsr")==0
       || strcmp(value[opt_pos],"velocity")==0
       || strcmp(value[opt_pos],"lsr")==0
       || strcmp(value[opt_pos],"LSR")==0)
      *x_unit=X_VELOCITY;
    else if(strcmp(keyword[opt_pos],"ch")==0
	    || strcmp(value[opt_pos],"channel")==0
	    || strcmp(value[opt_pos],"ch")==0)
      *x_unit=X_CHANNEL;
    StarOptionRange(value[opt_pos],x_range);
  } 

/* y_range */
  scan_flag[0]=5;
  n_scan=1;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    strcpy(buffer,value[opt_pos]);
    StarOptionRange(buffer,T_range);
  } 

/* GUI */
  scan_flag[0]=6; scan_flag[1]=7;
  scan_flag[2]= -8; scan_flag[4]= -9;
  scan_flag[5]= -10; scan_flag[6]= -11; 
  n_scan=7;
  flag=StarOptionFlag(n_scan,scan_flag,key_flag);
  if(flag!=0) *gui_flag=flag;

/* order */
  scan_flag[0]=12;
  n_scan=1;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    *f_order=atoi(value[opt_pos]);
  } 

/* fitting ranges in channel */
  scan_flag[0]=13;
  scan_flag[1]=14;
  n_scan=2;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    StarOptionRanges(value[opt_pos],&(n_range[0]),fit_range[0]);
  }

/* fitting ranges in velocity */
  scan_flag[0]=15;
  scan_flag[1]=16;
  n_scan=2;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    StarOptionRanges(value[opt_pos],&(n_range[1]),fit_range[1]);
  }

/* flag keep? */
  scan_flag[0]= 17; scan_flag[1]= -18;
  n_scan=2;
  flag=StarOptionFlag(n_scan,scan_flag,key_flag);
  if(flag!=0) *rangeclear_flag=flag;

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

/* set return values */

  return(0);
}


/* Main Body of process */

void write_profile(profile,sdb_flag,sdb_arg)
PROFILE *profile;
int sdb_flag;
char *sdb_arg;
{
  int	ii;
  HISTORY history;

  strcpy(history.step,"BSFIT");
  for(ii=0;ii<=GLf_order;ii++) {
    sprintf(history.prm[ii],"c%1d=%8.6e",ii,coefficent[ii]);
  }
  history.size = ii;

  StarAddHistory(profile,&history);
  StarProfileOut(profile,sdb_flag,sdb_arg);

  return;
}

void read_profile()
{
  static int i_file=0;
  int n_file;
  int err_code;

/* get global valuables */
  n_file=GLn_file;


/* write the previous profile before reading */
  if(i_file>0) {
    do_subtract_baseline();
    write_profile(&GLprofile,GLsdb_flag,GLsdb_arg);
  }

/* if no more, then end */
  if(i_file>=n_file) {
    StarPipeEndOut();
    exit(0);
  }

/* get a new profile */
  while(1) {
    err_code=StarProfileIn(&GLprofile,&GLsdb_flag,GLsdb_arg);
    if(err_code!=0) {
      fprintf(stderr,"baseline:fail to ProfileIn\n");
      exit(1);
    }

/* update the present count */
    i_file++;

    if(GLsdb_flag==GOOD_SCAN) break;

/* out for non-GOOD_SCAN */
    StarProfileOut(&GLprofile,GLsdb_flag,GLsdb_arg);
  }

/* preset baseline flag clear */
  if(GLrangeclear_flag==FLAG_YES)
    StarProfileClearBaselineFlag(&GLprofile,UNFLAGGED);

/* copy a profile to a backup and a work profile */
  StarCopyProfile(&backup_profile,&GLprofile);
  StarCopyProfile(&baseline_profile,&GLprofile);
  process_status=NOT_PROCESSED;

/* convert fit ranges in fit_ch_range */
  all_range_in_ch();

  return;
}

void actual_plot()
{
  static int read_flag=1;
  RANGE x_range,y_range;
  int x_unit;

  BOX_RANGE plot_box;

  static PxpPoint win_size;

  int output_mode;
  int window_x,window_y,window_h,window_w;

/* copy from global valuables */
  x_range.min=GLx_range.min;
  x_range.max=GLx_range.max;
  x_unit=GLx_unit;
  y_range.min=GLy_range.min;
  y_range.max=GLy_range.max;

/* read profile if the first */
  if(read_flag>0) {
    read_profile();
    read_flag= -1;
  }

/* plotsize auto scaling */
  PxpGetOutputMode(&output_mode);
  if(output_mode==X11WIN) {
    PxxPresentWindowSize(&window_x,&window_y,&window_w,&window_h);
    win_size.x=PxxGetPlotSpanX(window_w);
    win_size.y=PxxGetPlotSpanY(window_h);
  }

  plot_box.x.min=win_size.x*0.1;
  plot_box.y.min=win_size.y*0.1;
  plot_box.x.max=plot_box.x.min+win_size.x*0.8;
  plot_box.y.max=plot_box.y.min+win_size.y*0.8;

/* profile in standard form */
  local_StarPlotProfileStd(&GLprofile,x_unit,x_range,y_range,plot_box,1,1);

/* plot fitted curve */
  switch(process_status){ 
  case FIT_EXECUTED:
    plot_baseline();
  case RANGE_ASSIGNED:
    PxpSetColor("cyan");
    StarPlotProfileBaselineRange(&GLprofile,GLx_unit,GLphys_box,2);
    PxpSetColor("black");
  }

  return;
}

void help_function()
{
  fprintf(stderr,"help should be shown.\n");
  return;
}

int main(argc,argv)
int argc;
char *argv[];
{
  int err_code;
  int i_file,n_file;

  double	w_height, w_width;	/* window size (cm) */
  int	x_unit;
  RANGE	x_range,y_range;	/* for display */
  int range_unit;
  int f_order;
  int rangeclear_flag,gui_flag;

  char id_str[BUFFER_SIZE];

/* check option */
  err_code=options(argc,argv, &w_height,&w_width,&x_unit,&x_range,&y_range,&range_unit,GLn_range,GLfit_range,&f_order,&rangeclear_flag,&gui_flag);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

/* set global parameters */
  GLx_unit=x_unit;
  GLx_range.min=x_range.min;
  GLx_range.max=x_range.max;
  GLy_range.min=y_range.min;
  GLy_range.max=y_range.max;

  if(x_range.min==x_range.max)
    GLautorange_x= 1;
  else
    GLautorange_x= -1;
  if(y_range.min==y_range.max)
    GLautorange_y= 1;
  else
    GLautorange_y= -1;

  if(GLn_range[0]+GLn_range[1]==0)
    GLautorange_fit= 1;
  else
    GLautorange_fit= -1;

  GLrange_unit=range_unit;
  GLf_order=f_order;
  GLrangeclear_flag=rangeclear_flag;

/* multi-data header in */
  StarPipeHeadIn(id_str,&n_file);
  GLn_file=n_file;

  StarPipeHeadOut(n_file);

  if(gui_flag>0) {
/* GUI */
    star_PxpDisplayAll(argc,argv, 
      X11WIN,NULL,"baseline",w_width,w_height,
      actual_plot,read_profile,help_function);
  } else {
/* CUI processing */
    for(i_file=0;i_file<n_file;i_file++) {
      err_code=StarProfileIn(&GLprofile,&GLsdb_flag,GLsdb_arg);
      if(GLsdb_flag==GOOD_SCAN) {
	if(rangeclear_flag==FLAG_YES)
	  StarProfileClearBaselineFlag(&GLprofile,UNFLAGGED);
	StarCopyProfile(&backup_profile,&GLprofile);
	StarCopyProfile(&baseline_profile,&GLprofile);
	all_range_in_ch();
	do_fit();
	do_subtract_baseline();
      }
      write_profile(&GLprofile,GLsdb_flag,GLsdb_arg);
    }
    StarPipeEndOut();
    exit(0);
  }
  exit(0);
}
