/*
mathematical process for two cube datasets
A+B, A-B, a*A+b*B, A*B, A/B
where A is in stream, B is load from a file,
a and b are paramaters.

2002/5/28 T.Handa : ver.3.00
2006/5/22 T.Handa : ver.3.01 (debug) core dump via ssh; 2D array should be static
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"bicubprocess.prm"
#define N_KEY 30

#define BiCubProcess_ADD 1
#define BiCubProcess_SUB 2
#define BiCubProcess_LINEAR 11
#define BiCubProcess_PROCUCT 21
#define BiCubProcess_RATIO 31
#define BiCubProcess_RATIO_CLIP 32

#define DEBUG
#undef DEBUG

int options(argc,argv,base_cubname,cub_math,factors,clip_level,rms_flag)
/*
get parameters from options
*/
int argc;
char *argv[];
char *base_cubname;
int *cub_math;
double *factors;
double *clip_level;
int *rms_flag;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;
  RANGE tmp_range;
  int l_str;

/* default values */
  strcpy(base_cubname,"");
  *cub_math=BiCubProcess_RATIO_CLIP;
  factors[0]=1.0; factors[1]=1.0;
  *clip_level=3.0;
  *rms_flag=FLAG_YES;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"base_cube");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"base_cubname");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"process");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"math_type");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"factor");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"clip_level");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* base cubname */
  n_scan=2;
  scan_flag[0]=0;
  scan_flag[1]=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(base_cubname,got_value);
  }

/* mathematic type for two cube process */
  n_scan=2;
  scan_flag[0]=2;
  scan_flag[1]=3;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strcmp(got_value,"ratio")==0)
      *cub_math=BiCubProcess_RATIO;
    else if(strcmp(got_value,"ratio_clip")==0)
      *cub_math=BiCubProcess_RATIO_CLIP;
    else if(strcmp(got_value,"add")==0)
      *cub_math=BiCubProcess_ADD;
    else if(strncmp(got_value,"sub",3)==0)
      *cub_math=BiCubProcess_SUB;
    else if(strcmp(got_value,"product")==0 || strcmp(got_value,"by")==0)
      *cub_math=BiCubProcess_PROCUCT;
    else if(strcmp(got_value,"linear")==0)
      *cub_math=BiCubProcess_LINEAR;
  }

/* factors */
  n_scan=1;
  scan_flag[0]=4;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    StarOptionRange(got_value,&tmp_range);
    factors[0]=tmp_range.min;
    factors[1]=tmp_range.max;
  }

/* clip level */
  n_scan=1;
  scan_flag[0]=5;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    l_str=strlen(got_value);
    if(strcmp(&(got_value[l_str-3]),"rms")==0) {
      *rms_flag=FLAG_YES;
    } else {
      *rms_flag=FLAG_NO;
    }
    *clip_level=atof(got_value);
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return 0;
}

void StarCubeMathProcess(cube_add,cube_base,math_type,factors)
CUBE *cube_add; /* <i/o> */
CUBE *cube_base; /* <in> */
int math_type;
double *factors; /* <in> two factors for Liner combination of two cubes, or clip level */
{
  int ix,iy,iz;
  int mx,my,mz;
  double ta_base,ta_add;
  int valid_flag_base,valid_flag_add;
  float dummy_pixel;

  mx=(*cube_base).head.axis.x.size;
  my=(*cube_base).head.axis.y.size;
  mz=(*cube_base).head.axis.z.size;

  dummy_pixel=(*cube_add).head.data_value.dummy;

  for(ix=0;ix<mx;ix++) {
    for(iy=0;iy<my;iy++) {
      for(iz=0;iz<mz;iz++) {
	ta_base=StarGetCubePixelData(cube_base,ix,iy,iz,&valid_flag_base);
	ta_add=StarGetCubePixelData(cube_add,ix,iy,iz,&valid_flag_add);
	if(valid_flag_base==FLAG_YES && valid_flag_add==FLAG_YES) {
	  switch(math_type) {
	  case BiCubProcess_RATIO_CLIP:
	    if(ta_base>=factors[0])
	      ta_add=ta_add/ta_base;
	    else {
	      ta_add=dummy_pixel;
	    }
	    break;
	  case BiCubProcess_RATIO:
	    if(ta_base!=0.0)
	      ta_add=ta_add/ta_base;
	    else {
	      ta_add=dummy_pixel;
	    }
	    break;
	  case BiCubProcess_ADD:
	    ta_add=ta_add+ta_base;
	    break;
	  case BiCubProcess_SUB:
	    ta_add=ta_add-ta_base;
	    break;
	  case BiCubProcess_PROCUCT:
	    ta_add=ta_add*ta_base;
	    break;
	  case BiCubProcess_LINEAR:
	    ta_add=ta_base*factors[0]+ta_add*factors[1];
	  }
	  StarPutCubePixelData(cube_add,ix,iy,iz,ta_add);
	} else {
	  StarPutCubePixel(cube_add,ix,iy,iz,dummy_pixel);
	}
      }
    }
  }

  return;
}

void ModifyDatavalue(dv_add,dv_base,math_type)
DATA_VALUE *dv_add; /* <i/o> */
DATA_VALUE *dv_base; /* <in> */
int math_type;
{
  char buffer[BUFFER_SIZE];

#ifdef DEBUG
  fprintf(stderr,"%s and %s, mathtype=%d\n",(*dv_base).type,(*dv_add).type,math_type);
#endif

  if(strcmp((*dv_base).type,(*dv_add).type)==0) {
    switch(math_type) {
    case BiCubProcess_RATIO: case BiCubProcess_RATIO_CLIP:
      strcpy((*dv_add).type,"ratio");
      break;
    case BiCubProcess_PROCUCT:
      sprintf(buffer,"(%s)^2",(*dv_add).type);
      buffer[8]=(char)NULL;
      strcpy((*dv_add).type,buffer);
      break;
    }
  } else {
    switch(math_type) {
    case BiCubProcess_RATIO: case BiCubProcess_RATIO_CLIP:
      sprintf(buffer,"(%s)/(%s)",(*dv_add).type,(*dv_base).type);
      buffer[8]=(char)NULL;
      strcpy((*dv_add).type,buffer);
      break;
    case BiCubProcess_PROCUCT:
      sprintf(buffer,"(%s)*(%s)",(*dv_add).type,(*dv_base).type);
      buffer[8]=(char)NULL;
      strcpy((*dv_add).type,buffer);
      break;
    }
  }

/* unit */
  if(strcmp((*dv_base).unit,(*dv_add).unit)==0) {
    switch(math_type) {
    case BiCubProcess_RATIO: case BiCubProcess_RATIO_CLIP:
      strcpy((*dv_add).unit,"");
      break;
    case BiCubProcess_PROCUCT:
      sprintf(buffer,"(%s)^2",(*dv_add).unit);
      buffer[8]=(char)NULL;
      strcpy((*dv_add).unit,buffer);
      break;
    }
  } else {
    switch(math_type) {
    case BiCubProcess_RATIO: case BiCubProcess_RATIO_CLIP:
      sprintf(buffer,"(%s)/(%s)",(*dv_add).unit,(*dv_base).unit);
      buffer[8]=(char)NULL;
      strcpy((*dv_add).type,buffer);
      break;
    case BiCubProcess_PROCUCT:
      sprintf(buffer,"(%s)*(%s)",(*dv_add).unit,(*dv_base).unit);
      buffer[8]=(char)NULL;
      strcpy((*dv_add).type,buffer);
      break;
    }
  }

  return;
}

void main_filename(mainname,filename)
char *mainname; /* <out> */
char *filename; /* <in> */
{
  int ii,l_str,i_top;

  strcpy(mainname,StarFileMainName(filename));

  l_str=strlen(mainname);
  i_top=l_str-4;
  if(i_top<0) i_top=0;
  for(ii=l_str-1;ii>=i_top;ii--) {
    if(mainname[ii]=='.') {
      mainname[ii]=(char)NULL;
      break;
    }
  }
  return;
}

void AddCubratioHistory(add_head,base_head,math_type,factors)
CUBE_HEAD *add_head; /* <i/o> */
CUBE_HEAD *base_head; /* <in> */
int math_type; /* <in> */
double *factors; /* <in> */
{
  static char main_cubname[2][MAX_FILE_NAME_LENGTH];
  HISTORY	history;
  char buffer[80];
  int i_prm;

  main_filename(main_cubname[0],(*base_head).file_name);
  if(strlen(main_cubname[0])>16) main_cubname[0][16]=(char)NULL;
  main_filename(main_cubname[1],(*add_head).file_name);
  if(strlen(main_cubname[1])>16) main_cubname[1][16]=(char)NULL;

  strcpy(history.step,"cubmath");

  i_prm=0;

/* process */
  switch(math_type) {
  case BiCubProcess_RATIO:
    strcpy(buffer,"A/B");
    break;
  case BiCubProcess_RATIO_CLIP:
    strcpy(buffer,"A/B clipped");
    break;
  case BiCubProcess_ADD:
    strcpy(buffer,"A+B");
    break;
  case BiCubProcess_SUB:
    strcpy(buffer,"A-B");
    break;
  case BiCubProcess_PROCUCT:
    strcpy(buffer,"A*B");
    break;
  case BiCubProcess_LINEAR:
    strcpy(buffer,"a*A+b*B");
    break;
  }
  strcpy(history.prm[i_prm],buffer);
  i_prm++;

/* base and add cube */
  strcpy(buffer,"A and B are");
  buffer[16]=(char)NULL;
  strcpy(history.prm[i_prm],buffer);
  i_prm++;

  strncpy(buffer,main_cubname[0],15);
  buffer[16]=(char)NULL;
  strcpy(history.prm[i_prm],buffer);
  i_prm++;
  strncpy(buffer,main_cubname[1],15);
  buffer[16]=(char)NULL;
  strcpy(history.prm[i_prm],buffer);
  i_prm++;

/* factors */
  if(math_type==BiCubProcess_LINEAR) {
    sprintf(buffer,"a=%f",factors[0]);
    buffer[16]=(char)NULL;
    strcpy(history.prm[i_prm],buffer);
    i_prm++;
    sprintf(buffer,"b=%f",factors[1]);
    buffer[16]=(char)NULL;
    strcpy(history.prm[i_prm],buffer);
    i_prm++;
  } else if(math_type==BiCubProcess_RATIO_CLIP) {
    sprintf(buffer,"c=%f",factors[0]);
    buffer[16]=(char)NULL;
    strcpy(history.prm[i_prm],buffer);
    i_prm++;
  }

  history.size=i_prm;

  StarAddHistory4Cube(add_head,&history);
}

int CheckCubeHead(cube1,cube2,math_type)
CUBE_HEAD *cube1; /* <in> */
CUBE_HEAD *cube2; /* <in> */
int math_type; /* <in> */
{
  int ret_code;

  ret_code=0;

/* axis size */
  if((*cube1).axis.x.size!=(*cube2).axis.x.size) {
    fprintf(stderr,"unmatch x-size\n");
  }
  if((*cube1).axis.y.size!=(*cube2).axis.y.size) {
    fprintf(stderr,"unmatch y-size\n");
  }
  if((*cube1).axis.z.size!=(*cube2).axis.z.size) {
    fprintf(stderr,"unmatch z-size\n");
  }

/* axis type */
  if(strcmp((*cube1).axis.x.type,(*cube2).axis.x.type)!=0) {
    fprintf(stderr,"unmatch x-type\n");
  }
  if(strcmp((*cube1).axis.y.type,(*cube2).axis.y.type)!=0) {
    fprintf(stderr,"unmatch y-type\n");
  }
  if(strcmp((*cube1).axis.z.type,(*cube2).axis.z.type)!=0) {
    fprintf(stderr,"unmatch z-type\n");
  }

/* data unit */
  switch(math_type) {
  case BiCubProcess_ADD: case BiCubProcess_SUB:
    if(strcmp((*cube1).data_value.type,(*cube2).data_value.type)!=0) {
    fprintf(stderr,"unmatch data-type\n");
    }
  }

  return(ret_code);
}

int main(argc,argv)
int	argc;
char	*argv[];
{
  CUBE cube_base,cube_add;
  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];
  int err_code;
  int n_file;
  char id_str[BUFFER_SIZE];
  char base_cubname[MAX_FILE_NAME_LENGTH];
  int cube_mathtype;
  double factors[2];
  double clip_level;
  int rms_flag;

  int ii;

  FILE *fp_in;

/* get optional parameters */
  err_code=options(argc,argv,base_cubname,&cube_mathtype,factors,&clip_level,&rms_flag);

  fp_in=fopen(base_cubname,"r");
  if(fp_in==NULL) {
    fprintf(stderr,"**ERR** file %s is not found.\n",base_cubname);
    exit(99);
  }
  err_code=StarCubeHeadLoad(fp_in,&(cube_base.head));
  if(err_code!=0) {
    fprintf(stderr,"**ERR** failed to load header data for %s\n",base_cubname);
    exit(99);
  }
  cube_base.pixel=StarAllocCubePixel(&(cube_base.head));
  if(cube_base.pixel==NULL) {
    fprintf(stderr,"**ERR** cannot alloc pixel data for %s\n",base_cubname);
    exit(99);
  }
  err_code=StarCubePixelLoad(fp_in,&cube_base);
  if(err_code!=0) {
    fprintf(stderr,"**ERR** failed to load pixel data for %s\n",base_cubname);
    exit(99);
  }
  fclose(fp_in);
  main_filename(cube_base.head.file_name,base_cubname);
  

/* pipe header */
  StarPipeHeadIn(id_str,&n_file);

  StarPipeHeadOut(n_file);

/* .cub data */
  for(ii=0;ii<n_file;ii++) {

    StarCubeIn(&cube_add,&cdb_flag,cdb_arg);

    CheckCubeHead(&(cube_add.head),&(cube_base.head),cube_mathtype);

    if(cube_mathtype==BiCubProcess_RATIO_CLIP) {
      factors[0]=clip_level;
      if(rms_flag==FLAG_YES) 
	factors[0]=factors[0]*cube_base.head.data_value.rms;
    }

    StarCubeMathProcess(&cube_add,&cube_base,cube_mathtype,factors);

    ModifyDatavalue(&(cube_add.head.data_value),&(cube_base.head.data_value),cube_mathtype);
    StarCubeScanDatavalue(&cube_add);

    AddCubratioHistory(&(cube_add.head),&(cube_base.head),cube_mathtype,factors); 

    StarCubeOut(&cube_add,cdb_flag,cdb_arg);
    StarFreeCubePixel(&cube_add);
  }

  StarFreeCubePixel(&cube_base);

/* end tag of pipe */
  StarPipeEndIn(id_str);
  StarPipeEndOut();

  exit(0);
}
