/*
display color map of cub on X11 screen

2000/ 2/ 4 T.Handa : ver.3.00
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include <Xm/DrawingA.h>
#include <Xm/MainW.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#include <Xm/PushBG.h>

#include <Pxxlib.h>
#include <Pxplib.h>
#include <Ppslib.h>
#include <PxpGlib.h>

#include <StStdlib.h>
#include <StStdshow.h>
#include <StData.h>
#include <StProfileIO.h>
#include <StProfileShow.h>
#include <StProfileProc.h>
#include <StPipe.h>
#include <StCubeIO.h>
#include <StCubeShow.h>

#define	DEFAULT_PRMFILE	"cubmap.prm"
#define N_KEY 20

static CUBE GLcube;
static int GLcdb_flag;
static char GLcdb_arg[MAX_IDX_RECORD_LENGTH];
static int GLn_file=0;
static int GLpipeout= -1;		/* pipe out for the next command */
static RANGE GLx_range;
static RANGE GLy_range;

static char *GLimage_plane; /* image value quantized in 256 level */

extern void StarCube2RGBImage(char *, CUBE *,int, int, XY_SIZE_long, int);

/*-------Ext------*/
/*--------from Pxxlib--------------------*/
static void	(*do_plot)(), (*do_exit)(), (*do_help)();


/* define actions for buttons on the menu bar */
void local_StarPxxPushButtonActions(menubarWDG,client_data,call_data)
Widget	menubarWDG;
caddr_t	client_data,call_data;
/*
define actions for buttons on the menu bar
1997/10/12 T.Handa ver.1.00
1997/12/12 T.Handa ver.1.00 for display command
*/
{
  if(strcmp(client_data,"Exit")==0) {
    XClearWindow(PxxGetDisplay(),PxxGetWindow());
    if(do_exit!=NULL) (*do_exit)();
    if(do_plot!=NULL) (*do_plot)();
  }
  else if(strcmp(client_data,"Quit")==0) {
    exit(0);
  }
  else if(strcmp(client_data,"Print")==0)
    PxxExecPrintOut();
  else if(strcmp(client_data,"Help")==0)
    if(do_help!=NULL) (*do_help)();
}

#define MAX_ARG 100
#define	CHAR_SIZE_DEFAULT_RESOURCE	128

/* display on X11 */
void local_StarPxxDisplayAll(argc,argv, 
    app_class,width,height,plt_fnct,exit_fnct,help_fnct)
/*
create and plot an X11 window

2000/ 1/21 T.Handa ver.1.00
*/
int argc;
char *argv[];
char app_class[];	/* name of the application class (use in .Xresource) */
double width,height;
void (*plt_fnct)(),(*exit_fnct)(),(*help_fnct)();
{
  Widget  toplevelWDG, main_windowWDG, drawing_areaWDG, 
    manu_barWDG, mbBtn[4];
  Arg     args[10];
  int     ii;
  GC      graphic_context;

  int	width_in_dot,height_in_dot;
  int	mainwin_x,mainwin_y;

  XtAppContext    app_context;
  Arg fixed_res_args[MAX_ARG];     /* resource arguments */
  Cardinal        fixed_res_n_args;
  static char	window_w[CHAR_SIZE_DEFAULT_RESOURCE],
    window_h[CHAR_SIZE_DEFAULT_RESOURCE];
  static String   fb_res[] = {
    "*mainWindow.width:        500 ",
    "*mainWindow.height:       300 ",
    "*drawArea.background:	white",
    "*menubar.topAttachment:	ATTACH_FORM",
    "*menubar.rightAttachment:	ATTACH_FORM",
    "*menubar.leftAttachment:	ATTACH_FORM",
    NULL};
/* add for the display command only */
  static char psfile_name[]="";
  static char prt_name[]="";

/* Change initial window size */
  mainwin_x=(int)(width*CM2INCH*DVI_SCALE+0.5);
  mainwin_y=(int)(height*CM2INCH*DVI_SCALE+0.5);
  if(mainwin_x>0 && mainwin_x<9999999) {
    sprintf(window_w,"*mainWindow.width: %8d",mainwin_x);
    fb_res[0]=window_w;
  }
  if(mainwin_y>0 && mainwin_y<9999999) {
    sprintf(window_h,"*mainWindow.height:%8d",mainwin_y);
    fb_res[1]=window_h;
  }

/*	PxxView_width=width; */
/*	PxxView_height=height; */
/*	PxxView_printer[0]=NULL; */	/* set default printer */
/*	PxxView_psfile[0]=NULL;*/	/* set temporaly file */
/* add for the display command only */
  PxxPutWindowSizeInPixel(width,height);
  PxxSetPrinterName(prt_name);
  PxxSetPSFName(psfile_name);

/* printer name in argv */
  for(ii=1;ii<argc;ii++) {
    if(strcmp(argv[ii],"-P")==0) PxxSetPrinterName(argv[ii+1]);
    else if(argv[ii][0]=='-' && argv[ii][1]=='P')
      PxxSetPrinterName(&(argv[ii][2]));
  }

/* PS filename in argv */
  for(ii=1;ii<argc;ii++) {
    if(strcmp(argv[ii],"-F")==0) PxxSetPSFName(argv[ii+1]);
    else if(argv[ii][0]=='-' && argv[ii][1]=='F')
      PxxSetPSFName(&(argv[ii][2]));
  }

/* set plot, exit, and help functions */
  PxxSetPlotFunction(plt_fnct);
  PxxSetExitFunction(exit_fnct);
  PxxSetHelpFunction(help_fnct);
/* add for the display commnad only */
  do_plot=plt_fnct;
  do_exit=exit_fnct;
  do_help=help_fnct;

/* create X11 window */
  fixed_res_n_args=0;
  toplevelWDG=XtAppInitialize(&app_context,app_class, NULL, 0, &argc, argv,
			      fb_res,fixed_res_args,fixed_res_n_args);

  main_windowWDG=XmCreateMainWindow(toplevelWDG,"mainWindow",NULL,NULL);
  XtManageChild(main_windowWDG);

/* create menu bar */
  manu_barWDG=XmCreateMenuBar(main_windowWDG,"menubar",NULL,NULL);
  XtManageChild(manu_barWDG);

  mbBtn[0]=XmCreateCascadeButton(manu_barWDG,"help", args,0);
  XtAddCallback(mbBtn[0],XmNactivateCallback, (XtCallbackProc)local_StarPxxPushButtonActions,"Help");
  mbBtn[1]=XmCreateCascadeButton(manu_barWDG,"next", args,0);
  XtAddCallback(mbBtn[1],XmNactivateCallback, (XtCallbackProc)local_StarPxxPushButtonActions, "Exit");
  mbBtn[2]=XmCreateCascadeButton(manu_barWDG,"quit", args,0);
  XtAddCallback(mbBtn[2],XmNactivateCallback, (XtCallbackProc)local_StarPxxPushButtonActions, "Quit");
  mbBtn[3]=XmCreateCascadeButton(manu_barWDG,"print", args,0);
  XtAddCallback(mbBtn[3],XmNactivateCallback, (XtCallbackProc)local_StarPxxPushButtonActions, "Print");
  XtManageChildren(mbBtn,4);

  XtSetArg(args[0],XmNmenuHelpWidget,(XtArgVal)mbBtn[0]);
  XtSetValues(manu_barWDG,args,1);

/* create data ploting area */
  drawing_areaWDG=XmCreateDrawingArea(main_windowWDG,"drawArea",args,0);
  XtManageChild(drawing_areaWDG);

/* set alinement of the componets in a window */
  XmMainWindowSetAreas(main_windowWDG,manu_barWDG,NULL,NULL,NULL,drawing_areaWDG);

/* realize window */
  XtRealizeWidget(toplevelWDG);

  graphic_context=XCreateGC(XtDisplay(drawing_areaWDG),XtWindow(drawing_areaWDG),0,0);

  XtAddCallback(drawing_areaWDG,XmNexposeCallback,(XtCallbackProc)PxxExecDraw,graphic_context);

  XtAddCallback(drawing_areaWDG,XmNresizeCallback,(XtCallbackProc)PxxExecDraw,graphic_context);

/* keep showing */
  XtAppMainLoop(app_context);
}

/*--------from Pxplib--------------------*/
/* execute X11 display or PostScript output */
void local_StarPxpDisplayAll(argc,argv, output, filename,
	app_class,width,height,plt_fnct,exit_fnct,help_fnct)
/*
create the standared X11 window or PostScripte file

1997/10/14 T.Handa ver.1.00
1997/12/02 T.Handa ver.1.00 for the display command
*/
int argc;
char *argv[];
int output;
char *filename;	/* stdout for - or NULL, workfile for filename==NULL */
char app_class[];	/* name of the application class (use in .Xresource) */
double width,height;
void (*plt_fnct)(),(*exit_fnct)(),(*help_fnct)();
{
  void (*plot_function)();

  plot_function=plt_fnct;
/*	PxpView_output=output;*/
  PxpPutOutputMode(output);

  switch(output) {
  case X11WIN:
    local_StarPxxDisplayAll(argc,argv, app_class,width,height,plt_fnct,exit_fnct,help_fnct);
    break;
  case POSTSCRIPT:
    PpsInitialize(filename,width,height);
    plot_function();	/* execute the user-defined plot function */
    PpsPrintPage();
    PpsDoPrint();
  }
}

/*---------------------------------*/

int options(argc,argv, w_height,w_width,x_range,y_range,z_range,pipe_flag)
/*
get parameters from options
*/
int argc;
char *argv[];
double	*w_height, *w_width;	/* size of the window(cm) */
RANGE	*x_range,*y_range,*z_range;
int	*pipe_flag;
{
  int ii;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  int correction_flag;
  char *token;
  int opt_pos;
  char buffer[MAX_PRM_RECORD_LENGTH];

  int tmp_flag;
  double window_x,window_y;

  char *got_value;

/* default values */
  *w_height=15.0;	*w_width=20.0;
  (*x_range).min=0.0;	(*x_range).max=0.0;
  (*y_range).min=0.0;	(*y_range).max=0.0;
  *pipe_flag= -1;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"size");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"x");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"x_range");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y_range");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z_range");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"pipe");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"-");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"nopipe");	key_type[n_key-1]=0;


  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* window size */
  scan_flag[0]=0;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=NULL) {
    strcpy(buffer,got_value);
    token=strtok(buffer,"xX ,\t\n");
    if(token!=NULL) {
      window_x=atof(token);
      token=strtok(NULL," ,\t\n");
      if(token!=NULL) window_y=atof(token);
    }
    *w_height=window_y;
    *w_width=window_x;
  }

/* x_range */
  scan_flag[0]=1; scan_flag[1]=2;
  n_scan=2;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    strcpy(buffer,value[opt_pos]);
    StarOptionRange(buffer,x_range);
  } 

/* y_range */
  scan_flag[0]=3; scan_flag[1]=4;
  n_scan=2;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    strcpy(buffer,value[opt_pos]);
    StarOptionRange(buffer,y_range);
  } 

/* z_range */
  scan_flag[0]=5; scan_flag[1]=6;
  n_scan=2;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    strcpy(buffer,value[opt_pos]);
    StarOptionRange(buffer,z_range);
  }

/* pipe */
  scan_flag[0]=7; scan_flag[1]=8; scan_flag[2]= -9;
  n_scan=3;
  tmp_flag=StarOptionFlag(n_scan,scan_flag,key_flag);
  if(tmp_flag!=0) *pipe_flag=tmp_flag;

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);
/* set return values */

  return(0);
}


/* Main Body of process */

void read_cube()
{
  static int i_file=0;
  int n_file,n_act_file,pipeout;
  int err_code;

/* copy from global valuables */
  n_file=GLn_file;
  pipeout=GLpipeout;

  n_act_file=0;
  while(1) {
    if(i_file>=n_file) {
      if(pipeout>0) StarPipeEndOut();
      exit(0);
    }

    err_code=StarCubeIn(&GLcube,&GLcdb_flag,GLcdb_arg);
    if(err_code!=0) {
      fprintf(stderr,"fail to CubeIn\n");
      exit(1);
    }
    i_file++;

    if(pipeout>0)
      StarCubeOut(&GLcube,GLcdb_flag,GLcdb_arg);

    if(GLcdb_flag==GOOD_SCAN || GLcdb_flag==BAD_SCAN) {
      n_act_file++;
    }

    if(GLcdb_flag==GOOD_SCAN || GLcdb_flag==BAD_SCAN)
      break;
  }

  if(n_act_file==0) {
    if(pipeout>0) StarPipeEndOut();
    exit(0);
  }
}

#define CUT_XY 3
#define CUT_YZ 1
#define CUT_ZX 2
#define CUT_YX 13
#define CUT_ZY 11
#define CUT_XZ 12

void StarXColor2RGBImage15bpp(image,pixel)
char *image; /* <out> 2byte pixel */
XColor *pixel; /* <in> XColor structure; pixel.red,pixel.green,pixel.blue */
{
  unsigned char red,green,blue;

  red=(unsigned char)((unsigned long)(*pixel).red*31L/65535L);
  green=(unsigned char)((unsigned long)(*pixel).green*31L/65535L);
  blue=(unsigned char)((unsigned long)(*pixel).blue*31L/65535L);

  image[0]=blue;
  image[1]=(red*32) | green;

  return;
}

void StarCube2RGBImage(image,cube,cut_plane,cut_level,image_size,byte4pixel)
char *image;
CUBE *cube;
int cut_plane;
int cut_level;
XY_SIZE_long image_size;
int byte4pixel;
{
  int nx,ny,nz;
  int ix,iy,iz;
  int mx,my;
  int jx,jy;
  int ix0,iy0,jx0,jy0;
  int j_start;

  int ii;

  double data_value,data_scale,data_offset;
  float pixel_value,dummy;

  XColor pixel;

  nx=(*cube).head.axis.x.size;
  ny=(*cube).head.axis.y.size;
  nz=(*cube).head.axis.z.size;

  dummy=(*cube).head.data_value.dummy;
  data_scale=(*cube).head.data_value.scale;
  data_offset=(*cube).head.data_value.offset;

  mx=image_size.dx/nx+1;
  my=image_size.dy/ny+1;
  /*
  image_size.dx=(*cube).head.axis.x.size;
  image_size.dy=(*cube).head.axis.y.size;
  */

  for(iy=0;iy<image_size.dy;iy++) {
    iy0=image_size.dy*iy/ny;
    for(ix=0;ix<image_size.dx;ix++) {
      ix0=image_size.dx*ix/nx;

      pixel.red=StarGetCubePixelData(cube,ix,iy,3)*65355.0;
      pixel.green=StarGetCubePixelData(cube,ix,iy,3)*65355.0;
      pixel.blue=StarGetCubePixelData(cube,ix,iy,3)*65355.0;

      for(jy=0;jy<my;jy++) {
	jy0=iy0+jy;
	for(jx=0;jx<mx;jx++) {
	  jx0=ix0+jx;

	  j_start=(jy0*mx+jx0)*byte4pixel;
	  StarXColor2RGBImage15bpp(&(image[j_start]),&pixel);
	}
      }

    }
  }

/*
  switch(cut_plane) {
  case CUT_XY: case CUT_YX:
    if(cut_level<0) cut_level=0;
    else if(cut_level>=nz) cut_level=nz-1;
    iz=cut_level;

    mx=image_size.dx/nx;
    my=image_size.dy/ny;

    for(iy=0;iy<ny;iy++) {
      for(ix=0;ix<nx;ix++) {
	pixel_value=StarGetCubePixel(&GLcube,ix,iy,iz);
	i_start=(iy*mx*my*nx+ix*mx);
	if(pixel_value==dummy) {
	  for(jy=0;jy<my;jy++) {
	    for(jx=0;jx<mx;jx++) {
	      image[jy*mx+jx]=0;
	    }
	  }
	}
      }
    }
    break;

  case CUT_YZ: case CUT_ZY:
    if(cut_level<0) cut_level=0;
    else if(cut_level>=nx) cut_level=nx-1;
    ix=cut_level;

    for(iz=0;iz<nz;iz++) {
      for(iy=0;iy<ny;iy++) {
      }
    }
    break;

  case CUT_ZX: case CUT_XZ:
    if(cut_level<0) cut_level=0;
    else if(cut_level>=ny) cut_level=ny-1;
    iy=cut_level;

    for(iz=0;iz<nz;iz++) {
      for(ix=0;ix<nx;ix++) {
      }
    }
    break;
  }
*/
  return;
}

void actual_plot()
{
  static int read_flag=1;
  RANGE x_range,y_range;
  RANGE_long act_ch_range;

  BOX_RANGE phys_box,plot_box;
  BOX_RANGE phys_cbox,plot_cbox;

  static PxpPoint win_size;
  double x_step,y_step;
  double name_x,name_y;

  int output_mode;
  int window_x,window_y,window_h,window_w;
  double tick_length;

  static int window_h_old=0,window_w_old=0;
/*----------top of test--------*/
int      depth;
Display *disp;
Visual *visual;

static XImage *image;
unsigned char *image_plane;
static int byte4pixel;
static Pixmap pixmap;

int nx,ny,nz;
int ix,iy,iz;
double data_scale,data_offset;
XY_SIZE_long plot_image_size;
/*----------end of test--------*/


/* copy from global valuables */
  x_range.min=GLx_range.min;
  x_range.max=GLx_range.max;
  y_range.min=GLy_range.min;
  y_range.max=GLy_range.max;

/* read profile if the first */
  if(read_flag>0) {
    read_cube(&GLcube);
    read_flag= -1;
  }

/* plotsize auto scaling */
  PxpGetOutputMode(&output_mode);
  if(output_mode==X11WIN) {
    PxxPresentWindowSize(&window_x,&window_y,&window_w,&window_h);
    win_size.x=PxxGetPlotSpanX(window_w);
    win_size.y=PxxGetPlotSpanY(window_h);
  }

  plot_box.x.min=win_size.x*0.1;
  plot_box.y.min=win_size.y*0.1;
  plot_box.x.max=plot_box.x.min+win_size.x*0.8;
  plot_box.y.max=plot_box.y.min+win_size.y*0.8;

/* profile in standard form */
  /*
  StarPlotProfileStd(&GLcube,x_unit,x_range,y_range,plot_box,GLbase_show,GLspur_show);
  */
/*---top of test part------------*/
  disp=PxxGetDisplay();
  depth=DefaultDepth(disp,0);

  visual=DefaultVisual(disp,0);
  byte4pixel=depth/8; /* byte for a pixel, that is byte deps */
  if(depth==15) byte4pixel=4;

  if((window_h_old!=window_h || window_w_old!=window_w) && GLimage_plane!=NULL) {
    free(GLimage_plane);
    GLimage_plane=NULL;
  }
  if(GLimage_plane==NULL) {
    GLimage_plane=malloc(sizeof(char)*window_w*window_h*byte4pixel);
    if(GLimage_plane==NULL) {
      fprintf(stderr,"**ERR** cannot alloc memory for virtual VRAM\n");
      exit(999);
    }
    window_h_old=window_h;
    window_w_old=window_w;
  }

  nx=GLcube.head.axis.x.size;
  ny=GLcube.head.axis.y.size;
  nz=GLcube.head.axis.z.size;

  if(nx>window_w) nx=window_w;
  if(ny>window_h) ny=window_h;

  plot_image_size.dx=window_w;
  plot_image_size.dy=window_h;

  /*  StarCube2RGBImage(GLimage_plane,&GLcube,cut_plane,cut_level,plot_image_size,byte4pixel);*/
  StarCube2RGBImage(GLimage_plane,&GLcube,CUT_XY,nz/2,plot_image_size,byte4pixel);

  image=XCreateImage(disp,visual,depth,ZPixmap,0,GLimage_plane,(unsigned int)window_w,(unsigned int)window_h,byte4pixel*8,window_w*byte4pixel);
  image->bits_per_pixel=byte4pixel*8;
  image->bitmap_bit_order=MSBFirst;

  PxpSetColor("pink");
  PxpDrawLine(2.0,2.0,10.0,2.0);

  pixmap=XCreatePixmap(disp,PxxGetWindow(),window_w,window_h,depth);
  XPutImage(disp,pixmap,PxxGetGC(),image,0,0,0,0,window_w,window_h);
  /*  XPutImage(disp,pixmap,PxxGetGC(),image,0,0,100,100,window_w,window_h); */
  PxpSetColor("white");
  PxpDrawLine(1.0,1.0,2.0,2.0);

  XSetWindowBackgroundPixmap(disp,PxxGetWindow(),pixmap);
  XFreePixmap(disp,pixmap);

/*---end of test part------------*/

}

void help_function()
{
  fprintf(stderr,"help should be shown.\n");
  return;
}

main(argc,argv)
int argc;
char *argv[];
{
  int ii,jj;
  int err_code;
  int i_file,n_file;

  double w_height, w_width;	/* size of window(cm) */
  RANGE	x_range,y_range,z_range;	/* range to display */
  int pipeout;

  char id_str[BUFFER_SIZE];

/* check option */
  err_code=options(argc,argv, &w_height,&w_width,&x_range,&y_range,&z_range,&pipeout);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

/* set global parameters */
  GLpipeout=pipeout;

  if(z_range.min==z_range.max) {
    GLx_range.min=x_range.min;
    GLx_range.max=x_range.max;
    GLy_range.min=y_range.min;
    GLy_range.max=y_range.max;
  } else if(y_range.min==y_range.max) {
    GLx_range.min=x_range.min;
    GLx_range.max=x_range.max;
    GLy_range.min=z_range.min;
    GLy_range.max=z_range.max;
  } else {
    GLx_range.min=z_range.min;
    GLx_range.max=z_range.max;
    GLy_range.min=y_range.min;
    GLy_range.max=y_range.max;
  }

/* multi-data header in */
  StarPipeHeadIn(id_str,&n_file);
  GLn_file=n_file;

  if(pipeout>0) StarPipeHeadOut(n_file);

  GLimage_plane=NULL;

/* plot color map */
  local_StarPxpDisplayAll(argc,argv, X11WIN,NULL,"display",w_width,w_height,actual_plot,read_cube,help_function);

}
