/*
copy bad pixel in cubes
All bad pixels in a template cube are copied into the cube flow.
No check is applied such as cube size, cube position

2006/7/26 T.Handa : ver.3.00
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"cpbadpix.prm"
#define N_KEY 30

#define DEBUG
#undef DEBUG

int options(argc,argv,template_cube,check_flag)
/*
get parameters from options
*/
int argc;
char *argv[];
char *template_cube;
int *check_flag;
{
  int ii,next_pos;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  strcpy(template_cube,"");
  check_flag[0]=FLAG_YES;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"base_cube");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"base_cubname");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"template_cube");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"size_check");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

  next_pos=0;
/* base cubname */
  n_scan=3;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(template_cube,got_value);
  }

/* check criteria */
  n_scan=1;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strcasecmp(got_value,"no")!=0) {
      check_flag[0]=FLAG_YES;
    } else {
      check_flag[0]=FLAG_NO;
    }
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return 0;
}

void StarCubeProcess(cube_add,cube_base)
CUBE *cube_add; /* <i/o> */
CUBE *cube_base; /* <in> */
{
  int ix,iy,iz;
  int mx,my,mz;
  double ta_base,ta_add;
  int valid_flag_base,valid_flag_add;
  float dummy_pixel;

  mx=(*cube_base).head.axis.x.size;
  my=(*cube_base).head.axis.y.size;
  mz=(*cube_base).head.axis.z.size;
  if(mx>(*cube_add).head.axis.x.size)
    mx=(*cube_add).head.axis.x.size;
  if(my>(*cube_add).head.axis.y.size)
    my=(*cube_add).head.axis.y.size;
  if(mz>(*cube_add).head.axis.z.size)
    mz=(*cube_add).head.axis.z.size;

  dummy_pixel=(*cube_add).head.data_value.dummy;

  for(ix=0;ix<mx;ix++) {
    for(iy=0;iy<my;iy++) {
      for(iz=0;iz<mz;iz++) {
	ta_base=StarGetCubePixelData(cube_base,ix,iy,iz,&valid_flag_base);
	ta_add=StarGetCubePixelData(cube_add,ix,iy,iz,&valid_flag_add);
	if(valid_flag_base==FLAG_NO && valid_flag_add==FLAG_YES) {
	  StarPutCubePixel(cube_add,ix,iy,iz,dummy_pixel);
	}
      }
    }
  }

  return;
}

void main_filename(mainname,filename)
char *mainname; /* <out> */
char *filename; /* <in> */
{
  int ii,l_str,i_top;

  strcpy(mainname,StarFileMainName(filename));

  l_str=strlen(mainname);
  i_top=l_str-4;
  if(i_top<0) i_top=0;
  for(ii=l_str-1;ii>=i_top;ii--) {
    if(mainname[ii]=='.') {
      mainname[ii]=(char)NULL;
      break;
    }
  }
  return;
}

void AddProcessHistory(add_head,base_head)
CUBE_HEAD *add_head; /* <i/o> */
CUBE_HEAD *base_head; /* <in> */
{
  static char main_cubname[2][MAX_FILE_NAME_LENGTH];
  HISTORY	history;
  char buffer[80];
  int i_prm;

  main_filename(main_cubname[1],(*add_head).file_name);
  if(strlen(main_cubname[0])>16) main_cubname[0][16]=(char)NULL;
  main_filename(main_cubname[0],(*base_head).file_name);
  if(strlen(main_cubname[1])>16) main_cubname[1][16]=(char)NULL;

  strcpy(history.step,"badpix");

  i_prm=0;

/* base and add cube */
  strncpy(buffer,main_cubname[0],15);
  buffer[16]=(char)NULL;
  strcpy(history.prm[i_prm],buffer);
  i_prm++;

  strcpy(buffer,"copy dummy in");
  buffer[16]=(char)NULL;
  strcpy(history.prm[i_prm],buffer);
  i_prm++;

  strncpy(buffer,main_cubname[1],15);
  buffer[16]=(char)NULL;
  strcpy(history.prm[i_prm],buffer);
  i_prm++;

  history.size=i_prm;

  StarAddHistory4Cube(add_head,&history);
}

int CheckCubeHead(cube1,cube2)
CUBE_HEAD *cube1; /* <in> */
CUBE_HEAD *cube2; /* <in> */
{
  int ret_code;

  ret_code=0;

/* axis size */
  if((*cube1).axis.x.size!=(*cube2).axis.x.size) {
    fprintf(stderr,"unmatch x-size\n");
  }
  if((*cube1).axis.y.size!=(*cube2).axis.y.size) {
    fprintf(stderr,"unmatch y-size\n");
  }
  if((*cube1).axis.z.size!=(*cube2).axis.z.size) {
    fprintf(stderr,"unmatch z-size\n");
  }

/* axis type */
  if(strcmp((*cube1).axis.x.type,(*cube2).axis.x.type)!=0) {
    fprintf(stderr,"unmatch x-type\n");
  }
  if(strcmp((*cube1).axis.y.type,(*cube2).axis.y.type)!=0) {
    fprintf(stderr,"unmatch y-type\n");
  }
  if(strcmp((*cube1).axis.z.type,(*cube2).axis.z.type)!=0) {
    fprintf(stderr,"unmatch z-type\n");
  }

/* data unit */
  if(strcmp((*cube1).data_value.type,(*cube2).data_value.type)!=0) {
    fprintf(stderr,"unmatch data-type\n");
  }

  return(ret_code);
}

int main(argc,argv)
int argc;
char *argv[];
{
  CUBE cube_base,cube_add;
  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];
  int err_code;
  int n_file;
  char id_str[BUFFER_SIZE];
  char base_cubname[MAX_FILE_NAME_LENGTH];
  int check_flag[2];

  int ii;

  FILE *fp_in;

/* get optional parameters */
  err_code=options(argc,argv,base_cubname,check_flag);

  fp_in=fopen(base_cubname,"r");
  if(fp_in==NULL) {
    fprintf(stderr,"**ERR** file %s is not found.\n",base_cubname);
    exit(99);
  }
  err_code=StarCubeHeadLoad(fp_in,&(cube_base.head));
  if(err_code!=0) {
    fprintf(stderr,"**ERR** failed to load header data for %s\n",base_cubname);
    exit(99);
  }
  cube_base.pixel=StarAllocCubePixel(&(cube_base.head));
  if(cube_base.pixel==NULL) {
    fprintf(stderr,"**ERR** cannot alloc pixel data for %s\n",base_cubname);
    exit(99);
  }
  err_code=StarCubePixelLoad(fp_in,&cube_base);
  if(err_code!=0) {
    fprintf(stderr,"**ERR** failed to load pixel data for %s\n",base_cubname);
    exit(99);
  }
  fclose(fp_in);
  main_filename(cube_base.head.file_name,base_cubname);
  

/* pipe header */
  StarPipeHeadIn(id_str,&n_file);

  StarPipeHeadOut(n_file);

/* .cub data */
  for(ii=0;ii<n_file;ii++) {

    StarCubeIn(&cube_add,&cdb_flag,cdb_arg);

    if(check_flag[0]==FLAG_YES)
      CheckCubeHead(&(cube_add.head),&(cube_base.head));

    StarCubeProcess(&cube_add,&cube_base);

    StarCubeScanDatavalue(&cube_add);

    AddProcessHistory(&(cube_add.head),&(cube_base.head)); 

    StarCubeOut(&cube_add,cdb_flag,cdb_arg);
    StarFreeCubePixel(&cube_add);
  }

  StarFreeCubePixel(&cube_base);

/* end tag of pipe */
  StarPipeEndIn(id_str);
  StarPipeEndOut();

  exit(0);
}
