/*
concatinate several (effectively) 2D maps into single 3D map

2002/6/24 T.Handa : ver.3.00
2003/8/19 T.Handa : ver.3.01 (debug) 
2006/5/22 T.Handa : ver.3.02 (debug) core dump via ssh; 2D array should be static
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"cub2dto3d.prm"
#define N_KEY 30

#define DEBUG
/*#undef DEBUG*/

int options(argc,argv,z_range,z_step,auto_z_flag)
/*
get parameters from options
*/
int argc;
char *argv[];
RANGE *z_range;
double *z_step;
int *auto_z_flag;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  (*z_range).min=0.0;
  (*z_range).max=0.0;
  *z_step=1.0;
  *auto_z_flag=FLAG_NO;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"z");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z_range");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z_step");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"auto_z");   key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"auto_z");   key_type[n_key-1]=0;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* range along z-axis */
  n_scan=2;
  scan_flag[0]= 0;
  scan_flag[1]= 1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    StarOptionAngleRange(got_value,z_range);
  }

/* step along z-axis */
  n_scan=1;
  scan_flag[0]= 2;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    *z_step=StarStdAngle2deg(got_value);
  }

/* auto z : put z-position by the input sequence */
  n_scan=2;
  scan_flag[0]= 3;
  scan_flag[1]= 4;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strncmp(got_value,"y",1)==0 || strncmp(got_value,"Y",1)==0 || strcmp(got_value,"auto_z")==0) {
      *auto_z_flag=FLAG_YES;
    } else {
      *auto_z_flag=FLAG_NO;
    }
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}


int PutCubeletIntoCube(cube_base,cube_add)
CUBE *cube_base; /* <i/o> */
CUBE *cube_add; /* <in> */
/*
overwrite the z=0 plane of cube_add into cube_base

 <return> =(-1):out of z-range, >=0:input z_ch
2002/6/24 T.Handa
 */
{
  int ix,iy,iz;
  int mx,my,mz;
  float dummy_pixel;
  double zz,z_pixel;
  double ta;
  int valid_pixel;

  mx=(*cube_base).head.axis.x.size;
  my=(*cube_base).head.axis.y.size;
  mz=(*cube_base).head.axis.z.size;
  dummy_pixel=(*cube_base).head.data_value.dummy;

  zz=StarGetCubePixPosOnAxis(&((*cube_add).head.axis.z),0);
  z_pixel=StarPixelIndexPositionOnAxis(&((*cube_base).head.axis.z),zz);
  iz=(int)(rint(z_pixel));
  if(iz<0 || iz>=mz) return(-1);

  for(iy=0;iy<my;iy++) {
    for(ix=0;ix<mx;ix++) {
      ta=StarGetCubePixelData(cube_add,ix,iy,0,&valid_pixel);
      if(valid_pixel==FLAG_YES)
	StarPutCubePixelData(cube_base,ix,iy,iz,ta);
    }
  }

  return(iz);
}

void AddCubStackHistory(base_head,n_file,cubname,in_z)
CUBE_HEAD *base_head; /* <i/o> */
int n_file; /* <in> */
char cubname[MAX_FILE_NUMBER][MAX_FILE_NAME_LENGTH]; /* <in> */
int *in_z; /* <in> */
{
  HISTORY history;
  char buffer[80];
  int ii;
  int i_prm;

  static int max_i_prm=MAX_HISTORY_LINES_PER_STEP;


  strcpy(history.step,"2Dto3D");

  i_prm=0;

/* base and add cube */
  sprintf(buffer,"n_file=%d",n_file);
  buffer[16]=(char)NULL;
  strcpy(history.prm[i_prm],buffer);
  i_prm++;

  for(ii=0;ii<n_file;ii++) {
    strncpy(buffer,cubname[ii],15);
    buffer[16]=(char)NULL;
    strcpy(history.prm[i_prm],buffer);
    i_prm++;
    if(i_prm>=max_i_prm) {
      strcpy(history.prm[i_prm-1],"skip more");
      break;
    }

    sprintf(buffer,"at z=%d",in_z[ii]);
    buffer[16]=(char)NULL;
    strcpy(history.prm[i_prm],buffer);
    i_prm++;
    if(i_prm>=max_i_prm) {
      strcpy(history.prm[i_prm-1],"skip more");
      break;
    }
  }

  history.size=i_prm;

  StarAddHistory4Cube(base_head,&history);

  return;
}

int CheckCubeHead(cube1,cube2)
CUBE_HEAD *cube1; /* <in> */
CUBE_HEAD *cube2; /* <in> */
{
  int ret_code;

  ret_code=0;
/* axis size */
  if((*cube1).axis.x.size!=(*cube2).axis.x.size) {
    fprintf(stderr,"unmatch x-size\n");
  }
  if((*cube1).axis.y.size!=(*cube2).axis.y.size) {
    fprintf(stderr,"unmatch y-size\n");
  }

/* axis type */
  if(strcmp((*cube1).axis.x.type,(*cube2).axis.x.type)!=0) {
    fprintf(stderr,"unmatch x-type\n");
  }
  if(strcmp((*cube1).axis.y.type,(*cube2).axis.y.type)!=0) {
    fprintf(stderr,"unmatch y-type\n");
  }
  if(strcmp((*cube1).axis.z.type,(*cube2).axis.z.type)!=0) {
    fprintf(stderr,"unmatch z-type\n");
  }

/* data unit */
  if(strcmp((*cube1).data_value.type,(*cube2).data_value.type)!=0) {
    fprintf(stderr,"unmatch data-type\n");
  }

  return(ret_code);
}

int main(argc,argv)
int	argc;
char	*argv[];
{
  CUBE cube_base,cube_add;
  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];
  static char cubname[MAX_FILE_NUMBER][MAX_FILE_NAME_LENGTH];
  int z_ch[MAX_FILE_NUMBER];

  int err_code;
  int i_file,n_file;
  char id_str[BUFFER_SIZE];

  int z_size;
  double z_step;
  RANGE z_range;
  int auto_z_flag;

  int ii;
  float dummy_pixel;

  int in_z;

/* get optional parameters */
  err_code=options(argc,argv,&z_range,&z_step,&auto_z_flag);

  z_size=(int)(ceil(fabs((z_range.max-z_range.min)/z_step)))+1;
  if(z_size<=0) z_size=1;

/* pipe header */
  StarPipeHeadIn(id_str,&n_file);

  StarPipeHeadOut(1); /* final cube must be 1. */

/* .cub data */
  i_file=0;
  for(ii=0;ii<n_file;ii++) {

    err_code=StarCubeIn(&cube_add,&cdb_flag,cdb_arg);

    if(ii==0) {
      StarCopyCubeHead(&(cube_base.head),&(cube_add.head));
      cube_base.head.axis.z.size=z_size;
      cube_base.head.axis.z.delta=z_step;
      cube_base.head.axis.z.origin=z_range.min;
      cube_base.head.axis.z.pixel_origin=0.0;

      cube_base.pixel=StarAllocCubePixel(&(cube_base.head));
      if(cube_base.pixel==NULL) {
	fprintf(stderr,"%s: **ERR** cannot alloc pixel data\n",argv[0]);
	exit(99);
      }
      dummy_pixel=cube_base.head.data_value.dummy;
      StarCubePixelClear(&cube_base,dummy_pixel);

    } else {
      CheckCubeHead(&(cube_add.head),&(cube_base.head));
    }

/* put proper ordering position along z for input cube */
    if(auto_z_flag==FLAG_YES) {
      cube_add.head.axis.z.size=1;
      cube_add.head.axis.z.delta=z_step;
      cube_add.head.axis.z.origin=z_range.min+z_step*(double)ii;
      cube_add.head.axis.z.pixel_origin=0.0;

    }

/* copy to the resultant cube */

    in_z=PutCubeletIntoCube(&cube_base,&cube_add);
    if(in_z>=0 && in_z<z_size) {
      strcpy(cubname[i_file],StarFileMainName(cube_add.head.file_name));
      z_ch[i_file]=in_z;
      i_file++;
    }

    StarFreeCubePixel(&cube_add);
  }

/* scan and re-estimate head.data_value */ 
  StarCubeScanDatavalue(&cube_base);
  AddCubStackHistory(&(cube_base.head),i_file,cubname,z_ch);

  StarCubeOut(&cube_base,cdb_flag,cdb_arg);

  StarFreeCubePixel(&cube_base);

/* end tag of pipe */
  StarPipeEndIn(id_str);
  StarPipeEndOut();

  exit(0);
}
