/*
data convert from Ultra/STAR cub to FITS

1999/7/31 T.Handa : ver.3.00
2003/1/ 4 T.Handa : ver.3.01 (debug) extension name
2006/5/31 T.Handa : ver.3.10 (modify) short header in only 2880bytes
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>
#include <StFitsIO.h>

#define	DEFAULT_PRMFILE	"cub2fits.prm"
#define N_KEY 50
#define N_BLOCK	10

#ifdef _NO_PROTO
static int save_fits_head();
#else
static int save_fits_head(FILE *,CUBE_HEAD *,int);
#endif

int to_fits(cube,fp,short_head)
CUBE *cube;
FILE *fp;
int short_head;
{
  int err_code;


  err_code=save_fits_head(fp,&((*cube).head),short_head);
  if(err_code!=0) return(err_code);

  err_code=StarSaveFitsPixel(fp,cube);
  if(err_code!=0) return(err_code);

  return(0);
}

int StarWriteFitsHead4CubeHeadDataValue(fp,data)
/*
Write Fits header corresponding cube.data_value

1999/7/31 T.Handa
*/
FILE *fp; /* <in> for FITS */
DATA_VALUE *data; /* <in> cube.data_value */
{
  int n_line;
  char buffer[80];

  n_line=0;
  StarWriteFitsRecord("BUNIT",(*data).unit,FITS_VALUE_SHORT_STRING,"",fp);
  n_line++;
  sprintf(buffer,"%20.4e",(*data).scale);
  StarWriteFitsRecord("BSCALE",buffer,FITS_VALUE,"Real=fits_value*BSCALE+BZERO",fp);
  n_line++;
  sprintf(buffer,"%20.4e",(*data).offset);
  StarWriteFitsRecord("BZERO",buffer,FITS_VALUE,"",fp);
  n_line++;
  sprintf(buffer,"%20.0f",(*data).dummy);
  StarWriteFitsRecord("BLANK",buffer,FITS_VALUE,"meaning less",fp);
  n_line++;
  sprintf(buffer,"%20.4e",(*data).max);
  StarWriteFitsRecord("DATAMAX",buffer,FITS_VALUE,"",fp);
  n_line++;
  sprintf(buffer,"%20.4e",(*data).min);
  StarWriteFitsRecord("DATAMIN",buffer,FITS_VALUE,"",fp);
  n_line++;

  return(n_line);
}

int StarWriteFitsHead4CubeHeadObsPrm(fp,obs_prm)
/*
Write Fits header corresponding cube.obs_prm

1999/7/31 T.Handa
*/
FILE *fp; /* <in> for FITS */
OBS_PRM *obs_prm; /* <in> cube.obs_prm */
{
  int n_line;
  char buffer[80];
  int yy,mm,dd;

  n_line=0;

  sprintf(buffer,"%20.4f",(double)(*obs_prm).mjd);
  StarWriteFitsRecord("MJD-OBS",buffer,FITS_VALUE,"",fp);
  n_line++;

  StarMJD2Ymd(&yy,&mm,&dd,(long)((*obs_prm).mjd));
  sprintf(buffer,"%4d-%02d-%02d",yy,mm,dd);
  StarWriteFitsRecord("DATE-OBS",buffer,FITS_VALUE_STRING,"",fp);
  n_line++;

  if(strlen((*obs_prm).observer1)>0) {
    StarWriteFitsRecord("OBSERVER",(*obs_prm).observer1,FITS_VALUE_STRING,"",fp);
    n_line++;
  }
  if(strlen((*obs_prm).observer2)>0) {
    StarWriteFitsRecord("OBSERVER",(*obs_prm).observer2,FITS_VALUE_STRING,"",fp);
    n_line++;
  }
  if(strlen((*obs_prm).observer3)>0) {
    StarWriteFitsRecord("OBSERVER",(*obs_prm).observer3,FITS_VALUE_STRING,"",fp);
    n_line++;
  }
  if(strlen((*obs_prm).observer4)>0) {
    StarWriteFitsRecord("OBSERVER",(*obs_prm).observer4,FITS_VALUE_STRING,"",fp);
    n_line++;
  }

  return(n_line);
}

int StarWriteFitsHead4CubeHeadObject(fp,obj)
/*
Write Fits header corresponding cube.object

1999/7/31 T.Handa
*/
FILE *fp; /* <in> for FITS */
OBJECT *obj; /* <in> cube.object */
{
  int n_line;

  n_line=0;

  StarWriteFitsRecord("OBJECT",(*obj).name,FITS_VALUE_STRING,"",fp);
  n_line++;

  if(strlen((*obj).equinox)<=0) {
    StarWriteFitsRecord("EQUINOX","1950.0",FITS_VALUE,"",fp);
    n_line++;
  } else {
    if((*obj).equinox[0]=='B' || (*obj).equinox[0]=='J') {
      StarWriteFitsRecord("EQUINOX",&((*obj).equinox[1]),FITS_VALUE,"",fp);
      n_line++;
    } else {
      StarWriteFitsRecord("EQUINOX",(*obj).equinox,FITS_VALUE,"",fp);
      n_line++;
    }
  }

  return(n_line);
}

int StarWriteFitsHead4CubeHeadAntStatus(fp,ant)
/*
Write Fits header corresponding cube.ant

1999/7/31 T.Handa
*/
FILE *fp; /* <in> for FITS */
ANT_STATUS *ant; /* <in> cube.ant */
{
  int n_line;

  n_line=0;

  StarWriteFitsRecord("TELESCOP",(*ant).tele,FITS_VALUE_STRING,"",fp);
  n_line++;

  return(n_line);
}

int StarWriteFitsHead4CubeHeadRxStatus(fp,rx)
/*
Write Fits header corresponding cube.rx

1999/7/31 T.Handa
*/
FILE *fp; /* <in> for FITS */
RX_STATUS *rx; /* <in> cube.rx */
{
  int n_line;

  n_line=0;

  StarWriteFitsRecord("INSTRUME",(*rx).name,FITS_VALUE_STRING,"Receiver",fp);
  n_line++;

  return(n_line);
}

int StarWriteFitsHead4CubeHeadBeStatus(fp,be)
/*
Write Fits header corresponding cube.be

1999/7/31 T.Handa
*/
FILE *fp; /* <in> for FITS */
BE_STATUS *be; /* <in> cube.be */
{
  int n_line;

  n_line=0;

  StarWriteFitsRecord("INSTRUME",(*be).name,FITS_VALUE_STRING,"BackEnd",fp);
  n_line++;

  return(n_line);
}

int StarWriteFitsHead4CubeHeadObsLog(fp,log)
/*
Write Fits header corresponding cube.log

1999/7/31 T.Handa
*/
FILE *fp; /* <in> for FITS */
OBS_LOG *log; /* <in> cube.log */
{
  int n_line;

  n_line=0;

  StarWriteFitsRecord("ORIGIN",(*log).site_name,FITS_VALUE_STRING,"",fp);
  n_line++;
  if(strlen((*log).memo)>0) {
    StarWriteFitsRecord("COMMENT",(*log).memo,FITS_VALUE_STRING,"",fp);
    n_line++;
  }
  if(strlen((*log).comment)>0) {
    StarWriteFitsRecord("COMMENT",(*log).comment,FITS_VALUE_STRING,"",fp);
    n_line++;
  }

  return(n_line);
}

int save_fits_head(fp,cube_head,short_head)
FILE *fp;
CUBE_HEAD *cube_head;
int short_head; /* <in> =1 2880byte limited, =0 unlimited */
{
  int i_line,n_line,blank_line;
  int ii;
  char *blank;
  static int lmt_line=36; /* =2880/80; */

/* basic and axis part */
  n_line=0;
  StarWriteFitsRecord("SIMPLE","True",FITS_VALUE_BOOLIAN,"FITS STANDARD",fp);
  n_line++;
  StarWriteFitsRecord("BITPIX","                 -32",FITS_VALUE,"FITS BITS/PIXEL",fp);
  n_line++;

  if(n_line+3+3*6<lmt_line) {
    n_line=n_line+StarWriteFitsHead4CubeHeadAxes(fp,&((*cube_head).axis));
  }

/* Data value part */
  if(n_line+6<lmt_line) {
    n_line=n_line+StarWriteFitsHead4CubeHeadDataValue(fp,&((*cube_head).data_value));
  }
  if(n_line+7<lmt_line) {
    n_line=n_line+StarWriteFitsHead4CubeHeadObsPrm(fp,&((*cube_head).obs));
  }
  if(n_line+1<lmt_line) {
    n_line=n_line+StarWriteFitsHead4CubeHeadObject(fp,&((*cube_head).object));
  }
  if(n_line+1<lmt_line) {
    n_line=n_line+StarWriteFitsHead4CubeHeadAntStatus(fp,&((*cube_head).ant));
  }
  if(n_line+1<lmt_line) {
    n_line=n_line+StarWriteFitsHead4CubeHeadRxStatus(fp,&((*cube_head).rx));
  }
  if(n_line+1<lmt_line) {
    n_line=n_line+StarWriteFitsHead4CubeHeadBeStatus(fp,&((*cube_head).be));
  }
  if(n_line+3<lmt_line) {
    n_line=n_line+StarWriteFitsHead4CubeHeadObsLog(fp,&((*cube_head).log));
  }

/* End keyword */
  StarWriteFitsRecord("END","",FITS_VALUE,"",fp);
  n_line++;

/* estimate blank records */
  blank_line=(int)ceil((double)n_line/36.0)*36-n_line;

  if(blank_line>0) {
    blank=malloc(80);
    for(ii=0;ii<80;ii++) {
      blank[ii]=' ';
    }
    for(i_line=0;i_line<blank_line;i_line++) {
      fwrite(blank,1,80,fp);
    }
    free(blank);
  }

  return(0);
}

int options(argc,argv,filename,pipe_flag,short_head)
/*
get parameters from options
*/
int argc;
char *argv[];
char *filename;
int *pipe_flag;
int *short_head;
{
  int ii,next_pos;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;
  char *got_value;

/* default values */
  *pipe_flag= -1;
  filename[0]=(char)NULL;
  *short_head=1;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"outfits");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"fits_name");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"pipe");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"output");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"-");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"nopipe");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"shortheader");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"longheader");	key_type[n_key-1]=0;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

  next_pos=0;
/* FITS file name */
  got_value=StarKeylessOption(argc,argv,n_key,key_flag,key_type);
  if(got_value[0]!=(char)NULL) {
    strcpy(filename,got_value);
  }

  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(filename,got_value);
  }

/* pipe */
  n_scan=4;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
    if(ii==3) scan_flag[3]= -scan_flag[3];
  }
  next_pos=next_pos+n_scan;
  *pipe_flag=StarOptionFlag(n_scan,scan_flag,key_flag);

/* shortheader */
  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
    if(ii==1) scan_flag[ii]= -scan_flag[ii];
  }
  next_pos=next_pos+n_scan;
  *short_head=StarOptionFlag(n_scan,scan_flag,key_flag);

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

void StarRemoveFilenameExtension(filename,extension)
char *filename; /* <i/o> filename with extension --> without extension */
char *extension; /* <in> assigned extension with . */
{
  int l_str,l_ext;

  l_str=strlen(filename);
  l_ext=strlen(extension);

  if(l_str>=l_ext && strcmp(&(filename[l_str-l_ext]),extension)==0) {
    filename[l_str-l_ext]=(char)NULL;
  }
  return;
}

void StarSeparateFilenameExtension(filename,ext)
char *filename; /* <i/o> filename with extension --> without extension */
char *ext; /* <out> extension without '.' */
{
  int l_str;
  int ii;

  l_str=strlen(filename);

  for(ii=l_str-1;ii>=0;ii--) {
    if(filename[ii]=='.') {
      filename[ii]=(char)NULL;
      strcpy(ext,&(filename[ii+1]));
      return;
    }
  }
  strcpy(ext,"");
  return;
}

#define N_EXT_TYPE 3

int StarCheckExt4fits(fitsname)
char *fitsname;
{
  char mainname[MAX_FILE_NAME_LENGTH];
  char extension[MAX_FILE_NAME_LENGTH];
  static char good_ext[N_EXT_TYPE][5]={"fits","fit","fts"};
  int ii;

  strcpy(mainname,fitsname);
  StarSeparateFilenameExtension(mainname,extension);
  for(ii=0;ii<N_EXT_TYPE;ii++) {
    if(strcasecmp(extension,good_ext[ii])==0)
      return(FLAG_YES);
  }
  return(FLAG_NO);
}

int main(argc,argv)
int argc;
char *argv[];
{
  CUBE cube;

  FILE *fits_fp;
  int pipe_flag;
  int err_code;

  char fitsname[MAX_FILE_NAME_LENGTH];
  char full_filename[MAX_FILE_NAME_LENGTH];
  int short_head;

  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];

  int ii,n_file;
  char id_str[MAX_FILE_NAME_LENGTH];

/* check option */
  err_code=options(argc,argv,fitsname,&pipe_flag,&short_head);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }
  /*
  strcpy(cube.head.file_name,cubename);
  strcpy(cube.head.file_id,STAR_CUB_IN);
  */
/* pipe header */
  StarPipeHeadIn(id_str,&n_file);

  if(pipe_flag==1) {
    StarPipeHeadOut(n_file);
  }

/* .cub data */
  for(ii=0;ii<n_file;ii++) {
    StarCubeIn(&cube,&cdb_flag,cdb_arg);
    if(pipe_flag==1) {
      StarCubeOut(&cube,cdb_flag,cdb_arg);
    }

/* add STD extension for FITS */
    StarRemoveFilenameExtension(cube.head.file_name,".cub");
    if(strcmp(fitsname,"")==0) {
      err_code=StarStdFileExtName("STAR_DATA",cube.head.file_name,full_filename,".fit");
    } else {
      if(StarCheckExt4fits(fitsname)==FLAG_YES) {
	err_code=StarStdFileExtName("STAR_DATA",fitsname,full_filename,"");
      } else {
	err_code=StarStdFileExtName("STAR_DATA",fitsname,full_filename,".fit");
      }
    }
    if(err_code!=0) {
      fprintf(stderr,"Too long file name\n");
      exit(1);
    }
    fits_fp=fopen(full_filename,"w");

    to_fits(&cube,fits_fp,short_head);

    fclose(fits_fp);

    StarFreeCubePixel(&cube);
  }

/* end tag of pipe */
  StarPipeEndIn(id_str);
  if(pipe_flag==1) {
    StarPipeEndOut();
  }

  exit(0);
}
