/*
cub data process along velocity (assume z-axis is velocity)

2002/5/20 T.Handa : ver.3.00
2004/1/8 T.Handa : ver.3.01 (debug) output z-axis header
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"cub2map.prm"
#define N_KEY 10

#define MAP_INTEG   11
#define MAP_PEAK_TA 12
#define MAP_MEAN_V  21
#define MAP_PEAK_V  22

int options(argc,argv,map_type)
/*
get parameters from options
*/
int argc;
char *argv[];
int *map_type;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  *map_type=MAP_INTEG;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"map_type"); key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* smoothing width */
  n_scan=1;
  scan_flag[0]=0;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strncmp(got_value,"integ",5)==0)
      *map_type=MAP_INTEG;
    else if(strncmp(got_value,"peak_T",7)==0 || strncmp(got_value,"peak_t",7)==0 || strcmp(got_value,"peak")==0) 
      *map_type=MAP_PEAK_TA;
    else if(strncmp(got_value,"mean_v",7)==0 || strcmp(got_value,"v_mean")==0)
      *map_type=MAP_MEAN_V;
    else if(strncmp(got_value,"peak_v",7)==0 || strcmp(got_value,"v_peak")==0)
      *map_type=MAP_PEAK_V;
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return 0;
}

void StarCubeIntegrateAlongZ(cube_out,cube)
CUBE *cube;
CUBE *cube_out;
{
  double integ_ta;
  double ta;
  int ix,iy,iz;
  int mx,my,mz;
  int valid_flag;
  double v_step;

  mx=(*cube).head.axis.x.size;
  my=(*cube).head.axis.y.size;
  mz=(*cube).head.axis.z.size;
  v_step=(*cube).head.axis.z.delta;

  for(ix=0;ix<mx;ix++) {
    for(iy=0;iy<my;iy++) {
      integ_ta=0.0;
      for(iz=0;iz<mz;iz++) {
	ta=StarGetCubePixelData(cube,ix,iy,iz,&valid_flag);
	if(valid_flag==FLAG_YES) {
	  integ_ta=integ_ta+ta;
	}
      }
      integ_ta=integ_ta*v_step;
      StarPutCubePixelData(cube_out,ix,iy,0,integ_ta);
    }
  }

  return;
}

void StarCubeMeanVelAlongZ(cube_out,cube)
CUBE *cube;
CUBE *cube_out;
{
  double integ_ta,integ_vta;
  double ta,vel,mean_v;
  int ix,iy,iz;
  int mx,my,mz;
  int valid_flag;

  mx=(*cube).head.axis.x.size;
  my=(*cube).head.axis.y.size;
  mz=(*cube).head.axis.z.size;

  for(ix=0;ix<mx;ix++) {
    for(iy=0;iy<my;iy++) {
      integ_ta=0.0;
      integ_vta=0.0;
      for(iz=0;iz<mz;iz++) {
	ta=StarGetCubePixelData(cube,ix,iy,iz,&valid_flag);
	if(valid_flag==FLAG_YES) {
	  integ_ta=integ_ta+ta;
	  vel=StarGetCubePixPosOnAxis(&((*cube).head.axis.z),iz);
	  integ_vta=integ_vta+ta*vel;
	}
      }
      if(integ_ta!=0.0) {
	mean_v=integ_vta/integ_ta;
	StarPutCubePixelData(cube_out,ix,iy,0,mean_v);
      }
    }
  }

  return;
}

void StarCubePeakAlongZ(cube_out,cube,map_type)
CUBE *cube;
CUBE *cube_out;
int map_type;
{
  double ta,peak_ta;
  double peak_v;
  int ix,iy,iz;
  int mx,my,mz;
  int valid_flag;
  double v_step;

  mx=(*cube).head.axis.x.size;
  my=(*cube).head.axis.y.size;
  mz=(*cube).head.axis.z.size;
  v_step=(*cube).head.axis.z.delta;
  peak_v=0.0;

  for(ix=0;ix<mx;ix++) {
    for(iy=0;iy<my;iy++) {
      peak_ta=(*cube).head.data_value.min;
      for(iz=0;iz<mz;iz++) {
	ta=StarGetCubePixelData(cube,ix,iy,iz,&valid_flag);
	if(valid_flag==FLAG_YES) {
	  if(ta>peak_ta) {
	    peak_ta=ta;
	    peak_v=StarGetCubePixPosOnAxis(&((*cube).head.axis.z),iz);
	  }
	}
      }
      switch(map_type) {
      case MAP_PEAK_V:
	StarPutCubePixelData(cube_out,ix,iy,0,peak_v);
	break;
      case MAP_PEAK_TA:  default:
	StarPutCubePixelData(cube_out,ix,iy,0,peak_ta);
	break;
      }
    }
  }

  return;
}


void AddHistory(cube_head,map_type)
CUBE_HEAD *cube_head; /* <i/o> */
int map_type; /* <in> */
{
  HISTORY	history;
  int i_prm;

  strcpy(history.step,"cub2map");

  i_prm=0;
  switch(map_type) {
  case MAP_PEAK_TA:
    strcpy(history.prm[i_prm],"type=peak_t");
    break;
  case MAP_MEAN_V:
    strcpy(history.prm[i_prm],"type=v_mean");
    break;
  case MAP_PEAK_V:
    strcpy(history.prm[i_prm],"type=v_peak");
    break;
  case MAP_INTEG: default:
    strcpy(history.prm[i_prm],"type=integ");
    break;
  }
  i_prm++;

  history.size=i_prm;

  StarAddHistory4Cube(cube_head,&history);
}


int main(argc,argv)
int argc;
char *argv[];
{
  CUBE cube_in,cube_out;

  char id_str[MAX_FILE_NAME_LENGTH];
  char cdb_arg[MAX_IDX_RECORD_LENGTH];

  int ii;
  int n_file;
  int err_code;

  int cdb_flag;

  int map_type;
  char buffer[BUFFER_SIZE];

  RANGE z_span;
  int mz;


/* check option */
  err_code=options(argc,argv,&map_type);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

  n_file=0;

  StarPipeHeadIn(id_str,&n_file);

  StarPipeHeadOut(n_file);

  for(ii=0;ii<n_file;ii++) {

/* get cube from stdin */
    err_code=StarCubeIn(&cube_in,&cdb_flag,cdb_arg);
    if(err_code!=0) {
      StarPipeEndOut();
      fprintf(stderr,"%s: fail to CubeIn No.%d\n",argv[0],ii);
      exit(1);
    }

    if(cdb_flag!=GOOD_SCAN && cdb_flag!=BAD_SCAN)
      goto next_cube;

/* check axis type */
    if(strcmp(cube_in.head.axis.z.type,"velocity")!=0) {
      fprintf(stderr,"%s: **ERR** The command work only z-axis is velocity-axis, but is %s.\n",argv[0],cube_in.head.axis.z.type);
      goto next_cube;
    }

    StarCopyCubeHead(&(cube_out.head),&(cube_in.head));
    mz=cube_in.head.axis.z.size;
    z_span.min=StarGetCubePixPosOnAxis(&(cube_in.head.axis.z),0);
    z_span.max=StarGetCubePixPosOnAxis(&(cube_in.head.axis.z),mz-1);

    cube_out.head.axis.z.size=1;
    cube_out.head.axis.z.origin=(z_span.min+z_span.max)/2.0;
    cube_out.head.axis.z.pixel_origin=0.0;
    cube_out.head.axis.z.delta=fabs(z_span.max-z_span.min)+fabs(cube_in.head.axis.z.delta);

    cube_out.pixel=StarAllocCubePixel(&(cube_out.head));
    StarCubePixelClear(&cube_out,cube_out.head.data_value.dummy);

/* switch process */
    switch(map_type) {
    case MAP_PEAK_TA:
      StarCubePeakAlongZ(&cube_out,&cube_in,map_type);
      break;
    case MAP_MEAN_V:
      cube_out.head.data_value.scale=1.0;
      cube_out.head.data_value.offset=0.0;
      StarCubeMeanVelAlongZ(&cube_out,&cube_in);
      strcpy(cube_out.head.data_value.type,cube_in.head.axis.z.type);
      strcpy(cube_out.head.data_value.unit,cube_in.head.axis.z.unit);
      break;
    case MAP_PEAK_V:
      cube_out.head.data_value.scale=1.0;
      cube_out.head.data_value.offset=0.0;
      StarCubePeakAlongZ(&cube_out,&cube_in,map_type);
      strcpy(cube_out.head.data_value.type,cube_in.head.axis.z.type);
      strcpy(cube_out.head.data_value.unit,cube_in.head.axis.z.unit);
      break;
    case MAP_INTEG: default:
      cube_out.head.axis.z.resolution=cube_out.head.axis.z.delta;
      strcpy(cube_out.head.axis.z.psf,"integ");
      StarCubeIntegrateAlongZ(&cube_out,&cube_in);
      sprintf(buffer,"%s %s",cube_in.head.data_value.unit,cube_in.head.axis.z.unit);
      buffer[8]=(char)NULL;
      strcpy(cube_out.head.data_value.unit,buffer);
      break;
    }

    StarCubeScanDatavalue(&cube_out);

    AddHistory(&(cube_out.head),map_type);

/* do pipeout */
    err_code=StarCubeOut(&cube_out,cdb_flag,cdb_arg);
    StarFreeCubePixel(&cube_in);
    StarFreeCubePixel(&cube_out);

next_cube:;
  }

  StarPipeEndOut();
  exit(0);
}
