/*
print out a cub data in text format

2002/5/28 T.Handa : ver.3.00 (not finished to set an output filename)
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"cub2txt.prm"
#define N_KEY 10

int options(argc,argv,headline_flag,z_level,txt_filename)
/*
get parameters from options
*/
int argc;
char *argv[];
int *headline_flag;
double *z_level;
char *txt_filename;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  *headline_flag=FLAG_YES;
  *z_level=0.0;
  strcpy(txt_filename,"-");

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"with_headline"); key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"without_headline"); key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"headline"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"outfile"); key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* headline */
  n_scan=3;
  scan_flag[0]=0;
  scan_flag[1]=1;
  scan_flag[2]=2;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strcmp(got_value,"no")==0 || strncmp(got_value,"without_",8)==0) { 
      *headline_flag=FLAG_NO;
    } else {
      *headline_flag=FLAG_YES;
    }
  }

/* z level */
  n_scan=1;
  scan_flag[0]=3;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    *z_level=StarStdAngle2deg(got_value);
  }

/* outfilename */
  n_scan=1;
  scan_flag[0]=4;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(txt_filename,got_value);
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return 0;
}

void headline4map(fp,cube_head,iz,sep)
FILE *fp;
CUBE_HEAD *cube_head;
int iz;
char sep;
{
  double z_level;
  int ix,mx;
  double xx;

  z_level=StarGetCubePixPosOnAxis(&((*cube_head).axis.z),iz);
  mx=(*cube_head).axis.x.size;

  fprintf(fp,"%s%c%s%c%s=%d=%f%s\n",(*cube_head).file_name,sep,(*cube_head).object.name,sep,(*cube_head).axis.z.name,iz,z_level,(*cube_head).axis.z.unit);
  fprintf(fp,"%s(%s)%c%s(%s)\n",(*cube_head).axis.y.name,(*cube_head).axis.y.unit,sep,(*cube_head).axis.x.name,(*cube_head).axis.x.unit);
  for(ix=0;ix<mx;ix++) {
    xx=StarGetCubePixPosOnAxis(&((*cube_head).axis.x),ix);
    fprintf(fp,"%c%f",sep,xx);
  }

  return;
}

void CubePixeldatavalue2Text(fp,cube,z_level,separator,headline_flag)
FILE *fp;
CUBE *cube;
int z_level;
char separator;
int headline_flag;
{
  double ta,dummy_pixel;
  int ix,iy,iz;
  int mx,my,mz;
  int valid_flag;
  double yy;

  mx=(*cube).head.axis.x.size;
  my=(*cube).head.axis.y.size;
  mz=(*cube).head.axis.z.size;
  if(z_level<0) z_level=0;
  if(z_level>=mz) z_level=mz-1;
  dummy_pixel=(double)(*cube).head.data_value.dummy;

  iz=z_level;

/* single x; strip map */
  if(mx==1) {
    for(iy=0;iy<my;iy++) {
      if(headline_flag==FLAG_YES) {
	yy=StarGetCubePixPosOnAxis(&((*cube).head.axis.y),iy);
	fprintf(fp,"%f%c",yy,separator);
      }
      ta=StarGetCubePixelData(cube,0,iy,iz,&valid_flag);
      if(valid_flag!=FLAG_YES) {
	ta=dummy_pixel;
      }
      fprintf(fp,"%f\n",ta);
    }

/* multi x; usial case */
  } else {

    for(iy=0;iy<my;iy++) {
      if(headline_flag==FLAG_YES) {
	yy=StarGetCubePixPosOnAxis(&((*cube).head.axis.y),iy);
	fprintf(fp,"%f%c",yy,separator);
      }
      for(ix=0;ix<mx-1;ix++) {
	ta=StarGetCubePixelData(cube,ix,iy,iz,&valid_flag);
	if(valid_flag!=FLAG_YES) {
	  ta=dummy_pixel;
	}
	fprintf(fp,"%f%c",ta,separator);
      }

      ta=StarGetCubePixelData(cube,mx-1,iy,iz,&valid_flag);
      if(valid_flag!=FLAG_YES) {
	ta=dummy_pixel;
      }
      fprintf(fp,"%f\n",ta);
    }
  }

  return;
}

int main(argc,argv)
int argc;
char *argv[];
{
  CUBE cube_in;
  FILE *fp;

  char id_str[MAX_FILE_NAME_LENGTH];
  char cdb_arg[MAX_IDX_RECORD_LENGTH];

  int ii;
  int n_file;
  int err_code;

  int cdb_flag;
  int pipe_flag;

  int headline_flag;
  char fullfilename[MAX_FILE_NAME_LENGTH],outfilename[MAX_FILE_NAME_LENGTH];
  char txt_filename[MAX_FILE_NAME_LENGTH];
  double z_level;
  int mz,iz;

  pipe_flag=FLAG_NO;

/* check option */
  pipe_flag=FLAG_NO;
  err_code=options(argc,argv,&headline_flag,&z_level,txt_filename);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

  n_file=0;

  StarPipeHeadIn(id_str,&n_file);

  if(pipe_flag==FLAG_YES) StarPipeHeadOut(n_file);

  if(strcmp(txt_filename,"-")==0 || strcmp(txt_filename,"stdout")==0)
    fp=stdout;
  else if(strcmp(txt_filename,"stderr")==0)
    fp=stderr;
  else
    fp=NULL;

  for(ii=0;ii<n_file;ii++) {

/* get cube from stdin */
    err_code=StarCubeIn(&cube_in,&cdb_flag,cdb_arg);
    if(err_code!=0) {
      if(pipe_flag==FLAG_YES) StarPipeEndOut();
      fprintf(stderr,"%s: fail to CubeIn No.%d\n",argv[0],ii);
      exit(1);
    }

    if(cdb_flag!=GOOD_SCAN && cdb_flag!=BAD_SCAN)
      goto next_cube;

    if(fp!=stdout && fp!=stderr) {
      /*
	StarExtractParticalFilename(outfilename,cube_in.head.file_name,FileNameProcess_WithoutExt);
	*/
      strcpy(outfilename,cube_in.head.file_name);
      StarStdFileExtName("STAR_DATA",outfilename,fullfilename,".txt");
      fp=fopen(fullfilename,"w");
    }

    iz=StarPixelIndexPositionOnAxis(&(cube_in.head.axis.z),z_level);
    mz=cube_in.head.axis.z.size;
    if(iz<0) iz=0;
    if(iz>=mz) iz=mz-1;

    if(headline_flag==FLAG_YES) 
      headline4map(fp,&(cube_in.head),iz,'\t');
    CubePixeldatavalue2Text(fp,&cube_in,iz,'\t',headline_flag);

    if(fp!=stdout && fp!=stderr) {
      fclose(fp);
    }
    StarFreeCubePixel(&cube_in);

next_cube:;
  }

  if(pipe_flag==FLAG_YES) StarPipeEndOut();
  exit(0);
}
