/*
convert Ta* and Tmb in a cube data

2003/8/30 T.Handa : ver.3.00
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"cubTa2Tmb.prm"
#define N_KEY 10

#define TA2TMB 1
#define TMB2TA 2
#define FLIP   0

int options(argc,argv,conv_direction)
/*
get parameters from options
*/
int argc;
char *argv[];
int *conv_direction; /* <out> convert direction, =TA2TMB: to Tmb, =TMB2TA: to Ta*, others: flip between Ta* and Tmb */
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  *conv_direction=FLIP;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"mode");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"to_Ta");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"to_Tmb");	key_type[n_key-1]=0;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* cliping range */
  n_scan=3;
  scan_flag[0]=0;
  scan_flag[1]=1;
  scan_flag[2]=2;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strncmp(got_value,"to_Tmb",6)==0 || strncmp(got_value,"Tmb",3)==0) {
      *conv_direction=TA2TMB;
    } else if(strncmp(got_value,"to_Ta",5)==0 || strncmp(got_value,"Ta",2)==0) {
      *conv_direction=TMB2TA;
    } else {
      *conv_direction=0;
    }
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return 0;
}

void ConvTa2Tmb(cube,conv_dir)
CUBE *cube;
int conv_dir;
{
  double ta;
  int ix,iy,iz;
  int valid_pix_flag;

  double conv_factor;


  switch(conv_dir) {
  case TA2TMB: /* Ta*-->Tmb */
    conv_factor=1.0/(double)(*cube).head.ant.mb_eff;
    strcpy((*cube).head.data_value.type,"Tmb");
    break;
  case TMB2TA: /* Tmb-->Ta* */
    conv_factor=(double)(*cube).head.ant.mb_eff;
    strcpy((*cube).head.data_value.type,"Ta*");
    break;
  default:
    return;
  }
  for(iz=0;iz<(int)(*cube).head.axis.z.size;iz++) {
    for(iy=0;iy<(int)(*cube).head.axis.y.size;iy++) {
      for(ix=0;ix<(int)(*cube).head.axis.x.size;ix++) {
	ta=StarGetCubePixelData(cube,ix,iy,iz,&valid_pix_flag);
	if(valid_pix_flag==FLAG_YES) {
	  ta=ta*conv_factor;
	  StarPutCubePixelData(cube,ix,iy,iz,ta);
	}
      }
    }
  }

  /* retune the statistics */
  (*cube).head.data_value.max=(float)((double)(*cube).head.data_value.max*conv_factor);
  (*cube).head.data_value.min=(float)((double)(*cube).head.data_value.min*conv_factor);
  (*cube).head.data_value.mean=(float)((double)(*cube).head.data_value.mean*conv_factor);
  (*cube).head.data_value.rms=(float)((double)(*cube).head.data_value.rms*conv_factor);
  (*cube).head.data_value.noise=(float)((double)(*cube).head.data_value.noise*conv_factor);

  return;
}

void AddCubeTa2TmbHistory(cube_head,conv_dir)
CUBE_HEAD *cube_head; /* <i/o> */
int conv_dir; /* <in> */
{
  HISTORY	history;
  char buffer[80];
  int i_prm;

  i_prm=0;
  strcpy(history.step,"Ta2Tmb");

  switch(conv_dir) {
  case TA2TMB:
    strcpy(buffer,"Ta*2Tmb");
    break;
  case TMB2TA:
    strcpy(buffer,"Tmb2Ta*");
    break;
  default:
    strcpy(buffer,"unknown");
  }
  buffer[16]=(char)NULL;
  strcpy(history.prm[i_prm],buffer);
  i_prm++;

  sprintf(buffer,"eff_mb=%f",(double)(*cube_head).ant.mb_eff);
  buffer[16]=(char)NULL;
  strcpy(history.prm[i_prm],buffer);
  i_prm++;

  history.size=i_prm;

  StarAddHistory4Cube(cube_head,&history);
}

int main(argc,argv)
int	argc;
char	*argv[];
{
  CUBE cube;
  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];
  int err_code;
  int n_file;
  char id_str[BUFFER_SIZE];

  int ii;

  int conv_dir,conv_dir_init;

/* get optional parameters */
  err_code=options(argc,argv,&conv_dir_init);

/* pipe header */
  StarPipeHeadIn(id_str,&n_file);

  StarPipeHeadOut(n_file);

/* .cub data */
  for(ii=0;ii<n_file;ii++) {

    StarCubeIn(&cube,&cdb_flag,cdb_arg);

    if(cdb_flag==GOOD_SCAN) {
      if(strcmp(cube.head.data_value.type,"Ta*")==0) {
	if(conv_dir_init==FLIP || conv_dir_init==TA2TMB) {
	  conv_dir=TA2TMB;
	  ConvTa2Tmb(&cube,conv_dir);
	  AddCubeTa2TmbHistory(&(cube.head),conv_dir);
	}
      } else if(strcmp(cube.head.data_value.type,"Tmb")==0) {
	if(conv_dir_init==FLIP || conv_dir_init==TMB2TA) {
	  conv_dir=TMB2TA;
	  ConvTa2Tmb(&cube,conv_dir);
	  AddCubeTa2TmbHistory(&(cube.head),conv_dir);
	}
      }
      StarCubeOut(&cube,cdb_flag,cdb_arg);

    } else if(cdb_flag==BAD_SCAN) {
      StarCubeOut(&cube,cdb_flag,cdb_arg);
    } else {
      StarCubeOut(&cube,cdb_flag,cdb_arg);
    }

    StarFreeCubePixel(&cube);
  }

/* end tag of pipe */
  StarPipeEndIn(id_str);
  StarPipeEndOut();

  exit(0);
}
