/*
cube data: replace blank pixels using another cub

2002/2/27 T.Handa : ver.3.00 
2002/5/11 T.Handa : ver.3.01 (adjust for revise of StarGetCubePixelData)
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"cubcat.prm"
#define N_KEY 30

#ifdef _NO_PROTO
extern int IsInRange();
extern void StarCubeConcatinate();
extern void AddCubcatHistory();

#else
extern int IsInRange(RANGE *,double *,int);
extern void StarCubeConcatinate(CUBE *,CUBE *);
extern void AddCubcatHistory(CUBE_HEAD *,char *);

#endif

int options(argc,argv,add_cubname)
/*
get parameters from options
*/
int argc;
char *argv[];
char *add_cubname;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  strcpy(add_cubname,"");

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"cubname");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"base_cub");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"base_cubname");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* add cubname */
  n_scan=3;
  scan_flag[0]=0;
  scan_flag[1]=1;
  scan_flag[2]=2;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(add_cubname,got_value);
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return 0;
}

void StarCubeConcatinate(cube_base,cube_add)
CUBE *cube_base;
CUBE *cube_add;
{
  int ix,iy,iz;
  int jx,jy,jz;
  int nx,ny,nz;
  int mx,my,mz;
  double xx,yy,zz;
  float dummy_base,dummy_add;
  double ta_base,ta_add;
  float pixel_base,pixel_add;
  int valid_data_flag;

  nx=(*cube_add).head.axis.x.size;
  ny=(*cube_add).head.axis.y.size;
  nz=(*cube_add).head.axis.z.size;
  mx=(*cube_base).head.axis.x.size;
  my=(*cube_base).head.axis.y.size;
  mz=(*cube_base).head.axis.z.size;

  dummy_base=(*cube_base).head.data_value.dummy;
  dummy_add=(*cube_add).head.data_value.dummy;

  for(ix=0;ix<nx;ix++) {
    xx=StarGetCubePixPosOnAxis(&((*cube_add).head.axis.x),ix);
    jx=(int)rint(StarPixelIndexPositionOnAxis(&((*cube_base).head.axis.x),xx));

    if(jx>=0 && jx<mx) {
      for(iy=0;iy<ny;iy++) {
	yy=StarGetCubePixPosOnAxis(&((*cube_add).head.axis.y),iy);
	jy=(int)rint(StarPixelIndexPositionOnAxis(&((*cube_base).head.axis.y),yy));
	if(jy>=0 && jy<my) {
	  for(iz=0;iz<nz;iz++) {
	    zz=StarGetCubePixPosOnAxis(&((*cube_add).head.axis.z),iz);
	    jz=(int)rint(StarPixelIndexPositionOnAxis(&((*cube_base).head.axis.z),zz));
	    if(jz>=0 && jz<mz) {
	      pixel_base=StarGetCubePixel(cube_base,jx,jy,jz);
	      pixel_add=StarGetCubePixel(cube_add,ix,iy,iz);
	      if(pixel_add!=dummy_add && pixel_base==dummy_base) {
		ta_add=StarGetCubePixelData(cube_add,ix,iy,iz,&valid_data_flag);
		StarPutCubePixelData(cube_base,jx,jy,jz,ta_add);
		ta_base=StarGetCubePixelData(cube_base,jx,jy,jz,&valid_data_flag);

	      }
	    }
	  }
	}
      }
    }
  }

  return;
}

void main_filename(mainname,filename)
char *mainname; /* <out> */
char *filename; /* <in> */
{
  int ii,l_str,i_top;

  strcpy(mainname,StarFileMainName(filename));

  l_str=strlen(mainname);
  i_top=l_str-4;
  if(i_top<0) i_top=0;
  for(ii=l_str-1;ii>=i_top;ii--) {
    if(mainname[ii]=='.') {
      mainname[ii]=(char)NULL;
      break;
    }
  }
  return;
}

void AddCubcatHistory(cube_head,add_cubname)
CUBE_HEAD *cube_head; /* <i/o> */
char *add_cubname; /* <in> */
{
  HISTORY	history;
  char buffer[80];
  int i_prm;

  strcpy(history.step,"concat");

  i_prm=0;

  strncpy(buffer,add_cubname,15);
  buffer[16]=(char)NULL;
  strcpy(history.prm[i_prm],buffer);
  i_prm++;

  history.size=i_prm;

  StarAddHistory4Cube(cube_head,&history);

  return;
}

int CheckCubeHead(cube1,cube2)
CUBE_HEAD *cube1; /* <in> */
CUBE_HEAD *cube2; /* <in> */
{
  int ret_code;
  ret_code=0;

  if(strcmp((*cube1).axis.x.type,(*cube2).axis.x.type)!=0) {
    fprintf(stderr,"different type for x-axis\n");
    ret_code=1;
  }
  return(ret_code);
}

int main(argc,argv)
int	argc;
char	*argv[];
{
  CUBE cube_base,cube_add;
  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];
  int err_code;
  int n_file;
  char id_str[BUFFER_SIZE];
  char add_cubname[MAX_FILE_NAME_LENGTH],main_cubname[MAX_FILE_NAME_LENGTH];

  int ii;

  FILE *fp_in;

/* get optional parameters */
  err_code=options(argc,argv,add_cubname);

  fp_in=fopen(add_cubname,"r");
  if(fp_in==NULL) {
    fprintf(stderr,"**ERR** file %s is not found.\n",add_cubname);
    exit(99);
  }
  err_code=StarCubeHeadLoad(fp_in,&(cube_add.head));
  if(err_code!=0) {
    fprintf(stderr,"**ERR** failed to load header data for %s\n",add_cubname);
    exit(99);
  }
  cube_add.pixel=StarAllocCubePixel(&(cube_add.head));
  if(cube_add.pixel==NULL) {
    fprintf(stderr,"**ERR** cannot alloc pixel data for %s\n",add_cubname);
    exit(99);
  }
  err_code=StarCubePixelLoad(fp_in,&cube_add);
  if(err_code!=0) {
    fprintf(stderr,"**ERR** failed to load pixel data for %s\n",add_cubname);
    exit(99);
  }
  fclose(fp_in);

/* pipe header */
  StarPipeHeadIn(id_str,&n_file);

  StarPipeHeadOut(n_file);

/* .cub data */
  for(ii=0;ii<n_file;ii++) {

    StarCubeIn(&cube_base,&cdb_flag,cdb_arg);

      /*      CheckCubeHead(&(cube_base.head),&(cube_add.head)); */

    StarCubeConcatinate(&cube_base,&cube_add);

    main_filename(main_cubname,add_cubname);
    if(strlen(main_cubname)>16) main_cubname[16]=(char)NULL;
    AddCubcatHistory(&(cube_base.head),main_cubname); 
    StarCubeOut(&cube_base,cdb_flag,cdb_arg);
  }

/* end tag of pipe */
  StarPipeEndIn(id_str);
  StarPipeEndOut();

  exit(0);
}
