/*
clip a cube data

1999/10/27 T.Handa : ver.3.00
1999/12/27 T.Handa : ver.3.02 T and pixel value
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"cubclip.prm"
#define N_KEY 30

int options(argc,argv,clip_range,clip_unit)
/*
get parameters from options
*/
int argc;
char *argv[];
RANGE *clip_range; /* <out> cliping range (asigned by allow range) */
int *clip_unit; /* <out> clip unit =0 in pixel, =1 in physical */
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
/*  for(ii=0;ii<3;ii++) {
    clip_range[ii].min=0.0;
    clip_range[ii].max=0.0;
  }
*/
  (*clip_range).min=0.0;
  (*clip_range).max=0.0;
  *clip_unit=1;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"T");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"ta_range");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"clip_range");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"pixel_value_range");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* cliping range */
  n_scan=4;
  scan_flag[0]=0;
  scan_flag[1]=1;
  scan_flag[2]=2;
  scan_flag[3]=3;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    StarOptionAngleRange(got_value,clip_range);
    StarSortRange(clip_range);

    if(strcmp(argv[StarOptionKeyArgPosition(n_scan,scan_flag,key_flag)],"pixel_value_range")==0) {
      *clip_unit=0;
    } else {
      *clip_unit=1;
    }
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return 0;
}

void ClipCube(cube,clip_range,clip_unit)
CUBE *cube;
RANGE clip_range;
int clip_unit;
{
  float ta,dummy;
  int ix,iy,iz;

  dummy=(*cube).head.data_value.dummy;

  for(iz=0;iz<(int)(*cube).head.axis.z.size;iz++) {
    for(iy=0;iy<(int)(*cube).head.axis.y.size;iy++) {
      for(ix=(int)(*cube).head.axis.x.size-1;ix>=0;ix--) {
	ta=StarGetCubePixel(cube,ix,iy,iz);

	if(clip_unit!=0) {
	  ta=ta*(*cube).head.data_value.scale+(*cube).head.data_value.offset;
	}

	if(ta!=dummy) {
	  if(ta<clip_range.min || ta>clip_range.max) {
	    StarPutCubePixel(cube,ix,iy,iz,dummy);
	  }
	}

      }
    }
  }
}

void AddCubeClipHistory(cube_head,clip_range)
CUBE_HEAD *cube_head; /* <i/o> */
RANGE clip_range; /* <in> */
{
  HISTORY	history;
  char buffer[80];
  int i_prm;

  i_prm=0;
  strcpy(history.step,"clip");

  sprintf(buffer,"min=%7.3f",clip_range.min);
  buffer[16]=(char)NULL;
  strcpy(history.prm[i_prm],buffer);
  i_prm++;

  sprintf(buffer,"max=%7.3f",clip_range.max);
  buffer[16]=(char)NULL;
  strcpy(history.prm[i_prm],buffer);
  i_prm++;

  history.size=i_prm;

  StarAddHistory4Cube(cube_head,&history);
}

int main(argc,argv)
int	argc;
char	*argv[];
{
  CUBE cube;
  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];
  int err_code;
  int n_file;
  char id_str[BUFFER_SIZE];

  RANGE	clip_range;

  int ii;

  int clip_unit;

/* get optional parameters */
  err_code=options(argc,argv,&clip_range,&clip_unit);

/* pipe header */
  StarPipeHeadIn(id_str,&n_file);

  StarPipeHeadOut(n_file);

/* .cub data */
  for(ii=0;ii<n_file;ii++) {

    StarCubeIn(&cube,&cdb_flag,cdb_arg);

    if(cdb_flag==GOOD_SCAN) {
      ClipCube(&cube,clip_range,clip_unit);
      AddCubeClipHistory(&(cube.head),clip_range);
      StarCubeOut(&cube,cdb_flag,cdb_arg);

    } else if(cdb_flag==BAD_SCAN) {
      StarCubeOut(&cube,cdb_flag,cdb_arg);
    } else {
      StarCubeOut(&cube,cdb_flag,cdb_arg);
    }

    StarFreeCubePixel(&cube);
  }

/* end tag of pipe */
  StarPipeEndIn(id_str);
  StarPipeEndOut();

  exit(0);
}
