/*
clip a cube data

1999/10/27 T.Handa : ver.3.00
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStdio.h>
#include <StStdlib.h>
#include <StPipe.h>
#include <StCubeIO.h>
#include <StCubeProc.h>

#define	DEFAULT_PRMFILE	"cubclip.prm"
#define N_KEY 30

int options(argc,argv,clip_range)
/*
get parameters from options
*/
int argc;
char *argv[];
RANGE *clip_range; /* <out> cliping range (asigned by allow range) */
{
  int ii,jj;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  for(ii=0;ii<3;ii++) {
    clip_range[ii].min=0.0;
    clip_range[ii].max=0.0;
  }

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"T");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"ta_range");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"clip_range");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* cliping range */
  n_scan=3;
  scan_flag[0]=0;
  scan_flag[1]=1;
  scan_flag[2]=2;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=NULL) {
    StarOptionAngleRange(got_value,clip_range);
    StarSortRange(clip_range);
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return 0;
}

void ClipCube(cube,clip_range)
CUBE *cube;
RANGE clip_range;
{
  float ta,dummy;
  int ix,iy,iz;

  dummy=(*cube).head.data_value.dummy;

  for(ix=0;ix<(int)(*cube).head.axis.x.size;ix++) {
    for(iy=0;iy<(int)(*cube).head.axis.y.size;iy++) {
      for(iz=0;iz<(int)(*cube).head.axis.z.size;iz++) {
	ta=StarGetCubePixel(cube,ix,iy,iz);
	if(ta!=dummy) {

if(iy==0 && iz==367) fprintf(stderr,"(%d,%d,%d)=%f/%f",ix,iy,iz,ta,dummy);

	  if(ta<clip_range.min || ta>clip_range.max)
	    StarPutCubePixel(cube,ix,iy,iz,dummy);
	}
      }
    }
  }
}

void AddCubeClipHistory(cube_head,clip_range)
CUBE_HEAD *cube_head; /* <i/o> */
RANGE clip_range; /* <in> */
{
  HISTORY	history;
  char buffer[80];
  int ii;
  int i_prm;

  i_prm=0;
  strcpy(history.step,"clip");

  sprintf(buffer,"min=%7.3f",clip_range.min);
  buffer[16]=NULL;
  strcpy(history.prm[i_prm],buffer);
  i_prm++;

  sprintf(buffer,"max=%7.3f",clip_range.max);
  buffer[16]=NULL;
  strcpy(history.prm[i_prm],buffer);
  i_prm++;

  history.size=i_prm;

  StarAddHistory4Cube(cube_head,&history);
}

int main(argc,argv)
int	argc;
char	*argv[];
{
  CUBE cube;
  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];
  int err_code;
  int i_file,n_file;
  char id_str[BUFFER_SIZE];

  RANGE	clip_range;
  float **pixel_data;

  int ii,jj;

/* get optional parameters */
  err_code=options(argc,argv,&clip_range);

/* pipe header */
  StarPipeHeadIn(id_str,&n_file);

  StarPipeHeadOut(n_file);

/* .cub data */
  for(ii=0;ii<n_file;ii++) {

    StarCubeIn(&cube,&cdb_flag,cdb_arg);

    if(cdb_flag==GOOD_SCAN) {
      ClipCube(&cube,clip_range);
      AddCubeClipHistory(&(cube.head),clip_range);
      StarCubeOut(&cube,cdb_flag,cdb_arg);

    } else if(cdb_flag==BAD_SCAN) {
      StarCubeOut(&cube,cdb_flag,cdb_arg);
    } else {
      StarCubeOut(&cube,cdb_flag,cdb_arg);
    }

    StarFreeCubePixel(&cube);
  }

/* end tag of pipe */
  StarPipeEndIn(id_str);
  StarPipeEndOut();

  exit(0);
}
