/*
edot cube by hand

2006/2/12 T.Handa : ver.3.00
2006/5/22 T.Handa : ver.3.01 (debug) core dump via ssh; 2D array should be static
2006/7/27 T.Handa : ver.3.02 (modify) line interpolation
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define DEBUG
#undef DEBUG

#define	DEFAULT_PRMFILE	"cubeditk.prm"
#define N_KEY 30

#define DIRECTION_X 0
#define DIRECTION_Y 1
#define DIRECTION_Z 2

#define MAX_FIT_DIM 10

static int options(int, char **,char *,int *);
static void print_cmdlist(void);
static void main_filename(char *,char *);
extern void StarCubeSubtract(CUBE *, CUBE *);
extern int StarCubeShowWith1Digit(CUBE *,int,RANGE,RANGE_long,RANGE_long);
extern int StarCubeShowWithText(CUBE *,int,RANGE_long,RANGE_long);
extern int StarCubeBoxEdit(CUBE *,int,RANGE_long,RANGE_long,double);
extern int StarCubeBoxDummy(CUBE *,int,RANGE_long,RANGE_long);
extern int StarCubeInterpolateAlongX(CUBE *, int, int, int); 
extern int StarCubeInterpolateAlongY(CUBE *, int, int, int); 
static int get_box_area(char *,RANGE_long *,RANGE_long *);
static void print_cmdlist(void);
void StarCubeCopyPixel(CUBE *, CUBE *);

int options(argc,argv,std_cubname,scan_dir)
/*
get parameters from options
*/
int argc;
char *argv[];
char *std_cubname;
int *scan_dir;
{
  int ii,next_pos;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;
  int l_str;

/* default values */
  strcpy(std_cubname,"");
  *scan_dir=DIRECTION_X;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"cube");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"cubname");	key_type[n_key-1]=1;

  n_key=AddNewWord(keyword,"scan_dir");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"x_scan");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"y_scan");	key_type[n_key-1]=0;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* std cubname */
  next_pos=0;
  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(std_cubname,got_value);
  }

/* scan direction */
  n_scan=3;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    l_str=strlen(got_value);
    if(strncmp(got_value,"x",1)==0) {
      *scan_dir=DIRECTION_X;
    } else {
      *scan_dir=DIRECTION_Y;
    }
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return 0;
}

void main_filename(mainname,filename)
char *mainname; /* <out> */
char *filename; /* <in> */
{
  int ii,l_str,i_top;

  strcpy(mainname,StarFileMainName(filename));

  l_str=strlen(mainname);
  i_top=l_str-4;
  if(i_top<0) i_top=0;
  for(ii=l_str-1;ii>=i_top;ii--) {
    if(mainname[ii]=='.') {
      mainname[ii]=(char)NULL;
      break;
    }
  }
  return;
}

void StarCubeSubtract(cube_diff,cube_base)
CUBE *cube_diff; /* <i/o> */
CUBE *cube_base; /* <in> */
{
  int ix,iy,iz;
  int mx,my,mz;
  double ta_base,ta_diff;
  int valid_flag_base,valid_flag_diff;
  float dummy_pixel;

  mx=(*cube_base).head.axis.x.size;
  my=(*cube_base).head.axis.y.size;
  mz=(*cube_base).head.axis.z.size;

  dummy_pixel=(*cube_diff).head.data_value.dummy;

  for(ix=0;ix<mx;ix++) {
    for(iy=0;iy<my;iy++) {
      for(iz=0;iz<mz;iz++) {
	ta_base=StarGetCubePixelData(cube_base,ix,iy,iz,&valid_flag_base);
	ta_diff=StarGetCubePixelData(cube_diff,ix,iy,iz,&valid_flag_diff);
	if(valid_flag_base==FLAG_YES && valid_flag_diff==FLAG_YES) {
	  ta_diff=ta_diff-ta_base;
	  StarPutCubePixelData(cube_diff,ix,iy,iz,ta_diff);
	} else {
	  StarPutCubePixel(cube_diff,ix,iy,iz,dummy_pixel);
	}
      }
    }
  }

  return;
}

void AddCubDespikeHistory(add_head,base_head)
CUBE_HEAD *add_head; /* <i/o> */
CUBE_HEAD *base_head; /* <in> */
{
  static char main_cubname[2][MAX_FILE_NAME_LENGTH];
  HISTORY	history;
  char buffer[80];
  int i_prm;

  strcpy(history.step,"spike");

  i_prm=0;

/* process */
  strcpy(buffer,"autoflag");
  strcpy(history.prm[i_prm],buffer);
  i_prm++;

/* base and add cube */
  strcpy(buffer,"A and B are");
  buffer[16]=(char)NULL;
  strcpy(history.prm[i_prm],buffer);
  i_prm++;

  strncpy(buffer,main_cubname[0],15);
  buffer[16]=(char)NULL;
  strcpy(history.prm[i_prm],buffer);
  i_prm++;
  strncpy(buffer,main_cubname[1],15);
  buffer[16]=(char)NULL;
  strcpy(history.prm[i_prm],buffer);
  i_prm++;

  history.size=i_prm;

  StarAddHistory4Cube(add_head,&history);
}

int StarCubeShowWith1Digit(cube,iz,show_range,x_range,y_range)
CUBE *cube; /* <in> */
int iz; /* <in> depth along z-axis */
RANGE show_range; /* <in> show_range */
RANGE_long x_range,y_range; /* <in> */
{
  int ix,iy;
  int mx,my,mz;
  double pixel_value;
  int valid_flag;

  int pix_digit;
  double span10;

  mx=(*cube).head.axis.x.size;
  my=(*cube).head.axis.y.size;
  mz=(*cube).head.axis.z.size;

  if(iz<0) iz=0;
  if(iz>=mz) iz=mz-1;

  if(x_range.min<0) x_range.min=0;
  if(x_range.max>=mx) x_range.max=mx-1;
  if(y_range.min<0) y_range.min=0;
  if(y_range.max>=my) y_range.max=my-1;

  span10=(show_range.max-show_range.min)/10.0;

  /* print x-axis */
  fprintf(stderr,"x-");
  for(ix=x_range.min;ix<=x_range.max;ix++) {
    if(ix==ix/10*10) {
      fprintf(stderr,"%1d",ix/10-ix/100*10);
    } else {
      fprintf(stderr,"%c",(char)(ix-ix/10*10-1)+'A');
    }
  }
  fprintf(stderr,"\n");

  for(iy=y_range.min;iy<=y_range.max;iy++) {
  /* y-axis */
    if(iy==iy/10*10) {
      fprintf(stderr,"%1d ",iy/10-iy/100*10);
    } else {
      fprintf(stderr,"%c ",(char)(iy-iy/10*10-1)+'A');
    }
    /* map body */
    for(ix=x_range.min;ix<=x_range.max;ix++) {
      pixel_value=StarGetCubePixelData(cube,ix,iy,iz,&valid_flag);
      if(valid_flag==FLAG_YES) {
	pix_digit=(int)((pixel_value-show_range.min)/span10+0.5);
	if(pix_digit==0)
	  fprintf(stderr,".");
	else if(pix_digit<-9)
	  fprintf(stderr,"N");
	else if(pix_digit<0)
	  fprintf(stderr,"-");
	else if(pix_digit>9)
	  fprintf(stderr,"*");
	else
	  fprintf(stderr,"%1d",pix_digit);
      } else {
	fprintf(stderr,"X");
      }
    }
    fprintf(stderr,"\n");
  }

  /* x-axis */
  fprintf(stderr,"x-");
  for(ix=x_range.min;ix<=x_range.max;ix++) {
    if(ix==ix/10*10) {
      fprintf(stderr,"%1d",ix/10-ix/100*10);
    } else {
      fprintf(stderr,"%c",(char)(ix-ix/10*10-1)+'A');
    }
  }
  fprintf(stderr,"\n");

  return(0);
}

int StarCubeShowWithText(cube,iz,x_range,y_range)
CUBE *cube; /* <in> */
int iz; /* <in> depth along z-axis */
RANGE_long x_range,y_range; /* <in> */
{
  int ix,iy;
  int mx,my,mz;
  double pixel_value;
  int valid_flag;

  mx=(*cube).head.axis.x.size;
  my=(*cube).head.axis.y.size;
  mz=(*cube).head.axis.z.size;

  if(iz<0) iz=0;
  if(iz>=mz) iz=mz-1;

  if(x_range.min<0) x_range.min=0;
  if(x_range.max>=mx) x_range.max=mx-1;
  if(y_range.min<0) y_range.min=0;
  if(y_range.max>=my) y_range.max=my-1;

  /* print x-axis */
  fprintf(stderr,"x-- ");
  for(ix=x_range.min;ix<=x_range.max;ix++) {
    fprintf(stderr,"%13d ",ix);
  }
  fprintf(stderr,"\n");

  for(iy=y_range.min;iy<=y_range.max;iy++) {
  /* y-axis */
    fprintf(stderr,"%3d ",iy);
    /* map body */
    for(ix=x_range.min;ix<=x_range.max;ix++) {
      pixel_value=StarGetCubePixelData(cube,ix,iy,iz,&valid_flag);
      if(valid_flag==FLAG_YES) {
	fprintf(stderr,"%13.6e ",pixel_value);
      } else {
	fprintf(stderr,"  dummy       ");
      }
    }
    fprintf(stderr,"\n");
  }

  return(0);
}

int StarCubeBoxEdit(cube,iz,x_range,y_range,pixel_value)
CUBE *cube; /* <in> */
int iz; /* <in> depth along z-axis */
RANGE_long x_range,y_range; /* <in> */
double pixel_value;
{
  int ix,iy;
  int mx,my,mz;

  mx=(*cube).head.axis.x.size;
  my=(*cube).head.axis.y.size;
  mz=(*cube).head.axis.z.size;

  if(iz<0) iz=0;
  if(iz>=mz) iz=mz-1;

  if(x_range.min<0) x_range.min=0;
  if(x_range.max>=mx) x_range.max=mx-1;
  if(y_range.min<0) y_range.min=0;
  if(y_range.max>=my) y_range.max=my-1;

  for(iy=y_range.min;iy<=y_range.max;iy++) {
    for(ix=x_range.min;ix<=x_range.max;ix++) {
      StarPutCubePixelData(cube,ix,iy,iz,pixel_value);
    }
  }

  return(0);
}

int StarCubeBoxDummy(cube,iz,x_range,y_range)
CUBE *cube; /* <in> */
int iz; /* <in> depth along z-axis */
RANGE_long x_range,y_range; /* <in> */
{
  int ix,iy;
  int mx,my,mz;

  mx=(*cube).head.axis.x.size;
  my=(*cube).head.axis.y.size;
  mz=(*cube).head.axis.z.size;

  if(iz<0) iz=0;
  if(iz>=mz) iz=mz-1;

  if(x_range.min<0) x_range.min=0;
  if(x_range.max>=mx) x_range.max=mx-1;
  if(y_range.min<0) y_range.min=0;
  if(y_range.max>=my) y_range.max=my-1;

  for(iy=y_range.min;iy<=y_range.max;iy++) {
    for(ix=x_range.min;ix<=x_range.max;ix++) {
      StarPutCubePixelDummy(cube,ix,iy,iz);
    }
  }

  return(0);
}

int get_box_area(word,box_x,box_y)
char *word; /* <in> */
RANGE_long *box_x; /* <out> */
RANGE_long *box_y; /* <out> */
{
  char *token;
  char work[256];

  strcpy(work,word);

  token=strtok(work,",\n");
  if(token==NULL) {
    (*box_x).min=0;
    (*box_y).min=0;
    (*box_x).max=0;
    (*box_y).max=0;
    return(10);
  }
  (*box_x).min=atoi(token);

  token=strtok(NULL," -\n");
  if(token==NULL) {
    (*box_y).min=0;
    (*box_x).max=0;
    (*box_y).max=0;
    return(10);
  }
  (*box_y).min=atoi(token);

  token=strtok(NULL,",\n");
  if(token==NULL) {
    (*box_x).max=(*box_x).min;
    (*box_y).max=(*box_y).min;
    return(10);
  }
  (*box_x).max=atoi(token);

  token=strtok(NULL," \n");
  if(token==NULL) {
    (*box_y).max=(*box_y).min;
    return(10);
  }
  (*box_y).max=atoi(token);

  StarSortRangeLong(box_x);
  StarSortRangeLong(box_y);

  return(0);
}

void StarCubeCopyPixel(cube_new,cube_org)
CUBE *cube_new; /* <out> */
CUBE *cube_org; /* <in> */
{
  int nx,ny,nz;
  int ix,iy,iz;
  double ta;
  int valid_pixel_flag;
  float dummy;

  nx=(*cube_org).head.axis.x.size;
  ny=(*cube_org).head.axis.y.size;
  nz=(*cube_org).head.axis.z.size;

  dummy=(*cube_new).head.data_value.dummy;

  for(iz=0;iz<nz;iz++) {
    for(iy=0;iy<ny;iy++) {
      for(ix=0;ix<nx;ix++) {
	ta=StarGetCubePixelData(cube_org,ix,iy,iz,&valid_pixel_flag);
	if(valid_pixel_flag==FLAG_YES) {
	  StarPutCubePixelData(cube_new,ix,iy,iz,ta);
	} else {
	  StarPutCubePixel(cube_new,ix,iy,iz,dummy);
	}
      }
    }
  }

  return;
}

int StarCubeInterpolateAlongX(cube,ix,iz,allflag)
CUBE *cube; /* <i/o> */
int ix,iz; /* <in> */
int allflag; /* <in> =FLAG_YES all pixels, =FLAG_NO only dummy pixel */
{
  int iy;
  int mx,my,mz;
  float dummy_pixel;
  float *line1,*line2,*line3;

  mx=(*cube).head.axis.x.size;
  my=(*cube).head.axis.y.size;
  mz=(*cube).head.axis.z.size;

  if(ix<1 || ix>=mx-1) {
    fprintf(stderr,"invalid iy.\n");
    return(1);
  }
  if(iz<0 || iz>=mz) {
    fprintf(stderr,"invalid iz.\n");
    return(1);
  }

  dummy_pixel=(*cube).head.data_value.dummy;

  line1=malloc(my*sizeof(float));
  line2=malloc(my*sizeof(float));
  line3=malloc(my*sizeof(float));

  for(iy=0;iy<my;iy++) {
    line1[iy]=StarGetCubePixel(cube,ix-1,iy,iz);
  }
  for(iy=0;iy<my;iy++) {
    line3[iy]=StarGetCubePixel(cube,ix+1,iy,iz);
  }
  for(iy=0;iy<my;iy++) {
    if(line1[iy]==dummy_pixel || line3[iy]==dummy_pixel) {
      line2[iy]=dummy_pixel;
    } else {
      line2[iy]=(line1[iy]+line3[iy])/2.0;
    }
  }

  if(allflag==FLAG_YES) {
    for(iy=0;iy<my;iy++) {
      StarPutCubePixel(cube,ix,iy,iz,line2[iy]);
    }
  } else {
      for(iy=0;iy<my;iy++) {
      if(StarGetCubePixel(cube,ix,iy,iz)==dummy_pixel) {
	StarPutCubePixel(cube,ix,iy,iz,line2[iy]);
      }
    }
  }

  free(line3);
  free(line2);
  free(line1);

  return(0);
}

int StarCubeInterpolateAlongY(cube,iy,iz,allflag)
CUBE *cube; /* <i/o> */
int iy,iz; /* <in> */
int allflag; /* <in> =FLAG_YES all pixels, =FLAG_NO only dummy pixel */
{
  int ix;
  int mx,my,mz;
  float dummy_pixel;
  float *line1,*line2,*line3;

  mx=(*cube).head.axis.x.size;
  my=(*cube).head.axis.y.size;
  mz=(*cube).head.axis.z.size;

  if(iy<1 || iy>=my-1) {
    fprintf(stderr,"invalid iy.\n");
    return(1);
  }
  if(iz<0 || iz>=mz) {
    fprintf(stderr,"invalid iz.\n");
    return(1);
  }

  dummy_pixel=(*cube).head.data_value.dummy;

  line1=malloc(mx*sizeof(float));
  line2=malloc(mx*sizeof(float));
  line3=malloc(mx*sizeof(float));

  for(ix=0;ix<mx;ix++) {
    line1[ix]=StarGetCubePixel(cube,ix,iy-1,iz);
  }
  for(ix=0;ix<mx;ix++) {
    line3[ix]=StarGetCubePixel(cube,ix,iy+1,iz);
  }
  for(ix=0;ix<mx;ix++) {
    if(line1[ix]==dummy_pixel || line3[ix]==dummy_pixel) {
      line2[ix]=dummy_pixel;
    } else {
      line2[ix]=(line1[ix]+line3[ix])/2.0;
    }
  }

  if(allflag==FLAG_YES) {
    for(ix=0;ix<mx;ix++) {
      StarPutCubePixel(cube,ix,iy,iz,line2[ix]);
    }
  } else {
    for(ix=0;ix<mx;ix++) {
      if(StarGetCubePixel(cube,ix,iy,iz)==dummy_pixel) {
	StarPutCubePixel(cube,ix,iy,iz,line2[ix]);
      }
    }
  }

  free(line3);
  free(line2);
  free(line1);

  return(0);
}

void print_cmdlist()
{
  fprintf(stderr,"z value   : set the plain depth along z-axis\n");
  fprintf(stderr,"map       : show a whole map with ASCII char\n");
  fprintf(stderr,"map x,y   : show a 11x11 map centered at (x,y)\n");
  fprintf(stderr,"print x,y : show a 5x5 map centered at (x,y)\n");
  fprintf(stderr,"sense max,min : sensitivity to show a map\n");
  fprintf(stderr,"set b x,y : put pixel value to b at (x,y)\n");
  fprintf(stderr,"del x,y   : set dummy pixel at (x,y)\n");
  fprintf(stderr,"del x1,y1 x2,y2 : set dummy pixel in box (x1,y1)-(x2,y2)\n");
  fprintf(stderr,"interp x x1 all : interpolate along x for line x1 (all: for all pixels)\n");
  fprintf(stderr,"interp y y1 all : interpolate along y for line y1\n");
  fprintf(stderr,"undo      : undo the last editting process\n");
  fprintf(stderr,"exit      : end with pipeout\n");
  fprintf(stderr,"quit      : end without pipeout\n");
  return;
}

int main(argc,argv)
int argc;
char *argv[];
{
  CUBE cube_main,cube_backup;
  static int cdb_flag=GOOD_SCAN;
  static char cdb_arg[]="";
  char std_cubname[MAX_FILE_NAME_LENGTH];
  int err_code;
  char command[BUFFER_SIZE];
  int scan_dir;
  int l_str;

  double pixel_value;
  char *token;

  FILE *fp_in;

  static int z_position=0;
  RANGE sense_range;
  RANGE map_center;
  static RANGE_long show_x,show_y;
  static RANGE_long print_x,print_y;
  static RANGE_long edit_box_x,edit_box_y;

  int ix,iy,allflag;

/* get optional parameters */
  err_code=options(argc,argv,std_cubname,&scan_dir);

/* pipe header */
  fp_in=fopen(std_cubname,"r");
  if(fp_in==NULL) {
    fprintf(stderr,"**ERR** file %s is not found.\n",std_cubname);
    exit(99);
  }
  err_code=StarCubeHeadLoad(fp_in,&(cube_main.head));
  if(err_code!=0) {
    fprintf(stderr,"**ERR** failed to load header data for %s\n",std_cubname);
    exit(99);
  }
  cube_main.pixel=StarAllocCubePixel(&(cube_main.head));
  if(cube_main.pixel==NULL) {
    fprintf(stderr,"**ERR** cannot alloc pixel data for %s\n",std_cubname);
    exit(99);
  }
  err_code=StarCubePixelLoad(fp_in,&cube_main);
  if(err_code!=0) {
    fprintf(stderr,"**ERR** failed to load pixel data for %s\n",std_cubname);
    exit(99);
  }
  fclose(fp_in);
  main_filename(cube_main.head.file_name,std_cubname);
  strcpy(cube_main.head.file_id,STAR_CUB_IN);

/* make backup */
  StarCopyCubeHead(&(cube_backup.head),&(cube_main.head));
  cube_backup.pixel=StarAllocCubePixel(&(cube_backup.head));
  if(cube_backup.pixel==NULL) {
    fprintf(stderr,"**ERR** cannot alloc pixel data for %s\n",std_cubname);
    exit(99);
  }

/* .cub data */
  sense_range.min=cube_main.head.data_value.min;
  sense_range.max=cube_main.head.data_value.max;

  while(1) {
    fprintf(stderr,"cubeditk>");
    fgets(command,BUFFER_SIZE,stdin);
    l_str=strlen(command);
    if(command[l_str-1]=='\n') command[l_str-1]=(char)NULL;
    if(strcmp(command,"exit")==0 || strcmp(command,"quit")==0) {
      break;
    } else if(strncmp(command,"map",3)==0) {
      if(strcmp(command,"map")!=0) {
	StarOptionRange(&(command[3]),&map_center);
	show_x.min=(int)(map_center.min+0.5)-5;
	show_x.max=(int)(map_center.min+0.5)+5;
	show_y.min=(int)(map_center.max+0.5)-5;
	show_y.max=(int)(map_center.max+0.5)+5;
      } else {
	show_x.min=0;
	show_x.max=cube_main.head.axis.x.size;
	show_y.min=0;
	show_y.max=cube_main.head.axis.y.size;
      }
      StarCubeShowWith1Digit(&cube_main,z_position,sense_range,show_x,show_y);
    } else if(strncmp(command,"z ",2)==0) {
      z_position=atoi(&(command[2]));
    } else if(strncmp(command,"sense",5)==0) {
      fprintf(stderr,"map max=%f,min=%f\n",cube_main.head.data_value.max,cube_main.head.data_value.min);
      if(strcmp(command,"sense")==0) {
	sense_range.min=cube_main.head.data_value.min;
	sense_range.max=cube_main.head.data_value.max;
      } else {
	StarOptionRange(&(command[6]),&sense_range);
	StarSortRange(&sense_range);
      }
    } else if(strncmp(command,"print",5)==0) {
      if(strcmp(command,"print")!=0) {
	StarOptionRange(&(command[5]),&map_center);
	print_x.min=(int)(map_center.min+0.5)-2;
	print_x.max=(int)(map_center.min+0.5)+2;
	print_y.min=(int)(map_center.max+0.5)-2;
	print_y.max=(int)(map_center.max+0.5)+2;
      } else {
	print_x.min=0;
	print_x.max=cube_main.head.axis.x.size;
	print_y.min=0;
	print_y.max=cube_main.head.axis.y.size;
      }
      StarCubeShowWithText(&cube_main,z_position,print_x,print_y);
    } else if(strncmp(command,"del ",4)==0) {
      StarCubeCopyPixel(&cube_backup,&cube_main);

      get_box_area(&(command[4]),&edit_box_x,&edit_box_y);

      fprintf(stderr,"box=(%ld,%ld)-(%ld,%ld)\n",edit_box_x.min,edit_box_y.min,edit_box_x.max,edit_box_y.max);
      StarCubeBoxDummy(&cube_main,z_position,edit_box_x,edit_box_y);
    } else if(strncmp(command,"set ",4)==0) {
      StarCubeCopyPixel(&cube_backup,&cube_main);
      token=strtok(&(command[4])," ");
      pixel_value=atof(token);
      token=strtok(NULL," ");
      if(token!=NULL) {
	get_box_area(token,&edit_box_x,&edit_box_y);

	fprintf(stderr,"box=(%ld,%ld)-(%ld,%ld), value=%f\n",edit_box_x.min,edit_box_y.min,edit_box_x.max,edit_box_y.max,pixel_value);
	StarCubeBoxEdit(&cube_main,z_position,edit_box_x,edit_box_y,pixel_value);
      } else {
	fprintf(stderr,"pos is required!\n");
      }
    } else if(strncmp(command,"interp",6)==0) {
      if(strncmp(&(command[7]),"y ",2)==0) {
	iy=atoi(&(command[9]));
	l_str=strlen(command);
	if(strcmp(&(command[l_str-3]),"all")==0) {
	  allflag=FLAG_YES;
	  fprintf(stderr,"for all pixels in the line y=%d\n",iy);
	} else {
	  allflag=FLAG_NO;
	  fprintf(stderr,"for dummy pixels in the line y=%d\n",iy);
	}
	StarCubeInterpolateAlongY(&cube_main,iy,z_position,allflag);
      } else if(strncmp(&(command[7]),"x ",2)==0) {
	ix=atoi(&(command[9]));
	l_str=strlen(command);
	if(strcmp(&(command[l_str-3]),"all")==0) {
	  allflag=FLAG_YES;
	  fprintf(stderr,"for all pixels in the line x=%d\n",ix);
	} else {
	  allflag=FLAG_NO;
	  fprintf(stderr,"for dummy pixels in the line x=%d\n",ix);
	}
	StarCubeInterpolateAlongX(&cube_main,ix,z_position,allflag);
      }
    } else if(strncmp(command,"undo",4)==0) {
      StarCubeCopyPixel(&cube_main,&cube_backup);
    } else if(strcmp(command,"help")==0 || command[0]=='?') {
      print_cmdlist();
    } else {
      fprintf(stderr,"Invalid command\n");
      print_cmdlist();
    }
    
  }
  if(strcmp(command,"quit")!=0) {
    StarPipeHeadOut(1);
    StarCubeOut(&cube_main,cdb_flag,cdb_arg);
    StarPipeEndOut();
  }

  StarFreeCubePixel(&cube_main);

  /* end tag of pipe */
  exit(0);
}
