/*
cub head correcter
only for OHCUBE.FIT

1999/4/22 T.Handa : ver.3.00
1999/8/13 T.Handa
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StStdio.h>
#include <StStdlib.h>
#include <StPipe.h>
#include <StIndex.h>
#include <StData.h>
#include <StCubeIO.h>
#include <StCubeProc.h>

#define	DEFAULT_PRMFILE	"headcorr.prm"
#define N_KEY 50
#define N_BLOCK	10

void tmp_proc(cube)
CUBE *cube;
{
  int ii;

fprintf(stderr,"%d\n",(*cube).head.axis.y.size);
  for(ii=0;ii<(*cube).head.axis.y.size;ii++) {
    fprintf(stderr,"[0,%d,0]=%f,",ii,(double)StarGetCubePixel(cube,0,ii,0));
  }
}

void adjustBeStatus(head)
CUBE_HEAD *head;
{
  int ii;
  AXIS *axis;
  double freq_span;

  RANGE v_range;
  double obs_freq,center_freq;

  for(ii=0;ii<(*head).axis.dimension;ii++) {
    switch(ii) {
    case 0:
      axis= &((*head).axis.x);
      break;
    case 1:
      axis= &((*head).axis.y);
      break;
    case 2:
      axis= &((*head).axis.z);
      break;
    }

    if(strcmp((*axis).type,"VELO")==0) {
      strcpy((*axis).name,"velocity");
      strcpy((*axis).type,"velocity");
      strcpy((*axis).refer,"LSR");
      if(strcmp((*axis).unit,"m/s")==0 || strcmp((*axis).unit,"pixel")==0) {
	strcpy((*axis).unit,"km/s");
	(*axis).delta=(*axis).delta*1.0e-3;
	(*axis).origin=(*axis).origin*1.0e-3;
      }
      (*head).be.be_ch=(*axis).size;
      (*head).be.data_ch=(*axis).size;
      (*head).be.center_ch=((double)(*axis).size)/2.0;
      (*head).be.basis_ch=(short)(*head).be.center_ch;
      obs_freq=(*head).rx.obs_freq;
      v_range.min=(*axis).origin-(*axis).pixel_origin*(*axis).delta;
      v_range.max=(*axis).origin+((double)((*axis).size-1)-(*axis).pixel_origin)*(*axis).delta;
      freq_span=(v_range.min-v_range.max)/(LIGHT_SPEED)*obs_freq*1.0e3;
      if(freq_span<0.0) {
	freq_span= -freq_span;
	(*head).be.sign= (char)-1;
      } else {
	(*head).be.sign= (char)1;
      }
      if((*head).be.center_freq==0.0) {
	(*head).be.center_freq=freq_span; /* from 0.5*f_span to 1.5*f_span */
      }
      center_freq=(*head).be.center_freq;
      (*head).be.disp1=freq_span/(double)((*axis).size-1);
      (*head).be.disp0=(*head).be.center_freq;
    }
  }
  if((*head).be.disp0==0.0)
    (*head).be.disp0=(*head).be.center_freq;
}

void correct_header(argc,argv,cube_head)
int argc;
char *argv[];
CUBE_HEAD *cube_head;
{
  fprintf(stderr,"* WRN * valid only for OHCUBE data\n");

  strcpy((*cube_head).axis.y.name,"l");
  strcpy((*cube_head).axis.y.type,"long");
  strcpy((*cube_head).axis.y.unit,"degree");
  strcpy((*cube_head).axis.y.refer,"lb2");
  strcpy((*cube_head).axis.y.project,"sin");

  strcpy((*cube_head).axis.z.name,"b");
  strcpy((*cube_head).axis.z.type,"lat");
  strcpy((*cube_head).axis.z.unit,"degree");
  strcpy((*cube_head).axis.z.refer,"lb2");
  strcpy((*cube_head).axis.z.project,"sin");

  adjustBeStatus(cube_head);
}


int options(argc,argv)
/*
get parameters from options
*/
int argc;
char *argv[];
{
  int ii;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;
  int flag,all_flag;
  int opt_pos;
  char *got_value;

/* default values */

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"pipe");	key_type[n_key-1]=0;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* pipe */
  scan_flag[0]=0;

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

int main(argc,argv)
int argc;
char *argv[];
{
  CUBE cube;
  CUBE_HEAD list_flag;

  FILE *fp;
  int pipe_flag;
  int label_flag;
  int block_flag[N_BLOCK];
  int err_code;

  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];

  int ii,n_file;
  char id_str[MAX_FILE_NAME_LENGTH];

/* check option */
  err_code=options(argc,argv);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

/* pipe header */
  StarPipeHeadIn(id_str,&n_file);
  StarPipeHeadOut(n_file);

/* .cub data */
  for(ii=0;ii<n_file;ii++) {
    StarCubeIn(&cube,&cdb_flag,cdb_arg);

    correct_header(argc,argv,&(cube.head));
/*tmp_proc(&cube);*/
    StarCubeOut(&cube,cdb_flag,cdb_arg);

    StarFreeCubePixel(&cube);
  }

/* end tag of pipe */
  StarPipeEndIn(id_str);
  StarPipeEndOut();

  exit(0);
}
