/*
integrate pixel value over the whole area

2004/7/14 T.Handa : ver.3.00
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"cubinteg"
#define N_KEY 10

int options(argc,argv,pipe_flag,whole_flag,xx,yy,zz)
/*
get parameters from options
*/
int argc;
char *argv[];
int *pipe_flag;
int *whole_flag; /* <out> */
RANGE *xx,*yy,*zz; /* <out> integration range */
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  whole_flag[0]=FLAG_YES;
  whole_flag[1]=FLAG_YES;
  whole_flag[2]=FLAG_YES;
  *pipe_flag=FLAG_NO;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"x");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"pipe");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"output");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"-");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"nopipe");	key_type[n_key-1]=0; 

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* integration range */
  n_scan=1;
  scan_flag[0]=0;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strcmp(got_value,"whole")==0 || strcmp(got_value,"all")==0) {
      whole_flag[0]=FLAG_YES;
    } else {
      whole_flag[0]=FLAG_NO;
      StarOptionAngleRange(got_value,xx);
    }
  }

  n_scan=1;
  scan_flag[0]=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strcmp(got_value,"whole")==0 || strcmp(got_value,"all")==0) {
      whole_flag[1]=FLAG_YES;
    } else {
      whole_flag[1]=FLAG_NO;
      StarOptionAngleRange(got_value,yy);
    }
  }

  n_scan=1;
  scan_flag[0]=2;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strcmp(got_value,"whole")==0 || strcmp(got_value,"all")==0) {
      whole_flag[2]=FLAG_YES;
    } else {
      whole_flag[2]=FLAG_NO;
      StarOptionAngleRange(got_value,zz);
    }
  }

/* pipe */
  scan_flag[0]=3;
  scan_flag[1]=4;
  scan_flag[2]=5;
  scan_flag[3]= -6;
  *pipe_flag=StarOptionFlag(4,scan_flag,key_flag);

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

void convert_position(axis,xx,whole_flag,ll)
AXIS axis; /* <in> axis */
RANGE xx; /* <in> position in physical unit */
int whole_flag; /* <in> */
RANGE_long *ll; /* <out> */
{
  RANGE x_pix;

  if(whole_flag==FLAG_YES) {
    (*ll).min=0;
    (*ll).max=axis.size-1;
  } else {
    x_pix.min=StarPixelIndexPositionOnAxis(&axis,xx.min);
    x_pix.max=StarPixelIndexPositionOnAxis(&axis,xx.max);
    (*ll).min=(long)rint(x_pix.min);
    (*ll).max=(long)rint(x_pix.max);
    StarSortRangeLong(ll);
    if((*ll).min<0) (*ll).min=0;
    if((*ll).max<0) (*ll).max=0;
    if((*ll).min>=axis.size) (*ll).min=axis.size-1;
    if((*ll).max>=axis.size) (*ll).max=axis.size-1;
  }

  return;
}

int IntegOver(cube,ll,mm,nn,flux)
CUBE *cube; /* <in> */
RANGE_long ll,mm,nn; /* <in> range in pixel */
double *flux; /* <out> integrated value */
/* <return> effective pixels */
{
  int pixels;
  int ix,iy,iz;
  double ta,integ_ta;
  int valid_pixel_flag;

  pixels=0;
  integ_ta=0.0;

  for(iz=nn.min;iz<=nn.max;iz++) {
    for(iy=mm.min;iy<=mm.max;iy++) {
      for(ix=ll.min;ix<=ll.max;ix++) {
	ta=StarGetCubePixelData(cube,ix,iy,iz,&valid_pixel_flag);
	if(valid_pixel_flag==FLAG_YES) {
	  integ_ta=integ_ta+ta;
	  pixels++;
	}
      }
    }
  }

  *flux=integ_ta;

  return(pixels);
}

int main(argc,argv)
int	argc;
char	*argv[];
{
  CUBE cube;
  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];
  int err_code;
  int i_file,n_file;
  char id_str[BUFFER_SIZE];

  int pipe_flag;

  int whole_flag[3];
  RANGE xx,yy,zz;
  RANGE_long ll,mm,nn;
  double integ_flux;
  int pixels;

/* get optional parameters */
  err_code=options(argc,argv, &pipe_flag,whole_flag,&xx,&yy,&zz);

/* pipe header */
  StarPipeHeadIn(id_str,&n_file);

  if(pipe_flag==FLAG_YES) {
    StarPipeHeadOut(n_file);
  }

/* .cub data */
  for(i_file=0;i_file<n_file;i_file++) {

    StarCubeIn(&cube,&cdb_flag,cdb_arg);

    if(cdb_flag==GOOD_SCAN) {
      convert_position(cube.head.axis.x,xx,whole_flag[0],&ll);
      convert_position(cube.head.axis.y,yy,whole_flag[1],&mm);
      convert_position(cube.head.axis.z,zz,whole_flag[2],&nn);
      
      pixels=IntegOver(&cube,ll,mm,nn,&integ_flux);
      fprintf(stderr,"name=%s:",cube.head.file_name);
      fprintf(stderr,"integrated %s=%f %s",cube.head.data_value.type,integ_flux,cube.head.data_value.unit);
      fprintf(stderr,"; %f %s %s with z-depth ",integ_flux*cube.head.axis.z.delta,cube.head.data_value.unit,cube.head.axis.z.unit);
      fprintf(stderr," for %d pixles (%ld,%ld,%ld)-(%ld,%ld,%ld)\n",pixels,ll.min,mm.min,nn.min,ll.max,mm.max,nn.max);

      if(pipe_flag==FLAG_YES) {
	StarCubeOut(&cube,cdb_flag,cdb_arg);
      }

    } else if(cdb_flag==BAD_SCAN) {
      StarCubeOut(&cube,cdb_flag,cdb_arg);
    } else {
      StarCubeOut(&cube,cdb_flag,cdb_arg);
    }
    StarFreeCubePixel(&cube);
  }

/* end tag of pipe */
  if(pipe_flag==FLAG_YES) {
    StarPipeEndIn(id_str);
    StarPipeEndOut();
  }

  exit(0);
}
