/*
cub data loader into stdout

1998/11/28 T.Handa : ver.3.00
2002/6/23 T.Handa : ver.3.01 (modify) multi-file open
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define DEFAULT_PRMFILE "cubload.prm"
#define N_KEY 30

int options(argc,argv,input_filename)
/*
get parameters from options
1998/11/26 T.Handa
*/
int argc;
char *argv[];
char *input_filename;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  input_filename[0]=(char)NULL;

/* set keywords */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"cub"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"input_file"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"in_cub"); key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* name for input cube */
  scan_flag[0]=0;
  scan_flag[1]=1;
  scan_flag[2]=2;
  n_scan=3;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(input_filename,got_value);
  }

  got_value=StarKeylessOption(argc,argv,n_key,key_flag,key_type);
  if(got_value[0]!=(char)NULL) {
    strcpy(input_filename,got_value);
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

int cube_file2pipe(argc,argv, default_name,filename,cdb_flag,cdb_arg)
int argc;
char *argv[];
char *default_name;
char *filename;
int cdb_flag;
char *cdb_arg;
{
  FILE *in_fp;
  CUBE cube;
  int err_code;

  in_fp=fopen(filename,"r");
  if(in_fp==NULL) {
    fprintf(stderr,"%s : not open file %s\n",argv[0],filename);
    exit(ERR_UNEXIST_FILE);
  }
  err_code=StarCubeHeadLoad(in_fp,&(cube.head));
  cube.pixel=StarAllocCubePixel(&(cube.head));
  err_code=StarCubePixelLoad(in_fp,&cube);
  fclose(in_fp);

  strcpy(cube.head.file_id,STAR_CUB_IN);
  strcpy(cube.head.file_name,default_name);

/* pipeout cube data */
  err_code=StarCubeOut(&cube,cdb_flag,cdb_arg);

  StarFreeCubePixel(&cube);

  return(0);
}

int main(argc,argv)
int argc;
char *argv[];
{
  char **input_filename,**cdb_arg;
  int cdb_flag[MAX_FILE_NUMBER];
  char full_filename[MAX_FILE_NAME_LENGTH];

  int ii;
  int n_file;
  int err_code;


/* alloc work area */
  cdb_arg=AllocAllWords(MAX_FILE_NUMBER,MAX_IDX_RECORD_LENGTH);
  input_filename=AllocAllWords(MAX_FILE_NUMBER,MAX_FILE_NAME_LENGTH);

/* get option */
  err_code=options(argc,argv,input_filename[0]);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

  n_file=0;

/* command as cubload */
  for(ii=1;ii<argc;ii++) {
    strcpy(input_filename[n_file],argv[ii]);
    cdb_flag[n_file]=GOOD_SCAN;
    strcpy(cdb_arg[ii],"");
    n_file++;
    if(n_file>MAX_FILE_NUMBER) {
      fprintf(stderr,"%s:Too many files\n",argv[0]);
    }
  }

/* pipeout header */
  StarPipeHeadOut(n_file);

  for(ii=0;ii<n_file;ii++) {
/* add STD extension for cube */
    err_code=StarStdFileExtName("STAR_DATA",input_filename[ii],full_filename,".cub");
    if(err_code!=0) {
      fprintf(stderr,"Too long file name\n");
      exit(1);
    }

/* load cube data and flow out into the pipline */
  err_code=cube_file2pipe(argc,argv, input_filename[ii],full_filename,cdb_flag[ii],cdb_arg[ii]);
    if(err_code!=0) {
      switch(err_code) {
      case ERR_UNEXIST_FILE:
	fprintf(stderr,"%s: unexist file=%s\n",
		argv[0],full_filename);
	break;
      }
    }

  }
  StarPipeEndOut();
  FreeAllWords(input_filename,MAX_FILE_NUMBER);
  FreeAllWords(cdb_arg,MAX_FILE_NUMBER);
  exit(0);
}
