/*
edit a pixel value in a cube data by hand

2003/12/5 T.Handa : ver.3.00
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStdio.h>
#include <StStdlib.h>
#include <StPipe.h>
#include <StCubeIO.h>
#include <StCubeProc.h>

#define	DEFAULT_PRMFILE	"cubpixeledit.prm"
#define N_KEY 10

static void put_pixel_value_in_box(CUBE *,RANGE *,int *,double,int,RANGE_long *);
static void adjust_put_box(AXES *,RANGE *,int *,RANGE_long *);
static void AddCubePixelEditHistory(CUBE_HEAD *,RANGE *,int *,double,int,RANGE_long *);

int SeparateByColons(word,top_pos,n_pos)
char *word; /* <in> */
int *top_pos; /* <out> */
int *n_pos; /* <out> */
{
  char *token;
  int ii,jj;
  int i_pos;
  int l_str;

  ii=0;
  i_pos=0;
  l_str=strlen(word);
  for(jj=0;jj<=l_str;jj++) {
    switch(word[jj]) {
    case ':': case ';':
      word[jj]=(char)NULL;
    case (char)NULL:
      top_pos[ii]=i_pos;
      i_pos=jj+1;
      ii++;
      if(ii>MAX_RANGE_NUMBER) {
	*n_pos=ii;
	return(ERR_TOO_MANY_PRMS);
      }
    }
  }
  *n_pos=ii;
  return(0);
}

int options(argc,argv,put_box,whole_range,put_value,put_dummy)
/*
get parameters from options
*/
int argc;
char *argv[];
RANGE *put_box; /* <out> [0]:x,[1]:y,[2]:z range to edit */
int *whole_range; /* <out> [0]:x,[1]:y,[2]:z  =FLAG_YES if whole range in the cube */
double *put_value; /* <out> pixel value in physics unit */
int *put_dummy; /* <out> =FLAG_YES if put value is dummy pixel */
{
  int ii,jj;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

  int l_str;
  char *buffer;
  int top_pos[MAX_RANGE_NUMBER];
  int n_pos;

/* default values */
  *put_value=0.0;
  *put_dummy=FLAG_YES;
  whole_range[0]=FLAG_NO;
  whole_range[1]=FLAG_NO;
  whole_range[2]=FLAG_NO;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"box");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"value");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* box */
  n_scan=1;
  scan_flag[0]=0;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    l_str=strlen(got_value);
    buffer=(char *)malloc(sizeof(char)*l_str);
    strcpy(buffer,got_value);
    SeparateByColons(buffer,top_pos,&n_pos);
    for(ii=0;ii<3;ii++) {
      if(ii>=n_pos) {
	whole_range[ii]=FLAG_YES;
      } else if(strncmp(&(buffer[top_pos[ii]]),"whole",5)==0 || strncmp(&(buffer[top_pos[ii]]),"all",3)==0 || strcmp(&(buffer[top_pos[ii]]),"")==0) {
	whole_range[ii]=FLAG_YES;
      } else {
	whole_range[ii]=FLAG_NO;
	StarOptionAngleRange(&(buffer[top_pos[ii]]),&(put_box[ii]));
      }
    }
    free(buffer);
  }

/* value */
  n_scan=1;
  scan_flag[0]=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strcmp(got_value,"dummy")==0) {
      *put_dummy=FLAG_YES;
    } else {
      *put_dummy=FLAG_NO;
      *put_value=atof(got_value);
    }
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return 0;
}


void adjust_put_box(cube_axis,put_box,whole_range,actual_box)
AXES *cube_axis; /* <in> */
RANGE *put_box; /* <in> */
int *whole_range; /* <in> */
RANGE_long *actual_box; /* <out> adjusted box */
{
  double xx;
  int ii;

/* x-axis */
  ii=0;
  if(whole_range[ii]==FLAG_YES) {
    actual_box[ii].min=0;
    actual_box[ii].max=(*cube_axis).x.size-1;
  } else {
    xx=StarPixelIndexPositionOnAxis(&((*cube_axis).x),put_box[ii].min);
    actual_box[ii].min=(int)(rint(xx));
    xx=StarPixelIndexPositionOnAxis(&((*cube_axis).x),put_box[ii].max);
    actual_box[ii].max=(int)(rint(xx));
    StarSortRangeLong(&(actual_box[ii]));
    if(actual_box[ii].min<0) 
      actual_box[ii].min=0;
    if(actual_box[ii].max>=(*cube_axis).x.size) 
      actual_box[ii].max=(*cube_axis).x.size-1;
  }

/* y-axis */
  ii=1;
  if(whole_range[ii]==FLAG_YES) {
    actual_box[ii].min=0;
    actual_box[ii].max=(*cube_axis).y.size-1;
  } else {
    xx=StarPixelIndexPositionOnAxis(&((*cube_axis).y),put_box[ii].min);
    actual_box[ii].min=(int)(rint(xx));
    xx=StarPixelIndexPositionOnAxis(&((*cube_axis).y),put_box[ii].max);
    actual_box[ii].max=(int)(rint(xx));
    StarSortRangeLong(&(actual_box[ii]));
    if(actual_box[ii].min<0) 
      actual_box[ii].min=0;
    if(actual_box[ii].max>=(*cube_axis).x.size) 
      actual_box[ii].max=(*cube_axis).x.size-1;
  }

/* z-axis */
  ii=2;
  if(whole_range[ii]==FLAG_YES) {
    actual_box[ii].min=0;
    actual_box[ii].max=(*cube_axis).z.size-1;
  } else {
    xx=StarPixelIndexPositionOnAxis(&((*cube_axis).z),put_box[ii].min);
    actual_box[ii].min=(int)(rint(xx));
    xx=StarPixelIndexPositionOnAxis(&((*cube_axis).z),put_box[ii].max);
    actual_box[ii].max=(int)(rint(xx));
    StarSortRangeLong(&(actual_box[ii]));
    if(actual_box[ii].min<0) 
      actual_box[ii].min=0;
    if(actual_box[ii].max>=(*cube_axis).x.size) 
      actual_box[ii].max=(*cube_axis).x.size-1;
  }

  return;
}



void put_pixel_value_in_box(cube,put_box,whole_range,put_value,put_dummy,actual_box)
CUBE *cube; /* <i/o> */
RANGE *put_box; /* <in> */
int *whole_range; /* <in> */
double put_value; /* <in> */
int put_dummy; /* <in> */
RANGE_long *actual_box; /* <out> */
{
  int ix,iy,iz;
  float dummy_pix;

  adjust_put_box(&((*cube).head.axis),put_box,whole_range,actual_box);

  if(put_dummy==FLAG_YES) { /* set to dummy pixels */
    dummy_pix=(*cube).head.data_value.dummy;
    for(iz=actual_box[2].min;iz<=actual_box[2].max;iz++) {
      for(iy=actual_box[1].min;iy<=actual_box[1].max;iy++) {
	for(ix=actual_box[0].min;ix<=actual_box[0].max;ix++) {
	  StarPutCubePixel(cube,ix,iy,iz,dummy_pix);
	}
      }
    }
 

  } else { /* set to a given value */
    for(iz=actual_box[2].min;iz<=actual_box[2].max;iz++) {
      for(iy=actual_box[1].min;iy<=actual_box[1].max;iy++) {
	for(ix=actual_box[0].min;ix<=actual_box[0].max;ix++) {
	  StarPutCubePixelData(cube,ix,iy,iz,put_value);
	}
      }
    }
  }

  return;
}


void AddCubePixelEditHistory(cube_head,put_box,whole_range,put_value,put_dummy,actual_box)
CUBE_HEAD *cube_head; /* <i/o> */
RANGE *put_box; /* <in> */
int *whole_range; /* <in> */
double put_value; /* <in> */
int put_dummy; /* <in> */
RANGE_long *actual_box; /* <in> */
{
  HISTORY	history;
  char buffer[80],buff_head[3];
  int ii,jj;
  int i_prm;

  i_prm=0;
  strcpy(history.step,"EdPix");

  if(put_dummy==FLAG_YES) {
    strcpy(buffer,"put dummy");
  } else {
    sprintf(buffer,"set %f",put_value);
  }
  buffer[16]=(char)NULL;
  strcpy(history.prm[i_prm],buffer);
  i_prm++;

  for(jj=0;jj<3;jj++) {
    switch(jj) {
    case 0:
      strcpy(buff_head,"x");
      break;
    case 1:
      strcpy(buff_head,"y");
      break;
    case 2: default:
      strcpy(buff_head,"z");
      break;
    }
    if(whole_range[jj]==FLAG_YES) {
      sprintf(buffer,"%s=whole",buff_head);
      buffer[16]=(char)NULL;
      strcpy(history.prm[i_prm],buffer);
      i_prm++;

    } else {
      sprintf(buffer,"fm:%s=%f",buff_head,put_box[jj].min);
      buffer[16]=(char)NULL;
      strcpy(history.prm[i_prm],buffer);
      i_prm++;

      sprintf(buffer,"to:%s=%f",buff_head,put_box[jj].max);
      buffer[16]=(char)NULL;
      strcpy(history.prm[i_prm],buffer);
      i_prm++;
    }

    sprintf(buffer,"fm:i%s=%d",buff_head,actual_box[jj].min);
    buffer[16]=(char)NULL;
    strcpy(history.prm[i_prm],buffer);
    i_prm++;

    sprintf(buffer,"to:i%s=%d",buff_head,actual_box[jj].max);
    buffer[16]=(char)NULL;
    strcpy(history.prm[i_prm],buffer);
    i_prm++;

  }

  history.size=i_prm;

  StarAddHistory4Cube(cube_head,&history);
}

int main(argc,argv)
int	argc;
char	*argv[];
{
  CUBE cube;
  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];
  int err_code;
  int i_file,n_file;
  char id_str[BUFFER_SIZE];

  float **pixel_data;

  int ii,jj;

  RANGE put_box[3];
  int whole_range[3];
  double put_value;
  int put_dummy;

  int actual_max[3],actual_min[3];
  RANGE_long actual_box[3];

/* get optional parameters */
  err_code=options(argc,argv,put_box,whole_range,&put_value,&put_dummy);

/* pipe header */
  StarPipeHeadIn(id_str,&n_file);

  StarPipeHeadOut(n_file);

/* .cub data */
  for(ii=0;ii<n_file;ii++) {

    StarCubeIn(&cube,&cdb_flag,cdb_arg);

    if(cdb_flag==GOOD_SCAN) {
      put_pixel_value_in_box(&cube,put_box,whole_range,put_value,put_dummy,actual_box);
      AddCubePixelEditHistory(&(cube.head),put_box,whole_range,put_value,put_dummy,actual_box);
      StarCubeOut(&cube,cdb_flag,cdb_arg);
    } else if(cdb_flag==BAD_SCAN) {
      StarCubeOut(&cube,cdb_flag,cdb_arg);
    } else {
      StarCubeOut(&cube,cdb_flag,cdb_arg);
    }

    StarFreeCubePixel(&cube);
  }

/* end tag of pipe */
  StarPipeEndIn(id_str);
  StarPipeEndOut();

  exit(0);
}
