/*
cube data: smooth with considering the current beamsize

1998/ 9/29 T.Handa : ver.3.00
1999/10/28 T.Handa : ver.3.01 kernel for HUNNING,GAUSS,BOXCAR,BINNING in StarCubeSmoothedValue();
1999/12/27 T.Handa : ver.3.02 (debug) keyword mistype
2002/5/11 T.Handa : ver.3.03 (debug) scan data for datavalue header
2002/ 6/30 T.Handa : ver.3.03 (debug) logitude (360.0deg mod) not finished
2002/ 6/30 T.Handa : ver.3.04 (debug) input width direct for smoothing width
2003/ 3/ 3 T.Handa : ver.3.05 (debug/revise) default step is the same as original, default kernel PINPOINT
2003/ 8/28 T.Handa : ver.3.06 (revise) explicit assign for axes parameters in StarCubeSmoothAndConvert();
2003/ 8/30 T.Handa : ver.3.07 (debug) auto HPBW revision in MakeNewAxes();
2003/ 8/30 T.Handa : ver.3.07 (modify) 2D smoothing options (-xy_step, -xy_width, -xy_kernel, etc.)
2006/12/13 T.Handa : ver.3.08 (debug) avoid core dump even if the velocity axis is not found
*/
#define DEBUG
#undef DEBUG

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"smooth.prm"
#define N_KEY 70

#define HUNNING	1
#define GAUSS	2
#define BOXCAR	3
#define BINNING	4
#define PINPOINT	5

#ifdef _NO_PROTO
extern float StarCubeSmoothedValue();
extern double KernelWeight();
extern int IsInRange();
extern void StarCubeSmoothAndConvert();
extern void MakeNewAxes();
extern void AddCubeSmoothHistory();

#else
extern float StarCubeSmoothedValue(CUBE *,double *,int *,double *,double *);
extern double KernelWeight(double,double,int);
extern int IsInRange(RANGE *,double *,int);
extern void StarCubeSmoothAndConvert(CUBE *,CUBE *,double *,int *);
extern void MakeNewAxes(CUBE_HEAD *,RANGE *,double *,double *,int *);
extern void AddCubeSmoothHistory(CUBE_HEAD *,RANGE *,double *,double *,int *);

#endif

int options(argc,argv,xyz_range,xyz_step,xyz_width,kernel_type)
/*
get parameters from options
*/
int argc;
char *argv[];
RANGE *xyz_range; /* <out> new range along xyz [0],[1],[2] */
double *xyz_step,*xyz_width;
int *kernel_type;
{
  int ii;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  for(ii=0;ii<3;ii++) {
    xyz_range[ii].min=0.0;
    xyz_range[ii].max=0.0;
    xyz_step[ii]=0.0;
    xyz_width[ii]=0.0;
    kernel_type[ii]=PINPOINT;
  }

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"x");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"x_range");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y_range");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z_range");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"x_step");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"x_sampling");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y_step");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y_sampling");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z_step");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z_sampling");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"xy_step"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"xy_sampling"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"yz_step"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"yz_sampling"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"zx_step"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"zx_sampling"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"yx_step"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"yx_sampling"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"zy_step"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"zy_sampling"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"xz_step"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"xz_sampling"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"x_width");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y_width");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z_width");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"xy_width"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"yz_width"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"zx_width"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"yx_width"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"zy_width"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"xz_width"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"x_smooth_kernel");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"x_smooth_method");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"x_kernel");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y_smooth_kernel");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y_smooth_method");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y_kernel");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z_smooth_kernel");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z_smooth_method");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z_kernel");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"xy_smooth_kernel"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"xy_smooth_method"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"xy_kernel"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"yz_smooth_kernel"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"yz_smooth_method"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"yz_kernel"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"zx_smooth_kernel"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"zx_smooth_method"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"zx_kernel"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"yx_smooth_kernel"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"yx_smooth_method"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"yx_kernel"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"zy_smooth_kernel"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"zy_smooth_method"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"zy_kernel"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"xz_smooth_kernel"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"xz_smooth_method"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"xz_kernel"); key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* full span */
  n_scan=2;
  for(ii=0;ii<3;ii++) {
    scan_flag[0]=0+ii*n_scan;
    scan_flag[1]=1+ii*n_scan;
    got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
    if(got_value[0]!=(char)NULL) {
      StarOptionAngleRange(got_value,&(xyz_range[ii]));
    }
  }

/* resampling step */
  n_scan=10;
  for(ii=0;ii<3;ii++) {
    scan_flag[0]=6+ii*2;
    scan_flag[1]=7+ii*2;
    switch(ii) {
    case 0:
      scan_flag[2]=12;
      scan_flag[3]=13;
      scan_flag[4]=16;
      scan_flag[5]=17;
      scan_flag[6]=18;
      scan_flag[7]=19;
      scan_flag[8]=22;
      scan_flag[9]=23;
      break;
    case 1:
      scan_flag[2]=12;
      scan_flag[3]=13;
      scan_flag[4]=16;
      scan_flag[5]=17;
      scan_flag[6]=18;
      scan_flag[7]=19;
      scan_flag[8]=20;
      scan_flag[9]=21;
      break;
    case 2:
      scan_flag[2]=14;
      scan_flag[3]=15;
      scan_flag[4]=16;
      scan_flag[5]=17;
      scan_flag[6]=20;
      scan_flag[7]=21;
      scan_flag[8]=22;
      scan_flag[9]=23;
      break;
    }
    got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
    if(got_value[0]!=(char)NULL) {
      xyz_step[ii]=StarStdAngle2deg(got_value);
    }
  }

/* smoothing width */
  n_scan=5;
  for(ii=0;ii<3;ii++) {
    scan_flag[0]=24+ii*1;
    switch(ii) {
    case 0:
      scan_flag[1]=27;
      scan_flag[2]=29;
      scan_flag[3]=30;
      scan_flag[4]=32;
      break;
    case 1:
      scan_flag[1]=27;
      scan_flag[2]=28;
      scan_flag[3]=30;
      scan_flag[4]=31;
      break;
    case 2:
      scan_flag[1]=28;
      scan_flag[2]=29;
      scan_flag[3]=31;
      scan_flag[4]=32;
      break;
    }      
    got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
    if(got_value[0]!=(char)NULL) {
      xyz_width[ii]=StarStdAngle2deg(got_value);
    }
  }

/* smoothing kernel type */
  n_scan=15;
  for(ii=0;ii<3;ii++) {
    scan_flag[0]=33+ii*3;
    scan_flag[1]=34+ii*3;
    scan_flag[2]=35+ii*3;
    switch(ii) {
    case 0:
      scan_flag[3]=42;
      scan_flag[4]=43;
      scan_flag[5]=44;
      scan_flag[6]=48;
      scan_flag[7]=49;
      scan_flag[8]=50;
      scan_flag[9]=51;
      scan_flag[10]=52;
      scan_flag[11]=53;
      scan_flag[12]=57;
      scan_flag[13]=58;
      scan_flag[14]=59;
      break;
    case 1:
      scan_flag[3]=42;
      scan_flag[4]=43;
      scan_flag[5]=44;
      scan_flag[6]=45;
      scan_flag[7]=46;
      scan_flag[8]=47;
      scan_flag[9]=51;
      scan_flag[10]=52;
      scan_flag[11]=53;
      scan_flag[12]=54;
      scan_flag[13]=55;
      scan_flag[14]=56;
      break;
    case 2:
      scan_flag[3]=45;
      scan_flag[4]=46;
      scan_flag[5]=47;
      scan_flag[6]=48;
      scan_flag[7]=49;
      scan_flag[8]=50;
      scan_flag[9]=54;
      scan_flag[10]=55;
      scan_flag[11]=56;
      scan_flag[12]=57;
      scan_flag[13]=58;
      scan_flag[14]=59;
      break;
    }      
    got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
    if(got_value[0]!=(char)NULL) {
      if(strcmp(got_value,"hunning")==0)
	kernel_type[ii]=HUNNING;
      else if(strcmp(got_value,"gauss")==0)
	kernel_type[ii]=GAUSS;
      else if(strcmp(got_value,"box")==0)
	kernel_type[ii]=BOXCAR;
      else if(strcmp(got_value,"boxcar")==0)
	kernel_type[ii]=BOXCAR;
      else if(strcmp(got_value,"bin")==0)
	kernel_type[ii]=BINNING;
      else if(strcmp(got_value,"pin")==0)
	kernel_type[ii]=PINPOINT;
      else if(strcmp(got_value,"pinpoint")==0)
	kernel_type[ii]=PINPOINT;
    }
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return 0;
}

int IsInRange(fullspan,target,long_flag)
/*
check whether target is in range or not. 

1999/9/28 T.Handa
*/
RANGE *fullspan; /* <in> range to use the check */
double *target; /* <i/o> checked value, <out> 360deg moded value in range */ 
int long_flag; /* <in> =FLAG_YES apply 360deg ambiguity */
{
  double range_base,target_main;

  if(long_flag==FLAG_YES) {
    if(StarIsInLongitudeRange(fullspan,*target)==1) {
      range_base=floor((*fullspan).min/360.0)*360.0;
      target_main=fmod(*target,360.0);
      *target=range_base+target_main;
      return FLAG_YES;
    } else {
      return FLAG_NO;
    }
  } else {
    if((*fullspan).min<= *target && *target<=(*fullspan).max)
      return FLAG_YES;
    else
      return FLAG_NO;
  }
}

double KernelWeight(xx,width,kernel_type)
/*
calculate weight using verious kernels

1999/9/28 T.Handa
*/
double xx,width; /* <in> offset, width */
int kernel_type; /* <in> */
{
  double ff,yy;

  yy=xx/width;
  ff=0.0;

  switch(kernel_type) {
  case BOXCAR:
    if(fabs(yy)<0.5) ff=1.0;
    else ff=0.0;
    break;
  case HUNNING:
    if(fabs(yy)<0.5) ff=1.0;
    else if(ff<1.5) ff=0.5;
    else ff=0.0;
    break;
  case PINPOINT:
    if(fabs(yy)<=0.5) ff=1.0;
    else ff=0.0;
    break;
  case BINNING:
    if(fabs(yy)<=0.5) ff=1.0;
    else ff=0.0;
    break;
  case GAUSS: default:
    ff= -yy*yy;
    ff=exp(ff);
  }

  return(ff);
}

float StarCubeSmoothedValue(cube,pix_pos,kernel_type,pix_width,pix_window_width)
/*
1999/ 9/29 T.Handa
1999/10/27 T.Handa : ver.3.01 kernel for HUNNING,GAUSS,BOXCAR,BINNING
*/
CUBE *cube; /* <in> */
double *pix_pos; /* <in> [0][1][2] integration center in pixel index */
int *kernel_type; /* <in> [0][1][2] */
double *pix_width; /* <in> [0][1][2] integration width in pixel index */
double *pix_window_width; /* <in> [0][1][2] */
{
  float ta;
  double integ_ta,integ_weight;
  int ix,iy,iz;
  double factor[3],fxy,ff;
  RANGE pix_window[3];
  RANGE_long window[3];
  int ii;
  AXIS* axis[3];

  axis[0]= &((*cube).head.axis.x);
  axis[1]= &((*cube).head.axis.y);
  axis[2]= &((*cube).head.axis.z);

  for(ii=0;ii<3;ii++) {
    pix_window[ii].min=pix_pos[ii]-pix_window_width[ii];
    pix_window[ii].max=pix_pos[ii]+pix_window_width[ii];
    StarSortRange(&(pix_window[ii]));
  }
  for(ii=0;ii<3;ii++) {
    window[ii].min=floor((double)pix_window[ii].min);
    window[ii].max=ceil((double)pix_window[ii].max);
  }
  for(ii=0;ii<3;ii++) {
    if(window[ii].min<0)
      window[ii].min=0;
    if(window[ii].max>=(long)(*axis[ii]).size)
      window[ii].max=(long)(*axis[ii]).size-1;
  }

  integ_ta=0.0;
  integ_weight=0.0;

  for(ix=window[0].min;ix<=window[0].max;ix++) {
    factor[0]=KernelWeight((double)ix-pix_pos[0],pix_width[0],kernel_type[0]);
    for(iy=window[1].min;iy<=window[1].max;iy++) {
      factor[1]=KernelWeight((double)iy-pix_pos[1],pix_width[1],kernel_type[1]);
      fxy=factor[0]*factor[1];
      for(iz=window[2].min;iz<=window[2].max;iz++) {
	factor[2]=KernelWeight((double)iz-pix_pos[2],pix_width[2],kernel_type[2]);
	ff=fxy*factor[2];
	ta=StarGetCubePixel(cube,ix,iy,iz);
	if(ta!=(*cube).head.data_value.dummy) {
	  integ_ta=integ_ta+(double)ta*ff;
	  integ_weight=integ_weight+ff;
	}
      }
    }
  }

  if(integ_weight==0.0) {
    ta=(*cube).head.data_value.dummy;
  } else {
    ta=(float)(integ_ta/integ_weight);
  }

  return ta;
}

void StarCubeSmoothAndConvert(out_cube,in_cube,xyz_width,kernel_type)
/*
Smooth and resample

1999/9/29 T.Handa
2003/ 8/22 T.Handa : ver.3.06 (revise) explicit assign for axes parameters
*/
CUBE *out_cube,*in_cube; /* <i/o>,<in> */
double *xyz_width; /* <in> [0][1][2] smoothed widths along axes */
int *kernel_type; /* <in> [0][1][2] along axes */
{
  int jx,jy,jz;
  double xx,yy,zz;
  double in_pos[3];
  double pix_width[3],pix_window_width[3];
  float ta,dummy;
  RANGE in_fullspan[3];
  int is_longitude[3];

  int ii;

  double xyz_delta[2][3];

  dummy=(*out_cube).head.data_value.dummy;

  pix_width[0]=fabs(xyz_width[0]/(*in_cube).head.axis.x.delta);
  pix_width[1]=fabs(xyz_width[1]/(*in_cube).head.axis.y.delta);
  pix_width[2]=fabs(xyz_width[2]/(*in_cube).head.axis.z.delta);

  in_fullspan[0].min=StarGetCubePixPosOnAxis(&((*in_cube).head.axis.x),0);
  in_fullspan[0].max=StarGetCubePixPosOnAxis(&((*in_cube).head.axis.x),(int)((*in_cube).head.axis.x.size-1));
  in_fullspan[1].min=StarGetCubePixPosOnAxis(&((*in_cube).head.axis.y),0);
  in_fullspan[1].max=StarGetCubePixPosOnAxis(&((*in_cube).head.axis.y),(int)((*in_cube).head.axis.y.size-1));
  in_fullspan[2].min=StarGetCubePixPosOnAxis(&((*in_cube).head.axis.z),0);
  in_fullspan[2].max=StarGetCubePixPosOnAxis(&((*in_cube).head.axis.z),(int)((*in_cube).head.axis.z.size-1));

/* for x-axis */
  if(strcmp((*in_cube).head.axis.x.type,"long")==0) {
    is_longitude[0]=FLAG_YES;
    if((*in_cube).head.axis.x.delta<0.0) {
      xx=in_fullspan[0].min;
      in_fullspan[0].min=in_fullspan[0].max;
      in_fullspan[0].max=xx;
    } else {
      StarSortRange(&(in_fullspan[0]));
      is_longitude[0]=FLAG_NO;
    }
  }
/* for y-axis */
  if(strcmp((*in_cube).head.axis.y.type,"long")==0) {
    is_longitude[1]=FLAG_YES;
    if((*in_cube).head.axis.y.delta<0.0) {
      xx=in_fullspan[1].min;
      in_fullspan[1].min=in_fullspan[1].max;
      in_fullspan[1].max=xx;
    } else {
      StarSortRange(&(in_fullspan[1]));
      is_longitude[1]=FLAG_NO;
    }
  }
/* for z-axis */
  if(strcmp((*in_cube).head.axis.z.type,"long")==0) {
    is_longitude[2]=FLAG_YES;
    if((*in_cube).head.axis.z.delta<0.0) {
      xx=in_fullspan[2].min;
      in_fullspan[2].min=in_fullspan[2].max;
      in_fullspan[2].max=xx;
    } else {
      StarSortRange(&(in_fullspan[2]));
      is_longitude[2]=FLAG_NO;
    }
  }

  xyz_delta[0][0]=(*in_cube).head.axis.x.delta;
  xyz_delta[0][1]=(*in_cube).head.axis.y.delta;
  xyz_delta[0][2]=(*in_cube).head.axis.z.delta;

  xyz_delta[1][0]=(*out_cube).head.axis.x.delta;
  xyz_delta[1][1]=(*out_cube).head.axis.y.delta;
  xyz_delta[1][2]=(*out_cube).head.axis.z.delta;

/* integration window in channel */
  for(ii=0;ii<3;ii++) {
    switch(kernel_type[ii]) {
    case BOXCAR:
      pix_window_width[ii]=pix_width[ii];
      pix_width[ii]=pix_width[ii];
      break;
    case HUNNING:
      pix_window_width[ii]=pix_width[ii]*0.75;
      pix_width[ii]=pix_width[ii]/2.0;
      break;
    case PINPOINT:
      pix_window_width[ii]=0.0;
      pix_width[ii]=1.0;
      break;
    case BINNING:
      pix_window_width[ii]=fabs(xyz_delta[1][ii]/xyz_delta[0][ii]);
      pix_width[ii]=1.0;
      break;
    case GAUSS: default:
      pix_window_width[ii]=pix_width[ii]*5.0;
      pix_width[ii]=pix_width[ii]/1.665109222; /* 2*sqrt(log(2.0)) */
      break;
    }
  }

#ifdef DEBUG
  fprintf(stderr,"ori_width--> pix_window,width=");
  for(ii=0;ii<3;ii++)
    fprintf(stderr,"[%d]%f--> %f %f; ",ii,xyz_width[ii],pix_window_width[ii],pix_width[ii]);
  fprintf(stderr,"\n");
 
  for(ii=0;ii<3;ii++)
    fprintf(stderr,"xyz_delta[%d]=%f,",ii,xyz_delta[1][ii]);
  fprintf(stderr,"\n");
#endif

  jy=0;
  jz=0;

/* loop for every pixel in out_cube */
  for(jx=0;jx<(*out_cube).head.axis.x.size;jx++) {
    xx=StarGetCubePixPosOnAxis(&((*out_cube).head.axis.x),jx);
    if(IsInRange(&(in_fullspan[0]),&xx,is_longitude[0])==FLAG_YES) {
      in_pos[0]=StarPixelIndexPositionOnAxis(&((*in_cube).head.axis.x),xx);
      if(strncmp((*in_cube).head.axis.x.type,"long",4)==0) {
	while(in_pos[0]<0.0) in_pos[0]=in_pos[0]+360.0; 
	while(in_pos[0]>=360.0) in_pos[0]=in_pos[0]-360.0;
      } 

      for(jy=0;jy<(*out_cube).head.axis.y.size;jy++) {
	yy=StarGetCubePixPosOnAxis(&((*out_cube).head.axis.y),jy);
	if(IsInRange(&(in_fullspan[1]),&yy,is_longitude[1])==FLAG_YES) {
	  in_pos[1]=StarPixelIndexPositionOnAxis(&((*in_cube).head.axis.y),yy);
	  if(strncmp((*in_cube).head.axis.y.type,"long",4)==0) {
	    while(in_pos[1]<0.0) in_pos[1]=in_pos[1]+360.0; 
	    while(in_pos[1]>=360.0) in_pos[1]=in_pos[1]-360.0;
	  } 

	  for(jz=0;jz<(*out_cube).head.axis.z.size;jz++) {
	    zz=StarGetCubePixPosOnAxis(&((*out_cube).head.axis.z),jz);
	    if(IsInRange(&(in_fullspan[2]),&zz,is_longitude[2])==FLAG_YES) {
	      in_pos[2]=StarPixelIndexPositionOnAxis(&((*in_cube).head.axis.z),zz);
	      if(strncmp((*in_cube).head.axis.z.type,"long",4)==0) {
		while(in_pos[2]<0.0) in_pos[2]=in_pos[2]+360.0; 
		while(in_pos[2]>=360.0) in_pos[2]=in_pos[2]-360.0;
	      } 

	      ta=StarCubeSmoothedValue(in_cube,in_pos,kernel_type,pix_width,pix_window_width);

	      StarPutCubePixel(out_cube,jx,jy,jz,ta);

	    } else {
	      StarPutCubePixel(out_cube,jx,jy,jz,dummy);
	    }
	  } /* loop end along z-axis */
	} else {
	  StarPutCubePixel(out_cube,jx,jy,jz,dummy);
	}
      } /* loop end along y-axis */
    } else {
      StarPutCubePixel(out_cube,jx,jy,jz,dummy);
    }
  } /* loop end along x-axis */

  return;
}

void MakeNewAxes(cube_head,xyz_range,xyz_step,xyz_width,kernel_type)
/*
Make new AXES and RX_STATUS for re-sampling
convert xyz_range,xyz_step,xyz_width in axis physics unit
convert xyz_width[] under considering original resolution

1999/ 9/25 T.Handa : ver.3.00 (from MakeNewBEDispersion() ver3.0 in smooth.c)
2003/ 8/30 T.Handa : ver.3.07 (debug) auto HPBW revision
*/
CUBE_HEAD *cube_head; /* <i/o> in for rx_status.obs_freq */
RANGE *xyz_range; /* <i/o> in=deg,km/s; out=axis unit */
double *xyz_step; /* <i/o> in=deg,km/s; out=axis unit */
double *xyz_width; /* <i/o> in=deg,km/s resultant; out=axis unit kernel width */
int *kernel_type; /* <in> */
{
  int ii;
  AXIS *axis[3];
  static AXIS v_axis;
  double unit_factor[3];
  double fwhm;
  double hpbw_long,hpbw_lat;
  int axis_long_id,axis_lat_id;

  axis[0]= &((*cube_head).axis.x);
  axis[1]= &((*cube_head).axis.y);
  axis[2]= &((*cube_head).axis.z);

  axis_long_id= -1;
  axis_lat_id= -1;
  for(ii=0;ii<3;ii++)
    unit_factor[ii]=1.0;

  for(ii=0;ii<3;ii++) {
    if(strcmp((*axis[ii]).type,"velocity")==0 || strncmp((*axis[ii]).type,"freq",4)==0) {
      memcpy(&v_axis,&axis[ii],sizeof(AXIS));
      break;
    } else if(strcmp((*axis[ii]).type,"km/s")==0) {
      memcpy(&v_axis,axis[ii],sizeof(AXIS));
      strcpy(v_axis.type,"velocity");
      strcpy((*axis[ii]).type,"velocity");
      break;
    }
  }

#ifdef DEBUG
  fprintf(stderr,"unitconv.");
#endif

/* unit conversion */
  for(ii=0;ii<3;ii++) {
    if(xyz_range[ii].min==0.0 && xyz_range[ii].max==0.0 && xyz_step[ii]==0.0) {
      /* new axes are the same as the original */
      xyz_range[ii].min=(*axis[ii]).origin-(*axis[ii]).delta*(*axis[ii]).pixel_origin;
      xyz_range[ii].max=xyz_range[ii].min+(*axis[ii]).delta*((double)((*axis[ii]).size-1));
      xyz_step[ii]=(*axis[ii]).delta;

    } else if(xyz_step[ii]==0.0) {
      /* new axes are the same as the original */
      xyz_step[ii]=(*axis[ii]).delta;

    } else {
      if(strcmp((*axis[ii]).type,"long")==0 || strcmp((*axis[ii]).type,"lat")==0) {

	if(strcmp((*axis[ii]).type,"long")==0) {
	  axis_long_id=ii;
	} else if(strcmp((*axis[ii]).type,"lat")==0) {
	  axis_lat_id=ii;
	}

	if(strncmp((*axis[ii]).unit,"deg",3)==0) { /* default is in degree */
	  unit_factor[ii]=1.0;
	} else if(strncmp((*axis[ii]).unit,"arcmin",6)==0) {
	  unit_factor[ii]=60.0;
	} else if(strncmp((*axis[ii]).unit,"arcsec",6)==0) {
	  unit_factor[ii]=3600.0;
	} else {
	  unit_factor[ii]=1.0;
	}
      } else if(strcmp((*axis[ii]).type,"velocity")==0) {
	if(strcmp((*axis[ii]).unit,"km/s")==0) { /* default is in km/s */
	  unit_factor[ii]=1.0;
	} else if(strcmp((*axis[ii]).unit,"m/s")==0) {
	  unit_factor[ii]=1.0e3;
	} else if(strcmp((*axis[ii]).unit,"cm/s")==0) {
	  unit_factor[ii]=1.0e5;
	} else {
	  unit_factor[ii]=1.0;
	}
      } else if(strncmp((*axis[ii]).type,"freq",4)==0) {
	if(strcmp((*axis[ii]).unit,"GHz")==0) { /* default is in GHz */
	  unit_factor[ii]=1.0;
	} else if(strcmp((*axis[ii]).unit,"MHz")==0) {
	  unit_factor[ii]=1.0e3;
	} else if(strcmp((*axis[ii]).unit,"kHz")==0) {
	  unit_factor[ii]=1.0e6;
	} else if(strcmp((*axis[ii]).unit,"Hz")==0) {
	  unit_factor[ii]=1.0e9;
	} else {
	  unit_factor[ii]=1.0;
	}
      }
      xyz_range[ii].min=xyz_range[ii].min*unit_factor[ii];
      xyz_range[ii].max=xyz_range[ii].max*unit_factor[ii];
      xyz_step[ii]=xyz_step[ii]*unit_factor[ii];
      xyz_width[ii]=xyz_width[ii]*unit_factor[ii];
    }

/* basic */
    if(xyz_step[ii]==0.0) {
      (*axis[ii]).size=1;
    } else {
      (*axis[ii]).size=rint((xyz_range[ii].max-xyz_range[ii].min)/xyz_step[ii]+1.0);
    }
    (*axis[ii]).delta=xyz_step[ii];
    (*axis[ii]).origin=xyz_range[ii].min;
    (*axis[ii]).pixel_origin=0.0;
  }

#ifdef DEBUG
  fprintf(stderr,"resolution.");
#endif
/* resolution */
  for(ii=0;ii<3;ii++) {
    fwhm=(*axis[ii]).resolution;
    (*axis[ii]).resolution=sqrt(xyz_width[ii]*xyz_width[ii]+fwhm*fwhm);

    switch(kernel_type[ii]) {
    case HUNNING:
    case GAUSS:
      if(fwhm<xyz_width[ii]) {
	strcpy((*axis[ii]).psf,"gaussian");
      }
      break;
    case BOXCAR:
    case BINNING:
      if(fwhm<xyz_width[ii]) {
	strcpy((*axis[ii]).psf,"box");
      }
      break;
    case PINPOINT: /* resolution is original */
      (*axis[ii]).resolution=fwhm;
      xyz_width[ii]=0.0;
      break;
    }
  }

#ifdef DEBUG
  fprintf(stderr,"BEadjust.");
#endif
/* BE adjust */
  (*cube_head).be.data_ch=v_axis.size;
  (*cube_head).be.center_ch=v_axis.pixel_origin;
  (*cube_head).be.basis_ch=(unsigned short)(*cube_head).be.center_ch;
  (*cube_head).be.freq_res=v_axis.resolution;
  (*cube_head).be.disp0=(*cube_head).be.center_freq;

  if(strncmp(v_axis.type,"freq",3)==0) {
    (*cube_head).be.disp1=v_axis.delta*1.0e3; /* disp in MHz */
    (*cube_head).be.sign=1;
  } else { /* in velocity */
    (*cube_head).be.disp1=v_axis.delta/(LIGHT_SPEED)*(*cube_head).rx.obs_freq*1.0e3;
    (*cube_head).be.sign= -1;
  }
  (*cube_head).be.disp2=0.0;
  (*cube_head).be.disp3=0.0;
  (*cube_head).be.disp4=0.0;

#ifdef DEBUG
  fprintf(stderr,"RXadjust.");
#endif
/* RX adjust */
  if(strncmp(v_axis.type,"freq",3)==0) {
    (*cube_head).rx.v_lsr=((*cube_head).rx.rest_freq-v_axis.origin)/(*cube_head).rx.obs_freq*(LIGHT_SPEED);
  } else { /* in velocity */
    (*cube_head).rx.v_lsr=v_axis.origin;
  }
  (*cube_head).rx.v_off=(*cube_head).rx.v_lsr;
  (*cube_head).rx.scale=1.0;

#ifdef DEBUG
  fprintf(stderr,"ANTadjust.");
#endif
/* ANT adjust */
  if(axis_long_id>=0)
    hpbw_long=(*axis[axis_long_id]).resolution/unit_factor[axis_long_id];
  else
    hpbw_long=0.0;
  if(axis_lat_id>=0)
    hpbw_lat=(*axis[axis_lat_id]).resolution/unit_factor[axis_lat_id];
  else
    hpbw_lat=0.0;

  fwhm=sqrt(hpbw_long*hpbw_long+hpbw_lat*hpbw_lat);
  fwhm=fwhm*60.0; /* deg --> arcmin */

  if(fwhm>(*cube_head).ant.hpbw) {
    (*cube_head).ant.hpbw=fwhm;
  }

  return;
}

void AddCubeSmoothHistory(cube_head,xyz_range,xyz_step,xyz_width,kernel_type)
CUBE_HEAD *cube_head; /* <i/o> */
RANGE *xyz_range; /* <in> */
double *xyz_step,*xyz_width; /* <in> */
int *kernel_type; /* <in> */
{
  HISTORY	history;
  char buffer[80];
  int ii;
  static char xyz[3]={'x','y','z'};
  int i_prm;

  strcpy(history.step,"smooth");

  i_prm=0;
  for(ii=0;ii<3;ii++) {
    sprintf(buffer,"step=%7.3f",xyz_step[ii]);
    buffer[16]=(char)NULL;
    strcpy(history.prm[i_prm],buffer);
    i_prm++;

    sprintf(buffer,"W%c=%9.3f",xyz[ii],xyz_width[ii]);
    buffer[16]=(char)NULL;
    strcpy(history.prm[i_prm],buffer);
    i_prm++;

    sprintf(buffer,"%c1=%9.2f",xyz[ii],xyz_range[ii].min);
    buffer[16]=(char)NULL;
    strcpy(history.prm[i_prm],buffer);
    i_prm++;

    sprintf(buffer,"%c2=%9.2f",xyz[ii],xyz_range[ii].max);
    buffer[16]=(char)NULL;
    strcpy(history.prm[i_prm],buffer);
    i_prm++;

    switch(kernel_type[ii]) {
    case HUNNING:
      strcpy(history.prm[i_prm],"type=hunning"); break;
    case GAUSS:
      strcpy(history.prm[i_prm],"type=gauss"); break;
    case BOXCAR:
      strcpy(history.prm[i_prm],"type=boxcar"); break;
    case BINNING:
      strcpy(history.prm[i_prm],"type=binning"); break;
    case PINPOINT:
      strcpy(history.prm[i_prm],"type=pin"); break;
    default:
      strcpy(history.prm[i_prm],"type=unknow"); break;
    }
    i_prm++;
  }
  history.size=i_prm;

  StarAddHistory4Cube(cube_head,&history);
}

int main(argc,argv)
int argc;
char *argv[];
{
  CUBE cube_in,cube_out;
  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];
  int err_code;
  int n_file;
  char id_str[BUFFER_SIZE];

  RANGE	xyz_range[3],xyz_range_org[3];
  double xyz_step[3],xyz_width[3];
  double xyz_step_org[3],xyz_width_org[3];
  int kernel_type[3];
  float **pixel_data;

  int ii,jj;

/* get optional parameters */
  err_code=options(argc,argv,xyz_range_org,xyz_step_org,xyz_width_org,kernel_type);

  for(ii=0;ii<3;ii++) {
    StarSortRange(&(xyz_range_org[ii]));
    if(xyz_width_org[ii]<0.0) {
      xyz_width_org[ii]= -xyz_width_org[ii];
      fprintf(stderr,"%s: *err* negative width for axis=%d.\n",argv[0],ii);
      exit(1);
    } else if(xyz_width_org[ii]==0.0 && kernel_type[ii]!=PINPOINT) {
      fprintf(stderr,"%s: *wrn* width=0.0 means pinpoint sampling for axis=%d.\n",argv[0],ii);
      kernel_type[ii]=PINPOINT;
    }
  }

/* pipe header */
  StarPipeHeadIn(id_str,&n_file);

  StarPipeHeadOut(n_file);

#ifdef DEBUG
  fprintf(stderr,"input width=%f,%f,%f\n",xyz_width_org[0],xyz_width_org[1],xyz_width_org[2]);
#endif

/* .cub data */
  for(ii=0;ii<n_file;ii++) {

    StarCubeIn(&cube_in,&cdb_flag,cdb_arg);

    if(cube_in.head.axis.x.delta<=0.0) {
      fprintf(stderr,"dx=%fdeg is negative. Use flipaxis\n",cube_in.head.axis.x.delta);
      exit(10);
    }
    if(cube_in.head.axis.y.delta<=0.0) {
      fprintf(stderr,"dy=%fdeg is negative. Use flipaxis\n",cube_in.head.axis.y.delta);
      exit(10);
    }
    if(cube_in.head.axis.z.delta<=0.0) {
      fprintf(stderr,"dz=%fdeg is negative. Use flipaxis\n",cube_in.head.axis.z.delta);
      exit(10);
    }

    if(cdb_flag==GOOD_SCAN) {
      for(jj=0;jj<3;jj++) {
	xyz_range[jj].min=xyz_range_org[jj].min;
	xyz_range[jj].max=xyz_range_org[jj].max;
	xyz_step[jj]=xyz_step_org[jj];
	xyz_width[jj]=xyz_width_org[jj];
      }

#ifdef DEBUG
      fprintf(stderr,"before copy head\n");
#endif
      StarCopyCubeHead(&(cube_out.head),&(cube_in.head));
#ifdef DEBUG
      fprintf(stderr,"before NewAxes width=%f,%f,%f\n",xyz_width[0],xyz_width[1],xyz_width[2]);
#endif
      MakeNewAxes(&(cube_out.head),xyz_range,xyz_step,xyz_width,kernel_type);
#ifdef DEBUG
      fprintf(stderr,"NewAxes width=%f,%f,%f\n",xyz_width[0],xyz_width[1],xyz_width[2]);
      fprintf(stderr,"NewAxesSize=%ld,%ld,%ld\n",cube_out.head.axis.x.size,cube_out.head.axis.y.size,cube_out.head.axis.z.size);
#endif

      pixel_data=StarAllocCubePixel(&(cube_out.head));
      if(pixel_data==NULL) {
	fprintf(stderr,"%s: pixel data for %s cannot allocate.\n",argv[0],cube_out.head.file_name);
	exit(1);
      }
      cube_out.pixel=pixel_data;

      StarCubePixelClear(&cube_out,cube_out.head.data_value.dummy);
      StarCubeSmoothAndConvert(&cube_out,&cube_in,xyz_width,kernel_type);

      StarCubeScanDatavalue(&cube_out);

      AddCubeSmoothHistory(&(cube_out.head),xyz_range,xyz_step,xyz_width,kernel_type);

      StarCubeOut(&cube_out,cdb_flag,cdb_arg);

      StarFreeCubePixel(&cube_out);
    } else if(cdb_flag==BAD_SCAN) {
      StarCubeOut(&cube_in,cdb_flag,cdb_arg);
    } else {
      StarCubeOut(&cube_in,cdb_flag,cdb_arg);
    }

    StarFreeCubePixel(&cube_in);
  }

/* end tag of pipe */
  StarPipeEndIn(id_str);
  StarPipeEndOut();

  exit(0);
}
