/*
trim and extend the cube pixel size

2005/ 4/19 T.Handa : ver.3.00 
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"cubresample.prm"
#define N_KEY 50
#define N_BLOCK	10

#define DEBUG
#undef DEBUG

extern void StarCopyCubePixel(CUBE *,CUBE *);

void ExtendAxisRange(axis,i_range)
/*
extend and/or reduce pixel range along an axis
with proper axis range in physics unit

2005/4/19 T.Handa
*/
AXIS *axis; /* <i/o> */
RANGE_long i_range; /* <in> */
{
  (*axis).size=i_range.max-i_range.min+1;
  (*axis).pixel_origin=(*axis).pixel_origin-(double)i_range.min;

  return;
}

void ExtendAxesRange(axis,ix_range,iy_range,iz_range)
AXES *axis; /* <i/o> */
RANGE_long ix_range; /* <in> */
RANGE_long iy_range; /* <in> */
RANGE_long iz_range; /* <in> */
{
  ExtendAxisRange(&((*axis).x),ix_range);
  ExtendAxisRange(&((*axis).y),iy_range);
  ExtendAxisRange(&((*axis).z),iz_range);

  return;
}

void StarCopyCubePixel(cube_out,cube_in)
/*
2005/4/19 T.Handa
 */
CUBE *cube_out; /* <out> copied cube */
CUBE *cube_in; /* <in> original cube */
{
  int mx,my,mz;
  int ix,iy,iz;
  int jx,jy,jz;
  float pixel;

  mx=(*cube_in).head.axis.x.size;
  my=(*cube_in).head.axis.y.size;
  mz=(*cube_in).head.axis.z.size;

  for(ix=0;ix<mx;ix++) {
    jx=ix;
    for(iy=0;iy<my;iy++) {
      jy=iy;
      for(iz=0;iz<mz;iz++) {
	jz=iz;
	pixel=StarGetCubePixel(cube_in,ix,iy,iz);
	StarPutCubePixel(cube_out,jx,jy,jz,pixel);	
      }
    }
  }

  return;
}

void StarClearCubePixel(cube)
/*
2005/4/19 T.Handa
 */
CUBE *cube; /* <i/o> */
{
  int mx,my,mz;
  int ix,iy,iz;
  float dummy;

  mx=(*cube).head.axis.x.size;
  my=(*cube).head.axis.y.size;
  mz=(*cube).head.axis.z.size;

  dummy=(*cube).head.data_value.dummy;

  for(ix=0;ix<mx;ix++) {
    for(iy=0;iy<my;iy++) {
      for(iz=0;iz<mz;iz++) {
	StarPutCubePixel(cube,ix,iy,iz,dummy);	
      }
    }
  }

  return;
}

void StarShiftedCopyCubePixel(cube_out,cube_in,ix_range,iy_range,iz_range)
/*
2005/4/19 T.Handa
*/
CUBE *cube_out; /* <out> */
CUBE *cube_in; /* <in> */
RANGE_long ix_range; /* <in> */
RANGE_long iy_range; /* <in> */
RANGE_long iz_range; /* <in> */
{
  int mx,my,mz;
  int nx,ny,nz;
  int ix,iy,iz;
  int jx,jy,jz;
  float pixel;

  mx=(*cube_in).head.axis.x.size;
  my=(*cube_in).head.axis.y.size;
  mz=(*cube_in).head.axis.z.size;
  nx=(*cube_out).head.axis.x.size;
  ny=(*cube_out).head.axis.y.size;
  nz=(*cube_out).head.axis.z.size;

  for(iz=0;iz<mz;iz++) {
    jz=iz-iz_range.min;
    if(jz>=0 && jz<nz) { /* z_range inside */
      for(iy=0;iy<my;iy++) {
	jy=iy-iy_range.min;
	if(jy>=0 && jy<ny) { /* y_range inside */
	  for(ix=0;ix<mx;ix++) {
	    jx=ix-ix_range.min;
	    if(jx>=0 && jx<nx) { /* z_range inside */
	      pixel=StarGetCubePixel(cube_in,ix,iy,iz);
	      StarPutCubePixel(cube_out,jx,jy,jz,pixel);	
	    }
	  }
	}
      }
    }
  }

  return;
}

#define BUFF_SIZE 2048


int options(argc,argv,ix_org,iy_org,iz_org,ix_range,iy_range,iz_range)
/*
get parameters from options
*/
int argc;
char *argv[];
int *ix_org,*iy_org,*iz_org;
RANGE_long *ix_range;
RANGE_long *iy_range;
RANGE_long *iz_range;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  *ix_org=FLAG_YES; (*ix_range).min=0; (*ix_range).max=0;
  *iy_org=FLAG_YES; (*iy_range).min=0; (*iy_range).max=0;
  *iz_org=FLAG_YES; (*iz_range).min=0; (*iz_range).max=0;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"x");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* new range along x-axis */
  n_scan=1;
  scan_flag[0]=0;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strncmp(got_value,"orig",4)==0) {
      *ix_org=FLAG_YES;
      (*ix_range).min=0;
      (*ix_range).max=0;
    } else {
      *ix_org=FLAG_NO;
      StarOptionRange_long(got_value,ix_range);
    }
  }
  StarSortRangeLong(ix_range);
/* new range along y-axis */
  n_scan=1;
  scan_flag[0]=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strncmp(got_value,"orig",4)==0) {
      *iy_org=FLAG_YES;
      (*iy_range).min=0;
      (*iy_range).max=0;
    } else {
      *iy_org=FLAG_NO;
      StarOptionRange_long(got_value,iy_range);
    }
  }
  StarSortRangeLong(iy_range);
/* new range along z-axis */
  n_scan=1;
  scan_flag[0]=2;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strncmp(got_value,"orig",4)==0) {
      *iz_org=FLAG_YES;
      (*iz_range).min=0;
      (*iz_range).max=0;
    } else {
      *iz_org=FLAG_NO;
      StarOptionRange_long(got_value,iz_range);
    }
  }
  StarSortRangeLong(iz_range);

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

int main(argc,argv)
int argc;
char *argv[];
{
  CUBE cube_in,cube_out;

  RANGE_long ix_range,iy_range,iz_range;
  int ix_org,iy_org,iz_org;

  int err_code;

  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];

  int i_file,n_file;
  char id_str[MAX_FILE_NAME_LENGTH];

/* check option */
  err_code=options(argc,argv,&ix_org,&iy_org,&iz_org,&ix_range,&iy_range,&iz_range);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

/* pipe header */
  StarPipeHeadIn(id_str,&n_file);

  StarPipeHeadOut(n_file);

/* .cub data */
  for(i_file=0;i_file<n_file;i_file++) {
    err_code=StarCubeIn(&cube_in,&cdb_flag,cdb_arg);
    if(err_code!=0) {
      fprintf(stderr,"%s: CubeIn failed in %dth file. code=%d\n",argv[0],i_file+1,err_code);
      exit(999);
    }

/* copy cube header */
    StarCopyCubeHead(&(cube_out.head),&(cube_in.head));

    if(ix_org==FLAG_YES) {
      ix_range.min=0;
      ix_range.max=cube_in.head.axis.x.size-1;
    }
    if(iy_org==FLAG_YES) {
      iy_range.min=0;
      iy_range.max=cube_in.head.axis.y.size-1;
    }
    if(iz_org==FLAG_YES) {
      iz_range.min=0;
      iz_range.max=cube_in.head.axis.z.size-1;
    }

/* modify axes */
    ExtendAxesRange(&(cube_out.head.axis),ix_range,iy_range,iz_range);

/* copy pixel */
    cube_out.pixel=StarAllocCubePixel(&(cube_out.head));
    StarClearCubePixel(&cube_out);
    StarShiftedCopyCubePixel(&cube_out,&cube_in,ix_range,iy_range,iz_range);
    StarCubeScanDatavalue(&cube_out);

/* cube out */
    StarCubeOut(&cube_out,cdb_flag,cdb_arg);

    StarFreeCubePixel(&cube_in);
    StarFreeCubePixel(&cube_out);
  }

/* end tag of pipe */
  StarPipeEndIn(id_str);
  StarPipeEndOut();

  exit(0);
}
