/*
display profile data on X11 screen

originate T.OKA
1997/12/12 T.Handa : ver.3.00 multi_pipe and new library
1998/2/28 T.Handa : ver.3.01 debug for SUN-OS
1998/3/19 T.Handa : ver.3.01 debug for Solaris2 fcc
1998/4/28 T.Handa : ver.3.20 (revise) .scn,.sdb
1998/5/18 T.Handa : ver.3.21 (revise) show base/spur range
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include <Xm/DrawingA.h>
#include <Xm/MainW.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#include <Xm/PushBG.h>

#include <StData.h>
#include <StStd.h>

#include <Pxxlib.h>
#include <Pxplib.h>
#include <Ppslib.h>
#include <PxpGlib.h>
#include <StPxpShow.h>

#define	DEFAULT_PRMFILE	"display.prm"
#define N_KEY 20

static PROFILE gl_profile;
static int GLsdb_flag;
static char GLsdb_arg[MAX_IDX_RECORD_LENGTH];
static int GLn_file=0;
static int GLpipeout= -1;		/* pipe out for the next command */
static int GLx_unit;
static RANGE GLx_range;
static RANGE GLy_range;
static int GLbase_show,GLspur_show;

/*-------Ext------*/
/*--------from Pxxlib--------------------*/
static void	(*do_plot)(), (*do_exit)(), (*do_help)();


/* define actions for buttons on the menu bar */
void local_StarPxxPushButtonActions(menubarWDG,client_data,call_data)
Widget	menubarWDG;
caddr_t	client_data,call_data;
/*
define actions for buttons on the menu bar
1997/10/12 T.Handa ver.1.00
1997/12/12 T.Handa ver.1.00 for display command
*/
{
  if(strcmp(client_data,"Exit")==0) {
    XClearWindow(PxxGetDisplay(),PxxGetWindow());
    if(do_exit!=NULL) (*do_exit)();
    if(do_plot!=NULL) (*do_plot)();
  }
  else if(strcmp(client_data,"Quit")==0) {
    exit(0);
  }
  else if(strcmp(client_data,"Print")==0)
    PxxExecPrintOut();
  else if(strcmp(client_data,"Help")==0)
    if(do_help!=NULL) (*do_help)();
}

#define MAX_ARG 100
#define	CHAR_SIZE_DEFAULT_RESOURCE	128

/* display on X11 */
void local_StarPxxDisplayAll(argc,argv, 
    app_class,width,height,plt_fnct,exit_fnct,help_fnct)
/*
create and plot an X11 window

1997/10/12 T.Handa ver.1.00
1997/12/02 T.Handa ver.1.00 for display command
1998/ 5/ 6 T.Handa ver.1.02 (debug)
*/
int argc;
char *argv[];
char app_class[];	/* name of the application class (use in .Xresource) */
double width,height;
void (*plt_fnct)(),(*exit_fnct)(),(*help_fnct)();
{
  Widget  toplevelWDG, main_windowWDG, drawing_areaWDG, 
    manu_barWDG, mbBtn[4];
  Arg     args[10];
  int     ii;

  int	mainwin_x,mainwin_y;

  int i_btn_id;

  XtAppContext    app_context;
  Arg fixed_res_args[MAX_ARG];     /* resource arguments */
  Cardinal        fixed_res_n_args;
  static char	window_w[CHAR_SIZE_DEFAULT_RESOURCE],
    window_h[CHAR_SIZE_DEFAULT_RESOURCE];
  static String   fb_res[] = {
    "*mainWindow.width:        500 ",
    "*mainWindow.height:       300 ",
    "*drawArea.background:	white",
    "*menubar.topAttachment:	ATTACH_FORM",
    "*menubar.rightAttachment:	ATTACH_FORM",
    "*menubar.leftAttachment:	ATTACH_FORM",
    NULL};
/* add for the display command only */
  static char psfile_name[]="";
  static char prt_name[]="";

/* Change initial window size */
  mainwin_x=(int)(width*CM2INCH*DVI_SCALE+0.5);
  mainwin_y=(int)(height*CM2INCH*DVI_SCALE+0.5);
  if(mainwin_x>0 && mainwin_x<9999999) {
    sprintf(window_w,"*mainWindow.width: %8d",mainwin_x);
    fb_res[0]=window_w;
  }
  if(mainwin_y>0 && mainwin_y<9999999) {
    sprintf(window_h,"*mainWindow.height:%8d",mainwin_y);
    fb_res[1]=window_h;
  }

/*	PxxView_width=width; */
/*	PxxView_height=height; */
/*	PxxView_printer[0]=(char)NULL; */	/* set default printer */
/*	PxxView_psfile[0]=(char)NULL;*/	/* set temporaly file */
/* add for the display command only */
  PxxPutWindowSizeInPixel(width,height);
  PxxSetPrinterName(prt_name);
  PxxSetPSFName(psfile_name);

/* printer name in argv */
  for(ii=1;ii<argc;ii++) {
    if(strcmp(argv[ii],"-P")==0) PxxSetPrinterName(argv[ii+1]);
    else if(argv[ii][0]=='-' && argv[ii][1]=='P')
      PxxSetPrinterName(&(argv[ii][2]));
  }

/* PS filename in argv */
  for(ii=1;ii<argc;ii++) {
    if(strcmp(argv[ii],"-F")==0) PxxSetPSFName(argv[ii+1]);
    else if(argv[ii][0]=='-' && argv[ii][1]=='F')
      PxxSetPSFName(&(argv[ii][2]));
  }

/* set plot, exit, and help functions */
  PxxSetPlotFunction(plt_fnct);
  PxxSetExitFunction(exit_fnct);
  PxxSetHelpFunction(help_fnct);
/* add for the display commnad only */
  do_plot=plt_fnct;
  do_exit=exit_fnct;
  do_help=help_fnct;

/* create X11 window */
  fixed_res_n_args=0;
  toplevelWDG=XtAppInitialize(&app_context,app_class, NULL, 0, &argc, argv,
			      fb_res,fixed_res_args,fixed_res_n_args);

  main_windowWDG=XmCreateMainWindow(toplevelWDG,"mainWindow",NULL,(Cardinal)NULL);
  XtManageChild(main_windowWDG);

/* create menu bar */
  manu_barWDG=XmCreateMenuBar(main_windowWDG,"menubar",NULL,(Cardinal)NULL);
  XtManageChild(manu_barWDG);
  i_btn_id=0;

  mbBtn[i_btn_id]=XmCreateCascadeButton(manu_barWDG,"help", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, (XtCallbackProc)local_StarPxxPushButtonActions,"Help");
  i_btn_id++;

  mbBtn[i_btn_id]=XmCreateCascadeButton(manu_barWDG,"next", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, (XtCallbackProc)local_StarPxxPushButtonActions, "Exit");
  i_btn_id++;
  mbBtn[i_btn_id]=XmCreateCascadeButton(manu_barWDG,"print", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, (XtCallbackProc)local_StarPxxPushButtonActions, "Print");
  i_btn_id++;
  mbBtn[i_btn_id]=XmCreateCascadeButton(manu_barWDG,"quit", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, (XtCallbackProc)local_StarPxxPushButtonActions, "Quit");
  i_btn_id++;
  XtManageChildren(mbBtn,i_btn_id);

  XtSetArg(args[0],XmNmenuHelpWidget,(XtArgVal)mbBtn[0]);
  XtSetValues(manu_barWDG,args,1);

/* create data ploting area */
  XtSetArg(args[0],XmNresizePolicy,XmRESIZE_ANY);
  drawing_areaWDG=XmCreateDrawingArea(main_windowWDG,"drawArea",args,1);
  XtManageChild(drawing_areaWDG);

/* set alinement of the componets in a window */
  XmMainWindowSetAreas(main_windowWDG,manu_barWDG,NULL,NULL,NULL,drawing_areaWDG);

/* realize window */
  XtRealizeWidget(toplevelWDG);

  XtAddCallback(drawing_areaWDG,XmNexposeCallback,(XtCallbackProc)PxxExecDraw,&drawing_areaWDG);
  XtAddCallback(drawing_areaWDG,XmNresizeCallback,(XtCallbackProc)PxxExecDraw,&drawing_areaWDG);

/* keep showing */
  XtAppMainLoop(app_context);
}

/*--------from Pxplib--------------------*/
/* execute X11 display or PostScript output */
void local_StarPxpDisplayAll(argc,argv, output, filename,
	app_class,width,height,plt_fnct,exit_fnct,help_fnct)
/*
create the standared X11 window or PostScripte file

1997/10/14 T.Handa ver.1.00
1997/12/02 T.Handa ver.1.00 for the display command
*/
int argc;
char *argv[];
int output;
char *filename;	/* stdout for - or NULL, workfile for filename==NULL */
char app_class[];	/* name of the application class (use in .Xresource) */
double width,height;
void (*plt_fnct)(),(*exit_fnct)(),(*help_fnct)();
{
  void (*plot_function)();

  plot_function=plt_fnct;
/*	PxpView_output=output;*/
  PxpPutOutputMode(output);

  switch(output) {
  case X11WIN:
    local_StarPxxDisplayAll(argc,argv, app_class,width,height,plt_fnct,exit_fnct,help_fnct);
    break;
  case POSTSCRIPT:
    PpsInitialize(filename,width,height);
    plot_function();	/* execute the user-defined plot function */
    PpsPrintPage();
    PpsDoPrint();
  }
}

/*---------------------------------*/

int options(argc,argv, w_height,w_width,x_unit,x_range,T_range,pipe_flag,bs_flag,sp_flag)
/*
get parameters from options
*/
int argc;
char *argv[];
double	*w_height, *w_width;	/* size of the window(cm) */
int	*x_unit;
RANGE	*x_range,*T_range;
int	*pipe_flag;
int *bs_flag; /* show baseline range ? */
int *sp_flag; /* show spurious range ? */
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *token;
  int opt_pos;
  char buffer[MAX_PRM_RECORD_LENGTH];

  int tmp_flag;
  double window_x,window_y;

  char *got_value;

/* default values */
  *w_height=15.0;	*w_width=20.0;
  *x_unit=X_CHANNEL;
  (*x_range).min=0.0;	(*x_range).max=0.0;
  (*T_range).min=0.0;	(*T_range).max=0.0;
  *pipe_flag= -1;
  *bs_flag= 0; *sp_flag= 0;

  window_x=0;
  window_y=0;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"size");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"ch");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"v");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"lsr");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"x_unit");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"T");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"pipe");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"-");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"nopipe");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"baseline");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"nobaseline");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"spurious");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"nospurious");	key_type[n_key-1]=0;


  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* window size */
  scan_flag[0]=0;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(buffer,got_value);
    token=strtok(buffer,"xX ,\t\n");
    if(token!=NULL) {
      window_x=atof(token);
      token=strtok(NULL," ,\t\n");
      if(token!=NULL) window_y=atof(token);
    }
    *w_height=window_y;
    *w_width=window_x;
  }

/* ch,v,lsr,x_unit */
  scan_flag[0]=1;	scan_flag[1]=2; scan_flag[2]=3; scan_flag[3]=4;
  n_scan=4;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    if(strcmp(keyword[opt_pos],"v")==0
       || strcmp(keyword[opt_pos],"lsr")==0
       || strcmp(value[opt_pos],"velocity")==0
       || strcmp(value[opt_pos],"lsr")==0
       || strcmp(value[opt_pos],"LSR")==0)
      *x_unit=X_VELOCITY;
    else if(strcmp(keyword[opt_pos],"ch")==0
	    || strcmp(value[opt_pos],"channel")==0
	    || strcmp(value[opt_pos],"ch")==0)
      *x_unit=X_CHANNEL;
    StarOptionRange(value[opt_pos],x_range);
  } 

/* y_range */
  scan_flag[0]=5;
  n_scan=1;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    strcpy(buffer,value[opt_pos]);
    StarOptionRange(buffer,T_range);
  } 

/* show baseline range flag */
  scan_flag[0]=9; scan_flag[1]= -10;
  n_scan=2;
  tmp_flag=StarOptionFlag(n_scan,scan_flag,key_flag);
  if(tmp_flag!=0) *bs_flag=tmp_flag;

/* show spurious range flag */
  scan_flag[0]=11; scan_flag[1]= -12;
  n_scan=2;
  tmp_flag=StarOptionFlag(n_scan,scan_flag,key_flag);
  if(tmp_flag!=0) *sp_flag=tmp_flag;

/* pipe */
  scan_flag[0]=6; scan_flag[1]=7; scan_flag[2]= -8;
  n_scan=3;
  tmp_flag=StarOptionFlag(n_scan,scan_flag,key_flag);
  if(tmp_flag!=0) *pipe_flag=tmp_flag;

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);
/* set return values */

  return(0);
}


/* Main Body of process */

void read_profile()
{
  static int i_file=0;
  int n_file,n_act_file,pipeout;
  int err_code;

/* copy from global valuables */
  n_file=GLn_file;
  pipeout=GLpipeout;

  n_act_file=0;
  while(1) {
    if(i_file>=n_file) {
      if(pipeout>0) StarPipeEndOut();
      exit(0);
    }

    err_code=StarProfileIn(&gl_profile,&GLsdb_flag,GLsdb_arg);
    if(err_code!=0) {
      fprintf(stderr,"fail to ProfileIn\n");
      exit(1);
    }
    i_file++;

    if(pipeout>0)
      StarProfileOut(&gl_profile,GLsdb_flag,GLsdb_arg);

    if(GLsdb_flag==GOOD_SCAN || GLsdb_flag==BAD_SCAN) {
      n_act_file++;
    }

    if(GLsdb_flag==GOOD_SCAN || GLsdb_flag==BAD_SCAN)
      break;
  }

  if(n_act_file==0) {
    if(pipeout>0) StarPipeEndOut();
    exit(0);
  }
}


void actual_plot()
{
  static int read_flag=1;
  RANGE x_range,y_range;
  int x_unit;

  BOX_RANGE plot_box;

  static PxpPoint win_size;

  int output_mode;
  int window_x,window_y,window_h,window_w;

/* copy from global valuables */
  x_range.min=GLx_range.min;
  x_range.max=GLx_range.max;
  x_unit=GLx_unit;
  y_range.min=GLy_range.min;
  y_range.max=GLy_range.max;

/* read profile if the first */
  if(read_flag>0) {
    read_profile(&gl_profile);
    read_flag= -1;
  }

/* plotsize auto scaling */
  PxpGetOutputMode(&output_mode);
  if(output_mode==X11WIN) {
    PxxPresentWindowSize(&window_x,&window_y,&window_w,&window_h);
    win_size.x=PxxGetPlotSpanX(window_w);
    win_size.y=PxxGetPlotSpanY(window_h);
  }

  plot_box.x.min=win_size.x*0.1;
  plot_box.y.min=win_size.y*0.1;
  plot_box.x.max=plot_box.x.min+win_size.x*0.8;
  plot_box.y.max=plot_box.y.min+win_size.y*0.8;

/* profile in standard form */
  StarPlotProfileStd(&gl_profile,x_unit,x_range,y_range,plot_box,GLbase_show,GLspur_show);
}

void help_function()
{
  fprintf(stderr,"help should be shown.\n");
  return;
}

int main(argc,argv)
int argc;
char *argv[];
{
  int err_code;
  int n_file;

  double w_height, w_width;	/* size of window(cm) */
  int	x_unit;
  RANGE	x_range,y_range;	/* range to display */
  int pipeout;
  int bs_flag,sp_flag;

  char id_str[BUFFER_SIZE];

/* check option */
  err_code=options(argc,argv, &w_height,&w_width,&x_unit,&x_range,&y_range,&pipeout,&bs_flag,&sp_flag);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

/* set global parameters */
  GLpipeout=pipeout;
  GLx_unit=x_unit;
  GLx_range.min=x_range.min;
  GLx_range.max=x_range.max;
  GLy_range.min=y_range.min;
  GLy_range.max=y_range.max;

  GLbase_show=bs_flag;
  GLspur_show=sp_flag;

/* multi-data header in */
  StarPipeHeadIn(id_str,&n_file);
  GLn_file=n_file;

  if(pipeout>0) StarPipeHeadOut(n_file);

/* plot profile */
  local_StarPxpDisplayAll(argc,argv, X11WIN,NULL,"display",w_width,w_height,actual_plot,read_profile,help_function);

  exit(0);
}
