/*
exchange and rotate axis order

2006/ 4/28 T.Handa : ver.3.00
2006/12/12 T.Handa : ver.3.01 (modify) completely interpolation
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"fillblank.prm"
#define N_KEY 50
#define N_BLOCK	10

#define AXIS_NO 0
#define AXIS_X 1
#define AXIS_Y 2
#define AXIS_Z 3
#define AXIS_LONG 11
#define AXIS_LAT 12
#define AXIS_VELO 13

#define DEBUG
#undef DEBUG

#define FILL_CHECKER 0
#define FILL_FULL 256

double InterpolateSmallMap(cube_in,ix,iy,iz,type,flag)
/*
Interpolate the pixel only for the checker patturn blank

2006/ 4/28 T.Handa 
2006/12/12 T.Handa (modify) for full mode 
 */
CUBE *cube_in; /* <in> */
int ix,iy,iz; /* <in> */
int type; /* <in> fill type =FILL_CHECKER, =FILL_FULL */
int *flag; /* <out> */
{
  int nx,ny,nz;
  int jx,jy,jjx,jjy;
  int jx0,jx1,jy0,jy1;
  int valid_data[3][3];
  double small_map[3][3];
  double interp_data;
  int n_data;
  int min_valid_pixel;

  if(type==FILL_FULL) {
    min_valid_pixel=2;
  } else {
    min_valid_pixel=4;
  }

  nx=(*cube_in).head.axis.x.size;
  ny=(*cube_in).head.axis.y.size;
  nz=(*cube_in).head.axis.z.size;

  if(ix==0) jx0=0; else jx0= -1;
  if(ix==nx-1) jx1=0; else jx1=1;
  if(iy==0) jy0=0; else jy0= -1;
  if(iy==ny-1) jy1=0; else jy1=1;

  n_data=0;
  interp_data=0.0;
  for(jx=0;jx<3;jx++) {
    for(jy=0;jy<3;jy++) {
      valid_data[jx][jy]= -100;
    }
  }

  for(jx=jx0;jx<=jx1;jx++) {
    jjx=jx+1;
    for(jy=jy0;jy<=jy1;jy++) {
      jjy=jy+1;
      small_map[jjx][jjy]=StarGetCubePixelData(cube_in,ix+jx,iy+jy,iz,&(valid_data[jjx][jjy]));
      if(valid_data[jjx][jjy]==FLAG_YES) {
	n_data++;
	interp_data=interp_data+small_map[jjx][jjy];
      }
    }
  }

  if(n_data>=min_valid_pixel) {
    *flag= n_data;
    interp_data=interp_data/(double)n_data;
    if(valid_data[0][1]==FLAG_YES && valid_data[2][1]==FLAG_YES)
      *flag+=10;
    if(valid_data[1][0]==FLAG_YES && valid_data[1][2]==FLAG_YES)
      *flag+=10;
    if(valid_data[0][0]==FLAG_YES && valid_data[2][2]==FLAG_YES)
      *flag+=10;
    if(valid_data[0][2]==FLAG_YES && valid_data[2][0]==FLAG_YES)
      *flag+=10;
  } else {
    *flag= n_data;
  }

  return(interp_data);
}

int FillBlankPixel(cube_out,cube_in,type)
/*
<return>
the total number of filled pixels

2006/12/13 T.Handa
 */
CUBE *cube_out; /* <out> */
CUBE *cube_in; /* <in> */
int type; /* <in> fill type =FILL_CHECKER, =FILL_FULL */
{
  int ix,iy,iz;
  int nx,ny,nz;
  double data;
  int valid_data;
  int interp_flag;
  int n_pixel;

  nx=(*cube_in).head.axis.x.size;
  ny=(*cube_in).head.axis.y.size;
  nz=(*cube_in).head.axis.z.size;

  n_pixel=0;

  for(iz=0;iz<nz;iz++) {
    for(iy=0;iy<ny;iy++) {
      for(ix=0;ix<nx;ix++) {
	data=StarGetCubePixelData(cube_in,ix,iy,iz,&valid_data);
	if(valid_data==FLAG_YES) {
	  StarPutCubePixelData(cube_out,ix,iy,iz,data);
	} else {
	  data=InterpolateSmallMap(cube_in,ix,iy,iz,type,&interp_flag);
	  if(type==FILL_FULL && interp_flag>=12) {
	    StarPutCubePixelData(cube_out,ix,iy,iz,data);
	    n_pixel++;
	  } else if(type==FILL_CHECKER && interp_flag>=13) {
	    StarPutCubePixelData(cube_out,ix,iy,iz,data);
	    n_pixel++;
	  }
	}
      }
    }
  }

  return(n_pixel);
}

int options(argc,argv,fill_type)
/*
get parameters from options
*/
int argc;
char *argv[];
int *fill_type;
{
  int ii,next_pos;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  *fill_type=FILL_CHECKER;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"fill_type");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

  next_pos=0;
/* fill type */
  n_scan=1;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value!=NULL) {
    if(strcmp(got_value,"checker")==0 || strcmp(got_value,"dice5")==0) {
      *fill_type=FILL_CHECKER;
    } else if(strcmp(got_value,"full")==0) {
      *fill_type=FILL_FULL;
    }
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

int fillblank_add_history(cube_head,fill_type,n_pixel)
CUBE_HEAD *cube_head; /* <i/o> */
int fill_type; /* <in> */
int n_pixel; /* <in> */
{
  HISTORY hist;
  int h_size;

  h_size=0;

  strcpy(hist.step,"fillBlk");

  switch(fill_type) {
  case FILL_CHECKER:
    strcpy(hist.prm[h_size],"checker");
    break;
  case FILL_FULL:
    strcpy(hist.prm[h_size],"full");
    break;
  default:
    strcpy(hist.prm[h_size],"undefined");
  }
  h_size++;
  sprintf(hist.prm[h_size],"%d pix",n_pixel);
  h_size++;

  hist.size=h_size;
  StarAddHistory4Cube(cube_head,&hist);

  return(0);
}

int main(argc,argv)
int argc;
char *argv[];
{
  CUBE cube_in,cube_out;

  int err_code;

  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];

  int ii,n_file;
  char id_str[MAX_FILE_NAME_LENGTH];

  int fill_type;
  int n_pixel;

/* check option */
  err_code=options(argc,argv,&fill_type);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

/* pipe header */
  StarPipeHeadIn(id_str,&n_file);

  StarPipeHeadOut(n_file);

/* .cub data */
  for(ii=0;ii<n_file;ii++) {
    err_code=StarCubeIn(&cube_in,&cdb_flag,cdb_arg);
    if(err_code!=0) {
      fprintf(stderr,"%s: CubeIn failed in %dth file. code=%d\n",argv[0],ii+1,err_code);
      exit(999);
    }
    StarCopyCubeHead(&(cube_out.head),&(cube_in.head));

    cube_out.pixel=StarAllocCubePixel(&(cube_out.head));

    if(cube_out.pixel==NULL) {
      fprintf(stderr,"%s: Pixel allocation failed. more memory.\n",argv[0]);
      exit(999);
    }

    n_pixel=FillBlankPixel(&cube_out,&cube_in,fill_type);

    fillblank_add_history(&(cube_out.head),fill_type,n_pixel);

    StarCubeOut(&cube_out,cdb_flag,cdb_arg);

    StarFreeCubePixel(&cube_in);
    StarFreeCubePixel(&cube_out);
  }

/* end tag of pipe */
  StarPipeEndIn(id_str);
  StarPipeEndOut();

  exit(0);
}
