/*
3D FITS file to cub flow

1999/8/17 T.Handa : ver.3.01
2002/5/7 T.Handa : ver.3.02 (option for vel. in km/s even if without unit tag)
2002/6/24 T.Handa : ver.3.03 (for naxis=2 data)
2003/3/19 T.Handa : ver.3.04 (modify) auto calc data_value if no in the FITS header
2006/5/22 T.Handa : ver.3.05 (debug) core dump via ssh; 2D array should be static
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>
#include <StFitsIO.h>

#define DEFAULT_PRMFILE "fits2cub.prm"
#define N_KEY 30

#define MAX_DEBUG_FLAG 10

#ifdef _NO_PROTO

extern int StarFitsHead2CubeHead();
#else

extern int StarFitsHead2CubeHead(FILE *, CUBE_HEAD *, int *, int *, FILE *);
#endif

void force_to_unit_kms(x)
AXIS *x;
{
  if(strcmp((*x).type,"velocity")==0) {
    (*x).origin=(*x).origin*1.0e3;
    (*x).delta=(*x).delta*1.0e3;
    (*x).resolution=(*x).resolution*1.0e3;
    (*x).res_orig=(*x).res_orig*1.0e3;
  }
  return;
}

void StarDefaultAxes(axis)
AXES *axis;
{
  (*axis).dimension=3;
  strcpy((*axis).order,"xyz");

  strcpy((*axis).x.name,"x");
  (*axis).x.size=0;
  strcpy((*axis).x.type,"x");
  strcpy((*axis).x.unit,"pixel");
  strcpy((*axis).x.project,"linear");
  strcpy((*axis).x.refer,"");
  (*axis).x.delta=1.0;
  (*axis).x.origin=0.0;
  (*axis).x.pixel_origin=0.0;
  (*axis).x.rotation=0.0;
  (*axis).x.resolution=0.0;
  strcpy((*axis).x.psf,"gauss");

  strcpy((*axis).y.name,"y");
  (*axis).y.size=0;
  strcpy((*axis).y.type,"y");
  strcpy((*axis).y.unit,"pixel");
  strcpy((*axis).y.project,"linear");
  strcpy((*axis).y.refer,"");
  (*axis).y.delta=1.0;
  (*axis).y.origin=0.0;
  (*axis).y.pixel_origin=0.0;
  (*axis).y.rotation=0.0;
  (*axis).y.resolution=0.0;
  strcpy((*axis).y.psf,"gauss");
  (*axis).y.res_orig=0.0;

  strcpy((*axis).z.name,"z");
  (*axis).z.size=0;
  strcpy((*axis).z.type,"z");
  strcpy((*axis).z.unit,"pixel");
  strcpy((*axis).z.project,"linear");
  strcpy((*axis).z.refer,"");
  (*axis).z.delta=1.0;
  (*axis).z.origin=0.0;
  (*axis).z.pixel_origin=0.0;
  (*axis).z.rotation=0.0;
  (*axis).z.resolution=0.0;
  strcpy((*axis).z.psf,"gauss");
  (*axis).z.res_orig=0.0;

  return;
}

void StarDefaultDatavalue(data,dummy)
DATA_VALUE *data;
double dummy;
{
  strcpy((*data).type,"Ta*");
  strcpy((*data).unit,"K");
  (*data).scale=1.0;
  (*data).offset=0.0;
  (*data).dummy=(float)dummy;
  (*data).max=(*data).dummy;
  (*data).min=(*data).dummy;
  (*data).mean=(*data).dummy;
  (*data).rms=0.0;
  (*data).noise=0.0;

  return;
}

void set_default_cubehead(cube_head)
CUBE_HEAD *cube_head;
{
  StarDefaultSyshead(&((*cube_head).sys));
  StarDefaultObsprm(&((*cube_head).obs));
  StarDefaultObject(&((*cube_head).object));
  StarDefaultAntStatus(&((*cube_head).ant));
  StarDefaultRxStatus(&((*cube_head).rx));
  StarDefaultBeStatus(&((*cube_head).be));
  StarDefaultObsLog(&((*cube_head).log));
  StarDefaultAxes(&((*cube_head).axis));
  StarDefaultDatavalue(&((*cube_head).data_value),0.0);

  return;
}

void fits2Syshead(sys_head,key,type,value)
SYS_HEAD *sys_head;
char *key;
int type;
char *value;
{

  return;
}

void fits2Obsprm(obsprm,key,type,value)
OBS_PRM *obsprm;
char *key;
int type;
char *value;
{
/*
  strcpy((*obsprm).project,"from FITS");
  strcpy((*obsprm).scd_file,"");
  (*obsprm).line_numb=(unsigned short)0;
  (*obsprm).sw_mode=ONOFF_SW;
*/
  if(strcmp(key,"DATE")==0
  || ( strcmp(key,"DATE-OBS")==0 && (*obsprm).mjd==0) ) {
    (*obsprm).mjd=StarFitsDate2MJD(value);
  }
  if(strcmp(key,"DATE-OBS")==0) {
    (*obsprm).start=StarFitsDate2Time1970(value);
    if((*obsprm).stop==0L)
      (*obsprm).stop=(*obsprm).start;
  }
/*
  (*obsprm).on_time=(unsigned short)0;
  (*obsprm).off_time=(unsigned short)0;
*/
  if(strcmp(key,"OBSERVER")==0){
    if(strcmp((*obsprm).observer1,"")==0) {
      strncpy((*obsprm).observer1,value,8);
      (*obsprm).observer1[8]=(char)NULL;
    } else if(strcmp((*obsprm).observer2,"")==0) {
      strncpy((*obsprm).observer2,value,8);
      (*obsprm).observer2[8]=(char)NULL;
    } else if(strcmp((*obsprm).observer3,"")==0) {
      strncpy((*obsprm).observer3,value,8);
      (*obsprm).observer3[8]=(char)NULL;
    } else if(strcmp((*obsprm).observer4,"")==0) {
      strncpy((*obsprm).observer4,value,8);
      (*obsprm).observer4[8]=(char)NULL;
    }
  }

  return;
}

void fits2Object(obj,key,type,value)
OBJECT *obj;
char *key;
int type;
char *value;
{
  double equinox_year;

  if(strcmp(key,"OBJECT")==0) {
    strncpy((*obj).name,value,15);
    (*obj).name[15]=(char)NULL;
  }
/*
  (*obj).cood_sys=COORD_LB;
  (*obj).on_lb.x=0.0;
  (*obj).on_lb.y=0.0;
  (*obj).on_radec.x=0.0;
  (*obj).on_radec.y=0.0;
*/

  else if(strncmp(key,"EPOCH",5)==0 || strncmp(key,"EQUINOX",7)==0) {
    equinox_year=atof(value);
    if(equinox_year<1984.0)
      sprintf((*obj).equinox,"B%f6.1",equinox_year);
    else
      sprintf((*obj).equinox,"J%f6.1",equinox_year);
  }
/*
  (*obj).on_azel.x=0.0;
  (*obj).on_azel.y=0.0;
  (*obj).off_lb.x=0.0;
  (*obj).off_lb.y=0.0;
  (*obj).off_radec.x=0.0;
  (*obj).off_radec.y=0.0;
  (*obj).orig_xy.x=0.0;
  (*obj).orig_xy.y=0.0;
  (*obj).pa_xy=0.0;
  (*obj).on_xy.x=0.0;
  (*obj).on_xy.y=0.0;
  (*obj).azel_offset.x=0.0;
  (*obj).azel_offset.y=0.0;
*/

  return;
}

void fits2AntStatus(ant,key,type,value)
ANT_STATUS *ant;
char *key;
int type;
char *value;
{
  if(strcmp(key,"TELESCOP")==0) {
    strncpy((*ant).tele,value,8);
    (*ant).tele[8]=(char)NULL;
  }
/*
  (*ant).ap_eff=(float)0.0;
  (*ant).mb_eff=(float)0.0;
  (*ant).fss_eff=(float)0.0;
  (*ant).hpbw=(float)0.0;
*/

  return;
}

void fits2RxStatus(rx,key,type,value)
RX_STATUS *rx;
char *key;
int type;
char *value;
{
  if(strcmp(key,"INSTRUME")==0 && strcmp((*rx).name,"")==0) {
    strncpy((*rx).name,value,7);
    (*rx).name[7]=(char)NULL;
  }
/*
  (*rx).sb_mode=USB_SSB;
  (*rx).v_lsr=0.0;
  (*rx).v_off=0.0;
  (*rx).rest_freq=0.0;
  (*rx).obs_freq=0.0;
  (*rx).lo_freq1=0.0;
  (*rx).if_freq1=0.0;
  (*rx).lo_freq2=0.0;
  (*rx).if_freq2=0.0;
  (*rx).lo_freq3=0.0;
  (*rx).if_freq3=0.0;
  (*rx).mltplx1=(unsigned short)0;
  (*rx).mltplx2=(unsigned short)0;
  (*rx).mltplx3=(unsigned short)0;
  (*rx).tsys=(float)0.0;
  (*rx).rms=(float)0.0;
  (*rx).calib=(float)0.0;
  (*rx).weight=(float)1.0;
  (*rx).scale=(float)1.0;
*/

  return;
}

void fits2BeStatus(be,key,type,value)
BE_STATUS *be;
char *key;
int type;
char *value;
{
  if(strcmp(key,"INSTRUME")==0) {
    strncpy((*be).name,value,8);
    (*be).name[8]=(char)NULL;
  }
/*
  (*be).be_ch=(unsigned short)1;
  (*be).basis_ch=(unsigned short)0;
  (*be).sign=(char)1;
  (*be).data_ch=(unsigned short)1;
  (*be).center_ch=0.0;
  (*be).center_freq=0.0;
  (*be).disp0=(*be).center_freq;
  (*be).disp1=1.0;
  (*be).disp2=0.0;
  (*be).disp3=0.0;
  (*be).disp4=0.0;
  (*be).freq_res=0.0;
  (*be).ch_width=(float)1.0;
  (*be).ch_orig=(float)1.0;
*/

  return;
}

void fits2ObsLog(log,key,type,value)
OBS_LOG *log;
char *key;
int type;
char *value;
{
  if(strcmp(key,"ORIGIN")==0) {
    strncpy((*log).site_name,value,8);
    (*log).site_name[8]=(char)NULL;
  }
/*
  (*log).meteo.tmp=(float)0.0;
  (*log).meteo.water=(float)0.0;
  (*log).meteo.press=(float)0.0;
  (*log).tau0=(float)0.0;
  strcpy((*log).memo,"");
  strcpy((*log).comment,"");
*/
  if(strcmp(key,"COMMENT")==0) {
    strncpy((*log).comment,value,64);
    (*log).comment[64]=(char)NULL;
  }

  return;
}

void fits2Axes(axis,key,type,value,fits_axis)
AXES *axis;
char *key;
int type;
char *value;
FITS_AXIS *fits_axis;
{
  int fits_size[1000];
  int i_axis;
  char key_type[9];

  strncpy(key_type,key,8);
  key_type[8]=(char)NULL;

/* axis paramters */
  if(strncmp(key_type,"NAXIS",5)==0 || strncmp(key_type,"CTYPE",5)==0
  || strncmp(key_type,"CRPIX",5)==0 || strncmp(key_type,"CRVAL",5)==0
  || strncmp(key_type,"CDELT",5)==0 || strncmp(key_type,"CROTA",5)==0
  || strncmp(key_type,"CUNIT",5)==0) {
    i_axis=atoi(&(key_type[5]))-1;
    fits_size[i_axis]=atoi(value);
    key_type[5]=(char)NULL;
  } else {
    return;
  }

/* axis size */
  if(strcmp(key,"NAXIS")==0) {
    (*axis).dimension=atoi(value);
    return;
  }
  else if(strcmp(key_type,"NAXIS")==0)
    fits_axis[i_axis].naxis=atoi(value);
  else if(strcmp(key_type,"CTYPE")==0) {
    strncpy(fits_axis[i_axis].ctype,value,MAX_LWORD);
    fits_axis[i_axis].ctype[MAX_LWORD-1]=(char)NULL;
  }
  else if(strcmp(key_type,"CRPIX")==0)
    fits_axis[i_axis].crpix=atof(value);
  else if(strcmp(key_type,"CRVAL")==0)
    fits_axis[i_axis].crval=atof(value);
  else if(strcmp(key_type,"CDELT")==0)
    fits_axis[i_axis].cdelt=atof(value);
  else if(strcmp(key_type,"CROTA")==0)
    fits_axis[i_axis].crota=atof(value);
  else if(strcmp(key_type,"CUNIT")==0) {
    strncpy(fits_axis[i_axis].cunit,value,MAX_LWORD);
    fits_axis[i_axis].cunit[MAX_LWORD-1]=(char)NULL;
  }

  return;
}

void fits2Datavalue(data,key,type,value)
DATA_VALUE *data;
char *key;
int type;
char *value;
{
/*
  if(strcmp(key,"BUNIT")==0) {
    strncpy((*data).type,value,8);
    (*data).type[8]=(char)NULL;
  }
*/
  if(strcmp(key,"BUNIT")==0) {
    strncpy((*data).unit,value,8);
    (*data).unit[8]=(char)NULL;
    if(strcmp((*data).unit,"KELVIN")==0)
    strcpy((*data).unit,"K");
  } else if(strcmp(key,"BSCALE")==0) {
    (*data).scale=atof(value);
  } else if(strcmp(key,"BZERO")==0) {
    (*data).offset=atof(value);
  } else if(strcmp(key,"BLANK")==0) {
    (*data).dummy=atof(value);
  } else if(strcmp(key,"DATAMAX")==0) {
    (*data).max=atof(value);
  } else if(strcmp(key,"DATAMIN")==0) {
    (*data).min=atof(value);
  }
/*
  (*data).mean=(*data).dummy;
  (*data).rms=0.0;
  (*data).noise=0.0;
*/

  return;
}

void fits2cubehead(head,key,type,value,fits_axis)
CUBE_HEAD *head;
char *key;
int type;
char *value;
FITS_AXIS *fits_axis;
{
  fits2Syshead(&((*head).sys),key,type,value);
  fits2Obsprm(&((*head).obs),key,type,value);
  fits2Object(&((*head).object),key,type,value);
  fits2AntStatus(&((*head).ant),key,type,value);
  fits2RxStatus(&((*head).rx),key,type,value);
  fits2BeStatus(&((*head).be),key,type,value);
  fits2ObsLog(&((*head).log),key,type,value);
  fits2Axes(&((*head).axis),key,type,value,fits_axis);
  fits2Datavalue(&((*head).data_value),key,type,value);

  return;
}

void adjustObsprm(head)
CUBE_HEAD *head;
{
/*
  strcpy((*obsprm).project,"from FITS");
  strcpy((*obsprm).scd_file,"");
  (*obsprm).line_numb=(unsigned short)0;
  (*obsprm).sw_mode=ONOFF_SW;
  (*obsprm).on_time=(unsigned short)0;
  (*obsprm).off_time=(unsigned short)0;
*/

  return;
}

void adjustObject(head)
CUBE_HEAD *head;
{
  int ii;
  AXIS *axis;

  for(ii=0;ii<(*head).axis.dimension;ii++) {
    switch(ii) {
    case 0:
      axis= &((*head).axis.x);
      break;
    case 1:
      axis= &((*head).axis.y);
      break;
    case 2: default:
      axis= &((*head).axis.z);
      break;
    }

    if(strcmp((*axis).name,"l")==0) {
      (*head).object.cood_sys=COORD_LB;
      (*head).object.on_lb.x=(*axis).origin;
      (*head).object.orig_xy.x=(*axis).origin;
    } else if(strcmp((*axis).name,"b")==0) {
      (*head).object.cood_sys=COORD_LB;
      (*head).object.on_lb.y=(*axis).origin;
      (*head).object.orig_xy.y=(*axis).origin;
    } else if(strcmp((*axis).name,"RA")==0) {
      (*head).object.cood_sys=COORD_RADEC;
      (*head).object.on_radec.x=(*axis).origin;
      (*head).object.orig_xy.x=(*axis).origin;
    } else if(strcmp((*axis).name,"Dec")==0) {
      (*head).object.cood_sys=COORD_RADEC;
      (*head).object.on_radec.y=(*axis).origin;
      (*head).object.orig_xy.y=(*axis).origin;
    }
  }
/*
  (*obj).on_azel.x=0.0;
  (*obj).on_azel.y=0.0;
  (*obj).off_lb.x=0.0;
  (*obj).off_lb.y=0.0;
  (*obj).off_radec.x=0.0;
  (*obj).off_radec.y=0.0;
  (*obj).pa_xy=0.0;
  (*obj).on_xy.x=0.0;
  (*obj).on_xy.y=0.0;
  (*obj).azel_offset.x=0.0;
  (*obj).azel_offset.y=0.0;
*/

  return;
}

void adjustAntStatus(head)
CUBE_HEAD *head;
{
/*
  (*ant).ap_eff=(float)0.0;
  (*ant).mb_eff=(float)0.0;
  (*ant).fss_eff=(float)0.0;
  (*ant).hpbw=(float)0.0;
*/

  return;
}

void adjustRxStatus(head)
CUBE_HEAD *head;
{
  int ii;
  AXIS *axis;

  for(ii=0;ii<(*head).axis.dimension;ii++) {
    switch(ii) {
    case 0:
      axis= &((*head).axis.x);
      break;
    case 1:
      axis= &((*head).axis.y);
      break;
    case 2: default:
      axis= &((*head).axis.z);
      break;
    }

    if(strcmp((*axis).type,"velocity")==0) {
      (*head).rx.v_lsr=(*axis).origin+(*axis).delta*((double)((*axis).size+1)/2.0-1.5);
    }
    /* v_lsr is set at the ch (size+1)/2-0.5 (start ch=1) */
  }
  if((*head).rx.rest_freq==0.0) (*head).rx.rest_freq=128.0;
  if((*head).rx.obs_freq==0.0) (*head).rx.obs_freq=(*head).rx.rest_freq;
  if((*head).rx.v_off==0.0) (*head).rx.v_off=(*head).rx.v_lsr;
  (*head).rx.scale=(*head).data_value.scale;
/*
  (*rx).sb_mode=USB_SSB;
  (*rx).lo_freq1=0.0;
  (*rx).if_freq1=0.0;
  (*rx).lo_freq2=0.0;
  (*rx).if_freq2=0.0;
  (*rx).lo_freq3=0.0;
  (*rx).if_freq3=0.0;
  (*rx).mltplx1=(unsigned short)0;
  (*rx).mltplx2=(unsigned short)0;
  (*rx).mltplx3=(unsigned short)0;
  (*rx).tsys=(float)0.0;
  (*rx).rms=(float)0.0;
  (*rx).calib=(float)0.0;
  (*rx).weight=(float)1.0;
*/

  return;
}

void adjustBeStatus(head)
CUBE_HEAD *head;
{
  int ii;
  AXIS *axis;
  double freq_span;

  RANGE v_range;
  double obs_freq,center_freq;

  for(ii=0;ii<(*head).axis.dimension;ii++) {
    switch(ii) {
    case 0:
      axis= &((*head).axis.x);
      break;
    case 1:
      axis= &((*head).axis.y);
      break;
    case 2: default:
      axis= &((*head).axis.z);
      break;
    }

    if(strcmp((*axis).type,"velocity")==0) {
      (*head).be.be_ch=(*axis).size;
      (*head).be.data_ch=(*axis).size;
      (*head).be.center_ch=((double)(*axis).size)/2.0;
      (*head).be.basis_ch=(short)(*head).be.center_ch;
      obs_freq=(*head).rx.obs_freq;
      v_range.min=(*axis).origin-(*axis).pixel_origin*(*axis).delta;
      v_range.max=(*axis).origin+((double)((*axis).size-1)-(*axis).pixel_origin)*(*axis).delta;
      freq_span=(v_range.min-v_range.max)/(LIGHT_SPEED)*obs_freq*1.0e3;
      if(freq_span<0.0) {
	freq_span= -freq_span;
	(*head).be.sign= (char)-1;
      } else {
	(*head).be.sign= (char)1;
      }
      if((*head).be.center_freq==0.0) {
	(*head).be.center_freq=freq_span; /* from 0.5*f_span to 1.5*f_span */
      }
      center_freq=(*head).be.center_freq;
      (*head).be.disp1=freq_span/(double)((*axis).size-1);
      (*head).be.disp0=(*head).be.center_freq;
    }
  }
  if((*head).be.disp0==0.0)
    (*head).be.disp0=(*head).be.center_freq;

 /* default resolution is doulbe channel interval */
  if((*head).be.freq_res==0.0) {
    (*head).be.freq_res=fabs((*head).be.disp1*2.0);
  }

  return;
}

void adjustObsLog(head)
CUBE_HEAD *head;
{
/*
  (*log).meteo.tmp=(float)0.0;
  (*log).meteo.water=(float)0.0;
  (*log).meteo.press=(float)0.0;
  (*log).tau0=(float)0.0;
  strcpy((*log).memo,"");
  strcpy((*log).comment,"");
*/

  return;
}

void name2ntu(x,equinox)
AXIS *x;
char *equinox;
/*
equinox and velocity unit conversion

1999/8/3 T.Handa
*/
{
/* FITS standard names */
  if(strcmp((*x).type,"RA")==0 || strcmp((*x).type,"Dec")==0) {
    if(strcmp(equinox,"")==0) 
      strcpy((*x).refer,"B1950.0");
    else
      strcpy((*x).refer,equinox);
  }

/* velocity unit conversion m/s-->km/s */
  if(strcmp((*x).unit,"m/s")==0) {
    strcpy((*x).unit,"km/s");
    (*x).origin=(*x).origin/1.0e3;
    (*x).delta=(*x).delta/1.0e3;
    (*x).resolution=(*x).resolution/1.0e3;
    (*x).res_orig=(*x).res_orig/1.0e3;
  }

  return;
}

void adjustAxes(head,fits_axis,debug_flag)
CUBE_HEAD *head;
FITS_AXIS *fits_axis;
int *debug_flag; /* <in> [0]=default v unit, [1]=reduce naxes */
/*
convert global fits_axis to cube_head

1999/8/3 T.Handa
 */
{
  int n_axis,max_axis;
  int ii;
  int min_size;

  int fits_xyz[3];

/* estimate actual naxis */
  max_axis=(*head).axis.dimension;

  if(debug_flag[1]==FLAG_YES) {
    min_size=2;
  } else {
    min_size=1;
  }
  n_axis=0;
  for(ii=0;ii<max_axis;ii++) {
    if(fits_axis[ii].naxis>=min_size) {
      switch(n_axis) {
      case 0:
	StarHeadAxisFromFitsHead(&((*head).axis.x),&(fits_axis[ii]));
	fits_xyz[0]=ii;
	break;
      case 1:
	StarHeadAxisFromFitsHead(&((*head).axis.y),&(fits_axis[ii]));
	fits_xyz[1]=ii;
	break;
      case 2:
	StarHeadAxisFromFitsHead(&((*head).axis.z),&(fits_axis[ii]));
	fits_xyz[2]=ii;
	break;
      }
      n_axis++;
    }
  }

/* force to make 3D data */
  if(debug_flag[1]==FLAG_NO) {
    switch(max_axis) {
    case 1:
      (*head).axis.y.size=1;
      (*head).axis.y.delta=0.0;
      (*head).axis.y.origin=0.0;
      (*head).axis.y.pixel_origin=0.0;
    case 2:
      (*head).axis.z.size=1;
      (*head).axis.z.delta=0.0;
      (*head).axis.z.origin=0.0;
      (*head).axis.z.pixel_origin=0.0;
    }
    fprintf(stderr,"* WRN * force to make 3D data from %dD\n",(*head).axis.dimension);
    n_axis=3;
    (*head).axis.dimension=n_axis;
  }

/* the number of axes */
  if((*head).axis.dimension!=n_axis) {
    fprintf(stderr,"* WRN * omit single component axes in the FITS. Original=%d revised=%d\n",(*head).axis.dimension,n_axis);
    (*head).axis.dimension=n_axis;
  }
  if(n_axis>3)
    fprintf(stderr,"**ERR** too many axes in the FITS\n");

/* divide standard FITS axis name to name, type and unit */
  name2ntu(&((*head).axis.x),(*head).object.equinox);
  name2ntu(&((*head).axis.y),(*head).object.equinox);
  name2ntu(&((*head).axis.z),(*head).object.equinox);

/* debug for default velocity unit */
  if(debug_flag[0]==FLAG_YES) {
    force_to_unit_kms(&((*head).axis.x));
    force_to_unit_kms(&((*head).axis.y));
    force_to_unit_kms(&((*head).axis.z));
  }

  return;
}

void adjustDatavalue(head)
CUBE_HEAD *head;
{
/*
  if(strcmp(key,"BUNIT")==0) {
    strncpy((*data).type,value,8);
    (*data).type[8]=(char)NULL;
  }
*/

  return;
}

void adjustSyshead(head)
CUBE_HEAD *head;
{
  long nx,ny,nz;
  long total_pixel;

  nx=(*head).axis.x.size;
  ny=(*head).axis.y.size;
  nz=(*head).axis.z.size;

  total_pixel=nx*ny*nz;

  (*head).sys.l_file=(*head).sys.l_file+total_pixel*sizeof(float);

  return;
}

void adjust_cubehead(head,fits_axis,debug_flag)
CUBE_HEAD *head;
FITS_AXIS *fits_axis;
int *debug_flag; /* <in> */
{
  adjustAxes(head,fits_axis,debug_flag);
  adjustObsprm(head);
  adjustObject(head);
  adjustAntStatus(head);
  adjustRxStatus(head);
  adjustBeStatus(head);
  adjustObsLog(head);
  adjustDatavalue(head);
  adjustSyshead(head); /* must be the last of in this function */

  return;
}


void StarReduceBlankInObsPrm(obs)
OBS_PRM *obs;
/*
reduce top/bottom blanks in c-strings

2002/6/23 T.Handa
 */
{
  StarReduceBlankInCstrings((*obs).project);
  StarReduceBlankInCstrings((*obs).scd_file);
  StarReduceBlankInCstrings((*obs).observer1);
  StarReduceBlankInCstrings((*obs).observer2);
  StarReduceBlankInCstrings((*obs).observer3);
  StarReduceBlankInCstrings((*obs).observer4);

  return;
}

void StarReduceBlankInObject(obj)
OBJECT *obj;
/*
reduce top/bottom blanks in c-strings

2002/6/23 T.Handa
 */
{
  StarReduceBlankInCstrings((*obj).name);
  StarReduceBlankInCstrings((*obj).equinox);

  return;
}

void StarReduceBlankInAnt(ant)
ANT_STATUS *ant;
/*
reduce top/bottom blanks in c-strings

2002/6/23 T.Handa
 */
{
  StarReduceBlankInCstrings((*ant).tele);

  return;
}

void StarReduceBlankInRX(rx)
RX_STATUS *rx;
/*
reduce top/bottom blanks in c-strings

2002/6/23 T.Handa
 */
{
  StarReduceBlankInCstrings((*rx).name);

  return;
}

void StarReduceBlankInBE(be)
BE_STATUS *be;
/*
reduce top/bottom blanks in c-strings

2002/6/23 T.Handa
 */
{
  StarReduceBlankInCstrings((*be).name);

  return;
}

void StarReduceBlankInObsLog(log)
OBS_LOG *log;
/*
reduce top/bottom blanks in c-strings

2002/6/23 T.Handa
 */
{
  StarReduceBlankInCstrings((*log).site_name);
  StarReduceBlankInCstrings((*log).memo);
  StarReduceBlankInCstrings((*log).comment);

  return;
}

void StarReduceBlankAxis(axis)
AXIS *axis;
/*
reduce top/bottom blanks in c-strings

2002/6/23 T.Handa
 */
{
  StarReduceBlankInCstrings((*axis).name);
  StarReduceBlankInCstrings((*axis).type);
  StarReduceBlankInCstrings((*axis).unit);
  StarReduceBlankInCstrings((*axis).project);
  StarReduceBlankInCstrings((*axis).refer);
  StarReduceBlankInCstrings((*axis).psf);

  return;
}

void StarReduceBlankInAxes(axes)
AXES *axes;
/*
reduce top/bottom blanks in c-strings

2002/6/23 T.Handa
 */
{
  StarReduceBlankAxis(&((*axes).x));
  StarReduceBlankAxis(&((*axes).y));
  StarReduceBlankAxis(&((*axes).z));
  StarReduceBlankInCstrings((*axes).order);

  return;
}

void StarReduceBlankInDataValue(data)
DATA_VALUE *data;
/*
reduce top/bottom blanks in c-strings

2002/6/23 T.Handa
 */
{
  StarReduceBlankInCstrings((*data).type);
  StarReduceBlankInCstrings((*data).unit);

  return;
}


void StarReduceBlankInCubehead(head)
CUBE_HEAD *head; /* <i/o> */
/*
reduce top/bottom blanks in c-strings

2002/6/23 T.Handa
 */
{
  StarReduceBlankInObsPrm(&((*head).obs));
  StarReduceBlankInObject(&((*head).object));
  StarReduceBlankInAnt(&((*head).ant));
  StarReduceBlankInRX(&((*head).rx));
  StarReduceBlankInBE(&((*head).be));
  StarReduceBlankInObsLog(&((*head).log));
  StarReduceBlankInAxes(&((*head).axis));
  StarReduceBlankInDataValue(&((*head).data_value));

  return;
}


int StarFitsHead2CubeHead(fp,head,bitpix,debug_flag,fp_fitshead)
FILE *fp; /* <in> */
CUBE_HEAD *head; /* <out> */
int *bitpix; /* <out> */
int *debug_flag; /* <in> */
FILE *fp_fitshead; /* <in> */
{
  FITS_AXIS fits_axis[MAX_NAXIS];

  static char block[36][80];
  char card[81];
  char key[9],value[81];

  int ii,jj;
  int key_type;
  int err_code;
  HISTORY history;

  set_default_cubehead(head);
  *bitpix=0;
  while(1) {
    err_code=StarReadFitsBlock(fp,block);
    if(err_code!=0) { /* cannot read a block */
      return(9); /* not end by END card */
    }
    for(ii=0;ii<36;ii++) {
      for(jj=0;jj<79;jj++) {
	card[jj]=block[ii][jj];
      }
      card[80]=(char)NULL;
      StarCutFitsComment(card);
      key_type=StarDivideFitsCard(card,key,value);

      if(fp_fitshead!=NULL) fprintf(fp_fitshead,"%s\n",card);

      if(key_type==FITS_VALUE_END) { /* END card */
	goto loop_out;
      }
      fits2cubehead(head,key,key_type,value,fits_axis);
      if(strcmp(key,"BITPIX")==0) *bitpix=atoi(value);
    }
  }

loop_out:;

  adjust_cubehead(head,fits_axis,debug_flag);

  StarReduceBlankInCubehead(head);

  strcpy(history.step,"fit2cub");
  history.size=0;

  StarAddHistory4Cube(head,&history);

  return(0);
}

int StarSearchVariousFileExt4FITS(full_filename,filename)
char *full_filename; /* <out> */
char *filename; /* <in> */
{
  int l_str;
  static char std_ext[4][6]={".fit",".FIT",".fits",".FITS"};
  int ii;
  int err_code;

  l_str=strlen(filename);
  if(l_str>4) {
    if(strcasecmp(&(filename[l_str-4]),".fit")==0 || strcasecmp(&(filename[l_str-4]),".fts")==0 || strcasecmp(&(filename[l_str-5]),".fits")==0) {
      err_code=StarStdFileExtName("STAR_DATA",filename,full_filename,"");
      if(err_code!=0) {
	fprintf(stderr,"Too long file name\n");
	return(-1);
      }
    } else {
      err_code=StarStdFileExtName("STAR_DATA",filename,full_filename,".fit");
      if(err_code!=0) {
	fprintf(stderr,"Too long file name\n");
	return(-1);
      }
    }
  }

  if(StarDoesFileExist(full_filename)==FileExist) {
    return(0);
  } else {
    for(ii=0;ii<4;ii++) {
      err_code=StarStdFileExtName("STAR_DATA",filename,full_filename,std_ext[ii]);
      if(err_code!=0) {
	fprintf(stderr,"Too long file name\n");
	return(-1);
      }
      if(StarDoesFileExist(full_filename)==FileExist) return(0);
    }
    return(-2);
  }
}

int options(argc,argv,input_filename,output_filename,fitshead_name,debug_flag)
/*
get parameters from options
2002/5/7 T.Handa
*/
int argc;
char *argv[];
char *input_filename;
char *output_filename;
char *fitshead_name;
int *debug_flag; /* <in> [0]=default v unit, [1]=reduce naxes */
{
  int	ii;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

  int	flag;

/* default values */
  input_filename[0]=(char)NULL;
  strcpy(fitshead_name,"fitshead.txt");
  for(ii=0;ii<MAX_DEBUG_FLAG;ii++)
    debug_flag[ii]=FLAG_NO;

/* set keywords */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"cub"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"output_file"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"out_cub"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"fit"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"input_file"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"in_fits"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"fitsheader"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"unit_km/s"); key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"unit_m/s"); key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"naxes"); key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* name for input cube */
  scan_flag[0]=0;
  scan_flag[1]=1;
  scan_flag[2]=2;
  n_scan=3;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(output_filename,got_value);
  }

/* name for input cube */
  scan_flag[0]=3;
  scan_flag[1]=4;
  scan_flag[2]=5;
  n_scan=3;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(input_filename,got_value);
  }

  got_value=StarKeylessOption(argc,argv,n_key,key_flag,key_type);
  if(got_value[0]!=(char)NULL) {
    strcpy(input_filename,got_value);
  }

/* name for FITS header storage */
  scan_flag[0]=6;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(fitshead_name,got_value);
  }

/* force to km/s unit */
  scan_flag[0]= 7;
  scan_flag[1]= -8;
  n_scan=2;
  flag=StarOptionFlag(n_scan,scan_flag,key_flag);
  if(flag!=0) debug_flag[0]=FLAG_YES;

/* debug flag: reduce the number of axes of which size is 1 */
  scan_flag[0]=9;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strncmp(got_value,"reduce",6)==0) {
      debug_flag[1]=FLAG_YES;
    } else {
      debug_flag[1]=FLAG_NO;
    }
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

int main(argc,argv)
int argc;
char *argv[];
{
  FILE *in_fp;
  FILE *fp_fitshead;

  CUBE cube;

  int err_code;

  char output_cubename[MAX_FILE_NAME_LENGTH];
  char input_fitsname[MAX_FILE_NAME_LENGTH];
  char full_filename[MAX_FILE_NAME_LENGTH];
  char fitshead_name[MAX_FILE_NAME_LENGTH];

  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];

  int n_file;
  int bitpix;

  int debug_flag[MAX_DEBUG_FLAG]; /* [0]=default v unit, [1]=reduce naxis */

  n_file=1;
  StarPipeHeadOut(n_file);

/* get option */
  err_code=options(argc,argv,input_fitsname,output_cubename,fitshead_name,debug_flag);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

  if(strcmp(output_cubename,"")==0) strcpy(output_cubename,input_fitsname);

/* add STD extension for cube */
  err_code=StarSearchVariousFileExt4FITS(full_filename,input_fitsname);
  if(err_code<0) {
    fprintf(stderr,"%s: FITS file \"%s\" is not found.\n",argv[0],input_fitsname);
    exit(900);
  }

/* load FITS header into cube.head */
  in_fp=fopen(full_filename,"r");
  if(in_fp==NULL) {
    fprintf(stderr,"%s: cannot open FITS file \"%s\"\n",argv[0],full_filename);
    exit(900);
  }

/* open file for fits header to check */
  if(strcmp(fitshead_name,"")==0) {
    fp_fitshead=NULL;
  } else if(strcmp(fitshead_name,"-")==0) {
    fp_fitshead=stderr;
  } else {
    fp_fitshead=fopen(fitshead_name,"w");
    if(fp_fitshead==NULL) {
      fp_fitshead=stderr;
    }
  }

/* read and convert FITS header */
  err_code=StarFitsHead2CubeHead(in_fp,&(cube.head),&bitpix,debug_flag,fp_fitshead);

  strncpy(cube.head.obs.project,input_fitsname,16);
  cube.head.obs.project[16]=(char)NULL;
  strncpy(cube.head.obs.scd_file,input_fitsname,8);
  cube.head.obs.scd_file[8]=(char)NULL;

  cube.pixel=StarAllocCubePixel(&(cube.head));
  if(cube.pixel==NULL) {
    fprintf(stderr,"%s: cannot alloc memory for pixel.\n",argv[0]);
    exit(999);
  }
  err_code=StarFitsPixel2CubePixel(in_fp,&cube,bitpix);
  fclose(in_fp);

  strcpy(cube.head.file_id,STAR_CUB_IN);
  strcpy(cube.head.file_name,output_cubename);

  cdb_flag=GOOD_SCAN;
  strcpy(cdb_arg,"");

  if(cube.head.data_value.max==cube.head.data_value.min && cube.head.data_value.rms==(float)0.0) {
    StarCubeScanDatavalue(&cube);
  }

/* pipeout cube data */
  err_code=StarCubeOut(&cube,cdb_flag,cdb_arg);

  StarFreeCubePixel(&cube);

/* close the FITS file */
  if(fp_fitshead!=stderr && fp_fitshead!=stdout && fp_fitshead!=NULL) 
    fclose(fp_fitshead);

  StarPipeEndOut();

  exit(0);
}
