/*
3D FITS file to cub flow

1999/8/17 T.Handa : ver.3.01
1999/9/25 T.Handa : for CfA, wrong endian data, run only on Linux
2006/5/22 T.Handa : ver.3.03 (debug) core dump via ssh; 2D array should be static
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StStdio.h>
#include <StStdlib.h>
#include <StPipe.h>
#include <StIndex.h>
#include <StData.h>
#include <StCubeIO.h>
#include <StCubeProc.h>
#include <StFitsIO.h>

#define DEFAULT_PRMFILE "fits2cub.prm"
#define N_KEY 30

int WrongEndian_StarFitsPixel2CubePixel(fp,cube,bitpix)
FILE *fp; /* <in> */
CUBE *cube; /* <i/o> cube.head=in, cube.pixel=out */
int bitpix; /* <in> FITS bitpix */
/*
Read pixel value in FITS to cube.pixel

1999/8/3 T.Handa
1999/9/25 T.Handa : for CfA, wrong endian data, run only on Linux
*/
{
  int nx,ny,nz;
  int ix,iy,iz;
  double data_double;
  float data_float,data;
  long data_long;
  short data_short;
  unsigned char data_u_char;
  double scale,offset;

  nx=(*cube).head.axis.x.size;
  ny=(*cube).head.axis.y.size;
  nz=(*cube).head.axis.z.size;

  scale=(*cube).head.data_value.scale;
  offset=(*cube).head.data_value.offset;

  switch(bitpix) {
  case -64: /* double */
    for(iz=0;iz<nz;iz++){
      for(iy=0;iy<ny;iy++) {
	for(ix=0;ix<nx;ix++) {
/* 	  freadIntel(&data_double,sizeof(data_double),1,fp); */
 	  fread(&data_double,sizeof(data_double),1,fp);
	  data=(float)data_double;
	  StarPutCubePixel(cube,ix,iy,iz,data);
	}
      }
    }
    break;
  case -32: /* float */
    for(iz=0;iz<nz;iz++){
      for(iy=0;iy<ny;iy++) {
	for(ix=0;ix<nx;ix++) {
/*	  freadIntel(&data_float,sizeof(data_float),1,fp); */
	  fread(&data_float,sizeof(data_float),1,fp);
	  data=(float)data_float;
	  StarPutCubePixel(cube,ix,iy,iz,data);
	}
      }
    }
    break;
  case 32: /* long */
    for(iz=0;iz<nz;iz++){
      for(iy=0;iy<ny;iy++) {
	for(ix=0;ix<nx;ix++) {
/*	  freadIntel(&data_long,sizeof(data_long),1,fp); */
	  fread(&data_long,sizeof(data_long),1,fp);
	  data_double=(double)data_long*scale+offset;
	  data=(float)data_double;
	  StarPutCubePixel(cube,ix,iy,iz,data);
	}
      }
    }
    break;
  case 16: /* short */
    for(iz=0;iz<nz;iz++){
      for(iy=0;iy<ny;iy++) {
	for(ix=0;ix<nx;ix++) {
/*	  freadIntel(&data_short,sizeof(data_short),1,fp); */
	  fread(&data_short,sizeof(data_short),1,fp);
	  data_double=(double)data_short*scale+offset;
	  data=(float)data_double;
	  StarPutCubePixel(cube,ix,iy,iz,data);
	}
      }
    }
    break;
  case 8: /* unsigned char */
    for(iz=0;iz<nz;iz++){
      for(iy=0;iy<ny;iy++) {
	for(ix=0;ix<nx;ix++) {
/*	  freadIntel(&data_u_char,sizeof(data_u_char),1,fp); */
	  fread(&data_u_char,sizeof(data_u_char),1,fp);
	  data_double=(double)data_u_char*scale+offset;
	  data=(float)data_double;
	  StarPutCubePixel(cube,ix,iy,iz,data);
	}
      }
    }
    break;
  }

  if(bitpix>0) {
    (*cube).head.data_value.scale=1.0;
    (*cube).head.data_value.offset=0.0;
  }

  return(0);
}

#ifdef _NO_PROTO

extern int StarFitsHead2CubeHead();
#else

extern int StarFitsHead2CubeHead(FILE *, CUBE_HEAD *, int *);
#endif

void StarDefaultSyshead(sys_head)
SYS_HEAD *sys_head;
/*
copy from mkcube.c
1999/4/5 T.Handa
*/
{
  (*sys_head).l_file=CUBE_INITIAL_L_HEAD; /* file length */
  (*sys_head).l_head=(*sys_head).l_file; /* header part length */
  (*sys_head).format_id=XYZ_CUBE;
  (*sys_head).version=CUBE_FORMAT_VERSION;
  (*sys_head).data_type=TA_DATA;
  (*sys_head).hist_step=0;
  (*sys_head).hist_line=0;
  (*sys_head).scan=0;
  (*sys_head).backup=0;
  (*sys_head).num_scan=0;

  return;
}

void StarDefaultObsprm(obsprm)
OBS_PRM *obsprm;
{
  strcpy((*obsprm).project,"");
  strcpy((*obsprm).scd_file,"");
  (*obsprm).line_numb=(unsigned short)0;
  (*obsprm).sw_mode=ONOFF_SW;
  (*obsprm).mjd=0L;
  (*obsprm).start=0L;
  (*obsprm).stop=0L;
  (*obsprm).on_time=(unsigned short)0;
  (*obsprm).off_time=(unsigned short)0;
  strcpy((*obsprm).observer1,"");
  strcpy((*obsprm).observer2,"");
  strcpy((*obsprm).observer3,"");
  strcpy((*obsprm).observer4,"");

  return;
}

void StarDefaultObject(obj)
OBJECT *obj;
{
  strcpy((*obj).name,"");
  (*obj).cood_sys=COORD_LB;
  (*obj).on_lb.x=0.0;
  (*obj).on_lb.y=0.0;
  (*obj).on_radec.x=0.0;
  (*obj).on_radec.y=0.0;
  strcpy((*obj).equinox,"B1950.0");
  (*obj).on_azel.x=0.0;
  (*obj).on_azel.y=0.0;
  (*obj).off_lb.x=0.0;
  (*obj).off_lb.y=0.0;
  (*obj).off_radec.x=0.0;
  (*obj).off_radec.y=0.0;
  (*obj).orig_xy.x=0.0;
  (*obj).orig_xy.y=0.0;
  (*obj).pa_xy=0.0;
  (*obj).on_xy.x=0.0;
  (*obj).on_xy.y=0.0;
  (*obj).azel_offset.x=0.0;
  (*obj).azel_offset.y=0.0;

  return;
}

void StarDefaultAntStatus(ant)
ANT_STATUS *ant;
{
  strcpy((*ant).tele,"");
  (*ant).ap_eff=(float)0.0;
  (*ant).mb_eff=(float)0.0;
  (*ant).fss_eff=(float)0.0;
  (*ant).hpbw=(float)0.0;

  return;
}

void StarDefaultRxStatus(rx)
RX_STATUS *rx;
{
  strcpy((*rx).name,"");
  (*rx).sb_mode=USB_SSB;
  (*rx).v_lsr=0.0;
  (*rx).v_off=0.0;
  (*rx).rest_freq=0.0;
  (*rx).obs_freq=0.0;
  (*rx).lo_freq1=0.0;
  (*rx).if_freq1=0.0;
  (*rx).lo_freq2=0.0;
  (*rx).if_freq2=0.0;
  (*rx).lo_freq3=0.0;
  (*rx).if_freq3=0.0;
  (*rx).mltplx1=(unsigned short)0;
  (*rx).mltplx2=(unsigned short)0;
  (*rx).mltplx3=(unsigned short)0;
  (*rx).tsys=(float)0.0;
  (*rx).rms=(float)0.0;
  (*rx).calib=(float)0.0;
  (*rx).weight=(float)1.0;
  (*rx).scale=(float)1.0;

  return;
}

void StarDefaultBeStatus(be)
BE_STATUS *be;
{
  strcpy((*be).name,"");
  (*be).be_ch=(unsigned short)1;
  (*be).basis_ch=(unsigned short)0;
  (*be).sign=(char)1;
  (*be).data_ch=(unsigned short)1;
  (*be).center_ch=0.0;
  (*be).center_freq=0.0;
  (*be).disp0=(*be).center_freq;
  (*be).disp1=1.0;
  (*be).disp2=0.0;
  (*be).disp3=0.0;
  (*be).disp4=0.0;
  (*be).freq_res=0.0;
  (*be).ch_width=(float)1.0;
  (*be).ch_orig=(float)1.0;

  return;
}

void StarDefaultObsLog(log)
OBS_LOG *log;
{
  strcpy((*log).site_name,"");
  (*log).meteo.tmp=(float)0.0;
  (*log).meteo.water=(float)0.0;
  (*log).meteo.press=(float)0.0;
  (*log).tau0=(float)0.0;
  strcpy((*log).memo,"");
  strcpy((*log).comment,"");

  return;
}

void StarDefaultAxes(axis)
AXES *axis;
{
  (*axis).dimension=3;
  strcpy((*axis).order,"xyz");

  strcpy((*axis).x.name,"x");
  (*axis).x.size=0;
  strcpy((*axis).x.type,"x");
  strcpy((*axis).x.unit,"pixel");
  strcpy((*axis).x.project,"linear");
  strcpy((*axis).x.refer,"");
  (*axis).x.delta=1.0;
  (*axis).x.origin=0.0;
  (*axis).x.pixel_origin=0.0;
  (*axis).x.rotation=0.0;
  (*axis).x.resolution=0.0;
  strcpy((*axis).x.psf,"gauss");

  strcpy((*axis).y.name,"y");
  (*axis).y.size=0;
  strcpy((*axis).y.type,"y");
  strcpy((*axis).y.unit,"pixel");
  strcpy((*axis).y.project,"linear");
  strcpy((*axis).y.refer,"");
  (*axis).y.delta=1.0;
  (*axis).y.origin=0.0;
  (*axis).y.pixel_origin=0.0;
  (*axis).y.rotation=0.0;
  (*axis).y.resolution=0.0;
  strcpy((*axis).y.psf,"gauss");
  (*axis).y.res_orig=0.0;

  strcpy((*axis).z.name,"z");
  (*axis).z.size=0;
  strcpy((*axis).z.type,"z");
  strcpy((*axis).z.unit,"pixel");
  strcpy((*axis).z.project,"linear");
  strcpy((*axis).z.refer,"");
  (*axis).z.delta=1.0;
  (*axis).z.origin=0.0;
  (*axis).z.pixel_origin=0.0;
  (*axis).z.rotation=0.0;
  (*axis).z.resolution=0.0;
  strcpy((*axis).z.psf,"gauss");
  (*axis).z.res_orig=0.0;

  return;
}

void StarDefaultDatavalue(data,dummy)
DATA_VALUE *data;
double dummy;
{
  strcpy((*data).type,"Ta*");
  strcpy((*data).unit,"K");
  (*data).scale=1.0;
  (*data).offset=0.0;
  (*data).dummy=(float)dummy;
  (*data).max=(*data).dummy;
  (*data).min=(*data).dummy;
  (*data).mean=(*data).dummy;
  (*data).rms=0.0;
  (*data).noise=0.0;

  return;
}

void set_default_cubehead(cube_head)
CUBE_HEAD *cube_head;
{
  StarDefaultSyshead(&((*cube_head).sys));
  StarDefaultObsprm(&((*cube_head).obs));
  StarDefaultObject(&((*cube_head).object));
  StarDefaultAntStatus(&((*cube_head).ant));
  StarDefaultRxStatus(&((*cube_head).rx));
  StarDefaultBeStatus(&((*cube_head).be));
  StarDefaultObsLog(&((*cube_head).log));
  StarDefaultAxes(&((*cube_head).axis));
  StarDefaultDatavalue(&((*cube_head).data_value),0.0);

  return;
}

void fits2Syshead(sys_head,key,type,value)
SYS_HEAD *sys_head;
char *key;
int type;
char *value;
{

  return;
}

void fits2Obsprm(obsprm,key,type,value)
OBS_PRM *obsprm;
char *key;
int type;
char *value;
{
/*
  strcpy((*obsprm).project,"from FITS");
  strcpy((*obsprm).scd_file,"");
  (*obsprm).line_numb=(unsigned short)0;
  (*obsprm).sw_mode=ONOFF_SW;
*/
  if(strcmp(key,"DATE")==0
  || ( strcmp(key,"DATE-OBS")==0 && (*obsprm).mjd==0) ) {
    (*obsprm).mjd=StarFitsDate2MJD(value);
  }
  if(strcmp(key,"DATE-OBS")==0) {
    (*obsprm).start=StarFitsDate2Time1970(value);
    if((*obsprm).stop==0L)
      (*obsprm).stop=(*obsprm).start;
  }
/*
  (*obsprm).on_time=(unsigned short)0;
  (*obsprm).off_time=(unsigned short)0;
*/
  if(strcmp(key,"OBSERVER")==0){
    if(((*obsprm).observer1,"")==0) {
      strncpy((*obsprm).observer1,value,8);
      (*obsprm).observer1[8]=NULL;
    } else if(strcmp((*obsprm).observer2,"")==0) {
      strncpy((*obsprm).observer2,value,8);
      (*obsprm).observer2[8]=NULL;
    } else if(strcmp((*obsprm).observer3,"")==0) {
      strncpy((*obsprm).observer3,value,8);
      (*obsprm).observer3[8]=NULL;
    } else if(strcmp((*obsprm).observer4,"")==0) {
      strncpy((*obsprm).observer4,value,8);
      (*obsprm).observer4[8]=NULL;
    }
  }

  return;
}

void fits2Object(obj,key,type,value)
OBJECT *obj;
char *key;
int type;
char *value;
{
  double equinox_year;

  if(strcmp(key,"OBJECT")==0) {
    strncpy((*obj).name,value,15);
    (*obj).name[15]=NULL;
  }
/*
  (*obj).cood_sys=COORD_LB;
  (*obj).on_lb.x=0.0;
  (*obj).on_lb.y=0.0;
  (*obj).on_radec.x=0.0;
  (*obj).on_radec.y=0.0;
*/

  else if(strncmp(key,"EPOCH",5)==0 || strncmp(key,"EQUINOX",7)==0) {
    equinox_year=atof(value);
    if(equinox_year<1984.0)
      sprintf((*obj).equinox,"B%f6.1",equinox_year);
    else
      sprintf((*obj).equinox,"J%f6.1",equinox_year);
  }
/*
  (*obj).on_azel.x=0.0;
  (*obj).on_azel.y=0.0;
  (*obj).off_lb.x=0.0;
  (*obj).off_lb.y=0.0;
  (*obj).off_radec.x=0.0;
  (*obj).off_radec.y=0.0;
  (*obj).orig_xy.x=0.0;
  (*obj).orig_xy.y=0.0;
  (*obj).pa_xy=0.0;
  (*obj).on_xy.x=0.0;
  (*obj).on_xy.y=0.0;
  (*obj).azel_offset.x=0.0;
  (*obj).azel_offset.y=0.0;
*/

  return;
}

void fits2AntStatus(ant,key,type,value)
ANT_STATUS *ant;
char *key;
int type;
char *value;
{
  if(strcmp(key,"TELESCOP")==0) {
    strncpy((*ant).tele,value,8);
    (*ant).tele[8]=NULL;
  }
/*
  (*ant).ap_eff=(float)0.0;
  (*ant).mb_eff=(float)0.0;
  (*ant).fss_eff=(float)0.0;
  (*ant).hpbw=(float)0.0;
*/

  return;
}

void fits2RxStatus(rx,key,type,value)
RX_STATUS *rx;
char *key;
int type;
char *value;
{
  if(strcmp(key,"INSTRUME")==0 && strcmp((*rx).name,"")==0) {
    strncpy((*rx).name,value,7);
    (*rx).name[7]=NULL;
  }
/*
  (*rx).sb_mode=USB_SSB;
  (*rx).v_lsr=0.0;
  (*rx).v_off=0.0;
  (*rx).rest_freq=0.0;
  (*rx).obs_freq=0.0;
  (*rx).lo_freq1=0.0;
  (*rx).if_freq1=0.0;
  (*rx).lo_freq2=0.0;
  (*rx).if_freq2=0.0;
  (*rx).lo_freq3=0.0;
  (*rx).if_freq3=0.0;
  (*rx).mltplx1=(unsigned short)0;
  (*rx).mltplx2=(unsigned short)0;
  (*rx).mltplx3=(unsigned short)0;
  (*rx).tsys=(float)0.0;
  (*rx).rms=(float)0.0;
  (*rx).calib=(float)0.0;
  (*rx).weight=(float)1.0;
  (*rx).scale=(float)1.0;
*/

  return;
}

void fits2BeStatus(be,key,type,value)
BE_STATUS *be;
char *key;
int type;
char *value;
{
  if(strcmp(key,"INSTRUME")==0) {
    strncpy((*be).name,value,8);
    (*be).name[8]=NULL;
  }
/*
  (*be).be_ch=(unsigned short)1;
  (*be).basis_ch=(unsigned short)0;
  (*be).sign=(char)1;
  (*be).data_ch=(unsigned short)1;
  (*be).center_ch=0.0;
  (*be).center_freq=0.0;
  (*be).disp0=(*be).center_freq;
  (*be).disp1=1.0;
  (*be).disp2=0.0;
  (*be).disp3=0.0;
  (*be).disp4=0.0;
  (*be).freq_res=0.0;
  (*be).ch_width=(float)1.0;
  (*be).ch_orig=(float)1.0;
*/

  return;
}

void fits2ObsLog(log,key,type,value)
OBS_LOG *log;
char *key;
int type;
char *value;
{
  if(strcmp(key,"ORIGIN")==0) {
    strncpy((*log).site_name,value,8);
    (*log).site_name[8]=NULL;
  }
/*
  (*log).meteo.tmp=(float)0.0;
  (*log).meteo.water=(float)0.0;
  (*log).meteo.press=(float)0.0;
  (*log).tau0=(float)0.0;
  strcpy((*log).memo,"");
  strcpy((*log).comment,"");
*/
  if(strcmp(key,"COMMENT")==0) {
    strncpy((*log).comment,value,64);
    (*log).comment[64]=NULL;
  }

  return;
}

void fits2Axes(axis,key,type,value,fits_axis)
AXES *axis;
char *key;
int type;
char *value;
FITS_AXIS *fits_axis;
{
  int fits_size[1000];
  int i_axis;
  char key_type[9];

  strncpy(key_type,key,8);
  key_type[8]=NULL;

/* axis paramters */
  if(strncmp(key_type,"NAXIS",5)==0 || strncmp(key_type,"CTYPE",5)==0
  || strncmp(key_type,"CRPIX",5)==0 || strncmp(key_type,"CRVAL",5)==0
  || strncmp(key_type,"CDELT",5)==0 || strncmp(key_type,"CROTA",5)==0
  || strncmp(key_type,"CUNIT",5)==0) {
    i_axis=atoi(&(key_type[5]))-1;
    fits_size[i_axis]=atoi(value);
    key_type[5]=NULL;
  } else {
    return;
  }

/* axis size */
  if(strcmp(key,"NAXIS")==0) {
    (*axis).dimension=atoi(value);
    return;
  }
  else if(strcmp(key_type,"NAXIS")==0)
    fits_axis[i_axis].naxis=atoi(value);
  else if(strcmp(key_type,"CTYPE")==0) {
    strncpy(fits_axis[i_axis].ctype,value,MAX_LWORD);
    fits_axis[i_axis].ctype[MAX_LWORD-1]=NULL;
  }
  else if(strcmp(key_type,"CRPIX")==0)
    fits_axis[i_axis].crpix=atof(value);
  else if(strcmp(key_type,"CRVAL")==0)
    fits_axis[i_axis].crval=atof(value);
  else if(strcmp(key_type,"CDELT")==0)
    fits_axis[i_axis].cdelt=atof(value);
  else if(strcmp(key_type,"CROTA")==0)
    fits_axis[i_axis].crota=atof(value);
  else if(strcmp(key_type,"CUNIT")==0) {
    strncpy(fits_axis[i_axis].cunit,value,MAX_LWORD);
    fits_axis[i_axis].cunit[MAX_LWORD-1]=NULL;
  }

  return;
}

void fits2Datavalue(data,key,type,value)
DATA_VALUE *data;
char *key;
int type;
char *value;
{
/*
  if(strcmp(key,"BUNIT")==0) {
    strncpy((*data).type,value,8);
    (*data).type[8]=NULL;
  }
*/
  if(strcmp(key,"BUNIT")==0) {
    strncpy((*data).unit,value,8);
    (*data).unit[8]=NULL;
    if(strcmp((*data).unit,"KELVIN")==0)
    strcpy((*data).unit,"K");
  } else if(strcmp(key,"BSCALE")==0) {
    (*data).scale=atof(value);
  } else if(strcmp(key,"BZERO")==0) {
    (*data).offset=atof(value);
  } else if(strcmp(key,"BLANK")==0) {
    (*data).dummy=atof(value);
  } else if(strcmp(key,"DATAMAX")==0) {
    (*data).max=atof(value);
  } else if(strcmp(key,"DATAMIN")==0) {
    (*data).min=atof(value);
  }
/*
  (*data).mean=(*data).dummy;
  (*data).rms=0.0;
  (*data).noise=0.0;
*/

  return;
}

void fits2cubehead(head,key,type,value,fits_axis)
CUBE_HEAD *head;
char *key;
int type;
char *value;
FITS_AXIS *fits_axis;
{
  fits2Syshead(&((*head).sys),key,type,value);
  fits2Obsprm(&((*head).obs),key,type,value);
  fits2Object(&((*head).object),key,type,value);
  fits2AntStatus(&((*head).ant),key,type,value);
  fits2RxStatus(&((*head).rx),key,type,value);
  fits2BeStatus(&((*head).be),key,type,value);
  fits2ObsLog(&((*head).log),key,type,value);
  fits2Axes(&((*head).axis),key,type,value,fits_axis);
  fits2Datavalue(&((*head).data_value),key,type,value);

  return;
}

void adjustObsprm(head)
CUBE_HEAD *head;
{
/*
  strcpy((*obsprm).project,"from FITS");
  strcpy((*obsprm).scd_file,"");
  (*obsprm).line_numb=(unsigned short)0;
  (*obsprm).sw_mode=ONOFF_SW;
  (*obsprm).on_time=(unsigned short)0;
  (*obsprm).off_time=(unsigned short)0;
*/

  return;
}

void adjustObject(head)
CUBE_HEAD *head;
{
  int ii;
  AXIS *axis;

  for(ii=0;ii<(*head).axis.dimension;ii++) {
    switch(ii) {
    case 0:
      axis= &((*head).axis.x);
      break;
    case 1:
      axis= &((*head).axis.y);
      break;
    case 2:
      axis= &((*head).axis.z);
      break;
    }

    if(strcmp((*axis).name,"l")==0) {
      (*head).object.cood_sys=COORD_LB;
      (*head).object.on_lb.x=(*axis).origin;
      (*head).object.orig_xy.x=(*axis).origin;
    } else if(strcmp((*axis).name,"b")==0) {
      (*head).object.cood_sys=COORD_LB;
      (*head).object.on_lb.y=(*axis).origin;
      (*head).object.orig_xy.y=(*axis).origin;
    } else if(strcmp((*axis).name,"RA")==0) {
      (*head).object.cood_sys=COORD_RADEC;
      (*head).object.on_radec.x=(*axis).origin;
      (*head).object.orig_xy.x=(*axis).origin;
    } else if(strcmp((*axis).name,"Dec")==0) {
      (*head).object.cood_sys=COORD_RADEC;
      (*head).object.on_radec.y=(*axis).origin;
      (*head).object.orig_xy.y=(*axis).origin;
    }
  }
/*
  (*obj).on_azel.x=0.0;
  (*obj).on_azel.y=0.0;
  (*obj).off_lb.x=0.0;
  (*obj).off_lb.y=0.0;
  (*obj).off_radec.x=0.0;
  (*obj).off_radec.y=0.0;
  (*obj).pa_xy=0.0;
  (*obj).on_xy.x=0.0;
  (*obj).on_xy.y=0.0;
  (*obj).azel_offset.x=0.0;
  (*obj).azel_offset.y=0.0;
*/

  return;
}

void adjustAntStatus(head)
CUBE_HEAD *head;
{
/*
  (*ant).ap_eff=(float)0.0;
  (*ant).mb_eff=(float)0.0;
  (*ant).fss_eff=(float)0.0;
  (*ant).hpbw=(float)0.0;
*/

  return;
}

void adjustRxStatus(head)
CUBE_HEAD *head;
{
  int ii;
  AXIS *axis;

  for(ii=0;ii<(*head).axis.dimension;ii++) {
    switch(ii) {
    case 0:
      axis= &((*head).axis.x);
      break;
    case 1:
      axis= &((*head).axis.y);
      break;
    case 2:
      axis= &((*head).axis.z);
      break;
    }

    if(strcmp((*axis).type,"velocity")==0) {
      (*head).rx.v_lsr=(*axis).origin+(*axis).delta*((double)((*axis).size+1)/2.0-1.5);
    }
    /* v_lsr is set at the ch (size+1)/2-0.5 (start ch=1) */
  }
  if((*head).rx.rest_freq==0.0) (*head).rx.rest_freq=128.0;
  if((*head).rx.obs_freq==0.0) (*head).rx.obs_freq=(*head).rx.rest_freq;
  if((*head).rx.v_off==0.0) (*head).rx.v_off=(*head).rx.v_lsr;
  (*head).rx.scale=(*head).data_value.scale;
/*
  (*rx).sb_mode=USB_SSB;
  (*rx).lo_freq1=0.0;
  (*rx).if_freq1=0.0;
  (*rx).lo_freq2=0.0;
  (*rx).if_freq2=0.0;
  (*rx).lo_freq3=0.0;
  (*rx).if_freq3=0.0;
  (*rx).mltplx1=(unsigned short)0;
  (*rx).mltplx2=(unsigned short)0;
  (*rx).mltplx3=(unsigned short)0;
  (*rx).tsys=(float)0.0;
  (*rx).rms=(float)0.0;
  (*rx).calib=(float)0.0;
  (*rx).weight=(float)1.0;
*/

  return;
}

void adjustBeStatus(head)
CUBE_HEAD *head;
{
  int ii;
  AXIS *axis;
  double freq_span;

  RANGE v_range;
  double obs_freq,center_freq;

  for(ii=0;ii<(*head).axis.dimension;ii++) {
    switch(ii) {
    case 0:
      axis= &((*head).axis.x);
      break;
    case 1:
      axis= &((*head).axis.y);
      break;
    case 2:
      axis= &((*head).axis.z);
      break;
    }

    if(strcmp((*axis).type,"velocity")==0) {
      (*head).be.be_ch=(*axis).size;
      (*head).be.data_ch=(*axis).size;
      (*head).be.center_ch=((double)(*axis).size)/2.0;
      (*head).be.basis_ch=(short)(*head).be.center_ch;
      obs_freq=(*head).rx.obs_freq;
      v_range.min=(*axis).origin-(*axis).pixel_origin*(*axis).delta;
      v_range.max=(*axis).origin+((double)((*axis).size-1)-(*axis).pixel_origin)*(*axis).delta;
      freq_span=(v_range.min-v_range.max)/(LIGHT_SPEED)*obs_freq*1.0e3;
      if(freq_span<0.0) {
	freq_span= -freq_span;
	(*head).be.sign= (char)-1;
      } else {
	(*head).be.sign= (char)1;
      }
      if((*head).be.center_freq==0.0) {
	(*head).be.center_freq=freq_span; /* from 0.5*f_span to 1.5*f_span */
      }
      center_freq=(*head).be.center_freq;
      (*head).be.disp1=freq_span/(double)((*axis).size-1);
      (*head).be.disp0=(*head).be.center_freq;
    }
  }
  if((*head).be.disp0==0.0)
    (*head).be.disp0=(*head).be.center_freq;

 /* default resolution is doulbe channel interval */
  if((*head).be.freq_res==0.0) {
    (*head).be.freq_res=fabs((*head).be.disp1*2.0);
  }

  return;
}

void adjustObsLog(head)
CUBE_HEAD *head;
{
/*
  (*log).meteo.tmp=(float)0.0;
  (*log).meteo.water=(float)0.0;
  (*log).meteo.press=(float)0.0;
  (*log).tau0=(float)0.0;
  strcpy((*log).memo,"");
  strcpy((*log).comment,"");
*/

  return;
}

void name2ntu(x,equinox)
AXIS *x;
char *equinox;
/*
equinox and velocity unit conversion

1999/8/3 T.Handa
*/
{
  char name[MAX_LWORD],*name1,*name2;

/* FITS standard names */
  if(strcmp((*x).type,"RA")==0 || strcmp((*x).type,"Dec")==0) {
    if(strcmp(equinox,"")==0) 
      strcpy((*x).refer,"B1950.0");
    else
      strcpy((*x).refer,equinox);
  }

/* velocity unit conversion m/s-->km/s */
  if(strcmp((*x).unit,"m/s")==0) {
    strcpy((*x).unit,"km/s");
    (*x).origin=(*x).origin/1.0e3;
    (*x).delta=(*x).delta/1.0e3;
    (*x).resolution=(*x).resolution/1.0e3;
    (*x).res_orig=(*x).res_orig/1.0e3;
  }

  return;
}

void adjustAxes(head,fits_axis)
CUBE_HEAD *head;
FITS_AXIS *fits_axis;
/*
convert global fits_axis to cube_head

1999/8/3 T.Handa
 */
{
  int n_axis,max_axis;
  int ii;

  int fits_xyz[3];

/* estimate actual naxis */
  max_axis=(*head).axis.dimension;
  n_axis=0;
  for(ii=0;ii<max_axis;ii++) {
    if(fits_axis[ii].naxis>1) {
      switch(n_axis) {
      case 0:
	StarHeadAxisFromFitsHead(&((*head).axis.x),&(fits_axis[ii]));
	fits_xyz[0]=ii;
	break;
      case 1:
	StarHeadAxisFromFitsHead(&((*head).axis.y),&(fits_axis[ii]));
	fits_xyz[1]=ii;
	break;
      case 2:
	StarHeadAxisFromFitsHead(&((*head).axis.z),&(fits_axis[ii]));
	fits_xyz[2]=ii;
	break;
       }
     n_axis++;
    }
  }

/* axis size */
  if((*head).axis.dimension!=n_axis) {
    fprintf(stderr,"* WRN * omit single component axes in the FITS. Original=%d revised=%d\n",(*head).axis.dimension,n_axis);
    (*head).axis.dimension=n_axis;
  }
  if(n_axis>3)
    fprintf(stderr,"**ERR** too many axes in the FITS\n");

/* divide standard FITS axis name to name, type and unit */
  name2ntu(&((*head).axis.x),(*head).object.equinox);
  name2ntu(&((*head).axis.y),(*head).object.equinox);
  name2ntu(&((*head).axis.z),(*head).object.equinox);

  return;
}

void adjustDatavalue(head)
CUBE_HEAD *head;
{
/*
  if(strcmp(key,"BUNIT")==0) {
    strncpy((*data).type,value,8);
    (*data).type[8]=NULL;
  }
*/

  return;
}

void adjustSyshead(head)
CUBE_HEAD *head;
{
  long nx,ny,nz;
  long total_pixel;

  nx=(*head).axis.x.size;
  ny=(*head).axis.y.size;
  nz=(*head).axis.z.size;

  total_pixel=nx*ny*nz;

  (*head).sys.l_file=(*head).sys.l_file+total_pixel*sizeof(float);

  return;
}

void adjust_cubehead(head,fits_axis)
CUBE_HEAD *head;
FITS_AXIS *fits_axis;
{
  adjustAxes(head,fits_axis);
  adjustObsprm(head);
  adjustObject(head);
  adjustAntStatus(head);
  adjustRxStatus(head);
  adjustBeStatus(head);
  adjustObsLog(head);
  adjustSyshead(head); /* must be the last of in this function */

  return;
}


int StarFitsHead2CubeHead(fp,head,bitpix)
FILE *fp;
CUBE_HEAD *head;
int *bitpix;
{
  FITS_AXIS fits_axis[MAX_NAXIS];

  static char block[36][80];
  char card[81];
  char key[9],value[81];

  int ii,jj;
  int key_type;
  int err_code;
  HISTORY history;

  set_default_cubehead(head);
  *bitpix=0;
  while(1) {
    err_code=StarReadFitsBlock(fp,block);
    if(err_code!=0) { /* cannot read a block */
      return(9); /* not end by END card */
    }
    for(ii=0;ii<36;ii++) {
      for(jj=0;jj<79;jj++) {
	card[jj]=block[ii][jj];
      }
      card[80]=NULL;
      StarCutFitsComment(card);
      key_type=StarDivideFitsCard(card,key,value);
fprintf(stderr,"%s\n",card);
      if(key_type==FITS_VALUE_END) { /* END card */
	goto loop_out;
      }
      fits2cubehead(head,key,key_type,value,fits_axis);
      if(strcmp(key,"BITPIX")==0) *bitpix=atoi(value);
    }
  }

loop_out:;

  adjust_cubehead(head,fits_axis);

  strcpy(history.step,"fit2cub");
  history.size=0;

  StarAddHistory4Cube(head,&history);

  return(0);
}


int options(argc,argv,input_filename,output_filename)
/*
get parameters from options
1999/4/4 T.Handa
*/
int argc;
char *argv[];
char *input_filename;
char *output_filename;
{
  int	ii;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;
  int opt_pos;

  int	err_code,flag;

  RANGE grid_range,range;

/* default values */
  input_filename[0]=NULL;

/* set keywords */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"cub"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"output_file"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"out_cub"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"fit"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"input_file"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"in_fits"); key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* name for input cube */
  scan_flag[0]=0;
  scan_flag[1]=1;
  scan_flag[2]=2;
  n_scan=3;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=NULL) {
    strcpy(output_filename,got_value);
  }

/* name for input cube */
  scan_flag[0]=3;
  scan_flag[1]=4;
  scan_flag[2]=5;
  n_scan=3;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=NULL) {
    strcpy(input_filename,got_value);
  }

  got_value=StarKeylessOption(argc,argv,n_key,key_flag,key_type);
  if(got_value[0]!=NULL) {
    strcpy(input_filename,got_value);
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

int main(argc,argv)
int argc;
char *argv[];
{
  FILE *in_fp;

  CUBE cube;

  int err_code;

  char output_cubename[MAX_FILE_NAME_LENGTH];
  char input_fitsname[MAX_FILE_NAME_LENGTH];
  char full_filename[MAX_FILE_NAME_LENGTH];
  char cubename[MAX_FILE_NAME_LENGTH];

  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];

  int n_file;
  int bitpix;

  fprintf(stderr,"**Caution**\nfor CfA-FITS, wrong endian data\nrun only on Linux\n");

  n_file=1;
  StarPipeHeadOut(n_file);

/* get option */
  err_code=options(argc,argv,input_fitsname,output_cubename);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

  if(strcmp(output_cubename,"")==0) strcpy(output_cubename,input_fitsname);

/* add STD extension for cube */
  err_code=StarStdFileExtName("STAR_DATA",input_fitsname,full_filename,".fit");
  if(err_code!=0) {
    fprintf(stderr,"Too long file name\n");
    exit(1);
  }

/* load FITS header into cube.head */
  in_fp=fopen(full_filename,"r");
  if(in_fp==NULL) {
/* add STD extension for cube */
    err_code=StarStdFileExtName("STAR_DATA",input_fitsname,full_filename,".fits");
    if(err_code!=0) {
      fprintf(stderr,"Too long file name\n");
      exit(1);
    }
    in_fp=fopen(full_filename,"r");
    if(in_fp==NULL) {
      fprintf(stderr,"%s:**ERR** cannot find the FITS file: %s\n",argv[0],full_filename);
      exit(1);
    }
  }
  err_code=StarFitsHead2CubeHead(in_fp,&(cube.head),&bitpix);

  strncpy(cube.head.obs.project,input_fitsname,16);
  cube.head.obs.project[16]=NULL;
  strncpy(cube.head.obs.scd_file,input_fitsname,8);
  cube.head.obs.scd_file[8]=NULL;

  cube.pixel=StarAllocCubePixel(&(cube.head));
  if(cube.pixel==NULL) {
    fprintf(stderr,"%s: cannot alloc memory for pixel.\n",argv[0]);
    exit(999);
  }
  err_code=WrongEndian_StarFitsPixel2CubePixel(in_fp,&cube,bitpix);
  fclose(in_fp);

  strcpy(cube.head.file_id,STAR_CUB_IN);
  strcpy(cube.head.file_name,output_cubename);

  cdb_flag=GOOD_SCAN;
  strcpy(cdb_arg,"");

/* pipeout cube data */
  err_code=StarCubeOut(&cube,cdb_flag,cdb_arg);

  StarFreeCubePixel(&cube);

  StarPipeEndOut();

  exit(0);
}
