/*
exchange and rotate axis order

2002/ 5/28 T.Handa : ver.3.00 
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"flipaxis.prm"
#define N_KEY 50
#define N_BLOCK	10

#define DEBUG
#undef DEBUG

void FlipAxis(axis)
AXIS *axis; /* <i/o> */
{
#ifdef DEBUG
  fprintf(stderr,"flip=%s,delta=%f\n",(*axis).name,(*axis).delta);
#endif

  (*axis).pixel_origin= (double)((*axis).size-1)-(*axis).pixel_origin;
  (*axis).delta= -(*axis).delta;

  return;
}

void FlipPixel(cube_out,cube_in,flip_axis)
CUBE *cube_out; /* <out> flipped cube */
CUBE *cube_in; /* <in> original cube */
int *flip_axis; /* <in> */
{
  int mx,my,mz;
  int ix,iy,iz;
  int jx,jy,jz;
  float pixel;

#ifdef DEBUG
  fprintf(stderr,"flip=%d,%d,%d\n",flip_axis[0],flip_axis[1],flip_axis[2]);
#endif

  mx=(*cube_in).head.axis.x.size;
  my=(*cube_in).head.axis.y.size;
  mz=(*cube_in).head.axis.z.size;

  for(ix=0;ix<mx;ix++) {
    if(flip_axis[0]==FLAG_YES)
      jx=mx-ix-1;
    else
      jx=ix;

    for(iy=0;iy<my;iy++) {
      if(flip_axis[1]==FLAG_YES)
	jy=my-iy-1;
      else
	jy=iy;

      for(iz=0;iz<mz;iz++) {
	if(flip_axis[2]==FLAG_YES)
	  jz=mz-iz-1;
	else
	  jz=iz;

	pixel=StarGetCubePixel(cube_in,ix,iy,iz);
	StarPutCubePixel(cube_out,jx,jy,jz,pixel);	
      }
    }
  }

  return;
}

int options(argc,argv,flip_axis)
/*
get parameters from options
*/
int argc;
char *argv[];
int *flip_axis;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

  int ii;
  int l_str;

/* default values */
  flip_axis[0]=FLAG_NO;
  flip_axis[1]=FLAG_NO;
  flip_axis[2]=FLAG_NO;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"flip_axis");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* flip axis */
  n_scan=1;
  scan_flag[0]=0;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);

  if(got_value[0]!=(char)NULL) {
    l_str=strlen(got_value);
    for(ii=0;ii<l_str;ii++) {
      switch(got_value[ii]) {
      case 'x': case 'X':
	flip_axis[0]=FLAG_YES;
	break;
      case 'y': case 'Y':
	flip_axis[1]=FLAG_YES;
	break;
      case 'z': case 'Z':
	flip_axis[2]=FLAG_YES;
	break;
      }
    }
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return 0;
}

int main(argc,argv)
int argc;
char *argv[];
{
  CUBE cube_in,cube_out;

  int flip_axis[3];
  int err_code;

  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];

  int ii,n_file;
  char id_str[MAX_FILE_NAME_LENGTH];

/* check option */
  err_code=options(argc,argv,flip_axis);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

/* pipe header */
  StarPipeHeadIn(id_str,&n_file);

  StarPipeHeadOut(n_file);

/* .cub data */
  for(ii=0;ii<n_file;ii++) {
    err_code=StarCubeIn(&cube_in,&cdb_flag,cdb_arg);
    if(err_code!=0) {
      fprintf(stderr,"%s: CubeIn failed in %dth file. code=%d\n",argv[0],ii+1,err_code);
      exit(999);
    }

/* flip axis */
    StarCopyCubeHead(&(cube_out.head),&(cube_in.head));
    cube_out.pixel=StarAllocCubePixel(&(cube_out.head));

    if(flip_axis[0]==FLAG_YES)
      FlipAxis(&(cube_out.head.axis.x));
    if(flip_axis[1]==FLAG_YES)
      FlipAxis(&(cube_out.head.axis.y));
    if(flip_axis[2]==FLAG_YES)
      FlipAxis(&(cube_out.head.axis.z));
    FlipPixel(&cube_out,&cube_in,flip_axis);

    StarCubeOut(&cube_out,cdb_flag,cdb_arg);

    StarFreeCubePixel(&cube_in);
    StarFreeCubePixel(&cube_out);
  }

/* end tag of pipe */
  StarPipeEndIn(id_str);
  StarPipeEndOut();

  exit(0);
}
