/*
frequency shift in scn header

2004/5/18 T.Handa : ver.3.00
2004/12/2 T.Handa : ver.3.01
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"freqshift.prm"
#define N_KEY 30
#define N_BLOCK	10

int options(argc,argv,f_shift,f_rest)
/*
get parameters from options
*/
int argc;
char *argv[];
double *f_shift;
double *f_rest;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  *f_shift=0.0;
  *f_rest=0.0;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"freq_shift");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"f_shift");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"rest_f");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"f_rest");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"rest_freq");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* shift for rest freq */
  n_scan=2;
  scan_flag[0]= 0;
  scan_flag[1]= 1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
      *f_shift=atof(got_value);
  }

/* new rest freq */
  n_scan=3;
  scan_flag[0]= 2;
  scan_flag[1]= 3;
  scan_flag[2]= 4;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
      *f_rest=atof(got_value);
  }

  if(*f_rest>0.0) *f_shift=0.0;
  if(*f_shift!=0.0) *f_rest=0.0;

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

void add_history_shift(profile,f_shift)
PROFILE *profile;
double f_shift;
{
  HISTORY hist;
  char buffer[80];

  hist.size=1;
  strcpy(hist.step,"fShift");

  sprintf(buffer,"%fMHz",f_shift);
  buffer[16]=(char)NULL;
  strcpy(hist.prm[0],buffer);

  StarAddHistory(profile,&hist);
  return;
}

void add_history_new(profile,f_rest,f_rest_org)
PROFILE *profile;
double f_rest;
double f_rest_org;
{
  HISTORY hist;
  char buffer[80];

  hist.size=2;
  strcpy(hist.step,"fRest");

  sprintf(buffer,"%f-->",f_rest_org);
  buffer[16]=(char)NULL;
  strcpy(hist.prm[0],buffer);
  sprintf(buffer,"%fGHz",f_rest);
  buffer[16]=(char)NULL;
  strcpy(hist.prm[1],buffer);

  StarAddHistory(profile,&hist);
  return;
}

void freq_shift(prf,f_shift)
PROFILE *prf;
double f_shift;
{
  double f_rest,f_obs;
  double f_be_center;
  int f_direct;

  f_rest=(*prf).rx_status.rest_freq;
  f_obs=(*prf).rx_status.obs_freq;
  f_direct=(int)(*prf).be_status.sign;
  f_be_center=(*prf).be_status.center_freq;

  f_rest=f_rest+f_shift/1000.0;
  f_obs=f_obs+f_shift/1000.0;
  if(f_direct>0)
    f_be_center=f_be_center+f_shift;
  else
    f_be_center=f_be_center-f_shift;

  (*prf).rx_status.rest_freq=f_rest;
  (*prf).rx_status.obs_freq=f_obs;
  (*prf).be_status.center_freq=f_be_center;

  return;
}

void freq_reset(prf,f_new_rest)
PROFILE *prf;
double f_new_rest;
{
  double f_rest,f_obs;
  double f_be_center;
  int f_direct;
  double f_shift;

  f_rest=(*prf).rx_status.rest_freq;
  f_obs=(*prf).rx_status.obs_freq;
  f_direct=(int)(*prf).be_status.sign;
  f_be_center=(*prf).be_status.center_freq;

  f_shift=f_new_rest-f_rest;
  f_obs=f_obs+f_shift;
  if(f_direct>0)
    f_be_center=f_be_center+f_shift*1000.0;
  else
    f_be_center=f_be_center-f_shift*1000.0;

  (*prf).rx_status.rest_freq=f_new_rest;
  (*prf).rx_status.obs_freq=f_obs;
  (*prf).be_status.center_freq=f_be_center;

  return;
}

int main(argc,argv)
int argc;
char *argv[];
{
  PROFILE profile;
  int i_file,n_file;
  char id_str[BUFFER_SIZE];
  int err_code;

  double f_shift,f_rest;
  double f_rest_org;

  int sdb_flag;
  char sdb_arg[MAX_IDX_RECORD_LENGTH];

/* get option */
  options(argc,argv,&f_shift,&f_rest);

/* Pipeline head */
  StarPipeHeadIn(id_str,&n_file);
  StarPipeHeadOut(n_file);

  for(i_file=0;i_file<n_file;i_file++) {
/* ProfileIn */
    err_code=StarProfileIn(&profile,&sdb_flag,sdb_arg);

    if(err_code!=0) {
      StarPipeEndOut();
      exit(1);
    }

    if(sdb_flag==GOOD_SCAN) {
      if(f_rest<=0.0) {
	freq_shift(&profile,f_shift);
	add_history_shift(&profile,f_shift);
      } else {
	f_rest_org=profile.rx_status.rest_freq;
	freq_reset(&profile,f_rest);
	add_history_new(&profile,f_rest,f_rest_org);
      }
    }

/* ProfileOut */
    err_code=StarProfileOut(&profile,sdb_flag,sdb_arg);
  }

/* End of Pipeline */
  err_code=StarPipeEndIn(id_str);
  if(err_code!=0)
    fprintf(stderr,"%s: Pipeline end invalid=%s\n",argv[0],id_str);

  StarPipeEndOut();
  exit(0);
}
