/*
demodulation of frequency switched data
       Use a new method

1998/7/25 T.Handa : ver.3.00 
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define DEFAULT_PRMFILE "fswdem.prm"
#define N_KEY 10

#define DEM_BOTH_SUM 1
#define DEM_FOREWARD_SUM 2
#define DEM_BACKWARD_SUM 3
#define DEM_FOLD 11

int options(argc,argv,method_type)
/*
get parameters from options
*/
int argc;
char *argv[];
int *method_type; /* <out> demodulation type */
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  *method_type=DEM_BOTH_SUM;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"type");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* demodulation type */
  n_scan=1;
  scan_flag[0]=0;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strcmp(got_value,"fold")==0) {
      *method_type=DEM_FOLD;
    }else if(strcmp(got_value,"both")==0 ||
	     strcmp(got_value,"bothward")==0 ||
	     strcmp(got_value,"both_sum")==0) {
      *method_type=DEM_BOTH_SUM;
    }else if(strcmp(got_value,"foreward")==0 ||
	     strcmp(got_value,"+")==0 ||
	     strcmp(got_value,"positive")==0) {
      *method_type=DEM_FOREWARD_SUM;
    }else if(strcmp(got_value,"backward")==0 ||
	     strcmp(got_value,"-")==0 ||
	     strcmp(got_value,"nagative_sum")==0) {
      *method_type=DEM_BACKWARD_SUM;
    }
  }
/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

/*=========main processing part=============*/
void freqsw_fold(in_prof,out_prof,d_ch)
/*
frequency swith demodulation by folding
1998/7/23 T.Handa
*/
PROFILE *in_prof;
PROFILE *out_prof;
double d_ch;
{
  int data_ch;
  int ch;
  int off_ch;

  data_ch=(*in_prof).be_status.data_ch;

  for(ch=0;ch<data_ch;ch++) {
    off_ch=(int)(rint)((double)ch-d_ch);
    if(off_ch<0)
      off_ch=0;
    else if(off_ch>=data_ch)
      off_ch=data_ch-1;
    (*out_prof).data[ch]=((*in_prof).data[ch]-(*in_prof).data[off_ch])/2.0;
  }
}

void freqsw_foreward_sum(in_prof,out_prof,d_ch)
/*
frequency swith demodulation by summing up foreward
1998/7/25 T.Handa
*/
PROFILE *in_prof;
PROFILE *out_prof;
double d_ch;
{
  int data_ch;
  int ch,ii;
  int off_ch;
  double fore_sum;

  data_ch=(*in_prof).be_status.data_ch;

  for(ch=0;ch<data_ch;ch++) {

/* foreward sum */
    fore_sum=0.0;
    ii=1;
    while(1) {
      off_ch=(int)(rint((double)ii*d_ch));
      off_ch=ch+off_ch;
      if(off_ch>=data_ch || off_ch<0) break;
      fore_sum=fore_sum+(double)(*in_prof).data[off_ch];
      ii++;
    }

/* total sum */
    (*out_prof).data[ch]= (float)(-fore_sum);
  }
}

void freqsw_backward_sum(in_prof,out_prof,d_ch)
/*
frequency swith demodulation by summing up backward
1998/7/25 T.Handa
*/
PROFILE *in_prof;
PROFILE *out_prof;
double d_ch;
{
  int data_ch;
  int ch,ii;
  int off_ch;
  double back_sum;

  data_ch=(*in_prof).be_status.data_ch;

  for(ch=0;ch<data_ch;ch++) {

/* backward sum */
    back_sum=0.0;
    ii=0;
    while(1) {
      off_ch=(int)(rint((double)ii*d_ch));
      off_ch=ch-off_ch;
      if(off_ch>=data_ch || off_ch<0) break;
      back_sum=back_sum+(double)(*in_prof).data[off_ch];
      ii++;
    }

/* total sum */
    (*out_prof).data[ch]= (float)back_sum;
  }
}

void freqsw_both_sum(in_prof,out_prof,d_ch)
/*
frequency swith demodulation summing both way
1998/7/25 T.Handa
*/
PROFILE *in_prof;
PROFILE *out_prof;
double d_ch;
{
  int data_ch;
  int ch,ii;
  int off_ch;
  double ta,fore_sum,back_sum;

  data_ch=(*in_prof).be_status.data_ch;

  for(ch=0;ch<data_ch;ch++) {
/* foreward sum */
    fore_sum=0.0;
    ii=1;
    while(1) {
      off_ch=(int)(rint((double)ii*d_ch));
      off_ch=ch+off_ch;
      if(off_ch>=data_ch || off_ch<0) break;
      fore_sum=fore_sum+(double)(*in_prof).data[off_ch];
      ii++;
    }

/* backward sum */
    back_sum=0.0;
    ii=1;
    while(1) {
      off_ch=(int)(rint((double)ii*d_ch));
      off_ch=ch-off_ch;
      if(off_ch>=data_ch || off_ch<0) break;
      back_sum=back_sum+(double)(*in_prof).data[off_ch];
      ii++;
    }

/* total sum */
    ta=(*in_prof).data[ch]+back_sum-fore_sum;
    (*out_prof).data[ch]=(float)(ta/2.0);
  }
}

void AddFswdemHistory(profile,method_type,delta_ch)
PROFILE *profile;
int	method_type;
double	delta_ch;
{
  HISTORY	history;
  char buffer[80];

  history.size=2;
  strcpy(history.step,"Fswdem");
  switch(method_type) {
  case DEM_FOLD:
    strcpy(history.prm[0],"type=fold"); break;
  case DEM_FOREWARD_SUM:
    strcpy(history.prm[0],"type=Sfowd"); break;
  case DEM_BACKWARD_SUM:
    strcpy(history.prm[0],"type=Sback"); break;
  default:
    strcpy(history.prm[0],"type=Sboth"); break;
  }

  sprintf(buffer,"Dch=%7.3f",delta_ch);
  buffer[16]=(char)NULL;
  strcpy(history.prm[1],buffer);

  StarAddHistory(profile,&history);
}


int main(argc,argv)
int	argc;
char	*argv[];
{
  PROFILE in_prof,out_prof;
  HISTORY history;
  int err_code;
  int i_file,n_file;
  char id_str[BUFFER_SIZE];

  int	sdb_flag;
  char	sdb_arg[MAX_IDX_RECORD_LENGTH];

  int	method_type;
  int dem_flag;
  double delta_v; /* difference of frequency switch in km/s */
  double delta_f; /* difference of frequency switch in MHz */
  double delta_ch; /* difference of frequency switch in ch */

/* get option parameters */
  err_code=options(argc,argv,&method_type);
  if(StarPipeHeadIn(id_str,&n_file)!=NORMAL_END) {
    fprintf(stderr,"%s: Broken Pipe. Not scn files. ID=%s\n",argv[0],id_str);
    exit(1);
  }
  StarPipeHeadOut(n_file);

  strcpy(history.step,"fswdem");
  history.size=2;

  for(i_file=0;i_file<n_file;i_file++) {

/* profile in */
    err_code=StarProfileIn(&in_prof,&sdb_flag,sdb_arg);
    if(err_code!=0)	{
      StarPipeEndOut();
      exit(1);
    }
    dem_flag=0;
    switch(sdb_flag) {
    case GOOD_SCAN:
      if(in_prof.sys_head.format_id!=SPECTRUM ||
	 in_prof.sys_head.data_type!=FREQSW_RAW_DATA) {
	break;
      }

/* check version */
      if(in_prof.sys_head.version<3)
	break;

/* check linear dispersion */
      if(in_prof.be_status.disp2!=0.0 ||
	 in_prof.be_status.disp3!=0.0 ||
	 in_prof.be_status.disp4!=0.0 ) {
	fprintf(stderr,"%s: non-linear dispersion for %s\n",argv[0],in_prof.file_name);
	fprintf(stderr,"Use smooth command before applying this command.\n");
	break;
      }

/* header information copying */
      StarCopyProfile(&out_prof,&in_prof);

      delta_v=in_prof.rx_status.v_lsr-in_prof.rx_status.v_off;
      if(delta_v==0.0)
	break;

      delta_f=delta_v/LIGHT_SPEED*in_prof.rx_status.rest_freq*1.0e3;
      if((int)in_prof.be_status.sign<0) delta_f= -delta_f;
      delta_ch=delta_f/in_prof.be_status.disp1;

/* actual processing */
      dem_flag=1;
      switch(method_type) {
      case DEM_FOLD:
	freqsw_fold(&in_prof,&out_prof,delta_ch);
	strcpy(history.prm[0],"method=fold");
	break;
      case DEM_FOREWARD_SUM:
	freqsw_foreward_sum(&in_prof,&out_prof,-delta_ch);
	strcpy(history.prm[0],"method=back");
	break;
      case DEM_BACKWARD_SUM:
	freqsw_backward_sum(&in_prof,&out_prof,-delta_ch);
	strcpy(history.prm[0],"method=fore");
	break;
      default:
	strcpy(history.prm[0],"method=both");
	freqsw_both_sum(&in_prof,&out_prof,-delta_ch);
      }
      AddFswdemHistory(&out_prof,method_type,delta_ch);
      out_prof.sys_head.data_type=TA_DATA;
      break;
/* for not-good scans */
    default:
      dem_flag=0;
    }

/* profile out */
    if(dem_flag==1) { /* processed data */
      err_code=StarProfileOut(&out_prof,sdb_flag,sdb_arg);
    }else{
      err_code=StarProfileOut(&in_prof,sdb_flag,sdb_arg);
    }
  }
  StarPipeEndOut();
  exit(0);
}
