/*
display gray-scale/pseudo-color map of cub on X11 screen

2000/ 2/13 T.Handa : ver.3.00
2002/ 6/20 T.Handa : ver.3.   (modify) import from contourmap.c
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include <Xm/DrawingA.h>
#include <Xm/MainW.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#include <Xm/PushBG.h>
#include <Xm/ScrollBar.h>

#include <StData.h>
#include <StStd.h>

#include <Pxxlib.h>
#include <Pxplib.h>
#include <Ppslib.h>
#include <PxpGlib.h>
#include <StPxpShow.h>

#define	DEFAULT_PRMFILE	"cubmap.prm"
#define N_KEY 20

static CUBE GLcube;
static int GLcdb_flag;
static char GLcdb_arg[MAX_IDX_RECORD_LENGTH];
static int GLn_file=0;
static int GLpipeout= -1;		/* pipe out for the next command */
static RANGE GLx_range,GLy_range,GLz_range;
static int GLx_all,GLy_all,GLz_all;
static int GLcut_plane;
static double GLcut_level,GLcut_step;
static int GLacross_all;
static RANGE GLtotal_level;
static RANGE GLgray_range;

static Widget GLdrawing_areaWDG;

static char *GLimage_plane; /* image value quantized in 256 level */

static void	(*do_plot)(), (*do_exit)(), (*do_help)();
static void read_cube(void);

void ChangeCutLevel(menubarWDG,client_data,callback_data)
/*
change the showing profiles
*/
Widget	menubarWDG;
char	*client_data;
XmScrollBarCallbackStruct	*callback_data;
{
  double cut_level,cut_step;
  int cut_plane;
  RANGE total_level;

  cut_plane=GLcut_plane;
  cut_level=GLcut_level;
  cut_step=GLcut_step;
  total_level.min=GLtotal_level.min;
  total_level.max=GLtotal_level.max;
  if(cut_level<total_level.min) cut_level=total_level.min;
  if(cut_level>total_level.max) cut_level=total_level.max;

  switch(callback_data->reason) {
  case XmCR_INCREMENT:
    cut_level=cut_level+cut_step;
    if(cut_level>total_level.max) 
      cut_level=total_level.max;
/*      return; */
    break;
  case XmCR_PAGE_INCREMENT:
    cut_level=cut_level+cut_step*10.0;
    if(cut_level>total_level.max) 
      cut_level=total_level.max;
    break;
  case XmCR_DECREMENT:
    cut_level=cut_level-cut_step;
    if(cut_level<total_level.min) 
      cut_level=total_level.min;
/*      return; */
    break;
  case XmCR_PAGE_DECREMENT:
    cut_level=cut_level-cut_step*10.0;
    if(cut_level<total_level.min) 
      cut_level=total_level.min;
    break;
  case XmCR_VALUE_CHANGED:
    cut_level=(double)callback_data->value*cut_step+total_level.min;
    break;
  case XmCR_TO_TOP:
    cut_level=total_level.min;
    break;
  case XmCR_TO_BOTTOM:
    cut_level=total_level.max;
    break;

  default:
    cut_level=GLcut_level;
  }

  GLcut_level=cut_level;
  switch(cut_plane) {
  case XY_CUT: case YX_CUT:
    GLz_range.min=cut_level;
    GLz_range.max=cut_level;
    break;
  case YZ_CUT: case ZY_CUT:
    GLx_range.min=cut_level;
    GLx_range.max=cut_level;
    break;
  case ZX_CUT: case XZ_CUT: default:
    GLy_range.min=cut_level;
    GLy_range.max=cut_level;
    break;
  }
  

  (*callback_data).reason=XmCR_EXPOSE;
  XClearWindow(PxxGetDisplay(),PxxGetWindow());
  (*do_plot)();

  return;
}

/*-------Ext------*/
/*--------from Pxxlib--------------------*/
/*static void	(*do_plot)(), (*do_exit)(), (*do_help)();*/


/* define actions for buttons on the menu bar */
void local_StarPxxPushButtonActions(menubarWDG,client_data,call_data)
Widget	menubarWDG;
caddr_t	client_data,call_data;
/*
define actions for buttons on the menu bar
1997/10/12 T.Handa ver.1.00
1997/12/12 T.Handa ver.1.00 for display command
*/
{
  if(strcmp(client_data,"Exit")==0) {
    XClearWindow(PxxGetDisplay(),PxxGetWindow());
    if(do_exit!=NULL) (*do_exit)();
    if(do_plot!=NULL) (*do_plot)();
  }
  else if(strcmp(client_data,"Quit")==0) {
    exit(0);
  }
  else if(strcmp(client_data,"Print")==0)
    PxxExecPrintOut();
  else if(strcmp(client_data,"Help")==0)
    if(do_help!=NULL) (*do_help)();
}

#define MAX_ARG 100
#define	CHAR_SIZE_DEFAULT_RESOURCE	128

/* display on X11 */
void local_StarPxxDisplayAll(argc,argv, 
    app_class,width,height,plt_fnct,exit_fnct,help_fnct)
/*
create and plot an X11 window

2000/ 1/21 T.Handa ver.1.00
*/
int argc;
char *argv[];
char app_class[];	/* name of the application class (use in .Xresource) */
double width,height;
void (*plt_fnct)(),(*exit_fnct)(),(*help_fnct)();
{
  Widget  toplevelWDG, main_windowWDG, drawing_areaWDG, 
    manu_barWDG, mbBtn[4];
  Widget paging_barWDG;
  Arg     args[10];
  int     ii;
  GC      graphic_context;

  int	mainwin_x,mainwin_y;

  int n_cut_levels;
  static char fb_n_cut_levels[BUFFER_SIZE];
  static char fb_cut_level[BUFFER_SIZE];
  int cut_plane;
  double cut_step,cut_level;
  RANGE total_level;

  int i_btm_id;

  XtAppContext    app_context;
  Arg fixed_res_args[MAX_ARG];     /* resource arguments */
  Cardinal        fixed_res_n_args;
  static char	window_w[CHAR_SIZE_DEFAULT_RESOURCE],
    window_h[CHAR_SIZE_DEFAULT_RESOURCE];
  static String   fb_res[] = {
    "*mainWindow.width:        500 ",
    "*mainWindow.height:       300 ",
    "*drawArea.background:	white",
    "*menubar.topAttachment:	ATTACH_FORM",
    "*menubar.rightAttachment:	ATTACH_FORM",
    "*menubar.leftAttachment:	ATTACH_FORM",

    "*pagingBar.orientation: VERTICAL", /* No.6 */
    "*pagingBar.height: 20",
    "*pagingBar.maximum: 2048     ", /* No.8 */
    "*pagingBar.pageIncrement: 10", /* No.9 */
    "*pagingBar.sliderSize: 1", /* No.10 */
    "*pagingBar.value: 0", /* No.11 */

    NULL};
/* add for the display command only */
  static char psfile_name[]="";
  static char prt_name[]="";

/* Change initial window size */
  mainwin_x=(int)(width*CM2INCH*DVI_SCALE+0.5);
  mainwin_y=(int)(height*CM2INCH*DVI_SCALE+0.5);
  if(mainwin_x>0 && mainwin_x<9999999) {
    sprintf(window_w,"*mainWindow.width: %8d",mainwin_x);
    fb_res[0]=window_w;
  }
  if(mainwin_y>0 && mainwin_y<9999999) {
    sprintf(window_h,"*mainWindow.height:%8d",mainwin_y);
    fb_res[1]=window_h;
  }

/*	PxxView_width=width; */
/*	PxxView_height=height; */
/*	PxxView_printer[0]=(char)NULL; */	/* set default printer */
/*	PxxView_psfile[0]=(char)NULL;*/	/* set temporaly file */
/* add for the display command only */
  PxxPutWindowSizeInPixel(width,height);
  PxxSetPrinterName(prt_name);
  PxxSetPSFName(psfile_name);

/* printer name in argv */
  for(ii=1;ii<argc;ii++) {
    if(strcmp(argv[ii],"-P")==0) PxxSetPrinterName(argv[ii+1]);
    else if(argv[ii][0]=='-' && argv[ii][1]=='P')
      PxxSetPrinterName(&(argv[ii][2]));
  }

/* PS filename in argv */
  for(ii=1;ii<argc;ii++) {
    if(strcmp(argv[ii],"-F")==0) PxxSetPSFName(argv[ii+1]);
    else if(argv[ii][0]=='-' && argv[ii][1]=='F')
      PxxSetPSFName(&(argv[ii][2]));
  }

/* set plot, exit, and help functions */
  PxxSetPlotFunction(plt_fnct);
  PxxSetExitFunction(exit_fnct);
  PxxSetHelpFunction(help_fnct);
/* add for the display commnad only */
  do_plot=plt_fnct;
  do_exit=exit_fnct;
  do_help=help_fnct;

/* from global to local */
  cut_plane=GLcut_plane;
  cut_level=GLcut_level;

/* Cube In */
  read_cube();

/* Set slidebar length and its initial position */
  switch(cut_plane) {
  case XY_CUT: case YX_CUT:
    n_cut_levels=GLcube.head.axis.z.size;
    cut_step=fabs(GLcube.head.axis.z.delta);
    total_level.min=StarGetCubePixPosOnAxis(&(GLcube.head.axis.z),0);
    total_level.max=StarGetCubePixPosOnAxis(&(GLcube.head.axis.z),GLcube.head.axis.z.size-1);
    StarSortRange(&total_level);
    break;
  case YZ_CUT: case ZY_CUT:
    n_cut_levels=GLcube.head.axis.x.size;
    cut_step=fabs(GLcube.head.axis.x.delta);
    total_level.min=StarGetCubePixPosOnAxis(&(GLcube.head.axis.x),0);
    total_level.max=StarGetCubePixPosOnAxis(&(GLcube.head.axis.x),GLcube.head.axis.x.size-1);
    StarSortRange(&total_level);
    break;
  case ZX_CUT: case XZ_CUT: default:
    n_cut_levels=GLcube.head.axis.y.size;
    cut_step=fabs(GLcube.head.axis.y.delta);
    total_level.min=StarGetCubePixPosOnAxis(&(GLcube.head.axis.y),0);
    total_level.max=StarGetCubePixPosOnAxis(&(GLcube.head.axis.y),GLcube.head.axis.y.size-1);
    StarSortRange(&total_level);
    break;
  }

  if(GLacross_all==FLAG_YES) cut_level=total_level.min;

  if(cut_level<total_level.min) cut_level=total_level.min;
  if(cut_level>total_level.max) cut_level=total_level.max;

  sprintf(fb_n_cut_levels,"*pagingBar.maximum: %d ",n_cut_levels);
  fb_res[8]=fb_n_cut_levels;
  sprintf(fb_cut_level,"*pagingBar.value: %d ",(int)rint((fabs(cut_level-total_level.min)/cut_step)));
  fb_res[11]=fb_cut_level;

/* from local to global */
  GLcut_level=cut_level;
  GLcut_step=cut_step;
  GLtotal_level.min=total_level.min;
  GLtotal_level.max=total_level.max;

/* create X11 window */
  fixed_res_n_args=0;
  toplevelWDG=XtAppInitialize(&app_context,app_class, NULL, 0, &argc, argv,
			      fb_res,fixed_res_args,fixed_res_n_args);

  main_windowWDG=XmCreateMainWindow(toplevelWDG,"mainWindow",NULL,(Cardinal)NULL);
  XtManageChild(main_windowWDG);

/* create menu bar */
  manu_barWDG=XmCreateMenuBar(main_windowWDG,"menubar",NULL,(Cardinal)NULL);
  XtManageChild(manu_barWDG);
  i_btm_id=0;

  mbBtn[i_btm_id]=XmCreateCascadeButton(manu_barWDG,"help", args,0);
  XtAddCallback(mbBtn[i_btm_id],XmNactivateCallback, (XtCallbackProc)local_StarPxxPushButtonActions,"Help");
  i_btm_id++;
  mbBtn[i_btm_id]=XmCreateCascadeButton(manu_barWDG,"next", args,0);
  XtAddCallback(mbBtn[i_btm_id],XmNactivateCallback, (XtCallbackProc)local_StarPxxPushButtonActions, "Exit");
  i_btm_id++;
  mbBtn[i_btm_id]=XmCreateCascadeButton(manu_barWDG,"print", args,0);
  XtAddCallback(mbBtn[i_btm_id],XmNactivateCallback, (XtCallbackProc)local_StarPxxPushButtonActions, "Print");
  i_btm_id++;
  mbBtn[i_btm_id]=XmCreateCascadeButton(manu_barWDG,"quit", args,0);
  XtAddCallback(mbBtn[i_btm_id],XmNactivateCallback, (XtCallbackProc)local_StarPxxPushButtonActions, "Quit");
  i_btm_id++;
  XtManageChildren(mbBtn,i_btm_id);

  XtSetArg(args[0],XmNmenuHelpWidget,(XtArgVal)mbBtn[0]);
  XtSetValues(manu_barWDG,args,1);

/* create data ploting area */
  drawing_areaWDG=XmCreateDrawingArea(main_windowWDG,"drawArea",args,0);
  XtManageChild(drawing_areaWDG);

  GLdrawing_areaWDG=drawing_areaWDG;

/* set alinement of the componets in a window */
  XmMainWindowSetAreas(main_windowWDG,manu_barWDG,NULL,NULL,NULL,drawing_areaWDG);
  /*-------------*/

/* paging bar */
  paging_barWDG=XmCreateScrollBar(main_windowWDG,"pagingBar",args,0);

  XtAddCallback(paging_barWDG,XmNdecrementCallback,(XtCallbackProc)ChangeCutLevel,"prev");
  XtAddCallback(paging_barWDG,XmNpageDecrementCallback,(XtCallbackProc)ChangeCutLevel,"prev_page");
  XtAddCallback(paging_barWDG,XmNincrementCallback,(XtCallbackProc)ChangeCutLevel,"next");
  XtAddCallback(paging_barWDG,XmNpageIncrementCallback,(XtCallbackProc)ChangeCutLevel,"next_page");
  XtAddCallback(paging_barWDG,XmNvalueChangedCallback,(XtCallbackProc)ChangeCutLevel,"scroll_page");
  XtAddCallback(paging_barWDG,XmNtoTopCallback,(XtCallbackProc)ChangeCutLevel,"to_top");
  XtAddCallback(paging_barWDG,XmNtoBottomCallback,(XtCallbackProc)ChangeCutLevel,"to_bottom");
  XtManageChild(paging_barWDG);

  /*--------------*/

/* realize window */
  XtRealizeWidget(toplevelWDG);

  graphic_context=XCreateGC(XtDisplay(drawing_areaWDG),XtWindow(drawing_areaWDG),0,0);

  XtAddCallback(drawing_areaWDG,XmNexposeCallback,(XtCallbackProc)PxxExecDraw,graphic_context);

  XtAddCallback(drawing_areaWDG,XmNresizeCallback,(XtCallbackProc)PxxExecDraw,graphic_context);

/* keep showing */
  XtAppMainLoop(app_context);
}

/*--------from Pxplib--------------------*/
/* execute X11 display or PostScript output */
void local_StarPxpDisplayAll(argc,argv, output, filename,
	app_class,width,height,plt_fnct,exit_fnct,help_fnct)
/*
create the standared X11 window or PostScripte file

1997/10/14 T.Handa ver.1.00
1997/12/02 T.Handa ver.1.00 for the display command
*/
int argc;
char *argv[];
int output;
char *filename;	/* stdout for - or NULL, workfile for filename==NULL */
char app_class[];	/* name of the application class (use in .Xresource) */
double width,height;
void (*plt_fnct)(),(*exit_fnct)(),(*help_fnct)();
{
  void (*plot_function)();

  plot_function=plt_fnct;
/*	PxpView_output=output;*/
  PxpPutOutputMode(output);

  switch(output) {
  case X11WIN:
    local_StarPxxDisplayAll(argc,argv, app_class,width,height,plt_fnct,exit_fnct,help_fnct);
    break;
  case POSTSCRIPT:
    PpsInitialize(filename,width,height);
    plot_function();	/* execute the user-defined plot function */
    PpsPrintPage();
    PpsDoPrint();
  }
}

/*---------------------------------*/
int options(argc,argv, w_height,w_width,x_range,y_range,z_range,x_all,y_all,z_all,cut_plane,gray_range,pipe_flag)
/*
get parameters from options
*/
int argc;
char *argv[];
double	*w_height, *w_width;	/* size of the window(cm) */
RANGE	*x_range,*y_range,*z_range;
int *x_all,*y_all,*z_all;
int *cut_plane;
RANGE *gray_range;
int	*pipe_flag;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *token;
  int opt_pos;
  char buffer[MAX_PRM_RECORD_LENGTH];

  int tmp_flag;
  double window_x,window_y;

  char *got_value;

/* default values */
  *w_height=18.0;	*w_width=25.0;
  (*x_range).min=0.0;	(*x_range).max=0.0;
  (*y_range).min=0.0;	(*y_range).max=0.0;
  (*z_range).min=0.0;	(*z_range).max=0.0;
  *x_all=FLAG_YES; *y_all=FLAG_YES; *z_all=FLAG_YES;
  *cut_plane=XY_CUT;
  (*x_range).min=0.0;	(*x_range).max=0.0;
  (*y_range).min=0.0;	(*y_range).max=0.0;
  (*z_range).min=0.0;	(*z_range).max=0.0;
  (*gray_range).min=0.0; (*gray_range).max=0.0;
  *pipe_flag= -1;

  window_x=0.0;
  window_y=0.0;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"size");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"x");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"x_range");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y_range");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z_range");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"pipe");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"-");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"nopipe");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"cut_plane");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"gray_range"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"color_range"); key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* window size */
  scan_flag[0]=0;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(buffer,got_value);
    token=strtok(buffer,"xX ,\t\n");
    if(token!=NULL) {
      window_x=atof(token);
      token=strtok(NULL," ,\t\n");
      if(token!=NULL) window_y=atof(token);
    }
    *w_height=window_y;
    *w_width=window_x;
  }

/* x_range */
  scan_flag[0]=1; scan_flag[1]=2;
  n_scan=2;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    strcpy(buffer,value[opt_pos]);
    if(strcmp(buffer,"all")==0) {
      (*x_range).min=0.0;
      (*x_range).max=0.0;
      *x_all=FLAG_YES;
    } else {
      /*      StarOptionRange(buffer,x_range);*/
      StarOptionAngleRange(buffer,x_range);
      *x_all=FLAG_NO;
    }
  } 

/* y_range */
  scan_flag[0]=3; scan_flag[1]=4;
  n_scan=2;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    strcpy(buffer,value[opt_pos]);
    if(strcmp(buffer,"all")==0) {
      (*y_range).min=0.0;
      (*y_range).max=0.0;
      *y_all=FLAG_YES;
    } else {
      StarOptionRange(buffer,y_range);
      *y_all=FLAG_NO;
    }
  } 

/* z_range */
  scan_flag[0]=5; scan_flag[1]=6;
  n_scan=2;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    strcpy(buffer,value[opt_pos]);
    if(strcmp(buffer,"all")==0) {
      (*z_range).min=0.0;
      (*z_range).max=0.0;
      *z_all=FLAG_YES;
    } else {
      StarOptionRange(buffer,z_range);
      *z_all=FLAG_NO;
    }
  }

/* pipe */
  scan_flag[0]=7; scan_flag[1]=8; scan_flag[2]= -9;
  n_scan=3;
  tmp_flag=StarOptionFlag(n_scan,scan_flag,key_flag);
  if(tmp_flag!=0) *pipe_flag=tmp_flag;

/* cut plane */
  scan_flag[0]=10;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strcmp(got_value,"xy")==0)
      *cut_plane=XY_CUT;
    else if(strcmp(got_value,"yx")==0)
      *cut_plane=YX_CUT;
    else if(strcmp(got_value,"yz")==0)
      *cut_plane=YZ_CUT;
    else if(strcmp(got_value,"zy")==0)
      *cut_plane=ZY_CUT;
    else if(strcmp(got_value,"zx")==0)
      *cut_plane=ZX_CUT;
    else if(strcmp(got_value,"xz")==0)
      *cut_plane=XZ_CUT;
  }

/* gray range */
  scan_flag[0]=11;
  scan_flag[1]=12;
  n_scan=2;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    strcpy(buffer,value[opt_pos]);
    StarOptionRange(buffer,gray_range);
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);
/* set return values */

  return(0);
}


/* Main Body of process */

void read_cube()
{
  static int i_file=0;
  int n_file,n_act_file,pipeout;
  int err_code;

/* copy from global valuables */
  n_file=GLn_file;
  pipeout=GLpipeout;

  n_act_file=0;
  while(1) {
    if(i_file>=n_file) {
      if(pipeout>0) StarPipeEndOut();
      exit(0);
    }

    err_code=StarCubeIn(&GLcube,&GLcdb_flag,GLcdb_arg);
    if(err_code!=0) {
      fprintf(stderr,"fail to CubeIn\n");
      exit(1);
    }
    i_file++;

    if(pipeout>0)
      StarCubeOut(&GLcube,GLcdb_flag,GLcdb_arg);

    if(GLcdb_flag==GOOD_SCAN || GLcdb_flag==BAD_SCAN) {
      n_act_file++;
    }

    if(GLcdb_flag==GOOD_SCAN || GLcdb_flag==BAD_SCAN)
      break;
  }

  if(n_act_file==0) {
    if(pipeout>0) StarPipeEndOut();
    exit(0);
  }
}


void actual_plot()
{
  RANGE x_range,y_range,z_range;
  int cut_plane;
  double cut_level;
  RANGE gray_range;

  BOX_RANGE plot_box;

  static PxpPoint win_size;

  int output_mode;
  int window_x,window_y,window_h,window_w;

/* copy from global valuables */
  x_range.min=GLx_range.min;
  x_range.max=GLx_range.max;
  y_range.min=GLy_range.min;
  y_range.max=GLy_range.max;
  z_range.min=GLz_range.min;
  z_range.max=GLz_range.max;

  gray_range.min=GLgray_range.min;
  gray_range.max=GLgray_range.max;

  cut_plane=GLcut_plane;
  cut_level=GLcut_level;

  switch(cut_plane) {
  case XY_CUT: case YX_CUT:
    z_range.min=cut_level;
    z_range.max=cut_level;
    break;
  case YZ_CUT: case ZY_CUT:
    x_range.min=cut_level;
    x_range.max=cut_level;
    break;
  case ZX_CUT: case XZ_CUT: default:
    y_range.min=cut_level;
    y_range.max=cut_level;
    break;
  }

/* read profile if the first */
/*
  if(read_flag>0) {
    read_cube();
    read_flag= -1;
  }
*/

/* plotsize auto scaling */
  PxpGetOutputMode(&output_mode);
  if(output_mode==X11WIN) {
    PxxPresentWindowSize(&window_x,&window_y,&window_w,&window_h);
    win_size.x=PxxGetPlotSpanX(window_w);
    win_size.y=PxxGetPlotSpanY(window_h);
  }

  plot_box.x.min=win_size.x*0.1;
  plot_box.y.min=win_size.y*0.1;
  plot_box.x.max=plot_box.x.min+win_size.x*0.7;
  plot_box.y.max=plot_box.y.min+win_size.y*0.7;

/* cube-map in standard form */
  StarPlotCubeByGrayCellsStd(&GLcube,x_range,y_range,z_range,cut_plane,plot_box,gray_range);

  return;
}

void help_function()
{
  fprintf(stderr,"help should be shown.\n");
  return;
}

int main(argc,argv)
int argc;
char *argv[];
{
  int err_code;
  int n_file;

  double w_height, w_width;	/* size of window(cm) */
  RANGE	x_range,y_range,z_range;	/* range to display */
  int x_all,y_all,z_all;
  int cut_plane;
  RANGE gray_range; /* color code range */
  int pipeout;

  char id_str[BUFFER_SIZE];

/* check option */
  err_code=options(argc,argv, &w_height,&w_width,&x_range,&y_range,&z_range,&x_all,&y_all,&z_all,&cut_plane,&gray_range,&pipeout);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

/* set global parameters */
  GLpipeout=pipeout;

  GLx_range.min=x_range.min;
  GLx_range.max=x_range.max;
  GLy_range.min=y_range.min;
  GLy_range.max=y_range.max;
  GLz_range.min=z_range.min;
  GLz_range.max=z_range.max;
  GLx_all=x_all;
  GLy_all=y_all;
  GLz_all=z_all;

  GLcut_plane=cut_plane;
  GLgray_range.min=gray_range.min;
  GLgray_range.max=gray_range.max;

/* initial cut level */
  switch(cut_plane) {
  case XY_CUT: case YX_CUT:
    GLcut_level=z_range.min;
    GLacross_all=z_all;
    break;
  case YZ_CUT: case ZY_CUT:
    GLcut_level=x_range.min;
    GLacross_all=x_all;
    break;
  case ZX_CUT: case XZ_CUT: default:
    GLcut_level=y_range.min;
    GLacross_all=y_all;
    break;
  }

/* multi-data header in */
  StarPipeHeadIn(id_str,&n_file);
  GLn_file=n_file;

  if(pipeout>0) StarPipeHeadOut(n_file);

  GLimage_plane=NULL;

/* plot color map */
  local_StarPxpDisplayAll(argc,argv, X11WIN,NULL,"display",w_width,w_height,actual_plot,read_cube,help_function);

  exit(0);
}
