/*
.scn averager get from .sdb-PIPEflow

1992/12/10 T.Handa : ver.1.00 STAR_DATA option
1993/5/9 T.Handa : ver.1.30 -rms flag
1998/1/5 T.Handa : ver.3.00 new lib & multi-pipe
1998/3/19 T.Handa : ver.3.01 (debug) for Solaris2 fcc
1998/4/2 T.Handa : ver.3.10 (modify) direct .scn input
1998/4/10 T.Handa : ver.3.20 (revise) .scn,.sdb
2000/1/18 T.handa : ver.3.30 (revise) reconstruct and support command line
2002/5/8 T.Handa : ver.3.40 (revise) !integ support
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"integ.prm"
#define N_KEY 30

#define SDB_INPUT 0
#define SCN_INPUT 1

#define INTEG_EXE 100
#define INTEG_END 200
#define INTEG_STILL 1
#define INTEG_SKIP 0

#define SDB_EOF 100
#define SDB_BAD 1
#define SDB_GOOD 0

#ifdef _NO_PROTO
extern int StarDivideProfile();
extern int StarCompareProfileHeader();
extern int StarIntegrateProfile();
extern int GetScnUsingSdbEntry();
extern void PrintIntegErrorMassage();
extern int AverageProcess();
extern int options();
#else
extern int StarDivideProfile(PROFILE *, PROFILE *);
extern int StarCompareProfileHeader(PROFILE *, PROFILE *);
extern int StarIntegrateProfile(int ,PROFILE *, PROFILE  *, PROFILE *, int);
extern int GetScnUsingSdbEntry(char *,PROFILE *,char *,char *,int *);
extern void PrintIntegErrorMassage(int);
extern int AverageProcess(PROFILE *, PROFILE *, int, HISTORY *, int);
extern int options(int, char **, char *, int *, int *);
#endif

int options(argc,argv,default_scnname,weight_mode,input_format)
/*
get parameters from options
*/
int argc;
char *argv[];
char default_scnname[];
int *weight_mode;
int *input_format;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;
  int opt_pos;

/* default values */
  default_scnname[0]=(char)NULL;
  *weight_mode=0; /* weight mode = weighted by header */
  *input_format=SDB_INPUT;

/* set keywords */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"scn");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"scnname");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"scanname");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"weighted");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"uniform");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"rms");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"integ_mode");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"input_format");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* name for the integrated scan */
  scan_flag[0]=0;
  scan_flag[1]=1;
  scan_flag[2]=2;
  n_scan=3;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0)
    strcpy(default_scnname,value[opt_pos]);

/* weight mode */
  scan_flag[0]=3;
  scan_flag[1]=4;
  scan_flag[2]=5;
  scan_flag[3]=6;
  n_scan=4;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    if(strcmp(keyword[opt_pos],"weighted")==0)
      *weight_mode=0;	/* weighted by each header */
    else if(strcmp(keyword[opt_pos],"uniform")==0)
      *weight_mode=1;	/* 1: uniform weight */
    else if(strcmp(keyword[opt_pos],"rms")==0)
      *weight_mode=2;	/* 2: inverse of rms^2 weight */
    else if(strcmp(keyword[opt_pos],"integ_mode")==0) {
      if(strcmp(value[opt_pos],"weighteg")==0)
	*weight_mode=0;	/* weighted by each header */
      else if(strcmp(value[opt_pos],"uniform")==0)
	*weight_mode=1;	/* 1: uniform weight */
      else if(strcmp(value[opt_pos],"rms")==0)
	*weight_mode=2;	/* 2: inverse of rms^2 weight */
    }
  }

/* weight mode */
  scan_flag[0]=7;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(strcmp(got_value,"scn")==0 || strcmp(got_value,"scan")==0)
    *input_format=SCN_INPUT;
  else if(strcmp(got_value,"sdb")==0 || strcmp(got_value,"idx")==0)
    *input_format=SDB_INPUT;


/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return 0;
}

int StarDivideProfile(profile,div_profile)
PROFILE *profile,*div_profile;
{
  int ch;

  for(ch=0;ch<(*profile).be_status.data_ch;ch++) {
    if((*div_profile).data[ch]!=0.0)
      (*profile).data[ch]=(*profile).data[ch]/(*div_profile).data[ch];
    else
      (*profile).data[ch]=(float)(-9999.9);
  }

  return 0;
}


/* ------- main processing ------ */
int	StarCompareProfileHeader(prof_a,prof_b)
PROFILE	*prof_a, *prof_b;
/*
compare two headers of PROFILEs

return value

0=available to integrate
10=different on position (more than 1/4 beamsize)
6010=different backend channel numbers
6020=different backend dispersion function
6030=different backend resolution (more than 10%)
6041=different spectrum data channel
6042=different binning parameter
7010=different Vlsr
7020=different rest frequency
8000=different current beamsize (more than 10%)
9999=different data type
9990=different data switching mode

1993/1/19 T.Handa
*/
{
  double hpbw, freq_res;
  double on_pos,d_l,d_b;

/* data type check */
  if((*prof_a).sys_head.data_type!=(*prof_b).sys_head.data_type)
    return 9999;
/* swiching mode check */ 
  if((*prof_a).obs_prm.sw_mode!=(*prof_b).obs_prm.sw_mode)
    return 9990;
/* beamsize check */
  if((*prof_a).ant_status.hpbw>(*prof_b).ant_status.hpbw*1.1 || (*prof_a).ant_status.hpbw<(*prof_b).ant_status.hpbw*0.9)
    return 8000;

  hpbw=(double)(*prof_a).ant_status.hpbw;

/* frequency check */
  if((*prof_a).rx_status.v_lsr!=(*prof_b).rx_status.v_lsr)
    return 7010;
  if((*prof_a).rx_status.rest_freq!=(*prof_b).rx_status.rest_freq)
    return 7020;

/* Backend check */
  if((*prof_a).be_status.be_ch!=(*prof_b).be_status.be_ch)
    return 6010;
/*
  if((*prof_a).be_status.basis_ch!=(*prof_b).be_status.basis_ch)
    return 6020;
  if((*prof_a).be_status.center_freq!=(*prof_b).be_status.center_freq)
    return 6020;
  if((*prof_a).be_status.sign!=(*prof_b).be_status.sign)
    return 6020;
  if((*prof_a).be_status.disp0!=(*prof_b).be_status.disp0 || (*prof_a).be_status.disp1!=(*prof_b).be_status.disp1 || (*prof_a).be_status.disp2!=(*prof_b).be_status.disp2 || (*prof_a).be_status.disp3!=(*prof_b).be_status.disp3 || (*prof_a).be_status.disp4!=(*prof_b).be_status.disp4)
    return 6020;
*/

  if((*prof_a).be_status.freq_res>(*prof_b).be_status.freq_res*1.1 || (*prof_a).be_status.freq_res<(*prof_b).be_status.freq_res*0.9)
    return 6030;

  freq_res=(double)(*prof_a).be_status.freq_res;

  if((*prof_a).be_status.data_ch!=(*prof_b).be_status.data_ch)
    return 6041;
  if((*prof_a).be_status.ch_width!=(*prof_b).be_status.ch_width)
    return 6042;
  if((*prof_a).be_status.ch_orig!=(*prof_b).be_status.ch_orig)
    return 6042;

/* on position check */
  d_l=(*prof_a).object.on_lb.x-(*prof_b).object.on_lb.x;
  d_b=(*prof_a).object.on_lb.y-(*prof_b).object.on_lb.y;
  on_pos=sqrt(d_l*d_l+d_b*d_b);
  if(on_pos>hpbw/4.0)
    return 10;

  return 0;	
}


int StarIntegrateProfile(seq_numb,integ_prof,profile,weight_prof,weight_mode)
int seq_numb;
PROFILE	*profile,
  *integ_prof,
  *weight_prof;
int weight_mode;
/*
execute integration profiles by weight_mode

weight_mode=1	uniform weight
weight_mode=2	inverse of rms^2 weight
weight_mode=0	weighed in each header

return value
0=normal return
1=not integratable profiles

1993/1/19 T.Handa
1993/5/7 T.Handa weight mode
*/
{
  int ch;
  int err_code;

  double data_weight;

/* set weight value */

  switch(weight_mode) {
  case 1: /* uniform */
    data_weight=(*profile).rx_status.weight;
    break;
  case 2: /* inverse of rms^2 */
    data_weight=1.0/(double)(*profile).rx_status.rms;
    data_weight=data_weight*data_weight;
    break;
  default: /* weight in header */
    data_weight=(*profile).rx_status.weight;
  }

  if(seq_numb==0) {
    StarCopyProfile(integ_prof,profile);
    for(ch=1;ch<=(*profile).be_status.data_ch;ch++) {
      if(StarReadSpuriousFlag(profile,ch)==FLAGGED) {
	(*weight_prof).data[ch-1]=(float)0.0;
	(*integ_prof).data[ch-1]=(float)0.0;
      } else{
	(*weight_prof).data[ch-1]=(float)data_weight;
	(*integ_prof).data[ch-1]=(*profile).data[ch-1]*(float)data_weight;
      }
    }
  } else {
/* check profile type */
    err_code=StarCompareProfileHeader(integ_prof,profile);
    if(err_code==10) {
      fprintf(stderr,"integ: pos diff %s and %s but done.\n",
	      (*integ_prof).file_name,(*profile).file_name);
    } else if(err_code!=0){
      return err_code;
    }
/* increase original scan numbers */
    (*integ_prof).sys_head.num_scan+=(*profile).sys_head.num_scan;

/* integrate channel data */
    for(ch=1;ch<=(*profile).be_status.data_ch;ch++) {

      if(StarReadSpuriousFlag(profile,ch)==UNFLAGGED) {
	if(StarReadSpuriousFlag(integ_prof,ch)==FLAGGED) {
        /* current!=spurious && integ==spurious */
	  (*integ_prof).data[ch-1]=(*profile).data[ch-1]*(float)data_weight;
	  (*weight_prof).data[ch-1]=(*profile).rx_status.weight;
	  StarWriteSpuriousFlag(integ_prof,ch,UNFLAGGED);

	  StarWriteBaselineFlag(integ_prof,ch,StarReadBaselineFlag(profile,ch));

	} else {
	/* current!=spurious && integ!=spurious */

	  (*integ_prof).data[ch-1]=(*integ_prof).data[ch-1]+(*profile).data[ch-1]*(float)data_weight;

	  (*weight_prof).data[ch-1]=(*weight_prof).data[ch-1]+(float)data_weight;

	  if(StarReadBaselineFlag(profile,ch)==UNFLAGGED)
	    StarWriteBaselineFlag(integ_prof,ch,UNFLAGGED);

	}
	/* no modification for integ_prof, if current==spurious. */ 
      }
	/* next channel */
    }
/* revise profile weight */
    (*integ_prof).rx_status.weight=(*integ_prof).rx_status.weight+(*profile).rx_status.weight;
/* revise integration time */
    (*integ_prof).obs_prm.on_time=(*integ_prof).obs_prm.on_time+(*profile).obs_prm.on_time;
    (*integ_prof).obs_prm.off_time=(*integ_prof).obs_prm.off_time+(*profile).obs_prm.off_time;
  }

  return 0;
}

int GetScnUsingSdbEntry(cmd_name,profile,file_name,sdb_arg,flag)
char *cmd_name; /* <out> */
PROFILE *profile; /* <out> */
char *file_name; /* <out> */
char *sdb_arg; /* <out> */
int *flag; /* <out> GOOD_SCAN,BAD_SCAN,IDX_COMMENT,IDX_COMMAND */
{
  char input_file_name[MAX_FILE_NAME_LENGTH];
  FILE *file_p;	/* file to store data */

  int err_code;

/* get .sdb entries */
  err_code=StarSdbIn(file_name,flag,sdb_arg);
  switch(err_code) {
  case 201:
    *flag=GOOD_SCAN;	/* without flag means "good" */
  case 202:
    err_code=0;
    break;
  case 100:
    *flag=IDX_EOF;
    return SDB_EOF;	/* detect EOF */
  case 11:
    *flag=IDX_COMMENT;		/* blank line means comment */
  }

  if(*flag==GOOD_SCAN) { /* only for unflagged profiles */
/* make complete filename */
    err_code=StarStdFileExtName("STAR_DATA",file_name,input_file_name,".scn");
    if(err_code!=0) {
      fprintf(stderr,"%s: cannot make scn name.\n",cmd_name);
      return SDB_BAD;
    }
/* open files */
    file_p=fopen(input_file_name,"r");
    if(file_p==NULL) {
      fprintf(stderr,"%s: Don't exist file %s,\n",cmd_name,input_file_name);
    } else {
/* load profile */
      err_code=StarProfileLoad(file_p,profile);
      fclose(file_p);
      if(err_code!=0) {
	fprintf(stderr,"%s: failed to read scn file %s,\n",cmd_name,input_file_name);
	return SDB_BAD;
      }
    }
    return SDB_GOOD;
  }
  return SDB_BAD;
}

void PrintIntegErrorMassage(err_code)
int err_code;
{
  switch(err_code) {
  case 6010:
    fprintf(stderr,"Different AOS ch number.\n");
    break;
  case 6020:
    fprintf(stderr,"Different AOS prm.\n");
    break;
  case 6030:
    fprintf(stderr,"Different AOS resolution.\n");
    break;
  case 6041:
    fprintf(stderr,"Different data ch number.\n");
    break;
  case 6042:
    fprintf(stderr,"Different binning prm.\n");
    break;
  case 7010:
    fprintf(stderr,"Different Vlsr.\n");
    break;
  case 7020:
    fprintf(stderr,"Different rest freq.\n");
    break;
  case 8000:
    fprintf(stderr,"Different HPBW.\n");
    break;
  case 9990:
    fprintf(stderr,"Different switching mode.\n");
    break;
  case 9999:
    fprintf(stderr,"Different data type.\n");
    break;
  }
}

/* ---- main function ---- */
int AverageProcess(integ_prof,weight_prof,weight_mode,history,seq_numb)
PROFILE *integ_prof,*weight_prof;
int weight_mode;
HISTORY *history;
int seq_numb;
{
  RANGE_long ch_range;
  int err_code;

/* normarize the integrated profile */
  err_code=StarDivideProfile(integ_prof,weight_prof);

/* recalc rms */
  ch_range.min=1; ch_range.max=(*integ_prof).be_status.data_ch;
  StarCalcRms(integ_prof,ch_range);
  if(weight_mode!=0){	/* uniform or inverse of rms^2 */
    (*integ_prof).rx_status.weight=1.0/(*integ_prof).rx_status.rms;
    (*integ_prof).rx_status.weight=(*integ_prof).rx_status.weight*(*integ_prof).rx_status.weight;
  }

/* add history */
  sprintf((*history).prm[0],"%5d scns",seq_numb);

  if(seq_numb-1>=MAX_HISTORY_LINES_PER_STEP)
    seq_numb=MAX_HISTORY_LINES_PER_STEP-2;
  (*history).size=seq_numb+2;

  err_code=StarAddHistory(integ_prof,history);
  if(err_code!=0) {
    fprintf(stderr,"History step overflow!\n");
    return 999;
  }

  return 0;
}

int main(argc,argv)
int argc;
char *argv[];
{
  char	file_name[MAX_FILE_NAME_LENGTH],
    default_filename[MAX_FILE_NAME_LENGTH],
    default_sdb[MAX_FILE_NAME_LENGTH],
    primary_filename[MAX_FILE_NAME_LENGTH];

  int	sdb_flag;
  char	sdb_arg[MAX_IDX_RECORD_LENGTH];

  int	sdb_backver;

  PROFILE profile,integ_prof,weight_prof;
  HISTORY history;

  int err_code;
  int weight_mode; /* 1=uniform weight, 2=inverse of rms^2 weight, 0=weighed in each header */
  int input_format;
  int i_entry,n_entry; /* input entry, total entry */
  int i_file,j_file; /* input file, output file */
  int integ_out_flag;

  char id_str[BUFFER_SIZE];

  int prf_input_status;
  char *command_mainname;

/* check option */
  err_code=options(argc,argv,default_filename,&weight_mode,&input_format);
  if(err_code!=0 && err_code!=111) {
    StarPipeEndOut();
    exit(1);
  }

  command_mainname=StarFileMainName(argv[0]);
  if(strcmp(command_mainname,"sdbinteg")==0) {
    input_format=SDB_INPUT;
  } else if(strcmp(command_mainname,"scninteg")==0) {
    input_format=SCN_INPUT;
  }

  if(input_format==SDB_INPUT) {
/* header of .sdb pipe flow */
    err_code=StarSdbHeadIn(default_sdb,&n_entry,&sdb_backver);
    if(err_code!=0) {
      fprintf(stderr,"%s: Broken Pipe. Not a sdb file.\n",argv[0]);
      exit(1);
    }
    n_entry=MAX_FILE_NUMBER;
  } else {
    err_code=StarPipeHeadIn(id_str,&n_entry);
    if(err_code!=0) {
      fprintf(stderr,"%s: Broken Pipe. Not scn files. Id=%s\n",argv[0],id_str);
      exit(1);
    }
  }

/* add history */
  strcpy(history.step,"INTEG");
  switch(weight_mode) {
  case 1: /* uniform */
    sprintf(history.prm[1],"Uniform weight  ");
    break;
  case 2: /* inverse of rms^2 */
    sprintf(history.prm[1],"1/(rms^2) weight");
    break;
  default: /* weight in header */
    sprintf(history.prm[1],"Header weight   ");
  }

/* pipeline head */
  StarPipeHeadOut(n_entry);
  /*  StarPipeHeadOut(1); */

  i_entry=0;
  i_file=0;
  j_file=0;
  while(1) {

/* get a profile */

    if(input_format==SDB_INPUT) {
      prf_input_status=GetScnUsingSdbEntry(argv[0],&profile,file_name,sdb_arg,&sdb_flag);
      if((sdb_flag==IDX_COMMAND && strncmp(sdb_arg,"!integ",6)==0) || prf_input_status==SDB_EOF) {
	if(i_file==0)
	  integ_out_flag=INTEG_SKIP;
	else
	  integ_out_flag=INTEG_EXE;
      } else if((sdb_flag==IDX_COMMAND && strncmp(sdb_arg,"!end",4)==0) || prf_input_status==SDB_EOF) {
	prf_input_status=SDB_EOF; /* EOF of pipe */
	integ_out_flag=INTEG_EXE;
      } else if(sdb_flag==GOOD_SCAN) {
	integ_out_flag=INTEG_STILL;
      } else {
	integ_out_flag=INTEG_SKIP;
      }

      i_entry++; /* add entry counter */

    } else {
      if(i_entry<n_entry) {
	err_code=StarProfileIn(&profile,&sdb_flag,sdb_arg);
	if(err_code==111) {
	  prf_input_status=SDB_EOF; /* EOF of pipe */
	  integ_out_flag=INTEG_EXE;
	} else {
	  prf_input_status=SDB_GOOD;
	  if(err_code==0) {
	    i_entry++; /* add entry counter */
	    integ_out_flag=INTEG_STILL;
	  }
	}
      } else {
	prf_input_status=SDB_EOF; /* EOF of pipe */
	integ_out_flag=INTEG_EXE;
      }
      integ_out_flag=INTEG_SKIP;
    }

/* set default name */
    if(i_file==0) {
      if(input_format==SDB_INPUT)
	strcpy(primary_filename,file_name);
      else
	strcpy(primary_filename,profile.file_name);
    }

/* make an averaged profile */
    if(integ_out_flag==INTEG_EXE) {
      err_code=AverageProcess(&integ_prof,&weight_prof,weight_mode,&history,i_file);
      if(err_code!=0) exit(err_code);

      strcpy(integ_prof.file_id,"STAR.scn");
      if(strcmp(default_filename,"")==0) {
	strcpy(integ_prof.file_name,primary_filename);
      } else {
	sprintf(integ_prof.file_name,"%s%d",default_filename,i_entry);
      }
      err_code=StarProfileOut(&integ_prof,GOOD_SCAN,"");

      if(err_code!=0) exit(err_code);

      i_file=0; /* reset integ file count */
      j_file++; /* add output file count */

      if(prf_input_status==SDB_EOF) {
	break;
      }

    } else if(integ_out_flag==INTEG_STILL) {
/* integrate */
      err_code=StarIntegrateProfile(i_file,&integ_prof,&profile,&weight_prof,weight_mode);

      if(err_code!=0) {
	fprintf(stderr,"%s: %s is not integratable with others because of ",argv[0],file_name);
	PrintIntegErrorMassage(err_code);
      } else {
	i_file++; /* add integ file count */

	if(i_file+1<MAX_HISTORY_LINES_PER_STEP)
	  strncpy(history.prm[i_file+1],file_name,16);

      }
    }
  }

/* pipeflow file ended */
  StarPipeEndOut();

  exit(0);
}
