/*
output integrated intensity in ASCII characters from PROFILE in the assigned velocity range

output format
   divided by tab
   1 line per 1 profile

column (left to right)
   file name (tab) good (tab) longitude (tab) latitude (tab) integ intensity (tab) v_min (tab) v_max

The units are
   longitude & latitude [degree]
   integrated intensity [K km/s], averaged Ta* [K]
   v_min,v_max [km/s]

The column order is chaged if v_step is not 0.
   file name (tab) good (tab) longitude (tab) latitide (tab) v_min (tab) v_max (tab) v_step (tab) integ intensity (tab) integ intensity (tab)...

The units are
   longitude & latitude [degree]
   v_min,v_max,v_step [km/s]
   integrated intensity [K km/s], averaged Ta* [K]

1996/12/19 T.Handa ver.1.50 : re-create
1996/12/19 T.Handa ver.1.51 : selectable integ intensity/average intensity, v_step option
1997/01/14 T.Handa ver.1.52 : text_column,text_table output mode
1998/ 5/ 8 T.Handa ver.3.00 : new library, .sdb
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"lineint.prm"
#define N_KEY 100

#define HUNNING	1
#define GAUSS	2
#define BOXCAR	3
#define BINNING	4
#define PINPOINT	5

#define	CH_CENTER 0
#define OUTER_EDGE 1
#define INNER_EDGE 2


#undef	DEBUG

#define	LINEINT1	1
#define	LINEINT2	2
#define	LINEINT3	3
#define	TEXT_COLUMN	11
#define TEXT_TABLE	12

int options(argc,argv, v_range,v_step,coordinate_flag,avr_flag,out_format)
/*
get parameters from options
1992/11/25 T.Handa
*/
int	argc;
char	*argv[];
RANGE	*v_range;	/* to integrate */
double	*v_step;
int	*coordinate_flag;
int	*avr_flag;
int	*out_format;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char buffer[MAX_PRM_RECORD_LENGTH];

  char *got_value;

/* default values */
  (*v_range).min=0.0;	(*v_range).max=10.0;
  *v_step=0.0;
  *coordinate_flag=COORD_LB;
  *avr_flag=0;
  *out_format=LINEINT3;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"v");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"v_range");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"v_step");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"v_stampling");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"v_width");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"v_smooth_kernel");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"v_smooth_method");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"v_start_type");	key_type[n_key-1]=1;

  n_key=AddNewWord(keyword,"coordinate");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"lb");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"radec");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"radec-app");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"azel");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"xy");	key_type[n_key-1]=0;

  n_key=AddNewWord(keyword,"outunit");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"avr");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"average");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"int");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"integrated");	key_type[n_key-1]=0;

  n_key=AddNewWord(keyword,"format");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* velocity full span */
  n_scan=2;
  scan_flag[0]=0; scan_flag[1]=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    StarOptionRange(got_value,v_range);
  }

/* volocity step */
  n_scan=2;
  scan_flag[0]=2; scan_flag[1]=3;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    *v_step=atof(got_value);
  }

/* volocity width */
  n_scan=1;
  scan_flag[0]=4;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    /*		*v_width=atof(got_value);*/
    *v_step=atof(got_value);
  }

/* smoothing kernal type */
/*	n_scan=2;
	scan_flag[0]=5; scan_flag[1]=6;
	got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
	if(got_value[0]!=(char)NULL) {
		if(strcmp(got_value,"hunning")==0)
			*kernel_type=HUNNING;
		else if(strcmp(got_value,"gauss")==0)
			*kernel_type=GAUSS;
		else if(strcmp(got_value,"box")==0)
			*kernel_type=BOXCAR;
		else if(strcmp(got_value,"boxcar")==0)
			*kernel_type=BOXCAR;
		else if(strcmp(got_value,"bin")==0)
			*kernel_type=BINNING;
		else if(strcmp(got_value,"pin")==0)
			*kernel_type=PINPOINT;
		else if(strcmp(got_value,"pinpoint")==0)
			*kernel_type=PINPOINT;
	}
*/
/* edge type */
/*	n_scan=1;
	scan_flag[0]=7;
	got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
	if(got_value[0]!=(char)NULL) {
		if(strcmp(got_value,"edge")==0 || strcmp(got_value,"outer_edge")==0)
			*edge_type=OUTER_EDGE;
		else if(strcmp(got_value,"inner_edge")==0)
			*edge_type=INNER_EDGE;
	}
*/
/* coordinate */
  scan_flag[0]= 8;
  scan_flag[1]= 9;
  scan_flag[2]=10;
  scan_flag[3]=11;
  scan_flag[4]=12;
  scan_flag[5]=13;
  n_scan=6;
  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    strcpy(buffer,StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value));
    if(strcmp(buffer,"lb")==0) {
      *coordinate_flag=COORD_LB;
    } else if(strcmp(buffer,"radec")==0) {
      *coordinate_flag=COORD_RADEC;
    } else if(strcmp(buffer,"radec-app")==0) {
      *coordinate_flag=COORD_RADEC_APP;
    } else if(strcmp(buffer,"azel")==0) {
      *coordinate_flag=COORD_AZEL;
    } else if(strcmp(buffer,"xy")==0) {
      *coordinate_flag=COORD_XY;
    }
  }

/* output unit */
  scan_flag[0]=14;
  scan_flag[1]=15;
  scan_flag[2]=16;
  scan_flag[3]=17;
  scan_flag[4]=18;
  n_scan=5;
  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    strcpy(buffer,StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value));
    if(strcmp(buffer,"avr")==0 || strcmp(buffer,"average")==0) {
      *avr_flag=1;
    } else if(strcmp(buffer,"int")==0 || strcmp(buffer,"integrated")==0) {
      *avr_flag=0;
    }
  }

/* output format */
  scan_flag[0]=19;
  n_scan=1;
  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    strcpy(buffer,StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value));
    if(strcmp(buffer,"lineint1")==0)
      *out_format=LINEINT1;
    else if(strcmp(buffer,"lineint2")==0)
      *out_format=LINEINT2;
    else if(strcmp(buffer,"lineint3")==0)
      *out_format=LINEINT3;
    else if(strcmp(buffer,"text_column")==0)
      *out_format=TEXT_COLUMN;
    else if(strcmp(buffer,"text_table")==0)
      *out_format=TEXT_TABLE;
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

static double StarIntegratedIntensity(profile,v_range,avr_flag)
PROFILE *profile;
RANGE *v_range;
int avr_flag;
{
  RANGE v_for_ch;
  double ta,v_all;
  double scale;

  double v_width;
  int ch,data_ch;

  ta=0.0;
  v_all=0.0;
  scale=(double)(*profile).rx_status.scale;
  data_ch=(*profile).be_status.data_ch;

/* integrate over the assigned range */
  for(ch=1;ch<=data_ch;ch++) {
    v_for_ch.min=StarCh2Velocity(profile,(double)ch-0.5);
    v_for_ch.max=StarCh2Velocity(profile,(double)ch+0.5);
    StarSortRange(&v_for_ch);
/* integrate only non-spurious */
    if(StarReadSpuriousFlag(profile,ch)==UNFLAGGED) {
/* integration range */
      if(v_for_ch.min>=(*v_range).min && v_for_ch.max<=(*v_range).max) {
	/* whole width of a channel is in the range */
	v_width=v_for_ch.max-v_for_ch.min;
	ta=ta+(double)(*profile).data[ch-1]*v_width*scale;
	v_all=v_all+v_width;
      } else if(v_for_ch.min<(*v_range).min && v_for_ch.max>(*v_range).max) {
	/* the range is in a whole channel width */
	v_width=(*v_range).max-(*v_range).min;
	ta=ta+(double)(*profile).data[ch-1]*v_width*scale;
	v_all=v_all+v_width;
      } else if(v_for_ch.min<=(*v_range).min && v_for_ch.max>(*v_range).min) {
	/* v_min is in a middle of a channel */
	v_width=v_for_ch.max-(*v_range).min;
	ta=ta+(double)(*profile).data[ch-1]*v_width*scale;
	v_all=v_all+v_width;
      } else if(v_for_ch.min<=(*v_range).max && v_for_ch.max>(*v_range).max) {
	/* v_max is in a middle of a channel */
	v_width=(*v_range).max-v_for_ch.min;
	ta=ta+(double)(*profile).data[ch-1]*v_width*scale;
	v_all=v_all+v_width;
      }
    }

  } /* end if for spurious treatment */
  if(avr_flag!=0) {
    if(v_all!=0.0) ta=ta/v_all;
    else ta=0.0;
  }

  return(ta);
}

static void IntegTable(profile,coordinate_flag,avr_flag,v_range,v_step,out_format)
PROFILE *profile;
int  coordinate_flag;
int	avr_flag;
RANGE	v_range;
double	v_step;
int	out_format;
{
  RANGE	v_bin;
  FILE	*out_p;
  XY_POS	on_pos;
  double	ta;

  int	ii,ii_max;

  out_p=stdout;

/* get the coordinate position */
  if(coordinate_flag==COORD_RADEC) {
    on_pos.x=(*profile).object.on_radec.x;
    on_pos.y=(*profile).object.on_radec.y;
  } else if(coordinate_flag==COORD_AZEL) {
    on_pos.x=(*profile).object.on_azel.x;
    on_pos.y=(*profile).object.on_azel.y;
  } else if(coordinate_flag==COORD_XY) {
    on_pos.x=(*profile).object.on_xy.x;
    on_pos.y=(*profile).object.on_xy.y;
  } else {
    on_pos.x=(*profile).object.on_lb.x;
    on_pos.y=(*profile).object.on_lb.y;
  }

/* estimate the integrated intensity */
  if(v_step==0.0) {
    ta=StarIntegratedIntensity(profile,&v_range,avr_flag);

/* Pipe-out the integrated intensity data */
    if(out_format==TEXT_COLUMN) {
      fprintf(out_p,"%f\t%f\t%f",on_pos.x,on_pos.y,
	      (v_range.max+v_range.min)/2.0);
      fprintf(out_p,"\t%f",ta);
    } else if(out_format==TEXT_TABLE) {
      fprintf(out_p,"\t%f",ta);
    } else {
      fprintf(out_p,"%s\tgood\t%f\t%f",(*profile).file_name,on_pos.x,on_pos.y);
      fprintf(out_p,"\t%f",ta);
      fprintf(out_p,"\t%f\t%f\n",v_range.min,v_range.max);
    }
  } else {
    ii_max=(int)((v_range.max-v_range.min)/v_step);
    if(out_format==LINEINT3) {
      fprintf(out_p,"%s\tgood\t%f\t%f",(*profile).file_name,on_pos.x,on_pos.y);
      fprintf(out_p,"\t%f\t%f\t%f",v_range.min,v_range.max,v_step);
      for(ii=0;ii<ii_max;ii++) {
	v_bin.min=v_range.min+v_step*(double)ii;
	v_bin.max=v_bin.min+v_step;
	ta=StarIntegratedIntensity(profile,&v_bin,avr_flag);
	fprintf(out_p,"\t%f",ta);
      }
      fprintf(out_p,"\n");
    } else if(out_format==LINEINT1 || out_format==LINEINT2) {
      for(ii=0;ii<ii_max;ii++) {
	fprintf(out_p,"%s\tgood\t%f\t%f",(*profile).file_name,on_pos.x,on_pos.y);
	v_bin.min=v_range.min+v_step*(double)ii;
	v_bin.max=v_bin.min+v_step;
	ta=StarIntegratedIntensity(profile,&v_bin,avr_flag);
	fprintf(out_p,"\t%f",ta);
	fprintf(out_p,"\t%f\t%f\n",v_bin.min,v_bin.max);
      }
    } else if(out_format==TEXT_COLUMN) {
      fprintf(out_p,"%f\t%f\t%f",on_pos.x,on_pos.y,(v_range.max+v_range.min)/2.0);
      for(ii=0;ii<ii_max;ii++) {
	v_bin.min=v_range.min+v_step*(double)ii;
	v_bin.max=v_bin.min+v_step;
	ta=StarIntegratedIntensity(profile,&v_bin,avr_flag);
	fprintf(out_p,"\t%f",ta);
      }
      fprintf(out_p,"\n");
    } else if(out_format==TEXT_TABLE) {			
      for(ii=0;ii<ii_max;ii++) {
	v_bin.min=v_range.min+v_step*(double)ii;
	v_bin.max=v_bin.min+v_step;
	ta=StarIntegratedIntensity(profile,&v_bin,avr_flag);
	if(ii==0) fprintf(out_p,"%f",ta);
	else fprintf(out_p,"\t%f",ta);
      }
      fprintf(out_p,"\n");
    }
  }
  return;
}

int profile_load(scnname,profile,cmd)
char *scnname;
PROFILE *profile;
char *cmd;
{
  char full_filename[MAX_FILE_NAME_LENGTH];
  FILE *fp;
  int err_code;

/* add STD extension for scan */
  err_code=StarStdFileExtName("STAR_DATA",scnname,full_filename,".scn");
  if(err_code!=0) {
    fprintf(stderr,"%s:Too long file name\n",cmd);
    return(1);
  }
  fp=fopen(full_filename,"r");
  if(fp==NULL) {
    fprintf(stderr,"%s:Cannot open file=%s\n",cmd,full_filename);
    return(2);
  }
  StarProfileLoad(fp,profile);
  fclose(fp);

  StarReduceString((*profile).obs_prm.project);
  StarReduceString((*profile).obs_prm.scd_file);
  StarReduceString((*profile).obs_prm.observer1);
  StarReduceString((*profile).obs_prm.observer2);
  StarReduceString((*profile).obs_prm.observer3);
  StarReduceString((*profile).obs_prm.observer4);
  StarReduceString((*profile).object.name);
  StarReduceString((*profile).ant_status.tele);
  StarReduceString((*profile).rx_status.name);
  StarReduceString((*profile).be_status.name);
  StarReduceString((*profile).obs_log.site_name);
  StarReduceString((*profile).obs_log.memo);
  StarReduceString((*profile).obs_log.comment);

  return(0);
}


int main(argc,argv)
int argc;
char *argv[];
{
  PROFILE profile;

  char	default_sdbname[MAX_FILE_NAME_LENGTH];
  char	scnname[MAX_FILE_NAME_LENGTH];
  char argument[MAX_IDX_RECORD_LENGTH];

  int	sdb_flag;

  int	ii,n_line,err_code;
  int	backup_ver;

  RANGE	v_range;
  double	v_step;
  int	coordinate_flag,avr_flag,out_format;

/* get options */
  err_code=options(argc,argv, &v_range,&v_step,&coordinate_flag,&avr_flag,&out_format);


/* .sdb header */
  err_code=StarSdbHeadIn(default_sdbname,&n_line,&backup_ver);
  if(err_code!=0) {
    fprintf(stderr,"%s: Broken Pipe. Not a sdb file.\n",argv[0]);
    exit(1);
  } else if(n_line==0) {
    fprintf(stderr,"%s: Empty sdb flow.\n",argv[0]);
    exit(1);
  }

/* header lines in text output */
  if(out_format==TEXT_COLUMN) {
    fprintf(stdout,"sdb_file=%s\tx_pos\ty_pos\tvelocity",
	    default_sdbname);
    if(v_step!=0.0)
      fprintf(stdout," min=%f\tmax=%f\tstep=%f",
	      v_range.min,v_range.max,v_step);
    if(avr_flag!=0)
      fprintf(stdout,"Ta* (K)");
    else
      fprintf(stdout,"I (K km/s)");
    fprintf(stdout,"\n");
  } else if(out_format==TEXT_TABLE) {
    fprintf(stdout,"sdb_file=%s\tx_pos\ty_pos\t",default_sdbname);
    fprintf(stdout,"v min=%f\tv max=%f\tv step=%f",v_range.min,v_range.max,v_step);
    if(avr_flag!=0)
      fprintf(stdout,"Ta* (K)");
    else
      fprintf(stdout,"I (K km/s)");
    fprintf(stdout,"\n");
  }

/* main loop */
  for(ii=0;ii<n_line;ii++) {
    err_code=StarSdbIn(scnname,&sdb_flag,argument);
    switch(err_code) {
    case 201: case 301:
      sdb_flag=GOOD_SCAN;	/* without flag means "good" */
      err_code=0;
      break;
    case 100:
      goto loopend;	/* EOF */
    }

    if(sdb_flag==GOOD_SCAN) {
      err_code=profile_load(scnname,&profile,argv[0]);
      strcpy(profile.file_name,scnname);
      if(err_code==0)
	IntegTable(&profile,coordinate_flag,avr_flag,v_range,v_step,out_format);
      else
	fprintf(stderr,"%s: No.%d entry is invarid. name=%s, arg=%s\n",argv[0],ii,scnname,argument);
    }
  }
 loopend:;

  exit(0);
}
