/*
profile data loader into stdout

1992/4/17 T.Handa : ver.1.00
1994/7/6 Tomo OKA : ver.1.40 search also StarREF_DATA directory
1997/11/28 T.Handa : ver.3.00 multi_pipe and new library
1998/3/19 T.Handa : ver.3.01 (debug) for Solaris2 fcc
1998/4/ 3 T.Handa : ver.3.02 (debug) add include files
1998/4/23 T.Handa : ver.3.20 (revise) .scn,.sdb
1999/7/27 T.Handa : ver.3.31 (revise) alias sdb2scn
1999/7/30 T.Handa : ver.3.32 (revise) for ver4 scn
2006/2/ 7 T.Handa : ver.3.33 (modify) corrected for VST1 2SB RX before scan<=0006607 and MJD>=53610
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"load.prm"
#define N_KEY 10

int options(argc,argv,auto_crr)
/*
get parameters from options
*/
int argc;
char *argv[];
int *auto_crr;
{
  int ii,next_pos;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int scan_flag[N_KEY], n_scan;

  int correction_flag;

/* default values */
  correction_flag=1; /* auto correction = yes */
  *auto_crr=correction_flag;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"correction");
  n_key=AddNewWord(keyword,"nocorrection");
  n_key=AddNewWord(keyword,"input_file");

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

  next_pos=0;
/* correction or not */
  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {    
    scan_flag[ii]=next_pos+ii;
    if(ii==1) scan_flag[ii]= -scan_flag[ii];
  }
  next_pos=next_pos+n_scan;

  correction_flag=StarOptionFlag(n_scan,scan_flag,key_flag);
  if(correction_flag!=0) *auto_crr=correction_flag;

  scan_flag[0]=2;
  

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

void auto_correction(prf)
PROFILE *prf;
/*
correction headers for the latest version
1999/7/23 T.Handa
2006/2/ 7 T.Handa (modify) corrected for VST1 2SB RX
*/
{
  if((*prf).sys_head.format_id==SPECTRUM && (*prf).sys_head.version<SCAN_FORMAT_VERSION){
    switch((*prf).sys_head.version){
    case 1:	/* +4byte for rx_status.scale */
      (*prf).sys_head.version=2;
      (*prf).sys_head.l_file=(*prf).sys_head.l_file+4;
      (*prf).sys_head.l_head=(*prf).sys_head.l_head+4;
    case 2:	/* +16byte for rx_status.v_off, be_status.center_ch */
      (*prf).sys_head.version=3;
      (*prf).sys_head.l_file=(*prf).sys_head.l_file+16;
      (*prf).sys_head.l_head=(*prf).sys_head.l_head+16;
    case 3:	/* -31byte for dummy data reduced */
      (*prf).sys_head.version=4;
      (*prf).sys_head.l_file=(*prf).sys_head.l_file-23;
      (*prf).sys_head.l_head=(*prf).sys_head.l_head-23;
    case 4:	/* reverse be.sign for LSB data */
      (*prf).sys_head.version=5;
      if(((*prf).rx_status.sb_mode & USB_LSB_MASK) == 0) {
	(*prf).be_status.sign= -(*prf).be_status.sign;
      }
    }
  }

  if((*prf).rx_status.rest_freq==0.0)
    (*prf).rx_status.rest_freq=(*prf).rx_status.obs_freq;

  if((*prf).sys_head.version<=4 && strncmp((*prf).ant_status.tele,"VST1",4)==0 && (*prf).obs_prm.mjd>=53610 && (*prf).sys_head.scan<=6702) {
    if(((*prf).rx_status.sb_mode & USB_LSB_MASK) == 0)
      (*prf).be_status.sign= -(*prf).be_status.sign;
  }


  return;
}

int profile_file2pipe(argc,argv, default_name,filename,auto_crr,sdb_flag,sdb_arg)
int argc;
char *argv[];
char *default_name;
char *filename;
int auto_crr;
int sdb_flag;
char *sdb_arg;
{
  FILE *fp;
  PROFILE profile;
  int err_code;

  if(sdb_flag!=GOOD_SCAN && sdb_flag!=BAD_SCAN) {
    strcpy(profile.file_id,STAR_SCN_IN);
    strcpy(profile.file_name,"#!");
    err_code=StarProfileOut(&profile,sdb_flag,sdb_arg);
    return(0);
  }

/* open scan file */
  fp=fopen(filename,"r");
  if(fp==NULL) {
    fprintf(stderr,"%s : not open file %s\n",argv[0],filename);
    return(ERR_UNEXIST_FILE);
  }

/* load a scan profile */
  err_code=StarProfileLoad(fp,&profile);
  if(err_code!=0) {
    /* debug for "END..." c-strings debug for ASTROS ver.1.31,1.32 */
    if(err_code==111) {
      fprintf(stderr,"the last history is not END\n");
    } else {
      switch(err_code) {
      case 1:
	fprintf(stderr,"Garbage file! Broken PIPE.\n");
	break;
      case 101:
	fprintf(stderr,"too long history\n");
	break;
      case 102:
	fprintf(stderr,"too many steps in a history\n");
	break;
      default:
	fprintf(stderr,"Invarid file\n");
      }
      profile.sys_head.l_file=0;	/* garbage file */
      err_code=StarProfileOut(&profile,(char)NULL,"");
      exit(11);
    }
  }
  fclose(fp);
  strcpy(profile.file_id,STAR_SCN_IN);
  strcpy(profile.file_name,default_name);

/* auto recover header error */
  if(auto_crr==1) {
    auto_correction(&profile);
  }

/* output a scan profile to the PIPE */
  err_code=StarProfileOut(&profile,sdb_flag,sdb_arg);

  return(0);
}

char *command_main(command)
char *command;
/*
extract main part of the filename if it is assigned in absorute pass name.
1999/7/27 T.Handa
*/
{
  int l_str,ii,i_top;

  i_top=0;
  l_str=strlen(command);
  for(ii=l_str;ii>=0;ii--) {
    if(command[ii]=='/') {
      i_top=ii+1;
      break;
    }
  }
  return(&(command[i_top]));
}

int main(argc,argv)
int argc;
char *argv[];
{
  char **input_filename,**sdb_argument;
  int sdb_flag[MAX_FILE_NUMBER];
  char full_filename[MAX_FILE_NAME_LENGTH];
  char default_sdbfile[MAX_FILE_NAME_LENGTH];

  int ii,jj,kk;
  int n_pipe,n_file;
  int err_code;
  int auto_correction_flag;
  int stdin_flag;
  int n_line,backup_ver;

/* check option */
  err_code=options(argc,argv,&auto_correction_flag);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

  sdb_argument=AllocAllWords(MAX_FILE_NUMBER,MAX_IDX_RECORD_LENGTH);

/* get input_file names */
  input_filename=AllocAllWords(MAX_FILE_NUMBER,MAX_FILE_NAME_LENGTH);

  n_file=0;
  stdin_flag=0;

/* command as sdb2scn */
  if(strcmp(command_main(argv[0]),"sdb2scn")==0) {
/* pending for .sdb multi */
/*    StarPipeHeadIn(id_str,&n_pipe);*/n_pipe=1;
    for(jj=0;jj<n_pipe;jj++) {
      err_code=StarSdbHeadIn(default_sdbfile,&n_line,&backup_ver);
      for(kk=0;kk<n_line;kk++) {
	err_code=StarSdbIn(input_filename[n_file],&(sdb_flag[n_file]),sdb_argument[n_file]);
	if(err_code==ERR_EOF_DETECT) break;
	n_file++;
	if(n_file>MAX_FILE_NUMBER) 
	  fprintf(stderr,"%s:Too many files\n",argv[0]);
      }
    }
  }

/* command as load */
  for(ii=1;ii<argc;ii++) {
    if(strcmp(argv[ii],"-")==0) {
/* pending for .sdb multi */
/*      StarPipeHeadIn(id_str,&n_pipe);*/n_pipe=1;
      for(jj=0;jj<n_pipe;jj++) {
	err_code=StarSdbHeadIn(default_sdbfile,&n_line,&backup_ver);
	for(kk=0;kk<n_line;kk++) {
	  err_code=StarSdbIn(input_filename[n_file],&(sdb_flag[n_file]),sdb_argument[n_file]);
	  if(err_code==ERR_EOF_DETECT) break;
	  n_file++;
	  if(n_file>MAX_FILE_NUMBER) 
	    fprintf(stderr,"%s:Too many files\n",argv[0]);
	}
      }
    } else if(argv[ii][0]!='-') {
      strcpy(input_filename[n_file],argv[ii]);
      sdb_flag[n_file]=GOOD_SCAN;
      strcpy(sdb_argument[n_file],"");
      n_file++;
      if(n_file>MAX_FILE_NUMBER) {
	fprintf(stderr,"%s:Too many files\n",argv[0]);
      }
    }
  }

/* pipeout header */
  StarPipeHeadOut(n_file);

  for(ii=0;ii<n_file;ii++) {
/* add STD extension for scan */
    err_code=StarStdFileExtName("STAR_DATA",input_filename[ii],full_filename,".scn");

    if(err_code!=0) {
      fprintf(stderr,"%s:Too long file name\n",argv[0]);
    }

/* load profiles and flow out into the pipeline */
    err_code=profile_file2pipe(argc,argv,input_filename[ii],full_filename,auto_correction_flag,sdb_flag[ii],sdb_argument[ii]);
    if(err_code!=0) {
      switch(err_code) {
      case ERR_UNEXIST_FILE:
	fprintf(stderr,"%s: unexist file=%s\n",
		argv[0],full_filename);
	break;
      }
    }

  }
  StarPipeEndOut();
  FreeAllWords(input_filename,MAX_FILE_NUMBER);
  FreeAllWords(sdb_argument,MAX_FILE_NUMBER);
  exit(0);
}
