/*
Show many profiles on X11 simultenuously

1998/3/30 T.Handa : ver.3.00 from display.c(v3.01 1998/3/19)
1998/4/ 8 T.Handa : ver.3.20 (revise) .scn,.sdb
1998/4/28 T.Handa : ver.3.21 (debug) revise x-axis treat, .sdb flow 
1998/5/18 T.Handa : ver.3.22 (revise) show base/spur range
2006/1/31 T.Handa : ver.3.30 (modify) -rf, -bf
2006/3/21 T.Handa : ver.3.31 (modify) -n_scn
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include <Xm/DrawingA.h>
#include <Xm/MainW.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#include <Xm/PushBG.h>

#include <StData.h>
#include <StStd.h>

#include <Pxxlib.h>
#include <Pxplib.h>
#include <Ppslib.h>
#include <PxpGlib.h>
#include <StPxpShow.h>

#define	DEFAULT_PRMFILE	"display.prm"
#define N_KEY 40
#define MAX_SCN 20

static PROFILE GLprofs[MAX_SCN];
static int GLsdb_flag[MAX_SCN];
static char GLsdb_arg[MAX_SCN][MAX_IDX_RECORD_LENGTH];
static int GLn_file=0;
static int GLpipeout= -1;
static int GLbase_show,GLspur_show;
static int GLx_unit;
static RANGE GLx_range;
static RANGE GLy_range;

static int GLn_column;
static int GLn_raw;
static int GLoverlay;
static int GLn_scn1panel;
static int GLn_scn;

/*-------Ext------*/
/*--------from Pxxlib--------------------*/
static void	(*do_plot)(), (*do_exit)(), (*do_help)();


/* define button action on the menubar */
void local_StarPxxPushButtonActions(menubarWDG,client_data,call_data)
Widget	menubarWDG;
caddr_t	client_data,call_data;
/*
define actions of button on the menubar
1997/10/12 T.Handa ver.1.00
1997/12/12 T.Handa ver.1.00 for display
*/
{
  if(strcmp(client_data,"Exit")==0) {
    XClearWindow(PxxGetDisplay(),PxxGetWindow());
    if(do_exit!=NULL) (*do_exit)();
    if(do_plot!=NULL) (*do_plot)();
  }
  else if(strcmp(client_data,"Quit")==0) {
    exit(0);
  }
  else if(strcmp(client_data,"Print")==0)
    PxxExecPrintOut();
  else if(strcmp(client_data,"Help")==0)
    if(do_help!=NULL) (*do_help)();
}

#define MAX_ARG 100
#define	CHAR_SIZE_DEFAULT_RESOURCE	128

/* show on X */
void local_StarPxxDisplayAll(argc,argv, 
	app_class,width,height,plt_fnct,exit_fnct,help_fnct)
/*
show X11 window in a default style
1997/10/12 T.Handa ver.1.00
1997/12/02 T.Handa ver.1.00 for display
1998/ 5/ 6 T.Handa ver.1.02 (debug)
*/
int argc;
char *argv[];
char app_class[];	/* name of application class (used in .Xresource) */
double width,height;
void (*plt_fnct)(),(*exit_fnct)(),(*help_fnct)();
{
  Widget  toplevelWDG, main_windowWDG, drawing_areaWDG, manu_barWDG, mbBtn[4];
  Arg     args[10];
  int     ii;
  GC      graphic_context;

  int	mainwin_x,mainwin_y;

  int i_btn_id;

  XtAppContext    app_context;
  Arg fixed_res_args[MAX_ARG];     /* resource arguments */
  Cardinal        fixed_res_n_args;
  static char	window_w[CHAR_SIZE_DEFAULT_RESOURCE],window_h[CHAR_SIZE_DEFAULT_RESOURCE];
  static String   fb_res[] = {
    "*mainWindow.width:        500 ", /* NEVER change the character length and position */
    "*mainWindow.height:       300 ", /* NEVER change the character length and position */
    "*drawArea.background:	white",
    "*menubar.topAttachment:	ATTACH_FORM",
    "*menubar.rightAttachment:	ATTACH_FORM",
    "*menubar.leftAttachment:	ATTACH_FORM",
    NULL};
/* add for display */
  static char psfile_name[]="";
  static char prt_name[]="";

/* Change initial window size */
  mainwin_x=(int)(width*CM2INCH*DVI_SCALE+0.5);
  mainwin_y=(int)(height*CM2INCH*DVI_SCALE+0.5);
  if(mainwin_x>0.0 && mainwin_x<9999999.0) {
    sprintf(window_w,"*mainWindow.width: %8d",mainwin_x);
    fb_res[0]=window_w;
  }
  if(mainwin_y>0.0 && mainwin_y<9999999.0) {
    sprintf(window_h,"*mainWindow.height:%8d",mainwin_y);
    fb_res[1]=window_h;
  }

/*	PxxView_width=width; */
/*	PxxView_height=height; */
/*	PxxView_printer[0]=(char)NULL; */	/* set default printer */
/*	PxxView_psfile[0]=(char)NULL;*/	/* set temporaly file */
/* add for display */
  PxxPutWindowSizeInPixel(width,height);
  PxxSetPrinterName(prt_name);
  PxxSetPSFName(psfile_name);

/* printer with option -P */
  for(ii=1;ii<argc;ii++) {
    if(strcmp(argv[ii],"-P")==0) PxxSetPrinterName(argv[ii+1]);
    else if(argv[ii][0]=='-' && argv[ii][1]=='P')
      PxxSetPrinterName(&(argv[ii][2]));
  }

/* PS filename for option -F */
  for(ii=1;ii<argc;ii++) {
    if(strcmp(argv[ii],"-F")==0) PxxSetPSFName(argv[ii+1]);
    else if(argv[ii][0]=='-' && argv[ii][1]=='F')
      PxxSetPSFName(&(argv[ii][2]));
  }

/* define three functions; plot,exit,help */
  PxxSetPlotFunction(plt_fnct);
  PxxSetExitFunction(exit_fnct);
  PxxSetHelpFunction(help_fnct);
/* add for display */
  do_plot=plt_fnct;
  do_exit=exit_fnct;
  do_help=help_fnct;

/* create window */
  fixed_res_n_args=0;
  toplevelWDG=XtAppInitialize(&app_context,app_class, NULL, 0, &argc, argv,fb_res,fixed_res_args,fixed_res_n_args);

  main_windowWDG=XmCreateMainWindow(toplevelWDG,"mainWindow",NULL,(Cardinal)NULL);
  XtManageChild(main_windowWDG);

/* create menubar */
  manu_barWDG=XmCreateMenuBar(main_windowWDG,"menubar",NULL,(Cardinal)NULL);
  XtManageChild(manu_barWDG);
  i_btn_id=0;

  mbBtn[i_btn_id]=XmCreateCascadeButton(manu_barWDG,"help", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, (XtCallbackProc)local_StarPxxPushButtonActions,"Help");
  i_btn_id++;

  mbBtn[i_btn_id]=XmCreateCascadeButton(manu_barWDG,"next", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, (XtCallbackProc)local_StarPxxPushButtonActions, "Exit");
  i_btn_id++;
  mbBtn[i_btn_id]=XmCreateCascadeButton(manu_barWDG,"print", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, (XtCallbackProc)local_StarPxxPushButtonActions, "Print");
  i_btn_id++;
  mbBtn[i_btn_id]=XmCreateCascadeButton(manu_barWDG,"quit", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, (XtCallbackProc)local_StarPxxPushButtonActions, "Quit");
  i_btn_id++;
  XtManageChildren(mbBtn,i_btn_id);

  XtSetArg(args[0],XmNmenuHelpWidget,(XtArgVal)mbBtn[0]);
  XtSetValues(manu_barWDG,args,1);

/* creat drawing area */
  drawing_areaWDG=XmCreateDrawingArea(main_windowWDG,"drawArea",args,0);
  XtManageChild(drawing_areaWDG);

/* design window location */
  XmMainWindowSetAreas(main_windowWDG,manu_barWDG,NULL,NULL,NULL,drawing_areaWDG);

/* realize window */
  XtRealizeWidget(toplevelWDG);

  graphic_context=XCreateGC(XtDisplay(drawing_areaWDG),XtWindow(drawing_areaWDG),0,0);

  XtAddCallback(drawing_areaWDG,XmNexposeCallback,(XtCallbackProc)PxxExecDraw,graphic_context);

  XtAddCallback(drawing_areaWDG,XmNresizeCallback,(XtCallbackProc)PxxExecDraw,graphic_context);

/* main loop */
  XtAppMainLoop(app_context);
}

/*--------from Pxplib--------------------*/
/* execute X11 or PostScript output */
void local_StarPxpDisplayAll(argc,argv, output, filename,
	app_class,width,height,plt_fnct,exit_fnct,help_fnct)
/*
create a X11 window or a PostScript file in a default format

1997/10/14 T.Handa ver.1.00
1997/12/02 T.Handa ver.1.00 for display
*/
int argc;
char *argv[];
int output;
char *filename;	/* stdout for - or NULL name, work file for filename==NULL */
char app_class[];	/* application class (used in .Xresource) */
double width,height;
void (*plt_fnct)(),(*exit_fnct)(),(*help_fnct)();
{
  void (*plot_function)();

  plot_function=plt_fnct;
/*  PxpView_output=output;*/
  PxpPutOutputMode(output);

  switch(output) {
  case X11WIN:
    local_StarPxxDisplayAll(argc,argv, app_class,width,height,plt_fnct,exit_fnct,help_fnct);
    break;
  case POSTSCRIPT:
    PpsInitialize(filename,width,height);
    plot_function();	/* user defined plot function */
    PpsPrintPage();
    PpsDoPrint();
  }

  return;
}

/*---------------------------------*/

int options(argc,argv, w_height,w_width,x_unit,x_range,T_range,pipe_flag,bs_flag,sp_flag,n_column,n_raw,overlay_flag,n_scn1panel)
/*
get parameters from options
*/
int argc;
char *argv[];
double	*w_height, *w_width;	/* size of the window(cm) */
int	*x_unit;
RANGE	*x_range,*T_range;
int	*pipe_flag;
int *bs_flag; /* show baseline range ? */
int *sp_flag; /* show spurious range ? */
int	*n_column,*n_raw;	/* profiles in a page */
int	*overlay_flag;	/* overlay drawing */
int *n_scn1panel; /* overlayed scn in a panel */
{
  int ii,next_pos;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *token;
  int opt_pos;
  char buffer[MAX_PRM_RECORD_LENGTH];

  int tmp_flag;
  double window_x,window_y;

  char *got_value;
  static int default_n_column=4, default_n_raw=3;
  static double default_h[2]={20.0,15.0}, default_w[2]={30.0,20.0}; /* [0]multi-panel, [1]overlay */

/* default values */
  *w_height=default_h[0];	*w_width=default_w[0];
  *x_unit=X_CHANNEL;
  (*x_range).min=0.0;	(*x_range).max=0.0;
  (*T_range).min=0.0;	(*T_range).max=0.0;
  *pipe_flag= -1;
  *bs_flag= 0; *sp_flag= 0;
  *n_column=default_n_column; *n_raw=default_n_raw;
  *overlay_flag=UNFLAGGED;
  *n_scn1panel=0;
  window_x=7.0;
  window_y=5.0;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"size");	key_type[n_key-1]=1;

  n_key=AddNewWord(keyword,"x_unit");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"x");	key_type[n_key-1]=1;

  n_key=AddNewWord(keyword,"ch");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"v");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"lsr");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"lsr");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"rf");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"bf");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"backend_freq");	key_type[n_key-1]=1;

  n_key=AddNewWord(keyword,"T");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y");	key_type[n_key-1]=1;

  n_key=AddNewWord(keyword,"baseline");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"nobaseline");	key_type[n_key-1]=0;

  n_key=AddNewWord(keyword,"spurious");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"nospurious");	key_type[n_key-1]=0;

  n_key=AddNewWord(keyword,"x_number");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"n_column");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y_number");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"n_raw");	key_type[n_key-1]=1;

  n_key=AddNewWord(keyword,"overlay");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"n_scn");    key_type[n_key-1]=1;

  n_key=AddNewWord(keyword,"pipe");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"-");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"nopipe");	key_type[n_key-1]=0;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* window size */
  n_scan=1;
  scan_flag[0]=0;
  next_pos=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(buffer,got_value);
    token=strtok(buffer,"xX ,\t\n");
    if(token!=NULL) {
      window_x=atof(token);
      token=strtok(NULL," ,\t\n");
      if(token!=NULL) window_y=atof(token);
    }
    *w_height=window_y;
    *w_width=window_x;
  }
/* x_unit and range */
  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strcmp(got_value,"v")==0
       || strcmp(got_value,"lsr")==0
       || strcmp(got_value,"velocity")==0
       || strcmp(got_value,"lsr")==0
       || strcmp(got_value,"LSR")==0)
      *x_unit=X_VELOCITY;
    else if(strcmp(got_value,"ch")==0
	    || strcmp(got_value,"channel")==0
	    || strcmp(got_value,"ch")==0)
      *x_unit=X_CHANNEL;
  }

  n_scan=1;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    StarOptionRange(got_value,x_range);
  }
/* ch,v,lsr */
  n_scan=6;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    if(strcmp(keyword[opt_pos],"v")==0
       || strcmp(keyword[opt_pos],"lsr")==0
       || strcmp(keyword[opt_pos],"velocity")==0
       || strcasecmp(keyword[opt_pos],"lsr")==0)
      *x_unit=X_VELOCITY;
    else if(strcmp(keyword[opt_pos],"ch")==0
	    || strcmp(keyword[opt_pos],"channel")==0)
      *x_unit=X_CHANNEL;
    else if(strcmp(keyword[opt_pos],"rf")==0)
      *x_unit=X_RF_FREQ;
    else if(strncmp(keyword[opt_pos],"b",1)==0)
      *x_unit=X_BE_FREQ;
    StarOptionRange(value[opt_pos],x_range);
  } 

/* y_range */
  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(buffer,got_value);
    StarOptionRange(buffer,T_range);
  } 

/* show baseline range flag */
  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
    if(ii==1) scan_flag[ii]= -scan_flag[ii];
  }
  next_pos=next_pos+n_scan;
  tmp_flag=StarOptionFlag(n_scan,scan_flag,key_flag);
  if(tmp_flag!=0) *bs_flag=tmp_flag;

/* show spurious range flag */
  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
    if(ii==1) scan_flag[ii]= -scan_flag[ii];
  }
  next_pos=next_pos+n_scan;
  tmp_flag=StarOptionFlag(n_scan,scan_flag,key_flag);
  if(tmp_flag!=0) *sp_flag=tmp_flag;

/* columns */
  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    *n_column=atoi(got_value);
    if(*n_column<=0) *n_column=default_n_column;
  }

/* raws */
  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    *n_raw=atoi(got_value);
    if(*n_raw<=0) *n_raw=default_n_raw;
  }

/* overlay */
  n_scan=1;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(got_value[0]=='n' || got_value[0]=='N')
      *overlay_flag=UNFLAGGED;
    else if(got_value[0]=='y' || got_value[0]=='Y' || got_value[0]=='-')
      *overlay_flag=FLAGGED;
  }

  if(*overlay_flag==UNFLAGGED) {
    *w_height=default_h[0];
    *w_width=default_w[0];
  } else {
    *n_raw=MAX_SCN;
    *n_column=1;
    *w_height=default_h[1];
    *w_width=default_w[1];
  }

/* overlay and multi-panel */
  n_scan=1;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(*overlay_flag==FLAGGED) {
      *n_scn1panel=atoi(got_value);
    } else {
      *n_scn1panel=0;
    }
  }

/* pipe */
  n_scan=3;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
    if(ii==2) scan_flag[ii]= -scan_flag[ii];
  }
  next_pos=next_pos+n_scan;
  tmp_flag=StarOptionFlag(n_scan,scan_flag,key_flag);
  if(tmp_flag!=0) *pipe_flag=tmp_flag;

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);
/* set return values */

  return(0);
}


/* Main Body of process */

void read_profile()
{
  static int i_file=0;
  int n_file,n_act_file,pipeout;
  int err_code;

  int n_a_page;
  int ii;

/* from global valuables */
  n_file=GLn_file;	/* total files in a flow */
  pipeout=GLpipeout;

  n_a_page=GLn_column*GLn_raw;
  if(n_a_page>MAX_SCN) {
    n_a_page=MAX_SCN;
    GLn_raw=(n_a_page-1)/GLn_column+1;
  }

  if(i_file>=n_file) {
    if(pipeout>0) StarPipeEndOut();
    exit(0);
  }

/* read for a page */
  for(ii=0,n_act_file=0;n_act_file<n_a_page;ii++) {
    while(1) {
      err_code=StarProfileIn(&(GLprofs[ii]),&(GLsdb_flag[ii]),&(GLsdb_arg[ii][0]));
      if(err_code!=0) {
	fprintf(stderr,"fail to ProfileIn\n");
	exit(1);
      }
      i_file++;
	

      if(pipeout>0) StarProfileOut(&(GLprofs[ii]),GLsdb_flag[ii],&(GLsdb_arg[ii][0]));

      if(GLsdb_flag[ii]==GOOD_SCAN || GLsdb_flag[ii]==BAD_SCAN) {
	n_act_file++;
      }

      if(GLsdb_flag[ii]==GOOD_SCAN || GLsdb_flag[ii]==BAD_SCAN || i_file>=n_file) {
	break;
      }
    }

    if(n_act_file==0) {
      if(pipeout>0) StarPipeEndOut();
      exit(0);
    }
    if(i_file>=n_file) {
      if(pipeout>0) StarPipeEndOut();
      GLn_scn=n_act_file;
      return;
    }
  }
  GLn_scn=n_act_file;

  return;
}

void overlay_plot(x_unit,x_range,y_range,win_size,n_scn,last_scn)
RANGE x_range,y_range;
int x_unit;
PxpPoint win_size;
int n_scn,last_scn;
{
  BOX_RANGE plot_box;
  int i_scn,i_color;
  static char color[MAX_SCN][20]={"blue","SkyBlue","cyan","DarkGreen","green","yellow","orange","red","magenta","RoyalBlue","SeaGreen","gold","brown","gray","violet"};
  RANGE_long ch_range;

  BOX_RANGE phys_box;
  PxpPoint name_pos;


  plot_box.x.min=win_size.x*0.1;
  plot_box.y.min=win_size.y*0.1;
  plot_box.x.max=plot_box.x.min+win_size.x*0.8;
  plot_box.y.max=plot_box.y.min+win_size.y*0.8;

/* profile name position */
  name_pos.x=plot_box.x.min+0.2;
  name_pos.y=plot_box.y.max-0.5;

/* plot frame */
  StarPlotProfileStdFrame(&GLprofs[0],x_unit,x_range,y_range,plot_box,&ch_range,&phys_box);

/* profile name */
  PxpDrawString(name_pos.x,name_pos.y,GLprofs[0].file_name);

/* range auto setting */
  PxpSetColor("blue");

  if(x_range.min==x_range.max) {
    switch(x_unit) {
    case X_CHANNEL:
      x_range.min=(double)ch_range.min;
      x_range.max=(double)ch_range.max;
      break;
    case X_VELOCITY:
      x_range.min=StarCh2Velocity(&GLprofs[0],(double)ch_range.min);
      x_range.max=StarCh2Velocity(&GLprofs[0],(double)ch_range.max);
    }
  }

/* profile itself: first profile */
  StarPlotProfile(&GLprofs[0],x_unit,x_range);
  fprintf(stderr,"0 Blue = %s\n",GLprofs[0].file_name);

  PxpSetColor("black");
  if(last_scn==0) return;

/* second and later */
  for(i_scn=1;i_scn<n_scn;i_scn++) {
    if(i_scn>=last_scn) return;
    i_color=i_scn-i_scn/MAX_SCN*MAX_SCN;
    fprintf(stderr,"%d %s = %s\n",i_scn,color[i_color],GLprofs[i_scn].file_name);
    PxpSetColor(color[i_color]);

    StarPlotProfile(&GLprofs[i_scn],x_unit,x_range);
  }
  PxpSetColor("black");

  return;
}

void cell_plot(x_unit,x_range,y_range,win_size,n_column,n_raw,n_scn1panel,last_scn)
RANGE x_range,y_range;
int x_unit;
PxpPoint win_size;
int n_column,n_raw,n_scn1panel,last_scn;
{
  BOX_RANGE plot_box;
  int i_scn,j_scn;
  PxpPoint cell_size;
  int total_scn;

  cell_size.x=win_size.x/(double)n_column;
  cell_size.y=win_size.y/(double)n_raw;

  for(j_scn=0;j_scn<n_raw;j_scn++) {
    plot_box.y.min=cell_size.y*((double)(n_raw-j_scn)-0.9);
    plot_box.y.max=plot_box.y.min+cell_size.y*0.8;
    for(i_scn=0;i_scn<n_column;i_scn++) {
      total_scn=i_scn+j_scn*n_column;
      if(total_scn>=last_scn)
	return;

      plot_box.x.min=cell_size.x*((double)i_scn+0.1);
      plot_box.x.max=plot_box.x.min+cell_size.x*0.8;
      StarPlotProfileStd(&GLprofs[j_scn*n_column+i_scn],x_unit,x_range,y_range,plot_box,GLbase_show,GLspur_show);
    }
  }
  return;
}

void actual_plot()
{
  static int read_flag=1;
  RANGE x_range,y_range;
  int x_unit;

  static PxpPoint win_size;

  int output_mode;
  int window_x,window_y,window_h,window_w;

  int n_column,n_raw;
  int overlay_flag;
  int n_scn,last_scn;
  int n_scn1panel;

/* copy from global valuables */
  x_range.min=GLx_range.min;
  x_range.max=GLx_range.max;
  x_unit=GLx_unit;
  y_range.min=GLy_range.min;
  y_range.max=GLy_range.max;

  n_column=GLn_column;
  n_raw=GLn_raw;
  overlay_flag=GLoverlay;
  n_scn1panel=GLn_scn1panel;
  if(n_scn1panel==0) 
    n_scn=n_column*n_raw;
  else
    n_scn=n_column*n_raw*n_scn1panel;

/* read profile if the first */
  if(read_flag>0) {
    read_profile();
    read_flag= -1;
  }

  last_scn=GLn_scn;

/* plotsize auto scaling */
  PxpGetOutputMode(&output_mode);
  if(output_mode==X11WIN) {
    PxxPresentWindowSize(&window_x,&window_y,&window_w,&window_h);
    win_size.x=PxxGetPlotSpanX(window_w);
    win_size.y=PxxGetPlotSpanY(window_h);
  }

/* profile in standard form */
  if(overlay_flag==FLAGGED)
    overlay_plot(x_unit,x_range,y_range,win_size,n_scn,last_scn);
  else
    cell_plot(x_unit,x_range,y_range,win_size,n_column,n_raw,n_scn1panel,last_scn);

  return;
}

void help_function()
{
  fprintf(stderr,"help should be shown.\n");
  return;
}

int main(argc,argv)
int argc;
char *argv[];
{
  int err_code;
  int n_file;

  double	w_height, w_width;	/* window size in cm */
  int	x_unit;
  RANGE	x_range,y_range;	/* display range */
  int pipeout;
  int bs_flag,sp_flag;
  int	n_column,n_raw;	/* col and row of profiles in a page */
  int	overlay_flag;	/* overlay in a pannel or not */
  int n_scn1panel; /* overlayed scn in a panel */

  char id_str[BUFFER_SIZE];

/* check option */
  err_code=options(argc,argv, &w_height,&w_width,&x_unit,&x_range,&y_range,&pipeout,&bs_flag,&sp_flag, &n_column,&n_raw,&overlay_flag,&n_scn1panel);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

/* set global parameters */
  GLpipeout=pipeout;
  GLx_unit=x_unit;
  GLx_range.min=x_range.min;
  GLx_range.max=x_range.max;
  GLy_range.min=y_range.min;
  GLy_range.max=y_range.max;

  GLbase_show=bs_flag;
  GLspur_show=sp_flag;

  GLn_column=n_column;
  GLn_raw=n_raw;
  GLoverlay=overlay_flag;
  GLn_scn1panel=n_scn1panel;

/* multi-data header in */
  StarPipeHeadIn(id_str,&n_file);
  GLn_file=n_file;

  if(pipeout>0) StarPipeHeadOut(n_file);

/* show profile */
  local_StarPxpDisplayAll(argc,argv, X11WIN,NULL,"display",w_width,w_height,actual_plot,read_profile,help_function);

  exit(0);
}
