/*
make a scd file for a 5point obs

2008/4/17 T.Handa : ver.3.00
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"mk5ptscd.prm"
#define N_KEY 30

#define BUFF_SIZE 256

typedef struct {
  char linename[BUFF_SIZE];
  double rf;
} FreqSetupSSB;

typedef struct {
  FreqSetupSSB usb;
  FreqSetupSSB lsb;
} FreqSetup2SB;

typedef struct {
  char name[BUFF_SIZE];
  int on_coord;
  XY_POS on_xy;
  int off_coord;
  XY_POS off_xy;
  double v_lsr;
} ObjPosition;

int options(argc,argv,scd_name,obj,freq2sb,grid)
/*
get parameters from options
*/
int argc;
char *argv[];
char *scd_name;
ObjPosition *obj;
FreqSetup2SB *freq2sb;
XY_POS *grid;
{
  int ii,next_pos;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;
  char *token;
  char buffer[BUFF_SIZE];
  RANGE range;

  int off_input;

/* default values */
  strcpy(scd_name,"-");
  strcpy((*obj).name,"NorthPole");
  (*freq2sb).usb.rf=230.538000;
  (*freq2sb).lsb.rf=220.398681;
  strcpy((*freq2sb).usb.linename,"CO(J=2-1)");
  strcpy((*freq2sb).lsb.linename,"13CO(J=2-1)");
  (*obj).v_lsr=0.0;
  (*grid).x=4.0/60.0;
  (*grid).y=(*grid).x;
  (*obj).on_coord=COORD_RADEC1950;
  (*obj).off_coord=COORD_RADEC1950;
  off_input=0;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"scd");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"scd_file");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"filename");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"linename");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"line");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"rf");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"freq");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"object");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"obj");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"l");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"b");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"ra");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"dec");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"off_l");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"off_b");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"off_ra");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"off_dec");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"grid_size");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"grid");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

  next_pos=0;
/* scd name */
  n_scan=3;
  for(ii=0;ii<n_scan;ii++) {    
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
    strcpy(scd_name,got_value);
  }

/* RF setup: linename */
  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {    
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
    strcpy(buffer,got_value);
    token=strtok(buffer,",");
    strcpy((*freq2sb).usb.linename,token);
    token=strtok(NULL,",\n");
    if(token==NULL) {
      strcpy((*freq2sb).lsb.linename,(*freq2sb).usb.linename);
    } else {
      strcpy((*freq2sb).lsb.linename,token);
    }
  }
/* RF setup */
  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {    
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
    strcpy((*obj).name,got_value);
  }

/* object name */
  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {    
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
    strcpy((*obj).name,got_value);
  }
/* object position : l */
  n_scan=1;
  for(ii=0;ii<n_scan;ii++) {    
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);

    (*obj).on_xy.x=StarStdAngle2deg(got_value);
    (*obj).on_coord=COORD_LB;
  }
/* object position : b */
  n_scan=1;
  for(ii=0;ii<n_scan;ii++) {    
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);

    (*obj).on_xy.y=StarStdAngle2deg(got_value);
    (*obj).on_coord=COORD_LB;
  }
/* object position : RA(B1950) */
  n_scan=1;
  for(ii=0;ii<n_scan;ii++) {    
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);

    (*obj).on_xy.y=StarStdAngle2deg(got_value);
    (*obj).on_coord=COORD_RADEC1950;
  }
/* object position : Dec(B1950) */
  n_scan=1;
  for(ii=0;ii<n_scan;ii++) {    
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);

    (*obj).on_xy.y=StarStdAngle2deg(got_value);
    (*obj).on_coord=COORD_RADEC1950;
  }
/* off position : l */
  n_scan=1;
  for(ii=0;ii<n_scan;ii++) {    
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);

    (*obj).off_xy.x=StarStdAngle2deg(got_value);
    (*obj).off_coord=COORD_LB;
    off_input=1;
  }
/* off position : b */
  n_scan=1;
  for(ii=0;ii<n_scan;ii++) {    
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);

    (*obj).off_xy.y=StarStdAngle2deg(got_value);
    (*obj).off_coord=COORD_LB;
    off_input=1;
  }
/* off position : RA(B1950) */
  n_scan=1;
  for(ii=0;ii<n_scan;ii++) {    
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);

    (*obj).off_xy.x=StarStdAngle2deg(got_value);
    (*obj).off_coord=COORD_RADEC1950;
    off_input=1;
  }
/* off position : Dec(B1950) */
  n_scan=1;
  for(ii=0;ii<n_scan;ii++) {    
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);

    (*obj).off_xy.y=StarStdAngle2deg(got_value);
    (*obj).off_coord=COORD_RADEC1950;
    off_input=1;
  }
/* grid size */
  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {    
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);

    StarOptionRange(got_value,&range);
    (*grid).x=range.min;
    (*grid).y=range.max;
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  if(off_input==0) {
    (*obj).off_xy.x=(*obj).on_xy.x;
    (*obj).off_xy.y=(*obj).on_xy.y+1.0;
    (*obj).off_coord=(*obj).on_coord;
  }
  
  return(0);
}

int create5points(obj,grid,xy5pt)
ObjPosition obj;
XY_POS grid;
XY_POS *xy5pt;
{
  double dl_x;
  int ii;

  dl_x=grid.x/cos(obj.on_xy.y/180.0*PI);

  xy5pt[0].x=obj.on_xy.x;
  xy5pt[0].y=obj.on_xy.y;

  xy5pt[1].x=obj.on_xy.x+dl_x;
  xy5pt[1].y=obj.on_xy.y;

  xy5pt[2].x=obj.on_xy.x-dl_x;
  xy5pt[2].y=obj.on_xy.y;

  xy5pt[3].x=obj.on_xy.x;
  xy5pt[3].y=obj.on_xy.y+grid.y;

  xy5pt[4].x=obj.on_xy.x;
  xy5pt[4].y=obj.on_xy.y-grid.y;

  for(ii=1;ii<5;ii++) {
    if(xy5pt[ii].x<0.0) xy5pt[ii].x=xy5pt[ii].x+360.0;
    if(xy5pt[ii].x>=360.0) xy5pt[ii].x=xy5pt[ii].x-360.0;
  }

  return(0);
}

int write5points(fp,on_coord,xy5pt,off_coord,off_pt,v_lsr)
FILE *fp;
int on_coord;
XY_POS *xy5pt;
int off_coord;
XY_POS off_pt;
double v_lsr;
{
  static char on_coord_name[10],off_coord_name[10];
  char on_x[80],on_y[80];
  char off_x[80],off_y[80];
  char off_buffer[BUFF_SIZE];
  int ii;

  switch(on_coord) {
  case COORD_LB:
    strcpy(on_coord_name,"lb");
    break;
  case COORD_RADEC1950:
    strcpy(on_coord_name,"radec");
    break;
  case COORD_RADEC_APP:
    strcpy(on_coord_name,"radec-app");
    break;
  }
  switch(off_coord) {
  case COORD_LB:
    strcpy(off_coord_name,"lb");
    break;
  case COORD_RADEC1950:
    strcpy(off_coord_name,"radec");
    break;
  case COORD_RADEC_APP:
    strcpy(off_coord_name,"radec-app");
    break;
  }

  if(off_coord==COORD_LB) {
    Stardeg2StdAngle(off_x,off_pt.x,0);
  } else {
    Stardeg2StdAngle(off_x,off_pt.x,1);
  }
  Stardeg2StdAngle(off_y,off_pt.y,2);
  sprintf(off_buffer,"off,%s,%s,%s\n",off_coord_name,off_x,off_y);

  fprintf(fp,"r,%s,%s,%s,%f\n",off_coord_name,off_x,off_y,v_lsr);
  fprintf(fp,"%s",off_buffer);

  for(ii=0;ii<5;ii++) {
    if(on_coord==COORD_LB) {
      Stardeg2StdAngle(on_x,xy5pt[ii].x,0);
    } else {
      Stardeg2StdAngle(on_x,xy5pt[ii].x,1);
    }
    Stardeg2StdAngle(on_y,xy5pt[ii].y,2);
    fprintf(fp,"on, %s,%s,%s\n",on_coord_name,on_x,on_y);
    fprintf(fp,"%s",off_buffer);
  }
  fprintf(fp,"end\n");
  return(0);
}

int main(argc,argv)
int argc;
char *argv[];
{
  char scd_name[MAX_FILE_NAME_LENGTH];
  FreqSetup2SB freq2sb;
  ObjPosition obj;
  XY_POS grid_size;
  XY_POS xy5pt[5];

  FILE *fp;

  int err_code;

/* check option */
  err_code=options(argc,argv,scd_name,&obj,&freq2sb,&grid_size);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }
  printf("scd=%s;\n",scd_name);

  if(strcmp(scd_name,"-")==0) {
    fp=stdout;
  } else {
    fp=fopen(scd_name,"w");
  }

  fprintf(fp,"/* scd file created by %s */\n",argv[0]);
  fprintf(fp,"pointing\t%s\t\n",obj.name);

  fprintf(fp,"%s,%f,%s,%f,2sb\n",freq2sb.usb.linename,freq2sb.usb.rf,freq2sb.lsb.linename,freq2sb.lsb.rf);

  err_code=create5points(obj,grid_size,xy5pt);

  err_code=write5points(fp,obj.on_coord,xy5pt,obj.off_coord,obj.off_xy,obj.v_lsr);

  if(fp!=stdout)
    fclose(fp);

  exit(0);
}
