#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <StData.h>
#include <StStdio.h>
#include <StStdlib.h>
#include <StPipe.h>
#include <StIndex.h>
#include <StProfileIO.h>
#include <StProfileProc.h>
#include <StAstron.h>

#include <ns2star.h>

#define DEFAULT_PRMFILE "nsprf2scn.prm"
#define N_KEY 10

#define BUFFER_SIZE 256
#define CBFILE_SIZE 2048

typedef struct {
	char filename[13];
	char filetype[7];
	long n_sp;
	long max_ch;
} NS_CBdata;

void get_NS_CBfile(nscb,endian,fp)
NS_CBdata *nscb; /* <out> information in newstar CB file */
int endian; /* <in> data endian */
FILE *fp; /* <in> newstar CB file */
{
	char buffer[CBFILE_SIZE];
	size_t read_byte;

	read_byte=fread(buffer,sizeof(buffer[0]),CBFILE_SIZE,fp);

	minstrcpy((*nscb).filename,buffer,424,12);
	minstrcpy((*nscb).filetype,buffer,436,6);
	(*nscb).n_sp=convert_endian_long(endian,&(buffer[800]));
	(*nscb).max_ch=convert_endian_long(endian,&(buffer[808]));

	return;
}

int options(argc,argv,in_filename,data_endian)
/*
get parameters from options
*/
int argc;
char *argv[];
char *in_filename;
int *data_endian;
{
  int ii;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY],key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  *data_endian=BIG_ENDIAN;


/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"input_file"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"endian"); key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* input filename */
  strcpy(in_filename,argv[argc-1]);
  scan_flag[0]=0;
  got_value=StarOptionValue(1,scan_flag,keyword,key_type,key_flag,value);
  if(got_value!=NULL) {
    strcpy(in_filename,got_value);
  } else {
    strcpy(in_filename,argv[argc-1]);
  }

/* input filename */
  scan_flag[0]=1;
  got_value=StarOptionValue(1,scan_flag,keyword,key_type,key_flag,value);
  if(got_value!=NULL) {
    if(strncmp(got_value,"big",3)==0) {
      *data_endian=BIG_ENDIAN;
    } else {
      *data_endian=LITTLE_ENDIAN;
    }
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

main(argc,argv)
int argc;
char **argv;
{
	char filename[BUFFER_SIZE];
	FILE *fp;

	NS_CBdata NScb;
	int data_endian;

	options(argc,argv, filename,&data_endian);

	fp=fopen(filename,"r");
	if(fp==NULL) {
		fprintf(stderr,"file open faild. %s:::\n",filename);
		exit(99);
	}

	get_NS_CBfile(&NScb,data_endian,fp);

	fclose(fp);

	fprintf(stderr,"file=%s.%s\n",NScb.filename,NScb.filetype);
	fprintf(stderr,"n_sp=%ld,max_ch=%ld\n",NScb.n_sp,NScb.max_ch);
}
