/*
FITS used NewStar at NRO file to scn flow
random group

1999/8/23 T.Handa : ver.3.00
1999/9/21 T.Handa : ver.3.01 (debug)
1999/12/23 T.Handa : ver.3.02 (output_scanname)
2002/5/7 T.Handa : ver.3.03 (to adjust header bug in nsstar FITS, remove input filename extension part)
2002/5/9 T.Handa : ver.3.04 (auto caliculation if FITS has no HPBW for NRO45m data, trk_freq)
2006/ 5/22 T.Handa : ver.3.31 (debug) core dump via ssh; 2D array should be static
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>
#include <StFitsIO.h>

#define DEFAULT_PRMFILE "nsfits2scn.prm"
#define N_KEY 30

#define TOO_SHORT_FILE 100
#define TOO_SMALL_MEMORY_SPACE 9999
#define INVALID_BITPIX 9000

#define TOO_LARGE_DATA_CH 900

#define CORRECT_NO 0
#define CORRECT_DIRECT 1
#define CORRECT_BYFREQ 2


typedef struct {
  int kill_scfactor;
  int hpbw_flag; /* in deg */
  double hpbw;
  int freq_unit;
} BugFlags;

#ifdef INTEL_CPU
/*-------Top of StDataIO.c import---------*/
/*------ for INTEL CPU ------*/

static size_t freadIntel(pt,size,nn,fp)
char *pt;
size_t size;
int nn;
FILE *fp;
/*
Read Binary data written by Motrollor/MIPS CPU with Intel
1998/4/21 T.Handa
1998/7/14 T.Handa (return value revised)
*/
{
  char *buffer;
  size_t ret_code;
  int total_byte,ii,jj;

  if(size==1) {
    ret_code=fread(pt,1,nn,fp);
    return(ret_code);
  }

  total_byte=size*nn;
  buffer=malloc(total_byte);

  ret_code=fread(buffer,1,total_byte,fp);
  if(ret_code==total_byte) ret_code=nn;
  else ret_code=0;

  for(ii=0;ii<nn;ii++) {
    for(jj=0;jj<size;jj++) 
      pt[ii*size+jj]=buffer[(ii+1)*size-jj-1];
  }

  free(buffer);

  return(ret_code);
}
/*-------End of StDataIO.c import---------*/
#else
#define freadIntel(x1,x2,x3,x4) fread(x1,x2,x3,x4)
#endif

void set_default_fits_axis(fits_axis)
FITS_AXIS *fits_axis;
{
  int ii;

  for(ii=0;ii<MAX_NAXIS;ii++) {
    fits_axis[ii].naxis=0;
    strcpy(fits_axis[ii].ctype,"");
    fits_axis[ii].crpix=0.0;
    fits_axis[ii].crval=0.0;
    fits_axis[ii].cdelt=0.0;
    fits_axis[ii].crota=0.0;
    strcpy(fits_axis[ii].cunit,"");
  }

  return;
}

void StarFitsKeyword2FitsRGP(key,key_type,value,rg_prm)
/*
convert FITS keyword to FITS_RG_P data array

1999/8/19 T.Handa
*/
char *key; /* <in> */
int key_type; /* <in> */
char *value; /* <in> */
FITS_RG_P *rg_prm; /* <out> */
{
  int key_index;

  if(key[0]!='P') return;

  key_index=atoi(&(key[5]))-1;

  if(strncmp(key,"PTYPE",5)==0) {
    strcpy(rg_prm[key_index].ptype,value);
    StarReduceString(rg_prm[key_index].ptype);
  } else if(strncmp(key,"PSCAL",5)==0) {
    rg_prm[key_index].pscale=atof(value);
  } else if(strncmp(key,"PZERO",5)==0) {
    rg_prm[key_index].pzero=atof(value);
  }

  return;
}

void StarFitsKeyword2FitsAxis2(key,type,value,fits_axis)
/*
convert FITS keyword to FITS_AXIS data array

Remark: key2FitsAxis() in fits2cub.c is different specifications for fits_axis[0];

1999/8/20 T.Handa
*/
char *key; /* <in> */
int type; /* <in> */
double value; /* <in> */
FITS_AXIS *fits_axis; /* <out> */
{
  int i_axis;
  char key_type[9];

  strncpy(key_type,key,8);
  key_type[8]=(char)NULL;

/* axis paramters */
  if(strncmp(key_type,"NAXIS",5)==0 || strncmp(key_type,"CTYPE",5)==0
  || strncmp(key_type,"CRPIX",5)==0 || strncmp(key_type,"CRVAL",5)==0
  || strncmp(key_type,"CDELT",5)==0 || strncmp(key_type,"CROTA",5)==0
  || strncmp(key_type,"CUNIT",5)==0) {
    i_axis=atoi(&(key_type[5]));
    key_type[5]=(char)NULL;
  } else {
    return;
  }

/* axis size */
  if(strcmp(key,"NAXIS")==0) {
    fits_axis[0].naxis=(int)rint(value);
    return;
  }
  else if(strcmp(key_type,"NAXIS")==0)
    fits_axis[i_axis].naxis=(int)rint(value);
  else if(strcmp(key_type,"CRPIX")==0)
    fits_axis[i_axis].crpix=value;
  else if(strcmp(key_type,"CRVAL")==0)
    fits_axis[i_axis].crval=value;
  else if(strcmp(key_type,"CDELT")==0)
    fits_axis[i_axis].cdelt=value;
  else if(strcmp(key_type,"CROTA")==0)
    fits_axis[i_axis].crota=value;

  return;
}

void StarFitsKeyword2FitsAxis(key,type,value,fits_axis)
/*
convert FITS keyword to FITS_AXIS data array

Remark: key2FitsAxis() in fits2cub.c is different specifications for fits_axis[0];

1999/8/19 T.Handa
*/
char *key; /* <in> */
int type; /* <in> */
char *value; /* <in> */
FITS_AXIS *fits_axis; /* <out> */
{
  int i_axis;
  char key_type[9];

  strncpy(key_type,key,8);
  key_type[8]=(char)NULL;

/* axis paramters */
  if(strncmp(key_type,"NAXIS",5)==0 || strncmp(key_type,"CTYPE",5)==0
  || strncmp(key_type,"CRPIX",5)==0 || strncmp(key_type,"CRVAL",5)==0
  || strncmp(key_type,"CDELT",5)==0 || strncmp(key_type,"CROTA",5)==0
  || strncmp(key_type,"CUNIT",5)==0) {
    i_axis=atoi(&(key_type[5]));
    key_type[5]=(char)NULL;
  } else {
    return;
  }

/* axis size */
  if(strcmp(key,"NAXIS")==0) {
    fits_axis[0].naxis=atoi(value);
    return;
  }
  if(type==FITS_VALUE_INT || type==FITS_VALUE_FLOAT || type==FITS_VALUE_EXP) {
    StarFitsKeyword2FitsAxis2(key,type,atof(value),fits_axis);
  } else {
    if(strcmp(key_type,"CTYPE")==0) {
      strncpy(fits_axis[i_axis].ctype,value,MAX_LWORD);
      fits_axis[i_axis].ctype[MAX_LWORD-1]=(char)NULL;
    } else if(strcmp(key_type,"CUNIT")==0) {
      strncpy(fits_axis[i_axis].cunit,value,MAX_LWORD);
      fits_axis[i_axis].cunit[MAX_LWORD-1]=(char)NULL;
    }
  }

  return;
}

void fits2Syshead2(sys_head,key,type,value)
SYS_HEAD *sys_head;
char *key;
int type;
double value;
{
  if(strcmp(key,"NAVSP")==0) {
    (*sys_head).num_scan=(unsigned char)rint(value);
  }
  return;
}

void fits2Syshead(sys_head,key,type,value)
SYS_HEAD *sys_head;
char *key;
int type;
char *value;
{
  if(type==FITS_VALUE_INT || type==FITS_VALUE_FLOAT || type==FITS_VALUE_EXP) {
    fits2Syshead2(sys_head,key,type,atof(value));
    return;
  }
  return;
}

void fits2Obsprm2(obsprm,key,type,value)
OBS_PRM *obsprm;
char *key;
int type;
double value;
{
  char fits_date[20];
  int yy,mm,dd;
/*
  strcpy((*obsprm).project,"from FITS");
  strcpy((*obsprm).scd_file,"");
  (*obsprm).line_numb=(unsigned short)0;
  (*obsprm).sw_mode=ONOFF_SW;
*/
  if(strcmp(key,"DATE")==0
  || ( strcmp(key,"DATE-OBS")==0 && (*obsprm).mjd==0) ) {
    yy=(int)(rint)(value/10000.0);
    mm=(int)(rint)((value-(double)yy*10000.0)/100.0);
    dd=(int)(floor)(value-(double)yy*10000.0-(double)mm*100.0);
    if(yy<100) {
      sprintf(fits_date,"%02d/%02d/%02d",dd,mm,yy);
    } else {
      sprintf(fits_date,"%d-%02d-%02d",yy,mm,dd);
    }
    (*obsprm).mjd=StarFitsDate2MJD(fits_date);
  }
  if(strcmp(key,"DATE-OBS")==0) {
    yy=(int)(rint)(value/10000.0);
    mm=(int)(rint)((value-(double)yy*10000.0)/100.0);
    dd=(int)(floor)(value-(double)yy*10000.0-(double)mm*100.0);
    if(yy<100) {
      sprintf(fits_date,"%02d/%02d/%02d",dd,mm,yy);
    } else {
      sprintf(fits_date,"%d-%02d-%02d",yy,mm,dd);
    }
    (*obsprm).start=StarFitsDate2Time1970(fits_date);
    if((*obsprm).stop==0L)
      (*obsprm).stop=(*obsprm).start;
  }
  if(strcmp(key,"IAVTM")==0) {
    (*obsprm).on_time=(unsigned short)value;
  }
/*
  (*obsprm).off_time=(unsigned short)0;
*/

  return;
}

void fits2Obsprm(obsprm,key,type,value)
OBS_PRM *obsprm;
char *key;
int type;
char *value;
{
  if(type==FITS_VALUE_INT || type==FITS_VALUE_FLOAT || type==FITS_VALUE_EXP) {
    fits2Obsprm2(obsprm,key,type,atof(value));
    return;
  }

  if(strcmp(key,"DATE")==0
  || ( strcmp(key,"DATE-OBS")==0 && (*obsprm).mjd==0) ) {
    (*obsprm).mjd=StarFitsDate2MJD(value);
  }
  if(strcmp(key,"DATE-OBS")==0) {
    (*obsprm).start=StarFitsDate2Time1970(value);
    if((*obsprm).stop==0L)
      (*obsprm).stop=(*obsprm).start;
  }
  if(strcmp(key,"OBSERVER")==0){
    if(strcmp((*obsprm).observer1,"")==0) {
      strncpy((*obsprm).observer1,value,8);
      (*obsprm).observer1[8]=(char)NULL;
    } else if(strcmp((*obsprm).observer2,"")==0) {
      strncpy((*obsprm).observer2,value,8);
      (*obsprm).observer2[8]=(char)NULL;
    } else if(strcmp((*obsprm).observer3,"")==0) {
      strncpy((*obsprm).observer3,value,8);
      (*obsprm).observer3[8]=(char)NULL;
    } else if(strcmp((*obsprm).observer4,"")==0) {
      strncpy((*obsprm).observer4,value,8);
      (*obsprm).observer4[8]=(char)NULL;
    }
  }

  return;
}

void fits2Object2(obj,key,type,value)
OBJECT *obj;
char *key;
int type;
double value;
{
  if(strcmp(key,"L")==0 || strcmp(key,"GLON")==0) {
    (*obj).cood_sys=COORD_LB;
    (*obj).on_lb.x=value;
    lb2radec50(&((*obj).on_radec),(*obj).on_lb);
  } else if(strcmp(key,"B")==0 || strcmp(key,"GLAT")==0) {
    (*obj).cood_sys=COORD_LB;
    (*obj).on_lb.y=value;
    lb2radec50(&((*obj).on_radec),(*obj).on_lb);
  }
  else if(strcmp(key,"RA")==0) {
    (*obj).cood_sys=COORD_RADEC;
    (*obj).on_radec.x=value;
    radec502lb(&((*obj).on_lb),(*obj).on_radec);
  } else if(strcmp(key,"DEC")==0) {
    (*obj).cood_sys=COORD_RADEC;
    (*obj).on_radec.y=value;
    radec502lb(&((*obj).on_lb),(*obj).on_radec);
  }

  else if(strncmp(key,"EPOCH",5)==0 || strncmp(key,"EQUINOX",7)==0) {
    if(value<1984.0)
      sprintf((*obj).equinox,"B%f6.1",value);
    else
      sprintf((*obj).equinox,"J%f6.1",value);
  }
  else if(strcmp(key,"AZ")==0 || strncmp(key,"AZIM",4)==0) {
    (*obj).on_azel.x=value;
  } else if(strcmp(key,"EL")==0 || strncmp(key,"ELEV",4)==0) {
    (*obj).on_azel.y=value;
  }

  else if(strcmp(key,"LF")==0 || strcmp(key,"GLONF")==0) {
    (*obj).off_lb.x=value;
    lb2radec50(&((*obj).off_radec),(*obj).off_lb);
  } else if(strcmp(key,"BF")==0 || strcmp(key,"GLATF")==0) {
    (*obj).off_lb.y=value;
    lb2radec50(&((*obj).off_radec),(*obj).off_lb);
  }
  else if(strcmp(key,"RAF")==0) {
    (*obj).off_radec.x=value;
    radec502lb(&((*obj).off_lb),(*obj).off_radec);
  } else if(strcmp(key,"DECF")==0) {
    (*obj).off_radec.y=value;
    radec502lb(&((*obj).off_lb),(*obj).off_radec);
  }
/*
  (*obj).orig_xy.x=0.0;
  (*obj).orig_xy.y=0.0;
  (*obj).pa_xy=0.0;
  (*obj).on_xy.x=0.0;
  (*obj).on_xy.y=0.0;
  (*obj).azel_offset.x=0.0;
  (*obj).azel_offset.y=0.0;
*/

  return;
}

void fits2Object(obj,key,type,value)
OBJECT *obj;
char *key;
int type;
char *value;
{
  if(type==FITS_VALUE_INT || type==FITS_VALUE_FLOAT || type==FITS_VALUE_EXP) {
    fits2Object2(obj,key,type,atof(value));
    return;
  }
  if(strcmp(key,"OBJECT")==0) {
    strncpy((*obj).name,value,15);
    (*obj).name[15]=(char)NULL;
  }

  return;
}

void fits2AntStatus2(ant,key,type,value)
ANT_STATUS *ant;
char *key;
int type;
double value;
{
/*
  (*ant).ap_eff=(float)0.0;
  (*ant).mb_eff=(float)0.0;
  (*ant).fss_eff=(float)0.0;
*/
  if(strcmp(key,"HPBW")==0 || strcmp(key,"BEAMSIZE")==0) {
    (*ant).hpbw=(float)value;
  }

  return;
}

void fits2AntStatus(ant,key,type,value)
ANT_STATUS *ant;
char *key;
int type;
char *value;
{
  if(type==FITS_VALUE_INT || type==FITS_VALUE_FLOAT || type==FITS_VALUE_EXP) {
    fits2AntStatus2(ant,key,type,atof(value));
    return;
  }
  if(strcmp(key,"TELESCOP")==0) {
    strncpy((*ant).tele,value,8);
    (*ant).tele[8]=(char)NULL;
  }

  return;
}

void fits2RxStatus2(rx,key,type,value)
RX_STATUS *rx;
char *key;
int type;
double value;
{
/*
  (*rx).sb_mode=USB_SSB;
  (*rx).v_lsr=0.0;
  (*rx).v_off=0.0;
*/
  if(strcmp(key,"FREQ")==0) {
    (*rx).rest_freq=value*1.0e-9; /* default in Hz */
  } else if(strcmp(key,"FQTRK")==0) {
    (*rx).obs_freq=value*1.0e-9; /* default in Hz */
  } else if(strcmp(key,"FQIF1")==0) {
    (*rx).if_freq1=value*1.0e-9; /* default in Hz */
    (*rx).if_freq2=(*rx).if_freq1;
    (*rx).if_freq3=(*rx).if_freq1;
  }
/*
  (*rx).lo_freq1=0.0;
  (*rx).lo_freq2=0.0;
  (*rx).lo_freq3=0.0;
  (*rx).mltplx1=(unsigned short)0;
  (*rx).mltplx2=(unsigned short)0;
  (*rx).mltplx3=(unsigned short)0;
  (*rx).tsys=(float)0.0;
*/
    else if(strcmp(key,"RMS")==0) {
    (*rx).rms=(float)value;
    (*rx).weight=(float)(1.0/(value*value));
  } else if(strcmp(key,"TEMP-R")==0) {
    (*rx).calib=(float)value;
  }
/*
  (*rx).weight=(float)1.0;
*/
    else if(strcmp(key,"SCFACT")==0) {
    (*rx).scale=(float)value;
  }
  return;
}

void fits2RxStatus(rx,key,type,value)
RX_STATUS *rx;
char *key;
int type;
char *value;
{
  if(type==FITS_VALUE_INT || type==FITS_VALUE_FLOAT || type==FITS_VALUE_EXP) {
    fits2RxStatus2(rx,key,type,atof(value));
    return;
  }
  if(strcmp(key,"INSTRUME")==0 && strcmp((*rx).name,"")==0) {
    strncpy((*rx).name,value,7);
    (*rx).name[7]=(char)NULL;
  }

  return;
}

void fits2BeStatus2(be,key,type,value)
BE_STATUS *be;
char *key;
int type;
double value;
{
  if(strcmp(key,"ARRAY-NO")==0) {
    sprintf((*be).name,"%8.0f",value);
    (*be).name[8]=(char)NULL;
  }
/*
  (*be).be_ch=(unsigned short)1;
  (*be).basis_ch=(unsigned short)0;
  (*be).sign=(char)1;
  (*be).data_ch=(unsigned short)1;
  (*be).center_ch=0.0;
  (*be).center_freq=0.0;
  (*be).disp0=(*be).center_freq;
  (*be).disp1=1.0;
  (*be).disp2=0.0;
  (*be).disp3=0.0;
  (*be).disp4=0.0;
*/
  /* default resolution is doulbe channel interval */
  if((*be).freq_res==0.0) { 
    (*be).freq_res=fabs((*be).disp1*2.0);
  }
/*
  (*be).ch_width=(float)1.0;
  (*be).ch_orig=(float)1.0;
*/

  return;
}

void fits2BeStatus(be,key,type,value)
BE_STATUS *be;
char *key;
int type;
char *value;
{
  if(type==FITS_VALUE_INT || type==FITS_VALUE_FLOAT || type==FITS_VALUE_EXP) {
    fits2BeStatus2(be,key,type,atof(value));
    return;
  }
  if(strcmp(key,"INSTRUME")==0) {
    strncpy((*be).name,value,8);
    (*be).name[8]=(char)NULL;
  }

  return;
}

void fits2ObsLog2(log,key,type,value)
OBS_LOG *log;
char *key;
int type;
double value;
{
/*
  (*log).meteo.tmp=(float)0.0;
  (*log).meteo.water=(float)0.0;
  (*log).meteo.press=(float)0.0;
*/
  if(strcmp(key,"TAU")==0) {
    (*log).tau0=(float)value;
  }
/*
  strcpy((*log).memo,"");
*/
  return;
}

void fits2ObsLog(log,key,type,value)
OBS_LOG *log;
char *key;
int type;
char *value;
{
  if(type==FITS_VALUE_INT || type==FITS_VALUE_FLOAT || type==FITS_VALUE_EXP) {
    fits2ObsLog2(log,key,type,atof(value));
    return;
  }
  if(strcmp(key,"ORIGIN")==0) {
    strncpy((*log).site_name,value,8);
    (*log).site_name[8]=(char)NULL;
  }
  if(strcmp(key,"COMMENT")==0) {
    strncpy((*log).comment,value,64);
    (*log).comment[64]=(char)NULL;
  }

  return;
}

void StarFitsKeyword2PrfHeader(key,type,value,profile)
char *key;
int type;
char *value;
PROFILE *profile;
{
  fits2Syshead(&((*profile).sys_head),key,type,value);
  fits2Obsprm(&((*profile).obs_prm),key,type,value);
  fits2Object(&((*profile).object),key,type,value);
  fits2AntStatus(&((*profile).ant_status),key,type,value);
  fits2RxStatus(&((*profile).rx_status),key,type,value);
  fits2BeStatus(&((*profile).be_status),key,type,value);
  fits2ObsLog(&((*profile).obs_log),key,type,value);

  return;
}

void StarRGroupFits2Axis(rg_prm,pcount,pvalue,fits_axis)
/*
Convert axis values in RGroupFits header

1999/9/21 T.Handa
*/
FITS_RG_P *rg_prm; /* <in> */
int pcount; /* <in> */
double *pvalue; /* <in> */
FITS_AXIS *fits_axis; /* <out> */
{
  int ii;
  int type;

  type=FITS_VALUE_FLOAT;
  for(ii=0;ii<pcount;ii++) {
    StarFitsKeyword2FitsAxis2(rg_prm[ii].ptype,type,pvalue[ii],fits_axis);
  }

  return;
}

void StarRGroupFits2PrfHeader(rg_prm,pcount,pvalue,profile)
FITS_RG_P *rg_prm; /* <in> */
int pcount; /* <in> */
double *pvalue; /* <in> */
PROFILE *profile; /* <out> */
{
  int ii;
  int type;

  type=FITS_VALUE_FLOAT;
  for(ii=0;ii<pcount;ii++) {
    fits2Syshead2(&((*profile).sys_head),rg_prm[ii].ptype,type,pvalue[ii]);
    fits2Obsprm2(&((*profile).obs_prm),rg_prm[ii].ptype,type,pvalue[ii]);
    fits2Object2(&((*profile).object),rg_prm[ii].ptype,type,pvalue[ii]);
    fits2AntStatus2(&((*profile).ant_status),rg_prm[ii].ptype,type,pvalue[ii]);
    fits2RxStatus2(&((*profile).rx_status),rg_prm[ii].ptype,type,pvalue[ii]);
    fits2BeStatus2(&((*profile).be_status),rg_prm[ii].ptype,type,pvalue[ii]);
    fits2ObsLog2(&((*profile).obs_log),rg_prm[ii].ptype,type,pvalue[ii]);
  }

  return;
}

#define NO_END_CARD 9
#define NO_RG_FITS 9999
#define TOO_FEW_NAXIS 101
#define TOO_MANY_NAXIS 102
#define NON_ZERO_NAXIS1 103

int StarReadRGroupFitsHeader(in_fp,fits_axis,gcount,pcount,rg_prm,bitpix,profile,fits_bscale,fp_fitshead)
FILE *in_fp; /* <in> */
FITS_AXIS *fits_axis; /* <out> */
int *gcount,*pcount; /* <out> */
FITS_RG_P *rg_prm; /* <out> */
int *bitpix; /* <out> */
PROFILE *profile; /* <out> */
FITS_BSCALE *fits_bscale; /* <out> */
FILE *fp_fitshead; /* <in> */
{
  static char block[36][80];
  char card[81];
  char key[9],value[81];

  int ii,jj;
  int key_type;
  int err_code;

  int rg_flag;

/* set default */
  rg_flag=0;

  for(ii=0;ii<MAX_PSIZE;ii++) {
    strcpy(rg_prm[ii].ptype,"");
    rg_prm[ii].pscale=1.0;
    rg_prm[ii].pzero=0.0;
  }

/* convert FITS header */
  *bitpix=0;
  while(1) {
    err_code=StarReadFitsBlock(in_fp,block);
    if(err_code!=0) { /* cannot read a block */
      return(NO_END_CARD); /* not end by END card */
    }
    for(ii=0;ii<36;ii++) {
      for(jj=0;jj<79;jj++) {
	card[jj]=block[ii][jj];
      }
      card[80]=(char)NULL;
      StarCutFitsComment(card);
      key_type=StarDivideFitsCard(card,key,value);

      if(fp_fitshead!=NULL) fprintf(fp_fitshead,"%s\n",card);

      if(key_type==FITS_VALUE_END) { /* END card */
	goto loop_out;
      }
      StarFitsKeyword2FitsRGP(key,key_type,value,rg_prm);
      StarFitsKeyword2FitsAxis(key,key_type,value,fits_axis);
      if(strcmp(key,"BITPIX")==0) {
	*bitpix=atoi(value);
      } else if(strcmp(key,"GROUPS")==0 && strcmp(value,"T")==0) {
	rg_flag=1;
      } else if(strcmp(key,"GCOUNT")==0) {
	*gcount=atoi(value);
      } else if(strcmp(key,"PCOUNT")==0) {
	*pcount=atoi(value);

      } else if(strcmp(key,"BUNIT")==0) {
	strcpy((*fits_bscale).bunit,value);
      } else if(strcmp(key,"BSCALE")==0) {
	(*fits_bscale).bscale=atof(value);
      } else if(strcmp(key,"BZERO")==0) {
	(*fits_bscale).bzero=atof(value);
      } else if(strcmp(key,"BLANK")==0) {
	(*fits_bscale).blank=atof(value);
      }

      StarFitsKeyword2PrfHeader(key,key_type,value,profile);
    }
  }

loop_out:;

  if(rg_flag==0) return NO_RG_FITS;
  if(fits_axis[0].naxis<2) return TOO_FEW_NAXIS;
  if(fits_axis[0].naxis>2) return TOO_MANY_NAXIS;
  if(fits_axis[1].naxis!=0) return NON_ZERO_NAXIS1;

  return 0;
}

int ReadRGroupFitsPvalue(in_fp,pcount,rg_prm,bitpix,pvalue)
/*
read the RandomGroupFITS parameters and convert them in real values

1999/8/19 T.Handa
*/
FILE *in_fp; /* <in> */
int pcount; /* <in> */
FITS_RG_P *rg_prm; /* <in> */
int bitpix; /* <in> */
double *pvalue; /* <out> */
{
  int ii;
  float *data_float;
  long *data_long;
  short *data_short;
  unsigned char *data_u_char;
  int err_code;

  switch(bitpix) {
  case -64: /* double */
    err_code=freadIntel(pvalue,sizeof(double),pcount,in_fp);
    if(err_code!=pcount) return TOO_SHORT_FILE;
    for(ii=0;ii<pcount;ii++) {
      pvalue[ii]=pvalue[ii]*rg_prm[ii].pscale+rg_prm[ii].pzero;
    }
    break;
  case -32: /* float */
    data_float=malloc(sizeof(float)*pcount);
    if(data_float==NULL) return TOO_SMALL_MEMORY_SPACE;
    err_code=freadIntel(data_float,sizeof(float),pcount,in_fp);
    if(err_code!=pcount) {
      free(data_float);
      return TOO_SHORT_FILE;
    }
    for(ii=0;ii<pcount;ii++) {
      pvalue[ii]=(double)data_float[ii]*rg_prm[ii].pscale+rg_prm[ii].pzero;
    }
    free(data_float);
    break;
  case 32: /* long */
    data_long=malloc(sizeof(long)*pcount);
    if(data_long==NULL) return TOO_SMALL_MEMORY_SPACE;
    err_code=freadIntel(data_long,sizeof(long),pcount,in_fp);
    if(err_code!=pcount) {
      free(data_long);
      return TOO_SHORT_FILE;
    }
    for(ii=0;ii<pcount;ii++) {
      pvalue[ii]=(double)data_long[ii]*rg_prm[ii].pscale+rg_prm[ii].pzero;
    }
    free(data_long);
    break;
  case 16: /* short */
    data_short=malloc(sizeof(short)*pcount);
    if(data_short==NULL) return TOO_SMALL_MEMORY_SPACE;
    err_code=freadIntel(data_short,sizeof(short),pcount,in_fp);
    if(err_code!=pcount) {
      free(data_short);
      return TOO_SHORT_FILE;
    }
    for(ii=0;ii<pcount;ii++) {
      pvalue[ii]=(double)data_short[ii]*rg_prm[ii].pscale+rg_prm[ii].pzero;
    }
    free(data_short);
    break;
  case 8: /* unsigned char */
    data_u_char=malloc(sizeof(unsigned char)*pcount);
    if(data_u_char==NULL) return TOO_SMALL_MEMORY_SPACE;
    err_code=freadIntel(data_u_char,sizeof(unsigned char),pcount,in_fp);
    if(err_code!=pcount) {
      free(data_u_char);
      return TOO_SHORT_FILE;
    }
    for(ii=0;ii<pcount;ii++) {
      pvalue[ii]=(double)data_u_char[ii]*rg_prm[ii].pscale+rg_prm[ii].pzero;
    }
    free(data_u_char);
    break;
  default:
    return INVALID_BITPIX;
  }

  return 0;
}

int FitsRGroupParam2FitsAxis(fits_axis,rg_prm,pcount,pvalue)
/*
1999/ 8/20 T.Handa
*/
FITS_AXIS *fits_axis; /* <out> */
FITS_RG_P *rg_prm; /* <in> */
int pcount; /* <in> */
double *pvalue; /* <in> */
{
  int ii;

  for(ii=0;ii<pcount;ii++) {
    StarFitsKeyword2FitsAxis2(rg_prm[ii].ptype,FITS_VALUE_FLOAT,pvalue[ii],fits_axis);
  }

  return 0;
}

int FitsRGroupParam2FitsBscale(fits_bscale,rg_prm,pcount,pvalue)
/*
1999/ 8/22 T.Handa
*/
FITS_BSCALE *fits_bscale; /* <out> */
FITS_RG_P *rg_prm; /* <in> */
int pcount; /* <in> */
double *pvalue; /* <in> */
{
  int ii;

  for(ii=0;ii<pcount;ii++) {
      if(strcmp(rg_prm[ii].ptype,"BSCALE")==0) {
	(*fits_bscale).bscale=pvalue[ii];
      } else if(strcmp(rg_prm[ii].ptype,"BZERO")==0) {
	(*fits_bscale).bzero=pvalue[ii];
      } else if(strcmp(rg_prm[ii].ptype,"BLANK")==0) {
	(*fits_bscale).blank=pvalue[ii];
      }
  }

  return 0;
}

int fits_axisFreq2RxBe(rx,be,freq_axis)
/*
convert rx_status and be_status for FITS_AXIS with keyword FRE...
freq_axis.cunit : default assmue in Hz

1999/ 8/20 T.Handa
1999/ 9/21 T.Handa (debug) use freq reference (freq_axis.crval)
*/
BE_STATUS *be; /* <out> */
RX_STATUS *rx; /* <out> */
FITS_AXIS freq_axis; /* <in> */
{
  double freq_span;
  double freq_crval;

  (*be).be_ch=freq_axis.naxis;
  if(freq_axis.naxis>MAX_CH) return TOO_LARGE_DATA_CH;
  (*be).data_ch=freq_axis.naxis;

  (*be).center_ch=freq_axis.crpix;
  (*be).basis_ch=(unsigned short)(*be).center_ch;

  if(strcmp(freq_axis.cunit,"GHz")==0 || strcmp(freq_axis.cunit,"GHZ")==0) {
    (*be).disp1=freq_axis.cdelt*1.0e3;
    freq_crval=freq_axis.crval;
  } else if(strcmp(freq_axis.cunit,"MHz")==0 || strcmp(freq_axis.cunit,"MHZ")==0) {
    (*be).disp1=freq_axis.cdelt;
    freq_crval=freq_axis.crval*1.0e-3;
  } else if(strcmp(freq_axis.cunit,"kHz")==0 || strcmp(freq_axis.cunit,"KHZ")==0) {
    (*be).disp1=freq_axis.cdelt*1.0e-3;
    freq_crval=freq_axis.crval*1.0e-6;
  } else if(strcmp(freq_axis.cunit,"Hz")==0 || strcmp(freq_axis.cunit,"HZ")==0) {
    (*be).disp1=freq_axis.cdelt*1.0e-6;
    freq_crval=freq_axis.crval*1.0e-9;
  } else { /* default is in Hz */
    (*be).disp1=freq_axis.cdelt*1.0e-6;
    freq_crval=freq_axis.crval*1.0e-9;
  }
  freq_span=(*be).disp1*(double)((*be).data_ch-1);

  (*be).disp0=freq_span;  /* The backend freq width is assumed from 0.5 to 1.5 of its span. */
  if(freq_span<0.0)
    (*be).disp0= -freq_span;

  (*be).disp2=0.0; (*be).disp3=0.0; (*be).disp4=0.0;
  (*be).center_freq=(*be).disp0;


  (*rx).obs_freq=freq_crval;
  (*rx).v_lsr=((*rx).rest_freq-freq_crval)/(*rx).rest_freq*LIGHT_SPEED;

  return 0;
}

int fits_axis2FreqAndV(rx,be,fits_axis)
/*
convert be_status and rx_status for FITS_AXIS with keyword VEL... FRE...

1999/ 8/20 T.Handa
*/
RX_STATUS *rx; /* <out> */
BE_STATUS *be; /* <out> */
FITS_AXIS *fits_axis; /* <in> */
{
  int ii,ivel,ifrq;
  int err_code;
  FITS_AXIS freq_axis;
  double delta_v;

  ivel=0;
  ifrq=0;

  for(ii=2;ii<=fits_axis[0].naxis;ii++) {
    if(strncmp(fits_axis[ii].ctype,"VEL",3)==0) {
      ivel=ii;
    } else if(strncmp(fits_axis[ii].ctype,"FRE",3)==0) {
      ifrq=ii;
    }
  }

  if(ivel==0 && ifrq==0) return 1;

  if(ifrq==0) { /* only with velocity axis */
    freq_axis.naxis=fits_axis[ivel].naxis;
    freq_axis.crpix=fits_axis[ivel].crpix;
    if(strcmp(fits_axis[ivel].cunit,"km/s")==0 || strcmp(fits_axis[ivel].cunit,"KM/S")==0) {
      delta_v=fits_axis[ivel].cdelt;
    } else if(strcmp(fits_axis[ivel].cunit,"m/s")==0 || strcmp(fits_axis[ivel].cunit,"M/S")==0) {
      delta_v=fits_axis[ivel].cdelt*1.0e-3;
    } else if(strcmp(fits_axis[ivel].cunit,"cm/s")==0 || strcmp(fits_axis[ivel].cunit,"CM/S")==0) {
      delta_v=fits_axis[ivel].cdelt*1.0e-5;
    } else { /* default unit is m/s */
      delta_v=fits_axis[ivel].cdelt*1.0e-3;
    }
    freq_axis.crval=(*rx).obs_freq*1.0e3;
    freq_axis.cdelt=delta_v/LIGHT_SPEED*(*rx).obs_freq*1.0e3;
    strcpy(freq_axis.ctype,"MHz");

    err_code=fits_axisFreq2RxBe(rx,be,freq_axis);
    if(err_code!=0) return err_code;

    (*rx).v_lsr=fits_axis[ivel].crval;
  } 

  if(ivel==0) { /* only with the frequency axis */

    err_code=fits_axisFreq2RxBe(rx,be,fits_axis[ifrq]);
    if(err_code!=0) return err_code;
  }

  (*rx).v_off=(*rx).v_lsr;

  return 0;
}

void AdjustFileSize(profile)
PROFILE *profile; /* <i/o> */
{
  int data_ch,l_flags;

  data_ch=(*profile).be_status.data_ch;
  l_flags=(data_ch-1)/8+1;

  (*profile).sys_head.l_head=(*profile).sys_head.l_head+l_flags*2;
  (*profile).sys_head.l_file=(*profile).sys_head.l_head+data_ch*sizeof((*profile).data[0]);

  return;
}

int StarReadRGroupFitsParameter(in_fp,pcount,rg_prm,bitpix,fits_axis,fits_bscale,profile)
/* 
read a RamdomGroup FITS file and set profile header

1999/ 8/22 T.Handa
*/ 
FILE *in_fp; /* <in> */
int pcount; /* <in> */
FITS_RG_P *rg_prm; /* <in> */
int bitpix; /* <in> */
FITS_BSCALE *fits_bscale; /* <i/o> */
FITS_AXIS *fits_axis; /* <i/o> */
PROFILE *profile; /* <out> */
{
  HISTORY history;
  int err_code;
  double pvalue[MAX_PSIZE];

  ReadRGroupFitsPvalue(in_fp,pcount,rg_prm,bitpix,pvalue);
  err_code=FitsRGroupParam2FitsAxis(fits_axis,rg_prm,pcount,pvalue);
  if(err_code!=0) {
    return err_code;
  }

  err_code=FitsRGroupParam2FitsBscale(fits_bscale,rg_prm,pcount,pvalue);
  if(err_code!=0) {
    return err_code;
  }

  StarRGroupFits2Axis(rg_prm,pcount,pvalue,fits_axis);
  StarRGroupFits2PrfHeader(rg_prm,pcount,pvalue,profile);

  err_code=fits_axis2FreqAndV(&((*profile).rx_status),&((*profile).be_status),fits_axis);
  if(err_code!=0) {
    return err_code;
  }

  AdjustFileSize(profile);

  StarProfileClearBaselineFlag(profile,UNFLAGGED);
  StarProfileClearSpuriousFlag(profile,UNFLAGGED);

  strcpy(history.step,"NSfitP");
  history.size=0;

  StarAddHistory(profile,&history);

  return 0;
}

int ReadRGroupFitsPixel(in_fp,bitpix,fits_bscale,profile)
/*
read the RandomGroupFITS pixel values and convert them in real values

1999/8/22 T.Handa
*/
FILE *in_fp; /* <in> */
int bitpix; /* <in> */
FITS_BSCALE fits_bscale; /* <in> */
PROFILE *profile; /* <i/o> */
{
  int ii;
  double *data_double;
  long *data_long;
  short *data_short;
  unsigned char *data_u_char;
  int err_code;
  int data_ch;

  data_ch=(*profile).be_status.data_ch;

  switch(bitpix) {
  case -64: /* double */
    data_double=malloc(sizeof(double)*data_ch);
    if(data_double==NULL) return TOO_SMALL_MEMORY_SPACE;
    err_code=freadIntel(data_double,sizeof(double),data_ch,in_fp);
    if(err_code!=data_ch) {
      free(data_double);
      return TOO_SHORT_FILE;
    }
    for(ii=0;ii<data_ch;ii++) {
      (*profile).data[ii]=(float)(data_double[ii]*fits_bscale.bscale+fits_bscale.bzero);
    }
    free(data_double);
    break;
  case -32: /* float */
    err_code=freadIntel((*profile).data,sizeof(float),data_ch,in_fp);
    if(err_code!=data_ch) {
      return TOO_SHORT_FILE;
    }
    for(ii=0;ii<data_ch;ii++) {
      (*profile).data[ii]=(float)((double)(*profile).data[ii]*fits_bscale.bscale+fits_bscale.bzero);
    }
    break;
  case 32: /* long */
    data_long=malloc(sizeof(long)*data_ch);
    if(data_long==NULL) return TOO_SMALL_MEMORY_SPACE;
    err_code=freadIntel(data_long,sizeof(long),data_ch,in_fp);
    if(err_code!=data_ch) {
      free(data_long);
      return TOO_SHORT_FILE;
    }
    for(ii=0;ii<data_ch;ii++) {
      if((double)data_long[ii]==fits_bscale.blank) {
	(*profile).data[ii]=(float)fits_bscale.blank;
	StarWriteSpuriousFlag(profile,ii+1,FLAGGED);
      } else {
	(*profile).data[ii]=(float)((double)data_long[ii]*fits_bscale.bscale+fits_bscale.bzero);
      }
    }
    free(data_long);
    break;
  case 16: /* short */
    data_short=malloc(sizeof(short)*data_ch);
    if(data_short==NULL) return TOO_SMALL_MEMORY_SPACE;
    err_code=freadIntel(data_short,sizeof(short),data_ch,in_fp);
    if(err_code!=data_ch) {
      free(data_short);
      return TOO_SHORT_FILE;
    }
    for(ii=0;ii<data_ch;ii++) {
      if((double)data_short[ii]==fits_bscale.blank) {
	(*profile).data[ii]=(float)fits_bscale.blank;
	StarWriteSpuriousFlag(profile,ii+1,FLAGGED);
      } else {
	(*profile).data[ii]=(float)((double)data_short[ii]*fits_bscale.bscale+fits_bscale.bzero);
      }
    }
    free(data_short);
    break;
  case 8: /* unsigned char */
    data_u_char=malloc(sizeof(unsigned char)*data_ch);
    if(data_u_char==NULL) return TOO_SMALL_MEMORY_SPACE;
    err_code=freadIntel(data_u_char,sizeof(unsigned char),data_ch,in_fp);
    if(err_code!=data_ch) {
      free(data_u_char);
      return TOO_SHORT_FILE;
    }
    for(ii=0;ii<data_ch;ii++) {
      if((double)data_u_char[ii]==fits_bscale.blank) {
	(*profile).data[ii]=(float)fits_bscale.blank;
	StarWriteSpuriousFlag(profile,ii+1,FLAGGED);
      } else {
	(*profile).data[ii]=(float)((double)data_u_char[ii]*fits_bscale.bscale+fits_bscale.bzero);
      }
    }
    free(data_u_char);
    break;
  default:
    return INVALID_BITPIX;
  }

  return 0;
}

double Hpbw_byFreq(profile)
PROFILE *profile; /* <in> */
/*
estimate HPBW of NRO 45m using frequency

return: HPBW in arcsec
2002/5/9 T.Handa
 */
{
  double hpbw;
  double freq; /* frequency in GHz */

  freq=(*profile).rx_status.obs_freq;

  if(strncmp((*profile).ant_status.tele,"NRO45M",6)==0) {
    hpbw=15.0/(freq/115.3); /* 15arcsec at 115.3GHz */
  } else {
    hpbw=0.0;
  }

  return hpbw;
}


void HeaderBugFix(profile,bug_flags)
PROFILE *profile;
BugFlags *bug_flags;
{
/* SCFACT */
  if((*bug_flags).kill_scfactor==FLAG_YES) {
    (*profile).rx_status.scale=1.0;
  }

/* unit for FQTRK */
  if((*bug_flags).freq_unit==FLAG_YES) {
    (*profile).rx_status.obs_freq=(*profile).rx_status.obs_freq*1.0e6;
    (*profile).rx_status.v_lsr=(*profile).rx_status.v_lsr*1.0e6;
    (*profile).be_status.center_freq=(*profile).be_status.center_freq*1.0e6;
    (*profile).be_status.disp0=(*profile).be_status.disp0*1.0e6;
    (*profile).be_status.disp1=(*profile).be_status.disp1*1.0e6;
    (*profile).be_status.disp2=(*profile).be_status.disp2*1.0e6;
    (*profile).be_status.disp3=(*profile).be_status.disp3*1.0e6;
    (*profile).be_status.disp4=(*profile).be_status.disp4*1.0e6;
  }

/* no information on HPBW */
  if((*profile).ant_status.hpbw<=0.0) {
    if((*bug_flags).hpbw_flag==CORRECT_DIRECT) {
      fprintf(stderr,"HPBW is set to %f\'.\n",(*bug_flags).hpbw);
      (*profile).ant_status.hpbw=(*bug_flags).hpbw;
    } else if((*bug_flags).hpbw_flag==CORRECT_BYFREQ) {
      (*profile).ant_status.hpbw=Hpbw_byFreq(profile)/60.0;
      fprintf(stderr,"HPBW is set to %f\" using freq.\n",(*profile).ant_status.hpbw*60.0);
    }
  }

  return;
}

void rm_filenameext(filename)
char *filename;
{
  int l_str;
  int ii;

  l_str=strlen(filename);
  if(l_str<1) return;

  for(ii=l_str-1;ii>=0;ii--) {
    if(filename[ii]=='.') {
      if(ii>0) {
	filename[ii]=(char)NULL;
	return;
      }
    }
  }

  return;
}

int options(argc,argv,input_filename,output_filename,rest_freq,v_lsr,fitshead_name,bug_flags)
/*
get parameters from options
2002/5/7 T.Handa
*/
int argc;
char *argv[];
char *input_filename;
char *output_filename;
double *rest_freq,*v_lsr;
char *fitshead_name;
BugFlags *bug_flags;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

  int flag;

/* default values */
  input_filename[0]=(char)NULL;
  *rest_freq=0.0;
  (*bug_flags).kill_scfactor=FLAG_YES;
  strcpy(fitshead_name,"fitshead.txt");
  (*bug_flags).hpbw_flag=CORRECT_BYFREQ;
  (*bug_flags).freq_unit=FLAG_YES;

/* set keywords */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"scn"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"output_file"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"out_scn"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"fit"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"input_file"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"in_fits"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"rest_freq"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"v"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"fitsheader"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"kill_scale"); key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"use_scale"); key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"hpbw"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"freq_unit"); key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* name for input cube */
  scan_flag[0]=0;
  scan_flag[1]=1;
  scan_flag[2]=2;
  n_scan=3;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(output_filename,got_value);
  }

/* name for input cube */
  scan_flag[0]=3;
  scan_flag[1]=4;
  scan_flag[2]=5;
  n_scan=3;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(input_filename,got_value);
  }

  got_value=StarKeylessOption(argc,argv,n_key,key_flag,key_type);
  if(got_value[0]!=(char)NULL) {
    strcpy(input_filename,got_value);
  }

/* rest frequency in GHz */
  scan_flag[0]=6;
  got_value=StarOptionValue(1,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    *rest_freq=atof(got_value);
  }

/* LSR velocity in km/s */
  scan_flag[0]=7;
  got_value=StarOptionValue(1,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    *v_lsr=atof(got_value);
  }

/* name for FITS header output */
  scan_flag[0]=8;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(fitshead_name,got_value);
  }

/* FITS bug fix. kill SCFACT; force to profile.rx.scale=1.0 */
  scan_flag[0]= 9;
  scan_flag[1]= -10;
  flag=StarOptionFlag(2,scan_flag,key_flag);

/* HPBW correction */
  scan_flag[0]=11;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strcmp(got_value,"no")==0) {
      (*bug_flags).hpbw_flag=CORRECT_NO;
      (*bug_flags).hpbw=0.0;
    } else if(strcmp(got_value,"freq")==0) {
      (*bug_flags).hpbw_flag=CORRECT_BYFREQ;
      (*bug_flags).hpbw=0.0;
    } else {
      (*bug_flags).hpbw_flag=CORRECT_DIRECT;
      (*bug_flags).hpbw=atof(got_value)/60.0;
    }
  }

/* freq unit for FQTRK correction: should say in Hz but in kHz */
  scan_flag[0]=12;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strcmp(got_value,"no")==0) {
      (*bug_flags).freq_unit=FLAG_YES;
    } else {
      (*bug_flags).freq_unit=FLAG_NO;
    }
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

int main(argc,argv)
int argc;
char *argv[];
{
  FILE *in_fp;
  FILE *fp_fitshead;

  PROFILE profile;
  PROFILE stdprf;

  int err_code;

  char output_scanname[MAX_FILE_NAME_LENGTH];
  char input_fitsname[MAX_FILE_NAME_LENGTH];
  char full_filename[MAX_FILE_NAME_LENGTH];
  char fitshead_name[MAX_FILE_NAME_LENGTH];

  int sdb_flag;
  char sdb_arg[MAX_IDX_RECORD_LENGTH];

  int ii,n_file;
  int bitpix;

  int pcount,gcount;
  static FITS_RG_P rg_prm[MAX_PSIZE]; /* RandomGroup parameters, scale,zero */
  FITS_AXIS fits_axis[MAX_NAXIS]; /* FITS naxis,ctype,crpix,crval,cdelt,crota,cunit */
  FITS_BSCALE fits_bscale; /* scale,zero,blank for data */

  BugFlags bug_flags;

  double rest_freq,v_lsr;

/* default */
  StarSetDefaultProfile(&stdprf);
  set_default_fits_axis(fits_axis);

/* get option */
  err_code=options(argc,argv,input_fitsname,output_scanname,&rest_freq,&v_lsr,fitshead_name,&bug_flags);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

/* open file for fits header to check */
  if(strcmp(fitshead_name,"")==0) {
    fp_fitshead=NULL;
  } else if(strcmp(fitshead_name,"-")==0) {
    fp_fitshead=stderr;
  } else {
    fp_fitshead=fopen(fitshead_name,"w");
    if(fp_fitshead==NULL) {
      fp_fitshead=stderr;
    }
  }

  if(strcmp(output_scanname,"")==0) {
    strcpy(output_scanname,input_fitsname);
    rm_filenameext(output_scanname);
  }

  stdprf.rx_status.rest_freq=rest_freq;
  stdprf.rx_status.obs_freq=rest_freq;
  stdprf.rx_status.v_lsr=v_lsr;

/* add STD extension for cube */
  err_code=StarStdFileExtName("STAR_DATA",input_fitsname,full_filename,".fit");
  if(err_code!=0) {
    fprintf(stderr,"Too long file name\n");
    exit(1);
  }

/* open the FITS file */
  in_fp=fopen(full_filename,"r");
  if(in_fp==NULL) {
/* add STD extension for cube */
    err_code=StarStdFileExtName("STAR_DATA",input_fitsname,full_filename,".fits");
    if(err_code!=0) {
      fprintf(stderr,"Too long file name\n");
      exit(1);
    }
    in_fp=fopen(full_filename,"r");
    if(in_fp==NULL) {
      fprintf(stderr,"%s:**ERR** cannot find the FITS file: %s\n",argv[0],full_filename);
      exit(1);
    }
  }

/* read FITS header into work array */
  err_code=StarReadRGroupFitsHeader(in_fp,fits_axis,&gcount,&pcount,rg_prm,&bitpix,&stdprf,&fits_bscale,fp_fitshead);
  if(err_code!=0) {
    fclose(in_fp);
    fprintf(stderr,"%s:**ERR** %s in not RandomGroup FITS\n",argv[0],full_filename);
    switch(err_code) {
    case NO_END_CARD:
      fprintf(stderr,"No END keyword\n");
      break;
    case TOO_FEW_NAXIS:
      fprintf(stderr,"Too few NAXIS. %d\n",fits_axis[0].naxis);
      break;
    case TOO_MANY_NAXIS:
      fprintf(stderr,"Too many NAXIS. %d\n",fits_axis[0].naxis);
      break;
    case NON_ZERO_NAXIS1:
      fprintf(stderr,"None zero NAXIS1. %d\n",fits_axis[1].naxis);
      break;
    case NO_RG_FITS:
      fprintf(stderr,"GROUPS is not True\n");
    }
    exit(9);
  }

/* estimate the number of output profiles */
  n_file=gcount;
  StarPipeHeadOut(n_file);

  sdb_flag=GOOD_SCAN;
  strcpy(sdb_arg,"");

  for(ii=0;ii<n_file;ii++) {
    StarCopyProfile(&profile,&stdprf); /* copy the standard template */
    strcpy(profile.file_id,STAR_SCN_IN);
    sprintf(profile.file_name,"%s%06d",output_scanname,ii+1);

    if(fp_fitshead!=NULL) {
      fprintf(fp_fitshead,"FITS header for the scn: %s\n",profile.file_name);
    }

    err_code=StarReadRGroupFitsParameter(in_fp,pcount,rg_prm,bitpix,fits_axis,&fits_bscale,&profile);
    if(err_code!=0) {
      fprintf(stderr,"%s:**ERR** invalied RandomGroup FITS at %dth group\n",argv[0],ii+1);
      switch(err_code) {
      case TOO_LARGE_DATA_CH:
	fprintf(stderr,"Too many data channels ch=%d\n",profile.be_status.be_ch);
	break;
      }
      exit(999);
    }
    HeaderBugFix(&profile,&bug_flags);

    err_code=ReadRGroupFitsPixel(in_fp,bitpix,fits_bscale,&profile);
    if(err_code!=0) {
      fprintf(stderr,"%s:**ERR** invalied RandomGroup FITS at %dth group\n",argv[0],ii+1);
      switch(err_code) {
      case TOO_SHORT_FILE:
	fprintf(stderr,"Too short FITS file\n");
	break;
      case TOO_SMALL_MEMORY_SPACE:
	fprintf(stderr,"Too small memory space. Array size=%d\n",profile.be_status.data_ch);
	break;
      case INVALID_BITPIX:
	fprintf(stderr,"Invalid bitpix=%d\n",bitpix);
	break;
      }
      exit(999);
    }
    StarProfileOut(&profile,sdb_flag,sdb_arg);
  }

/* close the FITS file */
  fclose(in_fp);
  if(fp_fitshead!=stderr && fp_fitshead!=stdout && fp_fitshead!=NULL) 
    fclose(fp_fitshead);

  StarPipeEndOut();

  exit(0);
}
