/*
convert from the NEWSTAR spectrum data to .scn

1996/1/12 Toshihiro HANDA : ver.1.00
1996/2/28 Toshihiro HANDA : ver.1.01 for data ver3
1998/7/14 T.Handa : ver.3.01 (new lib, Linux support)
2000/7/21 J.I.Morino, T.Handa : ver.3.02 (debug)
2004/12/ 1 T.Handa : ver.3.11 (debug) revise from ns2star.c for data SSD
2004/12/19 T.Handa : ver.3.12 (debug) abs for IFfreq
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <StData.h>
#include <StStd.h>

#include <ns2star.h>

#define	BUFFER_SIZE	256

#define	DEFAULT_PRMFILE	"nsprf2scn.prm"
#define N_KEY 10

#ifdef _NO_PROTO
extern	int ns2star_convert();
extern	int STAR_AddTimeHistory();
extern	void calc_NScomb_fit();
#else
extern	int ns2star_convert(PROFILE *, NS_PROFILE *);
extern	int STAR_AddTimeHistory(PROFILE *, HISTORY *, long);
extern	void calc_NScomb_fit(PROFILE *, NS_PROFILE *);
#endif

int options(argc,argv,in_filename,data_endian,max_ch)
/*
get parameters from options
*/
int argc;
char *argv[];
char *in_filename;
int *data_endian;
int *max_ch;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY],key_type[N_KEY];
  int scan_flag[N_KEY];

  char *got_value;

/* default values */
  *data_endian=LITTLE_ENDIAN;
  *max_ch=0; /* auto read from CB-file */

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"input_file"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"endian"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"max_ch"); key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* input filename */
  scan_flag[0]=0;
  got_value=StarOptionValue(1,scan_flag,keyword,key_type,key_flag,value);
  if(got_value!=NULL && strcmp(got_value,"")!=0) {
    strcpy(in_filename,got_value);
  } else {
    strcpy(in_filename,argv[argc-1]);
  }

/* data endian */
  scan_flag[0]=1;
  got_value=StarOptionValue(1,scan_flag,keyword,key_type,key_flag,value);
  if(got_value!=NULL) {
    if(strncmp(got_value,"big",3)==0) {
      *data_endian=BIG_ENDIAN;
    } else {
      *data_endian=LITTLE_ENDIAN;
    }
  }

/* max channel */
  scan_flag[0]=2;
  got_value=StarOptionValue(1,scan_flag,keyword,key_type,key_flag,value);
  if(got_value!=NULL) {
    if(strcmp(got_value,"auto")==0)
      *max_ch=0;
    else
      *max_ch=atoi(got_value);
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

/* ephemeric conversion */
long ephem2mjd(iday)
long iday;
/*
convert from date in ephemerical format to modified Julian day 
correct from iday=000401,jday=15110 to iday=991231,jday=51543

1985/5/5 T.Handa : create
1988/2/12 T.Handa : revise
1995/7/24 T.Handa : convert from FORTRAN to C
*/
{
  long	jday;
  long	jd,jx,jm;
  long	iyy,imm,idd;

  iyy=iday/10000;
  idd=iday-iyy*10000;
  imm=idd/100;
  idd=idd-imm*100;
  jx=(14-imm)/12;
  jm=jx*12+imm-2;
  jm=jm*367;
  jm=jm/12;
  jd=(-jx)+6700+iyy;
  jx=jd/100+1;
  jx=jx*3;
  jx=jx/4;
  jd=jd*1461;
  jd=jd/4;
  jd=jd-2432076;
  jday=jd+jm-jx+idd;

  return(jday);
}

long mjd2time1970(mjday,timesec)
long mjday;	/* Modified Julian Day */
long timesec;	/* time in sec from 0:00 */
/*
Calcularte UNIX sequencial time from 1970 Jan. 1 0:00
from MJD and sequencial time
1970/1/1 0:00 UTC = 40587.0 MJD

1995/7/24 T.Handa
*/
{
  long time1970;

  time1970=(mjday-40587)*86400+timesec;
  return(time1970);
}


int stdfilenameAIPS2UNIX(unix_filename,aips_filename)
char *unix_filename; /* <out> correspond filename for scn */
char *aips_filename; /* <in> SSx file in AIPS system */
{
  int l_str;
  int ii,jj;

  l_str=strlen(aips_filename);
  if(l_str>5) l_str=5;

  for(ii=0,jj=0;ii<=l_str;ii++) {
    if(aips_filename[ii]==(char)NULL) {
      unix_filename[jj]=(char)NULL;
      break;
    }
    if(isalnum((int)aips_filename[ii])!=0) {
      unix_filename[jj]=(char)tolower((int)aips_filename[ii]);
      jj++;
    }
  }

  return(0);
}

int stdCBfilename(cb_filename,ss_filename)
char *cb_filename; /* <out> CBx file in AIPS system */
char *ss_filename; /* <in> SSx file in AIPS system */
{
  strcpy(cb_filename,ss_filename);

  cb_filename[0]='C';
  cb_filename[1]='B';

  return(0);
}

#define MAX_COMBS 10
#define FIT_ORDER 5

void calc_NScomb_fit(profile, NSprofile)
PROFILE *profile; /* <out> */
NS_PROFILE *NSprofile; /* <in> */
/*
2004/12/8 T.Handa (debug) direct linear fit for only 2 data
 */
{
  double coeff[FIT_ORDER+1];
  double comb_freq[MAX_COMBS],comb_ch[MAX_COMBS];
  int ncomb;

  int order,ii;

  ncomb=(*NSprofile).NSbe.n_freq_calib;

/* AOS dispersion parameters */
  for(ii=0;ii<ncomb;ii++){
    comb_freq[ii]=(*NSprofile).NSbe.freq_calib[ii]/1.0e6;
    comb_ch[ii]=(*NSprofile).NSbe.ch_calib_center[ii]-(*profile).be_status.basis_ch;
  }

/* adjuct fitting order */
  order=FIT_ORDER;
  if(ncomb<order){
    fprintf(stderr,"too few comb data =%d but order=%d. The order is adjusted.\n",ncomb,order);
    order=ncomb-1;
  }

/* fitting */
  /* linear backend such as the digital backend (AC) at NRO (Morino) */ 
  if((int)ncomb==2){
    (*profile).be_status.disp1=(comb_freq[1]-comb_freq[0])/(comb_ch[1]-comb_ch[0]);
    (*profile).be_status.disp0=comb_freq[0]-(*profile).be_status.disp1*comb_ch[0];
    (*profile).be_status.disp2=0.0;
    (*profile).be_status.disp3=0.0;
    (*profile).be_status.disp4=0.0;
  } else {
    LeastSquareFit(comb_ch,comb_freq,(int)ncomb,order,coeff);
    (*profile).be_status.disp0=coeff[0];
    (*profile).be_status.disp1=coeff[1];
    (*profile).be_status.disp2=coeff[2];
    (*profile).be_status.disp3=coeff[3];
    (*profile).be_status.disp4=coeff[4];
  }

  return;
}

int main(argc,argv)
int argc;
char *argv[];
{
  char base_filename[MAX_FILE_NAME_LENGTH],dirname[MAX_FILE_NAME_LENGTH];
  char ss_filename[MAX_FILE_NAME_LENGTH],cb_filename[MAX_FILE_NAME_LENGTH];
  char in_filename[MAX_FILE_NAME_LENGTH],out_filename[MAX_FILE_NAME_LENGTH];
  char buffer[MAX_FILE_NAME_LENGTH];
  char *cmd_name;

  PROFILE profile;
  HISTORY history;
  NS_PROFILE NSprofile;	/* profile data in NewStar format */
  NS_CBdata NScb;

  FILE *in_file;
  int n_file;

  int ii;
  int err_code;
  int data_endian;
  int max_ch;

  static int sdb_flag=GOOD_SCAN;
  static char sdb_arg[]="";

  cmd_name=argv[0];

/* check option */
  err_code=options(argc,argv,in_filename,&data_endian,&max_ch);
  if(err_code!=0 && err_code!=111){
    fprintf(stderr,"%s: option error",cmd_name);
    exit(1);
  }

  if(strcmp(in_filename,"-")==0) {
    fgets(in_filename,MAX_FILE_NAME_LENGTH,stdin);
  }

/* modify file name if it has the standard AIPS file name */
  StarExtractPartialFilename(in_filename,ss_filename,FileNameProcess_OnlyFullName);
  stdfilenameAIPS2UNIX(out_filename,ss_filename);
  StarExtractPartialFilename(in_filename,dirname,FileNameProcess_OnlyDir);

  if(strcmp(dirname,"")!=0) {
    sprintf(base_filename,"%s/%s",dirname,out_filename);
  } else {
    strcpy(base_filename,out_filename);
  }

  n_file=0;
  if(max_ch==0) {
    stdCBfilename(out_filename,ss_filename);
    if(strcmp(dirname,"")!=0) {
      sprintf(cb_filename,"%s/%s",dirname,out_filename);
    } else {
      strcpy(cb_filename,out_filename);
    }

    in_file=fopen(cb_filename,"r");
    if(in_file==NULL){
      fprintf(stderr,"*ERR* %s: invalid CB filename=%s\n",cmd_name,cb_filename);
      exit(1);
    }
    get_NS_CBfile(&NScb,data_endian,in_file);
    fclose(in_file);
    max_ch=NScb.max_ch;
    n_file=NScb.n_sp;
  }

  if(max_ch<=0 || max_ch>2048) {
    fprintf(stderr,"%s: CB file=%s,max_ch=%d force to 2048.\n",cmd_name,cb_filename,max_ch);
    max_ch=2048;
  }

/* add STD extension for scan */
  err_code=StarStdFileExtName("STAR_DATA",in_filename,buffer,"");
  if(err_code!=0) {
    fprintf(stderr,"*ERR* %s: Too long file name\n",cmd_name);
    exit(1);
  }
  strcpy(in_filename,buffer);

  in_file=fopen(in_filename,"r");
  if(in_file==NULL){
    fprintf(stderr,"%s: *ERR* invalid filename=%s\n",cmd_name,in_filename);
    exit(1);
  }

/* count total profiles in the file */
  in_file=fopen(in_filename,"r");
  if(in_file==NULL){
    fprintf(stderr,"*ERR* %s: invalid filename=%s\n",cmd_name,in_filename);
    exit(1);
  }

  if(n_file==0) {
    while(1) {
      if(feof(in_file)!=0){
	break;
      }
      err_code=get_NSprofile(&NSprofile,max_ch,data_endian,in_file);
      if(err_code!=0) {
	if(feof(in_file)!=0){
	  break;
	}
	fprintf(stderr,"%s: broken file. code=%d\n",cmd_name,err_code);
	break;
      }
      n_file++;
    }
    rewind(in_file);
  }

  fprintf(stderr,"%s:total scn=%d\n",cmd_name,n_file);

  StarPipeHeadOut(n_file);
 
/* get NSprofile data */
  for(ii=0;ii<n_file;ii++) {
    err_code=get_NSprofile(&NSprofile,max_ch,data_endian,in_file);
    if(err_code!=0) {
      switch(err_code) {
      case 101:
	fprintf(stderr,"%s is too short.\n",in_filename);
	exit(101);
      case 200:
	fprintf(stderr,"too many channels are required for %s.\n",in_filename);
	exit(200);
      }
    }

/* conversion NS-->STAR scn */
    err_code=ns2star_convert(&profile,&NSprofile);
    strcpy(history.step,"newstar");
    StarAddHistory(&profile,&history);

    sprintf(out_filename,"%s-%d",base_filename,ii+1);

/* set standard STAR pipe outflow */
    strcpy(profile.file_id,"STAR.scn");
    strcpy(profile.file_name,out_filename);

/* pipe line outflow */
    err_code=StarProfileOut(&profile,sdb_flag,sdb_arg);
  }
  StarPipeEndOut();
  fclose(in_file);

  exit(0);
}

int ns2star_convert(profile, NSprofile)
PROFILE *profile;
NS_PROFILE *NSprofile;
/*
1996/2/28 T.Handa for STAR data ver3
2004/12/19 T.Handa (debug) abs for IFfreq
*/
{
  HISTORY history;
  long when;

  int ii,jj;
  long start_timing,stop_timing;
  long mjd;
  long start_ch,stop_ch;
  char buffer[BUFFER_SIZE];

  double mean_width;

/* STAR system header */
/*  (*profile).sys_head.l_head=641+512;*/
    /* 641=main part(ver.2), 512=2*(2048/8)byte for flags */
  (*profile).sys_head.l_head=657+512;
  /* 657=main part(ver.3), 512=2*(2048/8)byte for flags */
  (*profile).sys_head.l_file=(*profile).sys_head.l_head+8192;
  /* 8192=2048*4byte for data */
  (*profile).sys_head.format_id=SPECTRUM;
  (*profile).sys_head.version=3;
  (*profile).sys_head.data_type=TA_DATA;
  (*profile).sys_head.hist_step=0;
  (*profile).sys_head.hist_line=0;
  (*profile).sys_head.scan=(*NSprofile).NSstatus.n_orig_sp;
  (*profile).sys_head.backup=0;
  (*profile).sys_head.num_scan=0;

/* STAR observation parameters */
  minstrcpy((*profile).obs_prm.project,(*NSprofile).NSsyshead.schedule[0],9);
  /* 15 chr <-- 9 chr */
  minstrcpy((*profile).obs_prm.scd_file,(*NSprofile).NSsyshead.chapter[0],9);
  /* 9 chr <-- 9 chr */
  (*profile).obs_prm.line_numb=0;	/* line in schedule */
  if(strcmp((*NSprofile).NSstatus.switch_mode,"MONOF")==0)
    (*profile).obs_prm.sw_mode=ONOFF_SW;
  else if(strcmp((*NSprofile).NSstatus.switch_mode,"ONOFF")==0)
    (*profile).obs_prm.sw_mode=ONOFF_SW;
  else if(strcmp((*NSprofile).NSstatus.switch_mode,"FREQ")==0)
    (*profile).obs_prm.sw_mode=FREQ_SW;
  else
    (*profile).obs_prm.sw_mode=ONOFF_SW;
  if((*NSprofile).NSsyshead.start_date[0]>20000101){
    (*profile).obs_prm.mjd=ephem2mjd((long)((*NSprofile).NSsyshead.start_date[0]-20000000+1000000));
  } else {
    (*profile).obs_prm.mjd=ephem2mjd((*NSprofile).NSsyshead.start_date[0]);
  }
  start_timing=hhmmss2sec((*NSprofile).NSsyshead.start_time[0]);
  stop_timing=hhmmss2sec((*NSprofile).NSsyshead.stop_time[0]);
  if(start_timing>stop_timing) stop_timing=stop_timing+24L*3600;
  (*profile).obs_prm.start=mjd2time1970((*profile).obs_prm.mjd,start_timing-9L*3600); /* newstar in JST */
  (*profile).obs_prm.stop=mjd2time1970((*profile).obs_prm.mjd,stop_timing-9L*3600); /* newstar in JST */
  (*profile).obs_prm.on_time=(*NSprofile).NSstatus.total_integ_time;
  (*profile).obs_prm.off_time=(*NSprofile).NSstatus.total_integ_time;
  minstrcpy((*profile).obs_prm.observer1,(*NSprofile).NSsyshead.observer[0],9);
  /* 9 chr <-- 13 chr */
  minstrcpy((*profile).obs_prm.observer2,(*NSprofile).NSsyshead.observer[1],9);
  /* 9 chr <-- 13 chr */
  minstrcpy((*profile).obs_prm.observer3,(*NSprofile).NSsyshead.observer[2],9);
  /* 9 chr <-- 13 chr */
  (*profile).obs_prm.observer4[0]=(char)NULL;

/* STAR object parameters */
  minstrcpy((*profile).object.name,(*NSprofile).NSobj.name,16);
  /* 16 chr <-- 17 chr */
  if(strncmp((*NSprofile).NSobj.xy_coordinate,"AD",2)==0 &&
     strncmp((*NSprofile).NSobj.epoch,"B1950",5)==0)
    (*profile).object.cood_sys=COORD_RADEC;
  /* (*NSprofile).NSobj.epoch == "B1950" || "J2000" */
  else if(strncmp((*NSprofile).NSobj.xy_coordinate,"LB",2)==0)
    (*profile).object.cood_sys=COORD_LB;
  else
    (*profile).object.cood_sys=0;

  (*profile).object.on_lb.x=(*NSprofile).NSobj.on.lb.x;
  (*profile).object.on_lb.y=(*NSprofile).NSobj.on.lb.y;
  (*profile).object.on_radec.x=(*NSprofile).NSobj.on.radec.x;
  (*profile).object.on_radec.y=(*NSprofile).NSobj.on.radec.y;
  (*profile).object.on_azel.x=(*NSprofile).NSobj.on.azel.x;
  (*profile).object.on_azel.y=(*NSprofile).NSobj.on.azel.y;
  (*profile).object.off_lb.x=(*NSprofile).NSobj.off[0].lb.x;
  (*profile).object.off_lb.y=(*NSprofile).NSobj.off[0].lb.y;
  (*profile).object.off_radec.x=(*NSprofile).NSobj.off[0].radec.x;
  (*profile).object.off_radec.y=(*NSprofile).NSobj.off[0].radec.y;
  if((*profile).object.cood_sys==COORD_LB){
    (*profile).object.orig_xy.x=(*NSprofile).NSobj.obj.lb.x;
    (*profile).object.orig_xy.y=(*NSprofile).NSobj.obj.lb.y;
  }
  else if((*profile).object.cood_sys==COORD_RADEC){
    (*profile).object.orig_xy.x=(*NSprofile).NSobj.obj.radec.x;
    (*profile).object.orig_xy.y=(*NSprofile).NSobj.obj.radec.y;
  }
  (*profile).object.pa_xy=(*NSprofile).NSobj.xy_pa;
  (*profile).object.on_xy.x=(*NSprofile).NSobj.d_on.xy.x;
  (*profile).object.on_xy.y=(*NSprofile).NSobj.d_on.xy.y;

/* STAR antena status */
  strcpy((*profile).ant_status.tele,"NRO45m");
  /* 9 chr <-- 0 chr */
  (*profile).ant_status.ap_eff=(*NSprofile).NSant.eta_app;
  (*profile).ant_status.mb_eff=(*NSprofile).NSant.eta_mb;
  (*profile).ant_status.fss_eff=(*NSprofile).NSant.eta_fss;
  (*profile).ant_status.hpbw=(*NSprofile).NSant.hpbw;

/* STAR receiver status */
  minstrcpy((*profile).rx_status.name,(*NSprofile).NSrx.name,8);
  if(strncmp((*profile).rx_status.name,"MULT2",5)==0) {
    int ii;
    for(ii=3;ii<8;ii++) {
      (*profile).rx_status.name[ii]=(*NSprofile).NSrx.name[ii+1];
    }
    (*profile).rx_status.name[8]=(char)NULL;
  }
  /* 8 chr <-- 9 chr */
  if(strncmp((*NSprofile).NSrx.sideband,"USB",3)==0)
    (*profile).rx_status.sb_mode=USB_SSB;
  else if(strncmp((*NSprofile).NSrx.sideband,"LSB",3)==0)
    (*profile).rx_status.sb_mode=LSB_SSB;
  else	/* assume DSB means USB_DSB */
    (*profile).rx_status.sb_mode=USB_DSB;

  (*profile).rx_status.v_lsr=(*NSprofile).NSobj.v/1.0e3;
  (*profile).rx_status.v_off=(*NSprofile).NSobj.v/1.0e3;	/* ver3 */
  (*profile).rx_status.rest_freq=(*NSprofile).NSbe.freq_rest/1.0e9;
/*  (*profile).rx_status.obs_freq=(*NSprofile).NSrx.f_track/1.0e9; */
  (*profile).rx_status.obs_freq=(*NSprofile).NSbe.freq_rest/1.0e9;
  (*profile).rx_status.lo_freq1=0.0;
  (*profile).rx_status.if_freq1=(*NSprofile).NSrx.first_if/1.0e9;
  (*profile).rx_status.lo_freq2=0.0;
  (*profile).rx_status.if_freq2=(*NSprofile).NSrx.first_if/1.0e9;
  (*profile).rx_status.lo_freq3=0.0;
  (*profile).rx_status.if_freq3=(*NSprofile).NSrx.first_if/1.0e9;
  (*profile).rx_status.mltplx1=0;
  (*profile).rx_status.mltplx2=0;
  (*profile).rx_status.mltplx3=0;
  if((*profile).rx_status.if_freq1<0.0) {
    (*profile).rx_status.if_freq1= -(*NSprofile).NSrx.first_if/1.0e9;
  }
  if((*profile).rx_status.if_freq2<0.0) {
    (*profile).rx_status.if_freq2= -(*NSprofile).NSrx.first_if/1.0e9;
  }
  if((*profile).rx_status.if_freq3<0.0) {
    (*profile).rx_status.if_freq3= -(*NSprofile).NSrx.first_if/1.0e9;
  }
  (*profile).rx_status.tsys=(*NSprofile).NSstatus.t_sys;
  (*profile).rx_status.trx=(*NSprofile).NSrx.t_rx;
  (*profile).rx_status.rms=(*NSprofile).NSstatus.noise_level;
  (*profile).rx_status.calib=(*NSprofile).NSbe.t_hot;
  (*profile).rx_status.weight=1.0 /((*profile).rx_status.rms * (*profile).rx_status.rms);
  (*profile).rx_status.scale=(*NSprofile).NSstatus.norm;	/* ver2 */

/* STAR backend status */
  strcpy((*profile).be_status.name,"AOS-");
  minstrcpy(buffer,(*NSprofile).NSbe.name,5);
  strcat((*profile).be_status.name,buffer);
  /* 9 chr <-- 5 chr */
  (*profile).be_status.be_ch=(*NSprofile).NSbe.ch;
  (*profile).be_status.data_ch=(*NSprofile).NSbe.ch;
  (*profile).be_status.basis_ch=(*profile).be_status.be_ch/2;
  switch((*profile).rx_status.sb_mode){
  case USB_SSB: case USB_DSB:
    (*profile).be_status.sign= -1;
    break;
  case LSB_SSB: case LSB_DSB:
    (*profile).be_status.sign=  1;
  default:
    (*profile).be_status.sign=  1;
  }

  /* AOS-H is inverted order in ch respected to IF */
  if((*NSprofile).NSbe.name[0]=='H')
    (*profile).be_status.sign= -(*profile).be_status.sign;
		
  (*profile).be_status.data_ch=(*NSprofile).NSbe.ch;
  (*profile).be_status.center_ch=(double)(*profile).be_status.basis_ch;	/* ver3 */
  (*profile).be_status.center_freq=(*NSprofile).NSbe.center_freq/1.0e6;

  (*profile).be_status.freq_res=(*NSprofile).NSbe.resolution/1.0e6;
  if((*profile).be_status.freq_res<=0.0)
    (*profile).be_status.freq_res=(*NSprofile).NSbe.separation*2.0;
  if((*profile).be_status.freq_res<=0.0){
    mean_width=0.0;
    for(ii=0;ii<(*NSprofile).NSbe.n_freq_calib;ii++){
      mean_width=mean_width+
	(*NSprofile).NSbe.ch_calib_width[ii];
    }
    (*profile).be_status.freq_res=
      mean_width/(*NSprofile).NSbe.n_freq_calib/1.0e6;
  }
  (*profile).be_status.ch_width=1.0;
  (*profile).be_status.ch_orig=1.0;

  calc_NScomb_fit(profile,NSprofile);

/* STAR observation log */
  strcpy((*profile).obs_log.site_name,"Nobeyama");
  /* 9 chr <-- 0 chr */
  (*profile).obs_log.meteo.tmp=(*NSprofile).NSstatus.meteo.tmp;
  (*profile).obs_log.meteo.water=(*NSprofile).NSstatus.meteo.water;
  (*profile).obs_log.meteo.press=(*NSprofile).NSstatus.meteo.press;
  (*profile).obs_log.tau0=(*NSprofile).NSstatus.tau_atm;
  (*profile).obs_log.memo[0]=(char)NULL;
  /* 65 chr <-- 0 chr */
  minstrcpy((*profile).obs_log.comment,(*NSprofile).comment,65);
  /* 65 chr <-- 81 chr */

/* baseline flag */
  for(ii=0;ii<(*profile).be_status.data_ch;ii++){
    StarWriteBaselineFlag(profile,ii+1,UNFLAGGED);
  }
  for(jj=0;jj<(*NSprofile).NSstatus.rms.n_range;jj++){
    start_ch=(*NSprofile).NSstatus.rms.start[jj];
    stop_ch=(*NSprofile).NSstatus.rms.stop[jj];
    /*      fprintf(stderr,"%ld %ld %ld %ld\n",ii,jj,start_ch,stop_ch);*/
    for(ii=start_ch;ii<stop_ch;ii++){
      StarWriteBaselineFlag(profile,ii+1,FLAGGED);
    }
  }

/* NEWSTAR save procedure has A BUG, so we use rms range in stead of basefit range */
/* debug by Morino */
  for(ii=0;ii<(*profile).be_status.data_ch;ii++){
    for(jj=0;jj<(*NSprofile).NSstatus.rms.n_range;jj++){
      start_ch=(*NSprofile).NSstatus.rms.start[jj];
      stop_ch=(*NSprofile).NSstatus.rms.stop[jj];
      if(ii>=start_ch && ii<=stop_ch)
	StarWriteBaselineFlag(profile,ii+1,FLAGGED);
      else
	StarWriteBaselineFlag(profile,ii+1,UNFLAGGED);
    }
  }

/* spurious flag */
/* all unflagged */
  for(ii=0;ii<(*profile).be_status.data_ch;ii++){
    StarWriteSpuriousFlag(profile,ii+1,UNFLAGGED);
  }

/* history */
  for(ii=0;ii<10;ii++){
    history.size=0;
    minstrcpy(history.step,(*NSprofile).NShistory[ii],7);
    /* 8 chr <-- first 7 chr */
    minstrcpy(buffer,&((*NSprofile).NShistory[ii][6]),7);
    if(buffer[0]==(char)NULL) break;
    mjd=ephem2mjd(atol(buffer));
    when=mjd2time1970(mjd,0);
    minstrcpy(history.prm[0],&((*NSprofile).NShistory[ii][13]),17);
    for(jj=0;jj<17;jj++){
      if(history.prm[0][jj]!=' ' && history.prm[0][jj]!=(char)NULL)
	history.size=1;
    }
    STAR_AddTimeHistory(profile,&history,when);
    /* 17 chr <-- 81 chr */
  }

/* data */
  for(ii=0;ii<(*profile).be_status.data_ch;ii++){
    (*profile).data[ii]=(*NSprofile).data[ii];
  }

  return(0);
}


int STAR_AddTimeHistory(profile,history,when)
/*
add HISTORY into a profile
END can be add by this function
END entry is automatically attached
0   normal return
100 too many history steps

1992/12/12 T.Handa
1995/7/26 T.Handa revised from STAR_AddHistory
*/
PROFILE	*profile;
HISTORY	*history;	/* history to be add */
long	when;		/* processed time */
{
  int	lines;
  long	pres_time;
  int	current_step;
  int	add_size,add_byte;

  current_step=(*profile).sys_head.hist_step;

/* ----correct increasing the sizes---- */
/* previous last step is "END" --> adjust sizes */
  if(strcmp((*profile).history[current_step].step,"END")==0 || strcmp((*profile).history[current_step].step,"end")==0) {
    (*profile).sys_head.hist_line--;
    (*profile).sys_head.l_file-=16;
    (*profile).sys_head.l_head-=16;
  }

  add_size=(*history).size+1;
  add_byte=add_size*16;

  (*profile).sys_head.hist_line+=add_size;
  (*profile).sys_head.l_file+=add_byte;
  (*profile).sys_head.l_head+=add_byte;

/* increase history step in normal history addition */
  if(strcmp((*history).step,"END")!=0 && strcmp((*history).step,"end")!=0) {
    (*profile).sys_head.hist_step++;
  } else {
    (*history).size=0;
  }

/* too many history steps */
  if((*profile).sys_head.hist_step>=MAX_HISTORY_STEP)
    return(100);

/* copy history to profile.history */
  (*profile).history[current_step].size=(*history).size;
  (*history).step[7]=(char)NULL;
  strcpy((*profile).history[current_step].step,(*history).step);
/*  time(&pres_time); */
  pres_time=when;
  (*profile).history[current_step].time=pres_time;
  for(lines=0;lines<(*history).size;lines++) {
    (*history).prm[lines][16]=(char)NULL;
    strcpy((*profile).history[current_step].prm[lines],(*history).prm[lines]);
  }

/* add history is alread "END" */
  if(strcmp((*history).step,"END")==0 || strcmp((*history).step,"end")==0)
    return(0);

/* ---add "END" step--- */

  (*profile).sys_head.hist_line++;
  (*profile).sys_head.l_file+=16;
  (*profile).sys_head.l_head+=16;
  current_step=(*profile).sys_head.hist_step;
/* too many history steps */
  if((*profile).sys_head.hist_step>=MAX_HISTORY_STEP)
    return(100);

  (*profile).history[current_step].size=0;
  strcpy((*profile).history[current_step].step,"END");	
  (*profile).history[current_step].time=pres_time;

  return(0);
}
