/*
convert from the NEWSTAR spectrum data to .scn

2004/12/ 1 T.Handa : ver.3.11
2004/12/ 6 T.Handa :(debug) print_NSbe() ch_calib_center,ch_calib_width
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <StData.h>
#include <StStd.h>

#include <ns2star.h>

#define	BUFFER_SIZE	256

#define	DEFAULT_PRMFILE	"nsprf2scn.prm"
#define N_KEY 10

int options(argc,argv,in_filename,data_endian,max_ch)
/*
get parameters from options
*/
int argc;
char *argv[];
char *in_filename;
int *data_endian;
int *max_ch;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY],key_type[N_KEY];
  int scan_flag[N_KEY];

  char *got_value;

/* default values */
  *data_endian=LITTLE_ENDIAN;
  *max_ch=0; /* auto read from CB-file */

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"input_file"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"endian"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"max_ch"); key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* input filename */
  scan_flag[0]=0;
  got_value=StarOptionValue(1,scan_flag,keyword,key_type,key_flag,value);
  if(got_value!=NULL && strcmp(got_value,"")!=0) {
    strcpy(in_filename,got_value);
  } else {
    strcpy(in_filename,argv[argc-1]);
  }

/* data endian */
  scan_flag[0]=1;
  got_value=StarOptionValue(1,scan_flag,keyword,key_type,key_flag,value);
  if(got_value!=NULL) {
    if(strncmp(got_value,"big",3)==0) {
      *data_endian=BIG_ENDIAN;
    } else {
      *data_endian=LITTLE_ENDIAN;
    }
  }

/* max channel */
  scan_flag[0]=2;
  got_value=StarOptionValue(1,scan_flag,keyword,key_type,key_flag,value);
  if(got_value!=NULL) {
    if(strcmp(got_value,"auto")==0)
      *max_ch=0;
    else
      *max_ch=atoi(got_value);
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

/* ephemeric conversion */
long ephem2mjd(iday)
long iday;
/*
convert from date in ephemerical format to modified Julian day 
correct from iday=000401,jday=15110 to iday=991231,jday=51543

1985/5/5 T.Handa : create
1988/2/12 T.Handa : revise
1995/7/24 T.Handa : convert from FORTRAN to C
*/
{
  long	jday;
  long	jd,jx,jm;
  long	iyy,imm,idd;

  iyy=iday/10000;
  idd=iday-iyy*10000;
  imm=idd/100;
  idd=idd-imm*100;
  jx=(14-imm)/12;
  jm=jx*12+imm-2;
  jm=jm*367;
  jm=jm/12;
  jd=(-jx)+6700+iyy;
  jx=jd/100+1;
  jx=jx*3;
  jx=jx/4;
  jd=jd*1461;
  jd=jd/4;
  jd=jd-2432076;
  jday=jd+jm-jx+idd;

  return(jday);
}

long mjd2time1970(mjday,timesec)
long mjday;	/* Modified Julian Day */
long timesec;	/* time in sec from 0:00 */
/*
Calcularte UNIX sequencial time from 1970 Jan. 1 0:00
from MJD and sequencial time
1970/1/1 0:00 UTC = 40587.0 MJD

1995/7/24 T.Handa
*/
{
  long time1970;

  time1970=(mjday-40587)*86400+timesec;
  return(time1970);
}

typedef struct {
  int year;
  int month;
  int day;
  int hour;
  int min;
  int sec;
} DT6DIGIT;

void date_time(dt6,date,time)
DT6DIGIT *dt6; /* <out> */
long date;
long time;
{
  (*dt6).year=date/10000;
  date=date-(*dt6).year*10000;
  (*dt6).month=date/100;
  date=date-(*dt6).month*100;
  (*dt6).day=date;

  (*dt6).hour=time/10000;
  time=time-(*dt6).hour*10000;
  (*dt6).min=time/100;
  time=time-(*dt6).min*100;
  (*dt6).sec=time;

  return;
}


int stdfilenameAIPS2UNIX(unix_filename,aips_filename)
char *unix_filename; /* <out> correspond filename for scn */
char *aips_filename; /* <in> SSx file in AIPS system */
{
  int l_str;
  int ii,jj;

  l_str=strlen(aips_filename);
  if(l_str>5) l_str=5;

  for(ii=0,jj=0;ii<=l_str;ii++) {
    if(aips_filename[ii]==(char)NULL) {
      unix_filename[jj]=(char)NULL;
      break;
    }
    if(isalnum((int)aips_filename[ii])!=0) {
      unix_filename[jj]=(char)tolower((int)aips_filename[ii]);
      jj++;
    }
  }

  return(0);
}

int stdCBfilename(cb_filename,ss_filename)
char *cb_filename; /* <out> CBx file in AIPS system */
char *ss_filename; /* <in> SSx file in AIPS system */
{
  strcpy(cb_filename,ss_filename);

  cb_filename[0]='C';
  cb_filename[1]='B';

  return(0);
}

void print_NSsyshead(fp,NSsys)
FILE *fp;
NS_SYSHEAD *NSsys;
{
  int ii;
  DT6DIGIT start,stop;

  fprintf(fp,"==system==\n");
  fprintf(fp,"name=%s, ver=%s\n",(*NSsys).sp_name,(*NSsys).version);
  for(ii=0;ii<3;ii++) {
    fprintf(fp,"chap %d:",ii);
    fprintf(fp,"sched=%s, ",(*NSsys).schedule[ii]);
    fprintf(fp,"chap=%s, ",(*NSsys).chapter[ii]);
    fprintf(fp,"obser=%s\n",(*NSsys).observer[ii]);
    date_time(&start,(*NSsys).start_date[ii],(*NSsys).start_time[ii]);
    date_time(&stop,(*NSsys).stop_date[ii],(*NSsys).stop_time[ii]);
    fprintf(fp,"date/time: %d/%d/%d %d:%d:%d - %d/%d/%d %d:%d:%d\n",start.year,start.month,start.day,start.hour,start.min,start.sec,stop.year,stop.month,stop.day,stop.hour,stop.min,stop.sec);
  }

  return;
}

void print_NSposition(fp,pos)
FILE *fp;
POSITION *pos;
{
  fprintf(fp,"lb=(%f,%f)[deg], ",(*pos).lb.x,(*pos).lb.y);
  fprintf(fp,"RADec=(%f,%f)[deg], ",(*pos).radec.x,(*pos).radec.y);
  fprintf(fp,"XY=(%f,%f)[deg], ",(*pos).xy.x,(*pos).xy.y);
  fprintf(fp,"AzEl=(%f,%f)[deg]\n",(*pos).azel.x,(*pos).azel.y);

  return;
}

void print_NSobj(fp,NSobj)
FILE *fp;
NS_OBJECT *NSobj;
{
  int ii;

  fprintf(fp,"==object==\n");
  fprintf(fp,"name=%s\n",(*NSobj).name);
  fprintf(fp,"epoch %s, ",(*NSobj).epoch);
  fprintf(fp,"xy coordinate=%s PA=%f[deg]\n",(*NSobj).xy_coordinate,(*NSobj).xy_pa);
  fprintf(fp,"center:");
  print_NSposition(fp,&((*NSobj).obj));
  fprintf(fp,"d_on  :");
  print_NSposition(fp,&((*NSobj).d_on));
  fprintf(fp,"on    :");
  print_NSposition(fp,&((*NSobj).on));
  for(ii=0;ii<2;ii++) {
    fprintf(fp,"d_off[%d]:",ii);
    print_NSposition(fp,&((*NSobj).d_off[ii]));
    fprintf(fp,"off[%d]  :",ii);
    print_NSposition(fp,&((*NSobj).off[ii]));
  }
  fprintf(fp,"offset of beamsw=%f\n",(*NSobj).d_offbeam);
  fprintf(fp,"v=%f[km/s], v_radial=%f[km/s] from %s by %s\n",(*NSobj).v,(*NSobj).v_radial,(*NSobj).v_reference,(*NSobj).v_definition);

  return;
}

void print_NSant(fp,NSant)
FILE *fp;
NS_ANT *NSant;
{
  fprintf(fp,"==antenna==\n");
  fprintf(fp,"HPBW=%f, ",(*NSant).hpbw);
  fprintf(fp,"eta_app=%f, eta_mb=%f, eta_ant=%f, eta_fss=%f, ant_gain=%f, eta_couple=%f\n",(*NSant).eta_app,(*NSant).eta_mb,(*NSant).eta_ant,(*NSant).eta_fss,(*NSant).gain,(*NSant).eta_couple);
  fprintf(fp,"subref x=%f[mm], y=%f[mm], z1=%f[mm], z2=%f[mm]\n",(*NSant).subref_offset.x,(*NSant).subref_offset.y,(*NSant).subref_offset.z1,(*NSant).subref_offset.z2);
  fprintf(fp,"pos corr x=%f, y=%f\n",(*NSant).pointing_corr.x,(*NSant).pointing_corr.y);

  return;
}

void print_NSrx(fp,NSrx)
FILE *fp;
NS_RX *NSrx;
{
  fprintf(fp,"==frontend==\n");
  fprintf(fp,"name=%s feed=%s\n",(*NSrx).name,(*NSrx).feed);
  fprintf(fp,"track=%f[Hz], 1st_IF=%f[Hz]\n",(*NSrx).f_track,(*NSrx).first_if);
  fprintf(fp,"sideband=%s with ratio=%f, ",(*NSrx).sideband,(*NSrx).ssb_ratio);
  fprintf(fp,"Trx=%f[K]\n",(*NSrx).t_rx);
  fprintf(fp,"freqsw at %f[Hz] with %f[Hz] offset\n",(*NSrx).freq_fsw,(*NSrx).d_freq_fsw);

  return;
}

void print_NSbe(fp,NSbe)
FILE *fp;
NS_BE *NSbe;
{
  int ii;

  fprintf(fp,"==backend==\n");
  fprintf(fp,"name=%s with function %s",(*NSbe).name,(*NSbe).function_table);
  fprintf(fp,"BE=%ldch\n",(*NSbe).ch);
  fprintf(fp,"bandwidht=%f, resolution=%f, ch_reparation=%f\n",(*NSbe).bandwidth,(*NSbe).resolution,(*NSbe).separation);
  fprintf(fp,"rest=%f[Hz], center=%f[Hz]\n",(*NSbe).freq_rest,(*NSbe).center_freq);
  if((*NSbe).n_freq_calib>10) {
    fprintf(fp,"**ERR** invalid value of n_freq_calib=%ld. must be <=10. Endian may be wrong.\n",(*NSbe).n_freq_calib);
    (*NSbe).n_freq_calib=10;
  }
  for(ii=0;ii<(*NSbe).n_freq_calib;ii++) {
    fprintf(fp,"%f[Hz] @ %f[ch] with %f[ch] width, ",(*NSbe).freq_calib[ii],(*NSbe).ch_calib_center[ii],(*NSbe).ch_calib_width[ii]);
  }
  fprintf(fp,"\n");
  fprintf(fp,"Thot=%f[K]\n",(*NSbe).t_hot);

  return;
}

void print_NSdatastatus(fp,NSstatus)
FILE *fp;
NS_DATA_STATUS *NSstatus;
{
  int ii;

  fprintf(fp,"==data status==\n");
  fprintf(fp,"integ=%ld[sec] from %ld prfs, ",(*NSstatus).total_integ_time,(*NSstatus).n_orig_sp);
  fprintf(fp,"noise lvl=%f[K] from \n",(*NSstatus).noise_level);
  if((*NSstatus).rms.n_range>6) {
    fprintf(fp,"**ERR** invalid value of n_rms_range=%ld. must be <=6. Endian may be wrong.\n",(*NSstatus).rms.n_range);
    (*NSstatus).rms.n_range=6;
  }
  for(ii=0;ii<(*NSstatus).rms.n_range;ii++) {
    fprintf(fp,"%ld[ch]-%ld[ch], ",(*NSstatus).rms.start[ii],(*NSstatus).rms.stop[ii]);
  }
  fprintf(fp,"\n");
  fprintf(fp,"basefit by %ldth-order %s from \n",(*NSstatus).basefit_dimension,(*NSstatus).basefit_function);
  if((*NSstatus).basefit.n_range>6) {
    fprintf(fp,"**ERR** invalid value of n_rms_range=%ld. must be <=6. Endian may be wrong.\n",(*NSstatus).basefit.n_range);
    (*NSstatus).basefit.n_range=6;
  }
  for(ii=0;ii<(*NSstatus).basefit.n_range;ii++) {
    fprintf(fp,"%ld[ch]-%ld[ch], ",(*NSstatus).basefit.start[ii],(*NSstatus).basefit.stop[ii]);
  }
  fprintf(fp,"\n");
  fprintf(fp,"Tsys=%f[K], tau_atm=%f, norm=%f\n",(*NSstatus).t_sys,(*NSstatus).tau_atm,(*NSstatus).norm);
  fprintf(fp,"Tatm=%f[K], Pwater=%f[hPa], Patm=%f[hPa]\n",(*NSstatus).meteo.tmp,(*NSstatus).meteo.water,(*NSstatus).meteo.press);
  fprintf(fp,"wind v=%f[m/min], dir=%f[deg], ",(*NSstatus).wind.speed,(*NSstatus).wind.direction);
  fprintf(fp,"Tb(atm)=%f[K]\n",(*NSstatus).tb_atm);
  fprintf(fp,"switch mode=%s\n",(*NSstatus).switch_mode);

  return;
}

void print_NSac45status(fp,NSac45)
FILE *fp;
NS_AC45_STATUS *NSac45;
{
  fprintf(fp,"==AC45 status==\n");
  fprintf(fp,"ALC time=%f[sec], voltage=%f[V], ",(*NSac45).alc_time,(*NSac45).alc_v);
  fprintf(fp,"lag window=%s\n",(*NSac45).lag_window);
  fprintf(fp,"norm for multiRX=%f\n",(*NSac45).norm4multrx);

  return;
}

void print_NSprofileHeader(fp,NSprofile)
FILE *fp;
NS_PROFILE *NSprofile;
{
  print_NSsyshead(fp,&((*NSprofile).NSsyshead));
  print_NSobj(fp,&((*NSprofile).NSobj));
  print_NSant(fp,&((*NSprofile).NSant));
  print_NSrx(fp,&((*NSprofile).NSrx));
  print_NSbe(fp,&((*NSprofile).NSbe));
  print_NSdatastatus(fp,&((*NSprofile).NSstatus));
  print_NSac45status(fp,&((*NSprofile).NSac45status));

  return;
}

int main(argc,argv)
int argc;
char *argv[];
{
  char base_filename[MAX_FILE_NAME_LENGTH],dirname[MAX_FILE_NAME_LENGTH];
  char ss_filename[MAX_FILE_NAME_LENGTH],cb_filename[MAX_FILE_NAME_LENGTH];
  char in_filename[MAX_FILE_NAME_LENGTH],out_filename[MAX_FILE_NAME_LENGTH];
  char buffer[MAX_FILE_NAME_LENGTH];
  char *cmd_name;

  NS_PROFILE NSprofile;	/* profile data in NewStar format */
  NS_CBdata NScb;

  FILE *in_file;
  int n_file;

  int ii;
  int err_code;
  int data_endian;
  int max_ch;

  cmd_name=argv[0];

/* check option */
  err_code=options(argc,argv,in_filename,&data_endian,&max_ch);
  if(err_code!=0 && err_code!=111){
    fprintf(stderr,"%s: option error",cmd_name);
    exit(1);
  }

  if(strcmp(in_filename,"-")==0) {
    fgets(in_filename,MAX_FILE_NAME_LENGTH,stdin);
  }

/* modify file name if it has the standard AIPS file name */
  StarExtractPartialFilename(in_filename,ss_filename,FileNameProcess_OnlyFullName);
  stdfilenameAIPS2UNIX(out_filename,ss_filename);
  StarExtractPartialFilename(in_filename,dirname,FileNameProcess_OnlyDir);

  if(strcmp(dirname,"")!=0) {
    sprintf(base_filename,"%s/%s",dirname,out_filename);
  } else {
    strcpy(base_filename,out_filename);
  }

  n_file=0;
  if(max_ch==0) {
    stdCBfilename(out_filename,ss_filename);
    if(strcmp(dirname,"")!=0) {
      sprintf(cb_filename,"%s/%s",dirname,out_filename);
    } else {
      strcpy(cb_filename,out_filename);
    }

    in_file=fopen(cb_filename,"r");
    if(in_file==NULL){
      fprintf(stderr,"*ERR* %s: invalid CB filename=%s\n",cmd_name,cb_filename);
      exit(1);
    }
    get_NS_CBfile(&NScb,data_endian,in_file);
    fclose(in_file);
    max_ch=NScb.max_ch;
    n_file=NScb.n_sp;
  }

  if(max_ch<=0 || max_ch>2048) {
    fprintf(stderr,"%s: CB file=%s,max_ch=%d force to 2048.\n",cmd_name,cb_filename,max_ch);
    max_ch=2048;
  }

/* add STD extension for scan */
  err_code=StarStdFileExtName("STAR_DATA",in_filename,buffer,"");
  if(err_code!=0) {
    fprintf(stderr,"*ERR* %s: Too long file name\n",cmd_name);
    exit(1);
  }
  strcpy(in_filename,buffer);

  in_file=fopen(in_filename,"r");
  if(in_file==NULL){
    fprintf(stderr,"%s: *ERR* invalid filename=%s\n",cmd_name,in_filename);
    exit(1);
  }

/* count total profiles in the file */
  in_file=fopen(in_filename,"r");
  if(in_file==NULL){
    fprintf(stderr,"*ERR* %s: invalid filename=%s\n",cmd_name,in_filename);
    exit(1);
  }

  if(n_file==0) {
    while(1) {
      if(feof(in_file)!=0){
	break;
      }
      err_code=get_NSprofile(&NSprofile,max_ch,data_endian,in_file);
      if(err_code!=0) {
	if(feof(in_file)!=0){
	  break;
	}
	fprintf(stderr,"%s: broken file. code=%d\n",cmd_name,err_code);
	break;
      }
      n_file++;
    }
    rewind(in_file);
  }

  fprintf(stderr,"%s:total scn=%d\n",cmd_name,n_file);

/* get NSprofile data */
  for(ii=0;ii<n_file;ii++) {
    err_code=get_NSprofile(&NSprofile,max_ch,data_endian,in_file);
    if(err_code!=0) {
      switch(err_code) {
      case 10:
	fprintf(stderr,"**ERR** %s: %s has an invalid ID.\n",cmd_name,in_filename);
	fclose(in_file);
	exit(10);
      case 99:
	fprintf(stderr,"**ERR** %s: The required number of channels are too large.\n",cmd_name);
	fclose(in_file);
	exit(99);
      }
      fprintf(stderr,"**ERR** %s: The file is endned improperly.",cmd_name);
      fprintf(stderr,"Broken file or wrong endian.\n");
      fprintf(stderr,"n_file=%d\n",n_file);
      fclose(in_file);
      exit(100);
    }
    if(feof(in_file)!=0) {
      fprintf(stderr,"**ERR** %s: The file is endned improperly.",cmd_name);
      fprintf(stderr,"Broken file or wrong endian.\n");
      fprintf(stderr,"n_file=%d\n",n_file);
      fclose(in_file);
      exit(100);
    }

/* show NSprf header */
    fprintf(stdout,"==============profile No.%d=================\n",ii);
    print_NSprofileHeader(stdout,&NSprofile);

  }
  fclose(in_file);

  exit(0);
}
