/*
correct off emission

1998/5/18 T.Handa : ver.3.01
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"offcorr.prm"
#define N_KEY 15

#define HUNNING	1
#define GAUSS	2
#define BOXCAR	3
#define BINNING	4
#define PINPOINT	5

#define	CH_CENTER 0
#define OUTER_EDGE 1
#define INNER_EDGE 2

int options(argc,argv,off_scnname,bad_flag)
/*
get parameters from options
*/
int	argc;
char	*argv[];
char *off_scnname;
int *bad_flag;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  *bad_flag=1;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"off_scn");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"off_name");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"off_scnname");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"bad_flag");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* off reference scn name */
  n_scan=3;
  scan_flag[0]=0;
  scan_flag[1]=1;
  scan_flag[2]=2;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(off_scnname,got_value);
  }

/* bad flag */
  n_scan=1;
  scan_flag[0]=3;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strcmp(got_value,"off")==0) {
      *bad_flag=0;
    } else {
      *bad_flag=1;
    }
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

void StarScnConvertAndCopy(out_prof,in_prof)
PROFILE	*out_prof,*in_prof;
{
  int ch,data_ch,old_ch;
  double	v_ch_center;
  int	error;

  data_ch=(*out_prof).be_status.data_ch;

  for(ch=1;ch<=data_ch;ch++) {
    v_ch_center=StarCh2Velocity(out_prof,(double)ch);
    old_ch=rint(StarVelocity2Ch(in_prof,&error,v_ch_center));
    if(error==0) {
      (*out_prof).data[ch-1]=(*in_prof).data[old_ch-1];
    } else {
      (*out_prof).data[ch-1]=0.0;
      StarWriteSpuriousFlag(out_prof,ch,FLAGGED);
    }
  }
}

void AddOffcorrHistory(profile,in_obj,vlsr_old)
PROFILE *profile;
OBJECT *in_obj;
double vlsr_old;
{
  HISTORY	history;
  char buffer[80];

  history.size=5;
  strcpy(history.step,"offcorr");
  sprintf(buffer,"l 0=%8.3f",(*in_obj).off_lb.x);
  buffer[16]=(char)NULL;
  strcpy(history.prm[0],buffer);
  sprintf(buffer,"b 0=%8.3f",(*in_obj).off_lb.y);
  buffer[16]=(char)NULL;
  strcpy(history.prm[1],buffer);
  sprintf(buffer,"RA 0=%8.3f",(*in_obj).off_radec.x);
  buffer[16]=(char)NULL;
  strcpy(history.prm[2],buffer);
  sprintf(buffer,"Dec0=%8.3f",(*in_obj).off_radec.y);
  buffer[16]=(char)NULL;
  strcpy(history.prm[3],buffer);
  sprintf(buffer,"Vref=%8.3f",vlsr_old);
  buffer[16]=(char)NULL;
  strcpy(history.prm[4],buffer);

  StarAddHistory(profile,&history);
}

double VlsrShift(on_prof,off_prof)
PROFILE *on_prof,*off_prof;
/*
 Calc new v_lsr when *on_prof 
1999/5/8 T.Handa
*/
{
  double v_sol_on,v_sol_off;
  EP_TIME et;
  DIRECT_COS radec_cos;
  int err_code;

  err_code=ephem_time((*on_prof).obs_prm.start,&et);
  if(err_code!=0) {
    fprintf(stderr,"invalid reap_sec file\n");
    exit(999);
  }
  dir_cos((*on_prof).object.on_radec,&radec_cos);
  v_sol_on=v_solar_motion(et,COORD_RADEC,radec_cos);

  err_code=ephem_time((*off_prof).obs_prm.start,&et);
  if(err_code!=0) {
    fprintf(stderr,"invalid reap_sec file\n");
    exit(999);
  }
  dir_cos((*off_prof).object.on_radec,&radec_cos);
  v_sol_off=v_solar_motion(et,COORD_RADEC,radec_cos);

  return(v_sol_on-v_sol_off);
}

int main(argc,argv)
int	argc;
char	*argv[];
{
  PROFILE in_prof,out_prof;
  int err_code;
  int i_file,n_file;
  char id_str[BUFFER_SIZE];

  int	sdb_flag;
  char	sdb_arg[MAX_IDX_RECORD_LENGTH];

  PROFILE org_off_prof,adjust_off_prof,work_off_prof;
  FILE *fp_off;
  char off_scnname[MAX_FILE_NAME_LENGTH],full_filename[MAX_FILE_NAME_LENGTH];
  double delta_vlsr;
  double hpbw;
  int bad_flag;

/* get options */
  err_code=options(argc,argv,off_scnname,&bad_flag);

/* new reference profile load */
/* add STD extension for scan */
  if(strcmp(off_scnname,"")==0) {
    fprintf(stderr,"%s: set new reference scn name by -off_scn\n",argv[0]);
    exit(99);
  }
  err_code=StarStdFileExtName("STAR_DATA",off_scnname,full_filename,".scn");
  if(err_code!=0) {
    fprintf(stderr,"%s:Too long file name\n",argv[0]);
  }

  fp_off=fopen(full_filename,"r");
  if(fp_off==NULL) {
    fprintf(stderr,"%s : not open file %s\n",argv[0],full_filename);
    exit(99);
  }
  err_code=StarProfileLoad(fp_off,&org_off_prof);
  if(err_code!=0) {
    exit(11);
  }
  fclose(fp_off);

/* pipe in */
  if(StarPipeHeadIn(id_str,&n_file)!=NORMAL_END) {
    fprintf(stderr,"%s: Broken Pipe. Not scn files. Id=%s\n",argv[0],id_str);
    exit(1);
  }
  StarPipeHeadOut(n_file);

  for(i_file=0;i_file<n_file;i_file++) {

/* profile in */
    err_code=StarProfileIn(&in_prof,&sdb_flag,sdb_arg);
    if(err_code!=0)	{
      fprintf(stderr,"%s: Failed for Pipe-in at %dth scn\n",argv[0],i_file);
      StarPipeEndOut();
      exit(1);
    }

    hpbw=(double)in_prof.ant_status.hpbw/60.0;
    hpbw=hpbw/10.0;
    StarCopyProfile(&out_prof,&in_prof);

    if(sdb_flag==GOOD_SCAN) {
/* check the off position matching */
      if(fabs(in_prof.object.off_lb.x-org_off_prof.object.on_lb.x)>hpbw
      || fabs(in_prof.object.off_lb.y-org_off_prof.object.on_lb.y)>hpbw
      || fabs(in_prof.object.off_radec.x-org_off_prof.object.on_radec.x)>hpbw
      || fabs(in_prof.object.off_radec.y-org_off_prof.object.on_radec.y)>hpbw)
      {
	fprintf(stderr,"%s: off pos not match for %s\n",argv[0],in_prof.file_name);
	if(bad_flag!=0) sdb_flag=BAD_SCAN;
	goto next_prof;
      }

/* make a velocity converted profile for the off_position */
      StarCopyProfile(&work_off_prof,&org_off_prof);
      StarCopyProfile(&adjust_off_prof,&in_prof);

      delta_vlsr=VlsrShift(&in_prof,&work_off_prof);
      work_off_prof.rx_status.v_lsr=in_prof.rx_status.v_lsr-delta_vlsr;

      StarScnConvertAndCopy(&adjust_off_prof,&work_off_prof);

/* subraction */
      StarSubtractProfile(&out_prof,&in_prof,&adjust_off_prof);
      out_prof.rx_status.v_off=org_off_prof.rx_status.v_off;
      out_prof.object.off_lb.x=org_off_prof.object.off_lb.x;
      out_prof.object.off_lb.y=org_off_prof.object.off_lb.y;
      out_prof.object.off_radec.x=org_off_prof.object.off_radec.x;
      out_prof.object.off_radec.y=org_off_prof.object.off_radec.y;

      AddOffcorrHistory(&out_prof,&(in_prof.object),in_prof.rx_status.v_lsr);
    }

    next_prof:;
    err_code=StarProfileOut(&out_prof,sdb_flag,sdb_arg);
  }
  StarPipeEndOut();
  exit(0);
}
