/*
correct off emission: with smooth process

1998/5/17 T.Handa : ver.3.00
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStdio.h>
#include <StStdlib.h>
#include <StProfileIO.h>
#include <StProfileProc.h>
#include <StPipe.h>
#include <StAstron.h>

#define	DEFAULT_PRMFILE	"offcorr.prm"
#define N_KEY 15

#define HUNNING	1
#define GAUSS	2
#define BOXCAR	3
#define BINNING	4
#define PINPOINT	5

#define	CH_CENTER 0
#define OUTER_EDGE 1
#define INNER_EDGE 2

int options(argc,argv,v_range,v_step,v_width,kernel_type,edge_type,off_scnname)
/*
get parameters from options
*/
int	argc;
char	*argv[];
RANGE	*v_range;
double	*v_step,*v_width;
int	*kernel_type,*edge_type;
char *off_scnname;
{
	int ii;
	char **keyword;
	char **value;
	int n_key;
	int key_flag[N_KEY];
	int key_type[N_KEY];
	int scan_flag[N_KEY], n_scan;

	char *got_value;

/* default values */
	(*v_range).min=0.0;
	(*v_range).max=0.0;
	*v_step=0.0;
	*v_width=0.0;
	*kernel_type=BINNING;
	*edge_type=CH_CENTER;

/* set keywords with scan_flag */
	keyword=AllocWordsStrage(N_KEY);
	value=AllocWordsStrage(N_KEY);

	n_key=AddNewWord(keyword,"v");	key_type[n_key-1]=1;
	n_key=AddNewWord(keyword,"v_range");	key_type[n_key-1]=1;
	n_key=AddNewWord(keyword,"v_step");	key_type[n_key-1]=1;
	n_key=AddNewWord(keyword,"v_stampling");	key_type[n_key-1]=1;
	n_key=AddNewWord(keyword,"v_width");	key_type[n_key-1]=1;
	n_key=AddNewWord(keyword,"v_smooth_kernel");	key_type[n_key-1]=1;
	n_key=AddNewWord(keyword,"v_smooth_method");	key_type[n_key-1]=1;
	n_key=AddNewWord(keyword,"v_start_type");	key_type[n_key-1]=1;
	n_key=AddNewWord(keyword,"off_scn");	key_type[n_key-1]=1;
	n_key=AddNewWord(keyword,"off_name");	key_type[n_key-1]=1;
	n_key=AddNewWord(keyword,"off_scnname");	key_type[n_key-1]=1;

	StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* velocity full span */
	n_scan=2;
	scan_flag[0]=0; scan_flag[1]=1;
	got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
	if(got_value[0]!=(char)NULL) {
		StarOptionRange(got_value,v_range);
	}

/* volocity step */
	n_scan=2;
	scan_flag[0]=2; scan_flag[1]=3;
	got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
	if(got_value[0]!=(char)NULL) {
		*v_step=atof(got_value);
	}

/* volocity width */
	n_scan=1;
	scan_flag[0]=4;
	got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
	if(got_value[0]!=(char)NULL) {
		*v_width=atof(got_value);
	}

/* smoothing kernal type */
	n_scan=2;
	scan_flag[0]=5; scan_flag[1]=6;
	got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
	if(got_value[0]!=(char)NULL) {
		if(strcmp(got_value,"hunning")==0)
			*kernel_type=HUNNING;
		else if(strcmp(got_value,"gauss")==0)
			*kernel_type=GAUSS;
		else if(strcmp(got_value,"box")==0)
			*kernel_type=BOXCAR;
		else if(strcmp(got_value,"boxcar")==0)
			*kernel_type=BOXCAR;
		else if(strcmp(got_value,"bin")==0)
			*kernel_type=BINNING;
		else if(strcmp(got_value,"pin")==0)
			*kernel_type=PINPOINT;
		else if(strcmp(got_value,"pinpoint")==0)
			*kernel_type=PINPOINT;
	}

/* edge type */
	n_scan=1;
	scan_flag[0]=7;
	got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
	if(got_value[0]!=(char)NULL) {
		if(strcmp(got_value,"edge")==0 || strcmp(got_value,"outer_edge")==0)
			*edge_type=OUTER_EDGE;
		else if(strcmp(got_value,"inner_edge")==0)
			*edge_type=INNER_EDGE;
	}

/* off reference scn name */
  n_scan=3;
  scan_flag[0]=8;
  scan_flag[1]=9;
  scan_flag[2]=10;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(off_scnname,got_value);
  }


/* free memory areas */
	StarFreeAllPrms(N_KEY,key_flag,value);
	free(keyword);

	return(0);
}

static double StarIntegratedIntensity(profile,v_range,avr_flag)
PROFILE	*profile;
RANGE	*v_range;
int	avr_flag;
{
	RANGE_long	ch_range;
	RANGE	v_for_ch;
	double	ta,v_all;
	double	scale;

	double	v_min,v_max,v,v_width;
	int	ch,data_ch;
	int	err_code;

	ta=0.0;
	v_all=0.0;
	scale=(double)(*profile).rx_status.scale;
	data_ch=(*profile).be_status.data_ch;

/* integrate over asigned part */
	for(ch=1;ch<=data_ch;ch++) {
		v_for_ch.min=StarCh2Velocity(profile,(double)ch-0.5);
		v_for_ch.max=StarCh2Velocity(profile,(double)ch+0.5);
		StarSortRange(&v_for_ch);
/* integrate only non-spurious channels */
		if(StarReadSpuriousFlag(profile,ch)==UNFLAGGED) {
/* range of integration */
		if(v_for_ch.min>=(*v_range).min && v_for_ch.max<=(*v_range).max) {
	/* an original channel is included in a target channel */
			v_width=v_for_ch.max-v_for_ch.min;
			ta=ta+(double)(*profile).data[ch-1]*v_width*scale;
			v_all=v_all+v_width;
		} else if(v_for_ch.min<(*v_range).min && v_for_ch.max>(*v_range).max) {
	/* an original channel includes a integration range */
			v_width=(*v_range).max-(*v_range).min;
			ta=ta+(double)(*profile).data[ch-1]*v_width*scale;
			v_all=v_all+v_width;
		} else if(v_for_ch.min<=(*v_range).min && v_for_ch.max>(*v_range).min) {
	/* an original channel includes lower boundary of the integration range */
			v_width=v_for_ch.max-(*v_range).min;
			ta=ta+(double)(*profile).data[ch-1]*v_width*scale;
			v_all=v_all+v_width;
		} else if(v_for_ch.min<=(*v_range).max && v_for_ch.max>(*v_range).max) {
	/* an original channel includes upper boundary of the integration range */
			v_width=(*v_range).max-v_for_ch.min;
			ta=ta+(double)(*profile).data[ch-1]*v_width*scale;
			v_all=v_all+v_width;
		}
	}

	} /* end of spurious desition */
	if(avr_flag!=0) {
		if(v_all!=0.0) ta=ta/v_all;
		else ta=0.0;
	}

	return(ta);
}

static double StarSmoothedTa(profile,v_center,v_width,kernel_type,error)
PROFILE	*profile;
double	v_center,v_width;
int	kernel_type;
int	*error;
{
	RANGE_long	ch_range;
	double	ta,weight;
	double	scale;

	int	ch,data_ch;
	int	err_code;
	double	ch_center,ch_width;
	double	ff,vv,xx;
	double	v_window,kernel_width;

	ta=0.0;
	weight=0.0;
	scale=(double)(*profile).rx_status.scale;
	data_ch=(*profile).be_status.data_ch;

/* center channel */
	ch_center=StarCh2Velocity(profile,v_center);

/* integration window in channel */
	switch(kernel_type) {
	case BOXCAR:
		v_window=v_width;
		kernel_width=v_width;
		break;
	case HUNNING:
		v_window=v_width*0.75;
		kernel_width=v_width/2.0;
		break;
	case GAUSS: default:
		v_window=v_width*3.0;
		kernel_width=v_width/1.665109222; /* 2*sqrt(log(2.0)) */
		break;
	}
	vv=v_center-v_window;
	xx=StarVelocity2Ch(profile,&err_code,vv);
	ch_range.min=(int)rint(xx);
	switch(err_code) {
	case NORMAL_END:
		break;
	case OVER_FLOW:
fprintf(stderr,"smooth:%s v_min=%f too high ch.\n",(*profile).file_name,vv);
		ch_range.min=data_ch;
		break;
	case UNDER_FLOW:
fprintf(stderr,"smooth:%s v_min=%f too low ch.\n",(*profile).file_name,vv);
		ch_range.min=1;
		break;
	case UN_CONVERGE:
fprintf(stderr,"smooth:%s v_min=%f unconverge.\n",(*profile).file_name,vv);
		*error=err_code;
		return(0.0);
	default:
fprintf(stderr,"smooth:%s v_min=%f code=%d.\n",(*profile).file_name,vv,err_code);
		*error=err_code;
		return(0.0);
	}

	vv=v_center+v_window;
	xx=StarVelocity2Ch(profile,&err_code,vv);
	ch_range.max=(int)rint(xx);
	switch(err_code) {
	case NORMAL_END:
		break;
	case OVER_FLOW:
fprintf(stderr,"smooth:%s v_max=%f too high ch.\n",(*profile).file_name,vv);
		ch_range.max=data_ch;
		break;
	case UNDER_FLOW:
fprintf(stderr,"smooth:%s v_max=%f too low ch.\n",(*profile).file_name,vv);
		ch_range.max=1;
		break;
	case UN_CONVERGE:
fprintf(stderr,"smooth:%s v_min=%f unconverge.\n",(*profile).file_name,vv);
		*error=err_code;
		return(0.0);
	default:
fprintf(stderr,"smooth:%s v_max=%f code=%d.\n",(*profile).file_name,vv,err_code);
		*error=err_code;
		return(0.0);

	}

	StarSortRangeLong(&ch_range);

/* integrate for a channel */
	weight=0.0;
	for(ch=ch_range.min;ch<=ch_range.max;ch++) {
		if(StarReadSpuriousFlag(profile,ch)==UNFLAGGED) {
			vv=StarCh2Velocity(profile,(double)ch);
			ff=(vv-v_center)/kernel_width;
			ff=fabs(ff);
			switch(kernel_type) {
			case BOXCAR:
				if(ff<0.5) ff=1.0;
				else ff=0.0;
				break;
			case HUNNING:
				if(ff<0.5) ff=1.0;
				else if(ff<1.5) ff=0.5;
				else ff=0.0;
				break;
			case GAUSS:
				ff= -ff*ff;
				ff=exp(ff);
				break;
			}
			ta=ta+(*profile).data[ch-1]*ff;
			weight=weight+ff;
		}
	}
	if(weight>0.0) {
		ta=ta/weight*scale;
		*error=0;
	} else {
		ta=0.0;
		*error=1;
	}
	return(ta);
}

void StarScnSmoothAndConvert(in_prof,out_prof,v_range,v_width,kernel_type)
PROFILE	*in_prof,*out_prof;
RANGE	v_range;	/* whole velocity range for the new backend */
double	v_width;	/* smoothed width for the old backend */
int	kernel_type;
{
	RANGE integ_range;
	double ta;
	int ch,data_ch,old_ch;
	static int	avr_flag=1;
	double	v_ch_center;
	int	error;

	data_ch=(*out_prof).be_status.data_ch;

	switch(kernel_type) {
	case BINNING:
		for(ch=1;ch<=data_ch;ch++) {
			v_ch_center=StarCh2Velocity(out_prof,(double)ch);
			integ_range.min=v_ch_center-v_width/2.0;
			integ_range.max=v_ch_center+v_width/2.0;
			ta=StarIntegratedIntensity(in_prof,&integ_range,avr_flag);
			(*out_prof).data[ch-1]=(float)ta;
		}
		break;
	case PINPOINT:
		for(ch=1;ch<=data_ch;ch++) {
			v_ch_center=StarCh2Velocity(out_prof,(double)ch);
			old_ch=rint(StarVelocity2Ch(in_prof,&error,v_ch_center));
			if(error==0) {
				(*out_prof).data[ch-1]=(*in_prof).data[old_ch-1];
			} else {
				(*out_prof).data[ch-1]=0.0;
				StarWriteSpuriousFlag(out_prof,ch,FLAGGED);
			}
		}
		break;
	default:
		for(ch=1;ch<=data_ch;ch++) {
			v_ch_center=StarCh2Velocity(out_prof,(double)ch);
			ta=StarSmoothedTa(in_prof,v_ch_center,v_width,kernel_type,&error);
			if(error==0) {
				(*out_prof).data[ch-1]=(float)ta;
			} else {
				(*out_prof).data[ch-1]=0.0;
				StarWriteSpuriousFlag(out_prof,ch,FLAGGED);
			}
		}
		break;
	}
}

void MakeNewBEDispersion(profile,v_range,v_step,kernel_type)
PROFILE *profile;
RANGE	v_range;
double	v_step;
int	kernel_type;
{
	double obs_freq;
	double v_span;

	RANGE freq_range;
	double freq_step;
	double freq_span,center_freq;

	long new_ch_numb,data_ch;

/* new data_ch */
	data_ch=(*profile).be_status.data_ch;

/* new center channel */
	(*profile).be_status.center_ch=(double)(data_ch+1)/2.0;
	(*profile).be_status.basis_ch=(int)(*profile).be_status.center_ch;

/* frequency range on the new backend */
	center_freq=(*profile).be_status.center_freq;

	obs_freq=(*profile).rx_status.obs_freq;
	freq_range.max=center_freq-v_range.min/(LIGHT_SPEED)*obs_freq*1.0e3;
	freq_range.min=center_freq-v_range.max/(LIGHT_SPEED)*obs_freq*1.0e3;
	freq_span=freq_range.max-freq_range.min;

/* always increase backend frequency in channel */
	if((*profile).be_status.disp1 <0.0)
		(*profile).be_status.sign= -(*profile).be_status.sign;

/* fix the linear dispersion */
	(*profile).be_status.disp2=0.0;
	(*profile).be_status.disp3=0.0;
	(*profile).be_status.disp4=0.0;

	(*profile).be_status.disp1=freq_span/(double)(data_ch-1);
	(*profile).be_status.disp0=(*profile).be_status.center_freq;

	(*profile).rx_status.scale=1.0;
}



void AddOffcorrHistory(profile,in_obj,vlsr_old,v_range,v_step,v_width,kernel_type)
PROFILE *profile;
OBJECT *in_obj;
double vlsr_old;
RANGE v_range;
double v_step,v_width;
int kernel_type;
{
  HISTORY	history;
  char buffer[80];

  history.size=9;
  strcpy(history.step,"offcorr");
  sprintf(buffer,"refL=%8.3f",(*in_obj).off_lb.x);
  buffer[16]=(char)NULL;
  strcpy(history.prm[0],buffer);
  sprintf(buffer,"refB=%8.3f",(*in_obj).off_lb.y);
  buffer[16]=(char)NULL;
  strcpy(history.prm[1],buffer);
  sprintf(buffer,"refR=%8.3f",(*in_obj).off_radec.x);
  buffer[16]=(char)NULL;
  strcpy(history.prm[2],buffer);
  sprintf(buffer,"refD=%8.3f",(*in_obj).off_radec.y);
  buffer[16]=(char)NULL;
  strcpy(history.prm[3],buffer);
  sprintf(buffer,"Vref=%8.3f",vlsr_old);
  buffer[16]=(char)NULL;
  strcpy(history.prm[4],buffer);
  sprintf(buffer,"Vstep=%7.3f",v_step);
  buffer[16]=(char)NULL;
  strcpy(history.prm[5],buffer);
  sprintf(buffer,"Dv=%8.3f",v_width);
  buffer[16]=(char)NULL;
  strcpy(history.prm[6],buffer);
  sprintf(buffer,"v1=%8.2f",v_range.min);
  buffer[16]=(char)NULL;
  strcpy(history.prm[7],buffer);
  sprintf(buffer,"v2=%8.2f",v_range.max);
  buffer[16]=(char)NULL;
  strcpy(history.prm[8],buffer);
  switch(kernel_type) {
  case HUNNING:
    strcpy(history.prm[8],"type=hunning"); break;
  case GAUSS:
    strcpy(history.prm[8],"type=gauss"); break;
  case BOXCAR:
    strcpy(history.prm[8],"type=boxcar"); break;
  case BINNING:
    strcpy(history.prm[8],"type=binning"); break;
  case PINPOINT:
    strcpy(history.prm[8],"type=pin"); break;
  default:
    strcpy(history.prm[8],"type=unknow"); break;
  }

  StarAddHistory(profile,&history);
}

void ResetOptions(v_range,v_step,v_width,kernel_type,edge_type)
RANGE *v_range;
double *v_step,*v_width;
int kernel_type,edge_type;
{
	int n_ch;

	if(*v_step<=0.0 && *v_width>0.0) {
		switch(kernel_type) {
		case BINNING: case PINPOINT:
			*v_step= *v_width; break;
		default:
			*v_step= *v_width/2.0; break;
		}
	} else if(*v_step>0.0 && *v_width<=0.0) {
		switch(kernel_type) {
		case BINNING: case PINPOINT:
			*v_width= *v_step; break;
		default:
			*v_width= *v_step*2.0; break;
		}
	} else if(*v_step<=0.0 && *v_width<=0.0) {
		if((*v_range).max==(*v_range).min) {
			(*v_range).max=(*v_range).min+(double)(MAX_CH-1);
		}
		*v_step= ((*v_range).max-(*v_range).min)/(double)(MAX_CH-1);
		switch(kernel_type) {
		case BINNING: case PINPOINT:
			*v_width= *v_step; break;
		default:
			*v_width= *v_step*2.0; break;
		}
	}

/* check MAX_CH */
	n_ch=(int)ceil(((*v_range).max - (*v_range).min)/ *v_step);
	if(n_ch>MAX_CH) (*v_range).max= (*v_range).min+ *v_step * (double)(MAX_CH-1);

/* edge type : range values are alway outer_edge */
	switch(edge_type) {
	case INNER_EDGE:
		(*v_range).min=(*v_range).min - *v_step;
		(*v_range).max=(*v_range).max + *v_step;
		break;
	case CH_CENTER:
		(*v_range).min=(*v_range).min - *v_step/2.0;
		(*v_range).max=(*v_range).max + *v_step/2.0;
		break;
	}

	return;
}

double VlsrShift(on_prof,off_prof)
PROFILE *on_prof,*off_prof;
/*
 Calc new v_lsr when *on_prof 
1999/5/8 T.Handa
*/
{
  double v_sol_on,v_sol_off;
  EP_TIME et;
  DIRECT_COS radec_cos;
  int err_code;

  err_code=ephem_time((*on_prof).obs_prm.start,&et);
  if(err_code!=0) {
    fprintf(stderr,"invalid reap_sec file\n");
    exit(999);
  }
  dir_cos((*on_prof).object.on_radec,&radec_cos);
  v_sol_on=v_solar_motion(et,COORD_RADEC,radec_cos);

  err_code=ephem_time((*off_prof).obs_prm.start,&et);
  if(err_code!=0) {
    fprintf(stderr,"invalid reap_sec file\n");
    exit(999);
  }
  dir_cos((*off_prof).object.on_radec,&radec_cos);
  v_sol_off=v_solar_motion(et,COORD_RADEC,radec_cos);

  return(v_sol_on-v_sol_off);
}

int main(argc,argv)
int	argc;
char	*argv[];
{
  PROFILE in_prof,out_prof;
  PRF_HEADER list_flag;
  int err_code;
  int i_file,n_file;
  char id_str[BUFFER_SIZE];

  int	sdb_flag;
  char	sdb_arg[MAX_IDX_RECORD_LENGTH];

  RANGE	v_range;
  double v_step,v_width;
  int kernel_type,edge_type;
  RANGE_long ch_range; /* for clear flags */

  PROFILE org_off_prof,new_off_prof,work_prof;
  FILE *fp_off;
  char off_scnname[MAX_FILE_NAME_LENGTH],full_filename[MAX_FILE_NAME_LENGTH];
  RX_STATUS org_rx;
  double new_vlsr,old_vlsr;
  double hpbw;

  ch_range.min=1;

/* get options */
  err_code=options(argc,argv,&v_range,&v_step,&v_width,&kernel_type,&edge_type,off_scnname);
  StarSortRange(&v_range);
  ResetOptions(&v_range,&v_step,&v_width,kernel_type,edge_type);

/* new reference profile load */
/* add STD extension for scan */
  if(strcmp(off_scnname,"")==0) {
    fprintf(stderr,"%s: set new reference scn name by -off_scn\n",argv[0]);
    exit(99);
  }
  err_code=StarStdFileExtName("STAR_DATA",off_scnname,full_filename,".scn");
  if(err_code!=0) {
    fprintf(stderr,"%s:Too long file name\n",argv[0]);
  }

  fp_off=fopen(full_filename,"r");
  if(fp_off==NULL) {
    fprintf(stderr,"%s : not open file %s\n",argv[0],full_filename);
    exit(99);
  }
  err_code=StarProfileLoad(fp_off,&org_off_prof);
  if(err_code!=0) {
    exit(11);
  }
  fclose(fp_off);
  StarCopyRxstatus(&org_rx,&(org_off_prof.rx_status));

/* pipe in */
  if(StarPipeHeadIn(id_str,&n_file)!=NORMAL_END) {
    fprintf(stderr,"%s: Broken Pipe. Not scn files. Id=%s\n",argv[0],id_str);
    exit(1);
  }
  StarPipeHeadOut(n_file);

  for(i_file=0;i_file<n_file;i_file++) {

    StarCopyRxstatus(&(org_off_prof.rx_status),&org_rx);
/* profile in */
    err_code=StarProfileIn(&in_prof,&sdb_flag,sdb_arg);
    if(err_code!=0)	{
      StarPipeEndOut();
      exit(1);
    }

    hpbw=(double)in_prof.ant_status.hpbw/60.0;
    hpbw=hpbw/10.0;

    if(sdb_flag==GOOD_SCAN) {
/* check the off position matching */
      if(fabs(in_prof.object.off_lb.x-org_off_prof.object.on_lb.x)<hpbw
      || fabs(in_prof.object.off_lb.y-org_off_prof.object.on_lb.y)<hpbw
      || fabs(in_prof.object.off_radec.x-org_off_prof.object.on_radec.x)<hpbw
      || fabs(in_prof.object.off_radec.y-org_off_prof.object.on_radec.y)<hpbw)
      {
	goto next_prof;
      }

/* make a smoothed (v_converted) profile for the off_position */
      StarCopyProfile(&new_off_prof,&org_off_prof);
      ch_range.max=new_off_prof.be_status.data_ch;
      new_off_prof.be_status.data_ch=(int)ceil((v_range.max-v_range.min)/v_step);
      new_off_prof.rx_status.v_lsr=(v_range.max+v_range.min)/2.0;

      StarRWriteSpuriousFlags(&new_off_prof,ch_range,UNFLAGGED);
      StarRWriteBaselineFlags(&new_off_prof,ch_range,UNFLAGGED);

      MakeNewBEDispersion(&new_off_prof,v_range,v_step,kernel_type);

/* calcurate new vlsr for reference */
      new_vlsr=in_prof.rx_status.v_lsr-VlsrShift(&in_prof,&org_off_prof);
      org_off_prof.rx_status.v_lsr=new_vlsr;

      StarScnSmoothAndConvert(&org_off_prof,&new_off_prof,v_range,v_width,kernel_type);

/* make a smoothed profile from on the position */
      StarCopyProfile(&out_prof,&in_prof);
      ch_range.max=in_prof.be_status.data_ch;
      new_off_prof.be_status.data_ch=(int)ceil((v_range.max-v_range.min)/v_step);
      out_prof.rx_status.v_lsr=(v_range.max+v_range.min)/2.0;

      StarRWriteSpuriousFlags(&out_prof,ch_range,UNFLAGGED);
      StarRWriteBaselineFlags(&out_prof,ch_range,UNFLAGGED);

      MakeNewBEDispersion(&out_prof,v_range,v_step,kernel_type);

/* calcurate new vlsr for reference */

      StarScnSmoothAndConvert(&in_prof,&out_prof,v_range,v_width,kernel_type);

/* subraction */
      StarCopyProfile(&work_prof,&out_prof); /* for header copy */
      StarSubtractProfile(&work_prof,&out_prof,&new_off_prof);
      strcpy(work_prof.file_id,in_prof.file_id);
      strcpy(work_prof.file_name,in_prof.file_name);
      work_prof.rx_status.v_off=org_off_prof.rx_status.v_off;
      work_prof.object.off_lb.x=org_off_prof.object.off_lb.x;
      work_prof.object.off_lb.y=org_off_prof.object.off_lb.y;
      work_prof.object.off_radec.x=org_off_prof.object.off_radec.x;
      work_prof.object.off_radec.y=org_off_prof.object.off_radec.y;

      old_vlsr=in_prof.rx_status.v_lsr;
      AddOffcorrHistory(&work_prof,&(in_prof.object),old_vlsr,v_range,v_step,v_width,kernel_type);

      err_code=StarProfileOut(&work_prof,sdb_flag,sdb_arg);
    } else if(sdb_flag==BAD_SCAN) {
      err_code=StarProfileOut(&in_prof,sdb_flag,sdb_arg);
    } else {
      err_code=StarProfileOut(&in_prof,sdb_flag,sdb_arg);
    }

    next_prof:;
  }
  StarPipeEndOut();
  exit(0);
}
