/*
set position alais
between 0 and 360, or between -180 and +180

2003/1/31 T.Handa : ver.3.00
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"posalias.prm"
#define N_KEY 30
#define N_BLOCK	10

#define ZERO_CENTER 0
#define ROUND_PRINCIPAL 1

int options(argc,argv,mode)
/*
get parameters from options
*/
int argc;
char *argv[];
int *mode;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  *mode=ROUND_PRINCIPAL;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"mode");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* mode */
  scan_flag[0]= 0;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(strncmp(got_value,"p",1)==0) {
    *mode=ROUND_PRINCIPAL;
  } else if(strncmp(got_value,"z",1)==0) {
    *mode=ZERO_CENTER;
  } else if(got_value[0]=='0') {
    *mode=ROUND_PRINCIPAL;
  } else if(got_value[0]=='-') {
    *mode=ZERO_CENTER;
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

double ReviseAngleAlias(angle,mode)
double angle; /* <in> deg */
int mode; /* <in> =0: principal value 0<=x<360, =1: 0 center -180<x<=+180 */
{
  switch(mode) {
  case ROUND_PRINCIPAL:
    while(angle <  0.0) angle=angle+360.0;
    while(angle >= 360.0) angle=angle-360.0;
    break;
  case ZERO_CENTER: default:
    while(angle <= -180.0) angle=angle+360.0;
    while(angle >  +180.0) angle=angle-360.0;
    break;
  }

  return(angle);
}

void Revise4LongLatPos(xy,mode)
XY_POS *xy; /* <i/o> */
int mode;
{
  (*xy).x=ReviseAngleAlias((*xy).x,mode); /* longitude 0<=x<360) */
  (*xy).y=ReviseAngleAlias((*xy).y,ZERO_CENTER); /* latitude -90=<y<=+90 */
  return;
}

void Revise4PositionAlias(profile,mode)
PROFILE *profile; /* <i/o> */
int mode;
{
  Revise4LongLatPos(&((*profile).object.on_lb),mode);
  Revise4LongLatPos(&((*profile).object.on_radec),mode);
  Revise4LongLatPos(&((*profile).object.on_azel),mode);
  Revise4LongLatPos(&((*profile).object.off_lb),mode);
  Revise4LongLatPos(&((*profile).object.off_radec),mode);
  Revise4LongLatPos(&((*profile).object.orig_xy),mode);
  Revise4LongLatPos(&((*profile).object.on_xy),mode);

  return;
}

int main(argc,argv)
int argc;
char *argv[];
{
  PROFILE profile;
  int i_file,n_file;
  char id_str[BUFFER_SIZE];
  int err_code;

  int mode;

  int sdb_flag;
  char sdb_arg[MAX_IDX_RECORD_LENGTH];

/* get option */
  options(argc,argv,&mode);

/* Pipeline head */
  StarPipeHeadIn(id_str,&n_file);
  StarPipeHeadOut(n_file);

  for(i_file=0;i_file<n_file;i_file++) {

/* ProfileIn */
    err_code=StarProfileIn(&profile,&sdb_flag,sdb_arg);

    if(err_code!=0)	{
      StarPipeEndOut();
      exit(1);
    }

    if(sdb_flag==GOOD_SCAN || sdb_flag==BAD_SCAN) {

/* revise position alias */
      Revise4PositionAlias(&profile,mode);
    }

/* ProfileOut */
    err_code=StarProfileOut(&profile,sdb_flag,sdb_arg);
  }

/* End of Pipeline */
  err_code=StarPipeEndIn(id_str);
  if(err_code!=0)
    fprintf(stderr,"%s: Pipeline end invalid=%s\n",argv[0],id_str);

  StarPipeEndOut();
  exit(0);
}
