/*
correct positions in scn header

2003/1/ 7 T.Handa : ver.3.00
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"poscorr.prm"
#define N_KEY 30
#define N_BLOCK	10

#define J2000toB1950 1
#define B1950toJ2000 2
#define AnytimetoB1950 11

int options(argc,argv,epoch_cnv)
/*
get parameters from options
*/
int argc;
char *argv[];
int *epoch_cnv;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  *epoch_cnv=J2000toB1950;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"convert_type");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* convert_type */
  n_scan=1;
  scan_flag[0]= 0;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strcmp(got_value,"toJ2000")==0) {
      *epoch_cnv=J2000toB1950;
    }
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

static DIRECT_COS matrix_vector(mat,vec)
double *mat;
DIRECT_COS vec;
{
  DIRECT_COS out;

  out.l=mat[0]*vec.l+mat[1]*vec.m+mat[2]*vec.n;
  out.m=mat[3]*vec.l+mat[4]*vec.m+mat[5]*vec.n;
  out.n=mat[6]*vec.l+mat[7]*vec.m+mat[8]*vec.n;

  return(out);
}

XY_POS StarAstronFromJ2000ToB1950(org_pos)
XY_POS org_pos;
{
  static double 
    matrix_J2000toB1950[9]={+0.9999257080,+0.0111789382,+0.0048590039,
                            -0.0111789382,+0.9999375133,-0.0000271579,
                            -0.0048590038,-0.0000271626,+0.9999881946};
  static double
    aa[3]={-0.0227/240.0,-0.035/240.0,-0.00085/240.0};
  static double
    bb[2]={-0.341/3600.0,-0.029/3600.0};
  static double ra_offset= +11.25*15.0;

  XY_POS new_pos,better_pos;
  DIRECT_COS org_dir,new_dir;
  double dec_rad,ra_rad;

  dir_cos(org_pos,&org_dir);
  new_dir=matrix_vector(matrix_J2000toB1950,org_dir);

  direction(new_dir,&new_pos);

  ra_rad=(new_pos.x+ra_offset)*PI/180.0;
  dec_rad=new_pos.y*PI/180.0;

  better_pos.x=new_pos.x+aa[0]*sin(ra_rad)/cos(dec_rad)+aa[1]+aa[2]*50.0;
  better_pos.y=new_pos.y+bb[0]*cos(ra_rad)*sin(dec_rad)+bb[1]*cos(new_pos.y*PI/180.0);

  return(better_pos);
}

void add_history(profile,epoch_cnv)
PROFILE *profile;
int epoch_cnv;
{
  HISTORY hist;
  char buffer[80];

  hist.size=1;
  strcpy(hist.step,"PosCor");

  switch(epoch_cnv) {
  case J2000toB1950:
  default:
    sprintf(buffer,"J2000toB1950");
    buffer[16]=(char)NULL;
    strcpy(hist.prm[0],buffer);
  }
  StarAddHistory(profile,&hist);
  return;
}

void convert_from_J2000(prf)
PROFILE *prf;
{
  XY_POS new_pos;

  new_pos=StarAstronFromJ2000ToB1950((*prf).object.on_radec);
  (*prf).object.on_radec.x=new_pos.x;
  (*prf).object.on_radec.y=new_pos.y;
  radec502lb(&((*prf).object.on_lb),(*prf).object.on_radec);
  new_pos=StarAstronFromJ2000ToB1950((*prf).object.off_radec);
  (*prf).object.off_radec.x=new_pos.x;
  (*prf).object.off_radec.y=new_pos.y;
  radec502lb(&((*prf).object.off_lb),(*prf).object.off_radec);

  return;
}

int main(argc,argv)
int argc;
char *argv[];
{
  PROFILE profile;
  int i_file,n_file;
  char id_str[BUFFER_SIZE];
  int err_code;

  int epoch_cnv;

  int sdb_flag;
  char sdb_arg[MAX_IDX_RECORD_LENGTH];

/* get option */
  options(argc,argv,&epoch_cnv);

/* Pipeline head */
  StarPipeHeadIn(id_str,&n_file);
  StarPipeHeadOut(n_file);

  for(i_file=0;i_file<n_file;i_file++) {
/* ProfileIn */
    err_code=StarProfileIn(&profile,&sdb_flag,sdb_arg);

    if(err_code!=0)	{
      StarPipeEndOut();
      exit(1);
    }

    if(sdb_flag==GOOD_SCAN) {
      switch(epoch_cnv) {
      case J2000toB1950:
      default:
	convert_from_J2000(&profile);
	break;
      }
    }

    add_history(&profile,epoch_cnv);

/* ProfileOut */
    err_code=StarProfileOut(&profile,sdb_flag,sdb_arg);
  }

/* End of Pipeline */
  err_code=StarPipeEndIn(id_str);
  if(err_code!=0)
    fprintf(stderr,"%s: Pipeline end invalid=%s\n",argv[0],id_str);

  StarPipeEndOut();
  exit(0);
}
