/*
reset position of scn on the nearest grid point

2003/2/3 T.Handa : ver.3.00
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"posgrid.prm"
#define N_KEY 30
#define N_BLOCK	10

int options(argc,argv,coord,orig,grid,msg_filename)
/*
get parameters from options
*/
int argc;
char **argv;
int *coord;
XY_POS *orig;
XY_POS *grid;
char *msg_filename;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

  RANGE tmp_range;

/* default values */
  *coord=COORD_LB;
  (*orig).x=0.0;
  (*orig).y=0.0;
  (*grid).x=0.0;
  (*grid).y=0.0;
  strcpy(msg_filename,"");

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"coordinates");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"coord_sys");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"origin");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"grid");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"msg_file");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* coordinate */
  scan_flag[0]= 0;
  scan_flag[1]= 1;
  n_scan=2;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strcmp(got_value,"lb")==0)
      *coord=COORD_LB;
    else if(strcmp(got_value,"radec")==0)
      *coord=COORD_RADEC;
    else if(strcmp(got_value,"radec-app")==0)
      *coord=COORD_RADEC_APP;
    else if(strcmp(got_value,"azel")==0)
      *coord=COORD_AZEL;
    else
      *coord=(unsigned char)atoi(got_value);
  }

/* origin */
  scan_flag[0]= 2;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    StarOptionAngleRange(got_value,&tmp_range);
    (*orig).x=tmp_range.min;
    (*orig).y=tmp_range.max;
  }

/* grid */
  scan_flag[0]= 3;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    StarOptionAngleRange(got_value,&tmp_range);
    (*grid).x=tmp_range.min;
    (*grid).y=tmp_range.max;
  }

/* message file */
  scan_flag[0]= 4;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(msg_filename,got_value);
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}


void RoundGrid(new_pt,old_pt,orig,grid)
XY_POS *new_pt; /* <out> rounded values */
XY_POS old_pt; /* <in> to be processed */
XY_POS orig; /* <in> grid origin */
XY_POS grid; /* <in> grid iterval */
{
  (*new_pt).x=rint((old_pt.x-orig.x)/grid.x)*grid.x+orig.x;
  (*new_pt).y=rint((old_pt.y-orig.y)/grid.y)*grid.y+orig.y;

  return;
}

void ResetOnTheGrid(profile,coord,orig,grid,msg_fp,delta_pos,delta_dist)
PROFILE *profile; /* <i/o> */
int coord; /* <in> coordinate ID */
XY_POS orig,grid; /* <in> grid origin and interval (deg) */
FILE *msg_fp; /* <in> message out file */
XY_POS *delta_pos; /* <out> shift along axes (arcsec) */
double *delta_dist; /* <out> shift distance (arcsec) */
{
  HISTORY hist;
  char buffer[256];
  XY_POS old_pt,new_pt;
  XY_POS delta;
  double dist,xx,yy;

/* set the grid size */
  if(grid.x==0.0) grid.x=(*profile).ant_status.hpbw;
  if(grid.y==0.0) grid.y=(*profile).ant_status.hpbw;

/* gridding */
  if(coord==COORD_LB) {
    old_pt.x=(*profile).object.on_lb.x;
    old_pt.y=(*profile).object.on_lb.y;
    RoundGrid(&new_pt,old_pt,orig,grid);
    (*profile).object.on_lb.x=new_pt.x;
    (*profile).object.on_lb.y=new_pt.y;
  } else if(coord==COORD_RADEC) {
    old_pt.x=(*profile).object.on_radec.x;
    old_pt.y=(*profile).object.on_radec.y;
    RoundGrid(&new_pt,old_pt,orig,grid);
    (*profile).object.on_radec.x=new_pt.x;
    (*profile).object.on_radec.y=new_pt.y;
  } else if(coord==COORD_RADEC_APP) {
    old_pt.x=(*profile).object.on_radec.x;
    old_pt.y=(*profile).object.on_radec.y;
    RoundGrid(&new_pt,old_pt,orig,grid);
    (*profile).object.on_radec.x=new_pt.x;
    (*profile).object.on_radec.y=new_pt.y;
  } else if(coord==COORD_AZEL) {
    old_pt.x=(*profile).object.on_azel.x;
    old_pt.y=(*profile).object.on_azel.y;
    RoundGrid(&new_pt,old_pt,orig,grid);
    (*profile).object.on_azel.x=new_pt.x;
    (*profile).object.on_azel.y=new_pt.y;
  }

/* Add History */
  hist.size=3;
  strcpy(hist.step,"PosGrd");

  if(coord==COORD_LB) {
    strcpy(hist.prm[0],"coord=lb");
  } else if(coord==COORD_RADEC) {
    strcpy(hist.prm[0],"coord=RADec");
  } else if(coord==COORD_RADEC_APP) {
    strcpy(hist.prm[0],"coord=RADec");
  } else if(coord==COORD_AZEL) {
    strcpy(hist.prm[0],"coord=AzEl");
  }
  sprintf(buffer,"long=%f",old_pt.x);
  buffer[16]=(char)NULL;
  strcpy(hist.prm[1],buffer);
  sprintf(buffer,"lat=%f",old_pt.y);
  buffer[16]=(char)NULL;
  strcpy(hist.prm[2],buffer);

  StarAddHistory(profile,&hist);

/* make a message */
  if(msg_fp!=NULL) {
    delta.x=(new_pt.x-old_pt.x)*3600.0;
    delta.y=(new_pt.y-old_pt.y)*3600.0;

    xx=delta.x*cos(new_pt.y/180.0*PI);
    xx=xx*xx;
    yy=delta.y;
    yy=yy*yy;
    dist=sqrt(xx+yy);

    fprintf(msg_fp,"%s delta=(%f\",%f\") dist=%f\"\n",(*profile).file_name,delta.x,delta.y,dist);

    (*delta_pos).x=delta.x;
    (*delta_pos).y=delta.y;
    *delta_dist=dist;
  }

  return;
}

int main(argc,argv)
int argc;
char *argv[];
{
  PROFILE profile;
  int i_file,n_file;
  char id_str[BUFFER_SIZE];
  int err_code;

  int coord;
  XY_POS orig,grid;

  int sdb_flag;
  char sdb_arg[MAX_IDX_RECORD_LENGTH];
  char msg_filename[MAX_FILE_NAME_LENGTH];
  FILE *msg_fp;
  XY_POS delta_pos,avr_pos; /* shift along axes (arcsec) */
  double delta_dist,avr_dist; /* shift distance (arcsec) */
  int n_point;
  XY_POS max_delta_pos;
  double max_shift;

/* get option */
  options(argc,argv,&coord,&orig,&grid,msg_filename);

  if(strcmp(msg_filename,"")==0 || strcmp(msg_filename,"no")==0 || strcmp(msg_filename," ")==0) {
    msg_fp=NULL;
  } else {
    if(strcmp(msg_filename,"-")==0) {
      msg_fp=stderr;
    } else {
      msg_fp=fopen(msg_filename,"w");
      if(msg_fp==NULL) {
	msg_fp=stderr;
	fprintf(stderr,"%s: Cannot open a file for messages. Use stderr.\n",argv[0]);
      }
    }
    
  }

/* Pipeline head */
  StarPipeHeadIn(id_str,&n_file);
  StarPipeHeadOut(n_file);

  n_point=0;
  avr_dist=0.0;
  max_shift=0.0;

  for(i_file=0;i_file<n_file;i_file++) {

/* ProfileIn */
    err_code=StarProfileIn(&profile,&sdb_flag,sdb_arg);

    if(err_code!=0)	{
      StarPipeEndOut();
      exit(1);
    }

    if(sdb_flag==GOOD_SCAN || sdb_flag==BAD_SCAN) {

/* revise position alias */
      ResetOnTheGrid(&profile,coord,orig,grid,msg_fp,&delta_pos,&delta_dist);
      if(n_point==0) {
	max_delta_pos.x=fabs(delta_pos.x);
	max_delta_pos.y=fabs(delta_pos.y);
	max_shift=delta_dist;
      } else {
	if(max_delta_pos.x<delta_pos.x)
	  max_delta_pos.x=delta_pos.x;
	if(max_delta_pos.y<delta_pos.y)
	  max_delta_pos.y=delta_pos.y;
	if(max_shift<delta_dist)
	  max_shift=delta_dist;
      }
      avr_pos.x=delta_pos.x+avr_pos.x;
      avr_pos.y=delta_pos.y+avr_pos.y;
      avr_dist=delta_dist+avr_dist;
      n_point++;
    }

/* ProfileOut */
    err_code=StarProfileOut(&profile,sdb_flag,sdb_arg);
  }

/* End of Pipeline */
  err_code=StarPipeEndIn(id_str);
  if(err_code!=0)
    fprintf(stderr,"%s: Pipeline end invalid=%s\n",argv[0],id_str);

  StarPipeEndOut();

  if(msg_fp!=NULL) {
    avr_pos.x=avr_pos.x/(double)n_point;
    avr_pos.y=avr_pos.y/(double)n_point;
    avr_dist=avr_dist/(double)n_point;
    fprintf(msg_fp,"average: delta=(%f\",%f\") dist=%f\"\n",avr_pos.x,avr_pos.y,avr_dist);
    fprintf(msg_fp,"max deviate: delta=(%f\",%f\") dist=%f\"\n",max_delta_pos.x,max_delta_pos.y,max_shift);
  }

  if(msg_fp!=stderr && msg_fp!=NULL) fclose(msg_fp);

  exit(0);
}
