/*
exchange and rotate axis order

1999/ 9/23 T.Handa : ver.3.00
2000/ 2/ 9 T.Handa : ver.3.01 (debug) some rotation cannot done, unresolved
2002/ 5/14 T.Handa : ver.3.02 (debug/revise) single pair of axes exchange
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"rotaxis.prm"
#define N_KEY 50
#define N_BLOCK	10

#define AXIS_NO 0
#define AXIS_X 1
#define AXIS_Y 2
#define AXIS_Z 3
#define AXIS_LONG 11
#define AXIS_LAT 12
#define AXIS_VELO 13

#define DEBUG
#undef DEBUG

void ReorderAxes(axis_out,axis_in,new_axis)
AXES *axis_out; /* <out> axes to reordaring */
AXES *axis_in; /* <in> axes to reordaring */
int *new_axis; /* <in> */
{
  int ii;

/* convert axis order */
  for(ii=0;ii<3;ii++) {
    switch(new_axis[ii]) {
    case AXIS_LONG:
      if(strcmp((*axis_in).x.type,"long")==0) {
	new_axis[ii]=AXIS_X;
      } else if(strcmp((*axis_in).y.type,"long")==0) {
	new_axis[ii]=AXIS_Y;
      } else if(strcmp((*axis_in).z.type,"long")==0) {
	new_axis[ii]=AXIS_Z;
      }
      break;
    case AXIS_LAT:
      if(strcmp((*axis_in).x.type,"lat")==0) {
	new_axis[ii]=AXIS_X;
      } else if(strcmp((*axis_in).y.type,"lat")==0) {
	new_axis[ii]=AXIS_Y;
      } else if(strcmp((*axis_in).z.type,"lat")==0) {
	new_axis[ii]=AXIS_Z;
      }
      break;
    case AXIS_VELO:
      if(strcmp((*axis_in).x.type,"velocity")==0) {
	new_axis[ii]=AXIS_X;
      } else if(strcmp((*axis_in).y.type,"velocity")==0) {
	new_axis[ii]=AXIS_Y;
      } else if(strcmp((*axis_in).z.type,"velocity")==0) {
	new_axis[ii]=AXIS_Z;
      }
      break;
    }
  }

  switch(new_axis[0]) {
  case AXIS_X:
    StarCopyAxis(&((*axis_out).x),&((*axis_in).x));
    break;
  case AXIS_Y:
    StarCopyAxis(&((*axis_out).x),&((*axis_in).y));
    break;
  case AXIS_Z:
    StarCopyAxis(&((*axis_out).x),&((*axis_in).z));
    break;
  }

  switch(new_axis[1]) {
  case AXIS_X:
    StarCopyAxis(&((*axis_out).y),&((*axis_in).x));
    break;
  case AXIS_Y:
    StarCopyAxis(&((*axis_out).y),&((*axis_in).y));
    break;
  case AXIS_Z:
    StarCopyAxis(&((*axis_out).y),&((*axis_in).z));
    break;
  }

  switch(new_axis[2]) {
  case AXIS_X:
    StarCopyAxis(&((*axis_out).z),&((*axis_in).x));
    break;
  case AXIS_Y:
    StarCopyAxis(&((*axis_out).z),&((*axis_in).y));
    break;
  case AXIS_Z:
    StarCopyAxis(&((*axis_out).z),&((*axis_in).z));
    break;
  }

  return;
}

void ReorderPixel(cube_out,cube_in,new_axis)
CUBE *cube_out; /* <i/o> re-ordered cube */
CUBE *cube_in; /* <in> original cube */
int *new_axis; /* <in> */
{
  int ix,iy,iz;
  float pixel;

#ifdef DEBUG
  fprintf(stderr,"RePixel new_axis=%d,%d,%d\n",new_axis[0],new_axis[1],new_axis[2]);
#endif

  if(new_axis[0]==AXIS_X) { /* xyz --> xzy */
    if((*cube_in).head.axis.x.size!=(*cube_out).head.axis.x.size) {
      fprintf(stderr,"**ERR** x-size is not equal.\n");
      exit(999);
    } else if((*cube_in).head.axis.y.size!=(*cube_out).head.axis.z.size) {
      fprintf(stderr,"**ERR** y-size is not equal to new z-size.\n");
      exit(999);
    } else if((*cube_in).head.axis.z.size!=(*cube_out).head.axis.y.size) {
      fprintf(stderr,"**ERR** z-size is not equal to new y-size.\n");
      exit(999);
    }

    for(iz=0;iz<(*cube_in).head.axis.z.size;iz++) {
      for(iy=0;iy<(*cube_in).head.axis.y.size;iy++) {
	for(ix=0;ix<(*cube_in).head.axis.x.size;ix++) {
	  pixel=StarGetCubePixel(cube_in,ix,iy,iz);

	  StarPutCubePixel(cube_out,ix,iz,iy,pixel);	
	}
      }
    }
    return;

  } else if(new_axis[1]==AXIS_Y) { /* xyz --> zyx */
    if((*cube_in).head.axis.y.size!=(*cube_out).head.axis.y.size) {
      fprintf(stderr,"**ERR** y-size is not equal.\n");
      exit(999);
    } else if((*cube_in).head.axis.x.size!=(*cube_out).head.axis.z.size) {
      fprintf(stderr,"**ERR** x-size is not equal to new z-size.\n");
      exit(999);
    } else if((*cube_in).head.axis.z.size!=(*cube_out).head.axis.x.size) {
      fprintf(stderr,"**ERR** z-size is not equal to new x-size.\n");
      exit(999);
    }

    for(iz=0;iz<(*cube_in).head.axis.z.size;iz++) {
      for(iy=0;iy<(*cube_in).head.axis.y.size;iy++) {
	for(ix=0;ix<(*cube_in).head.axis.x.size;ix++) {
	  pixel=StarGetCubePixel(cube_in,ix,iy,iz);

	  StarPutCubePixel(cube_out,iz,iy,ix,pixel);	
	}
      }
    }
    return;

  } else if(new_axis[2]==AXIS_Z) { /* xyz --> yxz */
    if((*cube_in).head.axis.z.size!=(*cube_out).head.axis.z.size) {
      fprintf(stderr,"**ERR** z-size is not equal.\n");
      exit(999);
    } else if((*cube_in).head.axis.x.size!=(*cube_out).head.axis.y.size) {
      fprintf(stderr,"**ERR** x-size is not equal to new y-size.\n");
      exit(999);
    } else if((*cube_in).head.axis.y.size!=(*cube_out).head.axis.x.size) {
      fprintf(stderr,"**ERR** y-size is not equal to new x-size.\n");
      exit(999);
    }

    for(iz=0;iz<(*cube_in).head.axis.z.size;iz++) {
      for(iy=0;iy<(*cube_in).head.axis.y.size;iy++) {
	for(ix=0;ix<(*cube_in).head.axis.x.size;ix++) {
	  pixel=StarGetCubePixel(cube_in,ix,iy,iz);

	  StarPutCubePixel(cube_out,iy,ix,iz,pixel);	
	}
      }
    }
    return;
  }
}

int options(argc,argv,new_axis)
/*
get parameters from options
*/
int argc;
char *argv[];
int *new_axis;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

  int ii;

/* default values */
  new_axis[0]=AXIS_NO;
  new_axis[1]=AXIS_NO;
  new_axis[2]=AXIS_NO;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"new_xyz");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* new axis order */
  scan_flag[0]=0;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);

  if(got_value[0]!=(char)NULL) {
    if(strcmp(got_value,"xyz")==0 || strcmp(got_value,"XYZ")==0) {
      new_axis[0]=AXIS_X;
      new_axis[1]=AXIS_Y;
      new_axis[2]=AXIS_Z;
    } else if(strcmp(got_value,"yxz")==0 || strcmp(got_value,"YXZ")==0) {
      new_axis[0]=AXIS_Y;
      new_axis[1]=AXIS_X;
      new_axis[2]=AXIS_Z;
    } else if(strcmp(got_value,"zyx")==0 || strcmp(got_value,"ZYX")==0) {
      new_axis[0]=AXIS_Y;
      new_axis[1]=AXIS_X;
      new_axis[2]=AXIS_Z;
    } else if(strcmp(got_value,"xzy")==0 || strcmp(got_value,"XZY")==0) {
      new_axis[0]=AXIS_X;
      new_axis[1]=AXIS_Z;
      new_axis[2]=AXIS_Y;
    } else if(strcmp(got_value,"lbv")==0 || strcmp(got_value,"radecv")==0) {
      new_axis[0]=AXIS_LONG;
      new_axis[1]=AXIS_LAT;
      new_axis[2]=AXIS_VELO;
    } else if(strcmp(got_value,"blv")==0 || strcmp(got_value,"decrav")==0) {
      new_axis[0]=AXIS_LAT;
      new_axis[1]=AXIS_LONG;
      new_axis[2]=AXIS_VELO;
    } else if(strcmp(got_value,"vbl")==0 || strcmp(got_value,"vdecra")==0) {
      new_axis[0]=AXIS_VELO;
      new_axis[1]=AXIS_LAT;
      new_axis[2]=AXIS_LONG;
    } else if(strcmp(got_value,"lvb")==0 || strcmp(got_value,"ravdec")==0) {
      new_axis[0]=AXIS_LONG;
      new_axis[1]=AXIS_VELO;
      new_axis[2]=AXIS_LAT;
    } 
  }

  for(ii=0;ii<3;ii++) {
    if(new_axis[ii]==AXIS_NO) {
      new_axis[ii]=ii;
      fprintf(stderr,"%s: Three characters with 1 fixed axis are required for -new_xyz.\n",argv[0]);
      StarFreeAllPrms(N_KEY,key_flag,value);
      free(keyword);
      exit(1);
    }
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

int main(argc,argv)
int argc;
char *argv[];
{
  CUBE cube_in,cube_out;

  int new_axis[3],new_axis_org[3]; /* new axis for x,y,z */
  int err_code;

  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];

  int ii,n_file;
  char id_str[MAX_FILE_NAME_LENGTH];

  int jj;

/* check option */
  err_code=options(argc,argv,new_axis_org);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

/* pipe header */
  StarPipeHeadIn(id_str,&n_file);

  StarPipeHeadOut(n_file);

/* .cub data */
  for(ii=0;ii<n_file;ii++) {
    err_code=StarCubeIn(&cube_in,&cdb_flag,cdb_arg);
    if(err_code!=0) {
      fprintf(stderr,"%s: CubeIn failed in %dth file. code=%d\n",argv[0],ii+1,err_code);
      exit(999);
    }
    StarCopyCubeHead(&(cube_out.head),&(cube_in.head));

    for(jj=0;jj<3;jj++) {
      new_axis[jj]=new_axis_org[jj];
    }

#ifdef DEBUG
    fprintf(stderr,"new_axis=%d,%d,%d\n",new_axis[0],new_axis[1],new_axis[2]);
#endif

    ReorderAxes(&(cube_out.head.axis),&(cube_in.head.axis),new_axis);

    cube_out.pixel=StarAllocCubePixel(&(cube_out.head));

    if(cube_out.pixel==NULL) {
      fprintf(stderr,"%s: Pixel allocation failed. more memory.\n",argv[0]);
      exit(999);
    }

    ReorderPixel(&cube_out,&cube_in,new_axis);

    StarCubeOut(&cube_out,cdb_flag,cdb_arg);

    StarFreeCubePixel(&cube_in);
    StarFreeCubePixel(&cube_out);
  }

/* end tag of pipe */
  StarPipeEndIn(id_str);
  StarPipeEndOut();

  exit(0);
}
