/*
profile data saver from stdin

1992/4/17 T.Handa : ver.1.00
1993/2/17 T.Handa : ver.1.41 non-backup option
1997/12/16 T.Handa : ver.3.00 multi_pipe and new library
1998/3/19 T.Handa : ver.3.01 debug for Solaris2 fcc
1998/4/ 8 T.Handa : ver.3.20 (revise) .scn,.sdb
2003/ 1/ 2 T.Handa : ver.3.30 (revise) Backup directory
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"save.prm"
#define N_KEY 10

int options(argc,argv,pipe_flag,backup_flag,with_backver)
/*
get parameters from options
*/
int argc;
char *argv[];
int *pipe_flag;
int *backup_flag;
int *with_backver;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int scan_flag[N_KEY];

  int flag;

/* default values */
  *pipe_flag=FLAG_NO;
  *backup_flag=FLAG_YES;
  *with_backver=FLAG_NO;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"pipe");
  n_key=AddNewWord(keyword,"output");
  n_key=AddNewWord(keyword,"-");
  n_key=AddNewWord(keyword,"nopipe");
  n_key=AddNewWord(keyword,"backup");
  n_key=AddNewWord(keyword,"nobackup");
  n_key=AddNewWord(keyword,"with_ver_ext");
  n_key=AddNewWord(keyword,"without_ver_ext");

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* pipe */
  scan_flag[0]= 0;
  scan_flag[1]= 1;
  scan_flag[2]= 2;
  scan_flag[3]= -3;
  flag=StarOptionFlag(4,scan_flag,key_flag);
  if(flag!=0) *pipe_flag=flag;

/* backup file */
  scan_flag[0]= 4;
  scan_flag[1]= -5;
  flag=StarOptionFlag(2,scan_flag,key_flag);
  if(flag!=0) *backup_flag=flag;

/* version extension for backup file */
  scan_flag[0]= 6;
  scan_flag[1]= -7;
  flag=StarOptionFlag(2,scan_flag,key_flag);
  if(flag!=0) *with_backver=flag;

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

int save_profile_data(profile,filename,sdb_flag)
PROFILE *profile; /* <i/o> */
char *filename; /* <in> full name to save */
int sdb_flag; /* <in> */
{
  FILE *fp;
  int err_code;

  err_code=0;
/* increase backup version number */
  (*profile).sys_head.backup++;

/* write a profile into the file */
  if(sdb_flag==GOOD_SCAN) {
    fp=fopen(filename,"w");
    if(fp==NULL) {
      fprintf(stderr,"Save: failed for %s at open\n",filename);
    } else {
      err_code=StarProfileSave(fp,profile);
      if(err_code!=0)
	fprintf(stderr,"Save: failed for %s\n",filename);
      fclose(fp);
    }
  }

  return(err_code);
}

int main(argc,argv)
int argc;
char *argv[];
{
  PROFILE profile;

  char output_filename[MAX_FILE_NAME_LENGTH],backup_filename[MAX_FILE_NAME_LENGTH];
  char full_output_filename[MAX_FILE_NAME_LENGTH];
  char id_str[MAX_FILE_NAME_LENGTH];
  /*  char default_sdbfile[MAX_FILE_NAME_LENGTH]; */
  char argument[MAX_IDX_RECORD_LENGTH];

  int ii,n_file,err_code;
  int pipe_flag,backup_flag;

  int sdb_flag;

  int backup_version,with_backver;

/* check option */
  err_code=options(argc,argv,&pipe_flag,&backup_flag,&with_backver);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

  n_file=0;

  StarPipeHeadIn(id_str,&n_file);

  if(pipe_flag==FLAG_YES) StarPipeHeadOut(n_file);

  for(ii=0;ii<n_file;ii++) {

/* profile input */
    err_code=StarProfileIn(&profile,&sdb_flag,argument);
    if(err_code!=0)	{
      if(pipe_flag==FLAG_YES) StarPipeEndOut();
      fprintf(stderr,"Save: fail to ProfileIn No.%d\n",ii);
      exit(1);
    }

    if(sdb_flag!=GOOD_SCAN && sdb_flag!=BAD_SCAN)
      goto next_profile;

/* get the file name to save */
    StarGetSavename(argc,argv,profile.file_name,output_filename);
    err_code=StarStdFileExtName("STAR_DATA",output_filename,full_output_filename,".scn"); 
    if(err_code!=0) {
      fprintf(stderr,"Too long file name\n");
      exit(1);
    }

    if(backup_flag==FLAG_YES) {
      backup_version=(int)(profile.sys_head.backup);
      StarGetBackupname(backup_filename,profile.file_name,backup_version,with_backver,".scn");
      StarRename4backup(full_output_filename,backup_filename);
    }

    err_code=save_profile_data(&profile,full_output_filename,sdb_flag);

next_profile:;

/* do pipe out */
    if(pipe_flag==FLAG_YES) 
      err_code=StarProfileOut(&profile,sdb_flag,argument);
  }
  if(pipe_flag==FLAG_YES) StarPipeEndOut();

  exit(0);
}
