/*
profile data saver from stdin

1992/4/17 T.Handa : ver.1.00
1993/2/17 T.Handa : ver.1.41 non-backup option
1997/12/16 T.Handa : ver.3.00 multi_pipe and new library
1998/3/19 T.Handa : ver.3.01 debug for Solaris2 fcc
1998/4/ 8 T.Handa : ver.3.20 (revise) .scn,.sdb
1999/5/18 T.Handa : savev2 (ver.3.20) based upon save.c ver.3.20(1998/4/8)
2002/1/4 T.Handa : separate the lib part out
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <StData.h>
#include <StStdio.h>
#include <StStdlib.h>
#include <StPipe.h>
#include <StIndex.h>
#include <StProfileIO.h>

#define	DEFAULT_PRMFILE	"save.prm"
#define N_KEY 10

#include "savev2.lib.c"

int options(argc,argv,pipe_flag,backup_flag,with_backver,version)
/*
get parameters from options
*/
int argc;
char *argv[];
int *pipe_flag;
int *backup_flag;
int *with_backver;
int *version;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY],key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;
  char *got_value;

  int flag;

/* default values */
  *pipe_flag= -1;
  *backup_flag= 1;
  *version=0;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"pipe"); key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"output"); key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"-"); key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"nopipe"); key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"backup"); key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"nobackup"); key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"with_ver_ext");
  n_key=AddNewWord(keyword,"without_ver_ext");
  n_key=AddNewWord(keyword,"ver"); key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* pipe */
  scan_flag[0]= 0;
  scan_flag[1]= 1;
  scan_flag[2]= 2;
  scan_flag[3]= -3;
  flag=StarOptionFlag(4,scan_flag,key_flag);
  if(flag!=0) *pipe_flag=flag;

/* backup file */
  scan_flag[0]= 4;
  scan_flag[1]= -5;
  flag=StarOptionFlag(2,scan_flag,key_flag);
  if(flag!=0) *backup_flag=flag;

/* version extension for backup file */
  scan_flag[0]= 6;
  scan_flag[1]= -7;
  flag=StarOptionFlag(2,scan_flag,key_flag);
  if(flag!=0) *with_backver=flag;

/* version */
  n_scan=1;
  scan_flag[0]=8;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    *version=atoi(got_value);
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

int save_profile_data(profile,filename,sdb_flag)
PROFILE *profile; /* <i/o> */
char *filename; /* <in> full name to save */
int sdb_flag; /* <in> */
{
  FILE *fp;
  int err_code;

  char full_filename[MAX_FILE_NAME_LENGTH];
  int backup_version;

/* increase backup version number */
  (*profile).sys_head.backup++;

/* write a profile into the file */
  if(sdb_flag==GOOD_SCAN) {
    fp=fopen(filename,"w");
    if(fp==NULL) {
      fprintf(stderr,"Save: failed for %s at open\n",filename);
    } else {
      err_code=StarProfileSave(fp,profile);
      if(err_code!=0)
	fprintf(stderr,"Save: failed for %s\n",filename);
      fclose(fp);
    }
  }

  return(err_code);
}

int StarGetSavenameV0(argc,argv, default_name,filename)
int argc; /* <in> the number of option arguments */
char **argv; /* <in> option arguments */
char *default_name; /* <in> */
char *filename; /* <out> */
/*
set a proper filename
skip just after -ver argument

2002/6/11 T.Handa
 */
{
  FILE *fp;
  int err_code;
  static int argi=0;

  argi++;
  if(argi>=argc) {
    strcpy(filename,default_name);
    return(1);
  }
  while(argv[argi][0]=='-' && argi<=argc-1) argi++;
  if(argi>=argc) {
    strcpy(filename,default_name);
    return(1);
  }

  if(strcmp(argv[argi-1],"-ver")==0) {
    argi++;
    while(argv[argi][0]=='-' && argi<=argc-1) argi++;
    if(argi>=argc) {
      strcpy(filename,default_name);
      return(1);
    }
  }

  strcpy(filename,argv[argi]);

  return(0);
}


int main(argc,argv)
int argc;
char *argv[];
{
  PROFILE profile;

  char output_filename[MAX_FILE_NAME_LENGTH],backup_filename[MAX_FILE_NAME_LENGTH];
  char full_output_filename[MAX_FILE_NAME_LENGTH];
  char id_str[MAX_FILE_NAME_LENGTH];
  /*  char default_sdbfile[MAX_FILE_NAME_LENGTH]; */
  char argument[MAX_IDX_RECORD_LENGTH];

  int ii,n_file,err_code;
  int pipe_flag,backup_flag;

  int sdb_flag;

  int backup_version,with_backver,rename_flag;
  int new_version;

/* check option */
  err_code=options(argc,argv,&pipe_flag,&backup_flag,&with_backver,&new_version);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

  n_file=0;

  StarPipeHeadIn(id_str,&n_file);

  if(pipe_flag==FLAG_YES) StarPipeHeadOut(n_file);

  for(ii=0;ii<n_file;ii++) {

/* profile input */
    err_code=StarProfileIn(&profile,&sdb_flag,argument);
    if(err_code!=0)	{
      if(pipe_flag==FLAG_YES) StarPipeEndOut();
      fprintf(stderr,"Save: fail to ProfileIn No.%d\n",ii);
      exit(1);
    }

    if(sdb_flag!=GOOD_SCAN && sdb_flag!=BAD_SCAN)
      goto next_profile;

/* get a file name to save */
    StarGetSavenameV0(argc,argv, profile.file_name,output_filename);
    err_code=StarStdFileExtName("STAR_DATA",output_filename,full_output_filename,".scn");
    if(err_code!=0) {
      fprintf(stderr,"Too long file name\n");
      exit(1);
    }

    if(backup_flag==FLAG_YES) {
      backup_version=(int)(profile.sys_head.backup);
      StarGetBackupname(backup_filename,profile.file_name,backup_version,with_backver,".scn");
      StarRename4backup(full_output_filename,backup_filename);
    }

    err_code=save_profile_data(&profile,full_output_filename,sdb_flag);

next_profile:;

/* pipe out */
    if(pipe_flag==FLAG_YES) 
      err_code=StarProfileOut(&profile,sdb_flag,argument);
  }
  if(pipe_flag==FLAG_YES) StarPipeEndOut();

  exit(0);
}
