/*
export the profile data into FITS format

2006/12/07 T.Handa : ver.3.00
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>
#include <StFitsIO.h>

#define	DEFAULT_PRMFILE	"scn2fits.prm"
#define N_KEY 10

#ifdef INTEL_CPU
/*------ for INTEL CPU ------*/
static size_t fwriteIntel(pt,size,nn,fp)
char *pt;
size_t size;
int nn;
FILE *fp;
/*
Write Binary data readable by Motrollor/MIPS CPU with Intel
1998/4/21 T.Handa
1998/7/14 T.Handa (return value revised)
*/
{
  char *buffer;
  size_t ret_code;
  int total_byte,ii,jj;

  if(size==1) {
    ret_code=fwrite(pt,1,nn,fp);
    return(ret_code);
  }

  total_byte=size*nn;
  buffer=malloc(total_byte);

  for(ii=0;ii<nn;ii++) {
    for(jj=0;jj<size;jj++)
      buffer[(ii+1)*size-jj-1]=pt[ii*size+jj];
  }

  ret_code=fwrite(buffer,1,total_byte,fp);
  if(ret_code==total_byte) 
    ret_code=nn;
  else 
    ret_code=0;

  free(buffer);
  return(ret_code);
}
#endif

/*---- for FITS header ----*/
int StarWriteFitsHead4CubeHeadObsPrm(fp,obs_prm)
/*
Write Fits header corresponding cube.obs_prm

1999/7/31 T.Handa
*/
FILE *fp; /* <in> for FITS */
OBS_PRM *obs_prm; /* <in> cube.obs_prm */
{
  int n_line;
  char buffer[80];
  int yy,mm,dd;

  n_line=0;

  sprintf(buffer,"%20.4f",(double)(*obs_prm).mjd);
  StarWriteFitsRecord("MJD-OBS",buffer,FITS_VALUE,"",fp);
  n_line++;

  StarMJD2Ymd(&yy,&mm,&dd,(long)((*obs_prm).mjd));
  sprintf(buffer,"%4d-%02d-%02d",yy,mm,dd);
  StarWriteFitsRecord("DATE-OBS",buffer,FITS_VALUE_STRING,"",fp);
  n_line++;

  if(strlen((*obs_prm).observer1)>0) {
    StarWriteFitsRecord("OBSERVER",(*obs_prm).observer1,FITS_VALUE_STRING,"",fp);
    n_line++;
  }
  if(strlen((*obs_prm).observer2)>0) {
    StarWriteFitsRecord("OBSERVER",(*obs_prm).observer2,FITS_VALUE_STRING,"",fp);
    n_line++;
  }
  if(strlen((*obs_prm).observer3)>0) {
    StarWriteFitsRecord("OBSERVER",(*obs_prm).observer3,FITS_VALUE_STRING,"",fp);
    n_line++;
  }
  if(strlen((*obs_prm).observer4)>0) {
    StarWriteFitsRecord("OBSERVER",(*obs_prm).observer4,FITS_VALUE_STRING,"",fp);
    n_line++;
  }

  return(n_line);
}

int StarWriteFitsHead4CubeHeadObject(fp,obj)
/*
Write Fits header corresponding cube.object

1999/7/31 T.Handa
*/
FILE *fp; /* <in> for FITS */
OBJECT *obj; /* <in> cube.object */
{
  int n_line;

  n_line=0;

  StarWriteFitsRecord("OBJECT",(*obj).name,FITS_VALUE_STRING,"",fp);
  n_line++;

  if(strlen((*obj).equinox)<=0) {
    StarWriteFitsRecord("EQUINOX","1950.0",FITS_VALUE,"",fp);
    n_line++;
  } else {
    if((*obj).equinox[0]=='B' || (*obj).equinox[0]=='J') {
      StarWriteFitsRecord("EQUINOX",&((*obj).equinox[1]),FITS_VALUE,"",fp);
      n_line++;
    } else {
      StarWriteFitsRecord("EQUINOX",(*obj).equinox,FITS_VALUE,"",fp);
      n_line++;
    }
  }

  return(n_line);
}

int StarWriteFitsHead4CubeHeadAntStatus(fp,ant)
/*
Write Fits header corresponding cube.ant

1999/7/31 T.Handa
*/
FILE *fp; /* <in> for FITS */
ANT_STATUS *ant; /* <in> cube.ant */
{
  int n_line;

  n_line=0;

  StarWriteFitsRecord("TELESCOP",(*ant).tele,FITS_VALUE_STRING,"",fp);
  n_line++;

  return(n_line);
}

int StarWriteFitsHead4CubeHeadRxStatus(fp,rx)
/*
Write Fits header corresponding cube.rx

1999/7/31 T.Handa
*/
FILE *fp; /* <in> for FITS */
RX_STATUS *rx; /* <in> cube.rx */
{
  int n_line;

  n_line=0;

  StarWriteFitsRecord("INSTRUME",(*rx).name,FITS_VALUE_STRING,"Receiver",fp);
  n_line++;

  return(n_line);
}

int StarWriteFitsHead4CubeHeadBeStatus(fp,be)
/*
Write Fits header corresponding cube.be

1999/7/31 T.Handa
*/
FILE *fp; /* <in> for FITS */
BE_STATUS *be; /* <in> cube.be */
{
  int n_line;

  n_line=0;

  StarWriteFitsRecord("INSTRUME",(*be).name,FITS_VALUE_STRING,"BackEnd",fp);
  n_line++;

  return(n_line);
}

int StarWriteFitsHead4CubeHeadObsLog(fp,log)
/*
Write Fits header corresponding cube.log

1999/7/31 T.Handa
*/
FILE *fp; /* <in> for FITS */
OBS_LOG *log; /* <in> cube.log */
{
  int n_line;

  n_line=0;

  StarWriteFitsRecord("ORIGIN",(*log).site_name,FITS_VALUE_STRING,"",fp);
  n_line++;
  if(strlen((*log).memo)>0) {
    StarWriteFitsRecord("COMMENT",(*log).memo,FITS_VALUE_STRING,"",fp);
    n_line++;
  }
  if(strlen((*log).comment)>0) {
    StarWriteFitsRecord("COMMENT",(*log).comment,FITS_VALUE_STRING,"",fp);
    n_line++;
  }

  return(n_line);
}

int WriteFitsAxis(fp,profile,fitsdim)
FILE *fp;
PROFILE *profile;
int fitsdim; /* <in> FITS dimension 1 or 3 */
{
  char buffer[80],tag_key[9];
  char axis2name[80],axis3name[80];
  int n_line;
  double vel_org,vel_delta;
  XY_POS pos4axis;

  n_line=0;
  sprintf(buffer,"                   %1d",fitsdim);
  StarWriteFitsRecord("NAXIS",buffer,FITS_VALUE,"NUMBER OF AXES",fp);
  n_line++;
  sprintf(buffer,"%20d",(*profile).be_status.data_ch);
  sprintf(tag_key,"NAXIS1");
  StarWriteFitsRecord(tag_key,buffer,FITS_VALUE," ",fp);
  n_line++;

  StarWriteFitsRecord("CTYPE1","VELOCITY",FITS_VALUE_SHORT_STRING,"Name of axis",fp);
  n_line++;
  StarWriteFitsRecord("CUNIT1","KM/S",FITS_VALUE_SHORT_STRING,"Unit of axis",fp);
  n_line++;

  sprintf(buffer,"%20.15f",(*profile).be_status.center_ch);
  StarWriteFitsRecord("CRPIX1",buffer,FITS_VALUE,"Reference in Index",fp);
  n_line++;
  vel_org=StarCh2Velocity(profile,(*profile).be_status.center_ch);
  sprintf(buffer,"%20.15f",vel_org);
  StarWriteFitsRecord("CRVAL1",buffer,FITS_VALUE,"at reference pixel",fp);
  n_line++;
  vel_delta=StarCh2Velocity(profile,2.0)-StarCh2Velocity(profile,1.0);
  sprintf(buffer,"%20.15f",vel_delta);
  StarWriteFitsRecord("CDELT1",buffer,FITS_VALUE,"",fp);
  n_line++;
  StarWriteFitsRecord("CROTA1","   0.000000000000000",FITS_VALUE,"",fp);
  n_line++;

  if(fitsdim==3) {
    if((*profile).object.cood_sys==COORD_LB) { /* Galactic Coordinate */
      strcpy(axis2name,"GLONG");
      strcpy(axis3name,"GLAT");
      memcpy(&pos4axis,&((*profile).object.on_lb),sizeof(pos4axis));
    } else if((*profile).object.cood_sys==COORD_RADEC_APP) { /* apparent RADec*/
      strcpy(axis2name,"RA_APP");
      strcpy(axis3name,"DEC_APP");
      memcpy(&pos4axis,&((*profile).object.on_radec),sizeof(pos4axis));
    } else { /* assume RADec1950 */
      strcpy(axis2name,"RA(B1950)");
      strcpy(axis3name,"DEC(B1950)");
      memcpy(&pos4axis,&((*profile).object.on_radec),sizeof(pos4axis));
    }

    StarWriteFitsRecord("NAXIS2","                   1",FITS_VALUE," ",fp);
    StarWriteFitsRecord("CTYPE2",axis2name,FITS_VALUE_SHORT_STRING,"Name of axis",fp);
    StarWriteFitsRecord("CUNIT2","DEGREE",FITS_VALUE_SHORT_STRING,"Unit of axis",fp);
    StarWriteFitsRecord("CRPIX2","   1.000000000000000",FITS_VALUE,"Refernce in Index",fp);
    sprintf(buffer,"%20.15f",pos4axis.x);
    StarWriteFitsRecord("CRVAL2",buffer,FITS_VALUE,"at reference pixel",fp);
    StarWriteFitsRecord("CDELT2","   1.000000000000000",FITS_VALUE,"",fp);
    StarWriteFitsRecord("CROTA2","   0.000000000000000",FITS_VALUE,"",fp);
    n_line=n_line+7;

    StarWriteFitsRecord("NAXIS3","                   1",FITS_VALUE," ",fp);
    StarWriteFitsRecord("CTYPE3",axis3name,FITS_VALUE_SHORT_STRING,"Name of axis",fp);
    StarWriteFitsRecord("CUNIT3","DEGREE",FITS_VALUE_SHORT_STRING,"Unit of axis",fp);
    StarWriteFitsRecord("CRPIX3","   1.000000000000000",FITS_VALUE,"Refernce in Index",fp);
    sprintf(buffer,"%20.15f",pos4axis.y);
    StarWriteFitsRecord("CRVAL3",buffer,FITS_VALUE,"at reference pixel",fp);
    StarWriteFitsRecord("CDELT3","   1.000000000000000",FITS_VALUE,"",fp);
    StarWriteFitsRecord("CROTA3","   0.000000000000000",FITS_VALUE,"",fp);
    n_line=n_line+7;

    StarWriteFitsRecord("EQUINOX","              1950.0",FITS_VALUE," ",fp);
  }

  sprintf(buffer,"%20.15f",(*profile).object.on_lb.x);
  StarWriteFitsRecord("GLONG",buffer,FITS_VALUE,"galactic longitude (deg)",fp);
  n_line++;
  sprintf(buffer,"%20.15f",(*profile).object.on_lb.y);
  StarWriteFitsRecord("GLAT",buffer,FITS_VALUE,"galactic latitued (deg)",fp);
  n_line++;

  sprintf(buffer,"%20.15f",(*profile).object.on_radec.x);
  StarWriteFitsRecord("RA1950",buffer,FITS_VALUE,"right ascension B1950 (deg)",fp);
  n_line++;
  sprintf(buffer,"%20.15f",(*profile).object.on_radec.y);
  StarWriteFitsRecord("DEC1950",buffer,FITS_VALUE,"declination B1950 (deg)",fp);
  n_line++;

  return(n_line);
}

int save_fits_head(fp,profile,short_head,fitsdim)
/*
Save FITS header for a spectral line profile

2006/12/07 T.Handa
 */
FILE *fp; /* <in> */
PROFILE *profile; /* <in> */
int short_head; /* <in> =1 2880byte limited, =0 unlimited */
int fitsdim; /* <in> FITS dimension 1 or 3 */
{
  int i_line,n_line,blank_line;
  int ii;
  char *blank;
  static int lmt_line=36; /* =2880/80; */

/* basic and axis part */
  n_line=0;
  StarWriteFitsRecord("SIMPLE","True",FITS_VALUE_BOOLIAN,"FITS STANDARD",fp);
  n_line++;
  StarWriteFitsRecord("BITPIX","                 -32",FITS_VALUE,"FITS BITS/PIXEL",fp);
  n_line++;

  n_line=n_line+WriteFitsAxis(fp,profile,fitsdim);


  if(n_line+7<lmt_line) {
    n_line=n_line+StarWriteFitsHead4CubeHeadObsPrm(fp,&((*profile).obs_prm));
  }
  if(n_line+1<lmt_line) {
    n_line=n_line+StarWriteFitsHead4CubeHeadObject(fp,&((*profile).object));
  }
  if(n_line+1<lmt_line) {
    n_line=n_line+StarWriteFitsHead4CubeHeadAntStatus(fp,&((*profile).ant_status));
  }
  if(n_line+1<lmt_line) {
    n_line=n_line+StarWriteFitsHead4CubeHeadRxStatus(fp,&((*profile).rx_status));
  }
  if(n_line+1<lmt_line) {
    n_line=n_line+StarWriteFitsHead4CubeHeadBeStatus(fp,&((*profile).be_status));
  }
  if(n_line+3<lmt_line) {
    n_line=n_line+StarWriteFitsHead4CubeHeadObsLog(fp,&((*profile).obs_log));
  }

/* End keyword */
  StarWriteFitsRecord("END","",FITS_VALUE,"",fp);
  n_line++;

/* estimate blank records */
  blank_line=(int)ceil((double)n_line/36.0)*36-n_line;

  if(blank_line>0) {
    blank=malloc(80);
    for(ii=0;ii<80;ii++) {
      blank[ii]=' ';
    }
    for(i_line=0;i_line<blank_line;i_line++) {
      fwrite(blank,1,80,fp);
    }
    free(blank);
  }

  return(0);
}

int save_fits_pixel(fp,profile)
/*
Save FITS pixel data for a spectral line profile

2006/10/17 T.Handa
 */
FILE *fp; /* <in> */
PROFILE *profile; /* <in> */
{
  int i_ch,n_ch;
  int blank_byte,total_byte;
  int ii;
  char *blank;

  size_t written_pixel,written;

  float data;
  static long nan=0xffffffff; /* IEEE NaN */

  n_ch=(*profile).be_status.data_ch;

  written_pixel=0;

  for(i_ch=1;i_ch<=n_ch;i_ch++) {
    data=(*profile).data[i_ch-1];
    if(StarReadSpuriousFlag(profile,i_ch)==FLAGGED) {
      written=fwrite(&nan,sizeof(nan),1,fp);
    } else {
#ifdef INTEL_CPU
      written=fwriteIntel(&data,sizeof(data),1,fp);
#else
      written=fwrite(&data,sizeof(data),1,fp);
#endif
    }
    if(written!=1) return((int)written_pixel);
    written_pixel=written_pixel+written;
  }
/* estimate blank byte */
  total_byte=n_ch*sizeof(float);
  blank_byte=(int)ceil((double)total_byte/2880.0)*2880-total_byte;

  if(blank_byte>0) {
    blank=malloc(blank_byte);
    for(ii=0;ii<blank_byte;ii++) {
      blank[ii]=' ';
    }
    written=fwrite(blank,1,blank_byte,fp);
    free(blank);
    written_pixel=written+written_pixel;
    if(written!=blank_byte) return((int)written_pixel);
  }

  return(0);
}


int save_scn_fits(fp,profile,short_head,fitsdim)
/*
Save a spectral line profile in the FITS format

2006/12/07 T.Handa
 */
FILE *fp; /* <in> */
PROFILE *profile; /* <in> */
int short_head; /* <in> =1 2880byte limited, =0 unlimited */
int fitsdim; /* <in> FITS dimension 1 or 3 */
{
  int err_code;

  err_code=save_fits_head(fp,profile,short_head,fitsdim);
  err_code=save_fits_pixel(fp,profile);

  return(0);
}

int check_dispersion(profile)
PROFILE *profile; /* <in> */
{
  if((*profile).be_status.disp2!=0.0) return(999);
  if((*profile).be_status.disp3!=0.0) return(999);
  if((*profile).be_status.disp4!=0.0) return(999);
  return(0);
}

int options(argc,argv,filename,pipe_flag,short_head,fitsdim)
/*
get parameters from options
*/
int argc;
char *argv[];
char *filename;
int *pipe_flag;
int *short_head;
int *fitsdim;
{
  int ii,next_pos;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;
  char *got_value;

  int flag;

/* default values */
  filename[0]=(char)NULL;
  *pipe_flag=FLAG_NO;
  *short_head=1;
  *fitsdim=1;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"outfits");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"fits_name");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"pipe");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"output");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"-");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"nopipe");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"shortheader");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"longheader");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"fitsdim");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"d");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

  next_pos=0;
/* FITS file name */
  got_value=StarKeylessOption(argc,argv,n_key,key_flag,key_type);
  if(got_value[0]!=(char)NULL) {
    strcpy(filename,got_value);
  }
  if(strcmp(got_value,"-")==0 || strcmp(got_value,"auto")==0) {
    filename[0]=(char)NULL;
  }

  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
  }
  next_pos=next_pos+n_scan;

  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(filename,got_value);
  }

/* pipe */
  n_scan=4;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
    if(ii==3) scan_flag[ii]= -scan_flag[ii];
  }
  next_pos=next_pos+n_scan;

  flag=StarOptionFlag(n_scan,scan_flag,key_flag);
  if(flag!=0) *pipe_flag=flag;

/* shortheader */
  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
    if(ii==1) scan_flag[ii]= -scan_flag[ii];
  }
  next_pos=next_pos+n_scan;
  *short_head=StarOptionFlag(n_scan,scan_flag,key_flag);

/* FITS dimension */
  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=next_pos+ii;
    if(ii==1) scan_flag[ii]= -scan_flag[ii];
  }
  next_pos=next_pos+n_scan;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    *fitsdim=atoi(got_value);
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

void StarRemoveFilenameExtension(filename,extension)
char *filename; /* <i/o> filename with extension --> without extension */
char *extension; /* <in> assigned extension with . */
{
  int l_str,l_ext;

  l_str=strlen(filename);
  l_ext=strlen(extension);

  if(l_str>=l_ext && strcmp(&(filename[l_str-l_ext]),extension)==0) {
    filename[l_str-l_ext]=(char)NULL;
  }
  return;
}

void StarSeparateFilenameExtension(filename,ext)
char *filename; /* <i/o> filename with extension --> without extension */
char *ext; /* <out> extension without '.' */
{
  int l_str;
  int ii;

  l_str=strlen(filename);

  for(ii=l_str-1;ii>=0;ii--) {
    if(filename[ii]=='.') {
      filename[ii]=(char)NULL;
      strcpy(ext,&(filename[ii+1]));
      return;
    }
  }
  strcpy(ext,"");
  return;
}

#define N_EXT_TYPE 3

int StarCheckExt4fits(fitsname)
char *fitsname;
{
  char mainname[MAX_FILE_NAME_LENGTH];
  char extension[MAX_FILE_NAME_LENGTH];
  static char good_ext[N_EXT_TYPE][5]={"fits","fit","fts"};
  int ii;

  strcpy(mainname,fitsname);
  StarSeparateFilenameExtension(mainname,extension);
  for(ii=0;ii<N_EXT_TYPE;ii++) {
    if(strcasecmp(extension,good_ext[ii])==0)
      return(FLAG_YES);
  }
  return(FLAG_NO);
}

int main(argc,argv)
int argc;
char *argv[];
{
  PROFILE profile;

  FILE *fits_fp;
  int pipe_flag;
  int err_code;

  char fitsname[MAX_FILE_NAME_LENGTH];
  char full_filename[MAX_FILE_NAME_LENGTH];
  int short_head;
  int fitsdim;

  int sdb_flag;
  char argument[MAX_IDX_RECORD_LENGTH];

  int ii,n_file;
  char id_str[MAX_FILE_NAME_LENGTH];

/* check option */
  err_code=options(argc,argv,fitsname,&pipe_flag,&short_head,&fitsdim);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

  n_file=0;

  StarPipeHeadIn(id_str,&n_file);

  if(pipe_flag==FLAG_YES) StarPipeHeadOut(n_file);

  for(ii=0;ii<n_file;ii++) {

/* profile input */
    err_code=StarProfileIn(&profile,&sdb_flag,argument);
    if(err_code!=0)	{
      if(pipe_flag==FLAG_YES) StarPipeEndOut();
      fprintf(stderr,"Save: fail to ProfileIn No.%d\n",ii);
      exit(1);
    }

    if(sdb_flag!=GOOD_SCAN && sdb_flag!=BAD_SCAN)
      goto next_profile;

    if(check_dispersion(&profile)!=0) {
      fprintf(stderr,"Skip! The backend of %s has a nonlinear dispersion.\n",profile.file_name);
      goto next_profile;
    }

/* add STD extension for FITS */
    StarRemoveFilenameExtension(profile.file_name,".scn");
    if(strcmp(fitsname,"")==0) {
      err_code=StarStdFileExtName("STAR_DATA",profile.file_name,full_filename,".fit");
    } else {
      if(StarCheckExt4fits(fitsname)==FLAG_YES) {
	err_code=StarStdFileExtName("STAR_DATA",fitsname,full_filename,"");
      } else {
	err_code=StarStdFileExtName("STAR_DATA",fitsname,full_filename,".fit");
      }
    }
    if(err_code!=0) {
      fprintf(stderr,"Too long file name\n");
      exit(1);
    }
    fits_fp=fopen(full_filename,"w");

    save_scn_fits(fits_fp,&profile,short_head,fitsdim);

    fclose(fits_fp);

next_profile:;

/* do pipe out */
    if(pipe_flag==FLAG_YES) 
      err_code=StarProfileOut(&profile,sdb_flag,argument);
  }
  if(pipe_flag==FLAG_YES) StarPipeEndOut();

  exit(0);
}
