/*
print profile in ASCII text.

output format
divided by tab, 1ch is in a line.
The first line is an index line for columns
Units are
km/s for velocity, GHz for RF frequency, MHz for BE frequency
K for a channel.

1999/ 8/10 T.Handa ver.3.00
1999/ 9/25 T.Handa ver.3.01 (debug) scaled value/ unscaled value
2002/ 5/28 T.Handa ver.3.02 (modify) change keyword definition for headline
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define SCALED 1
#define UNSCALED 0

#define	DEFAULT_PRMFILE	"scn2txt.prm"
#define N_KEY 30

void print_profile_data(profile,x_unit,scale_flag)
PROFILE *profile;
int x_unit;
int scale_flag;
{
  int ii,ch,max_ch;

  max_ch=(*profile).be_status.data_ch;

  for(ii=0;ii<max_ch;ii++) {
    ch=ii+1;

    switch(x_unit) {
    case X_VELOCITY:
      fprintf(stdout,"%f\t",StarCh2Velocity(profile,ch));
      break;
    case X_RF_FREQ:
      fprintf(stdout,"%f\t",StarCh2Frequency(profile,ch));
      break;
    case X_BE_FREQ:
      fprintf(stdout,"%f\t",StarCh2FreqOnBE(profile,ch));
      break;
    case X_CHANNEL: default:
      fprintf(stdout,"%d\t",ch);
      break;
    }
    if(scale_flag==UNSCALED) {
      fprintf(stdout,"%f\n",(*profile).data[ii]);
    } else {
      fprintf(stdout,"%f\n",(*profile).data[ii]*(*profile).rx_status.scale);
    }
  }
}

int options(argc,argv, x_unit,headline,scale_flag)
/*
get parameters from options

*/
int argc;
char *argv[];
int *x_unit;
int *headline;
int *scale_flag;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  *x_unit=X_CHANNEL;
  *headline=FLAG_NO;
  *scale_flag=SCALED;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"with_headline");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"without_headline");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"headline");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"v");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"rf");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"freq");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"bf");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"ch");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"output_scale");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"data_scale");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* headline */
  n_scan=3;
  scan_flag[0]=0;
  scan_flag[1]=1;
  scan_flag[2]=2;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strcmp(got_value,"no")==0 || strncmp(got_value,"without_",8)==0) { 
      *headline=FLAG_NO;
    } else {
      *headline=FLAG_YES;
    }
  }

/* display unit */
  n_scan=1;
  scan_flag[0]=3;
  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    *x_unit=X_VELOCITY;
  }
  n_scan=2;
  scan_flag[0]=4;
  scan_flag[1]=5;
  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    *x_unit=X_RF_FREQ;
  }
  n_scan=1;
  scan_flag[0]=6;
  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    *x_unit=X_BE_FREQ;
  }
  n_scan=1;
  scan_flag[0]=7;
  if(StarOptionFlag(n_scan,scan_flag,key_flag)>0) {
    *x_unit=X_CHANNEL;
  }

/* profile data scaling */
  n_scan=2;
  scan_flag[0]=8;
  scan_flag[1]=9;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value!=(char)NULL) {
    if(strcmp(got_value,"unscaled")==0) {
      *scale_flag=UNSCALED;
    } else {
      *scale_flag=SCALED;
    }
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return 0;
}

int main(argc,argv)
int argc;
char *argv[];
{
  PROFILE profile;

  char id_str[MAX_FILE_NAME_LENGTH];
  char argument[MAX_IDX_RECORD_LENGTH];

  int sdb_flag;

  int ii,n_file,err_code;

  int x_unit,headline,scale_flag;

/* get options */
  err_code=options(argc,argv, &x_unit,&headline,&scale_flag);

  n_file=0;
  StarPipeHeadIn(id_str,&n_file);

  for(ii=0;ii<n_file;ii++) {
/* header lines in text output */
    if(headline==FLAG_YES) {
      switch(x_unit) {
      case X_VELOCITY:
	fprintf(stdout,"v(km/s)\t");
	break;
      case X_RF_FREQ:
	fprintf(stdout,"RF(GHz)\t");
	break;
      case X_BE_FREQ:
	fprintf(stdout,"backend freq(MHz)\t");
	break;
      case X_CHANNEL: default:
	fprintf(stdout,"ch\t");
	break;
      }
      fprintf(stdout,"\tTa*(K)\n");
    }

    err_code=StarProfileIn(&profile,&sdb_flag,argument);
    if(err_code!=0)	{
      fprintf(stderr,"Save: fail to ProfileIn No.%d\n",ii);
      exit(1);
    }

    if(sdb_flag!=GOOD_SCAN && sdb_flag!=BAD_SCAN)
      goto next_profile;

    print_profile_data(&profile,x_unit,scale_flag);

next_profile:;
  }

  exit(0);
}
