/*
cub flow from sdb flow

1999/7/28 T.Handa : ver.3.02
2002/5/11 T.Handa : ver.3.03 (debug) StarCubeScanDatavalue() is in lib
2002/6/10 T.Handa : ver.3.04 (debug) cube.head.data_value.noise from baseline data
2003/1/5 T.Handa : ver.3.05 (revise) add -pos_error
2003/1/7 T.Handa : ver.3.06 (debug) for sdb with comment/command lines
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define DEFAULT_PRMFILE "sdb2cub.prm"
#define N_KEY 30

#define NEAREST 1

int options(argc,argv,all_flag,coord_code,pos_range,grid_size,fill_method,dummy_value,v_range,output_filename,pos_error)
/*
get parameters from options
1999/4/25 T.Handa
*/
int argc;
char *argv[];
int *all_flag;
int *coord_code;
BOX_RANGE *pos_range;
XY_POS *grid_size;
int *fill_method;
double *dummy_value;
RANGE *v_range;
char *output_filename;
double *pos_error;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;
  int opt_pos;

  RANGE grid_range,range;

/* default values */
  all_flag[0]=1; /* all for longitude */
  all_flag[1]=1; /* all for latitude */
  all_flag[2]=1; /* all for velocity */

  *fill_method=NEAREST;
  (*grid_size).x=1.0/8.0;	/* 1/8 deg */
  (*grid_size).y=(*grid_size).x;
  *coord_code=COORD_LB;
  (*pos_range).x.min=0.0;
  (*pos_range).x.max=0.0;
  (*pos_range).y.min=0.0;
  (*pos_range).y.max=0.0;
  *dummy_value= -9999.0;
  (*v_range).min= -100000.0;
  (*v_range).max= 100000.0;
  strcpy(output_filename,"");
  *pos_error= -0.1;

/* set keywords */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"l");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"b");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"ra");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"dec");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"az");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"el");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"x");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"method");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"grid");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"v");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"dummy");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"pos_error"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"cub"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"output_file"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"out_cub"); key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* coordinate */
  scan_flag[0]= 0;
  scan_flag[1]= 1;
  scan_flag[2]= 2;
  scan_flag[3]= 3;
  scan_flag[4]= 4;
  scan_flag[5]= 5;
  scan_flag[6]= 6;
  scan_flag[7]= 7;
  n_scan=8;

  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);

  if(opt_pos>=0) {
    if(strcmp(keyword[opt_pos],"l")==0 || strcmp(keyword[opt_pos],"b")==0) {
      *coord_code=COORD_LB;
    } else if(strcmp(keyword[opt_pos],"ra")==0 || strcmp(keyword[opt_pos],"dec")==0) {
      *coord_code=COORD_RADEC;
    } else if(strcmp(keyword[opt_pos],"az")==0 || strcmp(keyword[opt_pos],"el")==0) {
      *coord_code=COORD_AZEL;
    } else if(strcmp(keyword[opt_pos],"x")==0 || strcmp(keyword[opt_pos],"y")==0) {
      *coord_code=COORD_XY;
    }
  }

/* position range */
  n_scan=4;
  scan_flag[0]= 0;
  scan_flag[1]= 2;
  scan_flag[2]= 4;
  scan_flag[3]= 6;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    StarOptionAngleRange(got_value,&range);
    (*pos_range).x.min=range.min;
    (*pos_range).x.max=range.max;
    all_flag[0]=0;
  }

  n_scan=4;
  scan_flag[0]= 1;
  scan_flag[1]= 3;
  scan_flag[2]= 5;
  scan_flag[3]= 7;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    StarOptionAngleRange(got_value,&range);
    (*pos_range).y.min=range.min;
    (*pos_range).y.max=range.max;
    all_flag[1]=0;
  }

/* v range */
  n_scan=1;
  scan_flag[0]=10;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    StarOptionRange(got_value,&range);
    (*v_range).min=range.min;
    (*v_range).max=range.max;
    all_flag[2]=0;
  }

/* method */
  scan_flag[0]= 8;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(strcmp(got_value,"nearest")==0) {
    *fill_method=NEAREST;
  }

/* grid interval */
  scan_flag[0]=9;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    StarOptionAngleRange(got_value,&grid_range);
    (*grid_size).x=grid_range.min;
    (*grid_size).y=grid_range.max;
  }

/* dummy value */
  scan_flag[0]=11;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value!=NULL && got_value[0]!=(char)NULL) {
    *dummy_value=atof(got_value);
  }

/* allowed position error */
  scan_flag[0]=12;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    *pos_error=StarStdAngle2deg(got_value);
  }

/* name for output cube */
  scan_flag[0]=13;
  scan_flag[1]=14;
  scan_flag[2]=15;
  n_scan=3;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(output_filename,got_value);
  }

  got_value=StarKeylessOption(argc,argv,n_key,key_flag,key_type);
  if(got_value[0]!=(char)NULL) {
    strcpy(output_filename,got_value);
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

int estimate_noise_square(profile,noise_level_sq)
PROFILE *profile; /* <in> */
double *noise_level_sq; /* <out> */
{
  int n_base_pixel;
  int ch,max_ch;
  int baseline_flag;
  double noise_sq;
  double ta,scale_sq;

  max_ch=(*profile).be_status.data_ch;
  n_base_pixel=0;
  scale_sq=(double)(*profile).rx_status.scale;
  scale_sq=scale_sq*scale_sq;

  noise_sq=0.0;
  for(ch=1;ch<=max_ch;ch++) {
    baseline_flag=StarReadBaselineFlag(profile,ch);
    if(baseline_flag==FLAGGED) {
      ta=(double)(*profile).data[ch-1];
      noise_sq=noise_sq+ta*ta;
      n_base_pixel++;
    }
  }
  *noise_level_sq=noise_sq*scale_sq;

  return n_base_pixel;
}

int profile_load(scnname,profile,cmd)
char *scnname;
PROFILE *profile;
char *cmd;
{
  char full_filename[MAX_FILE_NAME_LENGTH];
  FILE *fp;
  int err_code;

/* add STD extension for scan */
  err_code=StarStdFileExtName("STAR_DATA",scnname,full_filename,".scn");
  if(err_code!=0) {
    fprintf(stderr,"%s:Too long file name\n",cmd);
    return(1);
  }
  fp=fopen(full_filename,"r");
  if(fp==NULL) {
    fprintf(stderr,"%s:Cannot open file=%s\n",cmd,full_filename);
    return(2);
  }
  StarProfileLoad(fp,profile);
  fclose(fp);

  strcpy((*profile).file_name,scnname);

  StarReduceString((*profile).obs_prm.project);
  StarReduceString((*profile).obs_prm.scd_file);
  StarReduceString((*profile).obs_prm.observer1);
  StarReduceString((*profile).obs_prm.observer2);
  StarReduceString((*profile).obs_prm.observer3);
  StarReduceString((*profile).obs_prm.observer4);
  StarReduceString((*profile).object.name);
  StarReduceString((*profile).ant_status.tele);
  StarReduceString((*profile).rx_status.name);
  StarReduceString((*profile).be_status.name);
  StarReduceString((*profile).obs_log.site_name);
  StarReduceString((*profile).obs_log.memo);
  StarReduceString((*profile).obs_log.comment);

  return(0);
}

void set_syshead(sys_head)
SYS_HEAD *sys_head;
{
  (*sys_head).l_file=CUBE_INITIAL_L_HEAD; /* file length */
  (*sys_head).l_head=(*sys_head).l_file; /* header part length */
  (*sys_head).format_id=XYZ_CUBE;
  (*sys_head).version=CUBE_FORMAT_VERSION;
  (*sys_head).data_type=TA_DATA;
  (*sys_head).hist_step=0;
  (*sys_head).hist_line=0;
  (*sys_head).scan=0;
  (*sys_head).backup=0;
  (*sys_head).num_scan=0;
}

void set_axis(axis,profile,pos_range,grid_size)
AXES *axis;
PROFILE *profile;
BOX_RANGE *pos_range;
XY_POS *grid_size;
{
  (*axis).dimension=3;
  strcpy((*axis).order,"xyz");

  strcpy((*axis).x.name,"l");
  if((*axis).x.size<0) (*axis).x.size= -(*axis).x.size;
  if((*grid_size).x==0.0) {
    (*axis).x.size=0;
  } else {
    (*axis).x.size=(short)ceil(((*pos_range).x.max-(*pos_range).x.min)/(*grid_size).x);
  }
  if((*axis).x.size<0) {
    (*axis).x.size= -(*axis).x.size;
    (*grid_size).x= -(*grid_size).x;
  }
  (*axis).x.size=(*axis).x.size+1;
  strcpy((*axis).x.type,"long");
  strcpy((*axis).x.unit,"degree");
  strcpy((*axis).x.project,"linear");
  strcpy((*axis).x.refer,"lb2");
  (*axis).x.delta=(*grid_size).x;
  (*axis).x.origin=(*pos_range).x.min;
  (*axis).x.pixel_origin=0.0;
  (*axis).x.rotation=0.0;
  (*axis).x.resolution=(double)(*profile).ant_status.hpbw/60.0;
  strcpy((*axis).x.psf,"gauss");
  (*axis).x.res_orig=(*axis).x.resolution;

  strcpy((*axis).y.name,"b");
  if((*grid_size).y==0.0) {
    (*axis).y.size=0;
  } else {
    (*axis).y.size=(short)ceil(((*pos_range).y.max-(*pos_range).y.min)/(*grid_size).y);
  }
  if((*axis).y.size<0) {
    (*axis).y.size= -(*axis).y.size;
    (*grid_size).y= -(*grid_size).y;
  }
  (*axis).y.size=(*axis).y.size+1;
  strcpy((*axis).y.type,"lat");
  strcpy((*axis).y.unit,"degree");
  strcpy((*axis).y.project,"linear");
  strcpy((*axis).y.refer,"lb2");
  (*axis).y.delta=(*grid_size).y;
  (*axis).y.origin=(*pos_range).y.min;
  (*axis).y.pixel_origin=0.0;
  (*axis).y.rotation=0.0;
  (*axis).y.resolution=(double)(*profile).ant_status.hpbw/60.0;
  strcpy((*axis).y.psf,"gauss");
  (*axis).y.res_orig=(*axis).y.resolution;

  strcpy((*axis).z.name,"LSR velocity");
  (*axis).z.size=(*profile).be_status.data_ch;
  strcpy((*axis).z.type,"velocity");
  strcpy((*axis).z.unit,"km/s");
  strcpy((*axis).z.project,"linear");
  strcpy((*axis).z.refer,"LSR");
  (*axis).z.delta=StarCh2Velocity(profile,2.0)-StarCh2Velocity(profile,1.0);
  (*axis).z.origin=StarCh2Velocity(profile,1.0);
  (*axis).z.pixel_origin=0.0;
  (*axis).z.rotation=0.0;
  (*axis).z.resolution=(*profile).be_status.freq_res/(*profile).rx_status.rest_freq*1.0e-3*LIGHT_SPEED;
  strcpy((*axis).z.psf,"gauss");
  (*axis).z.res_orig=(*axis).z.resolution;
}

void set_datavalue(data,dummy)
DATA_VALUE *data;
double dummy;
{
  strcpy((*data).type,"Ta*");
  strcpy((*data).unit,"K");
  (*data).scale=1.0;
  (*data).offset=0.0;
  (*data).dummy=(float)dummy;
  (*data).max=(*data).dummy;
  (*data).min=(*data).dummy;
  (*data).mean=(*data).dummy;
  (*data).rms=0.0;
  (*data).noise=0.0;
}

void add_datasize4syshead(cube_head)
CUBE_HEAD *cube_head;
{
  long nx,ny,nz;
  long total_pixel;

  nx=(*cube_head).axis.x.size;
  ny=(*cube_head).axis.y.size;
  nz=(*cube_head).axis.z.size;

  total_pixel=nx*ny*nz;

  (*cube_head).sys.l_file=(*cube_head).sys.l_file+total_pixel*sizeof(float);
}


void set_cubehead(cube_head,profile,pos_range,grid_size,dummy)
CUBE_HEAD *cube_head;
PROFILE *profile;
BOX_RANGE *pos_range;
XY_POS *grid_size;
double dummy;
{
  HISTORY history;

  strcpy(history.step,"sdb2cub");
  history.size=0;

/* Copy from profile */
  StarCopySyshead(&((*cube_head).sys),&((*profile).sys_head));
  StarCopyObsprm(&((*cube_head).obs),&((*profile).obs_prm));
  StarCopyObject(&((*cube_head).object),&((*profile).object));
  StarCopyAntstatus(&((*cube_head).ant),&((*profile).ant_status));
  StarCopyRxstatus(&((*cube_head).rx),&((*profile).rx_status));
  StarCopyBestatus(&((*cube_head).be),&((*profile).be_status));
  StarCopyObslog(&((*cube_head).log),&((*profile).obs_log));

/* set default */
  set_syshead(&((*cube_head).sys));

  set_axis(&((*cube_head).axis),profile,pos_range,grid_size);
  set_datavalue(&((*cube_head).data_value),dummy);
  add_datasize4syshead(cube_head);

  StarAddHistory4Cube(cube_head,&history);
}

void remove_sdb(file_name)
char *file_name;
{
  int l_str;

  l_str=strlen(file_name);
  if(l_str<4) return;

  if(strcmp(&(file_name[l_str-4]),".sdb")==0)
    file_name[l_str-4]=(char)NULL;
}

int is_profile_in_cube(cube_head,profile,coord_code,ix,iy,pos_error)
/*
return value
=1 profile is in cube
=0 profile is out of cube

1998/11/16 T.Handa
2004/1/5 T.Handa ; pos_error entry
*/
CUBE_HEAD *cube_head;
PROFILE *profile;
int coord_code;
int *ix,*iy;
double pos_error; /* <in> pos: allowance in deg, neg:allowance in HPBW unit */
{
  double hpbw;
  double xx,yy;
  double xx_new,yy_new;
  double dist;
  double allowance;

  hpbw=(double)(*cube_head).ant.hpbw/60.0;

  if(pos_error==0.0) pos_error= -0.1;
  if(pos_error<0.0) {
    allowance=hpbw*(-pos_error);
  } else {
    allowance=pos_error;
  }

  switch(coord_code) {
  case COORD_LB:
    xx=(*profile).object.on_lb.x;
    yy=(*profile).object.on_lb.y;
    break;
  case COORD_RADEC:
    xx=(*profile).object.on_radec.x;
    yy=(*profile).object.on_radec.y;
    break;
  case COORD_XY:
    xx=(*profile).object.on_xy.x;
    yy=(*profile).object.on_xy.y;
    break;
  case COORD_AZEL:
    xx=(*profile).object.on_azel.x;
    yy=(*profile).object.on_azel.y;
    break;
  default:
    xx=0.0;
    yy=0.0;
  }
  *ix=(int)rint((xx-(*cube_head).axis.x.origin)/(*cube_head).axis.x.delta);
  *iy=(int)rint((yy-(*cube_head).axis.y.origin)/(*cube_head).axis.y.delta);

  if(*ix<0 || *ix>=(int)(*cube_head).axis.x.size) {
    return(0);
  }
  if(*iy<0 || *iy>=(int)(*cube_head).axis.y.size) {
    return(0);
  }

  xx_new=(double)(*ix)*(*cube_head).axis.x.delta+(*cube_head).axis.x.origin;
  yy_new=(double)(*iy)*(*cube_head).axis.y.delta+(*cube_head).axis.y.origin;
  dist=(xx-xx_new)*(xx-xx_new)+(yy-yy_new)*(yy-yy_new);
  dist=sqrt(dist);

  if(dist<allowance) {
    return(1);
  } else {
    return(0);
  }
}

void copy_profile2cube(cube,ix,iy,profile)
CUBE *cube;
int ix,iy;
PROFILE *profile;
{
  int ch,data_ch;
  float data_value;

  data_ch=(*profile).be_status.data_ch;

  for(ch=0;ch<data_ch;ch++) {
    if(StarReadSpuriousFlag(profile,ch+1)==FLAGGED) {
      data_value=(*cube).head.data_value.dummy;
    } else {
      data_value=(*profile).data[ch];
    }

    StarPutCubePixel(cube,ix,iy,ch,data_value);
  }
}

int order_of_be_dispersion(profile)
PROFILE *profile;
{
  if((*profile).be_status.disp4!=0.0) return 4;
  if((*profile).be_status.disp3!=0.0) return 3;
  if((*profile).be_status.disp2!=0.0) return 2;
  if((*profile).be_status.disp1!=0.0) return 1;
  return 0;
}

int match_be_dispersion(cube,profile)
CUBE_HEAD *cube;
PROFILE *profile;
{
  if((*cube).be.disp0!=(*profile).be_status.disp0) return 0;
  if((*cube).be.disp1!=(*profile).be_status.disp1) return 1;
  if((*cube).be.disp2!=(*profile).be_status.disp2) return 2;
  if((*cube).be.disp3!=(*profile).be_status.disp3) return 3;
  if((*cube).be.disp4!=(*profile).be_status.disp4) return 4;
  return 5;
}

int debug_profileheader(cmd,profile,grid_size)
char *cmd;
PROFILE *profile;
XY_POS grid_size;
{
  double grid_span;

  if((*profile).ant_status.hpbw<=0.0) {
    grid_span=grid_size.x*grid_size.x+grid_size.y*grid_size.y;
    grid_span=sqrt(grid_span)*60.0;  /* deg --> arcmin */

    fprintf(stderr,"%s:* WRN * invalid HPBW %f\'. Set to %f\' using grid_size.\n",cmd,(*profile).ant_status.hpbw,grid_span);

    (*profile).ant_status.hpbw=grid_span;
    return 1;
  }

  return 0;
}


int main(argc,argv)
int argc;
char *argv[];
{
  PROFILE profile;

  CUBE cube;
  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];

  int err_code;
  int ix,iy,in_flag;

  int coord_code;
  BOX_RANGE pos_range;
  XY_POS grid_size;
  RANGE v_range;
  int all_flag[3];
  double pos_error;

  double noise_level_sq,noise_level_sq_in_prf;
  int n_base_pixel,n_base_in_prf;

  int fill_method;
  double dummy_value;
  char output_cubename[MAX_FILE_NAME_LENGTH];
  char default_sdbname[MAX_FILE_NAME_LENGTH];
  int i_file,n_file,backup_version;
  int n_cube_file;

  char scnname[MAX_FILE_NAME_LENGTH];
  char sdb_arg[MAX_IDX_RECORD_LENGTH];
  int sdb_flag;

  static int header_flag=1;

/* get option */
  err_code=options(argc,argv,all_flag,&coord_code,&pos_range,&grid_size,&fill_method,&dummy_value,&v_range,output_cubename,&pos_error);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

/* .sdb header */
  err_code=StarSdbHeadIn(default_sdbname,&n_file,&backup_version);
  if(err_code!=0) {
    fprintf(stderr,"%s: Bronken Pipe. Not a sdb file.\n",argv[0]);
    exit(1);
  } else if(n_file==0) {
    fprintf(stderr,"%s: Empty sdb flow.\n",argv[0]);
    exit(1);
  }
  remove_sdb(default_sdbname);
  if(output_cubename[0]==(char)NULL) strcpy(output_cubename,default_sdbname);
  strcpy(cube.head.file_name,default_sdbname);
  strcpy(cube.head.file_id,STAR_CUB_IN);

/* get profiles */
  n_base_pixel=0;
  noise_level_sq=0.0;

  for(i_file=0;i_file<n_file;i_file++) {
    err_code=StarSdbIn(scnname,&sdb_flag,sdb_arg);
    switch(err_code) {
    case 201: case 301:
      sdb_flag=GOOD_SCAN;
      err_code=0;
      break;
    case 100:
      goto loopend; /* EOF */
    }

/* load profile data */
    if(sdb_flag==GOOD_SCAN) {
      err_code=profile_load(scnname,&profile,argv[0]);
      if(err_code!=0) {
	sdb_flag=BAD_SCAN;
	exit(1);
      }

/* check backend dispersion */
      if(order_of_be_dispersion(&profile)!=1) {
	sdb_flag=BAD_SCAN;
	fprintf(stderr,"%s, with not linear BE dispersion.\n",profile.file_name);
      }
      if(header_flag==0) {
	if(match_be_dispersion(&(cube.head),&profile)<5) {
	  sdb_flag=BAD_SCAN;
	  fprintf(stderr,"%s, with unmatched BE dispersion.\n",profile.file_name);
	}
/* setup cub header using first GOOD_SCAN */
      }
      if(header_flag==1) {

	debug_profileheader(argv[0],&profile,grid_size);

	set_cubehead(&(cube.head),&profile,&pos_range,&grid_size,dummy_value);
	header_flag=0;

	cube.pixel=StarAllocCubePixel(&(cube.head));
	if(cube.pixel==NULL) {
	  fprintf(stderr,"%s: **ERR** cannot alloc memory for pixel data\n",argv[0]);
	  exit(999);
	}
	StarClearCubePixel(&cube);
      }
    }
    if(sdb_flag!=GOOD_SCAN) goto nextscn;

    in_flag=is_profile_in_cube(&(cube.head),&profile,coord_code,&ix,&iy,pos_error);
    if(in_flag==1) {
      copy_profile2cube(&cube,ix,iy,&profile);
      n_base_in_prf=estimate_noise_square(&profile,&noise_level_sq_in_prf);
      if(n_base_in_prf>0) {
	n_base_pixel=n_base_pixel+n_base_in_prf;
	noise_level_sq=noise_level_sq+noise_level_sq_in_prf;
      }
    } else {
      fprintf(stderr,"%s=(%d,%d),out of cube\n",profile.file_name,ix,iy);
    }

nextscn:;
loopend:;
  }

  StarCubeScanDatavalue(&cube);
  if(n_base_pixel>1) {
    noise_level_sq=noise_level_sq/(double)(n_base_pixel-1);
  } else if(n_base_pixel<=0) {
    noise_level_sq=0.0;
  }
  cube.head.data_value.noise=sqrt(noise_level_sq);

/* pipe line out */
  n_cube_file=1;
  StarPipeHeadOut(n_cube_file);

  cdb_flag=GOOD_SCAN;
  strcpy(cdb_arg,"");

/* pipeout cube data */
  err_code=StarCubeOut(&cube,cdb_flag,cdb_arg);

  StarFreeCubePixel(&cube);

  StarPipeEndOut();

  exit(0);
}
