/*
sdb file, header parameter sorting

2000/2/11 T.Handa : ver.3.00
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <StData.h>
#include <StStdio.h>
#include <StStdlib.h>
#include <StPipe.h>
#include <StProfileIO.h>
#include <StProfileOpt.h>

#define	DEFAULT_PRMFILE	"sdbgridsize.prm"
#define N_KEY 10

int options(argc,argv,pos_error,pipe_flag)
double *pos_error;
int *pipe_flag;
{
  int	ii;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;
  int opt_pos;

  int	err_code,flag;

/* default values */
  *pipe_flag=FLAG_NO;	/* pipe out = no */
  *pos_error= -0.1;


/* set keywords */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"pos_error"); key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* allowed position error */
  scan_flag[0]=12;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    *pos_error=StarStdAngle2deg(got_value);
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

int main(argc,argv)
int argc;
char *argv[];
{
  char	output_filename[MAX_FILE_NAME_LENGTH],backup_filename[MAX_FILE_NAME_LENGTH];
  char full_output_filename[MAX_FILE_NAME_LENGTH];
  /*  char id_str[MAX_FILE_NAME_LENGTH]; */
  char default_filename[MAX_FILE_NAME_LENGTH];
  char scnname[MAX_FILE_NAME_LENGTH];
  char argument[MAX_IDX_RECORD_LENGTH];

  int	ii,n_line,err_code;
  int	pipe_flag,backup_flag;

  int	sdb_flag;

  int backup_version,with_backver,rename_flag;

  FILE *fp;

/* option */
  err_code=options(argc,argv,&pipe_flag,&backup_flag,&with_backver);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

  n_line=1;

/* .sdb header */
  err_code=StarSdbHeadIn(default_filename,&n_line,&backup_version);
  if(err_code!=0) {
    fprintf(stderr,"%s: Broken Pipe. Not a sdb file.\n",argv[0]);
    exit(1);
  } else if(n_line==0) {
    fprintf(stderr,"%s: Empty sdb flow.\n",argv[0]);
    exit(1);
  }

  StarGetSavename(argc,argv,default_filename,output_filename);
  err_code=StarStdFileExtName("STAR_DATA",output_filename,full_output_filename,".sdb"); 
  if(err_code!=0) {
    fprintf(stderr,"Too long file name\n");
    exit(1);
  }

/* backup */
  if(backup_flag==FLAG_YES) {
    StarGetBackupname(backup_filename,default_filename,backup_version,with_backver,".sdb");
    StarRename4backup(full_output_filename,backup_filename);
  }
  backup_version++;

  fp=fopen(full_output_filename,"w");

/* do pipe out */
  if(pipe_flag==FLAG_YES) 
    StarSdbHeadOut(default_filename,n_line,backup_version);

  StarSdbHeadSave(fp,backup_version);

/* main loop */
  for(ii=0;ii<n_line;ii++) {
    err_code=StarSdbIn(scnname,&sdb_flag,argument);
    switch(err_code) {
    case 201: case 301:
      sdb_flag=GOOD_SCAN;	/* without flag means "good" */
      err_code=0;
      break;
    case 100:
      goto loopend;	/* too many flags are no problem */

    }

    if(pipe_flag==FLAG_YES) {
      err_code=StarSdbOut(scnname,sdb_flag,argument);
      if(err_code!=0) {
	fprintf(stderr,"%s: pipeout error for %s\n",argv[0],scnname);
      }
    }

    err_code=StarSdbSave(fp,scnname,sdb_flag,argument);
  }
loopend:;
  if(strncmp(argument,"!end",3)!=0) {
    fprintf(fp,"!end\n"); /* add endmark of sdb file */
  }

  fclose(fp);
  exit(0);
}
