/*
sdb file loader

1992/12/21 T.Handa ver.1.0
1998/ 4/ 3 T.Handa : ver.3.00 new lib & multi-pipe
1998/ 4/10 T.Handa : ver.3.20 (revise) command/comment lines
1998/ 4/28 T.Handa : ver.3.21 (revise) check max entries
2002/ 5/ 8 T.Handa : ver.3.30 (revise) !end command for the last line
2006/5/22 T.Handa : ver.3.31 (debug) core dump via ssh; 2D array should be static
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"sdbload.prm"
#define N_KEY 30

int options(argc,argv,in_sdbname)
/*
get parameters
*/
int	argc;
char	*argv[];
char	in_sdbname[];
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  in_sdbname[0]=(char)NULL;

/* set keywords */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"sdb");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"in_sdb");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"input_file");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* name for the integrated scan */
  scan_flag[0]=0;
  scan_flag[1]=1;
  scan_flag[2]=2;
  n_scan=3;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  strcpy(in_sdbname,got_value);

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

int main(argc,argv)
int argc;
char *argv[];
{
  char	in_sdbname[MAX_FILE_NAME_LENGTH],
    full_sdbname[MAX_FILE_NAME_LENGTH];
  static char	scnname[MAX_FILE_NUMBER][MAX_FILE_NAME_LENGTH];

  FILE	*fp;	/* for .sdb file */
  int	sdb_flag[MAX_FILE_NUMBER];
  static char argument[MAX_FILE_NUMBER][MAX_IDX_RECORD_LENGTH];

  int	ii,n_lines;
  int	err_code;
  int	back_ver;

/* option switches */
  err_code=options(argc,argv,in_sdbname);

  if(argc<2) {
    fprintf(stderr,"Usage: %s sdbfilename\n",argv[0]);
    exit(1);
  } else {
    for(ii=1;ii<argc;ii++) {
      if(argv[ii][0]!='-') {
	strcpy(in_sdbname,argv[1]);
	break;
      }
    }
  }

/* modify options */
  err_code=StarStdFileExtName("STAR_DATA",in_sdbname,full_sdbname,".sdb");
  fp=fopen(full_sdbname,"r");
  if(fp==NULL) {
    fprintf(stderr,"%s: Can't open input sdbfile %s\n",argv[0],in_sdbname);
    exit(1);
  }

/* read loop */
  n_lines=0;
  StarSdbHeadLoad(fp,&back_ver);
  while(1) {
looptop:;
    err_code=StarSdbLoad(fp,scnname[n_lines],&(sdb_flag[n_lines]),argument[n_lines]);
    switch(err_code) {
    case 201: case 301: /* no flag asigned */
      sdb_flag[n_lines]=GOOD_SCAN;
      err_code=0;
      break;
    case 100: /* EoF */
      goto loopend;
    case 0: /* normal */
      n_lines++;
      if(sdb_flag[n_lines-1]==IDX_COMMAND && strncmp(argument[n_lines-1],"!end",4)==0)
	goto readytoout;
      break;
    case 11: /* blank line */
      goto looptop;
    }

    if(n_lines>=MAX_FILE_NUMBER) {
      fprintf(stderr,"%s: too many files. MAX=%d. Skip the rest.\n",argv[0],n_lines);
      break;
    }
  }
loopend:;

  if(strncmp(argument[n_lines],"!end",4)!=0) {
    strcpy(scnname[n_lines],"");
    sdb_flag[n_lines]=IDX_COMMAND;
    strcpy(argument[n_lines],"!end");
    n_lines++;
  }

readytoout:;
/* pipe out loop */
  StarSdbHeadOut(in_sdbname,n_lines,back_ver);
  for(ii=0;ii<n_lines;ii++) {
    err_code=StarSdbOut(scnname[ii],sdb_flag[ii],argument[ii]);
    if(err_code!=0) {
      fprintf(stderr,"%s: pipeout error for %s\n",argv[0],scnname[ii]);
    }
  }

  exit(0);
}
