/*
sdb file saver

1992/12/21 T.Handa ver.1.0
1998/ 4/ 3 T.Handa : ver.3.00 new lib & multi-pipe
1998/ 4/10 T.Handa : ver.3.20 (revise) command/comment lines
1999/ 3/ 4 T.Handa : ver.3.21 (revise) change default from noback to back
2002/ 5/ 8 T.Handa : ver.3.22 (revise) add !end for the last line
2003/ 1/ 2 T.Handa : ver.3.30 (revise) Backup directory
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"sdbsave.prm"
#define N_KEY 30

int options(argc,argv,pipe_flag,backup_flag,with_backver)
/*
get parameters from options
*/
int argc;
char *argv[];
int *pipe_flag;
int *backup_flag;
int *with_backver;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int scan_flag[N_KEY];

  int flag;

/* default values */
  *pipe_flag=FLAG_NO;	/* pipe out = no */
  *backup_flag=FLAG_YES;	/* backup = yes */
  *with_backver=FLAG_YES;

/* set keywords */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"pipe");
  n_key=AddNewWord(keyword,"output");
  n_key=AddNewWord(keyword,"-");
  n_key=AddNewWord(keyword,"nopipe");
  n_key=AddNewWord(keyword,"backup");
  n_key=AddNewWord(keyword,"nobackup");
  n_key=AddNewWord(keyword,"with_ver_ext");
  n_key=AddNewWord(keyword,"without_ver_ext");

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* pipe */
  scan_flag[0]= 0;
  scan_flag[1]= 1;
  scan_flag[2]= 2;
  scan_flag[3]= -3;
  flag=StarOptionFlag(4,scan_flag,key_flag);
  if(flag!=0) *pipe_flag=flag;

/* backup file */
  scan_flag[0]= 4;
  scan_flag[1]= -5;
  flag=StarOptionFlag(2,scan_flag,key_flag);
  if(flag!=0) *backup_flag=flag;

/* version extension for backup file */
  scan_flag[0]= 6;
  scan_flag[1]= -7;
  flag=StarOptionFlag(2,scan_flag,key_flag);
  if(flag!=0) *with_backver=flag;

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

int main(argc,argv)
int argc;
char *argv[];
{
  char	output_filename[MAX_FILE_NAME_LENGTH],backup_filename[MAX_FILE_NAME_LENGTH];
  char full_output_filename[MAX_FILE_NAME_LENGTH];
  /*  char id_str[MAX_FILE_NAME_LENGTH]; */
  char default_filename[MAX_FILE_NAME_LENGTH];
  char scnname[MAX_FILE_NAME_LENGTH];
  char argument[MAX_IDX_RECORD_LENGTH];

  int	ii,n_line,err_code;
  int	pipe_flag,backup_flag;

  int	sdb_flag;

  int backup_version,with_backver;

  FILE *fp;

/* option */
  err_code=options(argc,argv,&pipe_flag,&backup_flag,&with_backver);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

  n_line=1;

/* .sdb header */
  err_code=StarSdbHeadIn(default_filename,&n_line,&backup_version);
  if(err_code!=0) {
    fprintf(stderr,"%s: Broken Pipe. Not a sdb file.\n",argv[0]);
    exit(1);
  } else if(n_line==0) {
    fprintf(stderr,"%s: Empty sdb flow.\n",argv[0]);
    exit(1);
  }

  StarGetSavename(argc,argv,default_filename,output_filename);
  err_code=StarStdFileExtName("STAR_DATA",output_filename,full_output_filename,".sdb"); 
  if(err_code!=0) {
    fprintf(stderr,"Too long file name\n");
    exit(1);
  }

/* backup */
  if(backup_flag==FLAG_YES) {
    StarGetBackupname(backup_filename,default_filename,backup_version,with_backver,".sdb");
    StarRename4backup(full_output_filename,backup_filename);
  }
  backup_version++;

  fp=fopen(full_output_filename,"w");

/* do pipe out */
  if(pipe_flag==FLAG_YES) 
    StarSdbHeadOut(default_filename,n_line,backup_version);

  StarSdbHeadSave(fp,backup_version);

/* main loop */
  for(ii=0;ii<n_line;ii++) {
    err_code=StarSdbIn(scnname,&sdb_flag,argument);
    switch(err_code) {
    case 201: case 301:
      sdb_flag=GOOD_SCAN;	/* without flag means "good" */
      err_code=0;
      break;
    case 100:
      goto loopend;	/* too many flags are no problem */

    }

    if(pipe_flag==FLAG_YES) {
      err_code=StarSdbOut(scnname,sdb_flag,argument);
      if(err_code!=0) {
	fprintf(stderr,"%s: pipeout error for %s\n",argv[0],scnname);
      }
    }

    err_code=StarSdbSave(fp,scnname,sdb_flag,argument);
  }
loopend:;
  if(strncmp(argument,"!end",3)!=0) {
    fprintf(fp,"!end\n"); /* add endmark of sdb file */
  }

  fclose(fp);
  exit(0);
}
