/*
sdb file, insert sdb-command between different groups

2002/ 5/ 9 T.Handa : ver.3.00 (only check HPBW, on position)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"sdbseparate.prm"
#define N_KEY 130

#define MAX_KEY 130
typedef struct { 
  int pos;
  unsigned long id;
  int size;
  int type;
} Key;

int options(argc,argv,sdbcommand,sdbcomment)
/*
get parameters from options
*/
int argc;
char *argv[];
char *sdbcommand; /* <out> sdb command to insert between groups */
char *sdbcomment; /* <out> sdb comment to insert between groups */
{
  int ii;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int scan_flag[N_KEY], n_scan;
  int key_type[N_KEY];

  char *got_value;
  int i_key_bottom;

/* default values */
  strcpy(sdbcommand,"!integ");
  strcpy(sdbcomment,"#");

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  /* system header part */
  n_key=0;

  i_key_bottom=n_key;
  n_key=AddNewWord(keyword,"sdbcommand");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"sdbcmd");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"sdbcomment");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"sdbcmt");	key_type[n_key-1]=1;

/* Option Scan */
  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* sdbcommand between groups */
  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=i_key_bottom+ii;
  }
  i_key_bottom=i_key_bottom+n_scan;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(strcmp(got_value,"")!=0) {
    if(strcmp(got_value,"NULL")==0 || strcmp(got_value,"null")==0) {
      strcpy(sdbcommand,"!");
    } else {
      sprintf(sdbcommand,"!%s",got_value);
    }
  }

/* sdbcomment between groups */
  n_scan=2;
  for(ii=0;ii<n_scan;ii++) {
    scan_flag[ii]=i_key_bottom+ii;
  }
  i_key_bottom=i_key_bottom+n_scan;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(strcmp(got_value,"")!=0) {
    sprintf(sdbcomment,"#%s",got_value);
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

int profile_load(scnname,profile,cmd)
char *scnname;
PROFILE *profile;
char *cmd;
{
  char full_filename[MAX_FILE_NAME_LENGTH];
  FILE *fp;
  int err_code;

/* add STD extension for scan */
  err_code=StarStdFileExtName("STAR_DATA",scnname,full_filename,".scn");
  if(err_code!=0) {
    fprintf(stderr,"%s:Too long file name\n",cmd);
    return(1);
  }
  fp=fopen(full_filename,"r");
  if(fp==NULL) {
    fprintf(stderr,"%s:Cannot open file=%s\n",cmd,full_filename);
    return(2);
  }
  StarProfileLoad(fp,profile);
  fclose(fp);

  StarReduceString((*profile).obs_prm.project);
  StarReduceString((*profile).obs_prm.scd_file);
  StarReduceString((*profile).obs_prm.observer1);
  StarReduceString((*profile).obs_prm.observer2);
  StarReduceString((*profile).obs_prm.observer3);
  StarReduceString((*profile).obs_prm.observer4);
  StarReduceString((*profile).object.name);
  StarReduceString((*profile).ant_status.tele);
  StarReduceString((*profile).rx_status.name);
  StarReduceString((*profile).be_status.name);
  StarReduceString((*profile).obs_log.site_name);
  StarReduceString((*profile).obs_log.memo);
  StarReduceString((*profile).obs_log.comment);

  return(0);
}

int IsSamePosition(prf0,prf1)
PROFILE *prf0;
PROFILE *prf1;
/* 
compare on-position in the header of two profiles

2002/5/9 T.Handa : assume float/double must differ beyond 0.001
*/
{
  unsigned char coord_sys;
  double hpbw;

  if((*prf0).object.cood_sys!=(*prf1).object.cood_sys) return FLAG_NO;
  coord_sys=(*prf0).object.cood_sys;

  hpbw=(double)(*prf0).ant_status.hpbw;
  if(fabs(hpbw-(double)((*prf1).ant_status.hpbw))>hpbw*0.1) return FLAG_NO;

  hpbw=hpbw/60.0; /* arcmin to deg */

  if(coord_sys==COORD_LB) {
    if(fabs((*prf0).object.on_lb.x-(*prf1).object.on_lb.x)>0.1*hpbw
       || fabs((*prf0).object.on_lb.y-(*prf1).object.on_lb.y)>0.1*hpbw) {
      return FLAG_NO;
    }
  } else if(coord_sys==COORD_RADEC) {
    if(fabs((*prf0).object.on_radec.x-(*prf1).object.on_radec.x)>0.1*hpbw
       || fabs((*prf0).object.on_radec.y-(*prf1).object.on_radec.y)>0.1*hpbw) {
      return FLAG_NO;
    }
  } else if(coord_sys==COORD_RADEC_APP) {
    if(fabs((*prf0).object.on_radec.x-(*prf1).object.on_radec.x)>0.1*hpbw
       || fabs((*prf0).object.on_radec.y-(*prf1).object.on_radec.y)>0.1*hpbw) {
      return FLAG_NO;
    }
  } else if(coord_sys==COORD_AZEL) {
    if(fabs((*prf0).object.on_xy.x-(*prf1).object.on_xy.x)>0.1*hpbw
       || fabs((*prf0).object.on_xy.y-(*prf1).object.on_xy.y)>0.1*hpbw) {
      return FLAG_NO;
    }
  } else if(coord_sys==COORD_XY) {
    if(fabs((*prf0).object.pa_xy-(*prf1).object.pa_xy)>0.1) 
      return FLAG_NO;

    if(fabs((*prf0).object.on_xy.x-(*prf1).object.on_xy.x)>0.1*hpbw
       || fabs((*prf0).object.on_xy.y-(*prf1).object.on_xy.y)>0.1*hpbw) {
      return FLAG_NO;
    }
  }

  return FLAG_YES;
}

int main(argc,argv)
int argc;
char *argv[];
{
  PROFILE profile,prev_prf;

  char default_sdbname[MAX_FILE_NAME_LENGTH];
  char scnname[MAX_FILE_NAME_LENGTH];
  char argument[MAX_IDX_RECORD_LENGTH];
  char sdbcommand[MAX_IDX_RECORD_LENGTH];
  char sdbcomment[MAX_IDX_RECORD_LENGTH];

  int sdb_flag;
  int samehead_flag;
  int do_cmd,do_cmt;

  int ii,n_line,err_code;
  int backup_ver;

/* get options */
  options(argc,argv,sdbcommand,sdbcomment);

  if(strcmp(sdbcommand,"!")==0) do_cmd=FLAG_NO;
  else do_cmd=FLAG_YES;
  if(strcmp(sdbcomment,"#")==0) do_cmt=FLAG_NO;
  else do_cmt=FLAG_YES;

/* sort key position */

/* .sdb header */
  err_code=StarSdbHeadIn(default_sdbname,&n_line,&backup_ver);
  if(err_code!=0) {
    fprintf(stderr,"%s: Broken Pipe. Not a sdb file.\n",argv[0]);
    exit(1);
  } else if(n_line==0) {
    fprintf(stderr,"%s: Empty sdb flow.\n",argv[0]);
    exit(1);
  }

/* header out */
  StarSdbHeadOut(default_sdbname,MAX_FILE_NUMBER,backup_ver);

/* main loop */
  for(ii=0;ii<n_line;ii++) {
    err_code=StarSdbIn(scnname,&sdb_flag,argument);
    switch(err_code) {
    case 201: case 301:
      sdb_flag=GOOD_SCAN;	/* without flag means "good" */
      err_code=0;
      break;
    case 100:
      goto loopend;	/* EOF */

    }

    if(sdb_flag==GOOD_SCAN || sdb_flag==BAD_SCAN) {
      err_code=profile_load(scnname,&profile,argv[0]);
      if(err_code==0) {
	if(ii>0) {
	  samehead_flag=IsSamePosition(&prev_prf,&profile);
	  if(samehead_flag==FLAG_NO){
	    if(do_cmd==FLAG_YES) {
	      StarSdbOut("",IDX_COMMAND,sdbcommand);
	    }
	    if(do_cmt==FLAG_YES) {
	      StarSdbOut("",IDX_COMMENT,sdbcomment);
	    }
	  }
	}
	StarCopyProfile(&prev_prf,&profile);

      } else {
	sdb_flag=BAD_SCAN;
      }
    }

    if(sdb_flag==IDX_COMMAND) {
      if(do_cmd==FLAG_YES && strcmp(sdbcommand,argument)==0) {
	/* skip to output */
      } else {
	err_code=StarSdbOut(scnname,sdb_flag,argument);
      }
    } else if(sdb_flag==IDX_COMMENT) {
      if(do_cmt==FLAG_YES && strcmp(sdbcomment,argument)==0) {
	/* skip to output */
      } else {
	err_code=StarSdbOut(scnname,sdb_flag,argument);
      }
    } else {
      err_code=StarSdbOut(scnname,sdb_flag,argument);
    }
    if(err_code!=0) {
      fprintf(stderr,"%s: pipeout error for %s\n",argv[0],scnname);
    }
  }
loopend:;

  exit(0);
}
