/*
shift axis origin value or axis origin pixel value

2006/12/13 T.Handa : ver.3.00 
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"shiftaxis.prm"
#define N_KEY 50
#define N_BLOCK	10

#define MODE_NO 0
#define MODE_PIXEL_ORIG 1
#define MODE_VALUE_ORIG 2

#define DEBUG
//#undef DEBUG

void ShiftAxisOrigin(axis,mode,value)
AXIS *axis; /* <i/o> */
int mode; /* <in> =0 no, =1 pixel_origin, =2 origin */
double value; /* <in> */
{
  double delta;

  delta=(*axis).delta;

  if(mode==MODE_VALUE_ORIG) {
    (*axis).pixel_origin= (value-(*axis).origin)/delta+(*axis).pixel_origin;
    (*axis).origin=value;
  } else if(mode==MODE_PIXEL_ORIG) {
    (*axis).origin= (value-(*axis).pixel_origin)*delta+(*axis).origin;
    (*axis).pixel_origin=value;
  }

  return;
}

int options(argc,argv,reset_axis,new_orig)
/*
get parameters from options
*/
int argc;
char *argv[];
int *reset_axis;
double *new_orig;
{
  int ii,next_pos;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;
  char *got_value;

  int l_str;
  int i_axis;

/* default values */
  reset_axis[0]=MODE_NO;
  reset_axis[1]=MODE_NO;
  reset_axis[2]=MODE_NO;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"x_orig");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"x_axis");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y_orig");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"y_axis");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z_orig");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"z_axis");	key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

  next_pos=0;

/* new origin value along x-axis */
  for(i_axis=0;i_axis<3;i_axis++) {
    n_scan=2;
    for(ii=0;ii<n_scan;ii++) {
      scan_flag[ii]=next_pos+ii;
    }
    next_pos=next_pos+n_scan;

    got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
    if(got_value[0]!=(char)NULL) {
      l_str=strlen(got_value);
      if(strcasecmp(got_value,"no")==0) {
	reset_axis[i_axis]=MODE_NO; /* no effect */
      } else if(strncasecmp(got_value,"pixel",5)==0) {
	reset_axis[i_axis]=MODE_PIXEL_ORIG; /* pixel origin */
	new_orig[i_axis]=atof(&(got_value[5]));
      } else if(got_value[0]=='p') {
	reset_axis[i_axis]=MODE_PIXEL_ORIG; /* pixel origin */
	new_orig[i_axis]=atof(&(got_value[1]));
      } else if(strcasecmp(&(got_value[l_str-5]),"pixel")==0 || strcasecmp(&(got_value[l_str-3]),"pix")==0 || strcasecmp(&(got_value[l_str-1]),"p")==0) {
	reset_axis[i_axis]=MODE_PIXEL_ORIG; /* pixel origin */
	new_orig[i_axis]=atof(got_value);
      } else {
	reset_axis[i_axis]=MODE_VALUE_ORIG; /* origin */
	new_orig[i_axis]=StarStdAngle2deg(got_value);
      }
    }
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return(0);
}

int shiftaxis_add_history(cube_head,reset_axis,new_org)
CUBE_HEAD *cube_head; /* <i/o> */
int *reset_axis; /* <in> */
double *new_org; /* <in> */
{
  HISTORY hist;
  int h_size,ii;
  static char axis_name[3]={'x','y','z'};

  h_size=0;

  strcpy(hist.step,"shiftAx");

  for(ii=0;ii<3;ii++) {
    if(reset_axis[ii]!=MODE_NO) {
      sprintf(hist.prm[h_size],"%c-axis",axis_name[ii]);
      h_size++;
      if(reset_axis[ii]==MODE_PIXEL_ORIG) {
	sprintf(hist.prm[h_size],"%14ep",new_org[ii]);
      } else if(reset_axis[ii]==MODE_VALUE_ORIG) {
	sprintf(hist.prm[h_size],"%15e",new_org[ii]);
      }
      h_size++;
    }
  }
  hist.size=h_size;
  StarAddHistory4Cube(cube_head,&hist);

  return(0);
}

int main(argc,argv)
int argc;
char *argv[];
{
  CUBE cube_in,cube_out;

  int reset_axis[3];
  double new_orig[3];
  int err_code;

  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];

  int ii,n_file;
  char id_str[MAX_FILE_NAME_LENGTH];

/* check option */
  err_code=options(argc,argv,reset_axis,new_orig);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

/* pipe header */
  StarPipeHeadIn(id_str,&n_file);

  StarPipeHeadOut(n_file);

/* .cub data */
  for(ii=0;ii<n_file;ii++) {
    err_code=StarCubeIn(&cube_in,&cdb_flag,cdb_arg);
    if(err_code!=0) {
      fprintf(stderr,"%s: CubeIn failed in %dth file. code=%d\n",argv[0],ii+1,err_code);
      exit(999);
    }

/* reset axis */
    StarCopyCubeHead(&(cube_out.head),&(cube_in.head));
    cube_out.pixel=StarAllocCubePixel(&(cube_out.head));

    ShiftAxisOrigin(&(cube_out.head.axis.x),reset_axis[0],new_orig[0]);
    ShiftAxisOrigin(&(cube_out.head.axis.y),reset_axis[1],new_orig[1]);
    ShiftAxisOrigin(&(cube_out.head.axis.z),reset_axis[2],new_orig[2]);

    err_code=StarCopyCubePixel(&cube_out,&cube_in);

    shiftaxis_add_history(&(cube_out.head),reset_axis,new_orig);

    StarCubeOut(&cube_out,cdb_flag,cdb_arg);

    StarFreeCubePixel(&cube_in);
    StarFreeCubePixel(&cube_out);
  }

/* end tag of pipe */
  StarPipeEndIn(id_str);
  StarPipeEndOut();

  exit(0);
}
