/*
Spurious Flagging : with interactive and background modes

1998/1/5 T.Handa : ver.3.00 multi_pipe and new library
1998/3/19 T.Handa : ver.3.01 debug for Solaris2 fcc
1998/4/24 T.Handa : ver.3.20 (revise) .scn,.sdb
1998/5/19 T.Handa : ver.3.22 (revise) range show
1998/5/21 T.Handa : ver.3.25 (revise) multi-unit for ranges
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>	/* to change the cursor font */

#include <Xm/DrawingA.h>
#include <Xm/MainW.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#include <Xm/PushBG.h>

#include <StData.h>
#include <StStd.h>

#include <Pxxlib.h>
#include <Pxplib.h>
#include <Ppslib.h>
#include <PxpGlib.h>
#include <StPxpShow.h>

#define	DEFAULT_PRMFILE	"spurious.prm"
#define N_KEY 25
#define	MAX_RANGES	256
#define MAX_UNIT 2

static PROFILE GLprofile;
static int GLsdb_flag;
static char GLsdb_arg[MAX_IDX_RECORD_LENGTH];
static PROFILE backup_profile;
static PROFILE spurious_profile;
static int GLn_file=0;
static int GLx_unit;
static RANGE GLx_range;
static RANGE GLy_range;

static BOX_RANGE GLchk_ch_range[MAX_RANGES];
static BOX_RANGE GLchk_range[MAX_UNIT][MAX_RANGES]; /* [0]ch, [1]v */
static int GLn_ch_range,GLrange_unit;
static int GLn_range[MAX_UNIT]; /* [0]ch, [1]v */
static int GLkeep_flag;
static int GLautorange_x,GLautorange_y,GLautorange_fit;
static BOX_RANGE GLphys_box;
static int GLrange_clear=1;

#define NOT_PROCESSED 0
#define RANGE_ASSIGNED 1
#define PROCESSED 10
static int process_status=NOT_PROCESSED;
	/* 0=unprocessed, 1=after range assigned, 10=after flagging */


/*-----------process undo---------------------*/
void do_reset()
/*
recover the original profile
1997/12/17 T.Handa
*/
{
/* copy profile from its backup */
  StarCopyProfile(&GLprofile,&backup_profile);
  process_status=RANGE_ASSIGNED;
  return;
}

/*-----------spurious flagging procedure---------------------*/
void do_sp_flag()
/*
do flagging
1997/12/17 T.Handa
*/
{
  int ch;
  int ii,n_range,range_unit;
  RANGE_long ch_range;
  int max_ch;
  int x_unit;
  RANGE x_range;
  double data,scale;

/* get global valuables */
  n_range=GLn_ch_range;
  range_unit=GLrange_unit;
  x_unit=GLx_unit;
  x_range.min=GLx_range.min;
  x_range.max=GLx_range.max;

  max_ch=GLprofile.be_status.data_ch;
  scale=GLprofile.rx_status.scale;

  for(ii=0;ii<n_range;ii++) {
/* copy fit range to spurious flag */
    ch_range.min=(long)(rint(GLchk_ch_range[ii].x.min));
    ch_range.max=(long)(rint(GLchk_ch_range[ii].x.max));
    for(ch=ch_range.min;ch<=ch_range.max;ch++) {
      data=GLprofile.data[ch-1]*scale;
      if(data<GLchk_ch_range[ii].y.min || data>GLchk_ch_range[ii].y.max) {
	StarWriteSpuriousFlag(&GLprofile,ch,FLAGGED);
      }
    }
  }

  process_status=PROCESSED;

  return;
}

void set_sp_profile()
{
  int ch,data_ch;
  int sp_ch;

  data_ch=spurious_profile.be_status.data_ch;
  spurious_profile.rx_status.scale=1.0;

  sp_ch=0;

  for(ch=1;ch<=data_ch;ch++) {
    if(StarReadSpuriousFlag(&GLprofile,ch)==FLAGGED) {
      spurious_profile.data[ch-1]=GLphys_box.y.max;
      StarWriteSpuriousFlag(&spurious_profile,ch,FLAGGED);
      sp_ch++;
    } else {
      spurious_profile.data[ch-1]=GLphys_box.y.min;
      StarWriteSpuriousFlag(&spurious_profile,ch,UNFLAGGED);
    }
  }

  fprintf(stderr,"spurious:%d ch are flagged for %s\n",sp_ch,GLprofile.file_name);

  return;
}

void plot_spurious()
/*
plot assigned spurious ranges
1997/12/25 T.Handa
*/
{
  RANGE full_plot_x;

  full_plot_x.min=GLphys_box.x.min;
  full_plot_x.max=GLphys_box.x.max;

  set_sp_profile();
  PxpSetColor("red");
  StarPlotProfileSpuriousRange(&spurious_profile,GLx_unit,GLphys_box,1);
  PxpSetColor("black");

  return;
}

void plot_last_spbox(profile,x_unit,check_point,phys_box)
PROFILE *profile; /* <in> */
int x_unit; /* <in> code of the unit of the x-axis */
PxpPoint check_point; /* <in> */
BOX_RANGE phys_box; /* <in> maxmum box size to plot */
{
/* plot line */
  PxpGMark(check_point.x,check_point.y,"+",0.5);

  return;
}

void plot_range_spbox(profile,x_unit,n_range,check_range,phys_box)
PROFILE *profile; /* <in> */
int x_unit; /* <in> code of the unit of the x-axis */
int n_range; /* <in> the numb of ranges */
BOX_RANGE *check_range; /* <in> */
BOX_RANGE phys_box; /* <in> maxmum box size to plot */
{
  RANGE plot_x,sp_x;
  BOX_RANGE fit_box;
  PxpPoint fit_box_min,fit_box_max;

  int ii;

  plot_x.min=phys_box.x.min;
  plot_x.max=phys_box.x.max;
  StarSortRange(&plot_x);

/* for each box */
  for(ii=0;ii<n_range;ii++) {
    if(x_unit==X_VELOCITY) {
      fit_box.x.min=StarCh2Velocity(profile,check_range[ii].x.min);
      fit_box.x.max=StarCh2Velocity(profile,check_range[ii].x.max);
    } else {
      fit_box.x.min=check_range[ii].x.min;
      fit_box.x.max=check_range[ii].x.max;
    }

    sp_x.min=fit_box.x.min;
    sp_x.max=fit_box.x.max;
    StarSortRange(&sp_x);

    if(!(sp_x.max<plot_x.min || sp_x.min>plot_x.max)) {
      if(sp_x.min<plot_x.min) sp_x.min=plot_x.min;
      if(sp_x.max>plot_x.max) sp_x.max=plot_x.max;
    }

    fit_box_min.x=sp_x.min;
    fit_box_max.x=sp_x.max;
    fit_box_min.y=phys_box.y.min;
    fit_box_max.y=check_range[ii].y.min;
    PxpGBox(fit_box_min,fit_box_max);
    fit_box_min.y=check_range[ii].y.max;
    fit_box_max.y=phys_box.y.max;
    PxpGBox(fit_box_min,fit_box_max);
  }

  return;
}

/*-----------reset assigned range---------------------*/
void clear_range()
/*
clear spurious flags
1997/12/17 T.Handa
*/
{
  int ii, i_unit;

/* clear assigned ranges */
  for(ii=0;ii<GLn_ch_range;ii++) {
    GLchk_ch_range[ii].x.min=0;
    GLchk_ch_range[ii].x.max=0;
    GLchk_ch_range[ii].y.min=0;
    GLchk_ch_range[ii].y.max=0;
  }
  GLn_ch_range=0;

  for(i_unit=0;i_unit<MAX_UNIT;i_unit++) {
    for(ii=0;ii<GLn_range[i_unit];ii++) {
      GLchk_range[i_unit][ii].x.min=0;
      GLchk_range[i_unit][ii].x.max=0;
      GLchk_range[i_unit][ii].y.min=0;
      GLchk_range[i_unit][ii].y.max=0;
    }
    GLn_range[i_unit]=0;
  }

/* clear flag in the profile */
  StarProfileClearSpuriousFlag(&GLprofile,UNFLAGGED);

/* set the global flag to notice range clear for other functions */
  GLrange_clear=1;

  process_status=NOT_PROCESSED;

  return;
}

/*-----------assign spurious range---------------------*/
void all_range_in_ch()
/*
convert all ranges asigned into the actual fitting ranges (in ch). 
All parameters are used by GLOBAL.

1998/5/21 T.Handa
*/
{
  int i_chk,ii,err[2];
  RANGE chk_ch;
  static char cmd[]="spurious";
  int data_ch;

  data_ch=GLprofile.be_status.data_ch;

/* ch to ch */
  for(ii=0,i_chk=0;ii<GLn_range[0];ii++) {
    GLchk_ch_range[i_chk].x.min=GLchk_range[0][ii].x.min;
    GLchk_ch_range[i_chk].x.max=GLchk_range[0][ii].x.max;
    GLchk_ch_range[i_chk].y.min=GLchk_range[0][ii].y.min;
    GLchk_ch_range[i_chk].y.max=GLchk_range[0][ii].y.max;
    i_chk++;
  }

/* v to ch */
  for(ii=0;ii<GLn_range[1];ii++) {
    chk_ch.min=StarVelocity2Ch(&GLprofile,&(err[0]),GLchk_range[1][ii].x.min);
    chk_ch.max=StarVelocity2Ch(&GLprofile,&(err[1]),GLchk_range[1][ii].x.max);

/* adjust over ranges */
/* actually all ranges */
    if((err[0]==OVER_FLOW && err[1]==UNDER_FLOW)
    || (err[0]==UNDER_FLOW && err[1]==OVER_FLOW)) {
      fprintf(stderr,"%s:cover all ch range. Adjust. : %s\n",cmd,GLprofile.file_name);
      chk_ch.min=1.0;
      chk_ch.max=(double)data_ch;
/* half over range */
    } else if(err[0]==NORMAL_END && err[1]==OVER_FLOW) {
      fprintf(stderr,"%s:v=%f is over range. Adjust: %s",cmd,GLchk_range[1][ii].x.max,GLprofile.file_name);
      chk_ch.max=(double)data_ch;
    } else if(err[0]==OVER_FLOW && err[1]==NORMAL_END) {
      fprintf(stderr,"%s:v=%f is over range. Adjust: %s",cmd,GLchk_range[1][ii].x.min,GLprofile.file_name);
      chk_ch.min=(double)data_ch;
/* half under range */
    } else if(err[0]==NORMAL_END && err[1]==UNDER_FLOW) {
      fprintf(stderr,"%s:v=%f is over range. Adjust: %s",cmd,GLchk_range[1][ii].x.max,GLprofile.file_name);
      chk_ch.max=1.0;
    } else if(err[0]==UNDER_FLOW && err[1]==NORMAL_END) {
      fprintf(stderr,"%s:v=%f is over range. Adjust: %s",cmd,GLchk_range[1][ii].x.min,GLprofile.file_name);
      chk_ch.min=1.0;
    } else if(!(err[0]==NORMAL_END && err[1]==NORMAL_END)) {
      fprintf(stderr,"%s:v=%f,%f is over range. Not use: %s",cmd,GLchk_range[1][ii].x.min,GLchk_range[1][ii].x.max,GLprofile.file_name);
      goto loop_btm;
    }

    GLchk_ch_range[i_chk].x.min=chk_ch.min;
    GLchk_ch_range[i_chk].x.max=chk_ch.max;
    GLchk_ch_range[i_chk].y.min=GLchk_range[1][ii].y.min;
    GLchk_ch_range[i_chk].y.max=GLchk_range[1][ii].y.max;

    i_chk++;
loop_btm:;
  }

  GLn_ch_range=i_chk;

  for(ii=0;ii<GLn_ch_range;ii++) {
    StarSortRange(&(GLchk_ch_range[ii].x));
    if(GLchk_ch_range[ii].x.min<1.0 && GLchk_ch_range[ii].x.max>1.0)
      GLchk_ch_range[ii].x.min=1.0;
    if(GLchk_ch_range[ii].x.max>(double)data_ch && GLchk_ch_range[ii].x.min<(double)data_ch)
      GLchk_ch_range[ii].x.max=(double)data_ch;
    StarSortRange(&(GLchk_ch_range[ii].y));
  }

  return;
}

void add_chk_range(x_unit,new_range)
/*
Add fit range arrays

1998/5/21 T.Handa
*/
int x_unit;
BOX_RANGE new_range;
{
  static char cmd[]="spurious";
  int i_unit;

  switch(x_unit) {
  case X_CHANNEL:
    i_unit=0;
    break;
  case X_VELOCITY: default:
    i_unit=1;
    break;
  }

  if(GLn_range[i_unit]>=MAX_RANGES || GLn_range[0]+GLn_range[1]>=MAX_RANGES) {
    fprintf(stderr,"%s: too many ranges.\n",cmd);
    return;
  }
  GLchk_range[i_unit][GLn_range[i_unit]].x.min=new_range.x.min;
  GLchk_range[i_unit][GLn_range[i_unit]].x.max=new_range.x.max;
  GLchk_range[i_unit][GLn_range[i_unit]].y.min=new_range.y.min;
  GLchk_range[i_unit][GLn_range[i_unit]].y.max=new_range.y.max;
  GLn_range[i_unit]++;

  all_range_in_ch();

  return;
}

int click_position_phys(ii,jj,result_phys_pos,x_unit,profile)
int ii,jj; /* <in> pixel position */
PxpPoint *result_phys_pos; /* <out> */
int x_unit; /* <in> */
PROFILE *profile; /* <in> */
{
  double xx,yy;
  PxpPoint plot_pos,phys_pos;
  int data_ch,ch;
  int err_code;

  PxxGetPlotPosition(ii,jj,&xx,&yy);
  plot_pos.x=xx;
  plot_pos.y=yy;
  phys_pos=PxpGPhysicsValue(plot_pos);

  data_ch=(*profile).be_status.data_ch;

  if(x_unit==X_CHANNEL) {

  } else if(x_unit==X_VELOCITY) {
    ch=(int)rint(StarVelocity2Ch(profile,&err_code,phys_pos.x));
    switch(err_code) {
    case OVER_FLOW:
      ch=(int)((*profile).be_status.data_ch);
      break;
    case UNDER_FLOW:
      ch=1.0;
      break;
    case NORMAL_END:
      break;
    default:
      return(err_code);
    }
    phys_pos.x=StarCh2Velocity(profile,(double)ch);
  }

  (*result_phys_pos).x=phys_pos.x;
  (*result_phys_pos).y=phys_pos.y;
  return(NORMAL_END);
}

void ClickActions(parentWDG,gc,cdata)
Widget	parentWDG;
GC	gc;
XmDrawingAreaCallbackStruct	*cdata;
/*
define actions for mouse click
1997/12/17 T.Handa ver.1.00
1998/5/21 T.Handa ver.1.10
*/
{
  int ii,jj;
  double xx,yy;
  static PxpPoint phys_pos;
  static BOX_RANGE new_range;
  static int onoff=1;

  Cursor	draw_cursor;

/* change cursor font */
  draw_cursor=XCreateFontCursor(PxxGetDisplay(),XC_crosshair);
  XDefineCursor(PxxGetDisplay(),PxxGetWindow(),draw_cursor);
  PxxFlush();

/* just after clearing the assigned ranges */
  if(GLrange_clear>0) {
    onoff=1;
    GLrange_clear= -1;
  }

/* action at mouse click */
  switch(cdata->reason) {
  case XmCR_INPUT:
    if(cdata->event->type==ButtonPress) {
      ii=cdata->event->xbutton.x;
      jj=cdata->event->xbutton.y;
      click_position_phys(ii,jj,&phys_pos,GLx_unit,&GLprofile);
      xx=phys_pos.x;
      yy=phys_pos.y;

      if(onoff>0) {
	new_range.x.min=xx;
	new_range.y.min=yy;
      } else {
	new_range.x.max=xx;
	new_range.y.max=yy;
	add_chk_range(GLx_unit,new_range);
process_status=RANGE_ASSIGNED;
      }
      onoff= -onoff;
    }

/* action at mouse click and expose,resize */
  case XmCR_EXPOSE:

    PxpSetColor("red");
    StarPlotProfileSpuriousRange(&GLprofile,GLx_unit,GLphys_box,2);

    plot_range_spbox(&GLprofile,GLx_unit,GLn_ch_range,GLchk_ch_range,GLphys_box);
    /* last */
    if(onoff<0) 
      plot_last_spbox(&GLprofile,GLx_unit,phys_pos,GLphys_box);

    PxpSetColor("black");
    break;
  }

  return;
}

/*-----------plot profile---------------------*/

void local_StarPlotProfileStd(profile,x_unit,x_range,y_range,plot_box,base,spur)
/*
plot profile in the standard format (localized for spurious)
1997/12/11 T.Handa
*/
PROFILE *profile;	/* <in> profile */
int x_unit;	/* <in>  */
RANGE x_range,y_range;	/* <in> */
BOX_RANGE plot_box;	/* <in> */
int base,spur; /* <in> base/spur plot switch (0=off,1=on) */
{
/*  BOX_RANGE phys_box;*/
  RANGE_long ch_range;
  PxpPoint name_pos;

/* profile name position */
  name_pos.x=plot_box.x.min+0.2;
  name_pos.y=plot_box.y.max-0.5;

/* plot frame */
  StarPlotProfileStdFrame(profile,x_unit,x_range,y_range,plot_box,&ch_range,&GLphys_box);

  if(base>0) {
/* plot baseline range */
    PxpSetColor("cyan");
    StarPlotProfileBaselineRange(profile,x_unit,GLphys_box,2);
  }

  if(spur>0) {
/* plot spurious range */
    PxpSetColor("pink");
    StarPlotProfileSpuriousRange(profile,x_unit,GLphys_box,1);
  }

  if(GLn_ch_range>0) {
/* plot ranges assigned in this procedure */
    PxpSetColor("red");
    plot_range_spbox(&GLprofile,GLx_unit,GLn_ch_range,GLchk_ch_range,GLphys_box);
  }
	
  PxpSetColor("black");
/* profile name */
  PxpDrawString(name_pos.x,name_pos.y,(*profile).file_name);

/* profile itself */
  if(process_status<PROCESSED)
    PxpSetColor("blue");	/* not yet processed */
  else
    PxpSetColor("green");	/* already processed */
  StarPlotProfileInCh(profile,ch_range,x_unit);
  PxpSetColor("black");

  return;
}

/*--------from Pxxlib--------------------*/
static void	(*do_plot)(), (*do_exit)(), (*do_help)();

/* define actions for buttons on the menubar */
void star_PxxPushButtonActions(menubarWDG,client_data,call_data)
Widget	menubarWDG;
caddr_t	client_data,call_data;
/*
define actions for buttons on the menubar
1997/10/12 T.Handa ver.1.00
1997/12/17 T.Handa ver.1.00 modify for spurious
*/
{
/*  RANGE full_plot_x;*/

  if(strcmp(client_data,"Exit")==0) {
    XClearWindow(PxxGetDisplay(),PxxGetWindow());
    if(do_exit!=NULL) (*do_exit)();
    if(do_plot!=NULL) (*do_plot)();
  }
  else if(strcmp(client_data,"Quit")==0) {
    exit(0);
  } else if(strcmp(client_data,"Flag")==0)	{ /* spurious flagging */
    do_sp_flag();
    plot_spurious();
  } else if(strcmp(client_data,"Reshow")==0){ /* show applied profile */
    XClearWindow(PxxGetDisplay(),PxxGetWindow());
    (*do_plot)();
  } else if(strcmp(client_data,"Clear")==0) { /* clear fitting range */
    XClearWindow(PxxGetDisplay(),PxxGetWindow());
    clear_range();
    (*do_plot)();
  } else if(strcmp(client_data,"Retry")==0) { /* reset profile data */
    XClearWindow(PxxGetDisplay(),PxxGetWindow());
    do_reset();
    (*do_plot)();
  } else if(strcmp(client_data,"Print")==0) {
    PxxExecPrintOut();
  } else if(strcmp(client_data,"Help")==0) {
    if(do_help!=NULL) (*do_help)();
  }

  return;
}

#define MAX_ARG 100
#define	CHAR_SIZE_DEFAULT_RESOURCE	128

/* show on X11 */
void star_PxxDisplayAll(argc,argv, app_class,width,height,plt_fnct,exit_fnct,help_fnct)
/*
create a window on X11 in the standard way
1997/10/12 T.Handa ver.1.00
1997/12/17 T.Handa ver.1.00 for spurious
1998/ 5/ 6 T.Handa ver.1.02 (debug)
*/
int argc;
char *argv[];
char app_class[];	/* application class used in .Xresource */
double width,height;
void (*plt_fnct)(),(*exit_fnct)(),(*help_fnct)();
{
  Widget  toplevelWDG, main_windowWDG, drawing_areaWDG, menubarWDG, mbBtn[9];
  Arg     args[10];
  int     ii;
  GC      graphic_context;

  int	mainwin_x,mainwin_y;

  int i_btn_id;

  XtAppContext    app_context;
  Arg fixed_res_args[MAX_ARG];     /* resource arguments */
  Cardinal        fixed_res_n_args;
  static char	window_w[CHAR_SIZE_DEFAULT_RESOURCE],window_h[CHAR_SIZE_DEFAULT_RESOURCE];
  static String   fb_res[] = {
    "*mainWindow.width:        500 ",
    "*mainWindow.height:       300 ",
    "*drawArea.background:	white",
    "*menubar.topAttachment:	ATTACH_FORM",
    "*menubar.rightAttachment:	ATTACH_FORM",
    "*menubar.leftAttachment:	ATTACH_FORM",
    NULL};
/* add for modification */
  static char psfile_name[]="";
  static char prt_name[]="";


/* Change initial window size */
  mainwin_x=(int)(width*CM2INCH*DVI_SCALE+0.5);
  mainwin_y=(int)(height*CM2INCH*DVI_SCALE+0.5);
  if(mainwin_x>0 && mainwin_x<9999999) {
    sprintf(window_w,"*mainWindow.width: %8d",mainwin_x);
    fb_res[0]=window_w;
  }
  if(mainwin_y>0 && mainwin_y<9999999) {
    sprintf(window_h,"*mainWindow.height:%8d",mainwin_y);
    fb_res[1]=window_h;
  }

/*  PxxView_width=width; */
/*  PxxView_height=height; */
/*  PxxView_printer[0]=(char)NULL; */	/* set default printer */
/*  PxxView_psfile[0]=(char)NULL;*/	/* set temporaly file */
/* add for modification */
  PxxPutWindowSizeInPixel(width,height);
  PxxSetPrinterName(prt_name);
  PxxSetPSFName(psfile_name);


/* printer assigned by option -P */
  for(ii=1;ii<argc;ii++) {
    if(strcmp(argv[ii],"-P")==0) PxxSetPrinterName(argv[ii+1]);
    else if(argv[ii][0]=='-' && argv[ii][1]=='P')
      PxxSetPrinterName(&(argv[ii][2]));
  }

/* PS filename assigned by option -F */
  for(ii=1;ii<argc;ii++) {
    if(strcmp(argv[ii],"-F")==0) PxxSetPSFName(argv[ii+1]);
    else if(argv[ii][0]=='-' && argv[ii][1]=='F')
      PxxSetPSFName(&(argv[ii][2]));
  }

/* give an actual function names for plot, exit and help */
  PxxSetPlotFunction(plt_fnct);
  PxxSetExitFunction(exit_fnct);
  PxxSetHelpFunction(help_fnct);
/* add for modification */
  do_plot=plt_fnct;
  do_exit=exit_fnct;
  do_help=help_fnct;

/* create the window */
  fixed_res_n_args=0;
  toplevelWDG=XtAppInitialize(&app_context,app_class, NULL, 0, &argc, argv,fb_res,fixed_res_args,fixed_res_n_args);

  main_windowWDG=XmCreateMainWindow(toplevelWDG,"mainWindow",NULL,(Cardinal)NULL);
  XtManageChild(main_windowWDG);

/* create the menu bar */
  menubarWDG=XmCreateMenuBar(main_windowWDG,"menubar",NULL,(Cardinal)NULL);
  XtManageChild(menubarWDG);
  i_btn_id=0;

  mbBtn[i_btn_id]=XmCreateCascadeButton(menubarWDG,"help", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, star_PxxPushButtonActions,"Help");
  i_btn_id++;

  mbBtn[i_btn_id]=XmCreateCascadeButton(menubarWDG,"next", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, star_PxxPushButtonActions,"Exit");
  i_btn_id++;
  mbBtn[i_btn_id]=XmCreateCascadeButton(menubarWDG,"flag", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, star_PxxPushButtonActions,"Flag");
  i_btn_id++;
  mbBtn[i_btn_id]=XmCreateCascadeButton(menubarWDG,"reshow", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, star_PxxPushButtonActions,"Reshow");
  i_btn_id++;
  mbBtn[i_btn_id]=XmCreateCascadeButton(menubarWDG,"clear", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, star_PxxPushButtonActions,"Clear");
  i_btn_id++;
  mbBtn[i_btn_id]=XmCreateCascadeButton(menubarWDG,"retry", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, star_PxxPushButtonActions,"Retry");
  i_btn_id++;
  mbBtn[i_btn_id]=XmCreateCascadeButton(menubarWDG,"print", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, star_PxxPushButtonActions,"Print");
  i_btn_id++;
  mbBtn[i_btn_id]=XmCreateCascadeButton(menubarWDG,"quit", args,0);
  XtAddCallback(mbBtn[i_btn_id],XmNactivateCallback, star_PxxPushButtonActions,"Quit");
  i_btn_id++;
  XtManageChildren(mbBtn,i_btn_id);

  XtSetArg(args[0],XmNmenuHelpWidget,(XtArgVal)mbBtn[0]);
  XtSetValues(menubarWDG,args,1);

/* create the drawing area */
  drawing_areaWDG=XmCreateDrawingArea(main_windowWDG,"drawArea",args,0);
  XtManageChild(drawing_areaWDG);

/* locate widgets */
  XmMainWindowSetAreas(main_windowWDG,menubarWDG,NULL,NULL,NULL,drawing_areaWDG);

/* realize the window */
  XtRealizeWidget(toplevelWDG);

  graphic_context=XCreateGC(XtDisplay(drawing_areaWDG),XtWindow(drawing_areaWDG),0,0);

  XtAddCallback(drawing_areaWDG,XmNexposeCallback,(XtCallbackProc)PxxExecDraw,graphic_context);

  XtAddCallback(drawing_areaWDG,XmNresizeCallback,(XtCallbackProc)PxxExecDraw,graphic_context);

/* action for mouse click in the drawing area */
  graphic_context=XCreateGC(XtDisplay(drawing_areaWDG),XtWindow(drawing_areaWDG),0,0);
  XtAddCallback(drawing_areaWDG,XmNinputCallback,ClickActions,graphic_context);
  XtAddCallback(drawing_areaWDG,XmNexposeCallback,ClickActions,graphic_context);

/* main loop */
  XtAppMainLoop(app_context);
}

/*--------from Pxplib--------------------*/
/* do X11 or PostScript output */
void star_PxpDisplayAll(argc,argv, output, filename,app_class,width,height,plt_fnct,exit_fnct,help_fnct)
/*
create an X11 window or a PostScript file
1997/10/14 T.Handa ver.1.00
1997/12/02 T.Handa ver.1.00 modify for display
*/
int argc;
char *argv[];
int output;
char *filename;	/* if "-" or NULL-string, stdout; if filename==NULL, temporaly work file */
char app_class[];	/* application class used in .Xresource */
double width,height;
void (*plt_fnct)(),(*exit_fnct)(),(*help_fnct)();
{
  void (*plot_function)();

  plot_function=plt_fnct;
/*  PxpView_output=output;*/
  PxpPutOutputMode(output);

  switch(output) {
  case X11WIN:
    star_PxxDisplayAll(argc,argv, app_class,width,height,plt_fnct,exit_fnct,help_fnct);
    break;
  case POSTSCRIPT:
    PpsInitialize(filename,width,height);
    plot_function();	/* actual plot defined by user */
    PpsPrintPage();
    PpsDoPrint();
  }

  return;
}

/*---------------------------------*/

int options(argc,argv, w_height,w_width,x_unit,x_range,T_range,range_unit,n_range,check_range,keep_flag,gui_flag)
/*
get parameters from options
*/
int argc;
char *argv[];
double	*w_height, *w_width;	/* window size (cm) */
int	*x_unit;
RANGE	*x_range,	/* channel range */
	*T_range;	/* Ta range */
int	*range_unit;	/* range unit */
int	*n_range;	/* number of ranges */
BOX_RANGE check_range[MAX_UNIT][MAX_RANGES]; /* <out> */
int	*keep_flag;
int	*gui_flag;
{
  int ii,i_unit;
  int i_limit,n_limit;
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *token;
  int opt_pos;
  char buffer[MAX_PRM_RECORD_LENGTH];

  int flag;
  double window_x,window_y;
  RANGE chk_range[MAX_RANGES];

  char *got_value;

/* default values */
  *w_height=15.0;	*w_width=20.0;
  *x_unit=X_CHANNEL;
  (*x_range).min=0.0;	(*x_range).max=0.0;
  (*T_range).min=0.0;	(*T_range).max=0.0;
  n_range[0]=0; n_range[1]=0;
  *keep_flag=1; /* keep the previous spurious flag? yes */
  *gui_flag= -1;	/* GUI? no */

  window_x=0;
  window_y=0;

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"size");	key_type[n_key-1]=1; /* No.0 */
  n_key=AddNewWord(keyword,"ch");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"v");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"lsr");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"x_unit");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"T");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"GUI");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"gui");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"nogui");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"CUI");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"cui");	key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"mute");	key_type[n_key-1]=0; /* No.11*/
  n_key=AddNewWord(keyword,"ch_check");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"ch_check_range");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"v_check");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"v_check_range");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"sp_limit");	key_type[n_key-1]=1; /* No.16*/
  n_key=AddNewWord(keyword,"clip_limit");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"clip_level");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"clear");	key_type[n_key-1]=0; /* No.19*/
  n_key=AddNewWord(keyword,"keep");	key_type[n_key-1]=0;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* window size */
  scan_flag[0]=0;
  n_scan=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(buffer,got_value);
    token=strtok(buffer,"xX ,\t\n");
    if(token!=NULL) {
      window_x=atof(token);
      token=strtok(NULL," ,\t\n");
      if(token!=NULL) window_y=atof(token);
    }
    *w_height=window_y;
    *w_width=window_x;
  }

/* ch,v,lsr,x_unit */
  scan_flag[0]=1;	scan_flag[1]=2; scan_flag[2]=3; scan_flag[3]=4;
  n_scan=4;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    if(strcmp(keyword[opt_pos],"v")==0
       || strcmp(keyword[opt_pos],"lsr")==0
       || strcmp(value[opt_pos],"velocity")==0
       || strcmp(value[opt_pos],"lsr")==0
       || strcmp(value[opt_pos],"LSR")==0)
      *x_unit=X_VELOCITY;
    else if(strcmp(keyword[opt_pos],"ch")==0
	    || strcmp(value[opt_pos],"channel")==0
	    || strcmp(value[opt_pos],"ch")==0)
      *x_unit=X_CHANNEL;
    StarOptionRange(value[opt_pos],x_range);
  } 

/* y_range */
  scan_flag[0]=5;
  n_scan=1;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    strcpy(buffer,value[opt_pos]);
    StarOptionRange(buffer,T_range);
  } 

/* GUI */
  scan_flag[0]=6; scan_flag[1]=7;
  scan_flag[2]= -8; scan_flag[4]= -9;
  scan_flag[5]= -10; scan_flag[6]= -11; 
  n_scan=7;
  flag=StarOptionFlag(n_scan,scan_flag,key_flag);
  if(flag!=0) *gui_flag=flag;

/* checking ranges in channel */
  scan_flag[0]=12;
  scan_flag[1]=13;
  n_scan=2;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    StarOptionRanges(value[opt_pos],&(n_range[0]),chk_range);
    for(ii=0;ii< n_range[0] ;ii++) {
      check_range[0][ii].x.min=chk_range[ii].min;
      check_range[0][ii].x.max=chk_range[ii].max;
    }
  }

/* checking ranges in velociy */
  scan_flag[0]=14;
  scan_flag[1]=15;
  n_scan=2;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    StarOptionRanges(value[opt_pos],&(n_range[1]),chk_range);
    for(ii=0;ii< n_range[0] ;ii++) {
      check_range[1][ii].x.min=chk_range[ii].min;
      check_range[1][ii].x.max=chk_range[ii].max;
    }
  }

/* clip limit */
  scan_flag[0]=16;
  scan_flag[1]=17;
  scan_flag[2]=18;
  n_scan=3;
  opt_pos=StarOptionKeyPosition(n_scan,scan_flag,key_flag);
  if(opt_pos>=0) {
    StarOptionRanges(value[opt_pos],&n_limit,chk_range);
    for(i_unit=0;i_unit<MAX_UNIT;i_unit++) {
      for(ii=0;ii< n_range[i_unit] ;ii++) {
	if(ii>=n_limit) i_limit=n_limit-1;
	else i_limit=ii;
	check_range[i_unit][ii].y.min=chk_range[i_limit].min;
	check_range[i_unit][ii].y.max=chk_range[i_limit].max;
      }
    }
  }

/* flag keep? */
  scan_flag[0]= -19; scan_flag[1]=20;
  n_scan=2;
  flag=StarOptionFlag(n_scan,scan_flag,key_flag);
  if(flag!=0) *keep_flag=flag;

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

/* set return values */

  return(0);
}


/* Main Body of process */

void write_profile(profile,sdb_flag,sdb_arg)
PROFILE *profile;
int sdb_flag;
char *sdb_arg;
{
  int	ii;
  HISTORY history;

  strcpy(history.step,"SPFLG");
  for(ii=0;ii<GLn_ch_range;ii++) {
    sprintf(history.prm[ii*2],"x=%f,%f",GLchk_ch_range[ii].x.min,GLchk_ch_range[ii].x.max);
    sprintf(history.prm[ii*2+1],"y=%f,%f",GLchk_ch_range[ii].y.min,GLchk_ch_range[ii].y.max);
  }
  history.size = ii*2;

  StarAddHistory(profile,&history);
  StarProfileOut(profile,sdb_flag,sdb_arg);

  return;
}

void read_profile()
{
  static int i_file=0;
  int n_file;
  int err_code;

/* get global valuables */
  n_file=GLn_file;


/* write the previous profile before reading */
  if(i_file>0) {
    write_profile(&GLprofile,GLsdb_flag,GLsdb_arg);
  }

/* if no more, then end */
  if(i_file>=n_file) {
    StarPipeEndOut();
    exit(0);
  }

/* get a new profile */
  while(1) {
    err_code=StarProfileIn(&GLprofile,&GLsdb_flag,GLsdb_arg);
    if(err_code!=0) {
      fprintf(stderr,"fail to ProfileIn\n");
      exit(1);
    }

/* update the present count */
    i_file++;

    if(GLsdb_flag==GOOD_SCAN) break;
    StarProfileOut(&GLprofile,GLsdb_flag,GLsdb_arg);
  }

/* preset baseline flag clear */
  if(GLkeep_flag!=1)
    StarProfileClearSpuriousFlag(&GLprofile,UNFLAGGED);

/* copy a profile to a backup and a work profile */
  StarCopyProfile(&backup_profile,&GLprofile);
  StarCopyProfile(&spurious_profile,&GLprofile);
  process_status=NOT_PROCESSED;

/* convert fit ranges in fit_ch_range */
  all_range_in_ch();

  return;
}

void actual_plot()
{
  static int read_flag=1;
  RANGE x_range,y_range;
  int x_unit;

  BOX_RANGE plot_box;

  static PxpPoint win_size;

  int output_mode;
  int window_x,window_y,window_h,window_w;

/* copy from global valuables */
  x_range.min=GLx_range.min;
  x_range.max=GLx_range.max;
  x_unit=GLx_unit;
  y_range.min=GLy_range.min;
  y_range.max=GLy_range.max;

/* read profile if the first */
  if(read_flag>0) {
    read_profile();
    read_flag= -1;
  }

/* plotsize auto scaling */
  PxpGetOutputMode(&output_mode);
  if(output_mode==X11WIN) {
    PxxPresentWindowSize(&window_x,&window_y,&window_w,&window_h);
    win_size.x=PxxGetPlotSpanX(window_w);
    win_size.y=PxxGetPlotSpanY(window_h);
  }

  plot_box.x.min=win_size.x*0.1;
  plot_box.y.min=win_size.y*0.1;
  plot_box.x.max=plot_box.x.min+win_size.x*0.8;
  plot_box.y.max=plot_box.y.min+win_size.y*0.8;

/* profile in standard form */
  local_StarPlotProfileStd(&GLprofile,x_unit,x_range,y_range,plot_box);

/* plot flagged curve */
  switch(process_status){ 
  case PROCESSED:
    plot_spurious();
  case RANGE_ASSIGNED:
    PxpSetColor("red");
    StarPlotProfileSpuriousRange(&GLprofile,GLx_unit,GLphys_box,2);
    PxpSetColor("black");
  }

  return;
}

void help_function()
{
  fprintf(stderr,"help should be shown.\n");
  return;
}

int main(argc,argv)
int argc;
char *argv[];
{
  int err_code;
  int i_file,n_file;

  double	w_height, w_width;	/* window size (cm) */
  int	x_unit;
  RANGE	x_range,y_range;	/* display ranges, horizontal and vertical */
  int range_unit;
  int n_range[2];
  int keep_flag,gui_flag;

  char id_str[BUFFER_SIZE];

/* check option */
  err_code=options(argc,argv, &w_height,&w_width,&x_unit,&x_range,&y_range,&range_unit,n_range,GLchk_range,&keep_flag,&gui_flag);
  if(err_code!=0 && err_code!=111) {
    exit(1);
  }

/* set global parameters */
  GLx_unit=x_unit;
  GLx_range.min=x_range.min;
  GLx_range.max=x_range.max;
  GLy_range.min=y_range.min;
  GLy_range.max=y_range.max;
  GLn_range[0]=n_range[0];
  GLn_range[1]=n_range[1];

  if(x_range.min==x_range.max)
    GLautorange_x= 1;
  else
    GLautorange_x= -1;
  if(y_range.min==y_range.max)
    GLautorange_y= 1;
  else
    GLautorange_y= -1;

  if(n_range[0]==0)
    GLautorange_fit= 1;
  else
    GLautorange_fit= -1;

  GLrange_unit=range_unit;
  GLkeep_flag=keep_flag;

/* multi-data header in */
  StarPipeHeadIn(id_str,&n_file);
  GLn_file=n_file;

  StarPipeHeadOut(n_file);

  if(gui_flag>0) {
/* GUI */
    star_PxpDisplayAll(argc,argv,X11WIN,NULL,"spurious",w_width,w_height,actual_plot,read_profile,help_function);
  } else {
/* CUI processing */
    for(i_file=0;i_file<n_file;i_file++) {
      err_code=StarProfileIn(&GLprofile,&GLsdb_flag,GLsdb_arg);
      if(GLsdb_flag==GOOD_SCAN) {
	if(keep_flag!=1)
	  StarProfileClearSpuriousFlag(&GLprofile,UNFLAGGED);
	all_range_in_ch();
	do_sp_flag();
      }
      write_profile(&GLprofile,GLsdb_flag,GLsdb_arg);
    }
    StarPipeEndOut();
    exit(0);
  }
  exit(0);
}

