/*
make a text file to show a corration plot in two cube datasets
T-T plot

2002/5/29 T.Handa : ver.3.00
2003/3/5 T.Handa : ver.3.01 (debug) output for txt_filename 
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <StData.h>
#include <StStd.h>

#define	DEFAULT_PRMFILE	"ttplot.prm"
#define N_KEY 30

#define DEBUG
#undef DEBUG

int options(argc,argv,base_cubname,headline_flag,txt_filename)
/*
get parameters from options
*/
int argc;
char *argv[];
char *base_cubname;
int *headline_flag;
char *txt_filename;
{
  char **keyword;
  char **value;
  int n_key;
  int key_flag[N_KEY];
  int key_type[N_KEY];
  int scan_flag[N_KEY], n_scan;

  char *got_value;

/* default values */
  strcpy(base_cubname,"");
  *headline_flag=FLAG_YES;
  strcpy(txt_filename,"-");

/* set keywords with scan_flag */
  keyword=AllocWordsStrage(N_KEY);
  value=AllocWordsStrage(N_KEY);

  n_key=AddNewWord(keyword,"base_cube");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"base_cubname");	key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"with_headline"); key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"without_headline"); key_type[n_key-1]=0;
  n_key=AddNewWord(keyword,"headline"); key_type[n_key-1]=1;
  n_key=AddNewWord(keyword,"outfile"); key_type[n_key-1]=1;

  StarGetOption(argc,argv,DEFAULT_PRMFILE,n_key,keyword,key_flag,value);

/* base cubname */
  n_scan=2;
  scan_flag[0]=0;
  scan_flag[1]=1;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(base_cubname,got_value);
  }

/* headline */
  n_scan=3;
  scan_flag[0]=2;
  scan_flag[1]=3;
  scan_flag[2]=4;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    if(strcmp(got_value,"no")==0 || strncmp(got_value,"without_",8)==0) { 
      *headline_flag=FLAG_NO;
    } else {
      *headline_flag=FLAG_YES;
    }
  }

/* outfilename */
  n_scan=1;
  scan_flag[0]=5;
  got_value=StarOptionValue(n_scan,scan_flag,keyword,key_type,key_flag,value);
  if(got_value[0]!=(char)NULL) {
    strcpy(txt_filename,got_value);
  }

/* free memory areas */
  StarFreeAllPrms(N_KEY,key_flag,value);
  free(keyword);

  return 0;
}


void main_filename(mainname,filename)
char *mainname; /* <out> */
char *filename; /* <in> */
{
  int ii,l_str,i_top;

  strcpy(mainname,StarFileMainName(filename));

  l_str=strlen(mainname);
  i_top=l_str-4;
  if(i_top<0) i_top=0;
  for(ii=l_str-1;ii>=i_top;ii--) {
    if(mainname[ii]=='.') {
      mainname[ii]=(char)NULL;
      break;
    }
  }
  return;
}

void headline4TTplot(fp,cube_x,cube_y,sep)
FILE *fp;
CUBE_HEAD *cube_x,*cube_y;
char sep;
{
  fprintf(fp,"%s, %s%c%s, %s\n",(*cube_x).file_name,(*cube_x).object.name,sep,(*cube_y).file_name,(*cube_y).object.name);
  fprintf(fp,"%s(%s)%c%s(%s)\n",(*cube_x).data_value.type,(*cube_x).data_value.unit,sep,(*cube_y).data_value.type,(*cube_y).data_value.unit);

  return;
}

void Pixeldata4TTPlot(fp,cube_x,cube_y,sep)
FILE *fp;
CUBE *cube_x,*cube_y;
char sep;
{
  int mx,my,mz;
  int ix,iy,iz;
  int valid_data_x,valid_data_y;
  double ta_x,ta_y;

  mx=(*cube_x).head.axis.x.size;
  my=(*cube_x).head.axis.y.size;
  mz=(*cube_x).head.axis.z.size;

  for(ix=0;ix<mx;ix++) {
    for(iy=0;iy<my;iy++) {
      for(iz=0;iz<mz;iz++) {
	ta_x=StarGetCubePixelData(cube_x,ix,iy,iz,&valid_data_x);
	ta_y=StarGetCubePixelData(cube_y,ix,iy,iz,&valid_data_y);
	if(valid_data_x==FLAG_YES && valid_data_y==FLAG_YES)
	  fprintf(fp,"%f%c%f\n",ta_x,sep,ta_y);
      }
    }
  }

  return;
}

int CheckCubeHead(cube1,cube2)
CUBE_HEAD *cube1; /* <in> */
CUBE_HEAD *cube2; /* <in> */
{
  int ret_code;

  ret_code=0;

/* axis size */
  if((*cube1).axis.x.size!=(*cube2).axis.x.size) {
    fprintf(stderr,"unmatch x-size\n");
  }
  if((*cube1).axis.y.size!=(*cube2).axis.y.size) {
    fprintf(stderr,"unmatch y-size\n");
  }
  if((*cube1).axis.z.size!=(*cube2).axis.z.size) {
    fprintf(stderr,"unmatch z-size\n");
  }

/* axis type */
  if(strcmp((*cube1).axis.x.type,(*cube2).axis.x.type)!=0) {
    fprintf(stderr,"unmatch x-type\n");
  }
  if(strcmp((*cube1).axis.y.type,(*cube2).axis.y.type)!=0) {
    fprintf(stderr,"unmatch y-type\n");
  }
  if(strcmp((*cube1).axis.z.type,(*cube2).axis.z.type)!=0) {
    fprintf(stderr,"unmatch z-type\n");
  }

  return(ret_code);
}

int main(argc,argv)
int	argc;
char	*argv[];
{
  CUBE cube_base,cube_add;
  int cdb_flag;
  char cdb_arg[MAX_IDX_RECORD_LENGTH];
  int err_code;
  int n_file;
  char id_str[BUFFER_SIZE];
  char base_cubname[MAX_FILE_NAME_LENGTH],txt_filename[MAX_FILE_NAME_LENGTH];

  int headline_flag;

  int ii;

  FILE *fp_in,*fp_txt;

/* get optional parameters */
  err_code=options(argc,argv,base_cubname,&headline_flag,txt_filename);

  fp_in=fopen(base_cubname,"r");
  if(fp_in==NULL) {
    fprintf(stderr,"**ERR** file %s is not found.\n",base_cubname);
    exit(99);
  }
  err_code=StarCubeHeadLoad(fp_in,&(cube_base.head));
  if(err_code!=0) {
    fprintf(stderr,"**ERR** failed to load header data for %s\n",base_cubname);
    exit(99);
  }
  cube_base.pixel=StarAllocCubePixel(&(cube_base.head));
  if(cube_base.pixel==NULL) {
    fprintf(stderr,"**ERR** cannot alloc pixel data for %s\n",base_cubname);
    exit(99);
  }
  err_code=StarCubePixelLoad(fp_in,&cube_base);
  if(err_code!=0) {
    fprintf(stderr,"**ERR** failed to load pixel data for %s\n",base_cubname);
    exit(99);
  }
  fclose(fp_in);

  main_filename(cube_base.head.file_name,base_cubname);
  
  if(strcmp(txt_filename,"-")==0 || strcmp(txt_filename,"stdout")==0)
    fp_txt=stdout;
  else if(strcmp(txt_filename,"stderr")==0)
    fp_txt=stderr;
  else {
    fp_txt=fopen(txt_filename,"w");
    if(fp_txt==NULL) {
      fprintf(stderr,"%s: cannot open output txt as %s\n",argv[0],txt_filename);
      exit(99);
    }
  }

/* pipe header */
  StarPipeHeadIn(id_str,&n_file);

/* .cub data */
  for(ii=0;ii<n_file;ii++) {

    StarCubeIn(&cube_add,&cdb_flag,cdb_arg);

    CheckCubeHead(&(cube_add.head),&(cube_base.head));

    if(headline_flag==FLAG_YES)
      headline4TTplot(fp_txt,&(cube_base.head),&(cube_add.head),'\t');
    Pixeldata4TTPlot(fp_txt,&cube_base,&cube_add,'\t');

    StarFreeCubePixel(&cube_add);
    if(fp_txt!=stdout && fp_txt!=stderr) {
      fclose(fp_txt);
    }
  }

  StarFreeCubePixel(&cube_base);

/* end tag of pipe */
  StarPipeEndIn(id_str);

  exit(0);
}
