/*
leap.sec keeper
1999/7/28 T.Handa : ver.1.00
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <StData.h>
#include <StStd.h>

#define MAX_LINE 1024

/* last UTC for frequency offset system */
#define UTC_F_OFFSET 63072000
/* difference ET-UTC at 1970/1/1 0h UTC */
#define D_ET_UTC 42

typedef struct {
  time_t utc;
  time_t leap_sec;
} LEAP;

static int time_order(line1,line2) 
LEAP *line1,*line2;
{
  if((*line1).utc<(*line2).utc) 
    return(-1);
  else if((*line1).utc==(*line2).utc) 
    return(0);
  else 
    return(1);
}


int main(argc,argv)
int argc;
char *argv[];
{
  FILE *fp_in,*fp_out;
  char buffer[BUFFER_SIZE];
  char *token,*got_c;
  LEAP entry[MAX_LINE];
  time_t utc,leap_sec;
  int ii,n_line;
  int l_str;

  static int ctime_flag=1;

  fp_in=fopen(LEAP_SEC_SOURCE_FILE,"r");
  if(fp_in==NULL) {
    fprintf(stderr,"%s: cannot open src of leap_sec, %s",argv[0],LEAP_SEC_SOURCE_FILE);
    exit(99);
  }
  fp_out=fopen(LEAP_SEC_FILE,"w");
  if(fp_in==NULL) {
    fprintf(stderr,"%s: cannot open output for leap_sec, %s",argv[0],LEAP_SEC_FILE);
    exit(99);
  }

  fprintf(stderr,"src file=%s\ntarget  =%s\n",LEAP_SEC_SOURCE_FILE,LEAP_SEC_FILE);

  /* check flag */
  if(argc>1) {
    if(strcmp(argv[1],"-h")==0 || strcmp(argv[1],"--help")==0) {
      fprintf(stderr,"Usage: %s [-c]\n",argv[0]);
      fprintf(stderr,"A flag -c is for ctime() which has already been corrected for leap_sec.\n");
      exit(1);
    } else if(strcmp(argv[1],"-c")==0) {
      ctime_flag=0;
    }
  }

/* load src file */
  for(ii=0;ii<MAX_LINE;ii++) {
    got_c=fgets(buffer,BUFFER_SIZE,fp_in);
    if(got_c==NULL) break;
    if(feof(fp_in)!=0) break;
    if(ii==MAX_LINE-1) {
      fprintf(stderr,"%s: **ERR** The src file %s has too many (%d) lines.\n",argv[0],LEAP_SEC_SOURCE_FILE,ii);
      exit(99);
    }
    if(buffer[0]=='#') {
      ii--;
      goto next_line;
    }
    if(strncasecmp(buffer,"end",3)==0) break;
    if(strncmp(buffer,"Mon ",4)!=0
    && strncmp(buffer,"Tue ",4)!=0
    && strncmp(buffer,"Wed ",4)!=0
    && strncmp(buffer,"Thu ",4)!=0
    && strncmp(buffer,"Fri ",4)!=0
    && strncmp(buffer,"Sat ",4)!=0
    && strncmp(buffer,"Sun ",4)!=0) {
      break;
    }

    token=strtok(buffer,"\t,\n=");
    if(token==NULL) {
      fprintf(stderr,"%s: **ERR** Line %d is invalid.\n",argv[0],ii);
      exit(99);
    }
    entry[ii].utc=ctime2utc(token);
    token=strtok(NULL," \t,\n");
    if(token==NULL) {
      fprintf(stderr,"%s: **ERR** Line %d is invalid.\n",argv[0],ii);
      exit(99);
    }
    entry[ii].leap_sec=atoi(token);

    next_line:;
  }
  n_line=ii;

  fprintf(stderr,"%d valid lines in total.\n",n_line);
  if(n_line<=0) {
    fprintf(stderr,"%s: **ERR** No data in src file.\n",argv[0]);
    exit(99);
  }

/* sort in increasing utc */
  qsort(entry,n_line,sizeof(entry[0]),time_order);

/* correct too simple ctime() to get UTC */
  if(ctime_flag==1) {
    for(ii=0;ii<n_line;ii++) {
      if(entry[ii].utc>UTC_F_OFFSET)
	entry[ii].utc=entry[ii].utc+entry[ii].leap_sec-D_ET_UTC;
    }
  }

/* write the target file */
  fprintf(fp_out,"# ET(TD)-UTC\tUTC\texp.Date\n");
  fprintf(fp_out,"%ld\t0\tnow\n",entry[0].leap_sec);
  for(ii=0;ii<n_line-1;ii++) {
    strcpy(buffer,ctime(&entry[ii].utc));
    l_str=strlen(buffer);
    if(buffer[l_str-1]=='\n') 
      buffer[l_str-1]=(char)NULL;
    fprintf(fp_out,"%ld\t%ld\t%s\n",entry[ii+1].leap_sec,entry[ii].utc,buffer);
  }

  fclose(fp_in);
  fclose(fp_out);

/* confirmation */
  fp_in=fopen(LEAP_SEC_FILE,"r");

  fprintf(stderr,"--------leap second list---------\n");
  fprintf(stderr," Expire date             == ET(TD)-UTC(sec), UTC\n");
  for(ii=0;ii<n_line;ii++) {
    fgets(buffer,BUFFER_SIZE,fp_in);
    if(buffer[0]=='#') {
      fprintf(stderr,"%s",buffer);
    } else {
      token=strtok(buffer,"\t");
      leap_sec=atoi(token);
      token=strtok(NULL,"\t\n");
      utc=atoi(token);
      if(utc==0) {
	fprintf(stderr,"until now                == %ld\n",leap_sec);
      } else {
	strcpy(buffer,ctime(&utc));
	l_str=strlen(buffer);
	if(buffer[l_str-1]=='\n') 
	  buffer[l_str-1]=(char)NULL;
	fprintf(stderr,"%s == %ld, %10ld\n",buffer,leap_sec,utc);
      }
    }
  }
  fclose(fp_in);

  exit(0);
}
